/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CAVSelect.m,v 1.8 2002/09/23 08:47:41 wprice Exp $
____________________________________________________________________________*/
#import <PGP/pgpOptionList.h>
#import <PGP/pgpKeyServer.h>
#import "CAVSelect.h"
#import "CRemoteAuth.h"
#import "pflPrefs.h"
#import "pgpClientLibCocoa.h"
#import "pgpClientPrefs.h"
#import "pflPrefTypes.h"

#define kCAVSelectNib				@"CAVSelect"
#define kMaxAVPairs					20
#define kNumDisplayedAttrs			16

	static PGPError
sCreateCommonAVPairs (
	PGPContextRef		context,
	PGPKeyServerClass	serverclass,
	PGPKeyDBObjRef		userid,
	PGPAttributeValue**	ppAVcur )
{
	PGPError			err			= kPGPError_NoErr;
	void				*pName;
	PGPSize				size;

	// first compute those attributes based on the userid
	if( PGPKeyDBObjRefIsValid( userid ) )
	{
		// get the "common name" attribute from the userid
		err = PGPGetKeyDBObjAllocatedDataProperty( userid, kPGPUserIDProperty_CommonName, &pName, &size );
		if( IsntPGPError( err ) && ( size > 0 ) )
		{
			if( serverclass == kPGPKeyServerClass_Verisign )
			{
				char* pFirstName;
				char* pMiddleName;
				char* pLastName;

				PGPclParseCommonName( PGPPeekContextMemoryMgr (context),
					(char *)pName, &pFirstName, &pMiddleName, &pLastName);

				if (pFirstName)
				{
					(*ppAVcur)->attribute = 
								kPGPAVAttribute_MailFirstName;
					(*ppAVcur)->size = strlen( pFirstName );
					(*ppAVcur)->value.pointervalue = pFirstName;
					++(*ppAVcur);
				}

				if (pMiddleName)
				{
					(*ppAVcur)->attribute = 
								kPGPAVAttribute_MailMiddleName;
					(*ppAVcur)->size = strlen( pMiddleName );
					(*ppAVcur)->value.pointervalue = pMiddleName;
					++(*ppAVcur);
				}

				if (pLastName)
				{
					(*ppAVcur)->attribute = 
								kPGPAVAttribute_MailLastName;
					(*ppAVcur)->size = strlen( pLastName );
					(*ppAVcur)->value.pointervalue = pLastName;
					++(*ppAVcur);
				}

				PGPFreeData (pName);
			}
			else
			{
				(*ppAVcur)->attribute = kPGPAVAttribute_CommonName;
				(*ppAVcur)->size = size;
				(*ppAVcur)->value.pointervalue = pName;
				++(*ppAVcur);
			}
		}
		else
			err = kPGPError_InvalidProperty;
		if( IsPGPError( err ) )
			goto done;

		// get the "email" attribute from the userid
		err = PGPGetKeyDBObjAllocatedDataProperty( userid,
						kPGPUserIDProperty_EmailAddress,
						&(*ppAVcur)->value.pointervalue, &size );

		if( IsntPGPError( err ) || IsntNull( (*ppAVcur)->value.pointervalue ) )
		{
			if( serverclass == kPGPKeyServerClass_Microsoft )
				(*ppAVcur)->attribute = kPGPAVAttribute_RFC822Name;
			else
				(*ppAVcur)->attribute = kPGPAVAttribute_Email;
			(*ppAVcur)->size = size;
			++(*ppAVcur);
		}
		else
			err = kPGPError_InvalidProperty;
		if( IsPGPError( err ) )
			goto done;
	}

done:
	return err;
}

	static PGPError
sGetAVPairsFromPrefs(
	PGPContextRef		context,
	PGPPrefRef			prefref,
	PGPAttributeValue**	ppAVcur )
{
	PGPError			err;
	PGPSize				size;
	void				*data;
	PGPUInt32			numAVs;
	PGPAttributeValue*	pAVlist;
	PGPUInt32			u;

	err = PGPGetPrefData (prefref, kPGPPrefExtraAVPairs, &size, &data);
	if (IsntPGPError (err))
	{
		err = PGPclDataToAVPairs(PGPPeekContextMemoryMgr (context),
					data, size, &pAVlist, &numAVs);
		PGPFreeData(data);

		if (IsntPGPError(err)) {
			for (u=0; u<numAVs; u++)
			{
				(*ppAVcur)->attribute			= pAVlist[u].attribute;
				(*ppAVcur)->size				= pAVlist[u].size;
				(*ppAVcur)->value.pointervalue	= pAVlist[u].value.pointervalue;
				++(*ppAVcur);
			}

			PGPFreeData (pAVlist);
		}
	}
	return err;
}

	static void
sCreateDefaultAVPairs (
	PGPContextRef		context,
	PGPKeyServerClass	serverclass,
	PGPAttributeValue**	ppAVcur)
{
	switch (serverclass)
	{
		case kPGPKeyServerClass_NetToolsCA:
		case kPGPKeyServerClass_Netscape:
		case kPGPKeyServerClass_Microsoft:
		case kPGPKeyServerClass_Entrust:
			break;
		case kPGPKeyServerClass_Verisign :
		{
			char	vs_certtype[]		= "end-user";
			PGPSize	len;
			
			(*ppAVcur)->attribute = kPGPAVAttribute_OrganizationName;
			(*ppAVcur)->size = 0;
			(*ppAVcur)->value.pointervalue = NULL;
			++(*ppAVcur);

			(*ppAVcur)->attribute = kPGPAVAttribute_OrganizationalUnitName;
			(*ppAVcur)->size = 0;
			(*ppAVcur)->value.pointervalue = NULL;
			++(*ppAVcur);

			len = strlen( vs_certtype );
			(*ppAVcur)->attribute = kPGPAVAttribute_CertType;
			(*ppAVcur)->size = len;
			(*ppAVcur)->value.pointervalue = PGPNewData (
						PGPPeekContextMemoryMgr( context ), len + 1, 0);
			strcpy( (char *)(*ppAVcur)->value.pointervalue, vs_certtype );
			++(*ppAVcur);
			break;
		}
		default:
			break;
	}
}

	static PGPError
sAppendClassAVPairs(
	PGPContextRef		context,
	PGPPrefRef			clientPrefs,
	PGPKeyServerClass	serverClass,
	PGPAttributeValue**	ppAVcur)
{
	PGPError			err					= kPGPError_NoErr;
	char *				passphrase			= NULL;
	PGPSize				len;
	PGPBoolean			enforceLength = FALSE,
						enforceQuality = FALSE;
	PGPUInt32			minLength,
						minQuality;

	switch( serverClass )
	{
		case kPGPKeyServerClass_NetToolsCA:
		case kPGPKeyServerClass_Netscape:
		case kPGPKeyServerClass_Microsoft:
		case kPGPKeyServerClass_Entrust:
			return err;
		case kPGPKeyServerClass_Verisign:
			if( PGPclEnterprise() )
			{
				PGPGetPrefBoolean( clientPrefs, kPGPPrefEnforceMinChars, &enforceLength);
				PGPGetPrefBoolean( clientPrefs, kPGPPrefEnforceMinQuality, &enforceQuality);
				PGPGetPrefNumber( clientPrefs, kPGPPrefMinChars, &minLength);
				PGPGetPrefNumber( clientPrefs, kPGPPrefMinQuality, &minQuality);
			}
			{
				char				cstr[256];

#define self	CAVSelect
				[LSCL( @"Please enter a challenge passphrase:", @"VeriSign Challenge request" )
					getCString:cstr maxLength:sizeof(cstr)-1];
#undef self
				err = PGPConfirmationPassphraseDialog( context,
						PGPOUIDialogPrompt( context, cstr ),
						PGPOUIOutputPassphrase( context, &passphrase ),
						enforceLength ?
							PGPOUIMinimumPassphraseLength( context, minLength ) :
							PGPONullOption( context ),
						enforceQuality ?
							PGPOUIMinimumPassphraseQuality( context, minQuality ) :
							PGPONullOption( context ),
						PGPOLastOption( context ) );
			}
			if (IsntPGPError (err))
			{
				len = strlen( passphrase );
				(*ppAVcur)->attribute = kPGPAVAttribute_Challenge;
				(*ppAVcur)->size = len;
				(*ppAVcur)->value.pointervalue = PGPNewData (
						PGPPeekContextMemoryMgr (context), len + 1, 0);
				strcpy( (char *)(*ppAVcur)->value.pointervalue, passphrase );
				++(*ppAVcur);
			}
			if( passphrase )
				PGPFreeData( passphrase );

			return err;
		default :
			return err;
	}
}

	PGPError
PGPclFreeCACertReqAVList(
	PGPAttributeValue *	pAVList,
	PGPUInt32			numAVs )
{
	PGPUInt32			u;

	if(pAVList) 
	{
		for(u=0; u<numAVs; u++)
		{
			if ((pAVList[u].attribute >= kPGPAVAttributeFirstPointer) &&
				(pAVList[u].attribute < kPGPAVAttributeFirstBoolean))
			{
				if(pAVList[u].value.pointervalue)
					PGPFreeData(pAVList[u].value.pointervalue);
			}
		}

		PGPFreeData(pAVList);
	}
	return kPGPError_NoErr;
}

	PGPError
PGPclGetCACertReqAVList(
	PGPContextRef			context,
	PGPPrefRef				prefRef,
	PGPBoolean				force,
	PGPKeyDBObjRef			userID,
	PGPKeyServerClass		serverClass,
	PGPUInt32 *				requestType,
	PGPAttributeValue **	pAVList,
	PGPUInt32 *				pNumAVs )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				numAVs = 0;
	PGPAttributeValue		tempAVList[kMaxAVPairs];
	PGPAttributeValue *		pAVCur = &tempAVList[0];
	
	if( IsntPGPError( err ) )
	{
		// if there isn't a user id, this won't add any pairs
		err = sCreateCommonAVPairs( context, serverClass, userID, &pAVCur );
		if( IsntPGPError( err ) )
		{
			if( PGPclEnterprise() )
			{
				if (IsntNull(*pAVList)) {
					pgpCopyMemory(*pAVList, tempAVList, (*pNumAVs * sizeof(PGPAttributeValue)));
					pAVCur = &tempAVList[*pNumAVs];
				} else {
					err = sGetAVPairsFromPrefs( context, prefRef, &pAVCur );
				}
			}
			if( !PGPclEnterprise() || IsPGPError( err ) || force )
			{
				if( !PGPclEnterprise() || IsPGPError(err) )
				{
					if (IsNull(*pAVList)) {
						sCreateDefaultAVPairs( context, serverClass, &pAVCur );
					}
				}
				{
					CAVSelect	*avSelect = [[CAVSelect alloc] initAVSelect:PGPPeekContextMemoryMgr(context)
									serverClass:serverClass
									requestType:*requestType
									attributesBegin:tempAVList
									attributesEnd:pAVCur];
					
					[[NSApplication sharedApplication] runModalForWindow:[avSelect window]];
					err = [avSelect resultCode];
					if( IsntPGPError( err ) )
						[avSelect getRequest:requestType endAttributes:&pAVCur];
					[avSelect release];
				}
			}
		}
		if( IsntPGPError( err ) )
		{
			if( PGPKeyDBObjRefIsValid( userID ) )
			{
				err = sAppendClassAVPairs( context, prefRef, serverClass, &pAVCur );
			}
		}
		numAVs = pAVCur - &tempAVList[0];
		if( IsntPGPError(err) )
		{
			PGPSize		avSize;

			avSize = numAVs * sizeof(PGPAttributeValue);
			if (IsntNull(*pAVList)) {
				PGPFreeData(*pAVList);
			}

			*pAVList = (PGPAttributeValue *)PGPNewData (
				PGPPeekContextMemoryMgr( context ), avSize, 0 );
			if( IsntNull( *pAVList ) )
			{
				pgpCopyMemory( tempAVList, *pAVList, avSize );
				*pNumAVs = numAVs;
			}
			else 
				err = kPGPError_OutOfMemory;
		}
		
		if( IsPGPError(err) )
		{
			PGPUInt32	u;

			for( u=0; u < numAVs; u++ )
			{
				if( tempAVList[u].value.pointervalue )
					PGPFreeData( tempAVList[u].value.pointervalue );
			}
		}
	}	
	return( err );
}

@implementation CAVSelect

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCAVSelectNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	if( !mLoadedNib )
		[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initAVSelect:(PGPMemoryMgrRef)memoryMgr
		serverClass:(PGPKeyServerClass)serverClass
		requestType:(PGPUInt32)requestType
		attributesBegin:(PGPAttributeValue *)attributesBegin
		attributesEnd:(PGPAttributeValue *)attributesEnd
{
	NSTextFieldCell			*tCell;
	NSPopUpButtonCell		*pCell;
	NSString				*caTypeStr = nil;
	PGPInt32				attrIndex;
	
	[self init];
	mResultCode	= kPGPError_UserAbort;
	mStartAV	= attributesBegin;
	mEndAV		= attributesEnd;
	mMemMgr		= memoryMgr;
	
	switch( serverClass )
	{
		case kPGPKeyServerClass_NetToolsCA:
			caTypeStr = LSCL( @"Net Tools PKI", @"Net Tools PKI" );
			break;
		case kPGPKeyServerClass_Verisign:
			caTypeStr = LSCL( @"VeriSign OnSite", @"VeriSign OnSite" );
			break;
		case kPGPKeyServerClass_Entrust:
			caTypeStr = LSCL( @"Entrust", @"Entrust" );
			break;
		case kPGPKeyServerClass_Netscape:
			caTypeStr = LSCL( @"iPlanet CMS", @"iPlanet CMS" );
			break;
		case kPGPKeyServerClass_Microsoft:
			caTypeStr = LSCL( @"Windows 2000", @"Windows 2000" );
			break;
		default:
			break;
	}
	[mCATypeField setStringValue:caTypeStr];
	
	if( requestType != kPGPPrefCertRequestCRS )
		[mReqTypeMatrix selectCellAtRow:0 column:1];
	
	[mRemoveButton setEnabled:NO];
	
	pCell = [[NSPopUpButtonCell alloc] init];
	[pCell setControlSize:NSSmallControlSize];
	[pCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	for( attrIndex = 0;attrIndex < kNumDisplayedAttrs; attrIndex++ )
		[pCell addItemWithTitle:[self attrString:[self indexToAttribute:attrIndex]]];
	[[[mAVTableView tableColumns] objectAtIndex:0] setDataCell:pCell];
	[pCell release];

	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[tCell setEditable:YES];
	[[[mAVTableView tableColumns] objectAtIndex:1] setDataCell:tCell];
	[tCell release];

	[mAVTableView reloadData];
	[mAVWindow center];
	[mAVWindow makeKeyAndOrderFront:self];
	return self;
}

- (BOOL)attributeInUse:(PGPAVAttribute)attribute exclude:(int)row
{
	PGPInt32		numPairs,
					pairIndex;
	
	numPairs = mEndAV - mStartAV;
	for( pairIndex = 0;pairIndex < numPairs;pairIndex++ )
	{
		if( ( pairIndex != row ) && ( mStartAV[pairIndex].attribute == attribute ) )
			break;
	}
	if( pairIndex == numPairs )
		return NO;
	else
		return YES;
}

- (IBAction)addButtonPressed:(id)sender
{
	PGPInt32	attrIndex;
	
	if( ( mEndAV - mStartAV ) < kNumDisplayedAttrs )
	{
		for( attrIndex = 0; attrIndex < kNumDisplayedAttrs; attrIndex++ )
			if( ![self attributeInUse:[self indexToAttribute:attrIndex] exclude:-1] )
			{
				mEndAV->attribute = [self indexToAttribute:attrIndex];
				break;
			}
		mEndAV->size = 0;
		mEndAV->value.pointervalue = NULL;
		mEndAV++;
		[mAVTableView reloadData];
	}
}

- (IBAction)removeButtonPressed:(id)sender
{
	PGPAttributeValue	*av = mStartAV + [mAVTableView selectedRow];
	
	if( IsntNull( av->value.pointervalue ) )
		PGPFreeData( av->value.pointervalue );
	if( mEndAV - av > 1 )
		pgpCopyMemory( av + 1, av, ( mEndAV - av - 1 ) * sizeof(PGPAttributeValue) );
	mEndAV--;
	[mAVTableView reloadData];
	[mAVTableView deselectAll:self];
	[mRemoveButton setEnabled:NO];
}

- (IBAction)actionButtonPressed:(id)sender
{
	if( ![sender tag] )
		mResultCode = kPGPError_NoErr;
	[mAVWindow performClose:self];
}

- (IBAction)selectionChanged:(id)sender
{
	[mRemoveButton setEnabled:( [mAVTableView selectedRow] >= 0 )];
}

- (PGPAVAttribute)indexToAttribute:(PGPInt32)index
{
	PGPAVAttribute		attr = kPGPAVAttributeFirstPointer;
	
	switch( index )
	{
		case 0:		attr = kPGPAVAttribute_CommonName;				break;
		case 1:		attr = kPGPAVAttribute_Email;					break;
		case 2:		attr = kPGPAVAttribute_OrganizationName;		break;
		case 3:		attr = kPGPAVAttribute_OrganizationalUnitName;	break;
		case 4:		attr = kPGPAVAttribute_Country;					break;
		case 5:		attr = kPGPAVAttribute_Locality;				break;
		case 6:		attr = kPGPAVAttribute_State;					break;
		case 7:		attr = kPGPAVAttribute_StreetAddress;			break;
		case 8:		attr = kPGPAVAttribute_PostalCode;				break;
		case 9:		attr = kPGPAVAttribute_POBOX;					break;
		case 10:	attr = kPGPAVAttribute_TelephoneNumber;			break;
		case 11:	attr = kPGPAVAttribute_Initials;				break;
		case 12:	attr = kPGPAVAttribute_DNSName;					break;
		case 13:	attr = kPGPAVAttribute_IPAddress;				break;
		case 14:	attr = kPGPAVAttribute_CertType;				break;
		case 15:	attr = kPGPAVAttribute_RFC822Name;				break;
	}
	return attr;
}

- (PGPInt32)attributeToIndex:(PGPAVAttribute)attribute
{
	PGPInt16				index = -1;
	
	switch( attribute )
	{
		default:
		case kPGPAVAttribute_CommonName:				index = 0;		break;
		case kPGPAVAttribute_Email:						index = 1;		break;
		case kPGPAVAttribute_OrganizationName:			index = 2;		break;
		case kPGPAVAttribute_OrganizationalUnitName:	index = 3;		break;
		case kPGPAVAttribute_Country:					index = 4;		break;
		case kPGPAVAttribute_Locality:					index = 5;		break;
		case kPGPAVAttribute_State:						index = 6;		break;
		case kPGPAVAttribute_StreetAddress:				index = 7;		break;
		case kPGPAVAttribute_PostalCode:				index = 8;		break;
		case kPGPAVAttribute_POBOX:						index = 9;		break;
		case kPGPAVAttribute_TelephoneNumber:			index = 10;		break;
		case kPGPAVAttribute_Initials:					index = 11;		break;
		case kPGPAVAttribute_DNSName:					index = 12;		break;
		case kPGPAVAttribute_IPAddress:					index = 13;		break;
		case kPGPAVAttribute_CertType:					index = 14;		break;
		case kPGPAVAttribute_RFC822Name:				index = 15;		break;
	}
	return index;
}

- (NSString *)attrString:(PGPAVAttribute)attr
{
	NSString		*str = nil;

	switch( attr )
	{
		case kPGPAVAttribute_CommonName:
			str = LSCL( @"Full Name", @"AV:Full Name" );
			break;
		case kPGPAVAttribute_Email:
			str = LSCL( @"Email Address", @"AV:Email Address" );
			break;
		case kPGPAVAttribute_OrganizationName:
			str = LSCL( @"Organization Name", @"AV:Organization Name" );
			break;
		case kPGPAVAttribute_OrganizationalUnitName:
			str = LSCL( @"Organization Unit", @"AV:Organization Unit Name" );
			break;
		case kPGPAVAttribute_Country:
			str = LSCL( @"Country", @"AV:Country" );
			break;
		case kPGPAVAttribute_Locality:
			str = LSCL( @"Locality", @"AV:Locality" );
			break;
		case kPGPAVAttribute_State:
			str = LSCL( @"State", @"AV:State" );
			break;
		case kPGPAVAttribute_StreetAddress:
			str = LSCL( @"Street Address", @"AV:Street Address" );
			break;
		case kPGPAVAttribute_PostalCode:
			str = LSCL( @"Postal Code", @"AV:Postal Code" );
			break;
		case kPGPAVAttribute_POBOX:
			str = LSCL( @"PO Box", @"AV:PO Box" );
			break;
		case kPGPAVAttribute_TelephoneNumber:
			str = LSCL( @"Telephone Number", @"AV:Telephone Number" );
			break;
		case kPGPAVAttribute_Initials:
			str = LSCL( @"Initials", @"AV:Initials" );
			break;
		case kPGPAVAttribute_DNSName:
			str = LSCL( @"Domain Name", @"AV:Domain Name" );
			break;
		case kPGPAVAttribute_IPAddress:
			str = LSCL( @"IP Address", @"AV:IP Address" );
			break;
		case kPGPAVAttribute_CertType:
			str = LSCL( @"Certificate Type", @"AV:Certificate Type" );
			break;
		case kPGPAVAttribute_RFC822Name:
			str = LSCL( @"Email Address (RFC822)", @"AV:Email Address (RFC822)" );
			break;
		default:
			break;
	}
	return str;
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return mEndAV - mStartAV;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	id					value = nil;
	PGPAttributeValue	*av = mStartAV + row;
	
	switch( [[tableColumn identifier] intValue] )
	{
		case 0:
			value = [NSNumber numberWithInt:[self attributeToIndex:av->attribute]];
			break;
		case 1:
			if( av->size )
				value = [NSString stringWithCString:av->value.pointervalue length:av->size];
			break;
	}
	return value;
}

- (BOOL)tableView:(NSTableView*)table shouldEditTableColumn:(NSTableColumn*)column row:(int)row
{
	return YES;
}

- (void)tableView:(NSTableView*)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn*)column row:(int)row
{
	PGPAttributeValue	*av = mStartAV + row;
	NSString			*str = object;
	PGPAVAttribute		attribute;

	switch( [[column identifier] intValue] )
	{
		case 0:
			attribute = [self indexToAttribute:[object intValue]];
			if( [self attributeInUse:attribute exclude:row] )
				NSBeep();
			else
				mStartAV[row].attribute = attribute;
			break;
		case 1:
			if( !object )
				break;
			if( IsntNull( av->value.pointervalue ) )
				PGPFreeData( av->value.pointervalue );
			av->value.pointervalue = PGPNewData( mMemMgr, [str length], 0);
			if( IsntNull( av->value.pointervalue ) )
				pgpCopyMemory( [str cString], av->value.pointervalue, [str length] );
			av->size = [str length];
			break;
	}
}

- (NSWindow *)window
{
	return mAVWindow;
}

- (PGPError)resultCode
{
	return mResultCode;
}

- (void)getRequest:(PGPUInt32 *)requestType endAttributes:(PGPAttributeValue **)endAttrP
{
	*requestType	= [mReqTypeMatrix selectedColumn] ? kPGPPrefCertRequestPKCS10 : kPGPPrefCertRequestCRS;
	*endAttrP		= mEndAV;
}

- (BOOL)windowShouldClose:(id)sender
{
	[[NSApplication sharedApplication] stopModal];
	return YES;
}

- (void)dealloc
{
	[super dealloc];
	[mAVWindow release];
	PGPLog(( @"CAVSelect dealloc" ));
}

@end

