/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CCertInfo.m,v 1.8 2002/11/11 05:26:30 wprice Exp $
____________________________________________________________________________*/
#import <PGPui/CPGPIcons.h>
#import "pgpClientLibCocoa.h"
#import "CKeyInfo.h"
#import "CCertInfo.h"
#import "CRemoteAuth.h"

#define kCCertInfoNib					@"CCertInfo"
#define kCCertInfoNAString				LSCL( @"N/A", @"N/A" )

static NSMutableArray	*sCertInfoArray = nil;

@implementation CCertInfo

+ (id)showCertInfo:(id)controller forCert:(PGPKeyDBObjRef)cert
{
	CCertInfo		*certInfo = nil;
	
	if( !sCertInfoArray )
		sCertInfoArray = [[NSMutableArray alloc] init];
	{
		NSEnumerator	*certInfoEnumerator = [sCertInfoArray objectEnumerator];

		while( certInfo = [certInfoEnumerator nextObject] )
		{
			if( cert == [certInfo cert] )
				break;
		}
		if( !certInfo )
		{
			certInfo = [[CCertInfo alloc] initWithController:controller forCert:cert];
			[sCertInfoArray addObject:certInfo];
			[certInfo release];
		}
		[[certInfo window] makeKeyAndOrderFront:controller];
	}
	return certInfo;
}

+ (void)closeAll:(id)sender forDB:(PGPKeyDBRef)keyDB
{
	NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
	if( sCertInfoArray )
	{
		NSEnumerator	*certInfoEnumerator = [sCertInfoArray objectEnumerator],
						*trashEnumerator;
		NSMutableArray	*trashArray = [[NSMutableArray alloc] init];
		CCertInfo		*certInfo;
	
		while( certInfo = [certInfoEnumerator nextObject] )
		{
			if( keyDB == [certInfo keyDB] )
				[trashArray addObject:certInfo];
		}
		trashEnumerator = [trashArray objectEnumerator];
		while( certInfo = [trashEnumerator nextObject] )
			[[certInfo window] performClose:sender];
		[trashArray release];
		if( ![sCertInfoArray count] )
		{
			[sCertInfoArray release];
			sCertInfoArray = nil;
		}
	}
	[pool release];
}

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCCertInfoNib];
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
    
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller
		forCert:(PGPKeyDBObjRef)keyDBObj
{
	[self init];
	[m509Box retain];
	[mPGPBox retain];
	mOwnerController = controller;
	mCert = keyDBObj;
	[self setInfo];
	return self;
}

- (void)setInfo
{
	PGPError			err;
	PGPBoolean			x509;
	NSBox				*theBox;
	NSRect				panelRect;
	float				addHeight;
	NSPoint				boxOrigin;
	PGPTime				pgpTime;
	PGPBoolean			exportable,
						expired,
						revoked;
	PGPInt32			trustLevel;
	PGPSize				len;
	char				cstr[kPGPcl_MaxPGPUserIDLength];
	NSUserDefaults		*defaults	= [NSUserDefaults standardUserDefaults];
	
	PGPGetKeyDBObjBooleanProperty( mCert, kPGPSigProperty_IsX509, &x509);
	theBox = x509 ? m509Box : mPGPBox;
	[theBox removeFromSuperview];
	addHeight = [theBox frame].size.height + 20;
	panelRect = [mCertInfoWindow frame];
	boxOrigin.x = 20;
	boxOrigin.y = panelRect.size.height - 20;
	panelRect.origin.y -= addHeight;
	panelRect.size.height += addHeight;
	[mCertInfoWindow setFrame:panelRect display:NO animate:NO];
	[theBox setFrameOrigin:boxOrigin];
	[[mCertInfoWindow contentView] addSubview:theBox];
	[mCertInfoWindow center];
	
	if( x509 )
	{
		err = PGPGetSigX509CertifierSig( mCert, PGPPeekKeyDBObjKeyDB( mCert ), &mSignerCert );
		if( mSignerCert == mCert )
			mSignerCert = kInvalidPGPKeyDBObjRef;
	}
	else
	{
		err = PGPGetSigCertifierKey( mCert, PGPPeekKeyDBObjKeyDB( mCert ), &mSignerKey );
	}
	if( ( !PGPKeyDBObjRefIsValid( mSignerKey ) && !PGPKeyDBObjRefIsValid( mSignerCert ) ) ||
		[[NSApplication sharedApplication] modalWindow] )
		[mShowSignerButton setEnabled:NO];
	PGPGetKeyDBObjTimeProperty( mCert, kPGPSigProperty_Creation, &pgpTime );
	[mCreatedField setObjectValue:
		[[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
		descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
		locale:[defaults dictionaryRepresentation]]];
	
	PGPGetKeyDBObjTimeProperty( mCert, kPGPSigProperty_Expiration, &pgpTime );
	if( pgpTime != 0 )
	{
		[mExpiresField setObjectValue:
			[[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
			descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
			locale:[defaults dictionaryRepresentation]]];
	}
	else
		[mExpiresField setStringValue:LSCL( @"Never", @"No expiration" )];
	
	[mCertImage setImage:[[CPGPIcons shared] getKeyDBObjIcon:mCert]];
	PGPGetKeyDBObjBooleanProperty( mCert, kPGPSigProperty_IsRevoked, &revoked);
	PGPGetKeyDBObjBooleanProperty( mCert, kPGPSigProperty_IsExportable, &exportable);
	PGPGetKeyDBObjBooleanProperty( mCert, kPGPSigProperty_IsExpired, &expired);
	PGPGetKeyDBObjNumericProperty( mCert, kPGPSigProperty_TrustLevel, &trustLevel);
	if( exportable )
		[mExportableCheckbox setState:1];
	if( expired )
		[mExpiredCheckbox setState:1];
	if( revoked )
		[mRevokedCheckbox setState:1];
	if( x509 )
	{
		PGPBoolean		hasCRL;
		PGPKeyDBObjRef	ownerKey;
		
		err = PGPGetKeyDBObjDataProperty( mCert, kPGPSigProperty_X509LongName,
				&cstr[0], sizeof(cstr) - 1, &len );
		if( IsntPGPError( err ) )
			[m509NameField setStringValue:[NSString stringWithCString:cstr length:len]];
		err = PGPGetKeyDBObjDataProperty( mCert,kPGPSigProperty_X509IssuerLongName,
				&cstr[0], sizeof(cstr) - 1, &len );
		if( IsntPGPError( err ) )
			[m509IssuerField setStringValue:[NSString stringWithCString:cstr length:len]];
		ownerKey = PGPPeekKeyDBObjKey( mCert );
		PGPGetKeyDBObjBooleanProperty( ownerKey, kPGPKeyProperty_HasCRL, &hasCRL );
		if( hasCRL )
		{
			PGPGetKeyDBObjTimeProperty( ownerKey, kPGPKeyProperty_CRLThisUpdate, &pgpTime );
			[m509LastCRLField setObjectValue:
				[[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
				descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
				locale:[defaults dictionaryRepresentation]]];
		}
		else
			[m509LastCRLField setStringValue:kCCertInfoNAString];
		if( hasCRL )
		{
			PGPGetKeyDBObjTimeProperty( ownerKey, kPGPKeyProperty_CRLNextUpdate, &pgpTime );
			[m509NextCRLField setObjectValue:
				[[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
				descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
				locale:[defaults dictionaryRepresentation]]];
		}
	}
	else
	{
		PGPKeyID	keyID;
		char		keyIDString[ kPGPMaxKeyIDStringSize ];
		
		err = PGPGetKeyDBObjDataProperty( mCert, kPGPSigProperty_KeyID, &keyID, sizeof(PGPKeyID), &len );
		if( IsntPGPError( err ) )
		{
			PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, keyIDString );
			[mPGPSignerIDField setStringValue:[NSString stringWithCString:keyIDString]];
		}
		if( PGPKeyDBObjRefIsValid( mSignerKey ) )
		{
			[mPGPSignerField setStringValue:PGPclGetPrimaryUserIDName( mSignerKey )];
		}
		else
			[mPGPSignerField setStringValue:kCCertInfoNAString];
		[mTrustDepthField setIntValue:trustLevel];
		err = PGPGetKeyDBObjDataProperty( mCert, kPGPSigProperty_RegularExpression,
					&cstr[0], sizeof(cstr) - 1, &len );
		if( IsntPGPError( err ) && cstr[0] )
			[mPGPDomainField setStringValue:[NSString stringWithCString:cstr length:len]];
		else
			[mPGPDomainField setStringValue:kCCertInfoNAString];
	}
}

- (IBAction)showSignerAction:(id)sender
{
	if( PGPKeyDBObjRefIsValid( mSignerKey ) )
	{
		[CKeyInfo showKeyInfo:mOwnerController forKey:mSignerKey
					mainDB:[mOwnerController mainDB]
					lock:NO];
	}
	else if( PGPKeyDBObjRefIsValid( mSignerCert ) )
	{
		[CCertInfo showCertInfo:mOwnerController forCert:mSignerCert];
	}
}

- (PGPKeyDBObjRef)cert
{
	return mCert;
}

- (PGPKeyDBRef)keyDB
{
	return PGPPeekKeyDBObjKeyDB( mCert );
}

- (NSWindow *)window
{
	return mCertInfoWindow;
}

- (BOOL)windowShouldClose:(id)sender
{
	[sCertInfoArray removeObject:self];
	return YES;
}

- (void)dealloc
{
	[m509Box release];
	[mPGPBox release];
	[super dealloc];
}

@end

