/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CCertSelect.m,v 1.6 2002/08/06 20:09:16 dallen Exp $
____________________________________________________________________________*/
#import "CCertSelect.h"
#import "pgpClientLibCocoa.h"
#import <PGPui/CBrowserCell.h>
#import <PGPui/CPGPIcons.h>

#define kCCertSelectNib		@"CCertSelect"

	void
PGPclBeginCertSelectSheet(
	NSString					*prompt,
	NSWindow					*docWindow,
	PGPKeyDBRef					fromDB,
	PGPUInt32					flags,
	id							target,
	SEL							didEndSelector )
{
	CCertSelect		*certSelector;

	certSelector = [[CCertSelect alloc] initCertSelect:prompt
					fromDB:fromDB
					usingFlags:flags
					target:target
					didEndSelector:didEndSelector];
	[certSelector runCertSelectModalTo:docWindow];
	[certSelector release];
}

@implementation CCertSelect

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCCertSelectNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initCertSelect:(NSString *)prompt
		fromDB:(PGPKeyDBRef)keyDB
		usingFlags:(PGPUInt32)flags
		target:(id)target
		didEndSelector:(SEL)didEndSelector;
{
	CBrowserCell			*bCell;
	PGPError				err;
	PGPFilterRef			filter = kInvalidPGPFilterRef,
							filter2 = kInvalidPGPFilterRef;
	PGPKeyListRef			keyList = kInvalidPGPKeyListRef;
	PGPKeyIterRef			keyIter = kInvalidPGPKeyIterRef;
	PGPKeyDBObjRef			curKey;
	PGPKeyDBObjRef			curUID;
	PGPKeyDBObjRef			curSig;
	
	[self init];
	mFlags = flags;
	mKeyDB = keyDB;
	mContext = PGPPeekKeyDBContext( keyDB );
	mTarget = target;
	mEndSelector = didEndSelector;
	mReturnCode = kPGPError_NoErr;
	[mPromptField setStringValue:prompt];
	
	bCell = [[CBrowserCell alloc] init];
	[bCell setLeaf:YES];
	[bCell setWraps:NO];
	[bCell setScrollable:NO];
	[[[mCertTableView tableColumns] objectAtIndex:0] setDataCell:bCell];
	[bCell release];
	mCertArray = [[NSMutableArray alloc] init];

	err = PGPNewKeyDBObjBooleanFilter( mContext, kPGPSigProperty_IsX509, TRUE, &filter ); CKERR;
	if( mFlags & kPGPclCertSelect_CanSignOnly )
	{
		err = PGPNewKeyDBObjBooleanFilter( mContext, kPGPKeyProperty_CanSign, TRUE, &filter2 ); CKERR;
		err = PGPIntersectFilters( filter, filter2, &filter ); CKERR;
		filter2 = kInvalidPGPFilterRef;
	}
	if( mFlags & kPGPclCertSelect_NoSplit )
	{
		err = PGPNewKeyDBObjBooleanFilter( mContext, kPGPKeyProperty_IsSecretShared, FALSE, &filter2 );CKERR;
		err = PGPIntersectFilters( filter, filter2, &filter );	CKERR;
		filter2 = kInvalidPGPFilterRef;
	}
	err = PGPFilterKeyDB( mKeyDB, filter, &m509KeySet ); CKERR;
	err = PGPOrderKeySet( m509KeySet, kPGPKeyOrdering_UserID, FALSE, &keyList );CKERR;
	err = PGPNewKeyIter( keyList, &keyIter );	CKERR;
	while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_Key, &curKey ) ) )
		while ( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_UserID, &curUID ) ) )
			while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_Signature, &curSig ) ) )
			{
				PGPBoolean	x509;
				
				PGPGetKeyDBObjBooleanProperty( curSig, kPGPSigProperty_IsX509, &x509 );
				if( x509 )
					[mCertArray addObject:[NSNumber numberWithInt:(int)curSig]];
			}
	[mOKButton setEnabled:NO];
done:
	if( PGPFilterRefIsValid( filter ) )
		PGPFreeFilter( filter );
	if( PGPFilterRefIsValid( filter2 ) )
		PGPFreeFilter( filter2 );
	if( PGPKeyIterRefIsValid( keyIter ) )
		PGPFreeKeyIter( keyIter );
	if( PGPKeyListRefIsValid( keyList ) )
		PGPFreeKeyList( keyList );
	[mCertTableView reloadData];
	return self;
}

- (void)certSelectSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
	mReturnCode = returnCode;
	[self autorelease];
}

- (void)runCertSelectModalTo:(NSWindow *)window
{
	[[NSApplication sharedApplication] beginSheet:mCertSelectWindow modalForWindow:window
		modalDelegate:self didEndSelector:@selector(certSelectSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
	[self retain];
}

- (IBAction)actionButtonPressed:(id)sender
{
	[[NSApplication sharedApplication] endSheet:mCertSelectWindow
		returnCode:[sender tag] ? kPGPError_NoErr : kPGPError_UserAbort];
}

- (IBAction)selectionChanged:(id)sender
{
	[mOKButton setEnabled:([mCertTableView selectedRow] >= 0)];
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [mCertArray count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	id				value = nil;
	char			cstr[kPGPcl_MaxPGPUserIDLength];
	PGPSize			len;
	PGPKeyDBObjRef	cert = (PGPKeyDBObjRef)[[mCertArray objectAtIndex:row] intValue];
	
	if( IsntPGPError( PGPGetKeyDBObjDataProperty( cert, kPGPSigProperty_X509LongName,
						&cstr[0], sizeof(cstr) - 1, &len ) ) )
	{
		if( len > sizeof(cstr) - 1 )
			cstr[sizeof(cstr) - 1] = '\0';
		else
			cstr[len] = '\0';
		value = [NSString stringWithCString:cstr];
	}
	return value;
}

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	PGPKeyDBObjRef	cert = (PGPKeyDBObjRef)[[mCertArray objectAtIndex:row] intValue];
	
	[cell setImage:[[CPGPIcons shared] getKeyDBObjIcon:cert]];
}

- (void)dealloc
{
	NSMethodSignature*		methodSignature;
	NSInvocation*			methodInvocation;
	PGPKeyDBObjRef			cert = kInvalidPGPKeyDBObjRef;
	int						row;
	
	row = [mCertTableView selectedRow];
	if( IsntPGPError( mReturnCode ) && ( row >= 0 ) )
		cert = (PGPKeyDBObjRef)[[mCertArray objectAtIndex:row] intValue];
	if( mEndSelector && [mTarget respondsToSelector:mEndSelector] )
	{
		methodSignature = [mTarget methodSignatureForSelector:mEndSelector];
		methodInvocation = [NSInvocation invocationWithMethodSignature:methodSignature];
		[methodInvocation setSelector:mEndSelector];
		[methodInvocation setTarget:mTarget];
		[methodInvocation setArgument:&mReturnCode atIndex:2];
		[methodInvocation setArgument:&cert atIndex:3];
		[methodInvocation invoke];
	}
	if( PGPKeySetRefIsValid( m509KeySet ) )
		PGPFreeKeySet( m509KeySet );
	[mCertArray release];
	[super dealloc];
	PGPLog(( @"CCertSelect dealloc" ));
}

@end

