/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CImageCell.m,v 1.6 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import "pgpClientLibCocoa.h"
#import "CImageCell.h"

@implementation CImageCell

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView*)controlView
{
	BOOL		highlighted = [self isHighlighted];
    NSImage*	sourceImage = [self image];
    NSPoint		compositePoint = cellFrame.origin;
	
    if (!sourceImage)
        return;

	if (highlighted || ([self state] == NSOnState))
	{
		NSImage*	darkImage;
		NSSize		darkImageSize;
		
        darkImage = [[NSImage alloc] initWithSize:[sourceImage size]];
		darkImageSize = [darkImage size];

        /*
        Create the highlighted image. To do this, we first draw the existing
        image into a blank image. We draw the source image because it contains
        size information which would be lost by simply copying the image. After
        making a copy of the image, we source-in the image with the color black.
        This causes the image to become a silhouette. After creating the image,
        we composite the highlight image with the source image using the source
        over operation.
        */
		[darkImage lockFocus];
        [sourceImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver fraction:1.0];
		[[NSColor blackColor] set];
		NSRectFillUsingOperation(NSMakeRect(0, 0, darkImageSize.width, darkImageSize.height), NSCompositeSourceIn);
		[darkImage unlockFocus];
        
        compositePoint.y = NSHeight(cellFrame) - (NSHeight(cellFrame) / 2) + (darkImageSize.height / 2);
    	compositePoint.x += (NSWidth(cellFrame) / 2) - (darkImageSize.width / 2);
        
		[darkImage compositeToPoint:compositePoint operation:NSCompositeSourceOver fraction:1.0];
		[sourceImage compositeToPoint:compositePoint operation:NSCompositeSourceOver fraction:0.6666];

		[darkImage release];
	}
	else
	{
        NSSize		imageSize = [sourceImage size];

        compositePoint.y = NSHeight(cellFrame) - (NSHeight(cellFrame) / 2) + (imageSize.height / 2);
    	compositePoint.x += (NSWidth(cellFrame) / 2) - (imageSize.width / 2);
        
		[sourceImage compositeToPoint:compositePoint operation:NSCompositeSourceOver fraction:1.0];
    }
}

@end
