/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyGen.h,v 1.14 2002/09/22 08:00:01 wprice Exp $
____________________________________________________________________________*/
#import <Cocoa/Cocoa.h>
#import <PGP/pgpKeys.h>
#import <PGPclient/PGPOpenPrefs.h>
#import <PGPclient/pgpReconstruct.h>
#import <PGPui/CPGPPassphraseTextView.h>

@class CWhiteView;

@protocol PGPKeyGen
- (PGPPrefRef)pgpPrefRef;
- (PGPtlsContextRef)tlsContext;
- (void)endKeyGen:(id)keysView success:(BOOL)generated;
@end

@protocol PGPKeyGenView
- (PGPKeyDBObjRef)lookupDefaultKey;
@end

typedef enum PGPGenViewID
{
	kPGPGenView_Invalid		= 0,
	kPGPGenView_Intro,
	kPGPGenView_Expert,
	kPGPGenView_Name,
	kPGPGenView_CorpWarn,
	kPGPGenView_Passphrase,
	kPGPGenView_PassError,
	kPGPGenView_Recon,
	kPGPGenView_Random,
	kPGPGenView_Generate,
	kPGPGenView_Conclusion,
	kPGPGenView_Exit
} PGPGenViewID;

@interface CKeyGen : NSObject <PGPPassphraseViewDelegate>
{
    IBOutlet NSWindow				*mKeyGenWindow;
	IBOutlet NSButton				*mNextButton;
    IBOutlet NSButton				*mPrevButton;
    IBOutlet NSBox					*mPanelBox;
	IBOutlet NSImageView			*mKeyImageView;
    IBOutlet NSTextField			*mPanelTitleField;
	
    IBOutlet NSTextField			*mIntroStateField;
    IBOutlet NSTextField			*mSetupStateField;
    IBOutlet NSTextField			*mPassphraseStateField;
    IBOutlet NSTextField			*mRandomStateField;
    IBOutlet NSTextField			*mGenerationStateField;
    IBOutlet NSTextField			*mConclusionStateField;
	
	IBOutlet NSView					*mIntroView;
	IBOutlet NSView					*mExpertView;
	IBOutlet NSView					*mNameView;
	IBOutlet NSView					*mCorpWarnView;
	IBOutlet NSView					*mPassphraseView;
	IBOutlet NSView					*mPhraseErrorView;
	IBOutlet NSView					*mRandomView;
	IBOutlet NSView					*mGenerationView;
	IBOutlet NSView					*mConclusionView;
	IBOutlet NSView					*mReconView;
	
	// Intro Panel
    IBOutlet NSButton				*mExpertButton;
	
	// Expert Panel
    IBOutlet NSTextField			*mExpertFullName;
    IBOutlet NSTextField			*mExpertEmail;
    IBOutlet NSTextField			*mExpertSize;
    IBOutlet NSTextField			*mExpertSizeRange;
    IBOutlet NSTextField			*mExpertExpiration;
	IBOutlet NSPopUpButton			*mExpertAlgPopup;
	IBOutlet NSMatrix				*mExpertExpirationMatrix;
	
	// Name Panel
    IBOutlet NSTextField			*mNameFullName;
    IBOutlet NSTextField			*mNameEmail;
	
	// CorpWarn Panel
	IBOutlet NSBox					*mCorpSigningBox;
	IBOutlet NSBox					*mCorpRevokerBox;
	IBOutlet NSBox					*mCorpADKBox;
	IBOutlet NSTextField			*mCorpSigningField;
	IBOutlet NSTextField			*mCorpRevokerField;
	IBOutlet NSTextField			*mCorpADKField;
	
	// Passphrase Panel
	IBOutlet NSScrollView			*mPassphraseScrollView;
	IBOutlet NSScrollView			*mPassConfirmationScrollView;
	IBOutlet NSProgressIndicator	*mPassQualityIndicator;
	IBOutlet NSProgressIndicator	*mPassMinQualityIndicator;
	IBOutlet NSTextField			*mPassMinQualityField;
	IBOutlet NSTextField			*mPassCapsLockField;
	IBOutlet NSTextField			*mPassMinLengthField;
	IBOutlet NSButton				*mPassHideTypingCheckbox;
	CPGPPassphraseTextView*			mPassphraseField;
	CPGPPassphraseTextView*			mSpaceField;
	CPGPPassphraseTextView*			mConfirmationPassphraseField;
	CPGPPassphraseTextView*			mConfirmationSpaceField;
	unsigned char					mSpaceCountList[kMaxPassphraseLength];
	
	// Phrase Error Panel
	IBOutlet NSBox					*mPEConfirmationBox;
	IBOutlet NSBox					*mPELengthBox;
	IBOutlet NSBox					*mPEQualityBox;
	IBOutlet NSProgressIndicator	*mPEQualityIndicator;
	IBOutlet NSTextField			*mPEMinLengthField;
	
	// Random Panel
	IBOutlet NSProgressIndicator	*mRandomProgress;
	IBOutlet NSTextField			*mRandomKeyboardField;
	
	// Generation Panel
	IBOutlet NSTextField			*mGenStatusField;
	IBOutlet NSTextField			*mGenStep1Field;
	IBOutlet NSTextField			*mGenStep2Field;
	IBOutlet NSTextField			*mGenStep3Field;
	IBOutlet NSTextField			*mGenStep4Field;
	IBOutlet NSTextField			*mGenContinueField;
	IBOutlet NSImageView			*mGenStep1Image;
	IBOutlet NSImageView			*mGenStep2Image;
	IBOutlet NSImageView			*mGenStep3Image;
	IBOutlet NSImageView			*mGenStep4Image;
	IBOutlet NSProgressIndicator	*mGenCurStatusIndicator;
	IBOutlet NSProgressIndicator	*mGenOverallIndicator;
	IBOutlet NSButton				*mGenCancelButton;
	
	// Recon Panel
	IBOutlet NSTextField			*mReconQ1Field;
	IBOutlet NSTextField			*mReconQ2Field;
	IBOutlet NSTextField			*mReconQ3Field;
	IBOutlet NSTextField			*mReconQ4Field;
	IBOutlet NSTextField			*mReconQ5Field;
	IBOutlet NSSecureTextField		*mReconA1Field;
	IBOutlet NSSecureTextField		*mReconA2Field;
	IBOutlet NSSecureTextField		*mReconA3Field;
	IBOutlet NSSecureTextField		*mReconA4Field;
	IBOutlet NSSecureTextField		*mReconA5Field;

	id <PGPKeyGen>					mOwnerController;
	id <PGPKeyGenView>				mKeysView;
	BOOL							mInited;
	BOOL							mLoadedNib;
	BOOL							mPassedKeyCheck;
	BOOL							mAddedKey;
	BOOL							mGenerating;
	NSTimer							*mEntropyTimer;
	PGPContextRef					mPGPContext;
	PGPKeyDBRef						mKeyDB;
	PGPKeyDBRef						mGeneratedKeyDB;
	PGPError						mKeyGenError;
	PGPPrefRef						mPrefRef;
	PGPGenViewID					mCurrentView;
	PGPBoolean						mExpert,
									mPreset,
									mRecon,
									mEnforceMinLength,
									mEnforceQuality;
	PGPInt32						mMinPassLength,
									mMinPassQuality,
									mNeededEntropy;
	PGPKeyDBObjRef					mADKKey;
	PGPKeyDBObjRef					mCorpKey;
	PGPKeyDBObjRef					mRootCAKey;
	PGPKeyDBObjRef					mRevokerKey;
	PGPReconPrompts					mReconPrompts;
	PGPReconPasses					mReconPasses;
	Point							mLastMouseLocation;
}

- (id)init;
- (id)initWithNib:(NSString*)nibFile;
- (id)initWithController:(id)controller keysView:(id)keysView forKeyDB:(PGPKeyDBRef)mainKeyDB;
- (IBAction)expertPressed:(id)sender;
- (IBAction)algorithmChanged:(id)sender;
- (IBAction)hideTypingPressed:(id)sender;
- (IBAction)switchPanelAction:(id)sender;
- (IBAction)cancelKeyGen:(id)sender;
- (NSWindow *)window;
- (BOOL)hasNeededEntropy;
- (BOOL)corpKeysAreValid;
- (void)switchToView:(PGPGenViewID)newView increment:(BOOL)increment;
- (BOOL)leaveView:(PGPGenViewID)newView;
- (void)enterView:(PGPGenViewID)viewID;
- (BOOL)phraseIsGood;
- (BOOL)phraseAttributes:(BOOL *)noMatch tooShort:(BOOL *)tooShort lowQuality:(BOOL *)lowQuality;
- (BOOL)getExpertParams:(PGPPublicKeyAlgorithm *)algorithm
	size:(PGPUInt32 *)size
	expires:(PGPUInt32 *)expires;
- (void)addEntropy:(NSTimer*)timer;
- (void)updatePassphraseQuality:(PGPUInt32)quality sender:(id)sender;

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldInsertText:(NSString*)string;
- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldDeleteBackward:(id)sender;
- (BOOL)shouldPassphraseViewDisplayMarkedText:(CPGPPassphraseTextView*)passphraseView;
- (void)capsLockStatusDidChange:(BOOL)enabled;

@end

