/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeySelect.m,v 1.14 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/
#import "CKeySelect.h"
#import "CKeyView.h"
#import "pgpClientPrefs.h"
#import "pgpClientLibCocoa.h"

#define kCKeySelectNib		@"CKeySelect"

	void
PGPclBeginKeySelectSheet(
	NSString		*prompt,
	NSWindow		*docWindow,
	PGPPrefRef		prefRef,
	PGPKeySetRef	selectFromSet,
	PGPKeyDBRef		mainDB,
	PGPUInt32		flags,
	id				owner,
	SEL				didEndSelector )
{
	CKeySelect		*keySelector;

	keySelector = [[CKeySelect alloc] initKeySelect:prompt
					fromSet:selectFromSet
					mainDB:mainDB
					prefRef:prefRef
					usingFlags:flags
					owner:owner
					didEndSelector:didEndSelector];
	[keySelector runKeySelectModalTo:docWindow];
	[keySelector release];
}

@implementation CKeySelect

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeySelectNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initKeySelect:(NSString *)prompt
		fromSet:(PGPKeySetRef)keySet
		mainDB:(PGPKeyDBRef)mainKeyDB
		prefRef:(PGPPrefRef)prefRef
		usingFlags:(PGPUInt32)flags
		owner:(id)owner
		didEndSelector:(SEL)didEndSelector;
{
	PGPSize					len;
	PGPclKeyListColumnInfo	*prefColumnInfo,
							columnInfo;

	[self init];
	mFlags = flags;
	mPrefRef = prefRef;
	mKeySet = keySet;
	mMainDB = mainKeyDB;
	mOwner = owner;
	PGPNewTLSContext( PGPPeekKeySetContext( keySet ), &mTLSContext );
	mEndSelector = didEndSelector;
	mReturnCode = kPGPError_NoErr;
	[mInfoText setStringValue:prompt];
	if( mFlags & kPGPclKeySelect_SingleKey )
	{
		[mSelectAllButton setTransparent:YES];
		[mSelectNoneButton setTransparent:YES];
	}
	mCKeyView = [[CKeyView alloc] initWithController:self];
	[[mCKeyView view] setFrameSize:[mKeyView frame].size];
	[mKeyView addSubview:[mCKeyView view]];
	[mCKeyView setFlags: kPGPclKeyList_ReadOnly | kPGPclKeyList_SelectKeysOnly |
		( ( mFlags & kPGPclKeySelect_SingleKey ) ? kPGPclKeyList_SingleSelect : 0 )];
	len = sizeof(PGPclKeyListColumnInfo);
	if( IsPGPError( PGPGetPrefData( mPrefRef,
		kPGPPrefPGPkeysMacColumnData, &len, (void **)&prefColumnInfo ) ) )
	{
		pgpCopyMemory( &kDefaultColumnInfo, &columnInfo, sizeof(PGPclKeyListColumnInfo) );
	}
	else
	{
		pgpCopyMemory( prefColumnInfo, &columnInfo, sizeof(PGPclKeyListColumnInfo) );
		PGPFreeData(prefColumnInfo);
	}
	[mCKeyView setColumns:columnInfo];
	if( PGPKeySetRefIsValid( keySet ) )
	{
		if( PGPKeyDBRefIsValid( mainKeyDB ) &&
			IsntPGPError( PGPCheckKeyRingSigs( keySet, mainKeyDB, TRUE, NULL, 0 ) ) )
			PGPCalculateTrust( keySet, mainKeyDB );
		[mCKeyView loadKeys:PGPPeekKeySetKeyDB(keySet) showKeySet:keySet];
	}
	if( mFlags & kPGPclKeySelect_ImportVariation )
		[mCKeyView selectAll:self];
	else
		[mImportButton setEnabled:NO];
	return self;
}

- (void)keySelectSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
	mReturnCode = returnCode;
	[self autorelease];
}

- (void)runKeySelectModalTo:(NSWindow *)window
{
    mIsSheet = YES;
	[[NSApplication sharedApplication] beginSheet:mKeySelectWindow modalForWindow:window
		modalDelegate:self didEndSelector:@selector(keySelectSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
	[self retain];
}

- (PGPError)runKeySelectModalDialog:(PGPKeySetRef *)ioSelectedKeys
{
    mIsSheet = NO;
	mReturnCode = [[NSApplication sharedApplication] runModalForWindow:mKeySelectWindow];
	[mKeySelectWindow close];
    if( IsntPGPError( mReturnCode ) )
        *ioSelectedKeys = [mCKeyView getEnumeratorKeySet:[mCKeyView selectedItemEnumerator]];
    return( mReturnCode );
}

- (void)selectionChanged
{
	BOOL		multiple;
	PGPUInt32	selectFlags;
	
	if( PGPKeyDBObjRefIsValid( [mCKeyView selectionInfo:&multiple withFlags:&selectFlags] ) )
		[mImportButton setEnabled:YES];
	else
		[mImportButton setEnabled:NO];
}

- (void)validityButtonPressed:(id)sender
{
}

- (void)becameKey:(CKeyView *)keyView
{
}

- (void)resignedKey:(CKeyView *)keyView
{
}

- (IBAction)cancelPressed:(id)sender
{
    if( mIsSheet )
        [[NSApplication sharedApplication] endSheet:mKeySelectWindow returnCode:kPGPError_UserAbort];
    else
        [[NSApplication sharedApplication] stopModalWithCode:kPGPError_UserAbort];
}

- (IBAction)okPressed:(id)sender
{
    if( mIsSheet )
        [[NSApplication sharedApplication] endSheet:mKeySelectWindow returnCode:kPGPError_NoErr];
    else
        [[NSApplication sharedApplication] stopModalWithCode:kPGPError_NoErr];
}

- (IBAction)selectAllPressed:(id)sender
{
	[mCKeyView selectAll:self];
}

- (IBAction)selectNonePressed:(id)sender
{
	[mCKeyView deselectAll:self];
}

- (PGPPrefRef)pgpPrefRef
{
	return mPrefRef;
}

- (PGPtlsContextRef)tlsContext
{
	return mTLSContext;
}

- (PGPKeyDBRef)mainDB
{
	return mMainDB;
}

- (void)dealloc
{
	NSMethodSignature*		methodSignature;
	NSInvocation*			methodInvocation;
	PGPKeySetRef			keySet = kInvalidPGPKeySetRef;
	
	if( IsntPGPError( mReturnCode ) )
		keySet = [mCKeyView getEnumeratorKeySet:[mCKeyView selectedItemEnumerator]];
	[mCKeyView release];
	if( PGPKeySetRefIsValid( mKeySet ) && ( mFlags & kPGPclKeySelect_FreeKeySet ) )
		PGPFreeKeySet( mKeySet );
	PGPFreeTLSContext( mTLSContext );
	if( mEndSelector && [mOwner respondsToSelector:mEndSelector] )
	{
		methodSignature = [mOwner methodSignatureForSelector:mEndSelector];
		methodInvocation = [NSInvocation invocationWithMethodSignature:methodSignature];
		[methodInvocation setSelector:mEndSelector];
		[methodInvocation setTarget:mOwner];
		[methodInvocation setArgument:&mReturnCode atIndex:2];
		[methodInvocation setArgument:&keySet atIndex:3];
		[methodInvocation invoke];
	}
	[super dealloc];
	PGPLog(( @"CKeySelect dealloc" ));
}

@end

