/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPPrefObj.m,v 1.4 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import "CPGPPrefObj.h"
#import <PGPclient/pgpClientLibCocoa.h>

@implementation CPGPPrefObj

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}

	return self;
}

- (id)initWithControl:(id)control prefId:(PGPPrefIndex)pref prefType:(PGPPrefType)type target:(id)target getter:(SEL)getter setter:(SEL)setter;
{
	mControl = control;
	mPrefId = pref;
	mPrefType = type;
	mTarget = target;
	mGetSelector = getter;
	mSetSelector = setter;

	return [self init];
}

- (id)initWithControl:(id)control prefId:(PGPPrefIndex)pref prefType:(PGPPrefType)type target:(id)target getter:(SEL)getter setter:(SEL)setter notification:(NSString*)notification
{
	mNotification = notification;
	
	return [self initWithControl:control prefId:pref prefType:type target:target getter:getter setter:setter];
}

- (id)control
{
	return mControl;
}

- (void)setControl:(id)control
{
	mControl = control;
}

- (PGPPrefIndex)prefId
{
	return mPrefId;
}

- (void)setPrefId:(PGPPrefIndex)pref
{
	mPrefId = pref;
}

- (PGPPrefType)prefType
{
	return mPrefType;
}

- (void)setPrefType:(PGPPrefType)type
{
	mPrefType = type;
}

- (id)target
{
	return mTarget;
}

- (void)setTarget:(id)target
{
	mTarget = target;
}

- (NSString*)notification
{
	return mNotification;
}

- (void)setNotification:(NSString*)notification
{
	if (mNotification)
		[mNotification release];
		
	if (notification)
		mNotification = [notification retain];
	else
		mNotification = nil;
}

- (void)updateControlWithData:(void*)data size:(PGPSize)size
{
	NSInvocation*		setInvocation;
	NSMethodSignature*	methodSig;
	
	if (!data)
		PGPLog((@"%s:%d: Invalid data specified for prefId %d", __FILE__, __LINE__, [self prefId]));
	else if (mTarget && mSetSelector)
	{
		if (methodSig = [mTarget methodSignatureForSelector:mSetSelector])
		{
			setInvocation = [NSInvocation invocationWithMethodSignature:methodSig];
			[setInvocation setSelector:mSetSelector];
			[setInvocation setTarget:mTarget];
			[setInvocation setArgument:&mControl atIndex:2];
			[setInvocation setArgument:&data atIndex:3];
			[setInvocation setArgument:&size atIndex:4];
			
			[setInvocation invoke];
		}
		else
			PGPLog((@"%s:%d: Unable to get method signature for selector", __FILE__, __LINE__));
	}
	else
		PGPLog((@"%s:%d: Invalid target or set selector", __FILE__, __LINE__));
}

- (void)retrieveControlData:(void**)data size:(PGPSize)size
{
	NSInvocation*		getInvocation;
	NSMethodSignature*	methodSig;
	if (!data)
		PGPLog((@"%s:%d: Invalid data specified for prefId %d", __FILE__, __LINE__, [self prefId]));
	else if (mTarget && mGetSelector)
	{
		if (methodSig = [mTarget methodSignatureForSelector:mGetSelector])
		{
			getInvocation = [NSInvocation invocationWithMethodSignature:methodSig];
			[getInvocation setSelector:mGetSelector];
			[getInvocation setTarget:mTarget];
			[getInvocation setArgument:&data atIndex:2];
			[getInvocation setArgument:&mControl atIndex:3];
			[getInvocation setArgument:&size atIndex:4];
			
			[getInvocation invoke];
		}
		else
			PGPLog((@"%s:%d: Unable to get method signature for selector", __FILE__, __LINE__));
	}
	else
		PGPLog((@"%s:%d: Invalid target or get selector", __FILE__, __LINE__));
}

- (void)dealloc
{
	if (mNotification)
		[mNotification release];
		
	[super dealloc];
}

@end
