/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPViewer.m,v 1.19 2002/11/17 23:51:52 wprice Exp $
____________________________________________________________________________*/

#import <stdlib.h>

#import "pgpPFLPriv.h"
#import <PGPclient/pgpClientLibCocoa.h>
#import "CRemoteAuth.h"
#import "CPGPViewer.h"

#define kPGPViewerNib			@"CPGPViewer"

#define kFYEOWarningStr						LSCL(@"The message you are decrypting is for your eyes only. It is recommended that this message only be read under the most secure circumstances.", @"FYEO Warning")
#define kCautionStr							LSCL(@"Caution", @"")
#define kOKStr								LSCL(@"OK", @"")
#define kCancelStr							LSCL(@"Cancel", @"")

#define kTempestFontRows					12
#define	kTempestFontColumns					16

	PGPError
PGPclForYourEyesOnlyDialog(
	PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
	char *						inString)
{
	PGPError			theError = kPGPError_NoErr;
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
    int					alertResult = 0;

	PGPValidatePtr(inString);
    
    alertResult = NSRunAlertPanel(kCautionStr, kFYEOWarningStr, kOKStr, kCancelStr, nil );
    
    if ( NSAlertDefaultReturn == alertResult )
    {
        CPGPViewer*		theDialog = nil;

        theDialog = [[CPGPViewer alloc] init];
        if( nil != theDialog )
        {
            [theDialog setTitle:LSCL(@"Secure Viewer", @"")];
            theError = [theDialog setInfo:inContext prefRef:inPrefRef data:inString fyeo:YES];
            if( IsntPGPError(theError) )
                [theDialog displayIfNeeded];
            
            [theDialog release];
        } else
            theError = kPGPError_OutOfMemory;
    }
    
	[pool release];
	return theError;
}

	PGPError
PGPclOutputDialog(
	PGPContextRef	inContext,
    PGPPrefRef		inPrefRef,
	const char *	inString)
{
	PGPError			theError = kPGPError_NoErr;
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
    CPGPViewer*			theDialog = nil;
    
	PGPValidatePtr(inString);
    
    theDialog = [[CPGPViewer alloc] init];
    if ( nil != theDialog )
    {
        [theDialog setTitle:LSCL(@"PGP", @"")];
        theError = [theDialog setInfo:inContext prefRef:inPrefRef data:inString fyeo:NO];
        if (IsntPGPError(theError))
            [theDialog displayIfNeeded];
        
        [theDialog release];
    } else
        theError = kPGPError_OutOfMemory;

	[pool release];
	return theError;
}

@implementation CPGPViewer

- (id)init
{
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }
    
    if( !mLoadedNib )
        [self initWithNib:kPGPViewerNib];

    return self;
}

- (id)initWithNib:(NSString *)nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
    
	[mTextScrollView retain];
	[mTempestScrollView retain];
	
	mLoadedNib = YES;

    return [self init];
}

- (void)dealloc
{
	[mTextScrollView release];
	[mTempestScrollView release];
	
	[mTempestFontImageArray release];
    [super dealloc];
}

- (BOOL)windowShouldClose:(NSWindow *)sender
{
    [self closeViewer:sender];
    return YES;
}

- (IBAction)closeViewer:(id)sender
{
    [NSApp stopModal];
	[mWindow setDelegate:nil];
    [mWindow close];
}

- (IBAction)switchView:(id)sender
{
    if( NSOnState == [mFontSwitch state])
    {
        mUseTempest = YES;
        PGPSetPrefBoolean( mPrefRef, kPGPPrefTempestFont, TRUE );
		[self switchToTempestView];
    }
    else
    {
        mUseTempest = NO;
        PGPSetPrefBoolean( mPrefRef, kPGPPrefTempestFont, FALSE );
		[self switchToTextView];
    }
}


- (void)switchToTextView
{
	[mTempestScrollView removeFromSuperview];
	[[mWindow contentView] addSubview:mTextScrollView];
	[mWindow makeFirstResponder:[mWindow contentView]];
}

- (void)switchToTempestView
{
	[mTextScrollView removeFromSuperview];
	[[mWindow contentView] addSubview:mTempestScrollView];
	[mWindow makeFirstResponder:[mWindow contentView]];
}

- (PGPError)setInfo:(PGPContextRef)inContext
    prefRef:(PGPPrefRef)inPrefRef
    data:(const char *)data
    fyeo:(BOOL)fyeo
{
    PGPError	theError = kPGPError_NoErr;

	mContext = inContext;
    mPrefRef = inPrefRef;
    mOutputString = [NSString stringWithUTF8String:data];
	[mTextView setString:mOutputString];
    
	if( !fyeo )
	{
		[mFontSwitch setEnabled:NO];
        [mFontSwitch setTransparent:YES];
        mUseTempest = NO;
		[self switchToTextView];
    }
	else
	{
		PGPBoolean			useTempest;
		NSSize				contentSize = [mTempestScrollView contentSize];
		NSArray *			lines;
		NSEnumerator *		enumerator;
		NSData *			latin1Bytes;
		NSImage *			img;
		const PGPByte *		cp;
		NSRect				frame = [mTempestView frame];
		NSSize				imgSize;
		NSPoint				cursor;
		unsigned			lineLength;
		NSImage *			fontImage;
		int					fontHeight;
		int					fontWidth;
		PGPUInt32 *			pixmap;
		NSImage *			noiseImage;
		PGPUInt32			numPixels;
		PGPUInt8			noise;
		PGPUInt8			alpha;
		PGPUInt32			randNum;
		PGPUInt32			x;
		NSRect				windowFrame = [mWindow frame];
		
		srandom(clock());
		[mWindow setShowsResizeIndicator:NO];
		[mWindow setMaxSize:windowFrame.size];
		[mWindow setMinSize:windowFrame.size];
		mTempestFontImageArray = [[NSArray alloc] initWithObjects:
			[[NSImage alloc] initByReferencingFile:[[NSBundle bundleForClass:[CPGPViewer class]] pathForImageResource:@"tempestfonta"]],
			[[NSImage alloc] initByReferencingFile:[[NSBundle bundleForClass:[CPGPViewer class]] pathForImageResource:@"tempestfontb"]],
			[[NSImage alloc] initByReferencingFile:[[NSBundle bundleForClass:[CPGPViewer class]] pathForImageResource:@"tempestfontc"]],
			[[NSImage alloc] initByReferencingFile:[[NSBundle bundleForClass:[CPGPViewer class]] pathForImageResource:@"tempestfontd"]],
			nil];
		fontHeight = [[mTempestFontImageArray objectAtIndex:0] size].height / kTempestFontRows;
		fontWidth = [[mTempestFontImageArray objectAtIndex:0]size].width / kTempestFontColumns;
		[mTempestScrollView setVerticalLineScroll:fontHeight];
		[mTempestScrollView setVerticalPageScroll:fontHeight];
		lines = [self wrapString:mOutputString withMaxChars:contentSize.width / fontWidth];
		enumerator = [lines objectEnumerator];
		img = [[NSImage alloc] initWithSize:NSMakeSize(contentSize.width, MAX([lines count] * fontHeight, contentSize.height))];
		imgSize = [img size];
		cursor = NSMakePoint(0, imgSize.height - fontHeight);
		[img lockFocus];
		while (latin1Bytes = [[enumerator nextObject] dataUsingEncoding:NSISOLatin1StringEncoding allowLossyConversion:YES]) {
			lineLength = [latin1Bytes length];
			cp = [latin1Bytes bytes];
			while (lineLength > 0) {
				fontImage = [mTempestFontImageArray objectAtIndex:(random() & 0x00000003)];
				[fontImage drawInRect:NSMakeRect(cursor.x, cursor.y, fontWidth, fontHeight)
					fromRect:NSMakeRect(((*cp - 32) % kTempestFontColumns) * fontWidth,
					[fontImage size].height - ((((*cp - 32) / kTempestFontColumns) + 1) * fontHeight),
					fontWidth, fontHeight) operation:NSCompositeSourceOver fraction:1];
				cp++;
				lineLength--;
				cursor.x += fontWidth;
			}
			cursor.x = 0;
			cursor.y -= fontHeight;
		}
		
		/* Add noise */
		numPixels = imgSize.width * imgSize.height;
		pixmap = PGPNewData(PGPPeekContextMemoryMgr(mContext), numPixels * sizeof(PGPUInt32), kPGPMemoryMgrFlags_None);
		for (x = 0; (x + 2) < numPixels; x+=3) {
			randNum = random();
			noise = (randNum >> 8) & 0x000000FF;
			alpha = ((noise ^ randNum) & 0x0000003F) ? 0x00 : 0xFF;
			pixmap[x] = (((((noise << 8) + noise) << 8) + noise) << 8) + alpha;
			noise = (randNum >> 16) & 0x000000FF;
			alpha = ((noise ^ randNum) & 0x0000005F) ? 0x00 : 0xFF;
			pixmap[x+1] = (((((noise << 8) + noise) << 8) + noise) << 8) + alpha;
			noise = (randNum >> 24) & 0x000000FF;
			alpha = ((noise ^ randNum) & 0x0000006F) ? 0x00 : 0xFF;
			pixmap[x+2] = (((((noise << 8) + noise) << 8) + noise) << 8) + alpha;
		}
		if (x < numPixels) {
			randNum = random();
			noise = (randNum >> 8) & 0x000000FF;
			alpha = ((noise ^ randNum) & 0x0000003F) ? 0x00 : 0xFF;
			pixmap[x] = (((((noise << 8) + noise) << 8) + noise) << 8) + alpha;
			if ((x + 1) < numPixels) {
				noise = (randNum >> 16) & 0x000000FF;
				alpha = ((noise ^ randNum) & 0x0000005F) ? 0x00 : 0xFF;
				pixmap[x+1] = (((((noise << 8) + noise) << 8) + noise) << 8) + alpha;
			}
		}
		noiseImage = [self imageFromPixmap:pixmap width:imgSize.width height:imgSize.height];
		PGPFreeData(pixmap);
		[noiseImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver];
		[noiseImage release];
		[img unlockFocus];
		[mTempestView setImage:img];
		[img release];
		[mTempestView setFrame:NSMakeRect(frame.origin.x, frame.origin.y + frame.size.height - imgSize.height, frame.size.width,
			imgSize.height)];
		[mTempestView setNeedsDisplay:YES];
		PGPGetPrefBoolean( mPrefRef, kPGPPrefTempestFont, &useTempest );
		if( !useTempest )
		{
            [mFontSwitch setState:NSOffState];
            mUseTempest = NO;
			[self switchToTextView];
		}
        else
        {
            [mFontSwitch setState:NSOnState];
            mUseTempest = YES;
			[self switchToTempestView];
        }
	}
    return theError;
}

- (void)displayIfNeeded
{
    if( ![mWindow isVisible] );
        [NSApp runModalForWindow:mWindow];
}

- (void)setTitle:(NSString *)title
{
    [mWindow setTitle:title];
}


- (NSArray *)wrapString:(NSString *)string withMaxChars:(PGPUInt32)maxChars
{
	NSMutableArray *	result = [NSMutableArray array];
	NSCharacterSet *	spaceAndNLSet = [NSCharacterSet whitespaceAndNewlineCharacterSet];
	NSCharacterSet *	spaceSet = [NSCharacterSet whitespaceCharacterSet];
	NSMutableString *	s = [NSMutableString stringWithString:string];
	NSArray *			linesByLF;
	NSEnumerator *		enumerator;
	NSString *			line;
	NSRange				r;
	
	[s replaceOccurrencesOfString:@"\r\n" withString:@"\n" options:0 range:NSMakeRange(0, [s length])];
	[s replaceOccurrencesOfString:@"\r" withString:@"\n" options:0 range:NSMakeRange(0, [s length])];
	[s replaceOccurrencesOfString:@"\t" withString:@"    " options:0 range:NSMakeRange(0, [s length])];
	linesByLF = [s componentsSeparatedByString:@"\n"];
	enumerator = [linesByLF objectEnumerator];
	while (line = [enumerator nextObject]) {
		r = NSMakeRange(0, [line length]);
		while ((r.length > 0) && [spaceAndNLSet characterIsMember:[line characterAtIndex:r.length - 1]]) {
			r.length--;
		}
		line = [line substringWithRange:r];
		while ([line length] > maxChars) {
			r = [line rangeOfCharacterFromSet:spaceSet options:NSBackwardsSearch range:NSMakeRange(0, maxChars)];
			if (r.location == NSNotFound) {
				r = NSMakeRange(0, maxChars);
			} else {
				r = NSMakeRange(0, r.location);
				while ((r.length > 0) && [spaceSet characterIsMember:[line characterAtIndex:r.length - 1]]) {
					r.length--;
				}
			}
			[result addObject:[line substringWithRange:r]];
			while ([spaceSet characterIsMember:[line characterAtIndex:r.length]]) {
				r.length++;
			}
			line = [line substringFromIndex:r.length];
		}
		[result addObject:line];
	}
	
	return result;
}


- (NSImage *)imageFromPixmap:(PGPUInt32 *)pixmapData width:(int)pixmapWidth height:(int)pixmapHeight
{
	NSImage *			result;
	NSBitmapImageRep *	bitmap;
	PGPByte *			pixels;
	
	bitmap = [[NSBitmapImageRep alloc]
				initWithBitmapDataPlanes: NULL // Let the class allocate it
				pixelsWide: pixmapWidth
				pixelsHigh: pixmapHeight
				bitsPerSample: 8 // Each component is 8 bits (one byte)
				samplesPerPixel: 4 // Number of components (R, G, B, alpha)
				hasAlpha: YES
				isPlanar: NO
				colorSpaceName: NSDeviceRGBColorSpace
				bytesPerRow: 0 // 0 means: Let the class figure it out
				bitsPerPixel: 0 // 0 means: Let the class figure it out
			];
	pixels = [bitmap bitmapData];
	memcpy(pixels, pixmapData, pixmapHeight * pixmapWidth * sizeof(PGPUInt32));
	result = [[NSImage alloc] initWithSize:NSMakeSize(pixmapWidth, pixmapHeight)];
	[result addRepresentation: bitmap];

	return result;
}

@end
