/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPassphraseField.m,v 1.4 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import "CPassphraseField.h"
#import <PGPclient/pgpClientLibCocoa.h>

static CPassphraseFieldEditor* sFieldEditor = nil;

@implementation CPassphraseField

/*
Because several CPassphraseFields can use the same formatter (CPassphraseFormatter),
we must ensure that the formatter knows which CPassphraseField it will be formatting.
*/
- (BOOL)becomeFirstResponder
{
	if ([[self cell] formatter])
		[[[self cell] formatter] setPassphraseField:self];
	
	[[CPassphraseFieldEditor sharedFieldEditor] setRepresentedTextField:self];
	
	return [super becomeFirstResponder];
}

- (NSProgressIndicator*)progressIndicator
{
	return mProgressIndicator;
}

- (void)setProgressIndicator:(NSProgressIndicator*)progressIndicator
{
	if (progressIndicator)
	{
		if (mProgressIndicator)
			[mProgressIndicator release];

		mProgressIndicator = [progressIndicator retain];
	}
}
	
- (BOOL)isTypingHidden
{
	return mTypingHidden;
}

- (void)setTypingHidden:(BOOL)flag
{
	mTypingHidden = flag;
}

- (NSString*)passphrase
{
	return mActualPassphraseString;
}

- (void)addCharacter:(unichar)character
{
	if (!mActualPassphraseString)
		mActualPassphraseString = [[NSMutableString alloc] initWithCapacity:32];
	
	[mActualPassphraseString appendFormat:@"%c", character];
	
	if (!mHiddenPassphraseString)
		mHiddenPassphraseString = [[NSMutableString alloc] initWithCapacity:32];
	
	[mHiddenPassphraseString appendString:@" "];
	
	if (mProgressIndicator)
		[mProgressIndicator incrementBy:5.0];
}

- (void)deleteCharacter
{
	NSRange		range;
	
	if (mActualPassphraseString)
	{
		range.location = [mActualPassphraseString length] ? [mActualPassphraseString length] - 1 : 0;
		range.length = 1;
		
		[mActualPassphraseString deleteCharactersInRange:range];
	}
	
	if (mHiddenPassphraseString)
	{
		range.location = [mHiddenPassphraseString length] ? [mHiddenPassphraseString length] - 1 : 0;
		range.length = 1;
		
		[mHiddenPassphraseString deleteCharactersInRange:range];
	}
	
	if (mProgressIndicator)
		[mProgressIndicator incrementBy:-5.0];
}

- (unsigned)lastStringLength
{
	return mLastStringLength;
}

- (void)updateStringLength
{
	if (mActualPassphraseString)
		mLastStringLength = [mActualPassphraseString length];
}

- (void)displayHiddenText
{
	if (mActualPassphraseString)
		[self setStringValue:mActualPassphraseString];
}

- (void)hideText
{
	if (mHiddenPassphraseString)
		[self setStringValue:mHiddenPassphraseString];
}

- (void)clearCurrentPassphrase
{
	if (mHiddenPassphraseString)
	{
		[mHiddenPassphraseString release];
		mHiddenPassphraseString = nil;
	}
	
	if (mActualPassphraseString)
	{
		[mActualPassphraseString release];
		mActualPassphraseString = nil;
	}
	
	mLastStringLength = 0;
}

- (id)windowWillReturnFieldEditor:(NSWindow*)window toObject:(id)object
{
	PGPLog((@"windowWillReturnFieldEditor"));
	if (!mFieldEditor)
		mFieldEditor = [[CPassphraseFieldEditor alloc] init];
		
	return mFieldEditor;
}

- (void)dealloc
{
	if (mHiddenPassphraseString)
		[mHiddenPassphraseString release];
		
	if (mActualPassphraseString)
		[mActualPassphraseString release];
		
	if (mProgressIndicator)
		[mProgressIndicator release];
		
	[super dealloc];
}

@end

@implementation CPassphraseFormatter

- (CPassphraseField*)passphraseField
{
	return mPassphraseField;
}

- (void)setPassphraseField:(CPassphraseField*)passphraseField
{
	if (passphraseField)
	{
		if (mPassphraseField)
			[mPassphraseField release];
		
		mPassphraseField = [passphraseField retain];
	}
}

- (NSString*)stringForObjectValue:(id)object
{
	if (!object)
		return nil;

	return object;
}

- (BOOL)getObjectValue:(id*)object forString:(NSString*)string errorDescription:(NSString**)error
{	
	*object = string;
	
	return YES;
}

- (NSAttributedString*)attributedStringForObjectValue:(id)object withDefaultAttributes:(NSDictionary*)attributes
{
	return nil;
}

- (BOOL)isPartialStringValid:(NSString*)string newEditingString:(NSString**)newString errorDescription:(NSString**)error
{
	unsigned			stringLength;
	unichar				character;
	NSMutableString*	tempString;
	int					i;

	if (!mPassphraseField)
	{
		PGPLog((@"%s:%d: Formatter doesn't have an associated passphrase field!", __FILE__, __LINE__));
		return YES;
	}
	
	if ([string length] <= [mPassphraseField lastStringLength])
	{
		[mPassphraseField deleteCharacter];
		[mPassphraseField updateStringLength];
		return YES;
	}
	
	character = [string characterAtIndex:([string length] ? [string length] - 1 : 0)];
	
	if (![mPassphraseField isTypingHidden])
	{
		[mPassphraseField addCharacter:character];
		[mPassphraseField updateStringLength];
		return YES;
	}
	
	*error = nil;
	
	if ((stringLength = [string length]) > 1)
	{
		tempString = [[NSMutableString alloc] initWithCapacity:stringLength];
		
		for (i = 0; i < stringLength; ++i)
			[tempString appendString:@" "];
			
		*newString = [[NSString alloc] initWithString:tempString];
		
		[tempString release];
	}
	
	else if (stringLength == 1)
		*newString = @" ";
	
	else if (!stringLength)
		*newString = @"";
		
	[mPassphraseField addCharacter:character];
	[mPassphraseField updateStringLength];
	
	return NO;
}

- (void)dealloc
{
	if (mPassphraseField)
		[mPassphraseField release];
	
	[super dealloc];
}

@end

@implementation CPassphraseFieldEditor

+ (id)sharedFieldEditor
{
	if (!sFieldEditor)
		sFieldEditor = [[CPassphraseFieldEditor alloc] init];
		
	return sFieldEditor;
}

- (void)setRepresentedTextField:(CPassphraseField*)field
{
	mField = field;
}

- (void)keyDown:(NSEvent*)theEvent
{
	NSString*		keyStr;
	unichar			key;
	BOOL			slurp = NO;
	
	keyStr = [theEvent charactersIgnoringModifiers];
	key = [keyStr characterAtIndex:0];

	if ((key == NSUpArrowFunctionKey) || (key == NSDownArrowFunctionKey) ||
		(key == NSLeftArrowFunctionKey) || (key == NSRightArrowFunctionKey))
		slurp = YES;	

	if (!slurp)
		[super keyDown:theEvent];
}

- (void)insertTab:(id)sender
{
	if (mField && [mField nextKeyView])
		[[self window] makeFirstResponder:[mField nextKeyView]];
}

- (void)cut:(id)sender
{
}

- (void)copy:(id)sender
{
}

- (void)paste:(id)sender
{
}

- (void)delete:(id)sender
{
}

- (void)selectAll:(id)sender
{
}

/*
This method gets called when the user tabs into our text field.
By default, the existing text is selected, but in our case it
will not be selected.
*/
- (void)setSelectedRange:(NSRange)charRange affinity:(NSSelectionAffinity)affinity stillSelecting:(BOOL)flag
{
	if (charRange.length)
		charRange.location = charRange.length;
	
	[super setSelectedRange:charRange affinity:affinity stillSelecting:flag];
}

/*
This method gets called on a mouseDown event. By default it
would place the insertion point wherever the mouseDown event
occurred. In our case, we always put the selection at the end
of the current string.
*/
- (NSRange)selectionRangeForProposedRange:(NSRange)proposed granularity:(NSSelectionGranularity)granularity
{
	NSString*		currentString = [self string];

	if (currentString)
	{
		proposed.location = [currentString length];
		proposed.length = 0;
	}
	
	return proposed;
}

@end