/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CRemoteAuth.m,v 1.7 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/
#import "CRemoteAuth.h"
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPui/CPGPIcons.h>

#define kCRemoteAuthNib		@"CRemoteAuth"

	PGPError
PGPclConfirmRemoteKeyDialog(
	const char				*prompt,
	const char				*remoteHostName,
	PGPKeyDBObjRef			keyToConfirm,
	PGPtlsCipherSuiteNum	tlsCipher,
	PGPBoolean				askConfirm )
{
	PGPError		theError = kPGPError_NoErr;
	CRemoteAuth		*remoteAuthController;

	remoteAuthController = [[CRemoteAuth alloc] initRemoteAuth:[NSString stringWithCString:prompt]
					hostName:[NSString stringWithCString:remoteHostName]
					authKey:keyToConfirm
					tlsCipher:tlsCipher
					confirm:askConfirm];
	
    theError = [remoteAuthController runRemoteAuthModalDialog];

    [remoteAuthController release];
	return( theError );
}

@implementation CRemoteAuth

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCRemoteAuthNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initRemoteAuth:(NSString *)prompt
		hostName:(NSString *)hostName
		authKey:(PGPKeyDBObjRef)authKey
		tlsCipher:(PGPtlsCipherSuiteNum)tlsCipher
		confirm:(BOOL)confirm
{
	char		cstr[kPGPcl_MaxPGPUserIDLength];
	PGPInt32	validity;
	
	[self init];
	[mCancelButton retain];
	[mPromptField setStringValue:prompt];
	[mAddressField setStringValue:hostName];
	if( !confirm )
		[mCancelButton removeFromSuperview];
	switch( tlsCipher )
	{
		case kPGPtls_TLS_PGP_DHE_DSS_WITH_CAST_CBC_SHA:
			[mSecurityField setStringValue:@"PGP\nDSS\nDiffie-Hellman (Ephemeral)\nCAST\nSHA"];
			break;
		case kPGPtls_TLS_PGP_DHE_RSA_WITH_CAST_CBC_SHA:
			[mSecurityField setStringValue:@"PGP\nRSA\nDiffie-Hellman (Ephemeral)\nCAST\nSHA"];
			break;
		case kPGPtls_TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA:
			[mSecurityField setStringValue:@"X.509\nDSS\nDiffie-Hellman (Ephemeral)\nTripleDES\nSHA"];
			break;
		case kPGPtls_TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA:
			[mSecurityField setStringValue:@"X.509\nRSA\nDiffie-Hellman (Ephemeral)\nTripleDES\nSHA"];
			break;
		case kPGPtls_TLS_RSA_WITH_3DES_EDE_CBC_SHA:
			[mSecurityField setStringValue:@"X.509\nRSA\nRSA\nTripleDES\nSHA"];
			break;
		case kPGPtls_TLS_RSA_WITH_IDEA_CBC_SHA:
			[mSecurityField setStringValue:@"X.509\nRSA\nRSA\nIDEA\nSHA"];
			break;
		case kPGPtls_TLS_PGP_RSA_WITH_CAST_CBC_SHA:
			[mSecurityField setStringValue:@"PGP\nRSA\nRSA\nCAST\nSHA"];
			break;
		case kPGPtls_TLS_PGP_DHE_DSS_WITH_NULL_SHA:
			[mSecurityField setStringValue:@"PGP\nDSS\nDiffie-Hellman (Ephemeral)\nNone\nSHA"];
			break;
		case kPGPtls_TLS_DHE_DSS_WITH_NULL_SHA:
			[mSecurityField setStringValue:@"X.509\nDSS\nDiffie-Hellman (Ephemeral)\nNone\nSHA"];
			break;
		default:
			break;
	}
	[mKeyField setStringValue:PGPclGetPrimaryUserIDName( authKey )];
	[mKeyImage setImage:[[CPGPIcons shared] getKeyDBObjIcon:authKey]];
	PGPclGetFingerprintString( authKey, cstr );
	[mFingerprintField setStringValue:[NSString stringWithCString:cstr]];
	PGPGetKeyDBObjNumericProperty( authKey, kPGPKeyProperty_Validity, &validity );
	switch( validity )
	{
		case kPGPValidity_Marginal:
			validity = 1;
			break;
		case kPGPValidity_Complete:
			validity = 2;
			break;
		default:
			validity = 0;
			break;
	}
	[mValidityIndicator setDoubleValue:validity];
	
	return self;
}

- (PGPError)runRemoteAuthModalDialog
{
	PGPError returnCode;
	
	returnCode = [[NSApplication sharedApplication] runModalForWindow:mRemoteAuthWindow];
	[mRemoteAuthWindow close];
    return( returnCode );
}

- (IBAction)dialogButtonPressed:(id)sender
{
	[[NSApplication sharedApplication] stopModalWithCode:[sender tag] ? kPGPError_UserAbort : kPGPError_NoErr];
}

- (void)dealloc
{
	[mCancelButton release];
	[super dealloc];
	PGPLog(( @"CRemoteAuth dealloc" ));
}

@end

