/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CSetupAssistant.m,v 1.12 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import "CSetupAssistant.h"

#import "CSetupAssistantPage.h"
#import "pgpClientLibCocoa.h"
#import "CRemoteAuth.h"

@implementation CSetupAssistant

- (id)init
{
    if (!mInited)
    {
        [super init];
        
        mInited = YES;
    }

    if (!mLoadedNib)
        [self initWithNib:kPGPSetupAssistantNib];
    
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	else
		PGPLog((@"%s:%d: Bad nibFile specified", __FILE__, __LINE__));

	mPreviousDelegate = [mWindow delegate];
    [mWindow setDelegate:self];

    [mHelp setEnabled:NO];

	[mNext retain];
	[mBack retain];
	[mHelp retain];
	[mCancel retain];

	mLoadedNib = YES;

    return [self init];
}

- (id)initWithPages:(id)firstPage, ...
{
    id			currentArg;
	id			lastPage;
    va_list		args;
    
    if (!mPages)
        mPages = [[NSMutableArray alloc] initWithCapacity:5];
    
    if (firstPage)
	{
		[(CSetupAssistantPage*) firstPage setSetupAssistant:self];
        [mPages addObject:firstPage];
	}
        
    va_start(args, firstPage);
    
    while (currentArg = va_arg(args, id))
	{
		[(CSetupAssistantPage*) currentArg setSetupAssistant:self];
        [mPages addObject:currentArg];
	}
    
    va_end(args);

	if (lastPage = [mPages lastObject])
		[self markPageAsLastPage:lastPage];
    
    return [self init];
}

- (void)beginSetupAssistantForTarget:(id)target didEndSelector:(SEL)endSelector isModal:(BOOL)modal
{
    mTarget = [target retain];
    mEndSelector = endSelector;
    mIsRunningModal = modal;
    
    mCurrentPageIndex = 0;
    
	[mWindow center];
	
    if (mIsRunningModal)
        [[NSApplication sharedApplication] runModalForWindow:mWindow];
    else
        [mWindow makeKeyAndOrderFront:self];
}

- (NSWindow*)window
{
	return mWindow;
}

- (NSString*)title
{
    return [mWindow title];
}

- (void)setTitle:(NSString*)title
{
    if (title)
        [mWindow setTitle:title];
}

- (BOOL)isRunningModal
{
    return mIsRunningModal;
}

- (NSArray*)pages
{
    return mPages;
}

- (CSetupAssistantPage*)currentPage
{
    unsigned		lastPageIndex;
    
    if (mPages)
    {
        lastPageIndex = [mPages count] ? [mPages count] - 1 : 0;
        
        if (mCurrentPageIndex <= lastPageIndex)
            return [mPages objectAtIndex:mCurrentPageIndex];
    }
    
	return nil;
}

- (unsigned)currentPageIndex
{
	return mCurrentPageIndex;
}

- (unsigned)returnCode
{
	return mReturnCode;
}

- (void)addPage:(CSetupAssistantPage*)page
{
	id		lastPage;
	
	if (page)
	{
		if (!mPages)
			mPages = [[NSMutableArray alloc] initWithCapacity:5];
	
		if (lastPage = [mPages lastObject])
			[self unmarkPageAsLastPage:lastPage];
			
		[page setSetupAssistant:self];
		[self markPageAsLastPage:page];
		[mPages addObject:page];
	}
}

- (void)insertPage:(CSetupAssistantPage*)page atIndex:(unsigned)index
{
    unsigned	lastPageIndex;
    
    if (mPages)
    {
        lastPageIndex = [mPages count] ? [mPages count] - 1 : 0;
        
        if (index == (lastPageIndex + 1))
            [self addPage:page];
        else if (index > lastPageIndex)
            PGPLog((@"%s:%d: Index out of range. Index must be within 0-%d", __FILE__, __LINE__, lastPageIndex + 1));
        else
		{
			[page setSetupAssistant:self];
            [mPages insertObject:page atIndex:index];
		}
    }
}

- (void)removePageAtIndex:(unsigned)index
{
	id			lastPage;
    unsigned	lastPageIndex;
    
    if (mPages)
    {
        lastPageIndex = [mPages count] ? [mPages count] - 1 : 0;
        
        if (index == lastPageIndex)
		{
			[self unmarkPageAtIndexAsLastPage:index];
            [mPages removeObjectAtIndex:index];
			
			if (lastPage = [mPages lastObject])
				[self markPageAsLastPage:lastPage];
		}
		else if (index < lastPageIndex)
			[mPages removeObjectAtIndex:index];
    }
}

- (void)setPage:(unsigned)pageIndex
{
    CSetupAssistantPage*	newPage = nil;
    unsigned				lastPageIndex;
    
    if (!pageIndex)
        [mBack setEnabled:NO];
    else
        [mBack setEnabled:YES];
    
    if (mPages)
    {
        NS_DURING
            newPage = [mPages objectAtIndex:pageIndex];
        NS_HANDLER
            newPage = nil;
            PGPLog((@"%s:%d: Invalid pageIndex (%d) specified. Out of bounds.", __FILE__, __LINE__, pageIndex));
        NS_ENDHANDLER
		
		if (newPage)
		{
			lastPageIndex = [mPages count] ? [mPages count] - 1 : 0;
			
			if ((pageIndex == lastPageIndex) || [newPage isLastPage])
			{
				[mNext setAction:@selector(finish:)];
				[mNext setTitle:LSCL(@"Finish", @"")];
			}
			else
			{
				[mNext setAction:@selector(next:)];
				[mNext setTitle:LSCL(@"Next", @"")];
			}
		}
    }
    
    if (newPage)
    {
        if (mCurrentView)
            [mCurrentView removeFromSuperview];
        
        [mView addSubview:[newPage view]];
		
		if ([newPage isLastPage])
			[mWindow setDefaultButtonCell:[mNext cell]];
		else
			[mWindow setDefaultButtonCell:nil];
            
        mCurrentView = [newPage view];
    }
}

- (void)markPageAsLastPage:(CSetupAssistantPage*)page
{
	if (page && ([mPages indexOfObject:page] != NSNotFound))
	{
		[page setIsLastPage:YES];
		
		if (page == [self currentPage])
			[self setPage:mCurrentPageIndex];
	}
}

- (void)markPageAtIndexAsLastPage:(unsigned)pageIndex
{
	CSetupAssistantPage*	lastPage = nil;
	
    if (mPages)
    {
        NS_DURING
            lastPage = [mPages objectAtIndex:pageIndex];
        NS_HANDLER
            lastPage = nil;
            PGPLog((@"%s:%d: Invalid pageIndex (%d) specified. Out of bounds.", __FILE__, __LINE__, pageIndex));
        NS_ENDHANDLER
	}
	
	if (lastPage)
		[self markPageAsLastPage:lastPage];
}

- (void)unmarkPageAsLastPage:(CSetupAssistantPage*)page
{
	if (page && ([mPages indexOfObject:page] != NSNotFound))
	{
		[page setIsLastPage:NO];
		
		if (page == [self currentPage])
			[self setPage:mCurrentPageIndex];
	}
}
- (void)unmarkPageAtIndexAsLastPage:(unsigned)pageIndex
{
	CSetupAssistantPage*	lastPage = nil;
	
	if (mPages)
	{
        NS_DURING
            lastPage = [mPages objectAtIndex:pageIndex];
        NS_HANDLER
            lastPage = nil;
            PGPLog((@"%s:%d: Invalid pageIndex (%d) specified. Out of bounds.", __FILE__, __LINE__, pageIndex));
        NS_ENDHANDLER
	}
	
	if (lastPage)
		[self unmarkPageAsLastPage:lastPage];
}

- (void)gotoNextPage
{
	[self setPage:++mCurrentPageIndex];
	[self setupAssistantPageDidBecomeActive];
}

- (void)gotoPreviousPage
{
	[self setPage:--mCurrentPageIndex];
	[self setupAssistantPageDidBecomeActive];
}

- (void)callEndSelector
{
    NSInvocation*		endInvocation;
    NSMethodSignature*	endSignature;
    
    if (mTarget && mEndSelector)
    {
        endSignature = [mTarget methodSignatureForSelector:mEndSelector];
        
        endInvocation = [NSInvocation invocationWithMethodSignature:endSignature];
        [endInvocation setTarget:mTarget];
        [endInvocation setSelector:mEndSelector];
        
        [mTarget release];
        mTarget = nil;		
		
        [endInvocation invoke];
    }
    else
        PGPLog((@"%s:%d: Invalid target or selector", __FILE__, __LINE__));
}

- (id)nextButton
{
	return mNext;
}

- (id)backButton
{
	return mBack;
}

- (id)helpButton
{
	return mHelp;
}

- (id)cancelButton
{
	return mCancel;
}

- (IBAction)next:(id)sender
{
    CSetupAssistantPage*	currentPage;
    BOOL					shouldProceed = YES;
    unsigned				lastPageIndex;
    
    if (mPages)
    {
        lastPageIndex = [mPages count] ? [mPages count] - 1 : 0;
        
        if (mCurrentPageIndex == lastPageIndex)
            [self finish:sender];
        else
        {
            if (currentPage = [mPages objectAtIndex:mCurrentPageIndex])
            {
                if ([currentPage respondsToSelector:@selector(assistantShouldGoToNextPage:)])
                    shouldProceed = [currentPage assistantShouldGoToNextPage:self];
                    
                if (shouldProceed)
				{
                    [self setPage:++mCurrentPageIndex];
					[self setupAssistantPageDidBecomeActive];					
				}
            }
        }
    }
}
    
- (IBAction)back:(id)sender
{
    CSetupAssistantPage*	currentPage;
    BOOL					shouldMoveBack = YES;
    
    if (mPages)
    {
        if (currentPage = [mPages objectAtIndex:mCurrentPageIndex])
        {
            if ([currentPage respondsToSelector:@selector(assistantShouldGoToPreviousPage:)])
                shouldMoveBack = [currentPage assistantShouldGoToPreviousPage:self];
                    
            if (shouldMoveBack)
			{
                [self setPage:--mCurrentPageIndex];
				[self setupAssistantPageDidBecomeActive];
			}
        }
    }
}

- (IBAction)help:(id)sender
{
    CSetupAssistantPage*	currentPage;

    if (currentPage = [mPages objectAtIndex:mCurrentPageIndex])
        PGPLog((@"help was pressed for page %d", mCurrentPageIndex));
}

- (IBAction)cancel:(id)sender
{
    if (mIsRunningModal)
        [[NSApplication sharedApplication] stopModal];

	mReturnCode = NSCancelButton;

	[mWindow setDelegate:mPreviousDelegate];
    [mWindow close];
	
	[self callEndSelector];
}
    
- (IBAction)finish:(id)sender
{
    CSetupAssistantPage*	currentPage;
    BOOL					shouldFinish = YES;

    if (mPages)
    {
        if (currentPage = [mPages objectAtIndex:mCurrentPageIndex])
        {
            if ([currentPage respondsToSelector:@selector(assistantShouldFinish:)])
                shouldFinish = [currentPage assistantShouldFinish:self];
        }
    }
    
    if (shouldFinish)
    {
        if (mIsRunningModal)
            [[NSApplication sharedApplication] stopModal];
		
		mReturnCode = NSOKButton;
		
        [mWindow close];
		
        [self callEndSelector];		
    }
}

- (void)setupAssistantPageDidBecomeActive
{
	[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclSetupAssistantPageBecameActive object:self];
}

- (void)windowDidBecomeKey:(NSNotification*)notification
{
    [self setPage:mCurrentPageIndex];
}

- (void)dealloc
{
    if (mPages)
        [mPages release];

    if (mTarget)
        [mTarget release];
        
    [super dealloc];
}

@end
