/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: pgpDialogs.m,v 1.36 2002/09/27 23:11:36 jeffc Exp $
____________________________________________________________________________*/
#import <PGP/PGP.h>
#import "pgpPFLPriv.h"
#import <PGPclient/pgpKeyServerPrefs.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import "CPGPViewer.h"
#import "CKeySelect.h"
#import "pgpDialogs.h"
#import "CRemoteAuth.h"

#define kCautionStr							LSCL(@"Caution", @"")
#define kOKStr								LSCL(@"OK", @"")

#define kDetachedSignatureTitleStr			LSCL(@"Detached Signature", @"")
#define kDetachedSignatureDescStr 			LSCL(@" \
When checked, the signature is placed in a separate file. When unchecked, the signature\n \
is merged with the source file into a single combined file.", @"")

#define kTextOutputTitleStr 				LSCL(@"Text Output", @"")
#define kTextOutputDescStr 					LSCL(@" \
When checked, the output data is created in an ASCII form suitable for including in the\n \
text of an email message. When unchecked, the output data is created in raw binary form.", @"")

#define kConvEncryptTitleStr 				LSCL(@"Conventional Encrypt", @"")
#define kConvEncryptDescStr 				LSCL(@" \
When checked, encryption is performed using an entered passphrase instead of one or\n \
more public keys. Decryption requires only knowing the correct passphrase.", @"")

#define kWipeOriginalTitleStr 				LSCL(@"Wipe Original", @"")
#define kWipeOriginalDescStr 				LSCL(@" \
When checked, all source files are overwritten and then deleted after encryption to\n \
prevent recovery of their unencrypted contents.", @"")

#define kFYEOTitleStr 						LSCL(@"Secure Viewer", @"")
#define kFYEODescStr 						LSCL(@" \
When checked, the decrypted text appears in a special viewer that keeps it in memory,\n \
immediately wipes it upon closure, and displays it in a tempest attack proof font. When\n \
unchecked, the decrypted text is displayed normally.", @"")

#define kSDATitleStr 						LSCL(@"Self-Decrypting Archive", @"")
#define kSDADescStr 						LSCL(@" \
When checked, the encrypted files are saved into a single archive which will decrypt\n \
itself when opened. Self-decrypting archives do not require PGP.", @"")

#define kMacBinaryTitleStr 					LSCL(@"MacBinary:", @"")
#define kMacBinaryNoStr 					LSCL(@"No", @"")
#define kMacBinarySmartStr 					LSCL(@"Smart", @"")
#define kMacBinaryYesStr 					LSCL(@"Yes", @"")
#define kMacBinaryDescStr 					LSCL(@" \
When No, only the data fork of the file is used. When Yes, the entire file is\n \
used, including Mac OS specific information. When Smart,  the file type determines\n \
if the Mac OS information is needed.", @"")

#define kCorporateADKDisabledOrMissingStr	LSCL(@"The required Additional Decryption Key is disabled or missing. You cannot encrypt without this key. Please contact your security administrator.", @"")
#define kNoDefaultPrivateKeyFoundStr		LSCL(@"The Always encrypt to default key option is enabled, however no default private key could be located. Use PGPkeys to set a default private key.", @"")
#define kMustSelectKeyOrConvEncryptStr		LSCL(@"You must select at least one key or use conventional encryption.", @"")
#define kCannotEncryptToInvalidKeyStr		LSCL(@"You are not allowed to encrypt to invalid keys.", @"")

enum
{
	kMacBinaryPopupNoneMenuItem	= 0,
	kMacBinaryPopupSmartMenuItem,
	kMacBinaryPopupYesMenuItem
};

    PGPError
PGPclSigningPassphraseDialog(
    PGPContextRef               inContext,
    PGPPrefRef					inPrefRef,
    PGPKeyDBRef                 inKeyDBRef, 
    const char                  *ioPrompt,
    PGPGetPassphraseOptions     inSignOptions,
    PGPGetPassphraseSettings	inSignSettings,
    PGPKeyDBObjRef              inDefaultSigningKey,
    char                        ioPassphrase[kPGPcl_MaxPGPUserIDLength],
    PGPGetPassphraseSettings    *ioUserSettings,
    PGPKeyDBObjRef              *ioActualSigningKey)
{
    PGPError				theError = kPGPError_NoErr;
    PGPOptionListRef    	theOptionList = kInvalidPGPOptionListRef;
    char					*thePassphrase = NULL;
    
	if( IsntNull( ioPassphrase ) )
        memset( ioPassphrase, (char)NULL, kPGPcl_MaxPGPUserIDLength );
        
	if( IsntNull( ioActualSigningKey ) )
        *ioActualSigningKey = NULL;
        
    if( IsntNull( ioUserSettings ) )
        pgpClearMemory( ioUserSettings, sizeof( *ioUserSettings ) );
    
    pgpClearMemory( ioPassphrase, kPGPcl_MaxPGPUserIDLength );
    
    theError = PGPBuildOptionList( inContext, &theOptionList,
            PGPOUIOutputPassphrase( inContext, &thePassphrase ),
            PGPOLastOption( inContext ) );
    
    if( IsntPGPError( theError ) )
    {
		PGPUInt32	detachedSignature	= 0;
		PGPUInt32	textOutput			= 0;
		PGPUInt32	macBinary			= 0;
		PGPBoolean	haveFileOptions 	= FALSE;
        
        if( IsntNull( ioPrompt ) )
        {
            theError = PGPAppendOptionList( theOptionList,
                PGPOUIDialogPrompt( inContext, ioPrompt ),
                PGPOLastOption( inContext ) );
        }
        
		if( IsntPGPError( theError ) && 
            ( inSignOptions & kPGPGetPassphraseOptionsHideFileOptions ) == 0 )
		{
			// Setup options dialog
			
			haveFileOptions = TRUE;

			if( (inSignSettings & kPGPGetPassphraseSettingsDetachedSig) != 0 )
				detachedSignature = 1;

			if( (inSignSettings & kPGPUISettingsTextOutput) != 0 )
				textOutput = 1;
			
			macBinary = GetMacBinaryPopupInitialValue( inSignSettings, inPrefRef );
								
			theError = PGPAppendOptionList( theOptionList,
                            PGPOUIDialogOptions( inContext,
                                DetachedSignatureCheckboxOptionList( inContext, 1, 
                                    detachedSignature, &detachedSignature ),
                                TextOutputCheckboxOptionList( inContext, 2,
                                    textOutput, &textOutput ),
                                MacBinaryPopupOptionList( inContext, 3, macBinary, &macBinary ),
                            PGPOLastOption( inContext ) ),
                        PGPOLastOption( inContext ) );
		}
        
        if( IsntPGPError( theError ) )
        {
            theError = PGPSigningPassphraseDialog( inContext, inKeyDBRef,
                        ioActualSigningKey,
                        theOptionList,
                        PGPOUIDefaultKey( inContext, inDefaultSigningKey ),
                        PGPOLastOption( inContext ) );
        }
        
        if( haveFileOptions && ( IsntPGPError( theError ) ||
        theError == kPGPError_KeyUnusableForSignature ) )
		{
			*ioUserSettings = 0;
			
			if( detachedSignature != 0 )
				*ioUserSettings |= kPGPGetPassphraseSettingsDetachedSig;
				
			if( textOutput != 0 )
				*ioUserSettings |= kPGPUISettingsTextOutput;
				
			if( macBinary == kMacBinaryPopupNoneMenuItem )
			{	
				*ioUserSettings |= kPGPUISettingsNoMacBinary;
			}
			else if( macBinary == kMacBinaryPopupSmartMenuItem )
			{
				*ioUserSettings |= kPGPUISettingsSmartMacBinary;
			}
			else
			{
				*ioUserSettings |= kPGPUISettingsForceMacBinary;
			}
            SetMacBinaryPopupValue( macBinary, inPrefRef );
		}
        
        if( IsntNull( thePassphrase ) )
            strcpy( ioPassphrase, thePassphrase );
        PGPFreeOptionList( theOptionList );
    }
    
    if( IsntNull( thePassphrase ) )
        PGPFreeData( thePassphrase );

    return( theError );
}

	static PGPUInt32
GetMacBinaryPopupInitialValue(
	PGPGetPassphraseSettings	inDefaultSettings,
	PGPPrefRef					inPrefsRef)
{
	PGPUInt32	macBinaryMenuItem = kMacBinaryPopupNoneMenuItem;
	
	// MacBinary always on by default
	macBinaryMenuItem = kMacBinaryPopupSmartMenuItem;

	if ( ( inDefaultSettings & kPGPUISettingsNoMacBinary ) != 0 )
	{
		macBinaryMenuItem = kMacBinaryPopupNoneMenuItem;
	}
	else if ( ( inDefaultSettings & kPGPUISettingsSmartMacBinary ) != 0 )
	{
		macBinaryMenuItem = kMacBinaryPopupSmartMenuItem;
	}
	else if ( ( inDefaultSettings & kPGPUISettingsForceMacBinary ) != 0 )
	{
		macBinaryMenuItem = kMacBinaryPopupYesMenuItem;
	}
	else
	{
		PGPUInt32	macBinaryPref = kPGPPrefMacBinaryOff;
		
		if( IsntPGPError( PGPGetPrefNumber( inPrefsRef, kPGPPrefMacBinaryDefault, &macBinaryPref ) ) )
		{
			switch( macBinaryPref )
			{
				case kPGPPrefMacBinaryOff:
					macBinaryMenuItem = kMacBinaryPopupNoneMenuItem;
					break;
				
				case kPGPPrefMacBinarySmart:
					macBinaryMenuItem = kMacBinaryPopupSmartMenuItem;
					break;
					
				case kPGPPrefMacBinaryOn:
					macBinaryMenuItem = kMacBinaryPopupYesMenuItem;
					break;
			}
		}
	}

	return( macBinaryMenuItem );
}

	static void
SetMacBinaryPopupValue(
	PGPInt32					inMacBinarySetting,
	PGPPrefRef					inPrefRef)
{
    PGPInt32		macBinaryPref = kPGPPrefMacBinaryOff;

    switch( inMacBinarySetting )
    {
        case kMacBinaryPopupNoneMenuItem:
            macBinaryPref = kPGPPrefMacBinaryOff;
            break;
        
        case kMacBinaryPopupSmartMenuItem:
            macBinaryPref = kPGPPrefMacBinarySmart;
            break;
            
        case kMacBinaryPopupYesMenuItem:
            macBinaryPref = kPGPPrefMacBinaryOn;
            break;
   }
   
	PGPSetPrefNumber( inPrefRef, kPGPPrefMacBinaryDefault, macBinaryPref );
}

	static PGPOptionListRef
NewCheckboxOption(
	PGPContextRef		context,
	PGPUInt32			itemID,
	NSString			*titleStr,
	NSString			*descStr,
	PGPBoolean			initialValue,
	PGPOptionListRef	subOptions,
	PGPUInt32			*resultPtr )
{
	char	title[256];
	char	desc[256];

	(void)strcpy( title, [titleStr cString] );
	
	if( descStr != nil)
	{
		(void)strcpy( desc, [descStr cString] );
	}
	else
	{
		*desc = 0;
	}
	
	return( PGPOUICheckbox( context, itemID, title, desc, initialValue,
					resultPtr, subOptions != NULL ? subOptions :
					PGPONullOption( context ), PGPOLastOption( context ) ) );
}

	static PGPOptionListRef
NewPopupListOption(
	PGPContextRef	context,
	PGPUInt32		itemID,
	NSString		*titleStr,
	NSString		*descStr,
    NSArray			*items,
	PGPBoolean		initialValue,
	PGPUInt32		*resultPtr )
{
    char				title[256];
	char				desc[256];
	const char				*listItems[32];
	PGPUInt32			numListItems = 0;
	PGPUInt32 			listIndex = 0;
	PGPOptionListRef	optionList;
	
    numListItems = [items count];
	pgpAssert( numListItems <= 32 );
	
	(void)strcpy( title, [titleStr cString] );
	
	if( descStr != nil )
	{
		(void)strcpy(desc, [descStr cString] );
	}
	else
	{
		*desc = 0;
	}
	
	pgpClearMemory( listItems, sizeof( listItems ) );
    
	for( listIndex = 0; listIndex < numListItems; listIndex++ )
	{
        NSString		*item = nil;
        item = [items objectAtIndex:listIndex];
        listItems[listIndex] = (char *)PGPNewData(
            PGPPeekContextMemoryMgr( context ),
            [item cStringLength] + 1, 0 );
        pgpCopyMemory( [item cString], (char *) listItems[listIndex], [item cStringLength] + 1);
		pgpAssert( IsntNull( listItems[listIndex] ) );
	}
	
	optionList = PGPOUIPopupList( context, itemID, title, desc, numListItems,
					listItems, initialValue, resultPtr,
					PGPOLastOption( context ) );
					
	for( listIndex = 0; listIndex < numListItems; listIndex++ )
	{
		// PGPOUIPopupList() copied our strings. Free our copies
		if( IsntNull( listItems[listIndex] ) )
			PGPFreeData( (char *) listItems[listIndex] );
	}

	return( optionList );
}

	static PGPOptionListRef
TextOutputCheckboxOptionList(
	PGPContextRef 	inContext,
	PGPUInt32		inItemID,
	PGPBoolean		inChecked,
	PGPUInt32		*ioResultPtr)
{

	return( NewCheckboxOption( inContext, inItemID, //kOptionDialogStringListResID,
					kTextOutputTitleStr, kTextOutputDescStr,
					inChecked, NULL, ioResultPtr ) );
}

	static PGPOptionListRef
DetachedSignatureCheckboxOptionList(
	PGPContextRef 	inContext,
	PGPUInt32		inItemID,
	PGPBoolean		inChecked,
	PGPUInt32		*ioResultPtr)
{
	return( NewCheckboxOption( inContext, inItemID, //kOptionDialogStringListResID,
					kDetachedSignatureTitleStr,
					kDetachedSignatureDescStr, inChecked, NULL, ioResultPtr ) );
}

	static PGPOptionListRef
MacBinaryPopupOptionList(
	PGPContextRef 	inContext,
	PGPUInt32		inItemID,
	PGPUInt32		inInitialValue,
	PGPUInt32		*ioResultPtr)
{
    NSMutableArray	*optionList = [NSMutableArray array];
    [optionList addObject:kMacBinaryNoStr];
    [optionList addObject:kMacBinarySmartStr];
    [optionList addObject:kMacBinaryYesStr];
    
	return( NewPopupListOption( inContext, inItemID, //kOptionDialogStringListResID,
					kMacBinaryTitleStr, kMacBinaryDescStr, optionList,
					inInitialValue, ioResultPtr ) );
                    
    [optionList removeAllObjects];
}

	static PGPOptionListRef
SDACheckboxOptionList(
	PGPContextRef 	context,
	PGPUInt32		itemID,
	PGPBoolean		checked,
	PGPUInt32		*resultPtr)
{
	return( NewCheckboxOption( context, itemID, //kOptionDialogStringListResID,
					kSDATitleStr, kSDADescStr,
					checked, NULL, resultPtr ) );
}

	static PGPOptionListRef
ConvEncryptCheckboxOptionList(
	PGPContextRef 	context,
	PGPUInt32		itemID,
	PGPBoolean		checked,
	PGPUInt32		*resultPtr,
	PGPUInt32		showSDA,
	PGPBoolean		sdaChecked,
	PGPUInt32		*sdaResultPtr)
{
	return( NewCheckboxOption( context, itemID, //kOptionDialogStringListResID,
					kConvEncryptTitleStr, kConvEncryptDescStr,
					checked, showSDA ? SDACheckboxOptionList( context, itemID + 1,
					sdaChecked, sdaResultPtr ) : NULL, resultPtr ) );
}

	static PGPOptionListRef
WipeOriginalCheckboxOptionList(
	PGPContextRef 	context,
	PGPUInt32		itemID,
	PGPBoolean		checked,
	PGPUInt32		*resultPtr)
{
	return( NewCheckboxOption( context, itemID, //kOptionDialogStringListResID,
					kWipeOriginalTitleStr, kWipeOriginalDescStr,
					checked, NULL, resultPtr ) );
}

	static PGPOptionListRef
FYEOCheckboxOptionList(
	PGPContextRef 	context,
	PGPUInt32		itemID,
	PGPBoolean		checked,
	PGPUInt32		*resultPtr)
{
	return( NewCheckboxOption( context, itemID, //kOptionDialogStringListResID,
					kFYEOTitleStr, kFYEODescStr,
					checked, NULL, resultPtr ) );
}

	static PGPError
SetupAdminParams(
	PGPContextRef		context,
	PGPOptionListRef	recipientOptions,
	PGPPrefRef			clientPrefsRef,
	PGPKeyDBRef			allKeys)
{
	PGPError			err = kPGPError_NoErr;
	
	pgpAssert( PGPPrefRefIsValid( clientPrefsRef ) );
	
	err = SetupOutgoingADK( context, recipientOptions, clientPrefsRef, allKeys );
	if( IsPGPError( err ) )
	{
	
	}
	
	return( err );
}

	static PGPError
SetupOutgoingADK(
	PGPContextRef		context,
	PGPOptionListRef	recipientOptions,
	PGPPrefRef			clientPrefsRef,
	PGPKeyDBRef			allKeys)
{
	PGPError	err = kPGPError_NoErr;
	PGPKeyID	*keyID 	= NULL;
	PGPSize		keyIDSize = 0;
	
	if( IsntPGPError( PGPGetPrefData( clientPrefsRef, kPGPPrefOutgoingADKID,
						&keyIDSize, (void *)&keyID ) ) )
	{
		PGPBoolean		enforceOutgoingADK = FALSE,
						bDisabled = FALSE;
		PGPKeyDBObjRef	outgoingADK = kInvalidPGPKeyDBObjRef;
		
		if( IsPGPError( PGPGetPrefBoolean( clientPrefsRef,
					kPGPPrefEnforceOutgoingADK, &enforceOutgoingADK ) ) )
		{
			enforceOutgoingADK = TRUE;
		}
		
		err = PGPFindKeyByKeyID( allKeys, keyID, &outgoingADK );
		if( IsntPGPError( err ) && PGPKeyDBObjRefIsValid( outgoingADK ) )
			err = PGPGetKeyDBObjBooleanProperty( outgoingADK, 
						kPGPKeyProperty_IsDisabled, &bDisabled );
		if( IsntPGPError( err ) && PGPKeyDBObjRefIsValid( outgoingADK ) && !bDisabled )
		{
			PGPRecipientSpec	spec;
		
			pgpClearMemory( &spec, sizeof( spec ) );
			
			spec.type	= kPGPRecipientSpecType_Key;
			spec.u.key	= outgoingADK;
			
			if( enforceOutgoingADK )
				spec.flags |= kPGPRecipientSpecFlags_Locked;
				
			err = PGPAppendOptionList( recipientOptions,
						PGPOUIDefaultRecipients( context, 1, &spec ),
						PGPOLastOption( context ) );
		}
		else
		{
			if( enforceOutgoingADK )
			{
 				NSBeep();
                
                NSRunAlertPanel(kCautionStr, kCorporateADKDisabledOrMissingStr, kOKStr, nil, nil);

				err = kPGPError_UserAbort;
			}
			else
			{
				err = kPGPError_NoErr;
			}
		}
		
		PGPFreeData( keyID );
	}
	
	return( err );
}

	PGPError
PGPclDecryptionPassphraseDialog(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
    PGPPrefRef			prefRef,
	const char			*prompt,
	PGPKeySetRef		recipientKeys,
	const PGPKeyID		keyIDList[],		
	PGPUInt32			keyIDCount,
	PGPOptionListRef	additionalOptions,
	char				**passphrase,
	PGPKeyDBObjRef 		*decryptionKey,
	PGPKeyDBRef			*newKeys)
{
	PGPError			err = kPGPError_NoErr;

	if( IsntNull( passphrase ) )
		*passphrase = NULL;
	if( IsntNull( decryptionKey ) )
		*decryptionKey = kInvalidPGPKeyDBObjRef;
	if( IsntNull( newKeys ) )
		*newKeys = kInvalidPGPKeyDBRef;
				
	PGPValidatePtr( context );
	PGPValidatePtr( recipientKeys );
	PGPValidatePtr( passphrase );
	PGPValidatePtr( decryptionKey );
    
	if( IsntPGPError( err ) )
	{
		err = PGPKeyServerInit();
		if( IsntPGPError( err ) )
		{
			err = pgpClientDecryptionPassphraseDialog( context,
						prefRef,
						tlsContext, prompt, recipientKeys, keyIDList,
						keyIDCount, additionalOptions, passphrase,
						decryptionKey, newKeys );
			PGPKeyServerCleanup();
		}
	}
				
	return( err );
}

    PGPError
PGPclRecipientDialog(
    PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
    PGPtlsContextRef			inTLSContext,
    PGPKeyDBRef					inKeyDBRef,
    PGPUInt32					inNumDefaultRecipients,
    const PGPRecipientSpec		*ioDefaultRecipients,
    PGPRecipientOptions			inDialogOptions,
    PGPRecipientSettings		inDefaultSettings,
    PGPRecipientSettings		*ioUserSettings,
    PGPKeyDBRef					*outActualRecipients,
    PGPKeyDBRef					*outNewKeys)
{
	PGPError			theError = kPGPError_NoErr;
		
	if( IsntNull( ioUserSettings ) )
		*ioUserSettings = 0;
	if( IsntNull( outActualRecipients ) )
		*outActualRecipients = kInvalidPGPKeyDBRef;
	if( IsntNull( outNewKeys ) )
		*outNewKeys = kInvalidPGPKeyDBRef;
				
	PGPValidatePtr( inContext );
    PGPValidatePtr( inPrefRef );
	PGPValidatePtr( inKeyDBRef );
	PGPValidatePtr( ioUserSettings );
	PGPValidatePtr( outActualRecipients );
	
	if( inNumDefaultRecipients != 0 )
	{
		PGPValidatePtr( ioDefaultRecipients );
	}
	else if( IsntNull( ioDefaultRecipients ) )
	{
		PGPValidateParam( inNumDefaultRecipients != 0 )
	}
    
    theError = PGPKeyServerInit();
    if( IsntPGPError( theError ) )
    {
        theError = pgpClientRecipientDialog( inContext, inPrefRef,
                    inTLSContext, inKeyDBRef,
                    inNumDefaultRecipients, ioDefaultRecipients,
                    inDialogOptions, inDefaultSettings, ioUserSettings,
                    outActualRecipients, outNewKeys );
			
        PGPKeyServerCleanup();
    }
    	
	return( theError );
}

	static PGPError
pgpClientRecipientDialog(
	PGPContextRef			context,					
	PGPPrefRef				clientPrefsRef,				
	PGPtlsContextRef		tlsContext,
	PGPKeyDBRef				allKeys,					
	PGPUInt32				numDefaultRecipients,		
	const PGPRecipientSpec	*defaultRecipients,			
	PGPRecipientOptions		dialogOptions,				
	PGPRecipientSettings	defaultSettings,
	PGPRecipientSettings	*userSettings,			
	PGPKeyDBRef				*actualRecipients,			
	PGPKeyDBRef				*newKeys)
{
	PGPError						err = kPGPError_NoErr;
	PGPOptionListRef				recipientOptions;
	PGPBoolean						showMarginalValidity = FALSE;
	PGPBoolean						marginalIsInvalid = FALSE;
	PGPBoolean						forceShowDialog = FALSE;
	
	if( IsntNull( newKeys ) )
		*newKeys = kInvalidPGPKeyDBRef;
		
	if( ( dialogOptions & kPGPRecipientOptionsAlwaysShowDialog ) != 0 )
	{
		forceShowDialog = TRUE;
	}
	else
	{
		forceShowDialog = ( numDefaultRecipients == 0 );;
	}
	
	(void) PGPGetPrefBoolean( clientPrefsRef, kPGPPrefDisplayMarginalValidity,
				&showMarginalValidity);
	(void) PGPGetPrefBoolean( clientPrefsRef, kPGPPrefMarginalIsInvalid,
				&marginalIsInvalid);

	err = PGPBuildOptionList( context, &recipientOptions,
				PGPOUIDisplayMarginalValidity( context, showMarginalValidity ),
				PGPOUIIgnoreMarginalValidity( context, marginalIsInvalid ),
				PGPOLastOption( context ) );
	if( IsntPGPError( err ) )
	{
		PGPBoolean			allowConvEncrypt 	= TRUE;
		PGPKeyServerEntry	*ksEntries 			= NULL;
		PGPKeyServerSpec	*serverList 		= NULL;
		PGPUInt32			numKSEntries 		= 0;
		PGPUInt32			serverIndex			= 0;
		PGPBoolean			allowInvalidEncrypt = TRUE;
	
		PGPAdditionalRecipientRequestEnforcement	arrEnforcement;
		
		arrEnforcement = kPGPARREnforcement_Warn;
		
		err = PGPGetKeyServerPrefs( clientPrefsRef, &ksEntries, &numKSEntries );
		if( IsntPGPError( err ) && IsntNull( ksEntries ) && numKSEntries > 0 )
		{
			serverList = (PGPKeyServerSpec *) PGPNewData(
								PGPPeekContextMemoryMgr( context ),
								numKSEntries * sizeof( *serverList ),
								kPGPMemoryMgrFlags_Clear );
			if( IsntNull( serverList ) )
			{
 				for( serverIndex = 0; serverIndex < numKSEntries; serverIndex++ )
				{
					err = PGPNewKeyServer( context,
								ksEntries[serverIndex].type,
								&serverList[serverIndex].server,
								PGPONetHostName( context,
									ksEntries[serverIndex].serverDNS,
									ksEntries[serverIndex].serverPort ),
								PGPOKeyServerProtocol( context,
									ksEntries[serverIndex].protocol ),
								PGPOKeyServerAccessType( context,
									kPGPKeyServerAccessType_Normal ),
								PGPOKeyServerKeySpace( context,
									kPGPKeyServerKeySpace_Normal ),
								PGPOLastOption( context ) );
                    if( IsPGPError( err ) )
						break;
					
					serverList[serverIndex].serverName =
							ksEntries[serverIndex].serverDNS;
					serverList[serverIndex].serverDomain =
							ksEntries[serverIndex].domain;
				}
				
				if( IsntPGPError( err ) )
				{
					PGPBoolean	searchForKeys = FALSE;
					
					(void) PGPGetPrefBoolean( clientPrefsRef,
								kPGPPrefKeyServerSyncUnknownKeys, &searchForKeys );
					err = PGPAppendOptionList( recipientOptions,
								PGPOUIKeyServerUpdateParams( context,
									numKSEntries, serverList, tlsContext,
									searchForKeys, newKeys, PGPOLastOption( context ) ),
								PGPOLastOption( context ) );
				}
			}
			else
			{
				err = kPGPError_OutOfMemory;
			}
		}

		if( IsntPGPError( err ) && numDefaultRecipients != 0 )
		{
			err = PGPAppendOptionList( recipientOptions,
					PGPOUIDefaultRecipients( context, numDefaultRecipients,
						defaultRecipients ),
					PGPOLastOption( context ) );
		}
		
		// Add default key, if set to do so.
		if( IsntPGPError( err ) )
		{
			PGPKeyID		*defaultKeyID;
			PGPSize			defaultKeyIDSize = 0;
			PGPKeyDBObjRef	defaultKey = kInvalidPGPKeyDBObjRef;
			
			if( IsntPGPError( PGPGetPrefData( clientPrefsRef,
						kPGPPrefDefaultKeyID, &defaultKeyIDSize, (void *)&defaultKeyID ) ) )
			{
				if( IsntPGPError( PGPFindKeyByKeyID( allKeys, defaultKeyID,
								&defaultKey ) ) )
				{
					err = PGPAppendOptionList( recipientOptions,
								PGPOUIDefaultKey( context, defaultKey ),
								PGPOLastOption( context ) );
				}

				PGPFreeData( defaultKeyID );
			}
			
			if( IsntPGPError( err ) )
			{
				PGPBoolean		encryptToSelf = FALSE;

				err = PGPGetPrefBoolean( clientPrefsRef, kPGPPrefEncryptToSelf,
							&encryptToSelf );
				if( IsntPGPError( err ) && encryptToSelf )
				{
					if( PGPKeyDBObjRefIsValid( defaultKey ) )
					{
						PGPRecipientSpec	spec;
					
						pgpClearMemory( &spec, sizeof( spec ) );
						
						spec.type	= kPGPRecipientSpecType_Key;
						spec.u.key	= defaultKey;
						
						err = PGPAppendOptionList( recipientOptions,
									PGPOUIDefaultRecipients( context, 1, &spec ),
									PGPOLastOption( context ) );
					}
					else
					{
						NSBeep();
                        NSRunAlertPanel(kCautionStr, kNoDefaultPrivateKeyFoundStr, kOKStr, nil, nil);
					}
				}
			}
		}
			
		if( PGPclEnterprise() && IsntPGPError( err ) && PGPPrefRefIsValid( clientPrefsRef ) )
		{
			PGPBoolean	enforceRemoteADKClass = FALSE;
			
			if( IsPGPError( PGPGetPrefBoolean( clientPrefsRef,
								kPGPPrefAllowConventionalEncryption,
								&allowConvEncrypt ) ) )
			{
				allowConvEncrypt = FALSE;
			}

			if( IsntPGPError( PGPGetPrefBoolean( clientPrefsRef,
								kPGPPrefEnforceRemoteADKClass,
								&enforceRemoteADKClass ) ) &&
				enforceRemoteADKClass )
			{
				arrEnforcement = kPGPARREnforcement_Strict;
			}
			
			err = SetupAdminParams( context, recipientOptions, clientPrefsRef, allKeys );
		}
		
		if( IsntPGPError( err ) )
		{
			PGPUInt32	conventionalEncrypt	= 0;
			PGPUInt32	textOutput			= 0;
			PGPUInt32	macBinary			= 0;
			PGPUInt32	wipeOriginal		= 0;
			PGPUInt32	showFYEO			= 0;
			PGPUInt32	fYEO 				= 0;
			PGPUInt32	sda					= 0;
			PGPBoolean	showSDA				= FALSE;
			PGPBoolean	haveFileOptions 	= FALSE;
			
			if( allowConvEncrypt )
			{
				if( (defaultSettings & kPGPRecipientSettingsConvEncrypt) != 0 )
					conventionalEncrypt = 1;
					
				if( ( dialogOptions & kPGPRecipientOptionsShowSDA ) != 0 )
				{
					showSDA = TRUE;
					
					if( (defaultSettings & kPGPRecipientSettingsSDA) != 0 )
						sda = 1;
				}
			}
			
			if( ( dialogOptions & kPGPRecipientOptionsShowFYEO ) != 0 )
			{
				showFYEO = 1;
				if( (defaultSettings & kPGPRecipientSettingsFYEO) != 0 )
					fYEO = 1;
			}

			if( ( dialogOptions & kPGPRecipientOptionsHideFileOptions ) == 0 )
			{
				// Setup file options
				haveFileOptions = TRUE;

				if( (defaultSettings & kPGPUISettingsTextOutput) != 0 )
					textOutput = 1;
				
				if( (defaultSettings & kPGPRecipientSettingsWipeOriginal) != 0 )
					wipeOriginal = 1;

				macBinary = GetMacBinaryPopupInitialValue( defaultSettings,
									clientPrefsRef );
									
				err = PGPAppendOptionList( recipientOptions,
							PGPOUIDialogOptions( context,
								allowConvEncrypt ? ConvEncryptCheckboxOptionList(
									context, 1, conventionalEncrypt, &conventionalEncrypt,
									showSDA, sda, &sda ) :
									PGPONullOption( context ),
								TextOutputCheckboxOptionList( context, 3,
									textOutput, &textOutput ),
								MacBinaryPopupOptionList( context, 4, macBinary,
									&macBinary ),
								WipeOriginalCheckboxOptionList( context, 5,
									wipeOriginal, &wipeOriginal ),
								(showFYEO) ? FYEOCheckboxOptionList( context, 6, fYEO, &fYEO ) :
									PGPONullOption( context ),
								PGPOLastOption( context ) ),
							PGPOLastOption( context ) );
			}
			else
			{

				err = PGPAppendOptionList( recipientOptions,
							PGPOUIDialogOptions( context,
								allowConvEncrypt ? ConvEncryptCheckboxOptionList(
									context, 1, conventionalEncrypt, &conventionalEncrypt, FALSE,
									0, NULL ) :
									PGPONullOption( context ),
								(showFYEO) ? FYEOCheckboxOptionList( context, 2, fYEO, &fYEO ) :
									PGPONullOption( context ),
								PGPOLastOption( context ) ),
							PGPOLastOption( context ) );
			}
			
			if( IsntPGPError( err ) )
			{
				PGPBoolean	warnOnADKs = FALSE;
				
				err = PGPGetPrefBoolean( clientPrefsRef, kPGPPrefWarnOnADK, &warnOnADKs );
				if( IsntPGPError( err ) )
				{			
					err = PGPRecipientDialog( context, allKeys,
								forceShowDialog, actualRecipients,
								recipientOptions,
								PGPOUIEnforceAdditionalRecipientRequests(
									context, arrEnforcement, warnOnADKs ),
								PGPOLastOption( context ) );
				}
				
				// Tell the user if there are no keys selected and no
				// conventional encryption.
				
				if( sda != 0 )
					conventionalEncrypt = 1;
					
				if( IsntPGPError( err ) && conventionalEncrypt == 0 )
				{
					PGPUInt32	numKeys = 0;
					
					err = PGPCountKeysInKeyDB( *actualRecipients, &numKeys );
					if( IsntPGPError( err ) && numKeys == 0 )
					{
						NSBeep();
                        NSRunAlertPanel(kCautionStr, kMustSelectKeyOrConvEncryptStr, kOKStr, nil, nil);
                        err = kPGPError_UserAbort;
					}
				}
				if( PGPclEnterprise() )
				{
					if ( IsntPGPError( err ) )
					{
						err = PGPGetPrefBoolean( clientPrefsRef, kPGPPrefAllowEncryptInvalidKeys, &allowInvalidEncrypt );
					}
					if (IsntPGPError(err) && (! allowInvalidEncrypt) && (! conventionalEncrypt))
					{
						PGPBoolean		allValid = true;
						PGPKeyIterRef	keyIter = kInvalidPGPKeyIterRef;
						PGPKeyDBObjRef	key;
						PGPBoolean		marginalIsInvalid = FALSE;
						PGPBoolean		isAxiomatic = FALSE;
						PGPInt32		validity = 0;
						
						
						err = PGPNewKeyIterFromKeyDB(*actualRecipients, &keyIter);
						if (IsntPGPError(err))
						{
							err = PGPGetPrefBoolean(clientPrefsRef, kPGPPrefMarginalIsInvalid,
									&marginalIsInvalid);
						}
						while( IsntPGPError(err) && IsntPGPError(err = PGPKeyIterNextKeyDBObj(keyIter,
									kPGPKeyDBObjType_Key, &key)))
						{
							err = PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsAxiomatic,
									&isAxiomatic);
							if( IsntPGPError(err) )
							{
								err = PGPGetKeyDBObjNumericProperty(key, kPGPKeyProperty_Validity,
										&validity);
							}
							if( IsntPGPError(err) )
							{
								if( isAxiomatic || (validity == kPGPValidity_Complete)
								|| ((validity == kPGPValidity_Marginal) && (! marginalIsInvalid)))
								{
									continue;
								} else {
									allValid = false;
									err = kPGPError_EndOfIteration;
								}
							}
						}
						if(err == kPGPError_EndOfIteration)
						{
							if( allValid )
							{
								err = kPGPError_NoErr;
							} else {
								NSBeep();
								NSRunAlertPanel(kCautionStr, kCannotEncryptToInvalidKeyStr, kOKStr, nil, nil);
								err = kPGPError_UserAbort;
							}
						}
						if (PGPKeyIterRefIsValid(keyIter))
						{
							PGPFreeKeyIter(keyIter);
						}
					}
				}
			
				if( IsntPGPError( err ) )
				{
					*userSettings = 0;
					
					if( conventionalEncrypt != 0 )
					{
						*userSettings |= kPGPRecipientSettingsConvEncrypt;
						
						if( PGPKeyDBRefIsValid( *actualRecipients ) )
						{
							PGPFreeKeyDB( *actualRecipients );
							*actualRecipients = kInvalidPGPKeyDBRef;
						}
					}
						
					if( textOutput != 0 )
						*userSettings |= kPGPUISettingsTextOutput;

					if( sda != 0 )
						*userSettings |= kPGPRecipientSettingsSDA;

					if( wipeOriginal != 0 )
						*userSettings |= kPGPRecipientSettingsWipeOriginal;
						
					if( fYEO != 0 )
						*userSettings |= kPGPRecipientSettingsFYEO;

					if( macBinary == kMacBinaryPopupNoneMenuItem )
					{	
						*userSettings |= kPGPUISettingsNoMacBinary;
					}
					else if( macBinary == kMacBinaryPopupSmartMenuItem )
					{
						*userSettings |= kPGPUISettingsSmartMacBinary;
					}
					else
					{
						*userSettings |= kPGPUISettingsForceMacBinary;
					}
                    SetMacBinaryPopupValue( macBinary, clientPrefsRef );
				}

				if( IsPGPError( err ) )
				{
					if( PGPKeyDBRefIsValid( *actualRecipients ) )
					{
						PGPFreeKeyDB( *actualRecipients );
						*actualRecipients = kInvalidPGPKeyDBRef;
					}
					
					if( IsntNull( newKeys ) && PGPKeyDBRefIsValid( *newKeys ) )
					{
						PGPFreeKeyDB( *newKeys );
						*newKeys = kInvalidPGPKeyDBRef;
					}
				}
			}
		}
		
		if( IsntNull( ksEntries ) )
			PGPFreeKeyServerList( ksEntries );

		if( IsntNull( serverList ) )
		{
			for( serverIndex = 0; serverIndex < numKSEntries; serverIndex++ )
			{
				if( PGPKeyServerRefIsValid( serverList[serverIndex].server ) )
					PGPFreeKeyServer( serverList[serverIndex].server );
			}

			PGPFreeData( serverList );
		}
		
		PGPFreeOptionList( recipientOptions );
	}
		
	return( err );
}

	PGPError
PGPclSelectKeysDialog(
	PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
	PGPSelectKeysSettings		settings,
	const char					*prompt,
	PGPKeyDBRef					fromDB,
	PGPKeyDBRef					validityDB,		// can be NULL
	PGPKeySetRef				*selectedSet)	// Caller is responsible for
                                                // freeing selectedSet
{
	PGPError		theError = kPGPError_NoErr;
	CKeySelect		*keySelector;

	PGPValidatePtr( selectedSet );
	*selectedSet	= kInvalidPGPKeySetRef;
	PGPValidateParam( PGPContextRefIsValid( inContext ) );
	PGPValidateParam( PGPKeyDBRefIsValid( fromDB ) );
	
	keySelector = [[CKeySelect alloc] initKeySelect:[NSString stringWithCString:prompt]
					fromSet:PGPPeekKeyDBRootKeySet(fromDB)
					mainDB:validityDB
					prefRef:inPrefRef
					usingFlags:settings
					owner:nil
					didEndSelector:nil];
                    
    theError = [keySelector runKeySelectModalDialog:selectedSet];

    [keySelector release];
	return( theError );
}

	static PGPError
pgpClientDecryptionPassphraseDialog(
	PGPContextRef		context,
	PGPPrefRef			clientPrefsRef,				
	PGPtlsContextRef	tlsContext,
	const char			*prompt,						
	PGPKeySetRef		recipientKeys,
	const PGPKeyID		keyIDList[],		
	PGPUInt32			keyIDCount,
	PGPOptionListRef	additionalOptions,
	char				**passphrase,
	PGPKeyDBObjRef 		*decryptionKey,
	PGPKeyDBRef			*newKeys)
{
	PGPError						err = kPGPError_NoErr;
	PGPOptionListRef				optionList 	= kInvalidPGPOptionListRef;
	PGPKeyServerSpec				*serverList = NULL;
	PGPUInt32						serverIndex = 0;
	PGPUInt32						numKSEntries = 0;
    			
	err = PGPBuildOptionList( context, &optionList,
				PGPOUIOutputPassphrase( context, passphrase ),
				IsntNull( prompt ) ? PGPOUIDialogPrompt( context, prompt ) : 
					PGPONullOption( context ),
				PGPOptionListRefIsValid( additionalOptions ) ?
					additionalOptions : PGPONullOption( context ),
				PGPOLastOption( context ) );
	if( IsntPGPError( err ) && IsntNull( newKeys ) )
	{
		PGPKeyServerEntry	*ksEntries 	= NULL;

		err = PGPCreateKeyServerPath( clientPrefsRef, "", &ksEntries,
					&numKSEntries);
		if( IsntPGPError( err ) && IsntNull( ksEntries ) && numKSEntries > 0 )
		{
			serverList = (PGPKeyServerSpec *) PGPNewData(
								PGPPeekContextMemoryMgr( context ),
								numKSEntries * sizeof( *serverList ),
								kPGPMemoryMgrFlags_Clear );
			if( IsntNull( serverList ) )
			{
				for( serverIndex = 0; serverIndex < numKSEntries;
							serverIndex++ )
				{
					err = PGPNewKeyServer( context,
								ksEntries[serverIndex].type,
								&serverList[serverIndex].server,
								PGPONetHostName( context,
									ksEntries[serverIndex].serverDNS,
									ksEntries[serverIndex].serverPort ),
								PGPOKeyServerProtocol( context,
									ksEntries[serverIndex].protocol ),
								PGPOKeyServerAccessType( context,
									kPGPKeyServerAccessType_Normal ),
								PGPOKeyServerKeySpace( context,
									kPGPKeyServerKeySpace_Normal ),
								PGPOLastOption( context ) );
					if( IsPGPError( err ) )
						break;
					
					serverList[serverIndex].serverName =
							ksEntries[serverIndex].serverDNS;
				}
				
				if( IsntPGPError( err ) )
				{
					err = PGPAppendOptionList( optionList,
								PGPOUIKeyServerUpdateParams( context,
									numKSEntries, serverList, tlsContext,
									FALSE, newKeys, PGPOLastOption( context ) ),
								PGPOLastOption( context ) );
				}
			}
			else
			{
				err = kPGPError_OutOfMemory;
			}
		}

		if( IsntNull( ksEntries ) )
			PGPDisposeKeyServerPath( ksEntries );
	}
	
	if( IsntPGPError( err ) )
	{
		err = PGPDecryptionPassphraseDialog( context, recipientKeys,
					keyIDCount, keyIDList, decryptionKey, optionList,
					PGPOLastOption( context ) );
	}
	
	if( PGPOptionListRefIsValid( optionList ) )
		PGPFreeOptionList( optionList );
	
	if( IsntNull( serverList ) )
	{
		for( serverIndex = 0; serverIndex < numKSEntries; serverIndex++ )
		{
			if( PGPKeyServerRefIsValid( serverList[serverIndex].server ) )
				PGPFreeKeyServer( serverList[serverIndex].server );
		}

		PGPFreeData( serverList );
	}
	
	return( err );
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
