/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: pgpKeyServerDialogs.m,v 1.6 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import "pgpPFLPriv.h"
#import "pgpKeyServerPrefs.h"
#import "pgpClientLibOSX.h"

#import <PGP/PGP.h>
#import <Cocoa/Cocoa.h>

#define kMaxUserIDLength				256

	static PGPError
sQueryServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			clientPrefsRef,
	const PGPKeyID*		keyID,         
	PGPKeyDBObjRef		keyRef,
	const char*			domain,        
	PGPKeyDBRef*		resultKeys)
{
	PGPError			err = kPGPError_NoErr;
	PGPKeyServerEntry*	ksEntries = NULL;
	PGPUInt32			numKSEntries;
	PGPKeyServerSpec*	serverList;
	PGPUInt32			serverIndex;
	PGPOptionListRef	searchObject;
	
	PGPValidatePtr(resultKeys);
	PGPValidatePtr(context);
	PGPValidatePtr(domain);

	*resultKeys = kInvalidPGPKeyDBRef;

	err = PGPCreateKeyServerPath(clientPrefsRef, domain, &ksEntries, &numKSEntries);
	if (IsntPGPError(err) && IsntNull(ksEntries) && (numKSEntries > 0))
	{
		serverList = (PGPKeyServerSpec*) PGPNewData(PGPPeekContextMemoryMgr(context),
												numKSEntries * sizeof(PGPKeyServerSpec),
												kPGPMemoryMgrFlags_Clear);
		
		if (!serverList)
			err = kPGPError_OutOfMemory; CKERR;

		for (serverIndex = 0; serverIndex < numKSEntries; ++serverIndex)
		{
			err = PGPNewKeyServer(context,
						ksEntries[serverIndex].type,
						&serverList[serverIndex].server,
						PGPONetHostName(context, ksEntries[serverIndex].serverDNS,
								ksEntries[serverIndex].serverPort),
						PGPOKeyServerProtocol(context, ksEntries[serverIndex].protocol),
						PGPOKeyServerAccessType(context, kPGPKeyServerAccessType_Normal),
						PGPOKeyServerKeySpace(context, kPGPKeyServerKeySpace_Normal),
						PGPOLastOption(context));
			if (IsPGPError(err))
				break;

			serverList[serverIndex].serverName = ksEntries[serverIndex].serverDNS;
			serverList[serverIndex].serverDomain = ksEntries[serverIndex].domain;
		}
	
		if (IsntPGPError(err))
		{
			if (IsntNull(keyID))
			{
				pgpAssert(!PGPKeyDBObjRefIsValid(keyRef));

				searchObject = PGPOUIKeyServerSearchKeyIDList(context, 1, keyID);
			}
			else
			{
				pgpAssert(IsNull(keyID));

				searchObject = PGPOUIKeyServerSearchKey(context, keyRef);
			}

			err = PGPSearchKeyServerDialog(context,
						numKSEntries,
						serverList,
						tlsContext,
						FALSE,
						resultKeys,
						searchObject,
						PGPOLastOption(context));
		}
			
		for (serverIndex = 0; serverIndex < numKSEntries; ++serverIndex)
		{
			if (PGPKeyServerRefIsValid(serverList[serverIndex].server))
				PGPFreeKeyServer(serverList[serverIndex].server);
		}

		PGPFreeData(serverList);
	}

done:
	if (IsntNull(ksEntries))
		PGPDisposeKeyServerPath(ksEntries);

	return err;
}

	static PGPError
sGetKeyFromServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			clientPrefsRef,
	PGPKeyID const*		keyID,
	PGPKeyDBObjRef		key,
	PGPKeyDBRef*		resultKeys)
{
	PGPError		err = kPGPError_NoErr;
	char			domainStr[kMaxUserIDLength];
	char			userID[kMaxUserIDLength];
	PGPSize			len;
	
	err = PGPKeyServerInit(); CKERR;

	domainStr[0] = '\0';
        
	if (IsntNull(key))
	{
		/* If we know the domain, let's see if we have a keyserver for that domain */
		len = kMaxUserIDLength;
		err = PGPGetPrimaryUserIDName(key, userID, sizeof(userID) - 1, &len);
		
		userID[len] = '\0';
        
		PGPFindEmailDomain(userID, domainStr);
	}

	if (IsntPGPError(err))
		err = sQueryServer(context, tlsContext, clientPrefsRef, keyID, key, domainStr, resultKeys);
	
	PGPKeyServerCleanup();

done:
	return err;
}

	static PGPError
sSendKeyToServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			clientPrefsRef,
	PGPKeyDBObjRef		key,
	PGPKeyServerEntry*	targetServer)
{       
	PGPKeySetRef		singleKeySet;
	PGPKeySetRef		failedKeys;
	char				userID[kMaxUserIDLength];
	char				domain[kMaxUserIDLength];
	PGPSize				len;
	PGPKeyServerEntry*	ksEntries;
	PGPUInt32			numKSEntries;
	PGPKeyServerEntry	serverInfo;
	PGPKeyServerSpec	serverSpec;
	PGPBoolean			foundServer = FALSE;
	PGPError			err = kPGPError_NoErr;
                                        
	PGPValidatePtr(key);          
	PGPValidatePtr( context );      

	if (IsntNull(targetServer))
	{
		pgpCopyMemory(targetServer, &serverInfo, sizeof(PGPKeyServerEntry));
		foundServer = TRUE;
	}
	else
	{
		domain[0] = '\0';
		len = kMaxUserIDLength;
		
		err = PGPGetPrimaryUserIDName(key, userID, sizeof(userID) - 1, &len);
		pgpAssertNoErr(err);
		
        userID[len] = 0;
		PGPFindEmailDomain(userID, domain);
		
		err = PGPCreateKeyServerPath(clientPrefsRef, domain, &ksEntries, &numKSEntries);
		pgpAssertNoErr(err);
		
        if (IsntPGPError(err))
		{
			if(IsntNull(ksEntries) && (numKSEntries > 0))
			{
				pgpCopyMemory(&ksEntries[0], &serverInfo, sizeof(PGPKeyServerEntry));
				foundServer = TRUE;
			}
			
			PGPDisposeKeyServerPath(ksEntries);
		}
	}

	if (IsntPGPError(err) && foundServer)
	{
		pgpClearMemory(&serverSpec, sizeof(serverSpec));

		serverSpec.serverName = serverInfo.serverDNS;

		err = PGPNewOneKeySet(key, &singleKeySet);
		pgpAssertNoErr(err);

		err = PGPNewKeyServer(context,
					serverInfo.type,
					&serverSpec.server,
					PGPONetHostName(context, serverInfo.serverDNS, serverInfo.serverPort),
					PGPOKeyServerProtocol(context, serverInfo.protocol),
					PGPOKeyServerAccessType(context, kPGPKeyServerAccessType_Normal),
					PGPOKeyServerKeySpace(context, kPGPKeyServerKeySpace_Normal),
					PGPOLastOption(context));
		pgpAssertNoErr(err);

		if (IsntPGPError(err))
		{
			err = PGPSendToKeyServerDialog(
						context,
						&serverSpec,
                        tlsContext,
						singleKeySet,
						&failedKeys,
                        PGPOLastOption(context));

			if (PGPKeySetRefIsValid(failedKeys))
				PGPFreeKeySet(failedKeys);

			PGPFreeKeyServer(serverSpec.server);
		}

		PGPFreeKeySet(singleKeySet);
	}

	return err;
}

	PGPError
PGPclGetKeyFromServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPKeyDBObjRef		key,
	PGPKeyDBRef			*resultDB)
{
	PGPError			err = kPGPError_NoErr;

	/* We let sGetKeyFromServer do further argument checking. */
	PGPValidatePtr(resultDB);
	
	*resultDB = kInvalidPGPKeyDBRef;

	err = sGetKeyFromServer(context, tlsContext, prefRef, NULL, key, resultDB);

	return err;
}

	PGPError
PGPclGetKeyIDFromServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPKeyID const*		keyID,
	PGPKeyDBRef*		resultDB)
{
	PGPError			err = kPGPError_NoErr;

	/* We let sGetKeyFromServer do further argument checking. */
	PGPValidatePtr(resultDB);
	
	*resultDB = kInvalidPGPKeyDBRef;

	err = sGetKeyFromServer(context, tlsContext, prefRef, keyID, kInvalidPGPKeyDBObjRef, resultDB);

	return err;
}

	PGPError
PGPclSendKeyToServer(   
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPKeyDBObjRef		key,
	PGPKeyServerEntry*	targetServer)
{                       
	PGPError			err = kPGPError_NoErr;
	
	/* We let sSendKeyToServer do further argument checking. */
	PGPValidatePtr(key);
                
	err = sSendKeyToServer(context, tlsContext, prefRef, key, targetServer);
	
	return err;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
