/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPDockMenu.m,v 1.13 2002/11/11 10:17:14 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPDockMenu.h"

#define kPGPDockMenu_AboutMenuItem				NSLocalizedString(@"About PGP...", @"")
#define kPGPDockMenu_PreferencesMenuItem		NSLocalizedString(@"Preferences...", @"")
#define kPGPDockMenu_PurgeMenuItem				NSLocalizedString(@"Purge Passphrase Cache", @"")
#define kPGPDockMenu_DiskMenuItem				NSLocalizedString(@"PGPdisk", @"")
#define kPGPDockMenu_KeysMenuItem				NSLocalizedString(@"PGPkeys", @"")
#define kPGPDockMenu_MailMenuItem				NSLocalizedString(@"PGPmail", @"")
#define kPGPDockMenu_NetMenuItem				NSLocalizedString(@"PGPnet", @"")
#define kPGPDockMenu_ClipboardMenuItem			NSLocalizedString(@"Clipboard", @"")
#define kPGPDockMenu_ClipboardEncrypt			NSLocalizedString(@"Encrypt", @"")
#define kPGPDockMenu_ClipboardSign				NSLocalizedString(@"Sign", @"")
#define kPGPDockMenu_ClipboardEncryptSign		NSLocalizedString(@"Encrypt & Sign", @"")
#define kPGPDockMenu_ClipboardDecryptVerify		NSLocalizedString(@"Decrypt/Verify", @"")

#define kPGPPrefsWindowTitle					NSLocalizedString(@"Preferences", @"")

static CPGPDockMenu*	sSharedInstance = nil;

@implementation CPGPDockMenu

+ (CPGPDockMenu*)dockMenuDelegate
{
	return sSharedInstance;
}

- (IBAction)about:(id)sender
{
	[NSApp activateIgnoringOtherApps:YES];
	[mInterface about:sender];
}

- (IBAction)decryptVerify:(id)sender
{
	[mMailController clipboardDecryptVerify:sender];
}

- (IBAction)encrypt:(id)sender
{
	[mMailController clipboardEncrypt:sender];
}

- (IBAction)encryptSign:(id)sender
{
	[mMailController clipboardEncryptSign:sender];
}

- (IBAction)sign:(id)sender
{
	[mMailController clipboardSign:sender];
}

- (IBAction)disk:(id)sender
{
	[mInterface disk:sender];
	[NSApp activateIgnoringOtherApps:YES];
}

- (IBAction)keys:(id)sender
{
	[mInterface keys:sender];
	[NSApp activateIgnoringOtherApps:YES];
}

- (IBAction)mail:(id)sender
{
	[mInterface mail:sender];
	[NSApp activateIgnoringOtherApps:YES];
}

- (IBAction)net:(id)sender
{
	[mInterface net:sender];
	[NSApp activateIgnoringOtherApps:YES];
}

- (IBAction)preferences:(id)sender
{
	[NSApp activateIgnoringOtherApps:YES];
	[mInterface showPreferences:sender];
}

- (IBAction)purgePassphraseCache:(id)sender
{
	[mAppController purgePassphraseCache];
}

- (BOOL)validateMenuItem:(NSMenuItem*)item
{
	if (!item || ![item action])
		return NO;
	
	if ([[item title] isEqualToString:kPGPDockMenu_AboutMenuItem])
	{
		if (!mInterface || [NSApp modalWindow] || ![mAppController finishedInitialization])
			return NO;
		
		return YES;
	}

	if ([[item title] isEqualToString:kPGPDockMenu_PreferencesMenuItem])
	{
		if (!mInterface || ![mAppController finishedInitialization] || [NSApp modalWindow] && ![[[NSApp modalWindow] title] isEqualToString:kPGPPrefsWindowTitle])
			return NO;
		
		return YES;
	}

	if ([[item title] isEqualToString:kPGPDockMenu_PurgeMenuItem])
	{
		if ([NSApp modalWindow] || ![mAppController finishedInitialization])
			return NO;
		
		return YES;
	}
	
	if ([[item title] isEqualToString:kPGPDockMenu_DiskMenuItem])
	{
		if (mDiskController)
			return YES;
	}
		
	if ([[item title] isEqualToString:kPGPDockMenu_KeysMenuItem])
	{
		if (mKeysController)
			return YES;
	}

	if ([[item title] isEqualToString:kPGPDockMenu_MailMenuItem])
	{
		if (mMailController)
			return YES;
	}
	
	if ([[item title] isEqualToString:kPGPDockMenu_NetMenuItem])
	{
		if (mNetController)
			return YES;
	}

	if ([[item title] isEqualToString:kPGPDockMenu_ClipboardMenuItem])
		return YES;

    if ([[item title] isEqualToString:kPGPDockMenu_ClipboardEncrypt])
	{
		if (!mInterface || [NSApp modalWindow])
			return NO;
		
		return YES;
	}
    
    if ([[item title] isEqualToString:kPGPDockMenu_ClipboardSign])
	{
		if (!mInterface || [NSApp modalWindow])
			return NO;
		
		return YES;
	}
    
    if ([[item title] isEqualToString:kPGPDockMenu_ClipboardEncryptSign])
	{
		if (!mInterface || [NSApp modalWindow])
			return NO;
		
		return YES;
	}
    
    if ([[item title] isEqualToString:kPGPDockMenu_ClipboardDecryptVerify])
	{
		if (!mInterface || [NSApp modalWindow])
			return NO;
		
		return YES;
	}

	return NO;
}

- (void)updateMenu
{
	mDiskController = [mBundleManager bundleByName:@"PGPdisk.bundle"];
	mKeysController = [mBundleManager bundleByName:@"PGPkeys.bundle"];
	mMailController = [mBundleManager bundleByName:@"PGPmail.bundle"];
	mNetController = [mBundleManager bundleByName:@"PGPnet.bundle"];
	
	if (mDiskController || mKeysController || mMailController || mNetController)
	{
		if (![mSeparator2 menu])
			[mDockMenu addItem:mSeparator2];
		
		if (![mSeparator3 menu])
			[mDockMenu addItem:mSeparator3];
	}
	
	if (mDiskController && ![mPGPdisk menu])
		[mDockMenu insertItem:mPGPdisk atIndex:[mDockMenu indexOfItem:mSeparator3]];
	
	if (mKeysController && ![mPGPkeys menu])
		[mDockMenu insertItem:mPGPkeys atIndex:[mDockMenu indexOfItem:mSeparator3]];
	
	if (mMailController && ![mPGPmail menu])
		[mDockMenu insertItem:mPGPmail atIndex:[mDockMenu indexOfItem:mSeparator3]];
	
	if (mNetController && ![mPGPnet menu])
		[mDockMenu insertItem:mPGPnet atIndex:[mDockMenu indexOfItem:mSeparator3]];
	
	if (mMailController && ![mClipboard menu])
		[mDockMenu addItem:mClipboard];
}

- (void)awakeFromNib
{
	mAppController = [NSApp delegate];
	mBundleManager = [mAppController bundleManager];
	mInterface = [mAppController interface];

	sSharedInstance = self;
	
	mDiskController = [mBundleManager bundleByName:@"PGPdisk.bundle"];
	mKeysController = [mBundleManager bundleByName:@"PGPkeys.bundle"];
	mMailController = [mBundleManager bundleByName:@"PGPmail.bundle"];
	mNetController = [mBundleManager bundleByName:@"PGPnet.bundle"];

	[mAbout retain];
	[mPreferences retain];
	[mSeparator1 retain];
	[mPurgePassphrase retain];
	[mSeparator2 retain];
	[mPGPdisk retain];
	[mPGPkeys retain];
	[mPGPmail retain];
	[mPGPnet retain];
	[mSeparator3 retain];
	[mClipboard retain];
	
	if (!mDiskController)
		[mDockMenu removeItem:mPGPdisk];
	else
		[mPGPdisk setSubmenu:[mDiskController dockMenu]];
	
	if (!mKeysController)
		[mDockMenu removeItem:mPGPkeys];
	//else
	//	[mPGPkeys setSubmenu:[mKeysController dockMenu]];
	
	if (!mMailController)
	{
		[mDockMenu removeItem:mPGPmail];
		[mDockMenu removeItem:mClipboard];
	}
	else
		[mPGPmail setSubmenu:[mMailController dockMenu]];
	
	if (!mNetController)
		[mDockMenu removeItem:mPGPnet];
	else
		[mPGPnet setSubmenu:[mNetController dockMenu]];
	
	if (!mDiskController && !mKeysController && !mMailController && !mNetController)
	{
		[mDockMenu removeItem:mSeparator2];
		[mDockMenu removeItem:mSeparator3];
	}
	
	[mDockMenu setAutoenablesItems:YES];
}

- (void)dealloc
{
	[mAbout release];
	[mPreferences release];
	[mSeparator1 release];
	[mPurgePassphrase release];
	[mSeparator2 release];
	[mPGPdisk release];
	[mPGPkeys release];
	[mPGPmail release];
	[mPGPnet release];
	[mSeparator3 release];
	[mClipboard release];
}

@end
