/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPInstaller.h,v 1.33 2002/08/06 20:09:21 dallen Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>
#import <Security/Authorization.h>
#import <Security/AuthorizationTags.h>

@protocol PGPInstaller

- (void)configureInstallChecklist;
- (void)updateControls:(NSDictionary*)controls;
- (void)setInstallMessage;
- (void)startPGPnet;
- (void)stopProgressBar;
- (void)installCompleted;
- (void)tearDownConnection;

@end

@interface CPGPInstaller : NSObject <PGPInstaller>
{
	AuthorizationRef                mAuthRef;
	AuthorizationRights             mRights;
	AuthorizationFlags              mFlags;
	
	/* Window outlets */
	IBOutlet NSWindow*				mWindow;
	IBOutlet NSImageView*           mImageView;
	IBOutlet NSTextField*           mPanelBanner;
	IBOutlet NSBox*                 mPanelBox;
	IBOutlet NSButton*              mMultiPurposeButton;
	
	/* Menu outlets */
	IBOutlet NSMenu*				mMenu;
	
	/* Install panels */
	IBOutlet NSBox*                 mAuthorizationPanel;
	IBOutlet NSBox*                 mLicensePanel;
	IBOutlet NSBox*                 mPreInstallPanel;
	IBOutlet NSBox*                 mInstallationPanel;
	IBOutlet NSBox*                 mKeysPanel;
	IBOutlet NSBox*                 mFinishUpPanel;
	
	/* Panel outlets */
	IBOutlet NSTextField*           mAuthorizationItem;
	IBOutlet NSTextField*           mLicenseItem;
	IBOutlet NSTextField*           mInstallingItem;
	IBOutlet NSTextField*           mFinishUpItem;
	IBOutlet NSImageView*           mAuthorizationItemImage;
	IBOutlet NSImageView*           mLicenseItemImage;
	IBOutlet NSImageView*           mInstallingItemImage;
	IBOutlet NSImageView*           mFinishUpItemImage;
	IBOutlet NSTextField*           mAuthMessage;
	IBOutlet NSTextField*           mLockText;
	IBOutlet NSButton*              mLockButton;
	IBOutlet NSTextView*            mLicenseView;
	IBOutlet NSScrollView*          mLicenseScrollView;
	IBOutlet NSWindow*              mLicenseSheet;
	IBOutlet NSButton*              mAgree;
	IBOutlet NSButton*              mDisagree;
	IBOutlet NSBox*                 mChecklistBox;
	IBOutlet NSBox*                 mDividingLine;
	IBOutlet NSTextField*           mInstallComplete;
	IBOutlet NSTextField*           mItemName;
	IBOutlet NSTextField*           mStatus;
	IBOutlet NSTextField*           mInstallMessage;
	IBOutlet NSTextField*           mProgressText;
	IBOutlet NSProgressIndicator*   mProgressIndicator;
	
	NSImage*						mLockImage;
	NSImage*						mUnlockImage;
	NSImage*						mBlueBall;
	NSImage*						mGrayBall;
	NSMutableDictionary*			mInstallInfo;
	NSMutableDictionary*			mControlsDictionary;
	NSConnection*					mConnection;
	unsigned						mCurrentPanelID;
	
	BOOL							mInstallCompleted;
	BOOL							mLoadedNib;
	BOOL							mInited;
}

- (id)init;
- (void)loadNib;

- (void)installIfNecessary;
- (void)createInstallDictionary;

- (BOOL)isPGPdiskPresent;
- (BOOL)isPGPkeysPresent;
- (BOOL)isPGPmailPresent;
- (BOOL)isPGPnetPresent;
- (BOOL)isPGPInstalled;
- (BOOL)isPGPdiskInstalled;
- (BOOL)isPGPkeysInstalled;
- (BOOL)isPGPmailInstalled;
- (BOOL)isPGPnetInstalled;

- (BOOL)isAdmin;
- (BOOL)haveAuthorization;
- (BOOL)obtainAuthorization;
- (BOOL)isProductInstalled;

- (void)setAuthText:(NSString*)string;
- (void)updateInstallItems;
- (void)configureInstallChecklist;
- (void)updateControls:(NSDictionary*)controls;
- (void)setInstallMessage;
- (void)startPGPnet;
- (void)stopProgressBar;
- (void)installCompleted;
- (void)configureItemField:(NSTextField*)item statusField:(NSTextField*)status checkField:(NSTextField*)check key:(NSString*)key;
- (char**)installtoolArguments;
- (void)communicateWithInstallTool:(NSString*)tool;
- (void)tearDownConnection;

- (IBAction)lock:(id)sender;
- (IBAction)close:(id)sender;
- (IBAction)gotoLicense:(id)sender;
- (IBAction)displayLicenseSheet:(id)sender;
- (IBAction)closeLicenseAgreement:(id)sender;
- (IBAction)gotoPreInstall:(id)sender;
- (IBAction)install:(id)sender;
- (IBAction)finish:(id)sender;

- (void)licenseAgreementDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)windowWillClose:(NSNotification*)notification;

- (void)startPGP;
- (void)dealloc;

@end

@interface PGPAppController : NSObject
{
	BOOL			mFinishedLaunching;
	BOOL			mDisplayedOpenAlert;
	BOOL			mInited;
}

- (id)init;
- (void)applicationDidFinishLaunching:(NSNotification*)notification;
- (BOOL)application:(NSApplication*)app openFile:(NSString*)filename;

- (BOOL)isRunningOnReadOnlyFS;
- (void)displayExpirationAlertAndExit;
- (void)displayReadOnlyAlertAndExit;

- (void)awakeFromNib;

@end

@interface PGPDocument : NSDocument
{
}

- (NSData*)dataRepresentationOfType:(NSString*)type;
- (BOOL)readFromFile:(NSString*)filename ofType:(NSString*)type;

@end