/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPInstaller.m,v 1.64 2002/08/06 20:09:21 dallen Exp $
____________________________________________________________________________*/

#import "CPGPInstaller.h"
#import "pgpBuild.h"
#import <pwd.h>
#import <grp.h>
#import <time.h>
#import <unistd.h>
#import <sys/un.h>
#import <sys/socket.h>
#import <sys/syslimits.h>
#import <sys/types.h>
#import <sys/stat.h>

#if PGP_DEBUG
#define DebugLog(x) NSLog x
#else
#define DebugLog(x)
#endif

#define kPGPAppString_CantOpenDocument				NSLocalizedString(@"Can't Open Document", @"")
#define kPGPAppString_CantOpenDocumentMsg			NSLocalizedString(@"The specified document could not be opened because PGP hasn't been installed. Click the Install button to begin the installation process.", @"")
#define kPGPAppString_BetaExpiration				NSLocalizedString(@"Beta Expiration", @"")
#define kPGPAppString_ExpirationMessage				NSLocalizedString(@"This beta version of PGP has expired. \
Please contact your software vendor to obtain a licensed version.", @"")
#define kPGPAppString_ReadOnlyFS					NSLocalizedString(@"Read-Only File System", @"")
#define kPGPAppString_ReadOnlyFSMessage				NSLocalizedString(@"PGP requires write access to the file system in which it resides. It is recommended that you copy PGP to the Applications folder of your main volume.", @"")
#define kPGPAppString_Quit							NSLocalizedString(@"Quit", @"")
#define kPGPInstallerString_NeedAuth				NSLocalizedString(@"PGP requires administrative access in order to function properly. To continue with the installation process, click the lock icon below and provide the administrator password for this computer.", @"Need authorization message")
#define kPGPInstallerString_HaveAuth				NSLocalizedString(@"Please click the Continue button to view PGP's software license", @"Have authorization message")
#define kPGPInstallerString_MailServicesMessage		NSLocalizedString(@"Please note that the PGP services may not appear in the Services menu until your next login session.", @"Services warning")
#define kPGPInstallerString_ServicesNotAvailable	NSLocalizedString(@"PGP services will not appear in the Services menu unless it is installed in the Applications folder.", @"Other services warning")
#define kPGPInstallerString_NotAdmin				NSLocalizedString(@"PGP requires administrative access in order to function properly. You do not have the sufficient privileges to install PGP. Please contact your machine's administrator.", @"Not An Admin")

#define kPGPPrefsDomain								@"com.pgp.PGP"
#define kPGPAppDefault_CheckExpiration				@"checkExpiration"
#define kPGPInstaller_ConnectionName				@"PGPinstaller.connection"
#define kPGPInstaller_ItemControlKey				@"Item"
#define kPGPInstaller_StatusControlKey				@"Status"
#define kPGPInstaller_CheckControlKey				@"Check"
#define kPGPInstaller_LaunchPath     				@"installtool"
#define kPGPInstaller_LibraryMailBundles			@"/Library/Mail/Bundles"
#define kPGPInstaller_PGPmailViewer					@"/Library/Mail/Bundles/PGPmail.mailbundle"
#define kPGPInstaller_LibraryFrameworks				@"/Library/Frameworks"
#define kPGPInstaller_PGPFramework					@"/Library/Frameworks/PGP.framework"
#define kPGPInstaller_UILib							@"/Library/Frameworks/PGPui.framework"
#define kPGPInstaller_ClientLib						@"/Library/Frameworks/PGPclient.framework"
#define kPGPInstaller_SDKService					@"/Library/Frameworks/PGP.framework/Resources/PGPservice.app/Contents/MacOS/PGPservice"
#define kPGPInstaller_CheckWidth					13
#define kPGPInstaller_ListItemWidth					58
#define kPGPInstaller_ListItemHeight				14
#define kPGPInstaller_ListItemVPadding				8
#define kPGPInstaller_AuthTextWidth					363
#define kPGPInstaller_AuthTextHeight				42
#define kPGPInstaller_GroupWheel					0
#define kPGPInstaller_GroupAdmin					80

enum
{       
	kPGPInstaller_AuthPanel             = 7777,
	kPGPInstaller_LicensePanel,
	kPGPInstaller_PreInstallPanel,
	kPGPInstaller_InstallPanel,
	kPGPInstaller_KeySelectionPanel,
	kPGPInstaller_FinishUpPanel
};

enum
{   
	kPGPInstaller_LicenseAgree          = 3333,
	kPGPInstaller_LicenseDisagree
};

static int sCompareInstallItems(id item1, id item2, void* context)
{
    return [item1 caseInsensitiveCompare:item2];
}

static BOOL sIsOSSupported(void)
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	BOOL					osIsSupported = YES;
    double					os10_1 = 620.0;

    if (NSAppKitVersionNumber < os10_1)
        osIsSupported = NO;
    
	if (!osIsSupported)
	{
		NSRunAlertPanel(NSLocalizedString(@"Unsupported OS version", @""),
			[NSString stringWithFormat:NSLocalizedString(@"PGP found that you are running a version of Mac OS X which is older than Mac OS X version 10.1. PGP requires Mac OS X version 10.1 or greater in order to function properly. It is recommended that you upgrade to the latest version of Mac OS X.", @"")],
			NSLocalizedString(@"Quit", @""),
			nil,
			nil);
	}

	[pool release];
	
	return osIsSupported;
}

static BOOL sTranslateDate(unsigned* month, unsigned* day, unsigned* year, char* dateString)
{
	char*		dateChunk;
	char*		tempBuffer;
	unsigned	bufferLen;
	
	if (!dateString || !year || !day || !month)
		return NO;

	*day = 0;
	*month = 0;
	*year = 0;

	bufferLen = strlen(dateString) + 1;
	tempBuffer = malloc(bufferLen);
	
	if (!tempBuffer)
		return NO;

	strncpy(tempBuffer, dateString, bufferLen);

	if (dateChunk = strtok(tempBuffer, "/"))
	{
		if (*dateChunk == '0')
			++dateChunk;
		
		*month = strtoq(dateChunk, NULL, 0);
	}
	
	if (dateChunk = strtok(NULL, "/"))
	{
		if (*dateChunk == '0')
			++dateChunk;
		
		*day = strtoq(dateChunk, NULL, 0);
	}
	
	if (dateChunk = strtok(NULL, "/"))
	{
		if (*dateChunk == '0')
			++dateChunk;
		
		*year = strtoq(dateChunk, NULL, 0);
	}
	
	free(tempBuffer);
	
	return YES;
}

static BOOL sIsExpired(void)
{
	unsigned		month, day, year;
	struct tm       tm;
	struct tm*		ptm;
	time_t			timeNow;
	time_t			timeOut;

#if PGP_BETA && PGP_BETA_DAYS

	/* Translate the compile date (MM/DD/YY), into a usable form. */
	if (!sTranslateDate (&month, &day, &year, __DATE__))
		return YES;

	/* Seed the tm structure with the current time. */
	time(&timeNow);
	ptm = localtime(&timeNow);
	memcpy(&tm, ptm, sizeof(tm));

	/* Set the dates to the compile time. */
	tm.tm_mon = month - 1;
	tm.tm_mday = day; 
	tm.tm_year = year + 100;

	/* Turn the compile time into number of secs since 1/1/1970. */
	timeOut = mktime(&tm);

	timeOut += PGP_BETA_DAYS * 24 * 60 * 60;

	if (timeOut < timeNow)
		return YES;

#endif  /* PGP_BETA && PGP_BETA_DAYS */

	return NO;
}

@implementation CPGPInstaller

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mInstallInfo)
		[self createInstallDictionary];
	
	if (!mLoadedNib)
		[self loadNib];
	
	if (!mControlsDictionary)
		mControlsDictionary = [[NSMutableDictionary alloc] init];
	
	if (!mConnection)
	{
		mConnection = [[NSConnection alloc] initWithReceivePort:[NSPort port] sendPort:[NSPort port]];
		[mConnection setRootObject:self];
	}
	
	return self;
}

- (void)loadNib
{
	NSBundle*		bundle = [NSBundle bundleForClass:[self class]];
	NSImage*        pgpLockImage;
	NSImage*        alphaPGPLockImage;
	
	if (mLoadedNib)
		return;
	
	[NSBundle loadNibNamed:@"PGPinstaller" owner:self];
	
	[NSApp setMainMenu:mMenu];
	
	[mWindow setDelegate:self];
	
	mCurrentPanelID = kPGPInstaller_AuthPanel;
	
	mLockImage = [[NSImage alloc] initWithContentsOfFile:
					[bundle pathForImageResource:@"lock"]];
	
	mUnlockImage = [[NSImage alloc] initWithContentsOfFile:
					[bundle pathForImageResource:@"unlock"]];
	
	mBlueBall = [[NSImage alloc] initWithContentsOfFile:
					[bundle pathForImageResource:@"blueball"]];
	
	mGrayBall = [[NSImage alloc] initWithContentsOfFile:
					[bundle pathForImageResource:@"grayball"]];
	
	[mAuthorizationPanel retain];
	[mAuthorizationPanel removeFromSuperview];
	
	[mLicensePanel retain];
	[mLicensePanel removeFromSuperview];
	
	[mPreInstallPanel retain];
	[mPreInstallPanel removeFromSuperview];   
	
	[mInstallationPanel retain];
	[mInstallationPanel removeFromSuperview];
	
	[mKeysPanel retain];
	[mKeysPanel removeFromSuperview];
	
	[mProgressText retain];
	[mProgressText removeFromSuperview];
	
	[mProgressIndicator retain];
	[mProgressIndicator removeFromSuperview];
	
	[mInstallComplete retain];
	[mInstallComplete removeFromSuperview];
	
	[mAuthorizationItem setTextColor:[NSColor disabledControlTextColor]];
	[mLicenseItem setTextColor:[NSColor disabledControlTextColor]];
	[mLicenseView setRichText:YES];
	[mLicenseView setImportsGraphics:NO];
	[mInstallingItem setTextColor:[NSColor disabledControlTextColor]];
	[mFinishUpItem setTextColor:[NSColor disabledControlTextColor]];
	
	[self updateInstallItems];
	
	[mPanelBanner setStringValue:NSLocalizedString(@"Authorization", @"")];
	[mPanelBox setContentView:[mAuthorizationPanel contentView]];
	[mPanelBox displayIfNeeded];
	
	[mMultiPurposeButton setTarget:self];
	
    if ([self isAdmin])
	{
		if ([self haveAuthorization])
		{
			[self setAuthText:kPGPInstallerString_HaveAuth];   
			[mAuthMessage setAlignment:NSCenterTextAlignment];   
			[mLockText setStringValue:NSLocalizedString(@"Click the lock to prevent further changes.", @"")];
			[mLockText displayIfNeeded];
			[mLockButton setImage:mUnlockImage];        
			[mMultiPurposeButton setAction:@selector(gotoLicense:)];
			[mMultiPurposeButton setTitle:NSLocalizedString(@"Continue", @"")];
		}
		else
		{
			[self setAuthText:kPGPInstallerString_NeedAuth];
			[mAuthMessage setAlignment:NSLeftTextAlignment];
			[mLockText setStringValue:NSLocalizedString(@"Click the lock to make changes.", @"")];
			[mLockText displayIfNeeded];
			[mLockButton setImage:mLockImage];
			[mMultiPurposeButton setAction:@selector(close:)];
			[mMultiPurposeButton setTitle:NSLocalizedString(@"Close", @"")];
		}
		
		[mLockButton setEnabled:YES];
		
		pgpLockImage = [[[NSImage alloc] initByReferencingFile:[bundle pathForImageResource:@"pgp"]] autorelease];
		alphaPGPLockImage = [[[NSImage alloc] initWithSize:[pgpLockImage size]] autorelease];
		
		[alphaPGPLockImage lockFocus];
		[pgpLockImage dissolveToPoint:NSZeroPoint fraction:0.25];
		[alphaPGPLockImage unlockFocus];
		
		[mImageView setImage:alphaPGPLockImage];
	}
	else
	{
		[self setAuthText:kPGPInstallerString_NotAdmin];
		[mMultiPurposeButton setAction:@selector(close:)];
		[mMultiPurposeButton setTitle:NSLocalizedString(@"Close", @"")];
		[mLockText removeFromSuperview];
		[mLockButton removeFromSuperview];
	}

	mLoadedNib = YES;
}

- (void)installIfNecessary
{
	if ([self isProductInstalled])
	{
		DebugLog((@"isProductInstalled returned YES"));
		[self startPGP];
	}
	else
	{
		DebugLog((@"isProductInstalled return NO"));
		[mWindow center];
		[mWindow makeKeyAndOrderFront:nil];
	}
}

- (void)createInstallDictionary
{
	if (mInstallInfo)
		return;
	
	mInstallInfo = [[NSMutableDictionary alloc] init];
	
	[mInstallInfo setObject:[NSNumber numberWithBool:[self isPGPInstalled]] forKey:@"PGPapp"];
	
	if ([self isPGPdiskPresent])
		[mInstallInfo setObject:[NSNumber numberWithBool:[self isPGPdiskInstalled]] forKey:@"PGPdisk.bundle"];
	
	if ([self isPGPkeysPresent])
		[mInstallInfo setObject:[NSNumber numberWithBool:[self isPGPkeysInstalled]] forKey:@"PGPkeys.bundle"];
	
	if ([self isPGPmailPresent])
		[mInstallInfo setObject:[NSNumber numberWithBool:[self isPGPmailInstalled]] forKey:@"PGPmail.bundle"];
	
	if ([self isPGPnetPresent])
		[mInstallInfo setObject:[NSNumber numberWithBool:[self isPGPnetInstalled]] forKey:@"PGPnet.bundle"];
	
	DebugLog((@"mInstallInfo = %@", mInstallInfo));
}

- (BOOL)isPGPdiskPresent
{
	return [[NSFileManager defaultManager] fileExistsAtPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPdisk.bundle"]];
}

- (BOOL)isPGPkeysPresent
{
	return [[NSFileManager defaultManager] fileExistsAtPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPkeys.bundle"]];
}

- (BOOL)isPGPmailPresent
{
	return [[NSFileManager defaultManager] fileExistsAtPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPmail.bundle"]];
}

- (BOOL)isPGPnetPresent
{
	return [[NSFileManager defaultManager] fileExistsAtPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPnet.bundle"]];
}

- (BOOL)isPGPInstalled
{
    return NO;
    /*
	NSString*		servicePath = [NSString stringWithString:kPGPInstaller_SDKService];
	NSFileManager*	fileManager = [NSFileManager defaultManager];
	BOOL			isDirectory;
	struct stat		fileStat;
	unsigned		failures = 0;
		
	if (![fileManager fileExistsAtPath:kPGPInstaller_LibraryFrameworks isDirectory:&isDirectory] || !isDirectory)
	{
		DebugLog((@"Missing /Library/Frameworks or path isn't a directory"));
		++failures;
	}
	
	if (![fileManager fileExistsAtPath:kPGPInstaller_PGPFramework isDirectory:&isDirectory] || !isDirectory)
	{
		DebugLog((@"Missing /Library/Frameworks/PGP.framework or path isn't a directory"));
		++failures;
	}
	
	if (![fileManager fileExistsAtPath:kPGPInstaller_UILib isDirectory:&isDirectory] || !isDirectory)
	{
		DebugLog((@"Missing /Library/Frameworks/PGPui.framework or path isn't a directory"));
		++failures;
	}
	
	if (![fileManager fileExistsAtPath:kPGPInstaller_ClientLib isDirectory:&isDirectory] || !isDirectory)
	{
		DebugLog((@"Missing /Library/Frameworks/PGPclient.framework or path isn't a directory"));
		++failures;
	}
	
	if (stat([servicePath cString], &fileStat) == -1)
	{
		DebugLog((@"Missing /Library/Frameworks/PGP.framework/Resources/PGPservice.app/Contents/MacOS/PGPservice"));
		++failures;
	}
	
	if (fileStat.st_uid != 0)
	{
		DebugLog((@"PGPservice isn't owned by root"));
		++failures;
	}
	
	if (access([servicePath cString], 04755) == -1)
	{
		DebugLog((@"PGPservice isn't suid root"));
		++failures;
	}
	
	return failures ? NO : YES;
    */
}

- (BOOL)isPGPdiskInstalled
{
	NSString*		bundlePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPdisk.bundle"];
	NSString*		prefPath;
	NSString*		loadPGPdiskPath;
	struct passwd*	pwent;
	struct stat		fileStat;
	unsigned		failures = 0;
	
	loadPGPdiskPath = [bundlePath stringByAppendingPathComponent:@"Contents/Resources/load_pgpdisk"];
	
	pwent = getpwnam("root");
	
	if (stat([loadPGPdiskPath cString], &fileStat) == -1)
	{
		DebugLog((@"Missing load_pgpdisk"));
		++failures;
	}
	
	if (fileStat.st_uid != pwent->pw_uid)
	{
		DebugLog((@"load_pgpdisk isn't owned by root"));
		++failures;
	}
	
	if (access([loadPGPdiskPath cString], 04755) == -1)
	{
		DebugLog((@"load_pgpdisk isn't suid root"));
		++failures;
	}
	
	prefPath = [[[NSString alloc] initWithContentsOfFile:@"/Library/Preferences/com.pgp.load_pgpdisk"] autorelease];
	if (![prefPath isEqualToString:loadPGPdiskPath])
	{
		DebugLog((@"Contents of /Library/Preferences/com.pgp.load_pgpdisk are invalid"));
		++failures;
	}
	
	return failures ? NO : YES;
}

- (BOOL)isPGPkeysInstalled
{
	return YES;
}

- (BOOL)isPGPmailInstalled
{
	NSFileManager*	fileManager = [NSFileManager defaultManager];
	NSString*		bundlePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPmail.bundle"];
	BOOL			isDirectory;
	unsigned		failures = 0;
	
	if ([fileManager fileExistsAtPath:[bundlePath stringByAppendingPathComponent:@"Contents/Resources/PGPmail.mailbundle"]])
	{
		if (![fileManager fileExistsAtPath:kPGPInstaller_LibraryMailBundles isDirectory:&isDirectory] || !isDirectory)
		{
			DebugLog((@"Missing /Library/Mail/Bundles or path isn't a directory"));
			++failures;
		}
		
		if (![fileManager fileExistsAtPath:kPGPInstaller_PGPmailViewer isDirectory:&isDirectory] || !isDirectory)
		{
			DebugLog((@"Missing /Library/Mail/Bundles/PGPmail.mailbundle or path isn't a directory"));
			++failures;
		}
	}
	
	return failures ? NO : YES;
}

- (BOOL)isPGPnetInstalled
{
	NSFileManager*	fileManager = [NSFileManager defaultManager];
	NSString*		bundlePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/PlugIns/PGPnet.bundle"];
	NSString*		path;
	NSScanner*		scanner;
	NSString*		kextPath;
	NSString*		servicePath;
	NSString*		startupScriptPath;
	NSString*		startupParamsPath;
	struct group*	grp;
	struct stat		fileStat;
	unsigned		failures = 0;
	
	kextPath = [bundlePath stringByAppendingPathComponent:@"Contents/Resources/PGPnet.kext"];
	servicePath = [bundlePath stringByAppendingPathComponent:@"Contents/Resources/PGPnetService"];
	startupScriptPath = @"/Library/StartupItems/PGPnet/PGPnet";
	startupParamsPath = @"/Library/StartupItems/PGPnet/StartupParameters.plist";
	
	if (stat([servicePath cString], &fileStat) == -1)
	{
		DebugLog((@"Missing PGPnetService"));
		++failures;
	}
	
	if (fileStat.st_uid != 0)
	{
		DebugLog((@"PGPnetService isn't owned by root"));
		++failures;
	}
	
	if (access([servicePath cString], 04755) == -1)
	{
		DebugLog((@"PGPnetService isn't suid root"));
		++failures;
	}
	
	grp = getgrnam("admin");
	
	if (stat([startupScriptPath cString], &fileStat) == -1)
	{
		DebugLog((@"Missing /Library/StartupItems/PGPnet/PGPnet"));
		++failures;
	}
	
	if (fileStat.st_uid != 0)
	{
		DebugLog((@"/Library/StartupItems/PGPnet/PGPnet isn't owned by root"));
		++failures;
	}
	
	if (fileStat.st_gid != grp->gr_gid)
	{
		DebugLog((@"/Library/StartupItems/PGPnet/PGPnet isn't group owned by admin"));
		++failures;
	}
	
	if (access([startupScriptPath cString], 0755) == -1)
	{
		DebugLog((@"/Library/StartupItems/PGPnet/PGPnet isn't executable"));
		++failures;
	}
	
	if (![fileManager fileExistsAtPath:startupParamsPath])
	{
		DebugLog((@"Missing /Library/StartupItems/PGPnet/StartupParameters.plist"));
		++failures;
	}
	
	if ([fileManager fileExistsAtPath:startupScriptPath])
	{
		scanner = [NSScanner scannerWithString:[NSString stringWithContentsOfFile:startupScriptPath]];
		[scanner scanUpToString:@"/sbin/kextload \"" intoString:nil];
		
		if ([scanner isAtEnd])
		{
			DebugLog((@"/Library/StartupItems/PGPnet/PGPnet contents are invalid"));
			++failures;
			goto done;
		}
		
		[scanner scanString:@"/sbin/kextload \"" intoString:nil];
		[scanner scanUpToString:@"\"\n" intoString:&path];
		
		if ([scanner isAtEnd])
		{
			DebugLog((@"/Library/StartupItems/PGPnet/PGPnet contents are invalid"));
			++failures;
			goto done;
		}
		
		[scanner scanString:@"\"\n" intoString:nil];
		
		if (![path isEqualToString:kextPath])
		{
			DebugLog((@"/Library/StartupItems/PGPnet/PGPnet contents are invalid"));
			++failures;
			goto done;
		}
		
		[scanner scanString:@"\"" intoString:nil];
		[scanner scanUpToString:@"\" &\n" intoString:&path];

		if (![path isEqualToString:servicePath])
		{
			DebugLog((@"/Library/StartupItems/PGPnet/PGPnet contents are invalid"));
			++failures;
			goto done;
		}
	}
	
done:
	return failures ? NO : YES;
}

- (BOOL)isAdmin
{           
	gid_t       groups[10];
	int         count;
	int			index;
	BOOL        admin = NO;
	
	count = getgroups(10, groups);
	
	for (index = 0; index < count; index++)
	{
		if ((kPGPInstaller_GroupWheel == groups[index]) ||
			(kPGPInstaller_GroupAdmin == groups[index]))
		{
			admin = YES;
			break;
		}
	}   
	
	return admin;
}

- (BOOL)haveAuthorization
{       
	AuthorizationRef    authRef;
	AuthorizationRights authRights;
	AuthorizationFlags  authFlags;
	AuthorizationItem   authItem;
	OSStatus            err;
			
	authRights.count = 0;
	authRights.items = NULL;
		
	authFlags = kAuthorizationFlagDefaults;
			
	err = AuthorizationCreate(&authRights, kAuthorizationEmptyEnvironment, authFlags, &authRef);
	if (err != errAuthorizationSuccess)
		return NO;
	
	authItem.name = kAuthorizationRightExecute;
	authItem.value = (char*) [kPGPInstaller_LaunchPath cString];
	authItem.valueLength = [kPGPInstaller_LaunchPath length];
	authItem.flags = 0;
		
	authRights.count = 1;
	authRights.items = &authItem; 
	
	authFlags = kAuthorizationFlagExtendRights;
	
	err = AuthorizationCopyRights(authRef, &authRights, kAuthorizationEmptyEnvironment, authFlags, NULL);
	
	if (mAuthRef)
		AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
	
	mAuthRef = authRef;
	
	if (err == errAuthorizationSuccess)
		return YES;
	else
		return NO;
}

- (BOOL)obtainAuthorization
{
	AuthorizationItem   authItem;
	OSStatus            err;
	
	if (!mAuthRef)
	{
		if ([self haveAuthorization])
			return YES;
	}
	
	authItem.name = kAuthorizationRightExecute;
	authItem.value = (char*) [kPGPInstaller_LaunchPath cString];
	authItem.valueLength = [kPGPInstaller_LaunchPath length];
	authItem.flags = 0;
	
	mRights.count = 1;
	mRights.items = &authItem;
	
	mFlags = kAuthorizationFlagExtendRights |
				kAuthorizationFlagInteractionAllowed |
				kAuthorizationFlagPreAuthorize;
	
	[mLockText setStringValue:NSLocalizedString(@"Authenticating...", @"")];
	[mLockText displayIfNeeded];
	
	err = AuthorizationCopyRights(mAuthRef, &mRights, kAuthorizationEmptyEnvironment, mFlags, NULL);
	if (err == errAuthorizationSuccess)
	{
		[mLockButton setImage:mUnlockImage];
		
		return YES;
	}
	else
	{
		[mLockButton setImage:mLockImage];
		
		return NO;
	}
}

/*
Iterates over the dictionary values in mInstallInfo. If a single component
is reported as not being installed, the installer is run.
*/
- (BOOL)isProductInstalled
{
	NSEnumerator*	enumerator;
	NSNumber*		installed;
	
	if (mInstallInfo)
	{
		enumerator = [mInstallInfo objectEnumerator];
		
		while (installed = [enumerator nextObject])
		{
			if (![installed boolValue])
				return NO;
		}
	}
	
	return YES;
}

- (void)setAuthText:(NSString*)string
{       
	NSSize      tfSize;
	NSRect      newFrame;
	
	[mAuthMessage setStringValue:string];
	
	tfSize = [[mAuthMessage cell] cellSizeForBounds:NSMakeRect(0, 0, kPGPInstaller_AuthTextWidth, kPGPInstaller_AuthTextHeight)];
	
	newFrame = [mAuthMessage frame];
	newFrame.size.width = kPGPInstaller_AuthTextWidth;
	newFrame.size.height = tfSize.height;
		
	newFrame.origin.y = (NSHeight([[mAuthMessage superview] frame]) / 2) - (NSHeight(newFrame) / 2);
		
	[mAuthMessage setFrame:newFrame];
	[[mAuthMessage superview] display];
}  

- (void)updateInstallItems
{   
	NSTextField*        textField = nil;
	NSImageView*        imageView = nil;
	
	[mAuthorizationItemImage setImage:mGrayBall];
	[mLicenseItemImage setImage:mGrayBall];
	[mInstallingItemImage setImage:mGrayBall];
	[mFinishUpItemImage setImage:mGrayBall];
	
	switch (mCurrentPanelID)
	{
		case kPGPInstaller_AuthPanel:
			textField = mAuthorizationItem;
			imageView = mAuthorizationItemImage;
			break;
		case kPGPInstaller_LicensePanel:
			textField = mLicenseItem;
			imageView = mLicenseItemImage;
			break;
		case kPGPInstaller_PreInstallPanel:
			textField = mInstallingItem;
			imageView = mInstallingItemImage;
			break;
		case kPGPInstaller_InstallPanel:
			textField = mInstallingItem;
			imageView = mInstallingItemImage;
			break;
		case kPGPInstaller_KeySelectionPanel:
			textField = mFinishUpItem;
			imageView = mFinishUpItemImage;
			break;
		case kPGPInstaller_FinishUpPanel:
			textField = mFinishUpItem;
			imageView = mFinishUpItemImage;
			break;
		default:
			break;
	}
	
	if (textField)
		[textField setTextColor:[NSColor controlTextColor]];
	
	if (imageView)
		[imageView setImage:mBlueBall];
}

- (void)configureInstallChecklist
{
	NSAutoreleasePool*			pool = [[NSAutoreleasePool alloc] init];
	NSMutableArray*             installItems = [[mInstallInfo allKeys] mutableCopy];
	NSEnumerator*               enumerator;
	NSString*                   installItem;
	NSRect                      boxFrame = NSZeroRect;
	NSRect                      installItemFrame = NSZeroRect;
	NSRect                      statusItemFrame = NSZeroRect;
	NSRect                      checkFrame = NSZeroRect;
	NSView*                     contentView;
	unsigned                    numItems;
	unsigned                    multiplier = 0;
	
	/* Determine the frame for the installation checklist box */
	numItems = [installItems count];
	boxFrame.size.width = NSWidth([mChecklistBox frame]);
	boxFrame.size.height = NSHeight([mChecklistBox frame]) + numItems * (kPGPInstaller_ListItemHeight + kPGPInstaller_ListItemVPadding);
	boxFrame.origin.x = (NSWidth([[mChecklistBox superview] frame]) / 2) - (NSWidth(boxFrame) / 2);
	boxFrame.origin.y = (NSHeight([[mChecklistBox superview] frame]) / 2) - (NSHeight(boxFrame) / 2) + kPGPInstaller_ListItemVPadding;
	
	[mChecklistBox setFrame:boxFrame];
	
	contentView = [mChecklistBox contentView];
	
	installItemFrame.size.width = kPGPInstaller_ListItemWidth;
	installItemFrame.size.height = kPGPInstaller_ListItemHeight;
	installItemFrame.origin.x = NSMinX([mItemName frame]);
	installItemFrame.origin.y = NSMinY([mDividingLine frame]) - kPGPInstaller_ListItemVPadding - kPGPInstaller_ListItemHeight;
	
	statusItemFrame.size.width = kPGPInstaller_ListItemWidth;
	statusItemFrame.size.height = kPGPInstaller_ListItemHeight;
	statusItemFrame.origin.x = NSMinX([mStatus frame]);
	statusItemFrame.origin.y = NSMinY([mDividingLine frame]) - kPGPInstaller_ListItemVPadding - kPGPInstaller_ListItemHeight;
	
	checkFrame.size.width = kPGPInstaller_CheckWidth;
	checkFrame.size.height = kPGPInstaller_ListItemHeight;
	checkFrame.origin.x = NSMinX([mItemName frame]) - kPGPInstaller_CheckWidth - 4;
	checkFrame.origin.y = NSMinY([mDividingLine frame]) - kPGPInstaller_ListItemVPadding - kPGPInstaller_ListItemHeight;
	
	/* We want our install list to be in alphabetical order, so we sort the keys of the install items. */
	[installItems sortUsingFunction:sCompareInstallItems context:nil];
	
	enumerator = [installItems objectEnumerator];
	while (installItem = [enumerator nextObject])
	{
		/* Unfortunately, -[NSTextField copy] isn't a valid method, so we need to customize each text field programatically. */
		NSTextField*            item = [[[NSTextField alloc] init] autorelease];
		NSTextField*            status = [[[NSTextField alloc] init] autorelease];
		NSTextField*            check = [[[NSTextField alloc] init] autorelease];
		NSMutableDictionary*    dict = [[[NSMutableDictionary alloc] init] autorelease];
		NSRect                  frame;
		
		/* Set the title for the install item and initial status for the corresponding status text field. */
		[self configureItemField:item statusField:status checkField:check key:installItem];
		
		[item setBordered:NO];
		[item setEditable:NO];
		[item setTextColor:[NSColor controlTextColor]];
		[item setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		[item setDrawsBackground:NO];
		
		frame = installItemFrame;
		frame.origin.y -= multiplier * (NSHeight(frame) + kPGPInstaller_ListItemVPadding);
		[item setFrame:frame];
		
		[status setBordered:NO];
		[status setEditable:NO];
		[status setTextColor:[NSColor controlTextColor]];
		[status setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		[status setDrawsBackground:NO];
		
		frame = statusItemFrame;
		frame.origin.y -= multiplier * (NSHeight(frame) + kPGPInstaller_ListItemVPadding);
		[status setFrame:frame];
		
		[check setBordered:NO];
		[check setEditable:NO];
		[check setTextColor:[NSColor controlTextColor]];
		[check setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		[check setDrawsBackground:NO];
		
		frame = checkFrame;
		frame.origin.y -= multiplier * (NSHeight(frame) + kPGPInstaller_ListItemVPadding);
		[check setFrame:frame];
		
		/* We need to update the text fields later on, so we store them in a dictionary for easy retrieval. */
		[dict setObject:item forKey:kPGPInstaller_ItemControlKey];
		[dict setObject:status forKey:kPGPInstaller_StatusControlKey];
		[dict setObject:check forKey:kPGPInstaller_CheckControlKey];
		
		[mControlsDictionary setObject:dict forKey:installItem];
		
		[contentView addSubview:item];
		[contentView addSubview:status];
		[contentView addSubview:check];
		
		++multiplier;
	}
	
	[pool release];
}

- (void)updateControls:(NSDictionary*)controls
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	NSTextField*			currentStatusField = [controls objectForKey:@"Status"];
	NSTextField*			currentCheckField = [controls objectForKey:@"Check"];
	
	[currentCheckField setStringValue:@"\xc3"];
	[currentCheckField displayIfNeeded];
	[currentStatusField setStringValue:NSLocalizedString(@"Installed", @"")];
	[currentStatusField displayIfNeeded];
	
	[pool release];
}

- (void)setInstallMessage
{
	BOOL                correctPath = NO;
	NSArray*            appPaths;
	NSString*           appPath;
	NSString*           installPath;
	NSString*           constructedPath;
	NSEnumerator*       pathEnumerator;
	
	installPath = [[NSBundle mainBundle] bundlePath];
	
	appPaths = [NSArray arrayWithObjects:@"/Applications", [NSString pathWithComponents:[NSArray arrayWithObjects:NSHomeDirectory(), @"Applications", nil]], nil];
	pathEnumerator = [appPaths objectEnumerator];
	
	while ((appPath = [pathEnumerator nextObject]) && !correctPath)
	{
		constructedPath = [NSString pathWithComponents:[NSArray arrayWithObjects:appPath, @"PGP.app", nil]];
		
		if ([installPath isEqualToString:constructedPath])
			correctPath = YES;
	}
	
	if (!correctPath)
		[mInstallMessage setStringValue:kPGPInstallerString_ServicesNotAvailable];
	else
		[mInstallMessage setStringValue:kPGPInstallerString_MailServicesMessage];
}

- (void)startPGPnet
{
	NSString*	bundlePath = [[NSBundle mainBundle] bundlePath];
	NSString*   installtoolPath = [[NSString alloc] initWithFormat:@"%@/Contents/Resources/%@", bundlePath, kPGPInstaller_LaunchPath];
	char**		args;
	
	[[mPanelBox contentView] addSubview:mProgressText];
	[[mPanelBox contentView] addSubview:mProgressIndicator];
	
	[mProgressIndicator setUsesThreadedAnimation:YES];
	[mProgressIndicator startAnimation:self];
	
	[mProgressText setStringValue:NSLocalizedString(@"Starting PGPnet", @"")];
	
	args = malloc(2 * sizeof(char*));
	args[0] = "-startnet";
	args[1] = NULL;
	
	AuthorizationExecuteWithPrivileges(mAuthRef, [installtoolPath cString], 0, args, NULL);
	free(args);
}

- (void)stopProgressBar
{
	[mProgressIndicator stopAnimation:self];
	[mProgressIndicator removeFromSuperview];
	[mProgressText removeFromSuperview];
}

- (void)installCompleted
{
	[[mPanelBox contentView] addSubview:mInstallComplete];
	
	[mMultiPurposeButton setEnabled:YES];
	[mMultiPurposeButton setTitle:NSLocalizedString(@"Finish", @"")];
	[mMultiPurposeButton setAction:@selector(finish:)];
	
	mCurrentPanelID = kPGPInstaller_FinishUpPanel;
	[self updateInstallItems];
    
    mInstallCompleted = YES;
}

- (void)configureItemField:(NSTextField*)item statusField:(NSTextField*)status checkField:(NSTextField*)check key:(NSString*)key
{
	if (item && status && key && check)
	{
		if ([key isEqualToString:@"PGPapp"])
			[item setStringValue:@"PGP"];
		else if ([key isEqualToString:@"PGPdisk.bundle"])
			[item setStringValue:@"PGPdisk"];
		else if ([key isEqualToString:@"PGPkeys.bundle"]) 
			[item setStringValue:@"PGPkeys"];
		else if ([key isEqualToString:@"PGPmail.bundle"])
			[item setStringValue:@"PGPmail"];
		else if ([key isEqualToString:@"PGPnet.bundle"])
			[item setStringValue:@"PGPnet"];
		
		if ([(NSNumber*) [mInstallInfo objectForKey:key] boolValue])
		{
			[status setStringValue:NSLocalizedString(@"Installed", @"")];
			[check setStringValue:@"\xc3"];
		}
		else
			[status setStringValue:NSLocalizedString(@"Installing", @"")];
	}
}

- (char**)installtoolArguments
{   
	NSEnumerator*		keyEnumerator;
	NSString*			key;
	NSString*			bundlePath = [[NSBundle mainBundle] bundlePath];
	NSMutableArray*		keys;
	unsigned			argIndex;
	char**				argv;
	
	if (!mInstallInfo)
		return NULL;
	
	argv = (char**) malloc(sizeof(char*) * ([[mInstallInfo allKeys] count] + 2));
	if (!argv)
		return NULL;
	
	argv[0] = (char*) [[NSString stringWithFormat:@"-installpath=\"%@\"", bundlePath] cString];
	argIndex = 1;
	
	keys = [[mInstallInfo allKeys] mutableCopy];
	[keys sortUsingFunction:sCompareInstallItems context:nil];
	keyEnumerator = [keys objectEnumerator];
	
	while (key = [keyEnumerator nextObject])
	{
		if ([key isEqualToString:@"PGPapp"])
		{
			if (![(NSNumber*) [mInstallInfo objectForKey:key] boolValue])
				argv[argIndex++] = (char*) [[NSString stringWithString:@"-app"] cString];
		}
		else if ([key isEqualToString:@"PGPdisk.bundle"])
		{
			if (![(NSNumber*) [mInstallInfo objectForKey:key] boolValue])
				argv[argIndex++] = (char*) [[NSString stringWithString:@"-disk"] cString];
		}
		else if ([key isEqualToString:@"PGPkeys.bundle"])
		{
			if (![(NSNumber*) [mInstallInfo objectForKey:key] boolValue])
				argv[argIndex++] = (char*) [[NSString stringWithString:@"-keys"] cString];
		}
		else if ([key isEqualToString:@"PGPmail.bundle"])
		{
			if (![(NSNumber*) [mInstallInfo objectForKey:key] boolValue])
				argv[argIndex++] = (char*) [[NSString stringWithString:@"-mail"] cString];
		}
		else if ([key isEqualToString:@"PGPnet.bundle"])
		{
			if (![(NSNumber*) [mInstallInfo objectForKey:key] boolValue])
				argv[argIndex++] = (char*) [[NSString stringWithString:@"-net"] cString];
		}
	}
	
	argv[argIndex] = NULL;
	
	return argv;
}

/*
This method sets up the install panel's UI, launches the installtool, and updates
the UI according to the output received from the installtool. This method gets
invoked in it's own thread, so we have to setup an NSAutoreleasePool.
*/
- (void)communicateWithInstallTool:(NSString*)tool
{
	NSAutoreleasePool*          pool = [[NSAutoreleasePool alloc] init];
	NSDictionary*               currentDictionary;
	NSConnection*				connection;
	NSNumber*                   installMail;
	NSNumber*                   installNet;
	char        				cmdLine[PATH_MAX];
	char                        statusBuf[256];
	char**                      args;
	FILE*                       iopipe;
	id							proxy;
	
	if (!mInstallInfo)
		return;
	
	if (!tool)
		return;
	
	if (![[NSFileManager defaultManager] fileExistsAtPath:tool])
		return;
	
	connection = [NSConnection connectionWithReceivePort:[mConnection sendPort] sendPort:[mConnection receivePort]];
	proxy = [connection rootProxy];
	
	[proxy configureInstallChecklist];
	
	/*
	We need to generate an argv array for the installtool.
	The array is malloc'd, so we free it after we're done.
	*/
	if (args = [self installtoolArguments])
	{
		AuthorizationExecuteWithPrivileges(mAuthRef, [tool cString], 0, args, &iopipe);
		free(args);
	}
	
	/*
	As the installtool runs, we receive the name of the item being installed, and a notification that the
	item was installed correctly. When we get the name of the item currently being installed, we update
	the corresponding status text field, and we also place the arrow text field such that it "points" to
	the current text field. When the item has been installed, that same status text field is updated to
	show "Status: Done", and the corresponding check box text field is filled in.
	*/
	
	while(!feof(iopipe))
	{
		fgets(statusBuf, sizeof(statusBuf), iopipe);
		
		if (strstr(statusBuf, "PGPapp"))
			currentDictionary = [mControlsDictionary objectForKey:@"PGPapp"];
		else if (strstr(statusBuf, "PGPdisk"))
			currentDictionary = [mControlsDictionary objectForKey:@"PGPdisk.bundle"];
		else if (strstr(statusBuf, "PGPkeys"))
			currentDictionary = [mControlsDictionary objectForKey:@"PGPkeys.bundle"];
		else if (strstr(statusBuf, "PGPmail"))
			currentDictionary = [mControlsDictionary objectForKey:@"PGPmail.bundle"];
		else if (strstr(statusBuf, "PGPnet"))
			currentDictionary = [mControlsDictionary objectForKey:@"PGPnet.bundle"];
		else if (strstr(statusBuf, "Done"))
		{
			[proxy updateControls:currentDictionary];
			
			currentDictionary = nil;
		}
		else
			currentDictionary = nil;
		
		statusBuf[0] = '\0';
	}
	
	if (installMail = [mInstallInfo objectForKey:@"PGPmail.bundle"])
		[proxy setInstallMessage];
	
	/* If PGPnet is installed, load the driver and start PGPnetService */
	if ((installNet = [mInstallInfo objectForKey:@"PGPnet.bundle"]) && ![installNet boolValue])
	{
		[proxy startPGPnet];
		sleep(3);
		[proxy stopProgressBar];
	}
	
	snprintf(cmdLine, sizeof(cmdLine), "/usr/sbin/chown %d \"%s\" ; /bin/chmod 755 \"%s\"", getuid(), [tool cString], [tool cString]);
	
	args = malloc(3 * sizeof(char*));
	args[0] = "-c";
	args[1] = cmdLine;
	args[2] = NULL;
	
	AuthorizationExecuteWithPrivileges(mAuthRef, "/bin/sh", 0, args, NULL);
	free(args);
	
	if (mAuthRef)
	{
		AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
		mAuthRef = NULL;
	}
	
	[proxy installCompleted];
    
	[pool release];
}

- (void)tearDownConnection
{
	if (mConnection)
	{
		[mConnection invalidate];
		[mConnection release];
		mConnection = nil;
	}
}

- (IBAction)lock:(id)sender
{
	if ([mLockButton image] == mLockImage)
	{   
		if ([self obtainAuthorization])
		{
			[self setAuthText:kPGPInstallerString_HaveAuth];
			[mAuthMessage setAlignment:NSCenterTextAlignment];
			[mLockText setStringValue:NSLocalizedString(@"Click the lock to prevent further changes.", @"")];
			[mMultiPurposeButton setTitle:NSLocalizedString(@"Continue", @"")];
			[mMultiPurposeButton setAction:@selector(gotoLicense:)];
		}
		else
		{
			[self setAuthText:kPGPInstallerString_NeedAuth];
			[mAuthMessage setAlignment:NSLeftTextAlignment];
			[mLockText setStringValue:NSLocalizedString(@"Click the lock to make changes.", @"")];
			[mMultiPurposeButton setTitle:NSLocalizedString(@"Close", @"")];
			[mMultiPurposeButton setAction:@selector(close:)];
		}
	}
	else
	{
		if (mAuthRef)
		{
			AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
			mAuthRef = NULL;
		}
		
		[self setAuthText:kPGPInstallerString_NeedAuth];
		[mAuthMessage setAlignment:NSLeftTextAlignment];
		[mLockText setStringValue:NSLocalizedString(@"Click the lock to make changes.", @"")];
		[mLockText displayIfNeeded];
		[mLockButton setImage:mLockImage];
		[mMultiPurposeButton setTitle:NSLocalizedString(@"Close", @"")];
		[mMultiPurposeButton setAction:@selector(close:)];
	}
	
	[[mPanelBox contentView] display];
}

- (IBAction)close:(id)sender
{
	[mWindow close];
}

- (IBAction)gotoLicense:(id)sender
{
	NSString*           licensePath;
	
	mCurrentPanelID = kPGPInstaller_LicensePanel;
	[self updateInstallItems];
	
	[mLicenseScrollView setDrawsBackground:NO];
	
	[[mPanelBox contentView] removeFromSuperview];
	[mPanelBox setContentView:[mLicensePanel contentView]];
	
	licensePath = [[NSBundle mainBundle] pathForResource:@"License" ofType:@"rtf"];
	if ([[NSFileManager defaultManager] fileExistsAtPath:licensePath])
		[mLicenseView readRTFDFromFile:licensePath];
	
	[mLicenseView setNeedsDisplay:YES];
	[mLicenseView scrollRangeToVisible:NSMakeRange(0, [[mLicenseView string] length])];
	[mLicenseScrollView setNeedsDisplay:YES];
	[mPanelBanner setStringValue:NSLocalizedString(@"Software License", @"")];
	[mMultiPurposeButton setTitle:NSLocalizedString(@"Continue", @"")];
	[mMultiPurposeButton setAction:@selector(displayLicenseSheet:)];
	[mMultiPurposeButton displayIfNeeded];
}

- (IBAction)displayLicenseSheet:(id)sender
{
	[NSApp beginSheet:mLicenseSheet
		   modalForWindow:mWindow
		   modalDelegate:self
		   didEndSelector:@selector(licenseAgreementDidEnd:returnCode:contextInfo:)
		   contextInfo:nil];
}

- (IBAction)closeLicenseAgreement:(id)sender
{
	int		returnCode;  
	
	if (sender == mAgree)
		returnCode = kPGPInstaller_LicenseAgree;
	else
		returnCode = kPGPInstaller_LicenseDisagree;
	
	[NSApp endSheet:mLicenseSheet returnCode:returnCode];
}

- (IBAction)gotoPreInstall:(id)sender                   
{
	mCurrentPanelID = kPGPInstaller_PreInstallPanel;
	[self updateInstallItems];
	
	[[mPanelBox contentView] removeFromSuperview];      
	[mPanelBox setContentView:[mPreInstallPanel contentView]];  
	
	[mPanelBanner setStringValue:NSLocalizedString(@"About to Install", @"")];
	[mMultiPurposeButton setTitle:NSLocalizedString(@"Install", @"")];
	[mMultiPurposeButton setAction:@selector(install:)];  
	[mMultiPurposeButton displayIfNeeded];
}

- (IBAction)install:(id)sender
{
	char        cmdLine[PATH_MAX];
	char*       args[3];
	NSString*   bundlePath = [[NSBundle mainBundle] bundlePath]; 
	NSString*   installtoolPath = [[NSString alloc] initWithFormat:@"%@/Contents/Resources/%@", bundlePath, kPGPInstaller_LaunchPath];
	
	mCurrentPanelID = kPGPInstaller_InstallPanel;       
	[self updateInstallItems];
	
	[[mPanelBox contentView] removeFromSuperview];      
	[mPanelBox setContentView:[mInstallationPanel contentView]];
	
	[mPanelBanner setStringValue:NSLocalizedString(@"Installing PGP", @"")];
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:installtoolPath])
	{   
		[mMultiPurposeButton setEnabled:NO];
		
		snprintf(cmdLine, sizeof(cmdLine), "/usr/sbin/chown root \"%s\" ; /bin/chmod 4755 \"%s\"",
			[installtoolPath cString], [installtoolPath cString]);
		
		args[0] = "-c";
		args[1] = cmdLine;
		args[2] = NULL;
		
		AuthorizationExecuteWithPrivileges(mAuthRef, "/bin/sh", 0, args, NULL);
		
		[NSThread detachNewThreadSelector:@selector(communicateWithInstallTool:) toTarget:self withObject:installtoolPath];
	}
	
	[installtoolPath release];
}

- (IBAction)finish:(id)sender
{
	if (mAuthRef)
	{
		AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
		mAuthRef = NULL;
	}
	
	[mWindow close];
}

- (void)licenseAgreementDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{          
	[mLicenseSheet close];
	
	if (returnCode == kPGPInstaller_LicenseAgree)
	{
		[mMultiPurposeButton setAction:@selector(gotoPreInstall:)];
		[mMultiPurposeButton performClick:nil];
	}
}

- (void)windowWillClose:(NSNotification*)notification
{
	if (!mInstallCompleted)
	{
		[self tearDownConnection];
		[self release];
	}
	else
		[self startPGP];
}

- (void)startPGP
{
	NSFileManager*		fileManager = [NSFileManager defaultManager];
	NSString*			bundlePath = [[NSBundle mainBundle] bundlePath];
	NSMutableArray*		args;
	
	[fileManager movePath:[bundlePath stringByAppendingPathComponent:@"Contents/MacOS/PGP"]
				 toPath:[bundlePath stringByAppendingPathComponent:@"Contents/MacOS/PGPinstaller"]
				 handler:nil];
	
	[fileManager movePath:[bundlePath stringByAppendingPathComponent:@"Contents/MacOS/PGPmain"]
				 toPath:[bundlePath stringByAppendingPathComponent:@"Contents/MacOS/PGP"]
				 handler:nil];
	
	args = [[[NSMutableArray alloc] init] autorelease];
	[args addObject:@"-c"];
	/*
	[args addObject:[NSString stringWithFormat:@"\"%@\" & ; /usr/bin/open \"%@\"",
			[bundlePath stringByAppendingPathComponent:@"Contents/MacOS/PGP"],
			bundlePath]];
	*/
	[args addObject:[NSString stringWithFormat:@"/usr/bin/open \"%@\"", bundlePath]];
	
	[NSTask launchedTaskWithLaunchPath:@"/bin/sh" arguments:args];
	
	[self tearDownConnection];
	[self release];
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	if (mInstallInfo)
		[mInstallInfo release];
	
	if (mControlsDictionary)
		[mControlsDictionary release];
	
	if (mLockImage)
		[mLockImage release];
	
	if (mUnlockImage)
		[mUnlockImage release];
	
	if (mBlueBall)
		[mBlueBall release];
	
	if (mGrayBall)
		[mGrayBall release];
	
	if (mAuthorizationPanel)
		[mAuthorizationPanel release];
	
	if (mLicensePanel)
		[mLicensePanel release];
	
	if (mPreInstallPanel)
		[mPreInstallPanel release];
	
	if (mInstallationPanel)
		[mInstallationPanel release];
	
	if (mKeysPanel)
		[mKeysPanel release];
	
	if (mFinishUpPanel)
		[mFinishUpPanel release];
	
	[super dealloc];
	
	[NSApp terminate:nil];
}

@end

@implementation PGPAppController

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	return self;
}

- (void)applicationDidFinishLaunching:(NSNotification*)notification
{
	mFinishedLaunching = YES;
	
	[[[CPGPInstaller alloc] init] installIfNecessary];
}

- (BOOL)application:(NSApplication*)app openFile:(NSString*)filename
{
	if (!mFinishedLaunching && !mDisplayedOpenAlert)
	{
		mDisplayedOpenAlert = YES;
		
		NSRunAlertPanel(kPGPAppString_CantOpenDocument,
						kPGPAppString_CantOpenDocumentMsg,
						@"Install",
						nil,
						nil);
	}
	
	return NO;
}

- (BOOL)isRunningOnReadOnlyFS
{
	NSScanner*			scanner;
	NSString*			bundlePath = [[NSBundle mainBundle] bundlePath];
	NSString*			tempString;
	NSMutableString*	volumeName = nil;
	
	/*
	Get the volume name. If we're running on the default volume, /Volumes won't
	appear in the path, and we just assume that the volume is writable.
	*/
	scanner = [[NSScanner alloc] initWithString:bundlePath];
	if ([scanner scanString:NSLocalizedString(@"/Volumes/", @"") intoString:&tempString])
	{
		volumeName = [[[NSMutableString alloc] init] autorelease];
		[volumeName appendString:tempString];
		
		if ([scanner scanUpToString:@"/" intoString:&tempString])
			[volumeName appendString:tempString];
		else
			volumeName = nil;
	}
	
	if (volumeName)
	{
		NSWorkspace*	workspace = [NSWorkspace sharedWorkspace];
		BOOL			removable;
		BOOL			writable;
		BOOL			unmountable;
		NSString*		description;
		NSString*		type;
		
		if ([workspace getFileSystemInfoForPath:volumeName
				isRemovable:&removable
				isWritable:&writable
				isUnmountable:&unmountable
				description:&description
				type:&type])
		{
			if (!writable)
				return YES;
		}
	}
	
	return NO;
}

- (void)displayExpirationAlertAndExit
{
	NSRunAlertPanel(kPGPAppString_BetaExpiration,
					kPGPAppString_ExpirationMessage,
					kPGPAppString_Quit,
					nil,
					nil);
	
	[NSApp terminate:nil];
}

- (void)displayReadOnlyAlertAndExit
{
	NSRunAlertPanel(kPGPAppString_ReadOnlyFS,
					kPGPAppString_ReadOnlyFSMessage,
					kPGPAppString_Quit,
					nil,
					nil);
	
	[NSApp terminate:nil];
}

- (void)awakeFromNib
{
	NSUserDefaults*		defaults = [NSUserDefaults standardUserDefaults];
	NSDictionary*		dict = [defaults persistentDomainForName:kPGPPrefsDomain];
	NSNumber*			checkExpiration;
	
	if (sIsExpired())
	{
		if (checkExpiration = [dict objectForKey:kPGPAppDefault_CheckExpiration])
		{
			if ([checkExpiration boolValue])
				[self displayExpirationAlertAndExit];
		}
		else
			[self displayExpirationAlertAndExit];
	}
	if (!sIsOSSupported())
		[NSApp terminate:nil];
	if ([self isRunningOnReadOnlyFS])
		[self displayReadOnlyAlertAndExit];
}

@end

@implementation PGPDocument

- (NSData*)dataRepresentationOfType:(NSString*)type
{
	return nil;
}

- (BOOL)readFromFile:(NSString*)filename ofType:(NSString*)type
{
	return NO;
}

@end

int main(int argc, const char* argv[])
{
    return NSApplicationMain(argc, argv);
}