/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPKeyringSelection.m,v 1.3 2002/10/25 01:10:18 jeffc Exp $
____________________________________________________________________________*/

#import <PGPclient/pgpClientLibCocoa.h>
#import "CPGPKeyringSelection.h"
#import "PGPAppController.h"

#define kPGPKeyringSelectionNib				@"CPGPKeyringSelection"

#define kPGPMacOSXDirectory					NSLocalizedString(@"/Documents/PGP", @"Default location for files")

#define kPGPMacOSPublicFile					NSLocalizedString(@"/Documents/PGP/PGP Public Keys", @"MacOS Public File Location")
#define kPGPMacOSPrivateFile				NSLocalizedString(@"/Documents/PGP/PGP Private Keys", @"MacOS Private File Location")
#define kPGPMacOSRandomSeedFile				NSLocalizedString(@"/Documents/PGP/PGP Random Seed", @"MacOS Random Seed File Location")

#define kPGPMacOSXPublicFile				NSLocalizedString(@"/Documents/PGP/PGP Public Keyring.pkr", @"MacOSX Public File Location")
#define kPGPMacOSXPrivateFile				NSLocalizedString(@"/Documents/PGP/PGP Private Keyring.skr", @"MacOSX Private File Location")
#define kPGPMacOSXRandomSeedFile			NSLocalizedString(@"/Documents/PGP/PGP Random Seed.rnd", @"MacOSX Random Seed File Location")

#define kPGPUnixPublicFile					NSLocalizedString(@"/.pgp/pubring.pkr", @"Unix Public File Location")
#define kPGPUnixPrivateFile					NSLocalizedString(@"/.pgp/secring.skr", @"Unix Private File Location")
#define kPGPUnixRandomSeedFile				NSLocalizedString(@"/.pgp/randseed.rnd", @"Unix Random Seed File Location")

@implementation CPGPKeyringSelection

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self loadNib:kPGPKeyringSelectionNib];
	
	mCurrentKeyringSelection = kPGPPrefPublicKeyringFile;
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	if (!mLoadedNib)
	{
		[NSBundle loadNibNamed:nibFile owner:self];
		
		mLoadedNib = YES;
	}
}

- (BOOL)haveKeyrings
{
	BOOL	keyringsSet = NO;
	
	[self validateRandSeedFile];	/* Sets the randseed file */
	
	/* If the keyring prefs are absent, or if the files don't exist, we begin the search. */
	if (![self prefKeyringsValid])
	{
        if ([self haveMacOSXKeyrings])
        {
            [self setMacOSXKeyrings];
            keyringsSet = YES;
        }
        else if ([self haveMacOSKeyrings])
		{
			[self setMacOSKeyrings];
			keyringsSet = YES;
		}
		else if ([self haveUnixKeyrings])
		{
			[self setUnixKeyrings];
			keyringsSet = YES;
		}
		else
			keyringsSet = NO;
	}
	else
		return YES;
	
	if (keyringsSet)
		[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_UpdateKeyrings object:nil];
	
	return keyringsSet;
}

- (void)validateRandSeedFile
{
	PGPError		err;
	PGPPrefRef		prefRef = [[PGPAppController appController] pgpPrefRef];
	PGPContextRef	context = [[PGPAppController appController] pgpContext];
	char			randSeedPath[kPGPcl_MaxPath];
	NSString*		filename;
	PFLFileSpecRef	randSeedFile = NULL;
	
	pgpClearMemory(randSeedPath, sizeof(randSeedPath));

	err = PGPGetPrefStringBuffer(prefRef, kPGPPrefRNGSeedFile, kPGPcl_MaxPath, randSeedPath);
	if (IsPGPError(err))
		PGPLog((@"Failed to get randseed file location"));
	else
	{
		if ((![[NSFileManager defaultManager] fileExistsAtPath:[NSString stringWithCString:randSeedPath]]) ||
			(randSeedPath[0] == '\0'))
		{
			filename = [NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXRandomSeedFile];
			if ([[NSFileManager defaultManager] fileExistsAtPath:filename])
			{
				err = PGPSetPrefString(prefRef, kPGPPrefRNGSeedFile, [filename cString]);
				if (IsPGPError(err))
					PGPLog((@"Failed to set randseed file"));
			}
			else
			{
				filename = [NSHomeDirectory() stringByAppendingPathComponent:kPGPUnixRandomSeedFile];
				if ([[NSFileManager defaultManager] fileExistsAtPath:filename])
				{
					err = PGPSetPrefString(prefRef, kPGPPrefRNGSeedFile, [filename cString]);
					if (IsPGPError(err))
						PGPLog((@"Failed to set randseed file"));
				}
			}
		}
		else
		{
			if (![[NSFileManager defaultManager] fileExistsAtPath:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXDirectory]] )
			{
				NSDictionary*	attributes = [[NSFileManager defaultManager] fileAttributesAtPath:[NSHomeDirectory() stringByAppendingPathComponent:NSLocalizedString(@"Documents", @"Documents Folder")] traverseLink:YES];
		
				[[NSFileManager defaultManager] createDirectoryAtPath:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXDirectory] attributes:attributes];
			}

			filename = [NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXRandomSeedFile];
			PGPSetPrefString(prefRef, kPGPPrefRNGSeedFile, [filename cString]);
			
			err = PFLNewFileSpecFromFullPath(PGPPeekContextMemoryMgr(context), [filename cString], &randSeedFile);
			if (IsntPGPError(err))
				PGPSetRandSeedFile(randSeedFile);
			
			if (randSeedFile)
				PFLFreeFileSpec(randSeedFile);
		}
	}
}

- (BOOL)prefKeyringsValid
{
	PGPError	err;
    PGPPrefRef	prefRef = [[PGPAppController appController] pgpPrefRef];
	char		pubringPath[kPGPcl_MaxPath];
	char		secringPath[kPGPcl_MaxPath];
	
	pgpClearMemory(pubringPath, sizeof(pubringPath));
	pgpClearMemory(secringPath, sizeof(secringPath));
	
	err = PGPGetPrefStringBuffer(prefRef, kPGPPrefPublicKeyringFile, kPGPcl_MaxPath, pubringPath);
	if (IsPGPError(err))
		return NO;
	else
	{
		if (![[NSFileManager defaultManager] fileExistsAtPath:[NSString stringWithCString:pubringPath]])
			return NO;
	}
	
	err = PGPGetPrefStringBuffer(prefRef, kPGPPrefPrivateKeyringFile, kPGPcl_MaxPath, secringPath);
	if (IsPGPError(err))
		return NO;
	else
	{
		if (![[NSFileManager defaultManager] fileExistsAtPath:[NSString stringWithCString:secringPath]])
			return NO;
	}
	
	return YES;
}

- (BOOL)fileExists:(NSString*)file
{
	return [[NSFileManager defaultManager] fileExistsAtPath:file];
}

- (BOOL)haveMacOSXKeyrings
{
	return [self fileExists:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXPublicFile]] &&
		   [self fileExists:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXPrivateFile]];
}

- (BOOL)haveMacOSKeyrings
{
	return [self fileExists:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSPublicFile]] &&
		   [self fileExists:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSPrivateFile]];
}

- (BOOL)haveUnixKeyrings
{
	return [self fileExists:[NSHomeDirectory() stringByAppendingPathComponent:kPGPUnixPublicFile]] &&
		   [self fileExists:[NSHomeDirectory() stringByAppendingPathComponent:kPGPUnixPrivateFile]];
}

- (void)setPubKeyring:(NSString*)pubring secKeyring:(NSString*)secring
{
	PGPError	err;
    PGPPrefRef	prefRef = [[PGPAppController appController] pgpPrefRef];
	
	if (pubring && secring)
	{
		err = PGPSetPrefString(prefRef, kPGPPrefPublicKeyringFile, [pubring cString]);
		if (IsPGPError(err))
			PGPLog((@"Failed to set pubring location"));
		
		err = PGPSetPrefString(prefRef, kPGPPrefPrivateKeyringFile, [secring cString]);
		if (IsPGPError(err))
			PGPLog((@"Failed to set secring location"));
	}
}

- (void)setMacOSXKeyrings
{
	[self setPubKeyring:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXPublicFile]
		  secKeyring:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXPrivateFile]];
}

- (void)setMacOSKeyrings
{
	[self setPubKeyring:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSPublicFile]
		  secKeyring:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSPrivateFile]];
}

- (void)setUnixKeyrings
{
	[self setPubKeyring:[NSHomeDirectory() stringByAppendingPathComponent:kPGPUnixPublicFile]
		  secKeyring:[NSHomeDirectory() stringByAppendingPathComponent:kPGPUnixPrivateFile]];
}

- (void)displayKeyringSelection
{
	[mWindow center];
	[mWindow makeKeyAndOrderFront:nil];
}

- (IBAction)createNewKeyrings:(id)sender
{
    PGPError		err;
    PGPContextRef	context = [[PGPAppController appController] pgpContext];
    PGPFileSpecRef	pubFileSpec = kInvalidPGPFileSpecRef;
	PGPFileSpecRef	privFileSpec = kInvalidPGPFileSpecRef;
    PGPKeyDBRef		keyDBRef = kInvalidPGPKeyDBRef;
    PGPPrefRef		prefRef = [[PGPAppController appController] pgpPrefRef];
    NSString*		pubring = nil;
	NSString*		secring = nil;
    NSFileManager*	fileManager = [NSFileManager defaultManager];
	
	[mCreate setEnabled:NO];
	[mSelect setEnabled:NO];
	[mWindow orderOut:nil];
	
    if (![fileManager fileExistsAtPath:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXDirectory]])
    {
        NSDictionary*	attr = [fileManager fileAttributesAtPath:[NSHomeDirectory() stringByAppendingPathComponent:NSLocalizedString(@"Documents", @"Documents Folder")] traverseLink:YES];

        [fileManager createDirectoryAtPath:[NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXDirectory] attributes:attr];
    }

    pubring = [NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXPublicFile];
    secring = [NSHomeDirectory() stringByAppendingPathComponent:kPGPMacOSXPrivateFile];
	
    PGPNewFileSpecFromFullPath(context, [pubring UTF8String], &pubFileSpec);
    PGPNewFileSpecFromFullPath(context, [secring UTF8String], &privFileSpec);
	
    err = PGPOpenKeyDBFile(context, kPGPOpenKeyDBFileOptions_Create, pubFileSpec, privFileSpec, &keyDBRef);
	if (IsPGPError(err))
		PGPLog((@"PGPOpenKeyDBFile failed with err=%d", err));
	else
	{
		PGPSetPrefString(prefRef, kPGPPrefPublicKeyringFile, [pubring UTF8String]);
		PGPSetPrefString(prefRef, kPGPPrefPrivateKeyringFile, [secring UTF8String]);
		
		[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_UpdateKeyrings object:nil];
	}
	
    if (PGPFileSpecRefIsValid(pubFileSpec))
    {
        PGPFreeFileSpec(pubFileSpec);
        pubFileSpec = kInvalidPGPFileSpecRef;
    }

    if (PGPFileSpecRefIsValid(privFileSpec))
    {
        PGPFreeFileSpec(privFileSpec);
        privFileSpec = kInvalidPGPFileSpecRef;
    }

    if (PGPKeyDBRefIsValid(keyDBRef))
    {
        PGPFreeKeyDB(keyDBRef);
        keyDBRef = kInvalidPGPKeyDBRef;
    }
	
	[[PGPAppController appController] keyringSelectionFinished:self];
}

- (IBAction)selectKeyrings:(id)sender
{
	[mCreate setEnabled:NO];
	[mSelect setEnabled:NO];
	
	if (mCurrentKeyringSelection == kPGPPrefPublicKeyringFile)
	{
		[self selectFile:kPGPPrefPublicKeyringFile
					withExtension:@"pkr"
					withPrompt:NSLocalizedString(@"Use Public Keyring", @"Install: Pubring button")];
	}
	else if (mCurrentKeyringSelection == kPGPPrefPrivateKeyringFile)
	{
		[self selectFile:kPGPPrefPrivateKeyringFile
					withExtension:@"skr"
					withPrompt:NSLocalizedString(@"Use Private Keyring", @"Install: Secring button")];
	}
}

- (IBAction)finish:(id)sender
{
	[mWindow orderOut:nil];
	[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_UpdateKeyrings object:nil];
	[[PGPAppController appController] keyringSelectionFinished:self];
}

- (void)selectFile:(PGPPrefIndex)inIndex withExtension:(NSString*)inExtension withPrompt:(NSString*)inMessage
{
	NSMutableArray*		types = [[[NSMutableArray alloc] init] autorelease];
	NSOpenPanel*		panel = [NSOpenPanel openPanel];
	
	[types addObject:inExtension];
	
	switch (inIndex)
	{
		case kPGPPrefPublicKeyringFile:
			[types addObject:NSFileTypeForHFSTypeCode(kPGPkeysPubringOSType)];
			break;
		case kPGPPrefPrivateKeyringFile:
			[types addObject:NSFileTypeForHFSTypeCode(kPGPkeysSecringOSType)];
			break;
		default:
			break;
	}
		
	[panel setAllowsMultipleSelection:NO];
	if (inMessage)
		[panel setPrompt:inMessage];
	
	[panel beginSheetForDirectory:nil
				file:nil
				types:types
				modalForWindow:mWindow
				modalDelegate:self
				didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:)
				contextInfo:[[NSNumber alloc] initWithInt:inIndex]];
}

- (void)openPanelDidEnd:(NSOpenPanel*)panel returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	PGPError		err;
	NSArray*		files;
	NSString*		file;
	NSNumber*		prefIndex;
	PGPPrefRef		prefRef = [[PGPAppController appController] pgpPrefRef];
	
	if (returnCode == NSOKButton)
	{
		files = [panel filenames];
		file =  [files objectAtIndex:0];
		prefIndex = (NSNumber*) contextInfo;
		
        err = PGPSetPrefString(prefRef, [prefIndex intValue], [file cString]);

		if ([prefIndex intValue] == kPGPPrefPublicKeyringFile)
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_PubKeyring object:nil];
			mCurrentKeyringSelection = kPGPPrefPrivateKeyringFile;
			[self performSelector:@selector(selectKeyrings:) withObject:nil afterDelay:0.0];
		}
		else if ([prefIndex intValue] == kPGPPrefPrivateKeyringFile)
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_SecKeyring object:nil];
			[self performSelector:@selector(finish:) withObject:nil afterDelay:0.0];
		}

		[prefIndex release];
	}
	else
	{
		[mCreate setEnabled:YES];
		[mSelect setEnabled:YES];
		
		mCurrentKeyringSelection = kPGPPrefPublicKeyringFile;
		PGPSetPrefString(prefRef, kPGPPrefPublicKeyringFile, "");
		PGPSetPrefString(prefRef, kPGPPrefPrivateKeyringFile, "");
	}
}

- (void)dealloc
{
	PGPLog((@"Deallocated CPGPKeyringSelection"));
	
	[super dealloc];
}

@end
