/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPLicenseAuth.m,v 1.16.2.1 2002/11/28 00:03:52 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPLicenseAuth.h"

#define kPGPLicenseAuthNib					@"CPGPLicenseAuth.nib"
#define kPGPLicenseAuthPurchaseURL			@"http://store.pgp.com/upgrade1"
#define kPGPLicenseAuthTriangleUp			@"triangleup.tiff"
#define kPGPLicenseAuthTriangleDown			@"triangledown.tiff"
#define kPGPLicenseAuthValidChars			@"ACDEFGHJKLMNPQRTUVWXYZ234679B0158OIS"

#define kPGPLicenseAuthMissingLA			NSLocalizedString(@"Missing License Authorization", @"")
#define kPGPLicenseAuthMissingLAMsg			NSLocalizedString(@"Please enter your License Authorization exactly as you received it from a Customer Service Representative.", @"")
#define kPGPLicenseAuthMissingName			NSLocalizedString(@"Missing Name", @"")
#define kPGPLicenseAuthMissingLN			NSLocalizedString(@"Missing License Number", @"")
#define kPGPLicenseAuthMissingNameOrgLNMsg	NSLocalizedString(@"Your License Authorize is encoded to your Name, Organization, and License Number. Please enter the specified information.", @"")
#define kPGPLicenseAuthBadLA				NSLocalizedString(@"There was a problem importing your license authorization", @"")
#define kPGPLicenseAuthBadLAMsg				NSLocalizedString(@"The License Authorization you entered does not appear to be formatted correctly. Please double-check the text and try again. Contact PGP Customer Service if you require assistance.", @"")
#define kPGPLicenseAuthBadLN				NSLocalizedString(@"There is a problem with the license number you entered", @"")
#define kPGPLicenseAuthBadLNGenericMsg		NSLocalizedString(@"Failed to import the license number because: %s", @"")
#define kPGPLicenseAuthCorruptLNMsg			NSLocalizedString(@"The License Number was not entered correctly or is not a valid License Number. Please double-check the number you entered. Contact PGP Customer Support if you require assistance.", @"")
#define kPGPLicenseAuthBadVerify			NSLocalizedString(@"There was a problem verifying the license authorization", @"")
#define kPGPLicenseAuthBadVerifyMsg			NSLocalizedString(@"License verification failed. Please make sure that the entered information matches the information given to the Customer Service Representative.", @"")
#define kPGPLicenseAuthLater				NSLocalizedString(@"Later", @"")
#define kPGPLicenseAuthInvalidProdID		NSLocalizedString(@"Invalid Product ID", @"")
#define kPGPLicenseAuthInvalidProdIDMsg		NSLocalizedString(@"The specified License Number is not valid for this product.", @"")
#define kPGPLicenseAuthLAExpired			NSLocalizedString(@"Expired License", @"")
#define kPGPLicenseAuthLAExpiredMsg			NSLocalizedString(@"The specified license has expired.", @"")
#define kPGPLicenseAuthLARefusedMsg			NSLocalizedString(@"The automatic License Authorization server has refused our request. Please try again later, or contact PGP Customer Service for further assistance.", @"")
#define kPGPLicenseAuthLAMismatchMsg		NSLocalizedString(@"The supplied Name and/or Organization do not match those on record for this License Number. Contact PGP Customer Service for assistance with this issue.", @"")
#define kPGPLicenseAuthManualMsg			NSLocalizedString(@"Click here to enter Authorization manually", @"")
#define kPGPLicenseAuthAutomaticMsg			NSLocalizedString(@"Click here to Authorize automatically", @"")
#define kPGPLicenseAuthAuthFailed			NSLocalizedString(@"There was a problem obtaining the license authorization", @"")
#define kPGPLicenseAuthNetworkProblemMsg	NSLocalizedString(@"PGP is unable to contact the License Authorization server. Please make sure your computer's internet connection is active. If you do not have internet access, or if you continue to have problems, contact PGP Customer Service to obtain a Manual License Authorization.", @"")
#define kPGPLicenseAuthGenericFailedMsg		NSLocalizedString(@"License authorization failed because the License Authorization Server could not be contacted. Please check your internet settings and try again. Please contact Customer Service if you continue to experience difficulties.", @"")
#define kPGPLicenseAuthTooManyRetrievals	NSLocalizedString(@"The number of License Authorization requests for your License Number has exceeded the maximum limit. Please try again later, or contact PGP Customer Service for assistance.", @"")

static CPGPLicenseAuth*		sSharedInstance = nil;

@implementation CPGPLicenseAuth

+ (CPGPLicenseAuth*)sharedLicenseAuth
{
	if (!sSharedInstance)
		sSharedInstance = [[CPGPLicenseAuth alloc] init];
	
	return sSharedInstance;
}

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self loadNib:kPGPLicenseAuthNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	NSBundle*		bundle = [NSBundle bundleForClass:[self class]];
	
	[NSBundle loadNibNamed:nibFile owner:self];
	
	[mTopView retain];
	[mLAView retain];
	[mBottomView retain];
	
	mTriangleUp = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:kPGPLicenseAuthTriangleUp]];
	mTriangleDown = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:kPGPLicenseAuthTriangleDown]];
	
	mLNFieldEditor = [[CPGPLicenseAuthFieldEditor alloc] init];
	[mLNFieldEditor setOwner:self];
	
	[mLA setFont:[NSFont fontWithName:@"Monaco" size:9]];
	
	[mNetProgress setUsesThreadedAnimation:YES];
	
	[mWindow setDelegate:self];
	
	mLoadedNib = YES;
}

+ (BOOL)isAuthorized
{
	NSString*		ln = [CPGPLicenseAuth prefLN];
	NSString*		la = [CPGPLicenseAuth prefLA];
	NSString*		name = [CPGPLicenseAuth prefName];
	NSString*		organization = [CPGPLicenseAuth prefOrganization];
	
	return [CPGPLicenseAuth authorize:ln licenseAuth:la name:name organization:organization];
}

+ (BOOL)evaluation
{
	return [CPGPLicenseAuth flags] & kPGPLicenseNumberFlag_Evaluation;
}

+ (BOOL)diskEnabled
{
	return [CPGPLicenseAuth flags] & kPGPLicenseNumberFlag_Disk;
}

+ (BOOL)mailEnabled
{
	return [CPGPLicenseAuth flags] & kPGPLicenseNumberFlag_Mail;
}

+ (BOOL)enterpriseEnabled
{
	return [CPGPLicenseAuth flags] & kPGPLicenseNumberFlag_Enterprise;
}

+ (BOOL)authorize:(NSString*)ln licenseAuth:(NSString*)la name:(NSString*)name organization:(NSString*)organization
{
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPError					err;
	
	if (!ln || !la || !name || !organization)
		return NO;
	
	if (IsPGPError(err = PGPLAImportBase64(context, [la UTF8String], &laRef)))
	{
		PGPLog((@"PGPLAImportBase64 returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPLNImportBase32(context, [ln UTF8String], &lnRef)))
	{
		PGPLog((@"PGPLNImportBase32 returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPLAVerify(context, lnRef, laRef, [name UTF8String], [organization UTF8String])))
	{
		PGPLog((@"PGPLAVerify returned err: %d", err));
		goto done;
	}

done:
	if (laRef)
		PGPLAFree(laRef);
	
	if (lnRef)
		PGPLNFree(lnRef);
	
	return IsPGPError(err) ? NO : YES;
}

- (void)displayLicenseEntryDialog:(BOOL)quitOnDeferral
{
	NSString*					la = [CPGPLicenseAuth prefLA];
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	PGPError					err;
	
	[self resetLicenseEntryDialog];
	
	if (la && IsntPGPError(err = PGPLAImportBase64(context, [la UTF8String], &laRef)))
	{
		[mLater setTitle:kPGPString_Cancel];
		
		PGPLAFree(laRef);
		laRef = kInvalidPGPLicenseAuthorizationRef;
	}
	
	if ([CPGPLicenseAuth daysLeft] <= 0)
	{
		if (quitOnDeferral)
		{
			[mLater setTitle:kPGPString_Quit];
			[mLater setAction:@selector(quit:)];
		}
	}
	
	[mWindow center];
	[mWindow makeKeyAndOrderFront:nil];
}

- (void)resetLicenseEntryDialog
{
	[mName setStringValue:@""];
	[mOrganization setStringValue:@""];
	[mLN1 setStringValue:@""];
	[mLN2 setStringValue:@""];
	[mLN3 setStringValue:@""];
	[mLN4 setStringValue:@""];
	[mLN5 setStringValue:@""];
	[mLN6 setStringValue:@""];
	[mLA setString:@""];
	
	[mLater setTitle:kPGPLicenseAuthLater];
	[mLater setAction:@selector(later:)];
	
	[mAuthorize setEnabled:NO];
	
	if ([mLAView superview])	/* If the LA view is onscreen */
		[self hideLicenseAuthorizationEntry:nil];
}

- (void)saveLicenseNumber:(NSString*)ln licenseAuth:(NSString*)la name:(NSString*)name organization:(NSString*)organization
{
	NSNotificationCenter*		center = [NSNotificationCenter defaultCenter];
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPPrefRef					prefRef = [[PGPAppController appController] pgpPrefRef];
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	char*						exportedLA = NULL;
	PGPError					err;
	
	if (!ln || !la || !name || !organization)
		return;
	
	if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefLicenseNumber, [ln UTF8String])))
		PGPLog((@"Failed to set license number pref (err=%d)", err));
	else
		[center postNotificationName:kPGPclPrefNotification_LicenseNumber object:nil];
	
	if (IsPGPError(err = PGPLAImportBlocked(context, [la UTF8String], &laRef)))
		PGPLog((@"Failed to import blocked LA (err=%d)", err));
	
	if (IsPGPError(err = PGPLAExportBase64(context, laRef, (PGPByte**) &exportedLA)))
		PGPLog((@"Failed to export LA (err=%d)", err));
	
	if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefLicenseAuthorization, exportedLA)))
		PGPLog((@"Failed to set license auth pref (err=%d)", err));
	else
		[center postNotificationName:kPGPclPrefNotification_LicenseAuthorization object:nil];
	
	if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefLicenseName, [name UTF8String])))
		PGPLog((@"Failed to set license name pref (err=%d)", err));
	else
		[center postNotificationName:kPGPclPrefNotification_LicenseName object:nil];

	if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefLicenseCompany, [organization UTF8String])))
		PGPLog((@"Failed to set license company pref (err=%d)", err));
	else
		[center postNotificationName:kPGPclPrefNotification_LicenseCompany object:nil];
	
	if (PGPPrefRefIsValid(prefRef))
		PGPclWritePrefs(prefRef);
}

- (void)rejectLicenseAuthorization:(NSString*)caption withMessage:(NSString*)message
{
	NSBeginAlertSheet(caption,
					  NSLocalizedString(@"OK", @""),
					  nil,
					  nil,
					  mWindow,
					  nil,
					  nil,
					  nil,
					  NULL,
					  message);
}

- (void)rejectLicenseNumber:(NSString*)caption withMessage:(NSString*)message
{
	NSBeginAlertSheet(caption,
					  NSLocalizedString(@"OK", @""),
					  nil,
					  nil,
					  mWindow,
					  nil,
					  nil,
					  nil,
					  NULL,
					  message);
}

- (void)finish
{
	[mWindow close];
	[[PGPAppController appController] licenseAuthFinished:self];
}

- (NSWindow*)window
{
	return mWindow;
}

- (NSString*)name
{
	NSString*	name = [mName stringValue];
	NSArray*	lines;
	NSString*	fixedName = name;
	
	if (strstr([fixedName UTF8String], "\r"))
	{
		lines = [fixedName componentsSeparatedByString:@"\r"];
		fixedName = [lines componentsJoinedByString:@""];
	}
	
	if (strstr([fixedName UTF8String], "\n"))
	{
		lines = [fixedName componentsSeparatedByString:@"\n"];
		fixedName = [lines componentsJoinedByString:@""];
	}
	
	return fixedName;
}

- (NSString*)organization
{
	NSString*	org = [mOrganization stringValue];
	NSArray*	lines;
	NSString*	fixedOrg = org;
	
	if (strstr([fixedOrg UTF8String], "\r"))
	{
		lines = [fixedOrg componentsSeparatedByString:@"\r"];
		fixedOrg = [lines componentsJoinedByString:@""];
	}
	
	if (strstr([fixedOrg UTF8String], "\n"))
	{
		lines = [fixedOrg componentsSeparatedByString:@"\n"];
		fixedOrg = [lines componentsJoinedByString:@""];
	}
	
	return fixedOrg;
}

- (NSString*)ln
{
	return [[NSArray arrayWithObjects:[mLN1 stringValue],
									  [mLN2 stringValue],
									  [mLN3 stringValue],
									  [mLN4 stringValue],
									  [mLN5 stringValue],
									  [mLN6 stringValue],
									  nil] componentsJoinedByString:@""];
}

- (NSString*)la
{
	NSString*	la = [mLA string];
	NSArray*	lines;
	NSString*	fixedLA = la;
	
	if (strstr([la UTF8String], "\r\n"))
		return la;
	else if (strchr([la UTF8String], '\r'))	/* Pasted text from Carbon source */
	{
		lines = [la componentsSeparatedByString:@"\r"];
		fixedLA = [lines componentsJoinedByString:@"\r\n"];
	}
	else if (strchr([la UTF8String], '\n')) /* Pasted text from Cocoa source */
	{
		lines = [la componentsSeparatedByString:@"\n"];
		fixedLA = [lines componentsJoinedByString:@"\r\n"];
	}
	
	return fixedLA;
}

+ (PGPError)prefLNRef:(PGPLicenseNumberRef*)lnRef
{
	NSString*		ln = [CPGPLicenseAuth prefLN];
	
	if (!lnRef)
		return kPGPError_BadParams;
	
	return PGPLNImportBase32([[PGPAppController appController] pgpContext], [ln UTF8String], lnRef);
}

+ (PGPError)prefLARef:(PGPLicenseAuthorizationRef*)laRef
{
	NSString*		la = [CPGPLicenseAuth prefLA];
	
	if (!laRef)
		return kPGPError_BadParams;
	
	return PGPLAImportBase64([[PGPAppController appController] pgpContext], [la UTF8String], laRef);
}

+ (PGPInt32)daysLeft
{
	NSString*					la = [CPGPLicenseAuth prefLA];
	NSString*					ln = [CPGPLicenseAuth prefLN];
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	PGPError					err;
	PGPInt32					days = 0;
	
	/* Failing to import the LA isn't necessarily an error. */
	if (IsPGPError(err = PGPLAImportBase64(context, [la UTF8String], &laRef)))
		laRef = kInvalidPGPLicenseAuthorizationRef;
	
	if (IsPGPError(err = PGPLNImportBase32(context, [ln UTF8String], &lnRef)))
	{
		PGPLog((@"PGPLNImportBase32 returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPLADaysLeft(context, lnRef, laRef, &days)))
		PGPLog((@"PGPLADaysLeft returned err: %d", err));
	
done:
	if (PGPLicenseAuthorizationRefIsValid(laRef))
	{
		PGPLAFree(laRef);
		laRef = kInvalidPGPLicenseAuthorizationRef;
	}
	
	if (PGPLicenseNumberRefIsValid(lnRef))
	{
		PGPLNFree(lnRef);
		lnRef = kInvalidPGPLicenseNumberRef;
	}
	
	return days;
}

+ (PGPUInt8)flags
{
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPUInt8					flags = 0;
	PGPError					err;
	NSString*					ln = [CPGPLicenseAuth prefLN];
	
	if (IsPGPError(err = PGPLNImportBase32(context, [ln UTF8String], &lnRef)))
	{
		PGPLog((@"PGPLNImportBase32 returned err: %d", err));
		return 0;
	}
	
	if (IsPGPError(err = PGPLNGetFlags(context, lnRef, &flags)))
		PGPLog((@"PGPLNGetFlags failed with err: %d", err));
	
	PGPLNFree(lnRef);
	lnRef = kInvalidPGPLicenseNumberRef;
	
	return flags;
}

+ (NSString*)prefName
{
	return [CPGPLicenseAuth getPrefString:kPGPPrefLicenseName];
}

+ (NSString*)prefOrganization
{
	return [CPGPLicenseAuth getPrefString:kPGPPrefLicenseCompany];
}

+ (NSString*)prefLN
{
	return [CPGPLicenseAuth getPrefString:kPGPPrefLicenseNumber];
}

+ (NSString*)prefLA
{
	return [CPGPLicenseAuth getPrefString:kPGPPrefLicenseAuthorization];
}

+ (NSString*)getPrefString:(PGPPrefIndex)prefID
{
	PGPPrefRef		prefRef = [[PGPAppController appController] pgpPrefRef];
	PGPError		err;
	char*			szData = NULL;
	NSString*		string = nil;
	
	if (IsPGPError(err = PGPGetPrefStringAlloc(prefRef, prefID, &szData)))
		PGPLog((@"Failed to get pref string %d (err=%d)", prefID, err));
	else
	{
		string = [NSString stringWithUTF8String:szData];
		
		PGPFreeData(szData);
	}
	
	return string;
}

- (IBAction)toggleLicenseAuthorizationEntry:(id)sender
{
	if ([mLAView superview])	/* If the LA view is onscreen */
		[self hideLicenseAuthorizationEntry:sender];
	else
		[self showLicenseAuthorizationEntry:sender];
}

- (IBAction)showLicenseAuthorizationEntry:(id)sender
{
	NSRect		windowFrame = [mWindow frame];
	NSRect		frame;
	int			numChars;
	
	windowFrame.size.height += NSHeight([mLAView frame]);
	windowFrame.origin.y -= NSHeight([mLAView frame]);
	
	frame = [mTopView frame];
	frame.origin.y += NSHeight([mLAView frame]);
	[mTopView setFrame:frame];
	
	[[mWindow contentView] addSubview:mLAView];
	
	[mDisclosureButton setImage:mTriangleUp];
	[mDisclosureTitle setStringValue:kPGPLicenseAuthAutomaticMsg];
	
	numChars = [[mLN1 stringValue] length] + [[mLN2 stringValue] length] + [[mLN3 stringValue] length] +
			   [[mLN4 stringValue] length] + [[mLN5 stringValue] length] + [[mLN6 stringValue] length];
	
	if (numChars == 28)
		[mAuthorize setEnabled:YES];
	else
		[mAuthorize setEnabled:NO];
	
	[mWindow setFrame:windowFrame display:YES];
}

- (IBAction)hideLicenseAuthorizationEntry:(id)sender
{
	NSRect		windowFrame = [mWindow frame];
	NSRect		frame;
	int			numChars;
	
	windowFrame.size.height -= NSHeight([mLAView frame]);
	windowFrame.origin.y += NSHeight([mLAView frame]);
	
	[mLAView removeFromSuperview];
	
	frame = [mTopView frame];
	frame.origin.y = NSMaxY([mBottomView frame]);
	[mTopView setFrame:frame];
	
	[mDisclosureButton setImage:mTriangleDown];
	[mDisclosureTitle setStringValue:kPGPLicenseAuthManualMsg];

	numChars = [[mLN1 stringValue] length] + [[mLN2 stringValue] length] + [[mLN3 stringValue] length] +
			   [[mLN4 stringValue] length] + [[mLN5 stringValue] length] + [[mLN6 stringValue] length];
	
	if (numChars == 28)
		[mAuthorize setEnabled:YES];
	else
		[mAuthorize setEnabled:NO];
	
	[mWindow setFrame:windowFrame display:YES];
}

- (IBAction)later:(id)sender
{
	[mWindow close];
	[[PGPAppController appController] licenseAuthDeferred:self];
}

- (IBAction)quit:(id)sender
{
	[mWindow close];
	[[PGPAppController appController] cleanupAndExit];
}

- (IBAction)purchaseNow:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:kPGPLicenseAuthPurchaseURL]];
}

- (IBAction)authorize:(id)sender
{
	if ([mLAView superview])	/* Validate entered License Authorization */
		[self authorizeLA:sender];
	else
		[self authorizeLN:sender];
}

- (IBAction)authorizeLA:(id)sender
{
	NSString*					licenseAuth = [self la];
	NSString*					name = [self name];
	NSString*					organization = [self organization];
	NSString*					licenseNum = [self ln];
	PGPUInt8					prodID;
	PGPError					err;
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	char						errStr[256];
	
	if (![licenseAuth length])
	{
		[self rejectLicenseAuthorization:kPGPLicenseAuthMissingLA withMessage:kPGPLicenseAuthMissingLAMsg];
		return;
	}
	
	if (![name length])
	{
		[self rejectLicenseAuthorization:kPGPLicenseAuthMissingName withMessage:kPGPLicenseAuthMissingNameOrgLNMsg];
		return;
	}
	
	if ([licenseNum length] < 28)
	{
		[self rejectLicenseAuthorization:kPGPLicenseAuthMissingLN withMessage:kPGPLicenseAuthMissingNameOrgLNMsg];
		return;
	}
	
	err = PGPLAImportBlocked(context, [licenseAuth UTF8String], &laRef);
	
	if (IsPGPError(err))
	{
		PGPLog((@"PGPLAImportBlocked returned err: %d", err));
		[self rejectLicenseAuthorization:kPGPLicenseAuthBadLA withMessage:kPGPLicenseAuthBadLAMsg];
		goto done;
	}
	
	if (IsPGPError(err = PGPLNImportBase32(context, [licenseNum UTF8String], &lnRef)))
	{
		PGPLog((@"PGPLNImportBase32 returned err: %d", err));
		if ((err == kPGPError_CorruptData) || (kPGPError_LNCorrupt))
			[self rejectLicenseAuthorization:kPGPLicenseAuthBadLN withMessage:kPGPLicenseAuthCorruptLNMsg];
		else
		{
			PGPGetErrorString(err, sizeof(errStr), errStr);
			[self rejectLicenseAuthorization:kPGPLicenseAuthBadLN withMessage:[NSString stringWithFormat:kPGPLicenseAuthBadLNGenericMsg, errStr]];
		}
		
		goto done;
	}
	
	if (IsPGPError(err = PGPLNGetProductID(context, lnRef, &prodID)))
	{
		PGPLog((@"PGPLNGetProductID failed with err: %d", err));
		goto done;
	}
	
	if (prodID != PGPPRODUCTID)
	{
		PGPLog((@"prodID != PGPPRODUCTID (prodID=%d)", prodID));
		[self rejectLicenseNumber:kPGPLicenseAuthInvalidProdID withMessage:kPGPLicenseAuthInvalidProdIDMsg];
		goto done;
	}
	
	err = PGPLAVerify(context, lnRef, laRef, [name UTF8String], [organization UTF8String]);
	
	if (IsPGPError(err))
	{
		if (err == kPGPError_KeyExpired)
		{
			PGPLog((@"PGPLAVerify returned err: kPGPError_KeyExpired"));
			[self rejectLicenseAuthorization:kPGPLicenseAuthBadVerify withMessage:kPGPLicenseAuthLAExpiredMsg];
		}
		else
		{
			PGPLog((@"PGPLAVerify returned err: %d", err));
			[self rejectLicenseAuthorization:kPGPLicenseAuthBadVerify withMessage:kPGPLicenseAuthBadVerifyMsg];
		}
	}
	
	[self saveLicenseNumber:licenseNum licenseAuth:licenseAuth name:name organization:organization];
	
done:
	if (laRef)
		PGPLAFree(laRef);
	
	if (lnRef)
		PGPLNFree(lnRef);
	
	if (IsntPGPError(err))
		[self finish];
}

- (IBAction)authorizeLN:(id)sender
{
	NSString*					name = [self name];
	NSString*					organization = [self organization];
	NSString*					licenseNum = [self ln];
	NSString*					server;
	NSArray*					lnPieces;
	NSArray*					servers;
	NSEnumerator*				enumerator;
	PGPUInt8					prodID;
	PGPByte*					exportedLA = NULL;
	PGPError					err;
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	BOOL						authorized = NO;
	char						errStr[256];
	
#if PGP_BETA
	servers = [NSArray arrayWithObject:@"http://authorize.foo.com/authorize1"];
#else
	servers = [NSArray arrayWithObjects:@"http://authorize1.pgp.com/authorize1",
										@"http://authorize2.pgp.com/authorize1",
										@"http://authorize3.pgp.com/authorize1",
										nil];
#endif
	
	if (![name length])
	{
		[self rejectLicenseAuthorization:kPGPLicenseAuthMissingName withMessage:kPGPLicenseAuthMissingNameOrgLNMsg];
		return;
	}
	
	if ([licenseNum length] < 28)
	{
		[self rejectLicenseAuthorization:kPGPLicenseAuthMissingLN withMessage:kPGPLicenseAuthMissingNameOrgLNMsg];
		return;
	}
	
	if (IsPGPError(err = PGPLNImportBase32(context, [licenseNum UTF8String], &lnRef)))
	{
		PGPLog((@"PGPLNImportBase32 returned err: %d", err));
		if ((err == kPGPError_CorruptData) || (kPGPError_LNCorrupt))
			[self rejectLicenseAuthorization:kPGPLicenseAuthBadLN withMessage:kPGPLicenseAuthCorruptLNMsg];
		else
		{
			PGPGetErrorString(err, sizeof(errStr), errStr);
			[self rejectLicenseAuthorization:kPGPLicenseAuthBadLN withMessage:[NSString stringWithFormat:kPGPLicenseAuthBadLNGenericMsg, errStr]];
		}
		
		goto done;
	}
	
	if (IsPGPError(err = PGPLNGetProductID(context, lnRef, &prodID)))
	{
		PGPLog((@"PGPLNGetProductID failed with err: %d", err));
		goto done;
	}
	
	if (prodID != PGPPRODUCTID)
	{
		PGPLog((@"prodID != PGPPRODUCTID (prodID=%d)", prodID));
		[self rejectLicenseNumber:kPGPLicenseAuthInvalidProdID withMessage:kPGPLicenseAuthInvalidProdIDMsg];
		err = kPGPError_FeatureNotAvailable;
		goto done;
	}
	
	[mNetName setStringValue:name];
	[mNetOrganization setStringValue:organization];
	
	lnPieces = [NSArray arrayWithObjects:[mLN1 stringValue],
										 [mLN2 stringValue],
										 [mLN3 stringValue],
										 [mLN4 stringValue],
										 [mLN5 stringValue],
										 [mLN6 stringValue],
										 nil];
	
	[mNetLN setStringValue:[lnPieces componentsJoinedByString:@"-"]];
	
	[mNetStatus setStringValue:@""];
	
	[NSApp beginSheet:mNetWindow
		   modalForWindow:mWindow
		   modalDelegate:self
		   didEndSelector:nil
		   contextInfo:nil];
	
	[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.5]];
	
	[mNetProgress startAnimation:nil];
	[mNetStatus setStringValue:NSLocalizedString(@"Requesting license authorization...", @"")];
	[mNetStatus display];
	
	enumerator = [servers objectEnumerator];
	
	while (!authorized && (server = [enumerator nextObject]))
	{
		PGPLog((@"Attempting to authorize with server %@", server));
		err = PGPLACreate(context,
					[server UTF8String],
					lnRef,
					[name UTF8String],
					[organization UTF8String],
					&laRef);
		
		PGPLog((@"PGPLACreate: %d", err));
		
		if ((err == kPGPError_NetLARefused) || (err == kPGPError_NetLAMismatch) || (err == kPGPError_NetLATooManyRetrievals))
			break;
		
		if (IsntPGPError(err))
			authorized = YES;
	}
	
	if (IsPGPError(err))
		[mNetStatus setStringValue:NSLocalizedString(@"Failed to obtain authorization...", @"")];
	else
		[mNetStatus setStringValue:NSLocalizedString(@"License successfully authorized.", @"")];
	
	[mNetStatus display];
	
	[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1.0]];
	
	[NSApp endSheet:mNetWindow];
	[mNetProgress stopAnimation:nil];
	[mNetWindow close];
	
	if (IsPGPError(err))
		PGPLog((@"PGPLACreate failed with err: %d", err));
	
	if (IsPGPError(err))
	{
		if ((err <= kPGPError_SocketsNetworkDown) && (err > kPGPError_X509AttributeNotSupported))
			[self rejectLicenseAuthorization:kPGPLicenseAuthAuthFailed withMessage:kPGPLicenseAuthNetworkProblemMsg];
		else if ((err == kPGPError_NetworkOpFailed) || (err == kPGPError_NetLARefused))
			[self rejectLicenseAuthorization:kPGPLicenseAuthAuthFailed withMessage:kPGPLicenseAuthLARefusedMsg];
		else if (err == kPGPError_NetLAMismatch)
			[self rejectLicenseAuthorization:kPGPLicenseAuthAuthFailed withMessage:kPGPLicenseAuthLAMismatchMsg];
		else if (err == kPGPError_NetLATooManyRetrievals)
			[self rejectLicenseAuthorization:kPGPLicenseAuthAuthFailed withMessage:kPGPLicenseAuthTooManyRetrievals];
		else
			[self rejectLicenseAuthorization:kPGPLicenseAuthAuthFailed withMessage:kPGPLicenseAuthGenericFailedMsg];
	}
	else
	{
		err = PGPLAVerify(context, lnRef, laRef, [name UTF8String], [organization UTF8String]);
		
		if (IsPGPError(err))
		{
			if (err == kPGPError_KeyExpired)
			{
				PGPLog((@"PGPLAVerify returned err: kPGPError_KeyExpired"));
				[self rejectLicenseAuthorization:kPGPLicenseAuthLAExpired withMessage:kPGPLicenseAuthLAExpiredMsg];
			}
			else
			{
				PGPLog((@"PGPLAVerify returned err: %d", err));
				[self rejectLicenseAuthorization:kPGPLicenseAuthBadVerify withMessage:kPGPLicenseAuthBadVerifyMsg];
			}
		}
		else
		{
			if (IsPGPError(err = PGPLAExportBlocked(context, laRef, &exportedLA)))
				PGPLog((@"PGPLAExportBase64 failed with err: %d", err));
			else
				[self saveLicenseNumber:licenseNum licenseAuth:[NSString stringWithUTF8String:exportedLA] name:name organization:organization];
		}
	}
	
done:
	if (laRef)
		PGPLAFree(laRef);
	
	if (lnRef)
		PGPLNFree(lnRef);
	
	if (IsntPGPError(err))
		[self finish];
}

- (BOOL)windowShouldClose:(id)sender
{
	[self performSelector:[mLater action] withObject:nil];
	
	return YES;
}

/* Let's us use a custom field editor for License Number entry. */
- (id)windowWillReturnFieldEditor:(NSWindow*)sender toObject:(id)object
{
	if ((object == mLN1) || (object == mLN2) || (object == mLN3) ||
		(object == mLN4) || (object == mLN5) || (object == mLN6))
	{
		return mLNFieldEditor;
	}
	
	return nil;
}

- (BOOL)shouldInsertText:(NSString*)text
{
	CPGPLicenseAuthFieldEditor*		fieldEditor = (CPGPLicenseAuthFieldEditor*) [mWindow firstResponder];
	NSTextField*					delegate = [fieldEditor delegate];
	NSString*						currentString = [delegate stringValue];
	NSString*						subString;
	BOOL							authorize = NO;
	BOOL							retval;
	int								numChars;
	
	numChars = [[mLN1 stringValue] length] + [[mLN2 stringValue] length] + [[mLN3 stringValue] length] +
			   [[mLN4 stringValue] length] + [[mLN5 stringValue] length] + [[mLN6 stringValue] length];
	
	if ((numChars == 28) || ((numChars + [text length]) >= 28))
		authorize = YES;
	
	if ([fieldEditor selectedRange].length)
	{
		if ([text length] < [fieldEditor selectedRange].length)
			[mAuthorize setEnabled:NO];
		
		retval = YES;
		goto done;
	}
	
	if ((delegate == mLN1) || (delegate == mLN2) || (delegate == mLN3) || (delegate == mLN4) || (delegate == mLN5))
	{
		if (([currentString length] + [text length]) <= 5)
		{
			retval = YES;
			goto done;
		}
		else
		{
			if ([currentString length] == 5)
			{
				if (![[(NSTextField*) [delegate nextKeyView] stringValue] length])
				{
					[mWindow makeFirstResponder:[delegate nextKeyView]];
					[fieldEditor moveToEndOfLine:self];
					[fieldEditor insertText:text];
				}
				else
				{
					if (numChars >= 28)
						authorize = YES;
					else
						authorize = NO;
				}
				
			}
			else
			{
				subString = [text substringWithRange:NSMakeRange(0, 5 - [currentString length])];
				[fieldEditor insertText:subString];
				
				if (![[(NSTextField*) [delegate nextKeyView] stringValue] length])
				{
					[mWindow makeFirstResponder:[delegate nextKeyView]];
					[fieldEditor moveToEndOfLine:self];
					
					subString = [text substringFromIndex:(5 - [currentString length])];
					
					if ([subString length])
						[fieldEditor insertText:subString];
				}
				else
				{
					numChars = [[mLN1 stringValue] length] + [[mLN2 stringValue] length] + [[mLN3 stringValue] length] +
							   [[mLN4 stringValue] length] + [[mLN5 stringValue] length] + [[mLN6 stringValue] length];
					
					if (numChars >= 28)
						authorize = YES;
					else
						authorize = NO;
				}
			}
			
			retval = NO;
			goto done;
		}
	}
	else if (delegate == mLN6)
	{
		if (([currentString length] + [text length]) <= 3)
		{
			retval = YES;
			goto done;
		}
		else
		{
			if ([currentString length] != 3)
			{
				subString = [text substringWithRange:NSMakeRange(0, 3 - [currentString length])];
				[fieldEditor insertText:subString];
			}
			
			retval = NO;
			goto done;
		}
	}
	
	retval = YES;
	
done:
	[mAuthorize setEnabled:authorize];
	
	return retval;
}

- (BOOL)shouldDeleteBackward:(id)sender
{
	CPGPLicenseAuthFieldEditor*		fieldEditor = (CPGPLicenseAuthFieldEditor*) [mWindow firstResponder];
	NSTextField*					delegate = [fieldEditor delegate];
	NSString*						currentString = [delegate stringValue];
	
	if ([currentString length])
	{
		[mAuthorize setEnabled:NO];
		return YES;
	}
	
	if ((delegate == mLN2) || (delegate == mLN3) || (delegate == mLN4) || (delegate == mLN5) || (delegate == mLN6))
	{
		[mWindow makeFirstResponder:[delegate previousKeyView]];
		[fieldEditor moveToEndOfLine:self];
		[fieldEditor deleteBackward:self];
	}
	
	return NO;
}

- (void)dealloc
{
	PGPLog((@"Deallocated CPGPLicenseAuth"));
	
	if (self == sSharedInstance)
		sSharedInstance = nil;
	
	[mWindow setDelegate:nil];
	
	[mTopView release];
	[mLAView release];
	[mBottomView release];
	
	[mLNFieldEditor release];
	
	[mTriangleUp release];
	[mTriangleDown release];
	
	[super dealloc];
}

@end

@implementation CPGPLicenseAuthFieldEditor

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mCharSet)
		mCharSet = [[NSCharacterSet characterSetWithCharactersInString:kPGPLicenseAuthValidChars] retain];
	
	[self setFieldEditor:YES];
	
	return self;
}

- (id)owner
{
	return mOwner;
}

- (void)setOwner:(id)owner
{
	mOwner = owner; /* Intentionally not retained. */
}

- (void)insertText:(NSString*)text
{
	NSString*	upperString = [text uppercaseString];
	NSArray*	hyphenatedText;
	unichar		character;
	BOOL		isValid = YES;
	int			len;
	int			i;
	
	hyphenatedText = [upperString componentsSeparatedByString:@"-"];
	upperString = [hyphenatedText componentsJoinedByString:@""];
	len = [upperString length];
	
	for (i = 0; (i < len) && isValid; ++i)
	{
		character = [upperString characterAtIndex:i];
		
		if (![mCharSet characterIsMember:character])
			isValid = NO;
	}
	
	if (isValid)
	{
		if (mOwner && [mOwner respondsToSelector:@selector(shouldInsertText:)])
		{
			if ([mOwner shouldInsertText:upperString])
				[super insertText:upperString];
		}
		else
			[super insertText:upperString];
	}
}

- (void)paste:(id)sender
{
	NSString*		text = [[NSPasteboard generalPasteboard] stringForType:NSStringPboardType];
	
	[self insertText:text];
}

- (void)cut:(id)sender
{
	[self deleteBackward:sender];
}

- (void)deleteBackward:(id)sender
{
	if (mOwner && [mOwner respondsToSelector:@selector(shouldDeleteBackward:)])
	{
		if ([mOwner shouldDeleteBackward:sender])
			[super deleteBackward:sender];
	}
	else
		[super deleteBackward:sender];
}

- (void)dealloc
{
	[mCharSet release];
	
	[super dealloc];
}

@end
