/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPLicenseDisplay.m,v 1.7 2002/11/22 08:44:36 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPLicenseDisplay.h"
#import "CPGPLicenseAuth.h"
#import <PGPclient/pgpBuild.h>

#define kPGPLicenseDisplayNib				@"CPGPLicenseDisplay.nib"
#define kPGPLicenseDisplayEvalBoxTitle		NSLocalizedString(@"Evaluation Product Information", @"")
#define kPGPLicenseDisplayNormalBoxTitle	NSLocalizedString(@"Product Information", @"")
#define kPGPLicenseDisplayEnterpriseFlag	NSLocalizedString(@"Enterprise Configuration:", @"")
#define kPGPLicenseDisplayDesktopFlag		NSLocalizedString(@"PGP Desktop:", @"")
#define kPGPLicenseDisplayDiskFlag			NSLocalizedString(@"PGP Disk:", @"")
#define kPGPLicenseDisplayMailFlag			NSLocalizedString(@"PGP Mail:", @"")

@implementation CPGPLicenseDisplay

static CPGPLicenseDisplay*		sSharedInstance = nil;

+ (CPGPLicenseDisplay*)sharedLicenseDisplay
{
	if (!sSharedInstance)
		sSharedInstance = [[CPGPLicenseDisplay alloc] init];
	
	return sSharedInstance;
}

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self loadNib:kPGPLicenseDisplayNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	NSTextFieldCell*	tfCell;
	
	[NSBundle loadNibNamed:nibFile owner:self];
	
	[mPurchaseNow retain];
	[mPurchaseMessage retain];
	
	tfCell = [[[NSTextFieldCell alloc] initTextCell:@""] autorelease];
	[tfCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	
	[mFlagNames setPrototype:tfCell];
	[mFlags setPrototype:tfCell];
	
	[mFlagNames addRow];
	[mFlagNames addRow];
	[mFlagNames addRow];
	[mFlagNames addRow];
	[mFlags addRow];
	[mFlags addRow];
	[mFlags addRow];
	[mFlags addRow];
	
	[mFlagNames removeRow:0];
	[mFlags removeRow:0];
	
	[[mFlagNames cellAtRow:0 column:0] setStringValue:kPGPLicenseDisplayEnterpriseFlag];
	[[mFlagNames cellAtRow:0 column:0] setTag:kPGPLicenseNumberFlag_Enterprise];
	[[mFlagNames cellAtRow:1 column:0] setStringValue:kPGPLicenseDisplayDesktopFlag];
	[[mFlagNames cellAtRow:1 column:0] setTag:kPGPLicenseNumberFlag_Desktop];	
	[[mFlagNames cellAtRow:2 column:0] setStringValue:kPGPLicenseDisplayDiskFlag];
	[[mFlagNames cellAtRow:2 column:0] setTag:kPGPLicenseNumberFlag_Disk];
	[[mFlagNames cellAtRow:3 column:0] setStringValue:kPGPLicenseDisplayMailFlag];
	[[mFlagNames cellAtRow:3 column:0] setTag:kPGPLicenseNumberFlag_Mail];
	
	[[mFlags cellAtRow:0 column:0] setTag:kPGPLicenseNumberFlag_Enterprise];
	[[mFlags cellAtRow:1 column:0] setTag:kPGPLicenseNumberFlag_Desktop];
	[[mFlags cellAtRow:2 column:0] setTag:kPGPLicenseNumberFlag_Disk];
	[[mFlags cellAtRow:3 column:0] setTag:kPGPLicenseNumberFlag_Mail];
	
	[mFlagNames setAutosizesCells:YES];
	[mFlags setAutosizesCells:YES];
	
	[mFlagNames sizeToCells];
	[mFlags sizeToCells];
	
	mLoadedNib = YES;
}

- (NSWindow*)window
{
	return mWindow;
}

- (void)setupDialog
{
	NSString*					licenseNum;
	NSString*					licenseAuth;
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	PGPLicenseAuthorizationRef	laRef = kInvalidPGPLicenseAuthorizationRef;
	PGPContextRef				context = [[PGPAppController appController] pgpContext];
	PGPUInt8					prodID = PGPPRODUCTID;
	PGPUInt8					flags = 0;
	PGPInt32					days = 0x7FFFFFFF;	/* Never */
	PGPUInt32					numLicenses = 1;
	PGPError					err;
	
	licenseNum = [CPGPLicenseAuth prefLN];
	licenseAuth = [CPGPLicenseAuth prefLA];
	
	if (IsPGPError(err = PGPLNImportBase32(context, [licenseNum UTF8String], &lnRef)))
	{
		PGPLog((@"PGPLNImportBase32 failed with err: %d", err));
		return;
	}
	
	/* Try to import the LA (if it exists). If it fails, we continue */
	if (IsPGPError(err = PGPLAImportBase64(context, [licenseAuth UTF8String], &laRef)))
		laRef = kInvalidPGPLicenseAuthorizationRef;
	
	if (IsPGPError(err = PGPLNGetProductID(context, lnRef, &prodID)))
		PGPLog((@"PGPLNGetProductID failed with err: %d", err));
	
	if (prodID != PGPPRODUCTID)
		PGPLog((@"prodID != PGPPRODUCTID (prodID=%d)", prodID));
	
	if (IsPGPError(err = PGPLADaysLeft(context, lnRef, laRef, &days)))
		PGPLog((@"PGPLADaysLeft failed with err: %d", err));
	
	if (IsPGPError(err = PGPLNGetNumLicenses(context, lnRef, &numLicenses)))
		PGPLog((@"PGPLNGetNumLicenses failed with err: %d", err));
	
	if (IsPGPError(err = PGPLNGetFlags(context, lnRef, &flags)))
		PGPLog((@"PGPLNGetFlags failed with err: %d", err));
	
	if (numLicenses == 1)
	{
		[mNameLabel setStringValue:NSLocalizedString(@"Name:", @"")];
		[mName setStringValue:[CPGPLicenseAuth prefName]];
	}
	else
	{
		[mNameLabel setStringValue:@""];
		[mName setStringValue:@""];
	}
	
	[mOrganization setStringValue:[CPGPLicenseAuth prefOrganization]];
	
	PGPLNFree(lnRef);
	lnRef = kInvalidPGPLicenseNumberRef;
	
	if (PGPLicenseAuthorizationRefIsValid(laRef))
	{
		PGPLAFree(laRef);
		laRef = kInvalidPGPLicenseAuthorizationRef;
	}
	
	[mLicenseSeats setStringValue:[NSString stringWithFormat:@"%d", numLicenses]];
	[self setExpirationDate:days flags:flags];
	
	if (!(flags & kPGPLicenseNumberFlag_Evaluation))
	{
		[mPurchaseNow removeFromSuperview];
		[mPurchaseMessage removeFromSuperview];
	}
	
	[self configureProductInfoBox:flags];
}

- (void)setExpirationDate:(PGPInt32)days flags:(PGPUInt8)flags
{
	NSCalendarDate*		date;
	
	if (days <= 0)
		[mExpiration setStringValue:kPGPLicenseDisplayExpired];
	else if (days == 0x7FFFFFFF)
		[mExpiration setStringValue:kPGPLicenseDisplayNever];
	else if (days <= 60)
	{
		if (flags & kPGPLicenseNumberFlag_Evaluation)
			[mExpiration setStringValue:[NSString stringWithFormat:kPGPLicenseDisplayEvalDays, days, (days == 1 ? NSLocalizedString(@" ", @"") : NSLocalizedString(@"s ", @""))]];
		else
			[mExpiration setStringValue:[NSString stringWithFormat:kPGPLicenseDisplayDaysRemaining, days, (days == 1 ? NSLocalizedString(@" ", @"") : NSLocalizedString(@"s ", @""))]];
	}
	else
	{
		date = [[NSCalendarDate calendarDate] dateByAddingYears:0 months:0 days:days hours:0 minutes:0 seconds:0];
		
		[mExpiration setStringValue:[date descriptionWithCalendarFormat:@"%b. %d, %Y"]];
	}
}

- (void)configureProductInfoBox:(PGPUInt8)flags
{
	NSAttributedString*		attrString;
	NSDictionary*			dict;
	NSFont*					font;
	
	if (flags & kPGPLicenseNumberFlag_Evaluation)
	{
		font = [NSFont boldSystemFontOfSize:11];
		dict = [NSDictionary dictionaryWithObjectsAndKeys:font,
							NSFontAttributeName,
							[NSColor redColor],
							NSForegroundColorAttributeName,
							nil];
		attrString = [[[NSAttributedString alloc] initWithString:kPGPLicenseDisplayEvalBoxTitle attributes:dict] autorelease];
		
		[mProductInfoBox setTitle:kPGPLicenseDisplayEvalBoxTitle];
		[[mProductInfoBox titleCell] setAttributedStringValue:attrString];
	}
	else
	{
		font = [NSFont boldSystemFontOfSize:11];
		dict = [NSDictionary dictionaryWithObjectsAndKeys:font,
							NSFontAttributeName,
							[NSColor blackColor],
							NSForegroundColorAttributeName,
							nil];
		attrString = [[[NSAttributedString alloc] initWithString:kPGPLicenseDisplayNormalBoxTitle attributes:dict] autorelease];
		
		[mProductInfoBox setTitle:kPGPLicenseDisplayNormalBoxTitle];
		[[mProductInfoBox titleCell] setAttributedStringValue:attrString];
	}
	
	[[mFlags cellWithTag:kPGPLicenseNumberFlag_Enterprise] setStringValue:(flags & kPGPLicenseNumberFlag_Enterprise ? @"Enabled" : @"Disabled")];
	[[mFlags cellWithTag:kPGPLicenseNumberFlag_Desktop] setStringValue:(flags & kPGPLicenseNumberFlag_Desktop ? @"Enabled" : @"Disabled")];
	[[mFlags cellWithTag:kPGPLicenseNumberFlag_Mail] setStringValue:(flags & kPGPLicenseNumberFlag_Mail ? @"Enabled" : @"Disabled")];
	[[mFlags cellWithTag:kPGPLicenseNumberFlag_Disk] setStringValue:(flags & kPGPLicenseNumberFlag_Disk ? @"Enabled" : @"Disabled")];
}

- (void)resetDialog
{
	[mName setStringValue:@""];
	[mOrganization setStringValue:@""];
	[mLicenseSeats setStringValue:@""];
	[mExpiration setStringValue:@""];
	
	[mProductInfoBox setTitle:kPGPLicenseDisplayNormalBoxTitle];
	
	[[mWindow contentView] addSubview:mPurchaseNow];
	[[mWindow contentView] addSubview:mPurchaseMessage];
}

- (void)display
{
	[self setupDialog];
	
	[mWindow center];
	[mWindow makeKeyAndOrderFront:nil];
}

- (IBAction)purchaseNow:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:kPGPLicenseDisplayPurchaseURL]];
}

- (IBAction)changeLicense:(id)sender
{
	[mWindow close];
	[[PGPAppController appController] changeLicense];
	[self autorelease];
}

- (IBAction)ok:(id)sender
{
	[mWindow close];
	[self autorelease];
}

- (void)dealloc
{
	PGPLog((@"Deallocated CPGPLicenseDisplay"));
	
	if (self == sSharedInstance)
		sSharedInstance = nil;
	
	[mPurchaseNow release];
	[mPurchaseMessage release];
	
	[super dealloc];
}

@end
