/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPUninstaller.m,v 1.19 2002/11/05 04:40:24 jeffc Exp $
____________________________________________________________________________*/

#import <unistd.h>
#import <sys/wait.h>
#import <sys/stat.h>
#import <Carbon/Carbon.h>

#import "CPGPUninstaller.h"
#import "PGPAppController.h"
#import "PGPBundleDisk.h"

#define kPGPUninstallerString_NeedAuth			NSLocalizedString(@"In order to proceed with the removal of this software, you will need to provide the administrator password for this computer. Please click the lock icon below, and enter the administrator password.", @"")
#define kPGPUninstallerString_BeginUninstall	NSLocalizedString(@"Click the Uninstall button to begin the uninstall process. If you have any PGPdisks mounted, it is recommended that you unmount them now or risk losing unsaved data.", @"")
#define kPGPUninstallerString_ContinueUninstall	NSLocalizedString(@"Please close any open documents and unmount any mounted PGPdisk volumes. Click the Continue button to continue the uninstall process.", @"")
#define kPGPUninstallerString_Abort				NSLocalizedString(@"Abort software removal", @"")
#define kPGPUninstallerString_AbortMsg			NSLocalizedString(@"Are you sure you want to abort the uninstall process? PGP operations may not behave in the expected manner.", @"")
#define kPGPUninstallerString_Finish			NSLocalizedString(@"The uninstall process has completed. It is highly recommended that you reboot your computer at this point.", @"")
#define kPGPUninstallerString_UnmountFailed		NSLocalizedString(@"Unmount failure", @"")
#define kPGPUninstallerString_UnmountFailedMsg	NSLocalizedString(@"PGPdisk could not shut down because one or more PGPdisks failed to unmount. Please close any open documents and unmount your PGPdisk(s) now, or they will be forcibly unmounted (possibly corrupting your data). Click the Continue button when you have unmounted your disk(s).", @"")

#define kPGPUninstallStatus_None			100
#define kPGPUninstallStatus_Uninstalling	101
#define kPGPUninstallStatus_Cancelling		102
#define kPGPUninstallStatus_Suspended		103
#define kPGPUninstallTextWidth				317
#define kPGPUninstallTextHeight				64
#define kPGPUninstallConditionWaiting		100
#define kPGPUninstallConditionProceed		101

@implementation CPGPUninstaller

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self initWithNib:kPGPUninstallerNib];

	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
    
	[mUninstallWindow setDelegate:self];
	
	[mCleanUninstall retain];
	[mCleanText retain];
	[mAuthButton retain];
	[mAuthText retain];
    [mUninstallProgress retain];
    [mUninstallButton retain];
    [mUninstallStatus retain];
    
	[mUninstallButton removeFromSuperview];
	[mCleanUninstall removeFromSuperview];
	[mCleanText removeFromSuperview];
	
    mLockImage = [[NSImage alloc] initWithContentsOfFile:
                    [[NSBundle bundleForClass:[self class]] pathForImageResource:@"lock"]];
    mUnlockImage = [[NSImage alloc] initWithContentsOfFile:
                    [[NSBundle bundleForClass:[self class]] pathForImageResource:@"unlock"]];
	
	[mUninstallProgress removeFromSuperview];
    [mUninstallProgress setUsesThreadedAnimation:YES];

    if ([self haveAuthorization])
    {
        [mAuthText setStringValue:NSLocalizedString(@"Click the lock to prevent further changes.", @"")];
        [self setUninstallText:kPGPUninstallerString_BeginUninstall];
        [mAuthButton setImage:mUnlockImage];
        [self addUninstallControls];
    }
    else
    {
        [mAuthText setStringValue:NSLocalizedString(@"Click the lock to make changes.", @"")];
        [self setUninstallText:kPGPUninstallerString_NeedAuth];
        [mAuthButton setImage:mLockImage];
        [self removeUninstallControls];
    }

    [mAuthButton setEnabled:YES];
	[mUninstallWindow displayIfNeeded];

	mLoadedNib = YES;
	
    return [self init];
}

- (void)setInfo:(NSArray*)info
{
	if (info)
		mInstallInfo = [info retain];
}

- (PGPError)displayUninstaller
{
	mUninstallReturnCode = kPGPError_UserAbort;
	mStatus = kPGPUninstallStatus_None;
    
	[NSApp runModalForWindow:mUninstallWindow];
	
	return mUninstallReturnCode;
}

- (BOOL)haveAuthorization
{
    AuthorizationRef	authRef;
    AuthorizationRights	authRights;
    AuthorizationFlags	authFlags;
    AuthorizationItem	authItem;
    OSStatus			err;
    
    authRights.count = 0;
    authRights.items = NULL;
    
    authFlags = kAuthorizationFlagDefaults;

    err = AuthorizationCreate(&authRights, kAuthorizationEmptyEnvironment, authFlags, &authRef);
    if (err != errAuthorizationSuccess)
        return NO;
    
    authItem.name = kAuthorizationRightExecute;
    authItem.value = (char*) [kPGPUninstallerLaunchPath cString];
    authItem.valueLength = [kPGPUninstallerLaunchPath length];
    authItem.flags = 0;

    authRights.count = 1; 
    authRights.items = &authItem;
        
    authFlags = kAuthorizationFlagExtendRights;
        
    err = AuthorizationCopyRights(authRef, &authRights, kAuthorizationEmptyEnvironment, authFlags, NULL);

    if (mAuthRef)
        AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
    mAuthRef = authRef;    
    
    if (err == errAuthorizationSuccess)
        return YES;
    else
        return NO;
}

- (BOOL)obtainAuthorization
{
    AuthorizationItem	authItem;
    OSStatus			err;
 
    if (!mAuthRef)
    {
        if ([self haveAuthorization])
            return YES;
    }
    
    authItem.name = kAuthorizationRightExecute;
    authItem.value = (char*) [kPGPUninstallerLaunchPath cString];
    authItem.valueLength = [kPGPUninstallerLaunchPath length];
    authItem.flags = 0;
    
    mRights.count = 1;
    mRights.items = &authItem;
    
    mFlags = kAuthorizationFlagExtendRights |
             kAuthorizationFlagInteractionAllowed |
             kAuthorizationFlagPreAuthorize;
    
    [mAuthText setStringValue:NSLocalizedString(@"Authenticating...", @"")];
    [mAuthText displayIfNeeded];
    
    err = AuthorizationCopyRights(mAuthRef, &mRights, kAuthorizationEmptyEnvironment, mFlags, NULL);
    if (err == errAuthorizationSuccess)
    {
        [mAuthButton setImage:mUnlockImage];
        
        return YES;
    }
    else
    {
        [mAuthButton setImage:mLockImage];
        
        return NO;
    }
}

- (void)addUninstallControls
{
    NSView*		contentView = [mUninstallWindow contentView];
    
	[contentView addSubview:mCleanUninstall];
	[contentView addSubview:mCleanText];
    [contentView addSubview:mUninstallButton];
    [contentView addSubview:mUninstallStatus];
}

- (void)removeUninstallControls
{
	[mCleanUninstall removeFromSuperview];
	[mCleanText removeFromSuperview];
	[mUninstallProgress removeFromSuperview];
    [mUninstallStatus removeFromSuperview];
}

- (PGPUInt32)status
{
	return mStatus;
}

- (void)setUninstallText:(NSString*)string
{
    NSSize		tfSize;
    NSRect		newFrame;
    float		yOffset;
    
    [mUninstallText setStringValue:string];
    
    tfSize = [[mUninstallText cell] cellSizeForBounds:NSMakeRect(0, 0, kPGPUninstallTextWidth, kPGPUninstallTextHeight)];
    
    newFrame = [mUninstallText frame];
    newFrame.size.width = kPGPUninstallTextWidth;
    newFrame.size.height = tfSize.height;
    
    if ((yOffset = NSHeight([mUninstallImage frame]) - NSHeight(newFrame)) > 0)
        newFrame.origin.y = [mUninstallImage frame].origin.y + (yOffset / 2);
    
    [mUninstallText setFrame:newFrame];
	[mUninstallWindow display];
}

- (void)setUninstallStatus:(NSString*)string
{
    [mUninstallStatus setStringValue:string];
}

- (void)uninstallThread
{
	PGPAppController*	appController = [PGPAppController appController];
    NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	OSStatus			err;
	NSString*			bundlePath = [[NSBundle mainBundle] bundlePath];
	NSString*			installtoolPath = [[NSString alloc] initWithFormat:@"%@/Contents/Resources/%@", bundlePath, kPGPUninstallerLaunchPath];
	NSMutableData*		data;
	id					diskController;
    char				statusBuf[256];
    char**				args;
	BOOL				unmountedAll = YES;
    FILE*				iopipe;
	
	mBusy = YES;
	
	if (diskController = [[appController bundleManager] bundleByName:@"PGPdisk.bundle"])
	{
		data = [[NSMutableData alloc] init];
		
		[self setUninstallStatus:NSLocalizedString(@"Shutting down PGPdisk", @"")];
		[diskController performSelectorOnMainThread:@selector(unmountAllDisks:) withObject:data waitUntilDone:YES];
		
		[data getBytes:&unmountedAll length:sizeof(BOOL)];
		[data release];
		
		if (!unmountedAll)
		{
			[self setUninstallText:kPGPUninstallerString_ContinueUninstall];
			[self setUninstallStatus:@""];
			[mUninstallProgress stopAnimation:nil];
			[mUninstallProgress removeFromSuperview];
			
			NSBeginAlertSheet(kPGPUninstallerString_UnmountFailed,
							NSLocalizedString(@"OK", @""),
							nil,
							nil,
							mUninstallWindow,
							nil,
							nil,
							nil,
							nil,
							kPGPUninstallerString_UnmountFailedMsg);
			
			[mUninstallButton setAction:@selector(cont:)];
			[mUninstallButton setEnabled:YES];
			[mUninstallWindow setDefaultButtonCell:[mUninstallButton cell]];
			[mUninstallButton setTitle:NSLocalizedString(@"Continue", @"")];
			
			mLock = [[NSConditionLock alloc] initWithCondition:kPGPUninstallConditionWaiting];
			[mLock lock];
			[mLock lockWhenCondition:kPGPUninstallConditionProceed];
			
			[mLock release];
			mLock = nil;
			
			[self setUninstallStatus:NSLocalizedString(@"Shutting down PGPdisk", @"")];
		}
		else
			PGPLog((@"Finished unmounting disks"));
	}
	
	/*
	We need to generate an argv array for the installtool.
	The array is malloc'd, so we free it after we're done.
	*/
	args = [self installtoolArguments];
    err = AuthorizationExecuteWithPrivileges(mAuthRef, [installtoolPath cString], 0, args, &iopipe);
	free(args);

	while(!feof(iopipe))
    {
        fgets(statusBuf, sizeof(statusBuf), iopipe);
		
		if (strstr(statusBuf, "PGPapp"))
			[self setUninstallStatus:NSLocalizedString(@"Removing PGP", @"")];
		else if (strstr(statusBuf, "PGPdisk"))
			[self setUninstallStatus:NSLocalizedString(@"Removing PGPdisk", @"")];
		else if (strstr(statusBuf, "PGPkeys"))
			[self setUninstallStatus:NSLocalizedString(@"Removing PGPkeys", @"")];
		else if (strstr(statusBuf, "PGPmail"))
			[self setUninstallStatus:NSLocalizedString(@"Removing PGPmail", @"")];
		else if (strstr(statusBuf, "PGPnet"))
			[self setUninstallStatus:NSLocalizedString(@"Removing PGPnet", @"")];
		
		statusBuf[0] = '\0';
    }

	wait(NULL);
	
	[mUninstallProgress stopAnimation:nil];
	[self removeUninstallControls];
	[self setUninstallText:kPGPUninstallerString_Finish];
	
	[mUninstallButton setAction:@selector(quit:)];
	[mUninstallButton setEnabled:YES];
	[mUninstallWindow setDefaultButtonCell:[mUninstallButton cell]];
	[mUninstallButton setTitle:NSLocalizedString(@"Quit", @"")];
	
	/*
	The return code is set to NoErr so that PGPAppController
	will exit the app once the Quit button is pressed.
	*/
	
	mUninstallReturnCode = kPGPError_NoErr;
    
	mBusy = NO;
	mFinished = YES;
	
	[pool release];
}

- (char**)installtoolArguments
{
	PGPAppController*	appController = [PGPAppController appController];
	NSEnumerator*		enumerator;
	NSString*			item;
	NSString*			bundlePath = [[NSBundle mainBundle] bundlePath];
	NSMutableData*		data;
	id					diskController;
	pid_t				pid;
	unsigned			argIndex;
	char**				argv;
	BOOL				havePID = NO;
	
	if (!mInstallInfo)
		return NULL;
	
	if (diskController = [[appController bundleManager] bundleByName:@"PGPdisk.bundle"])
	{
		data = [[NSMutableData alloc] init];
		
		[diskController performSelectorOnMainThread:@selector(diskEnginePID:) withObject:data waitUntilDone:YES];
		
		[data getBytes:&pid length:sizeof(pid_t)];
		[data release];
		
		havePID = YES;
	}
	
	argv = (char**) malloc(sizeof(char*) * ([mInstallInfo count] + (diskController ? 7 : 6)));
	if (!argv)
		return NULL;
	
	argv[0] = (char*) [[[NSString stringWithString:@"-installpath="] stringByAppendingString:bundlePath] cString];
	argv[1] = (char*) [[NSString stringWithFormat:@"-uid=%d", getuid()] cString];
	argv[2] = (char*) [[NSString stringWithFormat:@"-gid=%d", getgid()] cString];
	
	argIndex = 3;
	
	if (havePID)
	{
		argv[argIndex] = (char*) [[NSString stringWithFormat:@"-pid=%d", pid] cString];
		argIndex++;
	}
	
	argv[argIndex] = "-remove";
	argIndex++;
	
	if ([mCleanUninstall state] == NSOnState)
	{
		argv[argIndex] = "-clean";
		argIndex++;
	}
	
	enumerator = [mInstallInfo objectEnumerator];
	
	while (item = [enumerator nextObject])
	{
		if ([item isEqualToString:@"PGPapp"])
			argv[argIndex++] = (char*) [[NSString stringWithString:@"-app"] cString];
		else if ([item isEqualToString:@"PGPdisk.bundle"])
			argv[argIndex++] = (char*) [[NSString stringWithString:@"-disk"] cString];
		else if ([item isEqualToString:@"PGPkeys.bundle"])
			argv[argIndex++] = (char*) [[NSString stringWithString:@"-keys"] cString];
		else if ([item isEqualToString:@"PGPmail.bundle"])
			argv[argIndex++] = (char*) [[NSString stringWithString:@"-mail"] cString];
		else if ([item isEqualToString:@"PGPnet.bundle"])
			argv[argIndex++] = (char*) [[NSString stringWithString:@"-net"] cString];
	}
	
	argv[argIndex] = NULL;
	
	return argv;
}

- (IBAction)uninstall:(id)sender
{
	[self uninstallWarningDidEnd:nil returnCode:NSOKButton contextInfo:nil];
	/*
	NSBeginAlertSheet(kPGPUninstallerString_RebootWarning,
					  NSLocalizedString(@"Uninstall", @""),
					  NSLocalizedString(@"Cancel", @""),
					  nil,
					  mUninstallWindow,
					  self,
					  nil,
					  @selector(uninstallWarningDidEnd:returnCode:contextInfo:),
					  nil,
					  kPGPUninstallerString_RebootWarningMsg);
	*/
}

- (IBAction)restart:(id)sender
{
	[mUninstallWindow close];
	
	system("/usr/bin/osascript -e 'tell application \"Finder\" to restart' &");
	
	[NSApp terminate:nil];
}

- (IBAction)quit:(id)sender
{
	NSUserDefaults*			defaults;
	NSMutableDictionary*	dict;
	
	defaults = [NSUserDefaults standardUserDefaults];
	dict = [[[defaults persistentDomainForName:kPGPPrefsDomain] mutableCopy] autorelease];
	
	if (!dict)
		dict = [[[NSMutableDictionary alloc] init] autorelease];
	
	[dict setObject:[NSNumber numberWithBool:([mCleanUninstall state] == NSOnState ? YES : NO)] forKey:kPGPAppDefault_CleanUninstall];
	
	[defaults removePersistentDomainForName:kPGPPrefsDomain];
	[defaults setPersistentDomain:dict forName:kPGPPrefsDomain];
	[defaults synchronize];
	
	[mUninstallWindow close];
	
	[NSApp terminate:nil];
}

- (IBAction)cont:(id)sender
{
	[[mUninstallWindow contentView] addSubview:mUninstallProgress];
	[mUninstallButton setEnabled:NO];
	[mUninstallProgress startAnimation:nil];
	
	[mLock unlockWithCondition:kPGPUninstallConditionProceed];
}

- (IBAction)lock:(id)sender
{
    if ([mAuthButton image] == mLockImage)
    {
        if ([self obtainAuthorization])
		{
            [self setUninstallText:kPGPUninstallerString_BeginUninstall];
            [mAuthText setStringValue:NSLocalizedString(@"Click the lock to prevent further changes.", @"")];
            [self addUninstallControls];
		}
        else
		{
            [self setUninstallText:kPGPUninstallerString_NeedAuth];
            [mAuthText setStringValue:NSLocalizedString(@"Click the lock to make changes.", @"")];
			[mUninstallButton removeFromSuperview];
            [self removeUninstallControls];
		}
            
		[mAuthText displayIfNeeded];
    }
    else
    {
        if (mAuthRef)
        {
            AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
            mAuthRef = NULL;
        }
        
        [self setUninstallText:kPGPUninstallerString_NeedAuth];
        [mAuthText setStringValue:NSLocalizedString(@"Click the lock to make changes.", @"")];
        [mAuthText displayIfNeeded];
        [mAuthButton setImage:mLockImage];
		[mUninstallButton removeFromSuperview];
        [self removeUninstallControls];
    }        
}

- (BOOL)windowShouldClose:(id)sender
{
	if (mFinished)
	{
		[NSApp terminate:nil];
		
		return YES;
	}
	else if (!mBusy)
	{
		[NSApp stopModal];
		return YES;
	}
	
	return NO;
}

- (void)uninstallWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	if (returnCode != NSOKButton)
		return;
	
    if (mStatus == kPGPUninstallStatus_None)
        mStatus = kPGPUninstallStatus_Uninstalling;
    
    if (mStatus == kPGPUninstallStatus_Uninstalling)
    {
		[NSApp setServicesProvider:nil];	/* We no longer handle app services */
		
		[mAuthButton removeFromSuperview];
		[mAuthText removeFromSuperview];
		[[mUninstallWindow contentView] addSubview:mUninstallProgress];
        [mUninstallProgress startAnimation:nil];
		[mUninstallButton setEnabled:NO];
        [mAuthButton setEnabled:NO];
        [mAuthText setTextColor:[NSColor disabledControlTextColor]];
		[mCleanUninstall setEnabled:NO];
		[mCleanText setTextColor:[NSColor disabledControlTextColor]];
		
		[mUninstallWindow displayIfNeeded];

		mUninstallReturnCode = kPGPError_NoErr;

        if (!mThreadDetached)
        {
            mThreadDetached = YES;
            [NSThread detachNewThreadSelector:@selector(uninstallThread) toTarget:self withObject:nil];
        }
    }
}

- (void)restartWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	if (returnCode == NSCancelButton)
	{
		[mUninstallWindow close];
        
		system("/usr/bin/osascript -e 'tell application \"Finder\" to restart' &");
		
		[NSApp terminate:nil];
	}
}

- (void)dealloc
{
	if (mAuthRef)
        AuthorizationFree(mAuthRef, kAuthorizationFlagDestroyRights);
	
	if (mInstallInfo)
		[mInstallInfo release];
	
	if (mCleanUninstall)
		[mCleanUninstall release];
	
	if (mCleanText)
		[mCleanText release];
	
	if (mAuthButton)
		[mAuthButton release];
	
	if (mAuthText)
		[mAuthText release];
	
	if (mUninstallProgress)
		[mUninstallProgress release];
	
	if (mUninstallButton)
		[mUninstallButton release];
	
	if (mUninstallStatus)
		[mUninstallStatus release];
	
	[super dealloc];
}

@end
