/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPInterfaceController.m,v 1.76 2002/11/21 09:58:17 jeffc Exp $
____________________________________________________________________________*/

#import "PGPInterfaceController.h"
#import "CPGPDockMenu.h"
#import <Carbon/Carbon.h>
#import <PGPclient/CLicense.h>
#import "pgpLicenseNumber.h"
#import "CPGPLicenseAuth.h"

#define kPGPInterface_UninstallConfirmation		NSLocalizedString(@"Uninstall warning", @"")
#define kPGPInterface_UninstallConfirmationMsg	NSLocalizedString(@"Are you sure you want to uninstall PGP?", @"")

#define kPGPInterface_AboutMenuItem				NSLocalizedString(@"About PGP", @"")
#define kPGPInterface_LicenseMenuItem			NSLocalizedString(@"License", @"")
#define kPGPInterface_PreferencesMenuItem		NSLocalizedString(@"Preferences...", @"")
#define kPGPInterface_UninstallMenuItem			NSLocalizedString(@"Uninstall...", @"")
#define kPGPInterface_QuitMenuItem				NSLocalizedString(@"Quit PGP", @"")
#define kPGPInterface_BringAllToFront			NSLocalizedString(@"Bring All to Front", @"")
#define kPGPInterface_Disk						NSLocalizedString(@"PGPdisk", @"")
#define kPGPInterface_Keys						NSLocalizedString(@"PGPkeys", @"")
#define kPGPInterface_Net						NSLocalizedString(@"PGPnet", @"")
#define kPGPInterface_Mail						NSLocalizedString(@"PGPmail", @"")
#define kPGPInterface_WindowsMenuSeparator		100

@implementation PGPInterfaceController

- (id)init
{
    if (!mInited)
    {
        [super init];
        
        mInited = YES;
    }

	if (!mLoadedNib)
		[self initWithNib:kPGPInterfaceControllerNib];
        
	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
	NSAttributedString*		attrString;
	NSImage*				pgpLock;
	NSImage*				alphaLock;
	NSData*					rtfData;
	NSRect					frame;
	float					oldWidth;
	
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	
	pgpLock = [[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForImageResource:@"pgp"]] autorelease];
	alphaLock = [[[NSImage alloc] initWithSize:[pgpLock size]] autorelease];
	[alphaLock lockFocus];
	[pgpLock dissolveToPoint:NSZeroPoint fraction:0.2];
	[alphaLock unlockFocus];
	[mAboutImage setImage:alphaLock];
	
	[mClipView setDrawsBackground:NO];
	[mCreditsContainer addSubview:mCreditsView];
	
	rtfData = [NSData dataWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"credits" ofType:@"rtf"]];
	attrString = [[[NSAttributedString alloc] initWithRTF:rtfData documentAttributes:nil] autorelease];
	
	oldWidth = NSWidth([mCreditsText frame]);
	[mCreditsText setDrawsBackground:YES];
	[mCreditsText setAttributedStringValue:attrString];
	[mCreditsText sizeToFit];
	
	frame = [mCreditsText frame];
	frame.size.width = oldWidth;
	[mCreditsText setFrame:frame];
	
	mStartingPoint = (NSHeight([mCreditsText frame]) + NSHeight([mClipView frame])) - 30;
	
	[mVersionText setStringValue:[NSString stringWithFormat:NSLocalizedString(@"Version %s", @""), PGPVERSIONSTRING]];
	
	[mAboutWindow setDelegate:self];
	
	mLoadedNib = YES;

    return [self init];
}

- (id)initWithController:(id)controller
{
	mController = controller;
	
	return [self init];
}

- (id)controller
{
	return mController;
}

- (void)setController:(id)controller
{
	if (controller)
		mController = controller;
}

- (BOOL)initializeInterface
{
	NSMenu*				windowsMenu = [NSApp windowsMenu];
	NSMenuItem*			menuItem;
	NSArray*			bundleNames = [NSArray arrayWithObjects:@"PGPmail.bundle", @"PGPdisk.bundle", @"PGPkeys.bundle", nil];
	NSString*			bundleName;
	NSEnumerator*		enumerator;
	id <PGPBundle>		controller;
	id <PGPBundleDisk>	disk = [[mController bundleManager] bundleByName:@"PGPdisk.bundle"];
	id <PGPBundleKeys>	keys = [[mController bundleManager] bundleByName:@"PGPkeys.bundle"];
	PGPUInt32			index = 1;
	PGPUInt32			insertionIndex = [windowsMenu indexOfItemWithTag:kPGPInterface_WindowsMenuSeparator] + 1;
		
	[NSApp setMainMenu:mMainMenu];
	
	enumerator = [bundleNames objectEnumerator];
	
	while (bundleName = [enumerator nextObject])
	{
		if (!(controller = [[mController bundleManager] bundleByName:bundleName]))
			continue;
		
		[controller addMenus];
		
		if ([[[(id) controller class] identifier] isEqualToString:@"PGPdisk.bundle"])
		{
			menuItem = [[[NSMenuItem alloc] initWithTitle:kPGPInterface_Disk action:@selector(disk:) keyEquivalent:[NSString stringWithFormat:@"%d", index++]] autorelease];
		}
		else if ([[[(id) controller class] identifier] isEqualToString:@"PGPkeys.bundle"])
		{
			menuItem = [[[NSMenuItem alloc] initWithTitle:kPGPInterface_Keys action:@selector(keys:) keyEquivalent:[NSString stringWithFormat:@"%d", index++]] autorelease];
		}
		else if ([[[(id) controller class] identifier] isEqualToString:@"PGPnet.bundle"])
		{
			menuItem = [[[NSMenuItem alloc] initWithTitle:kPGPInterface_Net action:@selector(net:) keyEquivalent:[NSString stringWithFormat:@"%d", index++]] autorelease];
		}
		else
		{
			menuItem = [[[NSMenuItem alloc] initWithTitle:kPGPInterface_Mail action:@selector(mail:) keyEquivalent:[NSString stringWithFormat:@"%d", index++]] autorelease];
		}
		
		[menuItem setImage:[controller menuImage]];
		[menuItem setTarget:self];
		[menuItem setKeyEquivalentModifierMask:(NSCommandKeyMask | NSAlternateKeyMask)];
		[windowsMenu insertItem:menuItem atIndex:insertionIndex++];
	}
	
	if (keys)
	{
		if (menuItem = [(id <PGPBundle>) keys newMenuItem])
			[[[[NSApp mainMenu] itemWithTitle:NSLocalizedString(@"File", @"File Menu")] submenu] insertItem:menuItem atIndex:0];
	}
	
	if (disk)
	{
		if (menuItem = [(id <PGPBundle>) disk newMenuItem])
			[[[[NSApp mainMenu] itemWithTitle:NSLocalizedString(@"File", @"File Menu")] submenu] insertItem:menuItem atIndex:0];
	}
	
	[[mController prefsManager] initPreferencesPanel];
	
	mGUIInitialized = YES;
	
	return TRUE;
}

- (IBAction)showPreferences:(id)sender
{
	[[mController prefsManager] showPreferences];
}

- (id <PGPBundle>)currentBundle
{
    return mCurrentBundle;
}

- (void)setCurrentBundle:(id <PGPBundle>)controller passive:(BOOL)passive
{
	if (!controller)
		return;
	
	[controller activate:passive];
	
	mCurrentBundle = controller;
}

- (NSMenu*)mainMenu
{
	return mMainMenu;
}

- (NSArray*)menuItems
{
    if (mMainMenu)
        return [mMainMenu itemArray];
        
    return nil;
}

- (void)addMenu:(NSMenu*)menu withIdentifier:(NSString*)identifier;
{
    if (!menu || !identifier)
        PGPLog((@"%s:%d: Invalid parameters specified", __FILE__, __LINE__));
	else
		[self addMenu:menu withIdentifier:identifier atIndex:[mMainMenu numberOfItems]];
}

- (void)addMenu:(NSMenu*)menu withIdentifier:(NSString*)identifier atIndex:(unsigned)index;
{
    NSMenuItem*				menuItem;
	NSMenu*					menuCopy;
    
    if (!menu || !identifier)
        PGPLog((@"%s:%d: Invalid parameters specified", __FILE__, __LINE__));
	else
	{
		if (index > [mMainMenu numberOfItems])
		{
			PGPLog((@"%s:%d: Index extends beyond current size + 1. Adjusting index.", __FILE__, __LINE__));
			index = [mMainMenu numberOfItems];
		}
	
		menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:[menu title] action:nil keyEquivalent:@""];
		menuCopy = [menu copyWithZone:[NSMenu menuZone]];
		[menuItem setSubmenu:menuCopy];
		[self addMenuItem:menuItem withIdentifier:identifier atIndex:index];
		[menuCopy release];
			
		[menuItem release];
	}
}

- (void)addMenuItem:(NSMenuItem*)menuItem withIdentifier:(NSString*)identifier;
{
    if (!menuItem || !identifier)
        PGPLog((@"%s:%d: Invalid parameters specified", __FILE__, __LINE__));
	else
		[self addMenuItem:menuItem withIdentifier:identifier atIndex:[mMainMenu numberOfItems]];
}
	
- (void)addMenuItem:(NSMenuItem*)menuItem withIdentifier:(NSString*)identifier atIndex:(unsigned)index;
{
    if (!menuItem || !identifier)
        PGPLog((@"%s:%d: Invalid parameters specified", __FILE__, __LINE__));
	else
	{
		if (index > [mMainMenu numberOfItems])
		{
			PGPLog((@"%s:%d: Index extends beyond current size + 1. Adjusting index.", __FILE__, __LINE__));
			index = [mMainMenu numberOfItems];
		}
		
		[self registerMenuItem:menuItem withIdentifier:identifier];
		
		[mMainMenu insertItem:menuItem atIndex:index];
	}
}

- (void)removeMenusForIdentifier:(NSString*)identifier;
{
    NSMutableArray*			menusArray;
    NSEnumerator*			arrayEnumerator;
    NSMutableDictionary*	menusDictionary;
	NSMutableDictionary*	targetMenusDictionary = 0;
    NSArray*				menuItemArray = 0;
    NSMenuItem*				menuItem;
    
	if (!identifier)
		PGPLog((@"%s:%d: Invalid parameter specified", __FILE__, __LINE__));
	else
	{
		if (menusArray = [[mController attributes] objectForKey:kPGPAppMenusKey])
		{
			arrayEnumerator = [menusArray objectEnumerator];
        
			while ((menusDictionary = [arrayEnumerator nextObject]) && !menuItemArray)
			{
				if ([identifier isEqualToString:[menusDictionary objectForKey:kPGPAppMenuIdentifierKey]])
				{
					menuItemArray = [menusDictionary objectForKey:kPGPAppMenuBundleMenusKey];
					targetMenusDictionary = menusDictionary;
				}
			}
        
			if (menuItemArray)
			{
				arrayEnumerator = [menuItemArray objectEnumerator];
            
				while (menuItem = [arrayEnumerator nextObject])
					[mMainMenu removeItem:menuItem];
            
				[menusDictionary removeObjectForKey:kPGPAppMenuBundleMenusKey];
			}
			
			if (targetMenusDictionary)
				[menusArray removeObject:targetMenusDictionary];
		}
	}
}

- (void)registerMenuItem:(NSMenuItem*)menuItem withIdentifier:(NSString*)identifier
{
	NSMutableArray*			menusArray;
	NSMutableArray*			menuItemsArray;
	NSMutableDictionary*	currentMenuDictionary;
	NSMutableDictionary*	targetMenuDictionary = 0;
	NSEnumerator*			arrayEnumerator;
	NSString*				tempString;
	
	BOOL					allocatedMenusArray = NO;
	BOOL					allocatedMenuDictionary = NO;
	BOOL					allocatedMenuItemsArray = NO;

	if (!menuItem || !identifier)
		PGPLog((@"%s:%d: Invalid parameters specified", __FILE__, __LINE__));
	else
	{
		if (!(menusArray = [[mController attributes] objectForKey:kPGPAppMenusKey]))
		{
			menusArray = [[NSMutableArray alloc] initWithCapacity:5];
			[[mController attributes] setObject:menusArray forKey:kPGPAppMenusKey];
			allocatedMenusArray = YES;
		}
		
		arrayEnumerator = [menusArray objectEnumerator];
    
		while ((currentMenuDictionary = [arrayEnumerator nextObject]) && !targetMenuDictionary)
		{
			if ([identifier isEqualToString:[currentMenuDictionary objectForKey:kPGPAppMenuIdentifierKey]])
				targetMenuDictionary = currentMenuDictionary;
		}
		
		if (!targetMenuDictionary)
		{
			targetMenuDictionary = [[NSMutableDictionary alloc] initWithCapacity:2];
			allocatedMenuDictionary = YES;
			
			menuItemsArray = [[NSMutableArray alloc] initWithCapacity:5];
			allocatedMenuItemsArray = YES;        
        
			tempString = [[NSString alloc] initWithString:identifier];
			[targetMenuDictionary setObject:tempString forKey:kPGPAppMenuIdentifierKey];
			[tempString release];
			
			[targetMenuDictionary setObject:menuItemsArray forKey:kPGPAppMenuBundleMenusKey];
        
			[menusArray addObject:targetMenuDictionary];
		}
		else
		{
			if (!(menuItemsArray = [targetMenuDictionary objectForKey:kPGPAppMenuBundleMenusKey]))
			{
				menuItemsArray = [[NSMutableArray alloc] initWithCapacity:5];
				allocatedMenuItemsArray = YES;
				
				[targetMenuDictionary setObject:menuItemsArray forKey:kPGPAppMenuBundleMenusKey];
			}
		}
    
		[menuItemsArray addObject:menuItem];		
		
		if (allocatedMenusArray)
			[menusArray release];
			
		if (allocatedMenuDictionary)
			[targetMenuDictionary release];
			
		if (allocatedMenuItemsArray)
			[menuItemsArray release];
	}
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
	if ([[NSApplication sharedApplication] modalWindow] || !menuItem || !mGUIInitialized)
		return NO;
	if ([[menuItem title] isEqualToString:kPGPInterface_AboutMenuItem])
		return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_LicenseMenuItem])
	{
	#if PGP_LICENSENUMBERS
		return YES;
	#else
		return NO;
	#endif
	}
	if ([[menuItem title] isEqualToString:kPGPInterface_PreferencesMenuItem])
		return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_UninstallMenuItem])
		return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_QuitMenuItem])
		return YES;
    if ([[menuItem title] isEqualToString:kPGPInterface_BringAllToFront])
        return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_Disk])
		return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_Keys])
		return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_Net])
		return YES;
	if ([[menuItem title] isEqualToString:kPGPInterface_Mail])
		return YES;
	
	return NO;
}

- (IBAction)about:(id)sender
{
	NSCalendarDate*				date;
	PGPInt32					days = 0x7FFFFFFF;	/* Never */
	PGPUInt32					numLicenses = 1;
	PGPError					err;
	PGPLicenseNumberRef			lnRef = kInvalidPGPLicenseNumberRef;
	unsigned char				name[256];
	unsigned char				org[256];
	
	if ([mAboutWindow isVisible])
	{
		[mAboutWindow makeKeyAndOrderFront:nil];
		return;
	}
	
	days = PGPclLicenseDaysLeft();
	
	if (IsPGPError(err = [CPGPLicenseAuth prefLNRef:&lnRef]))
		[mAboutBox setContentView:mAboutFreeware];
	else if (PGPclIsAuthorized() || (days <= 0))
	{
		numLicenses = PGPclLicenseQuantity();
		
		[mSeats setStringValue:[NSString stringWithFormat:@"%d", numLicenses]];
		
		err = PGPclLicenseName(name, sizeof(name));
		if (IsntPGPError(err) && (numLicenses == 1) && name[0])
			[mName setStringValue:[NSString stringWithUTF8String:name]];
		else if (IsntPGPError(err) && (numLicenses == 1))
		{
		#if PGP_BETA
			[mName setStringValue:NSLocalizedString(@"PGP Beta", @"")];
		#else
			[mName setStringValue:@""];
		#endif
		}
		
		err = PGPclLicenseCorp(org, sizeof(org));
		if (IsntPGPError(err) && (numLicenses == 1) && org[0])
			[mOrganization setStringValue:[NSString stringWithUTF8String:org]];
		else if (IsntPGPError(err) && (numLicenses > 1))
			[mName setStringValue:[NSString stringWithUTF8String:org]];	/* Set licensee name to the organization name */
		else
			[mOrganization setStringValue:@""];
		
		days = PGPclLicenseDaysLeft();
		
		if (days <= 0)
		{
		#if PGP_BETA
			date = [NSCalendarDate dateWithNaturalLanguageString:[NSString stringWithCString:__DATE__]];
			date = [date dateByAddingYears:0 months:0 days:PGP_BETA_DAYS hours:0 minutes:0 seconds:0];
			
			[mExpDate setStringValue:[date descriptionWithCalendarFormat:@"%b. %d, %Y"]];
		#else
			[mExpDate setStringValue:kPGPLicenseDisplayExpired];
		#endif
		}
		else if (days == 0x7FFFFFFF)
			[mExpDate setStringValue:kPGPLicenseDisplayNever];
		else if (days <= 60)
		{
			if (PGPclEvaluation())
			{
				[mExpDate setStringValue:[NSString stringWithFormat:kPGPLicenseDisplayEvalDays,
														days,
														(days == 1 ? NSLocalizedString(@" ", @"") : NSLocalizedString(@"s ", @""))]];
			}
			else
			{
				[mExpDate setStringValue:[NSString stringWithFormat:kPGPLicenseDisplayDaysRemaining,
														days,
														(days == 1 ? NSLocalizedString(@" ", @"") : NSLocalizedString(@"s ", @""))]];
			}
		}
		else
		{
			date = [[NSCalendarDate calendarDate] dateByAddingYears:0 months:0 days:days hours:0 minutes:0 seconds:0];
			
			[mExpDate setStringValue:[date descriptionWithCalendarFormat:@"%b. %d, %Y"]];
		}
		
		[mAboutBox setContentView:mAboutLicensed];
	}
	
	if (PGPLicenseNumberRefIsValid(lnRef))
	{
		PGPLNFree(lnRef);
		lnRef = kInvalidPGPLicenseNumberRef; 
	}
	
	mScrollYPos = NSHeight([mCreditsText frame]) + 12;
	[mClipView scrollToPoint:NSMakePoint(0, mScrollYPos)];
	
	[self createScrollTimer];
	
	[mAboutWindow center];
	[mAboutWindow makeKeyAndOrderFront:sender];
}

- (IBAction)upgrade:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:kPGPLicenseDisplayPurchaseURL]];
}

- (IBAction)license:(id)sender
{
	[mController displayLicense];
}

- (IBAction)confirmUninstall:(id)sender
{
	int		returnCode;
	
	returnCode = NSRunAlertPanel(kPGPInterface_UninstallConfirmation,
						kPGPInterface_UninstallConfirmationMsg,
						NSLocalizedString(@"Cancel", @""),
						NSLocalizedString(@"Uninstall", @""),
						nil);
		
	[self uninstallWarningDidEnd:nil returnCode:returnCode contextInfo:nil];
}

- (IBAction)arrangeInFront:(id)sender
{
	NSArray*		windows = [NSApp windows];
	NSEnumerator*	enumerator = [windows objectEnumerator];
	NSWindow*		window;
	
	while (window = [enumerator nextObject])
	{
		if ([window isMiniaturized])
			[window deminiaturize:sender];
	}
}

- (IBAction)quit:(id)sender
{
	[mController cleanupAndExit];
}

- (IBAction)disk:(id)sender
{
	id <PGPBundle>		diskController = [[mController bundleManager] bundleByName:@"PGPdisk.bundle"];
	
	[self setCurrentBundle:diskController passive:NO];
}

- (IBAction)keys:(id)sender
{
	id <PGPBundle>		keysController = [[mController bundleManager] bundleByName:@"PGPkeys.bundle"];
	
	[self setCurrentBundle:keysController passive:NO];
}

- (IBAction)net:(id)sender
{
	id <PGPBundle>		netController = [[mController bundleManager] bundleByName:@"PGPnet.bundle"];
	
	[self setCurrentBundle:netController passive:NO];
}

- (IBAction)mail:(id)sender
{
	id <PGPBundle>		mailController = [[mController bundleManager] bundleByName:@"PGPmail.bundle"];
	
	[self setCurrentBundle:mailController passive:NO];
}

- (void)uninstallWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	if (returnCode == NSCancelButton)	/* The 'Uninstall' button */
		[mController uninstall:NO];
}

- (void)createScrollTimer
{
	mScrollTimer = [NSTimer scheduledTimerWithTimeInterval:0.05
							target:self
							selector:@selector(scrollCredits:)
							userInfo:nil
							repeats:YES];
	
	[mScrollTimer retain];
}

- (void)scrollCredits:(NSTimer*)timer
{
	KeyMap			keyboardState;
	
    GetKeys(keyboardState);
	
	if (keyboardState[1] & 0x4)	/* Options key */
	{
		mScrollYPos += 1.0;
		
		if (mScrollYPos == mStartingPoint)
			mScrollYPos = -30;
	}
	else
	{
		mScrollYPos -= 1.0;
		
		if (mScrollYPos == -30.0)
			mScrollYPos = mStartingPoint;
	}
	
	[mClipView scrollToPoint:NSMakePoint(0, mScrollYPos)];
	[mClipView setNeedsDisplay:YES];
}

- (void)windowWillClose:(NSNotification*)notification
{
	if (mScrollTimer)
	{
		[mScrollTimer invalidate];
		[mScrollTimer release];
		
		mScrollTimer = nil;
	}
}

- (void)dealloc
{
	[super dealloc];
}

@end
