/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPPrefsAdvanced.m,v 1.9 2002/11/26 01:23:19 jeffc Exp $
____________________________________________________________________________*/

#import "PGPPrefsAdvanced.h"

@implementation PGPPrefsAdvanced

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mIdentifiers = [[NSArray alloc] initWithObjects:kPGPPrefsAdvancedIdentifier, nil];

		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self initWithNib:kPGPPrefsAdvancedNib];
		
	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller;
{
	mController = controller;
	
	return [self init];
}

- (NSArray*)identifiers
{
	return mIdentifiers;
}

- (id)prefsViewForIdentifier:(NSString*)identifier
{
	if (identifier && [identifier isEqualToString:kPGPPrefsAdvancedIdentifier])
		return mPrefsView;
	
	return nil;
}

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier
{
	return mPreferredAlgorithm;
}

- (id)lastPrefsResponderForIdentifier:(NSString*)identifier
{
	return [mComplete controlView];
}

- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier
{
	[mPreferredAlgorithm setNextKeyView:mAES];
	[mAES setNextKeyView:mCAST];
	[mCAST setNextKeyView:mTripleDES];
	[mTripleDES setNextKeyView:mIDEA];
	[mIDEA setNextKeyView:mTwofish];
	[mTwofish setNextKeyView:mAllowMarginallyValidKeys];
	[mAllowMarginallyValidKeys setNextKeyView:mADKWarning];
	[mADKWarning setNextKeyView:[mCompatible controlView]];
}

- (void)restorePanelPrefs:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
    PGPPrefArray*		array;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	BOOL				enabled;
    
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPPrefsAdvancedIdentifier])
	{
		/* kPGPPrefPreferredAlgorithm */
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefPreferredAlgorithm, &nData)))
			NSLog(@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefPreferredAlgorithm, err);
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefPreferredAlgorithm, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefPreferredAlgorithm, err);
				enabled = NO;
			}
	
			[self setPreferredAlgorithm:nData enabled:enabled];
		}
		
		/* kPGPPrefAllowedAlgorithmsList */
		if (IsPGPError(err = PGPGetPrefArray(prefRef, kPGPPrefAllowedAlgorithmsList, &array)))
			NSLog(@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefAllowedAlgorithmsList, err);
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefAllowedAlgorithmsList, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefAllowedAlgorithmsList, err);
				enabled = NO;
			}
			
			[self setAllowedAlgorithms:array enabled:enabled];
			PGPFreePrefArray(array);
		}
				
		/* kPGPPrefMarginalIsInvalid */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefMarginalIsInvalid, &bData)))
			NSLog(@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefMarginalIsInvalid, err);
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMarginalIsInvalid, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMarginalIsInvalid, err);
				enabled = NO;
			}
			
			[self setInvalidateMarginallyValidKeys:bData enabled:enabled];
		}
		
		/* kPGPPrefWarnOnADK */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefWarnOnADK, &bData)))
			NSLog(@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefWarnOnADK, err);
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWarnOnADK, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWarnOnADK, err);
				enabled = NO;
			}
			
			[self setWarnOnADK:bData enabled:enabled];
		}
		
		/* kPGPPrefExportKeysCompatible */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefExportKeysCompatible, &bData)))
			NSLog(@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefExportKeysCompatible, err);
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefExportKeysCompatible, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefExportKeysCompatible, err);
				enabled = NO;
			}
			
			[self setExportKeysCompatible:bData enabled:enabled];
		}
	}
}

- (void)restorePanelDefaults:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
    PGPPrefArray*		array;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	PGPPrefDefinition	prefDef;
	BOOL				enabled;
    
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPPrefsAdvancedIdentifier])
	{
		/* kPGPPrefPreferredAlgorithm */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefPreferredAlgorithm, &prefDef)))
			NSLog(@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefPreferredAlgorithm, err);
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefPreferredAlgorithm, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefPreferredAlgorithm, err);
				enabled = NO;
			}
			
			[self setPreferredAlgorithm:nData enabled:enabled];
		}
	
		/* kPGPPrefAllowedAlgorithmsList */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefAllowedAlgorithmsList, &prefDef)))
			NSLog(@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefAllowedAlgorithmsList, err);
		else
		{
			array = (PGPPrefArray*)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefAllowedAlgorithmsList, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefAllowedAlgorithmsList, err);
				enabled = NO;
			}
			
			[self setAllowedAlgorithms:array enabled:enabled];
		}
	
		/* kPGPPrefMarginalIsInvalid */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefMarginalIsInvalid, &prefDef)))
			NSLog(@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefMarginalIsInvalid, err);
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMarginalIsInvalid, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMarginalIsInvalid, err);
				enabled = NO;
			}
			
			[self setInvalidateMarginallyValidKeys:bData enabled:enabled];
		}
		
		/* kPGPPrefWarnOnADK */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefWarnOnADK, &prefDef)))
			NSLog(@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefWarnOnADK, err);
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWarnOnADK, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWarnOnADK, err);
				enabled = NO;
			}
			
			[self setWarnOnADK:bData enabled:enabled];
		}
		
		/* kPGPPrefExportKeysCompatible */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefExportKeysCompatible, &prefDef)))
			NSLog(@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefExportKeysCompatible, err);
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefExportKeysCompatible, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				NSLog(@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefExportKeysCompatible, err);
				enabled = NO;
			}
			
			[self setExportKeysCompatible:bData enabled:enabled];
		}
	}
}

- (void)savePanelPrefs:(NSString*)panelIdentifier
{
	PGPError				err;
	PGPPrefRef				prefRef = [mController pgpPrefRef];
    PGPPrefArray*			array;
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];

	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPPrefsAdvancedIdentifier])
	{
		/* kPGPPrefPreferredAlgorithm */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefPreferredAlgorithm, [self getPreferredAlgorithm])))
			NSLog(@"Failed to set value for prefid %d (err=%d)", kPGPPrefPreferredAlgorithm, err);
		else
			[center postNotificationName:kPGPclPrefNotification_PreferredAlg object:nil];
		
		/* kPGPPrefAllowedAlgorithmsList */
		array = [self getAllowedAlgorithms];
		if (IsPGPError(err = PGPSetPrefArray(prefRef, kPGPPrefAllowedAlgorithmsList, array)))
			NSLog(@"Failed to set value for prefid %d (err=%d)", kPGPPrefAllowedAlgorithmsList, err);
		else
		{
			PGPFreePrefArray(array);
			[center postNotificationName:kPGPclPrefNotification_AllowedAlgs object:nil];
		}
					
		/* kPGPPrefMarginalIsInvalid */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefMarginalIsInvalid, [self getInvalidateMarginallyValidKeys])))
			NSLog(@"Failed to set value for prefid %d (err=%d)", kPGPPrefMarginalIsInvalid, err);
		else
			[center postNotificationName:kPGPclPrefNotification_InvalidateMVKeys object:nil];
			
		/* kPGPPrefWarnOnADK */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefWarnOnADK, [self getWarnOnADK])))
			NSLog(@"Failed to set value for prefid %d (err=%d)", kPGPPrefWarnOnADK, err);
		else
			[center postNotificationName:kPGPclPrefNotification_WarnOnADK object:nil];
		
		/* kPGPPrefExportKeysCompatible */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefExportKeysCompatible, [self getExportKeysCompatible])))
			NSLog(@"Failed to set value for prefid %d (err=%d)", kPGPPrefExportKeysCompatible, err);
		else
			[center postNotificationName:kPGPclPrefNotification_ExportCompatible object:nil];
	}
}

- (void)setPreferredAlgorithm:(PGPUInt32)value enabled:(BOOL)enabled
{
    [mPreferredAlgorithm selectItemAtIndex:[mPreferredAlgorithm indexOfItemWithTag:value]];
    [mPreferredAlgorithm setEnabled:enabled];
}

- (void)setAllowedAlgorithms:(PGPPrefArray*)value enabled:(BOOL)enabled
{
    PGPUInt32		index;
    
    pgpAssert(value != nil);
    
    [mIDEA setState:NSOffState];
    [mTripleDES setState:NSOffState];
    [mCAST setState:NSOffState];
    [mTwofish setState:NSOffState];
    [mAES setState:NSOffState];
    
    for (index = 0; index < value->numElements; ++index)
    {
        switch((PGPCipherAlgorithm) value->elements[index].data)
        {
            case kPGPCipherAlgorithm_IDEA:
                [mIDEA setState:NSOnState];
                break;
            case kPGPCipherAlgorithm_3DES:
            	[mTripleDES setState:NSOnState];
                break;
            case kPGPCipherAlgorithm_CAST5:
                [mCAST setState:NSOnState];
                break;
            case kPGPCipherAlgorithm_Twofish256:
                [mTwofish setState:NSOnState];
                break;
            case kPGPCipherAlgorithm_AES256:
                [mAES setState:NSOnState];
                break;
            default:
                break;
        }
    }

    [mIDEA setEnabled:enabled];
    [mTripleDES setEnabled:enabled];
    [mCAST setEnabled:enabled];
    [mTwofish setEnabled:enabled];
    [mAES setEnabled:enabled];
}

- (void)setInvalidateMarginallyValidKeys:(PGPBoolean)value enabled:(BOOL)enabled
{
    [self setCheckbox:mAllowMarginallyValidKeys state:value enabled:enabled];
}

- (void)setWarnOnADK:(PGPBoolean)value enabled:(BOOL)enabled
{
    [self setCheckbox:mADKWarning state:value enabled:enabled];
}

- (void)setExportKeysCompatible:(PGPBoolean)value enabled:(BOOL)enabled
{
	if (value)
		[mCompatible performClick:nil];
	else
		[mComplete performClick:nil];
	
	[mCompatible setEnabled:enabled];
	[mComplete setEnabled:enabled];
}

- (PGPUInt32)getPreferredAlgorithm
{
    return [[mPreferredAlgorithm selectedItem] tag];
}

- (PGPPrefArray*)getAllowedAlgorithms
{
    PGPError			err;
    PGPPrefRef			prefRef = [mController pgpPrefRef];
    PGPPrefArray*		array;
    PGPUInt32			prefCipher;
    PGPUInt32			index = 0;
    
    if (IsPGPError(err = PGPCreatePrefArray(prefRef, kPGPPrefType_Number, 1, NULL, &array)))
    {
		PGPLog((@"Failed to create pref array (err=%d)", err));
        return NULL;
    }
    
    prefCipher = [self getPreferredAlgorithm];
    
    PGPSetPrefArrayNumber(prefRef, array, index++, prefCipher);

    if (prefCipher != kPGPCipherAlgorithm_CAST5)
    {
        if ([mCAST state] == NSOnState)
            PGPSetPrefArrayNumber(prefRef, array, index++, kPGPCipherAlgorithm_CAST5);
    }
    if (prefCipher != kPGPCipherAlgorithm_AES256)
    {
        if ([mAES state] == NSOnState)
            PGPSetPrefArrayNumber(prefRef, array, index++, kPGPCipherAlgorithm_AES256);
    }
    if (prefCipher != kPGPCipherAlgorithm_3DES)
    {
        if ([mTripleDES state] == NSOnState)
            PGPSetPrefArrayNumber(prefRef, array, index++, kPGPCipherAlgorithm_3DES);
    }
    if (prefCipher != kPGPCipherAlgorithm_IDEA)
    {
        if ([mIDEA state] == NSOnState)
            PGPSetPrefArrayNumber(prefRef, array, index++, kPGPCipherAlgorithm_IDEA);
    }
    if (prefCipher != kPGPCipherAlgorithm_Twofish256)
    {
        if ([mTwofish state] == NSOnState)
            PGPSetPrefArrayNumber(prefRef, array, index++, kPGPCipherAlgorithm_Twofish256);
    }
    
    return array;
}

- (PGPBoolean)getInvalidateMarginallyValidKeys
{
    return [self getCheckbox:mAllowMarginallyValidKeys];
}

- (PGPBoolean)getWarnOnADK
{
    return [self getCheckbox:mADKWarning];
}

- (PGPBoolean)getExportKeysCompatible
{
	return [mCompatible state] == NSOnState ? TRUE : FALSE;
}

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled
{
	if (checkBox)
	{
		if (state)
			[checkBox setState:NSOnState];
		else
			[checkBox setState:NSOffState];
		
		if (enabled)
			[checkBox setEnabled:YES];
		else
			[checkBox setEnabled:NO];
	}
}

- (PGPBoolean)getCheckbox:(NSButton*)checkBox
{
	if (!checkBox)
		return FALSE;
	
	return [checkBox state] == NSOnState ? TRUE : FALSE;
}

- (void)dealloc
{
	if (mIdentifiers)
		[mIdentifiers release];
		
	[super dealloc];
}

@end
