/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPPrefsGeneral.m,v 1.18 2002/11/07 23:01:58 jeffc Exp $
____________________________________________________________________________*/

#import "PGPPrefsGeneral.h"

@implementation PGPPrefsGeneral

- (id)init
{
	NSNotificationCenter*		center = [NSNotificationCenter defaultCenter];
	
	if (!mInited)
	{
		[super init];
		
		mIdentifiers = [[NSArray alloc] initWithObjects:kPGPPrefsGeneralIdentifier, nil];

		[center addObserver:self
				selector:@selector(prefsViewBecameActive:)
				name:kPGPclPrefNotification_PrefsViewActive
				object:mPrefsView];

		[center addObserver:self
				selector:@selector(prefsViewBecameInactive:)
				name:kPGPclPrefNotification_PrefsViewInactive
				object:mPrefsView];

		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self initWithNib:kPGPPrefsGeneralNib];

	if (!mHoursFieldEditor)
	{
		mHoursFieldEditor = [[CDurationFieldEditor alloc] init];
		[mHoursFieldEditor setMaxValue:23];
		[mHoursFieldEditor setFieldEditor:YES];
	}

	if (!mMinutesFieldEditor)
	{
		mMinutesFieldEditor = [[CDurationFieldEditor alloc] init];
		[mMinutesFieldEditor setMaxValue:59];
		[mMinutesFieldEditor setFieldEditor:YES];
	}

	if (!mSecondsFieldEditor)
	{
		mSecondsFieldEditor = [[CDurationFieldEditor alloc] init];
		[mSecondsFieldEditor setMaxValue:59];
		[mSecondsFieldEditor setFieldEditor:YES];
	}

	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	
	if (!mIntFormatter)
	{
		mIntFormatter = [[CIntegerFormatter alloc] init];
		[mIntFormatter setFormat:@"00"];
		[mIntFormatter setMinimum:[NSDecimalNumber zero]];
	}
	
	[mCacheHours setFormatter:mIntFormatter];
	[mCacheMinutes setFormatter:mIntFormatter];
	[mCacheSeconds setFormatter:mIntFormatter];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller;
{
	mController = controller;
	
	return [self init];
}

- (NSArray*)identifiers
{
	return mIdentifiers;
}

- (id)prefsViewForIdentifier:(NSString*)identifier
{
	if (identifier && [identifier isEqualToString:kPGPPrefsGeneralIdentifier])
		return mPrefsView;
	
	return nil;
}

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier
{
	return mEncryptToDefaultKey;
}

- (id)lastPrefsResponderForIdentifier:(NSString*)identifier
{
	return mPassesStepper;
}

- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier
{
	[mEncryptToDefaultKey setNextKeyView:mDefaultToMacBinary];
	[mDefaultToMacBinary setNextKeyView:mFasterKeyGen];
	[mFasterKeyGen setNextKeyView:mCommentBlock];
	
	if ([mCacheWhileLoggedOn state] == NSOnState)
	{
		[mCommentBlock setNextKeyView:[mCacheWhileLoggedOn controlView]];
		[[mCacheWhileLoggedOn controlView] setNextKeyView:mShareCache];
	}
	else if ([mCacheForDuration state] == NSOnState)
	{
		[mCommentBlock setNextKeyView:[mCacheForDuration controlView]];
		[self cacheForDuration:nil];
	}
	else
	{
		[mCommentBlock setNextKeyView:[mDoNotCache controlView]];
		[[mDoNotCache controlView] setNextKeyView:mShareCache];
	}
	
	[mShareCache setNextKeyView:mWarnOnWipe];
	[mWarnOnWipe setNextKeyView:mNumPasses];
	[mNumPasses setNextKeyView:mPassesStepper];
}

- (void)restorePanelPrefs:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
	char*				szData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	BOOL				enabled;
	BOOL				allocated;
	
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPPrefsGeneralIdentifier])
	{
		/* kPGPPrefEncryptToSelf */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefEncryptToSelf, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefEncryptToSelf, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefEncryptToSelf, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefEncryptToSelf, err));
				enabled = NO;
			}
			
			[self setEncryptToDefaultKey:bData enabled:enabled];
		}
			
		/* kPGPPrefMacBinaryDefault */
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefMacBinaryDefault, &nData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefMacBinaryDefault, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMacBinaryDefault, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMacBinaryDefault, err));
				enabled = NO;
			}
	
			[self setDefaultToMacBinary:nData enabled:enabled];
		}
	
		/* kPGPPrefFastKeyGen */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefFastKeyGen, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefFastKeyGen, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefFastKeyGen, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefFastKeyGen, err));
				enabled = NO;
			}
			
			[self setFasterKeyGen:bData enabled:enabled];
		}
	
		/* kPGPPrefComment */
		allocated = NO;
		szData = NULL;
		
		if (IsPGPError(err = PGPGetPrefStringAlloc(prefRef, kPGPPrefComment, &szData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefComment, err));
		else
			allocated = YES;
		
		[mCommentBlock setStringValue:@""];
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefComment, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefComment, err));
				enabled = NO;
			}
			
			[self setCommentBlock:szData enabled:enabled];
		}
		
		if (allocated)
			PGPFreeData(szData);
	
		/* kPGPPrefCacheType */
		nData = 0;
		
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefCacheType, &nData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefCacheType, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCacheType, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCacheType, err));
				enabled = NO;
			}
			
			[self setCacheType:nData enabled:enabled];
		}
	
		/* kPGPPrefCacheSeconds */
		nData = 0;
		
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefCacheSeconds, &nData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefCacheSeconds, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCacheSeconds, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCacheSeconds, err));
				enabled = NO;
			}
			
			[self setCacheSeconds:nData enabled:enabled];
		}
	
		/* kPGPPrefShareCache */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefShareCache, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefShareCache, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefShareCache, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefShareCache, err));
				enabled = NO;
			}
			
			[self setSharedCache:bData enabled:enabled];
		}
	
		/* kPGPPrefWarnOnWipe */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefWarnOnWipe, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefWarnOnWipe, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWarnOnWipe, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWarnOnWipe, err));
				enabled = NO;
			}
			
			[self setWarnOnWipe:bData enabled:enabled];
		}
	
		/* kPGPPrefFileWipePasses */
		nData = 0;
		
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefFileWipePasses, &nData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefFileWipePasses, err));
		
		[mNumPasses setIntValue:0];
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefFileWipePasses, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefFileWipePasses, err));
				enabled = NO;
			}
			
			[self setFileWipePasses:nData enabled:enabled];
		}
	}
}

- (void)restorePanelDefaults:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
	char*				szData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	PGPPrefDefinition	prefDef;
	BOOL				enabled;
	
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPPrefsGeneralIdentifier])
	{
		/* kPGPPrefEncryptToSelf */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefEncryptToSelf, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefEncryptToSelf, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefEncryptToSelf, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefEncryptToSelf, err));
				enabled = NO;
			}
			
			[self setEncryptToDefaultKey:bData enabled:enabled];
		}
		
		/* kPGPPrefMacBinaryDefault */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefMacBinaryDefault, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefMacBinaryDefault, err));
		else
		{
			nData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMacBinaryDefault, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMacBinaryDefault, err));
				enabled = NO;
			}
			
			[self setDefaultToMacBinary:nData enabled:enabled];
		}
	
		/* kPGPPrefFastKeyGen */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefFastKeyGen, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefFastKeyGen, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefFastKeyGen, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefFastKeyGen, err));
				enabled = NO;
			}
			
			[self setFasterKeyGen:bData enabled:enabled];
		}
		
		/* kPGPPrefComment */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefComment, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefComment, err));
		else
		{
			szData = (char*)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefComment, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefComment, err));
				enabled = NO;
			}
			
			[self setCommentBlock:szData enabled:enabled];
		}
	
		/* kPGPPrefCacheType */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefCacheType, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCacheType, err));
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCacheType, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCacheType, err));
				enabled = NO;
			}
			
			[self setCacheType:nData enabled:enabled];
		}
	
		/* kPGPPrefCacheSeconds */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefCacheSeconds, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCacheSeconds, err));
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCacheSeconds, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCacheSeconds, err));
				enabled = NO;
			}
			
			[self setCacheSeconds:nData enabled:enabled];
		}
	
		/* kPGPPrefShareCache */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefShareCache, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefShareCache, err));
		else
		{
			bData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefShareCache, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefShareCache, err));
				enabled = NO;
			}
			
			[self setSharedCache:bData enabled:enabled];
		}
		
		/* kPGPPrefWarnOnWipe */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefWarnOnWipe, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefWarnOnWipe, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWarnOnWipe, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWarnOnWipe, err));
				enabled = NO;
			}
			
			[self setWarnOnWipe:bData enabled:enabled];
		}
	
		/* kPGPPrefFileWipePasses */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefFileWipePasses, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefFileWipePasses, err));
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefFileWipePasses, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefFileWipePasses, err));
				enabled = NO;
			}
			
			[self setFileWipePasses:nData enabled:enabled];
		}
	}
}

- (void)savePanelPrefs:(NSString*)panelIdentifier
{
	PGPError				err;
	PGPPrefRef				prefRef = [mController pgpPrefRef];
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];
	
	if (panelIdentifier && [panelIdentifier isEqualToString:panelIdentifier])
	{
		/* kPGPPrefEncryptToSelf */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefEncryptToSelf, [self getEncryptToDefaultKey])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefEncryptToSelf, err));
		else
			[center postNotificationName:kPGPclPrefNotification_EncryptToSelf object:nil];
		
		/* kPGPPrefMacBinaryDefault */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefMacBinaryDefault, [self getDefaultToMacBinary])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefMacBinaryDefault, err));
		else
			[center postNotificationName:kPGPclPrefNotification_DefaultMacBinary object:nil];
	
		/* kPGPPrefFastKeyGen */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefFastKeyGen, [self getFasterKeyGen])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefFastKeyGen, err));
		else
			[center postNotificationName:kPGPclPrefNotification_FastKeyGen object:nil];
		
		/* kPGPPrefComment */
		if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefComment, [self getCommentBlock])))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefComment, err));
		else
			[center postNotificationName:kPGPclPrefNotification_Comment object:nil];
	
		/* kPGPPrefCacheType */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefCacheType, [self getCacheType])))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCacheType, err));
		else
			[center postNotificationName:kPGPclPrefNotification_CacheType object:nil];
	
		/* kPGPPrefCacheSeconds */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefCacheSeconds, [self getCacheSeconds])))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCacheSeconds, err));
		else
			[center postNotificationName:kPGPclPrefNotification_CacheSeconds object:nil];
			
		/* kPGPPrefShareCache */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefShareCache, [self getSharedCache])))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefShareCache, err));
		else
			[center postNotificationName:kPGPclPrefNotification_ShareCache object:nil];
		
		/* kPGPPrefWarnOnWipe */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefWarnOnWipe, [self getWarnOnWipe])))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefWarnOnWipe, err));
		else
			[center postNotificationName:kPGPclPrefNotification_WarnOnWipe object:nil];
	
		/* kPGPPrefFileWipePasses */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefFileWipePasses, [self getFileWipePasses])))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefFileWipePasses, err));
		else
			[center postNotificationName:kPGPclPrefNotification_FileWipePasses object:nil];
	}
}

- (void)setEncryptToDefaultKey:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mEncryptToDefaultKey state:value enabled:enabled];
}

- (void)setDefaultToMacBinary:(PGPUInt32)value enabled:(BOOL)enabled
{
	BOOL		state;
	
	if (value == kPGPPrefMacBinaryOff)
		state = NO;
	else
		state = YES;
		
	[self setCheckbox:mDefaultToMacBinary state:state enabled:enabled];
}

- (void)setFasterKeyGen:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mFasterKeyGen state:value enabled:enabled];
}

- (void)setCommentBlock:(char*)value enabled:(BOOL)enabled
{
	[mCommentBlock setStringValue:@""];
	[mCommentBlock setStringValue:[NSString stringWithCString:value]];
	
	if (enabled)
		[mCommentBlock setEnabled:YES];
	else
		[mCommentBlock setEnabled:NO];
}

- (void)setCacheType:(PGPUInt32)value enabled:(BOOL)enabled
{
	[mCacheWhileLoggedOn setEnabled:enabled];
	[mCacheForDuration setEnabled:enabled];
	[mDoNotCache setEnabled:enabled];

	switch (value)
	{
		case kPGPPrefCacheTypeNone:
			[mDoNotCache performClick:nil];
			break;
		case kPGPPrefCacheTypeSeconds:
			[mCacheForDuration performClick:nil];
			break;
		case kPGPPrefCacheTypeLogon:
			[mCacheWhileLoggedOn performClick:nil];
			break;
		default:
			break;
	}
	
	[mCacheMatrix sendAction];
}

- (void)setCacheSeconds:(PGPUInt32)value enabled:(BOOL)enabled
{
	PGPUInt32		temp;
	
	temp = value / 3600;
	[mCacheHours setIntValue:temp];
	
	value -= temp * 3600;
	
	temp = value / 60;
	[mCacheMinutes setIntValue:temp];
	
	value -= temp * 60;
	
	[mCacheSeconds setIntValue:value];
	
	[mCacheHours setEnabled:enabled];
	[mCacheMinutes setEnabled:enabled];
	[mCacheSeconds setEnabled:enabled];
	[mDurationStepper setEnabled:enabled];
}

- (void)setSharedCache:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mShareCache state:value enabled:enabled];
}

- (void)setWarnOnWipe:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mWarnOnWipe state:value enabled:enabled];
}

- (void)setFileWipePasses:(PGPUInt32)value enabled:(BOOL)enabled
{
	[mNumPasses setIntValue:value];
	
	if (enabled)
		[mNumPasses setEnabled:YES];
	else
		[mNumPasses setEnabled:NO];
}

- (PGPBoolean)getEncryptToDefaultKey
{
	return [self getCheckbox:mEncryptToDefaultKey];
}

- (PGPUInt32)getDefaultToMacBinary
{
	PGPBoolean		state = [self getCheckbox:mDefaultToMacBinary];
	
	if (state)
		return kPGPPrefMacBinaryOn;
	else
		return kPGPPrefMacBinaryOff;
}

- (PGPBoolean)getFasterKeyGen
{
	return [self getCheckbox:mFasterKeyGen];
}

- (char*)getCommentBlock
{
	return (char*) [[mCommentBlock stringValue] cString];
}

- (PGPUInt32)getCacheType
{
	if ([mCacheWhileLoggedOn state] == NSOnState)
		return kPGPPrefCacheTypeLogon;
	else if ([mCacheForDuration state] == NSOnState)
		return kPGPPrefCacheTypeSeconds;
	else
		return kPGPPrefCacheTypeNone;
}

- (PGPUInt32)getCacheSeconds
{
	return ([mCacheHours intValue] * 60 * 60) + ([mCacheMinutes intValue] * 60) + [mCacheSeconds intValue];
}

- (PGPBoolean)getSharedCache
{
	return [self getCheckbox:mShareCache];
}

- (PGPBoolean)getWarnOnWipe
{
	return [self getCheckbox:mWarnOnWipe];
}

- (PGPUInt32)getFileWipePasses
{
	return [mNumPasses intValue];
}

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled
{
	if (checkBox)
	{
		if (state)
			[checkBox setState:NSOnState];
		else
			[checkBox setState:NSOffState];
		
		if (enabled)
			[checkBox setEnabled:YES];
		else
			[checkBox setEnabled:NO];
	}
}

- (PGPBoolean)getCheckbox:(NSButton*)checkBox
{
	if (!checkBox)
		return FALSE;
	
	return [checkBox state] == NSOnState ? TRUE : FALSE;
}

- (IBAction)cacheForDuration:(id)sender
{
	[mCacheMatrix setNextKeyView:mCacheHours];
	[mCacheHours setNextKeyView:mCacheMinutes];
	[mCacheMinutes setNextKeyView:mCacheSeconds];
	[mCacheSeconds setNextKeyView:mDurationStepper];
	[mDurationStepper setNextKeyView:mShareCache];
}

- (IBAction)otherCacheType:(id)sender
{
	if ([mCacheWhileLoggedOn state] == NSOnState)
	{
		[mCommentBlock setNextKeyView:[mCacheWhileLoggedOn controlView]];
		[mCacheMatrix setNextKeyView:mShareCache];
	}
	else
	{
		[mCommentBlock setNextKeyView:[mDoNotCache controlView]];
		[mCacheMatrix setNextKeyView:mShareCache];
	}
}

- (IBAction)incrementDurationField:(id)sender
{
	id				firstResponder;
	NSTextField*	textField = nil;
    int				increment;
    int				currentValue;
    int				stepperValue = [mDurationStepper intValue];

	firstResponder = [[mPrefsView window] firstResponder];

	if ((firstResponder == mHoursFieldEditor) || (firstResponder == mMinutesFieldEditor))
	{
		if (firstResponder == mHoursFieldEditor)
			textField = mCacheHours;
		else if (firstResponder == mMinutesFieldEditor)
			textField = mCacheMinutes;
	}
	else
	{
		[[mPrefsView window] makeFirstResponder:mCacheSeconds];
		textField = mCacheSeconds;
	}
	
	currentValue = [textField intValue];
	
    if (stepperValue < 0)
        increment = -1;
    else
        increment = 1;

    [mDurationStepper setIntValue:0];

	if (textField == mCacheSeconds)
		[self incrementSeconds:increment];
	else if (textField == mCacheMinutes)
		[self incrementMinutes:increment];
	else if (textField == mCacheHours)
		[self incrementHours:increment];
}

- (IBAction)numPassesPressed:(id)sender
{
    int		increment;
    int		currentSize = [mNumPasses intValue];
    int		stepperValue = [mPassesStepper intValue];
    
    if (stepperValue < 0)
        increment = -1;
    else
        increment = 1;

    [mPassesStepper setIntValue:0];
    
    if ((currentSize + increment) < 1)
        [mNumPasses setIntValue:1];
    else if ((currentSize + increment) >= 32)
		[mNumPasses setIntValue:32];
	else
        [mNumPasses setIntValue:(currentSize + increment)];
}

- (void)incrementSeconds:(PGPInt32)increment
{
	PGPUInt32		currentValue;
	PGPInt32		sum;
	
	currentValue = [mCacheSeconds intValue];
	sum = currentValue + increment;
	
	if (sum < 0)
		[mCacheSeconds setIntValue:59];
	else if (sum >= 60)
		[mCacheSeconds setStringValue:@"00"];
	else
		[mCacheSeconds setIntValue:sum];
}

- (void)incrementMinutes:(PGPInt32)increment
{
	PGPUInt32		currentValue;
	PGPInt32		sum;
	
	currentValue = [mCacheMinutes intValue];
	sum = currentValue + increment;
	
	if (sum < 0)
		[mCacheMinutes setIntValue:59];
	else if (sum >= 60)
		[mCacheMinutes setStringValue:@"00"];
	else
		[mCacheMinutes setIntValue:sum];
}

- (void)incrementHours:(PGPInt32)increment
{
	PGPUInt32		currentValue;
	PGPInt32		sum;
	
	currentValue = [mCacheHours intValue];
	sum = currentValue + increment;

	if (sum < 0)
		[mCacheHours setIntValue:23];
	else if (sum >= 24)
		[mCacheHours setStringValue:@"00"];
	else
		[mCacheHours setIntValue:sum];
}

- (void)prefsViewBecameActive:(NSNotification*)notification
{
	if ([notification object] == mPrefsView)
	{
		mPreviousWindowDelegate = [[mPrefsView window] delegate];
		[[mPrefsView window] setDelegate:self];		
		[[mPrefsView window] makeFirstResponder:nil];
	}
}

- (void)prefsViewBecameInactive:(NSNotification*)notification
{
	if ([notification object] == mPrefsView)
	{
		[[mPrefsView window] makeFirstResponder:nil];
		[[mPrefsView window] setDelegate:mPreviousWindowDelegate];
		mPreviousWindowDelegate = nil;
	}
}

- (id)windowWillReturnFieldEditor:(NSWindow*)sender toObject:(id)object
{
	if (object == mCacheHours)
		return mHoursFieldEditor;
	else if (object == mCacheMinutes)
		return mMinutesFieldEditor;
	else if (object == mCacheSeconds)
		return mSecondsFieldEditor;
	
	return nil;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	if (mIdentifiers)
		[mIdentifiers release];

	if (mHoursFieldEditor)
		[mHoursFieldEditor release];
	
	if (mMinutesFieldEditor)
		[mMinutesFieldEditor release];
	
	if (mSecondsFieldEditor)
		[mSecondsFieldEditor release];

	if (mIntFormatter)
		[mIntFormatter release];
		
	[super dealloc];
}

@end
