/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPPrefsManager.m,v 1.42 2002/08/06 20:09:21 dallen Exp $
____________________________________________________________________________*/

#import "PGPPrefsManager.h"

enum
{
	kPGPPrefPanel_General		= 0,
	kPGPPrefPanel_Files,
	kPGPPrefPanel_Email,
	kPGPPrefPanel_Menu,
	kPGPPrefPanel_Server,
	kPGPPrefPanel_CertAuthority,
	kPGPPrefPanel_Advanced,
	kPGPPrefPanel_IDS,
	kPGPPrefPanel_VPN,
	kPGPPrefPanel_VPNAuth,
	kPGPPrefPanel_VPNAdvanced,
	kPGPPrefPanel_PGPdisk,
	
	kPGPPrefPanel_Invalid
};	

@implementation PGPPrefsManager

- (id)init
{
    if (!mInited)
    {
        [super init];
        mInited = YES;
    }
	if (!mPrefRef)
		[self openPrefs];
    if (!mPrefDictionary)
		mPrefDictionary = [[NSMutableDictionary alloc] init];
    if (!mPrefControllers)
		mPrefControllers = [[NSMutableDictionary alloc] init];
	if (!mControllers)
		mControllers = [[NSMutableArray alloc] init];
	if (!mPrefPanels)
		mPrefPanels = [[NSMutableArray alloc] init];
	if (!mGeneralPrefs)
		mGeneralPrefs = [[PGPPrefsGeneral alloc] initWithController:mAppController];
    if (!mAdvancedPrefs)
        mAdvancedPrefs = [[PGPPrefsAdvanced alloc] initWithController:mAppController];
	if (!mLoadedNib)
		[self initWithNib:kPGPPrefsManagerNib];
	
	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
	NSTableColumn*		column;
	NSTextFieldCell*	dataCell;
	
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	
	if (column = [mPanelTable tableColumnWithIdentifier:kPGPPrefsPanelColumnIdentifier])
	{
		dataCell = [[NSTextFieldCell alloc] init];
		[dataCell setFont:[NSFont systemFontOfSize:11]];
		[column setDataCell:dataCell];
		[dataCell release];
	}
	
	[mPanelTable setDataSource:self];
	[mPanelTable setDelegate:self];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller
{
    mAppController = controller;

    return [self init];
}

- (id)controller
{
    return mAppController;
}

- (void)setController:(id)controller
{
    if (controller)
        mAppController = controller;
}

- (PGPPrefRef)pgpPrefRef
{
    return mPrefRef;
}

- (void)setPrefRef:(PGPPrefRef)prefRef
{
    if (PGPPrefRefIsValid(prefRef))
    {
        if (PGPPrefRefIsValid(mPrefRef))
        {
            PGPclWritePrefs(mPrefRef);
            PGPFreePrefs(mPrefRef);
        }
		
        mPrefRef = prefRef;
    }
}

- (PGPPrefRef)pgpNetPrefRef
{
    return mNetPrefRef;
}

- (void)setNetPrefRef:(PGPPrefRef)prefRef
{
    if (PGPPrefRefIsValid(prefRef))
        mNetPrefRef = prefRef;
}

- (NSMutableArray*)prefPanels
{
	return mPrefPanels;
}

- (void)setPrefPanels:(NSArray*)prefPanels
{
	if (prefPanels)
	{
		if (mPrefPanels)
			[mPrefPanels release];
			
		mPrefPanels = [prefPanels retain];
	}
}

- (void)sortPrefPanels
{
	NSMutableArray*		newArray = [[[NSMutableArray alloc] init] autorelease];
	NSEnumerator*		enumerator;
	NSString*			searchIdentifier;
	NSString*			identifier;
	unsigned			panelID;
	BOOL				found;
	
	pgpAssert(mPrefPanels != nil);
		
	for (panelID = 0; panelID != kPGPPrefPanel_Invalid; ++panelID)
	{
		switch (panelID)
		{
			case kPGPPrefPanel_General:
				searchIdentifier = NSLocalizedString(@"General", @"General Pref Panel");
				break;
			case kPGPPrefPanel_Files:
				searchIdentifier = NSLocalizedString(@"Files", @"Files Pref Panel");
				break;
			case kPGPPrefPanel_Email:
				searchIdentifier = NSLocalizedString(@"Email", @"Email Pref Panel");
				break;
			case kPGPPrefPanel_Menu:
				searchIdentifier = NSLocalizedString(@"Menu", @"Menu Pref Panel");
				break;
			case kPGPPrefPanel_Server:
				searchIdentifier = NSLocalizedString(@"Server", @"Server Pref Panel");
				break;
			case kPGPPrefPanel_CertAuthority:
				searchIdentifier = NSLocalizedString(@"CA", @"CertAuth Pref Panel");
				break;
			case kPGPPrefPanel_Advanced:
				searchIdentifier = NSLocalizedString(@"Advanced", @"Advanced Pref Panel");
				break;
			case kPGPPrefPanel_IDS:
				searchIdentifier = NSLocalizedString(@"IDS", @"IDS Pref Panel");
				break;
			case kPGPPrefPanel_VPN:
				searchIdentifier = NSLocalizedString(@"VPN", @"VPN Pref Panel");
				break;
			case kPGPPrefPanel_VPNAuth:
				searchIdentifier = NSLocalizedString(@"VPN Authentication", @"VPNAuth Pref Panel");
				break;
			case kPGPPrefPanel_VPNAdvanced:
				searchIdentifier = NSLocalizedString(@"VPN Advanced", @"VPNAdvanced Pref Panel");
				break;
			case kPGPPrefPanel_PGPdisk:
				searchIdentifier = NSLocalizedString(@"PGPdisk", @"PGPdisk Pref Panel");
				break;
			default:
				searchIdentifier = nil;
				break;
		}
		
		if (searchIdentifier)
		{
			found = NO;
			enumerator = [mPrefPanels objectEnumerator];
			while ((identifier = [enumerator nextObject]) && !found)
			{
				if ([identifier isEqualToString:searchIdentifier])
				{
					[newArray addObject:identifier];
					found = YES;
				}	
			}
		}
	}
	
	[self setPrefPanels:newArray];
}

- (NSTableView*)tableView
{
	return mPanelTable;
}

- (PGPError)openPrefs
{
	PGPError		err;
	PGPPrefRef		prefRef = kInvalidPGPPrefRef;
	
	if (!mAppController)
		return kPGPError_BadParams;
	
	err = PGPclOpenPrefs(PGPPeekContextMemoryMgr([mAppController pgpContext]), kPGPclPrefsClientKey, nil, &prefRef, FALSE);

	if (IsPGPError(err))
		return err;
	else
		[self setPrefRef:prefRef];
		
	return kPGPError_NoErr;
}

- (PGPError)closePrefs
{
	PGPError		err;
	
	if (!PGPPrefRefIsValid(mPrefRef))
		return kPGPError_BadParams;
	
	err = PGPclWritePrefs(mPrefRef);
	
	PGPFreePrefs(mPrefRef);
	mPrefRef = kInvalidPGPPrefRef;
	
	return err;
}

- (void)initPreferencesPanel
{
	PGPError				err;
	NSArray*				identifiers;
	NSString*				identifier;
	NSArray*				bundleControllers;
	NSEnumerator*			bundleEnumerator;
	NSEnumerator*			identifierEnumerator;
	id <PGPPrefsController>	prefController;
	id <PGPBundle>			bundleController;

	pgpAssert(mPrefPanels != nil);

	/* Register the General prefs panel */
	if (mGeneralPrefs)
	{
		identifiers = [mGeneralPrefs identifiers];
        pgpAssert(identifiers != nil);
		
		[mControllers addObject:mGeneralPrefs];
		
		identifierEnumerator = [identifiers objectEnumerator];
		while (identifier = [identifierEnumerator nextObject])
		{
			[mGeneralPrefs restorePanelPrefs:identifier];
			
			err = [self registerIdentifier:identifier withController:mGeneralPrefs];
			if (IsPGPError(err))
				NSLog(@"Failed to register prefcontroller (err=%d)", err);
			else
				[mPrefPanels addObject:identifier];
		}
	}

	/* Register the Advanced prefs panel */
	if (mAdvancedPrefs)
	{
		identifiers = [mAdvancedPrefs identifiers];
        pgpAssert(identifiers != nil);
		
		[mControllers addObject:mAdvancedPrefs];
		
		identifierEnumerator = [identifiers objectEnumerator];
		while (identifier = [identifierEnumerator nextObject])
		{
			[mAdvancedPrefs restorePanelPrefs:identifier];
			
			err = [self registerIdentifier:identifier withController:mAdvancedPrefs];
			if (IsPGPError(err))
				NSLog(@"Failed to register prefcontroller (err=%d)", err);
			else
				[mPrefPanels addObject:identifier];
		}
	}
	
	if (bundleControllers = [[mAppController bundleManager] bundleControllers])
	{
		bundleEnumerator = [bundleControllers objectEnumerator];
		
		while (bundleController = [bundleEnumerator nextObject])
		{
			if (!(prefController = [bundleController prefsController]))
				continue;
            
			identifiers = [prefController identifiers];
            pgpAssert(identifiers != nil);
		
			[mControllers addObject:prefController];
			
			identifierEnumerator = [identifiers objectEnumerator];
			
			while (identifier = [identifierEnumerator nextObject])
			{
				[prefController restorePanelPrefs:identifier];
				
				err = [self registerIdentifier:identifier withController:prefController];
				if (IsPGPError(err))
					NSLog(@"Failed to register prefcontroller (err=%d)", err);
				else
					[mPrefPanels addObject:identifier];
			}
		}
	}

	[self sortPrefPanels];
}

- (void)showPreferences
{
	NSEnumerator*				enumerator;
	NSString*					identifier;
	id <PGPPrefsController>		prefController;
	
	enumerator = [mPrefPanels objectEnumerator];
	
	while (identifier = [enumerator nextObject])
	{
		prefController = [self controllerForIdentifier:identifier];
		[prefController restorePanelPrefs:identifier];
	}
	
	identifier = [mPrefPanels objectAtIndex:0];
	prefController = [self controllerForIdentifier:identifier];
	
	[mPanelBox setTitle:identifier];
	[self setPrefsView:[prefController prefsViewForIdentifier:identifier]];
	
	[self performSelector:@selector(selectFirstPanel)
		  withObject:nil
		  afterDelay:0.0
		  inModes:[NSArray arrayWithObject:NSModalPanelRunLoopMode]];
	
    [NSApp runModalForWindow:mWindow];
	
	[mPanelTable selectRow:0 byExtendingSelection:NO];
}

- (void)selectFirstPanel
{
	[mPanelTable selectRow:0 byExtendingSelection:NO];
	[mWindow makeFirstResponder:mPanelTable];
}

- (PGPError)setPrefsView:(NSView*)view
{
	NSNotificationCenter*		center = [NSNotificationCenter defaultCenter];
	
	if (!view)
		return kPGPError_BadParams;
	
	if (mCurrentView)
	{
		[center postNotificationName:kPGPclPrefNotification_PrefsViewInactive object:mCurrentView];
		[mCurrentView release];
	}
	
	[mPanelBox setContentView:view];
	mCurrentView = [view retain];
	
	[center postNotificationName:kPGPclPrefNotification_PrefsViewActive object:mCurrentView];
	
	return kPGPError_NoErr;
}

- (PGPError)setActivePrefs:(NSString*)identifier
{
	PGPError					err;
	id <PGPPrefsController>		controller;
	
	if (!identifier)
		return kPGPError_BadParams;
		
	if (controller = [self controllerForIdentifier:identifier])
	{
		[mPanelBox setTitle:identifier];
		err = [self setPrefsView:[controller prefsViewForIdentifier:identifier]];
		
		[mPanelTable setNextKeyView:[controller firstPrefsResponderForIdentifier:identifier]];
		[controller setupKeyViewLoopForIdentifier:identifier];
		[[controller lastPrefsResponderForIdentifier:identifier] setNextKeyView:mDefaultButton];
		[mDefaultButton setNextKeyView:mRevertButton];
		[mRevertButton setNextKeyView:mCancelButton];
		[mCancelButton setNextKeyView:mSaveButton];
		[mSaveButton setNextKeyView:mPanelTable];
	}
		
	return kPGPError_UnknownError;
}

- (PGPError)registerIdentifier:(NSString*)identifier withController:(id)controller
{
	PGPError		err = kPGPError_NoErr;
	
	if (!controller || !identifier || !mPrefControllers)
		return kPGPError_BadParams;
	
	[mPrefControllers setObject:controller forKey:identifier];
		
	return err;
}

- (id <PGPPrefsController>)controllerForIdentifier:(NSString*)identifier
{
	if (!identifier || !mPrefControllers)
		return nil;
	
	return [mPrefControllers objectForKey:identifier];
}

- (IBAction)defaultButtonPressed:(id)sender
{
	int							row = [mPanelTable selectedRow];
	NSString*					panelIdentifier = nil;
	id <PGPPrefsController>		controller = nil;
	
	if (row != -1)
	{
		panelIdentifier = [mPrefPanels objectAtIndex:row];
		controller = [self controllerForIdentifier:panelIdentifier];
	}

	if (controller)
		[controller restorePanelDefaults:panelIdentifier];
}

- (IBAction)revertButtonPressed:(id)sender
{
	int							row = [mPanelTable selectedRow];
	NSString*					panelIdentifier = nil;
	id <PGPPrefsController>		controller = nil;

	if (row != -1)
	{
		panelIdentifier = [mPrefPanels objectAtIndex:row];
		controller = [self controllerForIdentifier:[mPrefPanels objectAtIndex:row]];
	}
		
	if (controller)
		[controller restorePanelPrefs:panelIdentifier];
}

- (IBAction)saveButtonPressed:(id)sender
{
	NSEnumerator*				controllerEnumerator;
	NSEnumerator*				identifierEnumerator;
	NSString*					panelIdentifier = nil;
	NSArray*					identifiers;
	id <PGPPrefsController>		currentController;
	id <PGPBundleNet>			netController;

	[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_PrefsViewInactive object:mCurrentView];

    [NSApp abortModal];
    [mWindow orderOut:nil];
	
	controllerEnumerator = [mControllers objectEnumerator];
	
	while (currentController = [controllerEnumerator nextObject])
	{
		identifiers = [currentController identifiers];
		identifierEnumerator = [identifiers objectEnumerator];
		
		while (panelIdentifier = [identifierEnumerator nextObject])
			[currentController savePanelPrefs:panelIdentifier];
	}
	
	if (PGPPrefRefIsValid(mPrefRef))
		PGPclWritePrefs(mPrefRef);
	
	if (PGPPrefRefIsValid(mNetPrefRef))
	{
		netController = [[mAppController bundleManager] bundleByName:@"PGPnet.bundle"];
		
		if (netController)
			[netController writeNetPrefs];
	}
}

- (IBAction)cancelButtonPressed:(id)sender
{
	[self revertButtonPressed:sender];

	[[NSNotificationCenter defaultCenter] postNotificationName:kPGPclPrefNotification_PrefsViewInactive object:mCurrentView];

    [NSApp abortModal];
    [mWindow orderOut:nil];
}

- (void)tableViewSelectionDidChange:(NSNotification*)notification
{
	int			row = [mPanelTable selectedRow];
	NSString*	identifier;
	
	if (row != -1)
	{
		identifier = [mPrefPanels objectAtIndex:row];		
		[self setActivePrefs:identifier];
		
		[mWindow makeFirstResponder:mPanelTable];
	}
}

- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	if (mPrefPanels)
		return [mPrefPanels count];
		
	return 0;
}

- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row
{
	if (!mPrefPanels)
		return nil;
		
	NSParameterAssert((row >= 0) && (row < [mPrefPanels count]));
	
	return [mPrefPanels objectAtIndex:row];
}

- (void)dealloc
{
	[self closePrefs];

	if (mGeneralPrefs)
		[mGeneralPrefs release];

    if (mAdvancedPrefs)
        [mAdvancedPrefs release];

	if (mControllers)
		[mControllers release];

	if (mPrefPanels)
		[mPrefPanels release];

	if (mPrefDictionary)
		[mPrefDictionary release];
	
	if (mPrefControllers)
		[mPrefControllers release];

	if (mCurrentView)
		[mCurrentView release];

	[super dealloc];
}

@end
