/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPCMPlugin.c,v 1.4 2002/10/16 06:27:15 jeffc Exp $
____________________________________________________________________________*/
#include <CoreFoundation/CoreFoundation.h>
#include <CoreFoundation/CFPlugInCOM.h>
#include <Carbon/Carbon.h>
#include <syslog.h>
#include "PGPCMPlugin.h"

	static HRESULT
PGPCMPluginQuery(
	void			*context,
	REFIID			iid,
	LPVOID			*ppv )
{
	CFUUIDRef		interfaceID = CFUUIDCreateFromUUIDBytes( NULL, iid );
	PGPCMPlugin		*plugin = (PGPCMPlugin *)context;
	HRESULT			result = S_OK;
	
	if( CFEqual( interfaceID, kContextualMenuInterfaceID ) ||
		CFEqual( interfaceID, IUnknownUUID ) )
	{
		plugin->cmInterface->AddRef( context );
		*ppv = context;
	}
	else
	{
		*ppv = NULL;
		result = E_NOINTERFACE;
	}
	CFRelease( interfaceID );
	return result;
}

	static ULONG
PGPCMPluginRetain(
	void			*context )
{
	PGPCMPlugin		*plugin = (PGPCMPlugin *)context;

	plugin->cmPluginRefCount += 1;
	return plugin->cmPluginRefCount;
}

	static void
freePGPCMPlugin(
	PGPCMPlugin			*plugin )
{
	CFUUIDRef			factoryID = plugin->cmPluginFactoryID;
	
	free( plugin );
	if( factoryID )
	{
		CFPlugInRemoveInstanceForFactory( factoryID );
		CFRelease( factoryID );
	}
}

	static ULONG
PGPCMPluginRelease(
	void			*context )
{	
	PGPCMPlugin		*plugin = (PGPCMPlugin *)context;

	plugin->cmPluginRefCount -= 1;
	if( plugin->cmPluginRefCount == 0 )
	{
		freePGPCMPlugin( plugin );
		return 0;
	}
	else
		return plugin->cmPluginRefCount;
}

	static OSStatus
PGPCMPluginExamineContext(
	void				*context,
	const AEDesc		*inContext,
	AEDescList			*outCommandPairs )
{
	return PGPCMPluginExamineContextImp( context, inContext, outCommandPairs );
}

	static OSStatus
PGPCMPluginHandleSelection(
	void				*context,
	AEDesc				*inContext,
	SInt32				inCommandID )
{
	return PGPCMPluginHandleSelectionImp( context, inContext, inCommandID );
}

	static void
PGPCMPluginPostMenuCleanup(
	void				*context )
{
}

static ContextualMenuInterfaceStruct pgpCMPluginInterface =
{
		NULL,							// Required padding for COM
		PGPCMPluginQuery,				// These three are the required COM functions
		PGPCMPluginRetain,
		PGPCMPluginRelease,
		PGPCMPluginExamineContext,
		PGPCMPluginHandleSelection,
		PGPCMPluginPostMenuCleanup
};

	static PGPCMPlugin *
newPGPCMPlugin()
{
	PGPCMPlugin					*plugin = (PGPCMPlugin *)malloc( sizeof(PGPCMPlugin) );
	
	plugin->cmInterface			= &pgpCMPluginInterface;
	plugin->cmPluginFactoryID	= CFRetain( kPGPCMPluginFactoryID );
	CFPlugInAddInstanceForFactory( kPGPCMPluginFactoryID );
	plugin->cmPluginRefCount	= 1;
	return plugin;
}

	void *
PGPCMPluginFactory(
	CFAllocatorRef		allocator,
	CFUUIDRef			typeID)
{
#if PGP_DEBUG
    syslog( LOG_ERR, "PGPCMPlugin: PGPCMPluginFactory called\n" );
#endif
	if( CFEqual( typeID, kContextualMenuTypeID ) )
		return newPGPCMPlugin();
	else
		return NULL;
}

