/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskAddUser.m,v 1.16 2002/09/24 03:41:56 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPdiskAddUser.h"
#import <PGPclient/pgpClientLibCocoa.h>

#define kPGPdiskString_NeedPassphraseUser		LSPD(@"Username needed", @"Username needed caption")
#define kPGPdiskString_NeedPassphraseUserMsg	LSPD(@"Your disk's user must have a username.", @"Username needed message")
#define kPGPdiskString_NeedPassphrase			LSPD(@"Passphrase needed", @"Passphrase needed caption")
#define kPGPdiskString_NeedPassphraseMsg		LSPD(@"A passphrase is needed to authenticate the user's access to your disk.", @"Passphrase needed message")
#define kPGPdiskString_NeedPublicKeyUser		LSPD(@"Public Key needed", @"Public key needed caption")
#define kPGPdiskString_NeedPublicKeyUserMsg		LSPD(@"Please select the user's Public Key.", @"Public key needed message")

static CPGPdiskAddUser*		sharedInstance = nil;

@implementation CPGPdiskAddUser

+ (id)sharedInstance
{
	if (!sharedInstance)
		sharedInstance = [[CPGPdiskAddUser alloc] init];
	
	return sharedInstance;
}

- (id)init
{	
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}

	[[NSNotificationCenter defaultCenter] addObserver:self
			selector:@selector(textDidChange:)
			name:NSControlTextDidChangeNotification
			object:nil];
	
	if (!mLoadedNib)
		[self initWithNib:kPGPdiskAddUserNib];
	
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];

	[mUserName retain];
	[mKeyBox retain];
	[mKeyBox removeFromSuperview];

	mOriginalUsernameRect = [mUserName frame];
	
	[self authTypeSwitched:self];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (void)displayForWindow:(NSWindow*)window target:(PGPdiskController*)target adminPassphrase:(char*)passphrase addEndSelector:(SEL)addSelector cancelEndSelector:(SEL)cancelSelector
{
	mTarget = target;
	mAddSelector = addSelector;
	mCancelSelector = cancelSelector;
	mAdminPassphrase = passphrase;
	
	[NSApp runModalForWindow:mAddUserWindow];
}

- (void)reset
{
	[mReadOnly setState:NSOffState];
	[mAuthenticationType selectItemAtIndex:[mAuthenticationType indexOfItemWithTag:kPGPdiskCommand_Passphrase]];
	
	[self authTypeSwitched:self];
}

- (IBAction)authTypeSwitched:(id)sender
{
	NSMenuItem*		selectedItem;
	
	selectedItem = [mAuthenticationType selectedItem];
	
	if (mLastSelectedAuthType != [selectedItem tag])
    {
		if ([[mAuthenticationType selectedItem] tag] == kPGPdiskCommand_Passphrase)
		{
			[mKeyBox removeFromSuperview];
			[[mAddUserWindow contentView] addSubview:mUserName];
			
			[mUserNameCaption setStringValue:kPGPdiskString_UsernamePrompt];
			
			if ([[mUserName stringValue] length])
				[mSelectKeyButton setEnabled:YES];
			else
				[mSelectKeyButton setEnabled:NO];
			
			[mSelectKeyButton setTitle:kPGPdiskString_SetPassphrase];
			[mSelectKeyButton setAction:@selector(setPassphrase:)];
		}
		else
		{
			[mUserName removeFromSuperview];
			[[mAddUserWindow contentView] addSubview:mKeyBox];
			
			[mUserNameCaption setStringValue:kPGPdiskString_PublicKeyPrompt];
			[mSelectKeyButton setEnabled:YES];
			[mSelectKeyButton setTitle:kPGPdiskString_SelectKey];
			[mSelectKeyButton setAction:@selector(selectKey:)];
		}
		
		mLastSelectedAuthType = [selectedItem tag];
	}
}

- (IBAction)setPassphrase:(id)sender
{
	PGPError		err;
	PGPContextRef	context = [mTarget pgpContext];
	
	if (context)
	{
		err = PGPConfirmationPassphraseDialog(context,
					PGPOUIOutputPassphrase(context, &mPassphrase),
					PGPOUIMinimumPassphraseLength(context, kPGPdiskMinimumPassphraseLength),
					PGPOLastOption(context));
	}
}

- (IBAction)selectKey:(id)sender
{
	PGPclBeginKeySelectSheet(kPGPdiskString_KeySelectPrompt,
		mAddUserWindow,
        [mTarget pgpPrefRef],
        PGPPeekKeyDBRootKeySet([mTarget pgpKeyDB]),
		[mTarget pgpKeyDB],
        kPGPclKeySelect_SingleKey,
		self,
        @selector(keySelectDidEnd:selectedKeys:));
}

- (IBAction)ok:(id)sender
{
	unsigned char*		utf8Name = NULL;
	
	if ([[mAuthenticationType selectedItem] tag] == kPGPdiskCommand_Passphrase)
	{
		if ([mUserName stringValue] && [[mUserName stringValue] length])
			utf8Name = (unsigned char*) [[mUserName stringValue] UTF8String];
		
		if (![mUserName stringValue] || ![[mUserName stringValue] length])
		{
			NSBeginAlertSheet(kPGPdiskString_NeedPassphraseUser,
                kPGPdiskString_OK,		/* defaultButton */
                nil,					/* alternateButton */
                nil,					/* otherButton */
                mAddUserWindow,			/* docWindow */
                nil,					/* modalDelegate */
                nil,					/* willEndSelector */
                nil,					/* didEndSelector */
                nil,					/* contextInfo */
                kPGPdiskString_NeedPassphraseUserMsg);
			
			return;
		}
		if (strlen(utf8Name) > (kPGPdiskMaxUserNameSize - 1))
		{
			NSBeginAlertSheet(kPGPdiskString_UserNameTooLong,
                kPGPdiskString_OK,		/* defaultButton */
                nil,					/* alternateButton */
                nil,					/* otherButton */
                mAddUserWindow,			/* docWindow */
                nil,					/* modalDelegate */
                nil,					/* willEndSelector */
                nil,					/* didEndSelector */
                nil,					/* contextInfo */
                kPGPdiskString_UserNameTooLongMsg);
			
			return;
		}
		if (!mPassphrase)
		{
			NSBeginAlertSheet(kPGPdiskString_NeedPassphrase,
                kPGPdiskString_OK,		/* defaultButton */
                nil,					/* alternateButton */
                nil,					/* otherButton */
                mAddUserWindow,			/* docWindow */
                nil,					/* modalDelegate */
                nil,					/* willEndSelector */
                nil,					/* didEndSelector */
                nil,					/* contextInfo */
                kPGPdiskString_NeedPassphraseMsg);
			
			return;
		}
	}
	else if ([[mAuthenticationType selectedItem] tag] == kPGPdiskCommand_PublicKey)
	{
		if (!mKeyDBObj)
		{
			NSBeginAlertSheet(kPGPdiskString_NeedPublicKeyUser,
                kPGPdiskString_OK,		/* defaultButton */
                nil,					/* alternateButton */
                nil,					/* otherButton */
                mAddUserWindow,			/* docWindow */
                nil,					/* modalDelegate */
                nil,					/* willEndSelector */
                nil,					/* didEndSelector */
                nil,					/* contextInfo */
                kPGPdiskString_NeedPublicKeyUserMsg);
			
			return;
		}
	}
	
	[NSApp abortModal];
	[mAddUserWindow close];
	
	[self callEndSelector:mAddSelector];
	[self reset];
	
	if (mAdminPassphrase)
	{
		PGPFreeData(mAdminPassphrase);
		mAdminPassphrase = NULL;
	}
	
	if (mPassphrase)
	{
		PGPFreeData(mPassphrase);
		mPassphrase = NULL;
	}
	
	mKeyDBObj = NULL;
	mTarget = nil;
	mAddSelector = nil;
	mCancelSelector = nil;
	
	[self autorelease];
}

- (IBAction)cancel:(id)sender
{
	[NSApp abortModal];	
	[mAddUserWindow close];
	
	[self callEndSelector:mCancelSelector];
	
	[self reset];

	if (mAdminPassphrase)
	{
		PGPFreeData(mAdminPassphrase);
		mAdminPassphrase = NULL;
	}
	
	if (mPassphrase)
	{
		PGPFreeData(mPassphrase);
		mPassphrase = NULL;
	}
	
	mKeyDBObj = NULL;
	mTarget = nil;
	mAddSelector = nil;
	mCancelSelector = nil;
	
	[self autorelease];
}

- (void)callEndSelector:(SEL)selector
{
	NSMethodSignature*		methodSig;
	NSInvocation*			invocation;
	NSString*				username = [mUserName stringValue];
	BOOL					readOnly;
	
	if (mTarget && selector && [mTarget respondsToSelector:selector])
	{
		readOnly = [mReadOnly state] == NSOnState ? YES : NO;
		
		methodSig = [mTarget methodSignatureForSelector:selector];
		
		invocation = [NSInvocation invocationWithMethodSignature:methodSig];
		[invocation setSelector:selector];
		[invocation setTarget:mTarget];
		
		if (selector == mAddSelector)
		{
			[invocation setArgument:&username atIndex:2];
			[invocation setArgument:&mKeyDBObj atIndex:3];
			[invocation setArgument:&mPassphrase atIndex:4];
			[invocation setArgument:&mAdminPassphrase atIndex:5];
			[invocation setArgument:&readOnly atIndex:6];
		}
		else if (selector == mCancelSelector)
			[invocation setArgument:&self atIndex:2];
			
		[invocation invoke];
	}
}

- (void)keySelectDidEnd:(PGPError)resultCode selectedKeys:(PGPKeySetRef)keySet
{
	PGPError		err;
	PGPKeyIterRef	keyIter;
	PGPKeyDBObjRef	keyDBObj;
	NSString*		userID = nil;
	
	if (IsntPGPError(resultCode) && PGPKeySetRefIsValid(keySet))
	{
		if (IsntPGPError(err = PGPNewKeyIterFromKeySet(keySet, &keyIter)))
		{
			if (IsntPGPError(err = PGPKeyIterNextKeyDBObj(keyIter, kPGPKeyDBObjType_Key, &keyDBObj)))
			{
				mKeyDBObj = keyDBObj;
				
				[mKeyIcon setImage:[[CPGPIcons shared] getKeyDBObjIcon:mKeyDBObj]];
				
				userID = PGPclGetPrimaryUserIDName(keyDBObj);
				
				if (userID)
				{
					NSDictionary*	attributes = [NSDictionary dictionaryWithObjectsAndKeys:
														[NSColor blackColor], NSForegroundColorAttributeName, nil];
														
					[[mKeyUserID cell] setAttributedStringValue:[[[NSAttributedString alloc]
											initWithString:userID attributes:attributes] autorelease]];
				}
				else
					[mKeyUserID setStringValue:@""];
			}
			
			PGPFreeKeyIter(keyIter);			
		}
		
		PGPFreeKeySet(keySet);
	}
}

- (void)textDidChange:(NSNotification*)notification
{
	if ([mUserName stringValue] && [[mUserName stringValue] length])
		[mSelectKeyButton setEnabled:YES];
	else
		[mSelectKeyButton setEnabled:NO];
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	if (self == sharedInstance)
		sharedInstance = nil;
		
	if (mPassphrase)
	{
		PGPFreeData(mPassphrase);
		mPassphrase = NULL;
	}

	if (mUserName)
		[mUserName release];

	if (mKeyBox)
		[mKeyBox release];
		
	[super dealloc];
}

@end
