/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskDiskTable.m,v 1.33 2002/08/22 08:55:19 jeffc Exp $
____________________________________________________________________________*/

#import "PGPdiskController.h"
#import "PGPdiskMountDisk.h"
#import "CPGPdiskDiskTable.h"
#import "CPGPdiskObject.h"
#import "CPGPdiskUsers.h"

#import <PGPui/CPGPIcons.h>

@implementation CPGPdiskDiskTable

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mDiskCellIcon)
	{
		mDiskCellIcon = [[NSImage alloc] initWithContentsOfFile:
			[[NSBundle mainBundle] pathForImageResource:kPGPdiskDiskCellIcon]];
		mOS9DiskCellIcon = [[NSImage alloc] initWithContentsOfFile:
			[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPdiskOS9DiskCellIcon]];
		
		[mDiskCellIcon setSize:NSMakeSize(16.0, 16.0)];
	}
    
    PGPclHideFileExtensions(&mHideExtensions);
	
	return self;
}

- (id)initWithController:(PGPdiskController*)controller
{
	mController = controller;
	
	return [self init];
}

- (id)table
{
	return mDiskTable;
}

- (void)setTable:(id)table
{
	NSTableColumn*	diskColumn;
	
	if (table)
		mDiskTable = table;
	
	if (diskColumn = [mDiskTable tableColumnWithIdentifier:@"PGPdisk"])
		mColumnSize = [diskColumn width];
}

- (void)orderListByAscendingOrder:(BOOL)flag
{
	NSArray*			array = [mController disks];
	NSMutableArray*		newArray = nil;
	NSEnumerator*		diskEnumerator;
	NSEnumerator*		newDiskEnumerator;
	CPGPdiskObject*		disk;
	CPGPdiskObject*		newDisk;
	
	if (array)
	{
		newArray = [[NSMutableArray alloc] initWithCapacity:[array count]];
		diskEnumerator = [array objectEnumerator];
		
		while (disk = [diskEnumerator nextObject])
		{
			NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
			NSComparisonResult		result;
			BOOL					inserted = NO;
			int						numNewDisks;
			int						index;
						
			newDiskEnumerator = [newArray objectEnumerator];
			numNewDisks = [newArray count];
			
			if (!numNewDisks)
			{
				[newArray addObject:disk];
				[pool release];
				
				continue;
			}
			
			while (!inserted && (newDisk = [newDiskEnumerator nextObject]))
			{
				index = [newArray indexOfObject:newDisk];
				result = [[disk shortName] compare:[newDisk shortName] options:NSCaseInsensitiveSearch];
				
				if (result == NSOrderedAscending)
				{
					if (flag)
					{
						[newArray insertObject:disk atIndex:index];
						inserted = YES;
					}
				}
				else	/* NSOrderedDescending */
				{
					if (!flag)
					{
						[newArray insertObject:disk atIndex:index];
						inserted = YES;
					}
				}
			}
			
			if (!inserted)
				[newArray addObject:disk];
			
			[pool release];
		}
		
		[mController setDisks:newArray];
		[newArray release];
	}
}

- (BOOL)tableView:(NSTableView*)tableView shouldSelectRow:(int)row
{
	/*
	PGPError			err = kPGPError_NoErr;
	CPGPdiskObject*		disk;
	CPGPdiskObject*		selectedDisk = [mController selectedDisk];
	
	if (row != -1)
	{
		NS_DURING
			disk = [[mController disks] objectAtIndex:row];
		NS_HANDLER
			return NO;	/* row exceeds our array's boundaries. */
	/*	NS_ENDHANDLER
		
		/*
		If we don't have a valid diskRef, we assume that the disk hasn't been opened.
		If we fail to open the disk, we remove it from our array and consequently,
		from the table.
		*/
	/*	if (![disk diskRef])
		{
			err = [disk openDiskWithContext:[mController pgpDiskContext]];
			
			if (IsPGPError(err))
			{
				[mController displayOpenFailedAlertWithError:err forDisk:disk];
				[[mController disks] removeObject:disk];
				[tableView deselectAll:self];
				[tableView reloadData];
				return NO;
			}
			
			if ([disk isBeingReEncrypted])
                [mController continueReEncryptingDisk:disk];
		}
	}
	
	if (selectedDisk && ![selectedDisk isMounted])
		[selectedDisk closeDisk];
	*/
	return YES;
}

- (void)tableViewSelectionDidChange:(NSNotification*)notification
{
	/*
	CPGPdiskObject*		disk;
	
	[[mController userTable] deselectAll:self];

	if ([mDiskTable selectedRow] == -1)
	{
        if (![mLastSelectedDisk isMounted])
        {
            [mLastSelectedDisk closeDisk];
            mLastSelectedDisk = nil;
        }
	}
	*/
	/*
	Whenever the disk selection changes, we update the users for that
	disk if it isn't currently being re-encrypted. If the disk is being
	re-encrypted, we clear the user table (by sending updateUsersForDisk:nil).
	*/
	/*
	if (disk = [mController selectedDisk])
	{
		mLastSelectedDisk = disk;
		
		if ([disk diskIsBusy])
		{
			[mController updateUsersForDisk:nil];
			[mController updateTabViewForDisk:nil];
		}
		else
		{
			[mController updateUsersForDisk:disk];
			[mController updateTabViewForDisk:disk];
		}
	}
	else
	{
		[mController updateUsersForDisk:nil];
		[mController updateTabViewForDisk:nil];
	}
	
	[mController setMountButtonForDisk:disk];
	*/
}

- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row
{
	/*
	CPGPdiskObject*		diskEntry;
	NSSize				size;
	NSColor*			color;
	NSDictionary*		attributes;
	NSAttributedString*	attrString;

	if (diskEntry = [[mController disks] objectAtIndex:row])
	{
        if ([[mController mountedDisks] indexOfObject:diskEntry] != NSNotFound)
        {
            if ([diskEntry isMounted])
                color = [NSColor darkGrayColor];
            else
			{
				[[mController mountedDisks] removeObject:diskEntry];
				
				if ([mController selectedDisk] != diskEntry)
					[diskEntry closeDisk];
				
                color = [NSColor blackColor];
			}
        }
		else
			color = [NSColor blackColor];
		
		attributes = [[NSDictionary alloc] initWithObjectsAndKeys:color, NSForegroundColorAttributeName, nil];
		attrString = [[NSAttributedString alloc] initWithString:[cell stringValue] attributes:attributes];

		if ([diskEntry diskFormat] == kPGPdiskDiskFormat_OS9)
			[cell setImage:mOS9DiskCellIcon];
		else
			[cell setImage:mDiskCellIcon];
		
		[cell setAttributedStringValue:attrString];
		
		[attributes release];
		[attrString release];
	}
	
	size = [cell cellSize];
	
	if (size.width > mColumnSize)
	{
		mColumnSize = size.width;
		[column setWidth:mColumnSize];
	}
	*/
}

- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	/*
	NSArray*	disks = [mController disks];
	
	if (disks)
		return [disks count];
	*/
	return 0;
}

- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row
{
	//NSArray*		disks;
	//int				selectedRow;
	//CPGPdiskObject*	diskEntry;
	id				value = nil;
	
	/*
	selectedRow = [tableView selectedRow];
	[self orderListByAscendingOrder:YES];
	disks = [mController disks];

	NSParameterAssert((row >= 0) && (row < [disks count]));
	if (diskEntry = [disks objectAtIndex:row])
	{
		if ([[column identifier] isEqualToString:@"PGPdisk"])
		{
            if (mHideExtensions)
            {
                if ([diskEntry hideFileExtension])
                    value = [diskEntry shortName];
                else
                    value = [[diskEntry diskPath] lastPathComponent];
            }
            else
                value = [[diskEntry diskPath] lastPathComponent];
		}
	}
	*/
	return value;
}

- (NSDragOperation)tableView:(NSTableView*)table validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
{
    NSPasteboard*	pboard = [info draggingPasteboard];
    NSString*		type;
    NSString*		plist;
    NSString*		hfsFileType;
    NSString*		filename;
    NSArray*		filenames;
    BOOL			atLeastOneValidFile = NO;
    unsigned		numFilenames;
    unsigned		i;
    
    type = [pboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]];

    if (type)
    {
		plist = [[[NSString alloc] initWithData:[pboard dataForType:type]
                                   encoding:[NSString defaultCStringEncoding]] autorelease];
        if (filenames = [plist propertyList])
        {
            numFilenames = [filenames count];
            for (i = 0; (i < numFilenames) && !atLeastOneValidFile; ++i)
            {
                filename = [filenames objectAtIndex:i];
                hfsFileType = NSHFSTypeOfFile(filename);
                if ([[filename pathExtension] isEqualToString:@"pgd"])
                    atLeastOneValidFile = YES;
                else if ([hfsFileType isEqualToString:NSFileTypeForHFSTypeCode(kPGPdiskFileOSType)])
                    atLeastOneValidFile = YES;
            }
        }
    }
    
	if (atLeastOneValidFile)
		[table setDropRow:-1 dropOperation:NSTableViewDropOn];
	
    return atLeastOneValidFile ? NSDragOperationGeneric : NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView*)table acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation
{
    NSPasteboard*	pboard = [info draggingPasteboard];
    NSString*		type;
    NSString*		plist;
    NSString*		hfsFileType;
    NSString*		filename;
    NSArray*		filenames;
    NSMutableArray*	disks = [mController disks];
    CPGPdiskObject*	disk = nil;
	PGPError		err = kPGPError_NoErr;
    BOOL			alreadyInList;
    BOOL			atLeastOneValidFile = NO;
    unsigned		numDisks = [disks count];
    unsigned		numFilenames;
    unsigned		i, j;
    
    type = [pboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]];

    if (type)
    {
        plist = [[[NSString alloc] initWithData:[pboard dataForType:type]
                                   encoding:[NSString defaultCStringEncoding]] autorelease];
        if (filenames = [plist propertyList])
        {
            numFilenames = [filenames count];
            for (i = 0; i < numFilenames; ++i)
            {
                filename = [filenames objectAtIndex:i];
                hfsFileType = NSHFSTypeOfFile(filename);
                
                if (([[filename pathExtension] isEqualToString:@"pgd"]) ||
					([hfsFileType isEqualToString:NSFileTypeForHFSTypeCode(kPGPdiskFileOSType)]))
                {
                    alreadyInList = NO;
                    
                    for (j = 0; (j < numDisks) && !alreadyInList; ++j)
                    {
                        if ([[[disks objectAtIndex:j] diskPath] isEqualToString:filename])
                            alreadyInList = YES;
                    }
                    
                    if (!alreadyInList)
                    {
                        disk = [CPGPdiskObject diskAtPath:filename];
						[disk setController:mController];
						err = [disk openDiskWithContext:[mController pgpDiskContext]];
						
						if (IsPGPError(err))
						{
							[mController displayOpenFailedAlertWithError:err forDisk:disk];
							disk = nil;
						}
						else
						{
							[disk closeDisk];
							atLeastOneValidFile = YES;
						}
                    }
				}
                
                if (disk)
                {
                    if (!numDisks || (row > numDisks))
                        [[mController disks] addObject:disk];
                    else
                        [[mController disks] insertObject:disk atIndex:row];
					
                    disk = nil;
                }
            }
        }
    }
    
    if (atLeastOneValidFile)
        [table reloadData];
    
    return atLeastOneValidFile;
}

- (void)dealloc
{
	if (mDiskCellIcon)
		[mDiskCellIcon release];
		
	if (mOS9DiskCellIcon)
		[mOS9DiskCellIcon release];
		
	[super dealloc];
}

@end
