/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskObject.h,v 1.26 2002/09/30 22:32:18 jeffc Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>
#import <PGP/PGP.h>

#import "CPGPdiskProgressPanel.h"
#import "PGPdiskController.h"
#import "pgpDiskEngine.h"

@class CPGPdiskUser, CPGPdiskObject;

@interface CPGPdiskObject : NSObject
{
	PGPdiskController*	mDiskController;
	PGPdiskDiskRef		mDiskRef;
	PGPdiskUserSetRef	mDiskUserSet;
	NSString*			mDiskPath;
	NSMutableArray*		mDiskUsers;
	
	PGPUInt32			mDiskFormat;
	
	BOOL				mHideExtension;
	BOOL				mDiskIsBusy;
	BOOL				mDiskNeedsConversion;
	BOOL				mInited;
}

+ (id)diskAtPath:(NSString*)path;

- (id)init;
- (id)initWithPath:(NSString*)path;

- (PGPdiskController*)diskController;
- (void)setController:(PGPdiskController*)controller;

- (NSMutableArray*)diskUsers;
- (void)setDiskUsers:(NSMutableArray*)users;

- (PGPdiskDiskRef)diskRef;
- (void)setDiskRef:(PGPdiskDiskRef)diskRef;

- (PGPdiskUserSetRef)userSet;
- (void)setUserSet:(PGPdiskUserSetRef)userSet;

- (BOOL)diskIsBusy;
- (void)setDiskIsBusy:(BOOL)flag;

/* Get boolean properties */
- (BOOL)isMounted;
- (BOOL)isReadOnly;
- (BOOL)isBeingReEncrypted;
- (BOOL)hasTimedOut;
- (BOOL)usesCustomTimeout;
- (BOOL)usesCustomEjectOnLogout;
- (BOOL)ejectOnLogout;

/* Set boolean properties */
- (void)setUsesCustomTimeout:(PGPBoolean)flag;
- (void)setUsesCustomEjectOnLogout:(PGPBoolean)flag;
- (void)setEjectOnLogout:(PGPBoolean)flag;

- (BOOL)getBooleanProperty:(PGPdiskDiskPropName)property;
- (PGPError)setBooleanProperty:(PGPdiskDiskPropName)property value:(PGPBoolean)flag;

/* Get numeric properties */
- (PGPdiskDiskFormat)diskFormat;
- (PGPdiskEncryptionAlgorithm)diskAlgorithm;
- (PGPUInt32)customTimeout;
- (PGPUInt32)blocksDiskLow;
- (PGPUInt32)blocksDiskHigh;

/* Set numeric properties */
- (void)setCustomTimeout:(PGPUInt32)timeout;

- (PGPUInt32)getNumericProperty:(PGPdiskDiskPropName)property;
- (PGPError)setNumericProperty:(PGPdiskDiskPropName)property value:(PGPUInt32)value;

/* Get string properties */
- (NSString*)diskPath;
- (NSString*)diskRoot;
- (NSString*)defaultRoot;
- (NSString*)devicePath;

/* Set string properties */
- (void)setDiskRoot:(NSString*)root;

- (NSString*)getStringProperty:(PGPdiskDiskPropName)property;
- (PGPError)setStringProperty:(PGPdiskDiskPropName)property value:(NSString*)string;

/* Convenient little accessors */
- (NSString*)diskSize;
- (NSString*)shortName;
- (BOOL)isWritable;	/* Example: Mounting a remote share as a guest */
- (BOOL)hasPubKeyUser;
- (BOOL)hideFileExtension;
- (BOOL)diskNeedsReEncryption;
- (BOOL)diskHasMoved;

- (void)setDiskPath:(NSString*)path;
- (void)updateUserSet;

- (PGPError)mountWithPassphrase:(const char*)passphrase;
- (PGPError)unmount;
- (PGPError)importUserSet:(PGPdiskUserSetRef)userSet adminPassphrase:(const char*)passphrase adminUser:(PGPdiskUserRef)adminUser;
- (PGPError)getAdminPassphrase:(char**)passphrasePtr;
- (PGPError)getPassphrase:(char**)passphrasePtr forUser:(PGPdiskUserRef)user;
- (PGPError)openDiskWithContext:(PGPdiskContextRef)contextRef;
- (PGPError)openDiskWithContext:(PGPdiskContextRef)contextRef diskRef:(PGPdiskDiskRef*)diskRef ignoreReEncryption:(BOOL)flag;
- (void)closeDisk;

- (NSComparisonResult)compareName:(CPGPdiskObject*)disk;

- (void)dealloc;

@end
