/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskProgressPanel.m,v 1.9 2002/08/06 20:09:29 dallen Exp $
____________________________________________________________________________*/

#import "CPGPdiskProgressPanel.h"
#import <PGPclient/pgpClientLibCocoa.h>

static CPGPdiskProgressPanel*	sharedInstance = nil;

@implementation CPGPdiskProgressPanel

+ (id)sharedPanel
{
	if (!sharedInstance)
		sharedInstance = [[CPGPdiskProgressPanel alloc] init];
		
	return sharedInstance;
}

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}

	if (!mLoadedNib)
		[self initWithNib:kPGPdiskProgressPanelNib];
	
	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];

	[mWindow setExcludedFromWindowsMenu:YES];

	mLoadedNib = YES;
	
    return [self init];
}

- (NSString*)caption
{
	return [mCaption stringValue];
}

- (void)setCaption:(NSString*)caption
{
	NSRect		windowFrame = [[mWindow contentView] frame];
	NSRect		captionFrame = [mCaption frame];
	NSRect		newCaptionFrame;
	NSSize		cellSize;
	
	[mCaption setStringValue:caption];
	
	cellSize = [[mCaption cell] cellSize];
	
	if (cellSize.width > NSWidth(captionFrame))
	{
		cellSize.height = NSHeight(captionFrame) * (int) (cellSize.width / NSWidth(captionFrame) + 1);
			
		newCaptionFrame.origin.x = captionFrame.origin.x;
		newCaptionFrame.origin.y = captionFrame.origin.y;
		newCaptionFrame.size.width = NSWidth(captionFrame);
		newCaptionFrame.size.height = cellSize.height;			
			
		windowFrame.size.height += (cellSize.height - NSHeight(captionFrame));
		[mWindow setContentSize:windowFrame.size];
		
		[mCaption setFrame:newCaptionFrame];
	}
}

- (NSWindow*)window
{
    return mWindow;
}

- (NSProgressIndicator*)progressIndicator
{
	return mProgressIndicator;
}

- (void)runSheetModalToWindow:(NSWindow*)window modalDelegate:(id)delegate didEndSelector:(SEL)endSelector stopSelector:(SEL)stopSelector contextInfo:(void*)contextInfo
{
    mDelegate = delegate;
    mEndSelector = endSelector;
    mStopSelector = stopSelector;
	mContextInfo = contextInfo;

    [NSApp beginSheet:mWindow
           modalForWindow:window
           modalDelegate:self
           didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
           contextInfo:nil];
}

- (void)endSheetWithReturnCode:(PGPError)returnCode
{
	[NSApp endSheet:mWindow returnCode:returnCode];
}

- (IBAction)stop:(id)sender
{
    NSInvocation*			invocation = nil;
    NSMethodSignature*		methodSig;
    
    if (mDelegate && mStopSelector)
    {
        methodSig = [mDelegate methodSignatureForSelector:mStopSelector];
        invocation = [NSInvocation invocationWithMethodSignature:methodSig];
        [invocation setSelector:mStopSelector];
        [invocation setTarget:mDelegate];
        
        [invocation invoke];
    }
}

- (void)sheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
    NSInvocation*			invocation = nil;
    NSMethodSignature*		methodSig;
    
    if (mDelegate && mEndSelector)
    {
        methodSig = [mDelegate methodSignatureForSelector:mEndSelector];
        invocation = [NSInvocation invocationWithMethodSignature:methodSig];
        [invocation setSelector:mEndSelector];
        [invocation setTarget:mDelegate];
        [invocation setArgument:&returnCode atIndex:2];
		[invocation setArgument:&mContextInfo atIndex:3];
        [invocation invoke];
    }
    
    [self close];
}

- (void)close
{
    mDelegate = nil;
    mEndSelector = nil;
    mStopSelector = nil;
    
	[mWindow close];
}

- (void)dealloc
{
	PGPLog((@"Deallocated CPGPdiskProgressPanel"));
	
	if (self == sharedInstance)
		sharedInstance = nil;
		
	[super dealloc];
}

@end
