/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskUsers.m,v 1.25 2002/11/09 03:05:39 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPdiskUsers.h"
#import "CPGPdiskObject.h"
#import "pgpDebug.h"

#define kPGPdiskString_DisableUser				LSPD(@"Disable user", @"Disable user caption")
#define kPGPdiskString_DisableUserMsg			LSPD(@"Are you sure you want to disable the user \"%@\"? The user will not be able to mount the disk.", @"Disable user message")
#define kPGPdiskString_RemoveUser				LSPD(@"Confirm: User Removal", @"Remove user caption")
#define kPGPdiskString_RemoveUserMsg			LSPD(@"Remove the user \"%@\" from the disk?", @"Remove user message")
#define kPGPdiskString_AddUserReadOnly			LSPD(@"Unable to add a new user to the selected disk because you do not have write access to the disk image.", @"")
#define kPGPdiskString_CantDisableReadOnly		LSPD(@"Unable to disable the selected user because you do not have write access to the selected disk's disk image.", @"")
#define kPGPdiskString_ChangePasswdReadOnly		LSPD(@"Unable to change the passphrase for the selected user because you do not have write access to the selected disk's disk image.", @"")
#define kPGPdiskString_RemoveUserReadOnly		LSPD(@"Unable to remove the selected user because you do not have write access to the selected disk's disk image.", @"")
#define kPGPdiskString_ReadOnlyPressedReadOnly	LSPD(@"Unable to change the selected user's read-only status because you do not have write access to the selected disk's disk image.", @"")
#define kPGPdiskString_AdminPressedReadOnly		LSPD(@"Unable to change the selected disk's administrator because you do not have write access to the disk image.", @"")
#define	kPGPdiskString_CantChangeAdmin			LSPD(@"Cannot Change Administrator", @"Can't change admin user caption")
#define kPGPdiskString_PrivateKeyNotPresent		LSPD(@"Your main keyring doesn't contain a private key for the specified public key disk user. If you wish to make a public key user the disk's administrator, you must have the corresponding private key on your main keyring.", @" Private key missing admin alert")

@implementation CPGPdiskUser

+ (PGPdiskUserRef)findAdminInUserSet:(PGPdiskUserSetRef)userSet
{
	PGPError			err;
	PGPdiskUserRef		user = NULL;
	PGPdiskUserIterRef	userIter;
	PGPBoolean			isAdmin = FALSE;
	
	if (!userSet)
		return NULL;
		
	if (IsPGPError(err = PGPdiskNewUserIter(userSet, &userIter)))
		return NULL;
		
	while (!isAdmin && IsntPGPError(err = PGPdiskUserIterNext(userIter, &user)))
		err = PGPdiskGetUserBooleanProp(user, kPGPdiskUserProp_IsAdminUser, &isAdmin);
		
	if (IsPGPError(err) && (err != kPGPError_EndOfIteration))
		return NULL;
	
	if (userIter)
		PGPdiskFreeUserIter(userIter);
	
	return user;
}

+ (PGPdiskUserRef)findUser:(PGPdiskUserRef)userRef inUserSet:(PGPdiskUserSetRef)userSet
{
	PGPError			err;
	PGPdiskUserRef		user = NULL;
	PGPdiskUserIterRef	userIter;
	BOOL				found = NO;

	if (!userRef || !userSet)
		return NULL;
		
	if (IsPGPError(err = PGPdiskNewUserIter(userSet, &userIter)))
		return NULL;
		
	while (!found && IsntPGPError(err = PGPdiskUserIterNext(userIter, &user)))
	{
		if (PGPdiskUserAreUsersEqual(user, userRef))
			found = YES;
	}
		
	if (IsPGPError(err) && (err != kPGPError_EndOfIteration))
		return NULL;
	
	if (userIter)
		PGPdiskFreeUserIter(userIter);
	
	return user;
}

+ (PGPdiskUserRef)findUserAtIndex:(int)index inUserSet:(PGPdiskUserSetRef)userSet
{
	PGPError			err;
	PGPdiskUserRef		user = NULL;
	PGPdiskUserIterRef	userIter;
	int					i;
	
	if (!userSet)
		return NULL;
	
	if (IsPGPError(err = PGPdiskNewUserIter(userSet, &userIter)))
		return NULL;
	
	for (i = 0; i <= index; ++i)
	{
		if (IsPGPError(err = PGPdiskUserIterNext(userIter, &user)))
			return NULL;
	}
	
	return user;
}

- (id)initWithUserRef:(PGPdiskUserRef)userRef
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	[self setUserRef:userRef];
	
	return self;
}

- (void)updateProperties
{
	NSData*			data;
	PGPError		err;
	PGPSize			actualSize;
	char			name[kPGPdiskMaxUserNameSize + 1];
	
	if (!mUserRef)
	{
		PGPLog((@"updateProperties called with invalid mUserRef"));
		return;
	}
	
	if (IsPGPError(err = PGPdiskGetUserDataProp(mUserRef, kPGPdiskUserProp_UserName, name, kPGPdiskMaxUserNameSize, &actualSize)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_UserName (err=%d)", err));
		name[0] = '\0';
	}
	else
		name[actualSize] = '\0';
	
	if (!(mName = [NSString stringWithUTF8String:name]))
	{
		/* Attempt Mac OS Roman encoding */
		
		data = [NSData dataWithBytes:name length:sizeof(name)];
		mName = [[[NSString alloc] initWithData:data encoding:NSMacOSRomanStringEncoding] autorelease];
	}
	
	[mName retain];
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_HasPublicKey, &mHasPublicKey)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_HasPublicKey (err=%d)", err));
		mHasPublicKey = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_HasSymmetricKey, &mHasSymmetricKey)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_HasSymmetricKey (err=%d)", err));
		mHasSymmetricKey = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_IsReadOnly, &mIsReadOnly)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_IsReadOnly (err=%d)", err));
		mIsReadOnly = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_IsLocked, &mIsLocked)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_IsLocked (err=%d)", err));
		mIsLocked = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_IsAdminUser, &mIsAdminUser)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_IsAdminUser (err=%d)", err));
		mIsAdminUser = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_IsDisabled, &mIsDisabled)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_IsDisabled (err=%d)", err));
		mIsDisabled = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_CanSetReadOnly, &mCanSetReadOnly)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_CanSetReadOnly (err=%d)", err));
		mCanSetReadOnly = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_CanSetLocked, &mCanSetLocked)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_CanSetLocked (err=%d)", err));
		mCanSetLocked = FALSE;
	}
	
	if (IsPGPError(err = PGPdiskGetUserBooleanProp(mUserRef, kPGPdiskUserProp_CanSetUserName, &mCanSetUserName)))
	{
		PGPLog((@"Failed to retrieve kPGPdiskUserProp_CanSetUserName (err=%d)", err));
		mCanSetUserName = FALSE;
	}
}

- (NSString*)name
{
	return mName;
}

- (void)setName:(NSString*)name
{
	PGPError		err;
	
	err = PGPdiskSetUserDataProp(mUserRef, kPGPdiskUserProp_UserName, [name UTF8String], [name length] + 1);
	
	if (IsPGPError(err))
		PGPLog((@"Failed to set kPGPdiskUserProp_UserName (err=%d)", err));
	
	[self updateProperties];
}

- (PGPdiskUserRef)userRef
{
	return mUserRef;
}

- (void)setUserRef:(PGPdiskUserRef)userRef
{
	mUserRef = userRef;
}

- (BOOL)isAdmin
{
	return mIsAdminUser;
}

- (BOOL)isReadOnly
{
	return mIsReadOnly;
}

- (BOOL)isSymmetricKeyUser
{
	return mHasSymmetricKey;
}

- (BOOL)isPublicKeyUser
{
	return mHasPublicKey;
}

- (BOOL)isDisabled
{
	return mIsDisabled;
}

- (void)dealloc
{
	[super dealloc];
}

@end

@implementation PGPdiskController (PGPdiskUserExtensions)

- (BOOL)updateUsersForDisk:(CPGPdiskObject*)disk
{
	NSMutableArray*		users;
	CPGPdiskUser*		newUser;
	PGPdiskDiskRef		diskRef = NULL;
	PGPdiskUserSetRef	userSet = NULL;
	PGPdiskUserIterRef	userIter = NULL;
	PGPdiskUserRef		userRef = NULL;	
	PGPError			err;
	BOOL				openedDisk = NO;
	
	if (!disk)
	{
		mAddUser = NO;
		mRemoveUser = NO;
		
        mUsers = nil;
        [mUserTable reloadData];
        
        return YES;
    }

	if (mDiskEngineIsBusy)
		return YES;
	
	if (!(diskRef = [disk diskRef]))
	{
		if (!mDiskContext)
		{
			PGPLog((@"Creating disk context a little late here..."));
			[self createDiskContext];
		}
		
		err = [disk openDiskWithContext:mDiskContext];
		
		if (IsPGPError(err))
		{
			[self displayOpenFailedAlertWithError:err forDisk:disk];
			return NO;
		}
		else
		{
			diskRef = [disk diskRef];
			openedDisk = YES;
		}
	}
	
	if (!diskRef)
		return NO;
	
	if ([disk hasPubKeyUser])
		[self updateKeyrings];
	
	[disk updateUserSet];
	
	userSet = [disk userSet];
	
	if (!PGPdiskUserSetRefIsValid(userSet))
	{
		PGPLog((@"%s:%d: Invalid user set", __FILE__, __LINE__));
		
		if (openedDisk)
			[disk closeDisk];
		
		return NO;
	}

	if (IsPGPError(err = PGPdiskNewUserIter(userSet, &userIter)))
	{
		PGPLog((@"%s:%d: Failed to create user iterator (err=%d)", __FILE__, __LINE__, err));
		
		if (openedDisk)
			[disk closeDisk];
		
		return NO;
	}
	
	users = [[NSMutableArray alloc] init];
	
	while (IsntPGPError(err = PGPdiskUserIterNext(userIter, &userRef)))
	{
		newUser = [[[CPGPdiskUser alloc] initWithUserRef:userRef] autorelease];
		[newUser updateProperties];
		
		[users addObject:newUser];
	}
	
	if (err != kPGPError_EndOfIteration)
		PGPLog((@"%s:%d: Loop terminated abnormally (err=%d)", __FILE__, __LINE__, err));

	[disk setDiskUsers:users];
	[users release];
	
	mUsers = [disk diskUsers];
	
	mAddUser = YES;
	[mUserTable reloadData];
	
	PGPdiskFreeUserIter(userIter);
	
	if (openedDisk)
		[disk closeDisk];
	
	return YES;
}

- (IBAction)disablePressed:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	CPGPdiskUser*		user = [self selectedUser];

	if (disk && user)
	{
		if (![disk isWritable])
			[self displayAlert:kPGPdiskString_ReadOnlyAccess message:kPGPdiskString_CantDisableReadOnly critical:NO];
		else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
			[self convertDisk:disk displayAlert:YES];
		else
		{
			if ([user isDisabled])
				[self disableUserWarningDidEnd:nil returnCode:NSCancelButton contextInfo:user];
			else
			{
				NSBeginAlertSheet(kPGPdiskString_DisableUser,
								  kPGPdiskString_Cancel,	/* defaultButton */
								  kPGPdiskString_Disable,	/* alternateButton */
								  nil,						/* otherButton */
								  mWindow,					/* docWindow */
								  self,						/* modalDelegate */
								  nil,						/* willEndSelector */
								  @selector(disableUserWarningDidEnd:returnCode:contextInfo:),	/* didEndSelector */
								  user,						/* contextInfo */
								  [NSString stringWithFormat:kPGPdiskString_DisableUserMsg, [user name]]);
			}
		}
	}
}

- (IBAction)changePassphrase:(id)sender
{
	PGPContextRef		context = [self pgpContext];
	CPGPdiskObject*		disk = [self selectedDisk];
	CPGPdiskUser*		user;
	PGPError			err = kPGPError_NoErr;
	PGPdiskUserRef		userRef = NULL;
	PGPdiskUserSetRef	userSet = NULL;
	char*				oldPassphrase = NULL;
	char*				newPassphrase = NULL;
	BOOL				openedDisk = NO;
	
	if (![disk diskRef])
	{
		[disk openDiskWithContext:[self pgpDiskContext]];
		openedDisk = YES;
	}
	
	[self updateUsersForDisk:disk];
	
	if (![disk isWritable])
		[self displayAlert:kPGPdiskString_ReadOnlyAccess message:kPGPdiskString_ChangePasswdReadOnly critical:NO];		
	else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
		[self convertDisk:disk displayAlert:YES];
	else
	{
		[self setDiskEngineIsBusy:YES];
		
		userSet = [disk userSet];
		
		if (user = [mUsers objectAtIndex:[mUserTable selectedRow]])
			userRef = [CPGPdiskUser findUserAtIndex:[mUserTable selectedRow] inUserSet:userSet];
		
		if (IsPGPError(err = [disk getPassphrase:&oldPassphrase forUser:userRef]))
		{
			if (oldPassphrase)
				PGPFreeData(oldPassphrase);
		}
		else
		{                    
			err = PGPConfirmationPassphraseDialog(context,
						PGPOUIOutputPassphrase(context, &newPassphrase),
						PGPOUIMinimumPassphraseLength(context, kPGPdiskMinimumPassphraseLength),
						PGPOLastOption(context));
			
			if (IsntPGPError(err))
			{
				err = PGPdiskUserChangePassphrase(userRef, oldPassphrase, newPassphrase);
				
				if (IsPGPError(err))
					PGPLog((@"Failed to change passphrase for user (err=%d)", err));
			}
			
			if (oldPassphrase)
				PGPFreeData(oldPassphrase);
			
			if (newPassphrase)
				PGPFreeData(newPassphrase);
		}
		
		[self setDiskEngineIsBusy:NO];
	}
	
	if (openedDisk)
		[disk closeDisk];
	
	[self updateUsersForDisk:disk];
}

- (IBAction)addUserPressed:(id)sender
{
	CPGPdiskAddUser*	addUser = [CPGPdiskAddUser sharedInstance];
	CPGPdiskObject*		disk = [self selectedDisk];
	PGPError			err = kPGPError_NoErr;
	char*				passphrase;
	BOOL				openedDisk = NO;
	
	if (![disk isWritable])
		[self displayAlert:kPGPdiskString_ReadOnlyAccess message:kPGPdiskString_AddUserReadOnly critical:NO];
	else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
		[self convertDisk:disk displayAlert:YES];
	else if ([disk diskNeedsReEncryption])
		[self continueReEncryptingDisk:disk];
	else
	{
		if (![disk diskRef])
		{
			[disk openDiskWithContext:[self pgpDiskContext]];
			openedDisk = YES;
		}
		
		[self updateUsersForDisk:disk];
		
		if (IsPGPError(err = [disk getAdminPassphrase:&passphrase]))
		{
			if (passphrase)
				PGPFreeData(passphrase);
			
			[self displayUserModAlertWithError:err forDisk:disk];
		}
		else
		{
			[addUser displayForWindow:[mController mainWindow]
					target:self
					adminPassphrase:passphrase
					addEndSelector:@selector(newUser:keyDBObj:passphrase:adminPassphrase:readOnly:)
					cancelEndSelector:@selector(addUserCancelled:)];
		}
		
		if (openedDisk)
			[disk closeDisk];
	}
}

- (IBAction)removeUserPressed:(id)sender
{
    CPGPdiskObject*		disk;
	CPGPdiskUser*		user = [self selectedUser];
	int					selectedRow;
	
	selectedRow = [mUserTable selectedRow];
	
	if (selectedRow != -1)
	{
        disk = [self selectedDisk];
        
		if (![disk isWritable])
			[self displayAlert:kPGPdiskString_ReadOnlyAccess message:kPGPdiskString_RemoveUserReadOnly critical:NO];
        else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
            [self convertDisk:disk displayAlert:YES];
		else if ([disk diskNeedsReEncryption])
			[self continueReEncryptingDisk:disk];
        else
        {
            user = [mUsers objectAtIndex:selectedRow];
        
            NSBeginAlertSheet(kPGPdiskString_RemoveUser,
                            kPGPdiskString_Cancel,		/* defaultButton */
                            kPGPdiskString_Remove,		/* alternateButton */
                            nil,						/* otherButton */
                            mWindow,					/* docWindow */
                            self,						/* modalDelegate */
                            nil,						/* willEndSelector */
                            @selector(removeUserWarningDidEnd:returnCode:contextInfo:),	/* didEndSelector */
                            user,						/* contextInfo */
                            [NSString stringWithFormat:
                                    kPGPdiskString_RemoveUserMsg,
                                    [user name]]);
        }
	}
}

- (IBAction)readonlyPressed:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	CPGPdiskUser*		user;
	PGPError			err = kPGPError_NoErr;
	PGPBoolean			readOnly = FALSE;
	PGPdiskUserSetRef	userSet = NULL;
	PGPdiskUserRef		userRef = NULL;
	char*				passphrase;
	BOOL				openedDisk = NO;
	
	if (![disk diskRef])
	{
		[disk openDiskWithContext:[self pgpDiskContext]];
		openedDisk = YES;
	}
	
	[self updateUsersForDisk:disk];
	
	if (![disk isWritable])
		[self displayAlert:kPGPdiskString_ReadOnlyAccess message:kPGPdiskString_ReadOnlyPressedReadOnly critical:NO];
	else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
		[self convertDisk:disk displayAlert:YES];
	else
	{
		[self setDiskEngineIsBusy:YES];
		
		if (IsPGPError(err = [disk getAdminPassphrase:&passphrase]))
		{
			if (passphrase)
				PGPFreeData(passphrase);
			
			[self displayUserModAlertWithError:err forDisk:disk];
		}
		else
		{
			userSet = [disk userSet];
			
			if (user = [mUsers objectAtIndex:[mUserTable selectedRow]])
			{
				readOnly = [user isReadOnly];
				userRef = [CPGPdiskUser findUserAtIndex:[mUserTable selectedRow] inUserSet:userSet];
			}
			
			if (userRef)
			{
				if (IsntPGPError(err = PGPdiskSetUserBooleanProp(userRef, kPGPdiskUserProp_IsReadOnly, !readOnly)))
				{
					if (IsPGPError(err = [disk importUserSet:userSet adminPassphrase:passphrase adminUser:nil]))
						PGPLog((@"Failed to import modified user set (err=%d)", err));
				}
				else
					PGPLog((@"Failed to set readonly attribute for user (err=%d)", err));
			}
			
			PGPFreeData(passphrase);
		}
		
		[self setDiskEngineIsBusy:NO];
	}
	
	if (openedDisk)
		[disk closeDisk];
	
	[self updateUsersForDisk:disk];
}

- (IBAction)adminPressed:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	CPGPdiskUser*		user;
	PGPError			err = kPGPError_NoErr;
	PGPdiskUserSetRef	userSet = NULL;
	PGPdiskUserRef		userRef = NULL;	
	PGPKeyID			keyID;
	PGPKeyDBObjRef		key;
	PGPBoolean			isSecret;
	char*				passphrase;
	int					userRow = [mUserTable selectedRow];
	BOOL				openedDisk = NO;
	
	if (userRow == -1)
		return;
	
	if (![disk diskRef])
	{
		[disk openDiskWithContext:[self pgpDiskContext]];
		openedDisk = YES;
	}
	
	[self updateUsersForDisk:disk];
	
	user = [mUsers objectAtIndex:[mUserTable selectedRow]];
	
	if (![user isAdmin])
	{
		userSet = [disk userSet];
		userRef = [CPGPdiskUser findUserAtIndex:[mUserTable selectedRow] inUserSet:userSet];
		
		/* If the user has a public key, but no associated private key, we deny the request. */
		if ([user isPublicKeyUser])
		{
			if (IsPGPError(err = PGPdiskUserGetPGPKeyID(userRef, &keyID)))
			{
				PGPLog((@"PGPdiskUserGetPGPKeyID failed with err: %d", err));
				goto done;
			}
			
			if (IsPGPError(err = PGPFindKeyByKeyID([self pgpKeyDB], &keyID, &key)))
			{
				PGPLog((@"PGPFindKeyByKeyID failed with err: %d", err));
				goto done;
			}
			
			if (IsPGPError(err = PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsSecret, &isSecret)))
			{
				PGPLog((@"PGPGetKeyDBObjBooleanProperty failed with err: %d", err));
				goto done;
			}
			
			if (!isSecret)
			{
				[self displayAlert:kPGPdiskString_CantChangeAdmin message:kPGPdiskString_PrivateKeyNotPresent critical:NO];
				goto done;
			}
		}
		
		if (![disk isWritable])
			[self displayAlert:kPGPdiskString_ReadOnlyAccess message:kPGPdiskString_AdminPressedReadOnly critical:NO];
		else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
			[self convertDisk:disk displayAlert:YES];
		else
		{
			[self setDiskEngineIsBusy:YES];
			
			if (IsPGPError(err = [disk getAdminPassphrase:&passphrase]))
			{
				if (passphrase)
					PGPFreeData(passphrase);
				
				[self displayUserModAlertWithError:err forDisk:disk];
			}
			else
			{
				userSet = [disk userSet];
				
				if (user = [mUsers objectAtIndex:[mUserTable selectedRow]])
					userRef = [CPGPdiskUser findUserAtIndex:[mUserTable selectedRow] inUserSet:userSet];
				
				if (userRef)
				{
					if (IsPGPError(err = [disk importUserSet:userSet adminPassphrase:passphrase adminUser:userRef]))
						PGPLog((@"Failed to set new disk admin"));
				}
				
				PGPFreeData(passphrase);
			}
			
			[self setDiskEngineIsBusy:NO];
		}
	}
	
done:
	if (openedDisk)
		[disk closeDisk];
	
	[self updateUsersForDisk:disk];
}

- (PGPError)newUser:(NSString*)username keyDBObj:(PGPKeyDBObjRef)keyDBObj passphrase:(const char*)passphrase adminPassphrase:(const char*)adminPassphrase readOnly:(BOOL)flag
{
	PGPError			err = kPGPError_NoErr;
	PGPdiskUserSetRef	userSet = NULL;
	PGPdiskUserRef		newUser;
	PGPdiskUserRef		adminUser;
	PGPKeyID			keyID;
	CPGPdiskObject*		disk = [self selectedDisk];
	BOOL				openedDisk = NO;
	
	if (!disk)
	{
		err = kPGPError_BadParams;
		goto done;
	}
	
	if (![disk diskRef])
	{
		[disk openDiskWithContext:[self pgpDiskContext]];
		openedDisk = YES;
	}
	
    if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
        [self convertDisk:disk displayAlert:YES];
    else
    {
        if (IsPGPError(err = PGPdiskNewUserSet([self pgpDiskContext], &userSet)))
            goto done;
            
        if (IsPGPError(err = PGPdiskDiskExportUserSet([disk diskRef], userSet)))
            goto done;
		
        adminUser = [CPGPdiskUser findAdminInUserSet:userSet];
		
        if (keyDBObj)
        {
			err = PGPGetKeyID(keyDBObj, &keyID);
			pgpAssertNoErr(err);
			
            err = PGPdiskNewUserWithPublicKey(userSet,
                        &keyID,
                        flag,
                        FALSE,
                        &newUser);
                        
            if (IsPGPError(err))
                goto done;
        }
        else if (username && [username length] && passphrase)
        {
            err = PGPdiskNewUserWithSymmetricKey(userSet,
                        [username UTF8String],
                        passphrase,
                        flag,
                        &newUser);
            
            if (IsPGPError(err))
                goto done;
        }
        
        err = PGPdiskDiskImportUserSet([disk diskRef], adminPassphrase, userSet, adminUser);
        
        if (IsPGPError(err))
            goto done;
    }
    
done:
	if (userSet)
		PGPdiskFreeUserSet(userSet);
	
	[self updateUsersForDisk:disk];
	
	if (openedDisk)
		[disk closeDisk];
	
	return err;
}

- (void)addUserCancelled:(id)sender
{
}

- (void)removeUserWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	CPGPdiskUser*		user = (CPGPdiskUser*) contextInfo;
	CPGPdiskObject*		disk;
	PGPContextRef		context;
	PGPdiskUserSetRef	userSet;
	PGPdiskUserRef		newUserRef;
	PGPdiskUserRef		adminUser;
	PGPError			err = kPGPError_NoErr;
	char*				passphrase;
	BOOL				openedDisk = NO;
	int					userIndex = 1;
	
	disk = [self selectedDisk];
	
	userIndex = [mUsers indexOfObject:user];
	
	pgpAssert(disk != nil);
	
	/*
	NSCancelButton signifies that the user pressed the Remove button.
	The CPGPdiskUser object is stowed away in contextInfo.
	*/
	if ((returnCode == NSCancelButton) && disk && user)
	{
		if (![disk diskRef])
		{
			[disk openDiskWithContext:[self pgpDiskContext]];
			openedDisk = YES;
		}
		
		context = [self pgpContext];
		
		[self updateUsersForDisk:disk];
		userSet = [disk userSet];
		newUserRef = [CPGPdiskUser findUserAtIndex:userIndex inUserSet:userSet];
		adminUser = [CPGPdiskUser findAdminInUserSet:userSet];
		
		if (IsPGPError(err = [disk getAdminPassphrase:&passphrase]))
		{
			if (passphrase)
				PGPFreeData(passphrase);
		}
		else
		{
			if (IsPGPError(err = PGPdiskUserSetRemoveUser(userSet, newUserRef)))
				PGPLog((@"Failed to remove user (err=%d)", err));
			else
			{
				err = PGPdiskDiskImportUserSet([disk diskRef], passphrase, userSet, adminUser);
				
				if (IsPGPError(err))
					PGPLog((@"Failed to import modified user set (err=%d)", err));
			}
			
			PGPFreeData(passphrase);
		}
		
		if (openedDisk)
			[disk closeDisk];
		
		[self updateUsersForDisk:disk];
	}
}

- (void)disableUserWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	CPGPdiskUser*		user = (CPGPdiskUser*) contextInfo;
	CPGPdiskObject*		disk = [self selectedDisk];
	PGPdiskUserSetRef	userSet;
	PGPdiskUserRef		userRef;
	PGPError			err = kPGPError_NoErr;
	char*				passphrase;
	BOOL				openedDisk = NO;
	
	/*
	NSCancelButton signifies that the user pressed the Remove button.
	The CPGPdiskUser object is stowed away in contextInfo.
	*/
	if ((returnCode == NSCancelButton) && disk && user)
	{
		if (![disk diskRef])
		{
			[disk openDiskWithContext:[self pgpDiskContext]];
			openedDisk = YES;
		}
		
		[self updateUsersForDisk:disk];
		
		[self setDiskEngineIsBusy:YES];
			
		if (IsPGPError(err = [disk getAdminPassphrase:&passphrase]))
		{
			if (passphrase)
				PGPFreeData(passphrase);
			
			[self displayUserModAlertWithError:err forDisk:disk];
		}
		else
		{
			userSet = [disk userSet];
			userRef = [CPGPdiskUser findUserAtIndex:[mUserTable selectedRow] inUserSet:userSet];
			user = [mUsers objectAtIndex:[mUserTable selectedRow]];
			
			if (userRef)
			{
				if (IsPGPError(err = PGPdiskSetUserBooleanProp(userRef, kPGPdiskUserProp_IsDisabled, ![user isDisabled])))
					PGPLog((@"Failed to disable/enable user (err=%d)", err));
				else
				{
					if (IsPGPError(err = [disk importUserSet:userSet adminPassphrase:passphrase adminUser:nil]))
						PGPLog((@"Failed to set readonly attribute for user"));
				}
			}
			
			PGPFreeData(passphrase);
		}
		
		[self setDiskEngineIsBusy:NO];
		
		if (openedDisk)
			[disk closeDisk];
		
		[self updateUsersForDisk:disk];
	}
}

@end
