/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPdiskController.m,v 1.113 2002/11/09 03:05:39 jeffc Exp $
____________________________________________________________________________*/

#import "PGPdiskController.h"
#import "PGPdiskPreferences.h"
#import "CPGPdiskReEncryption.h"
#import "CPGPdiskConversion.h"
#import "pgpDiskEngine.h"

#import <PGP/PGP.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPui/CBrowserCell.h>
#import <pwd.h>
#import <unistd.h>
#import <sys/stat.h>
#import <sys/param.h>
#import <sys/mount.h>

#define kPGPdiskString_ResumeReEncryption			LSPD(@"Resume Re-Encryption", @"Resume re-encryption caption")
#define kPGPdiskString_ResumeReEncryptionMsg		LSPD(@"The selected disk has not been completely re-encrypted. In order to use this disk, the re-encryption process must finish.", @"Resume re-encryption message")
#define kPGPdiskString_OS9Conversion				LSPD(@"Conversion Needed", @"Conversion needed caption")
#define kPGPdiskString_OS9ConversionMsg				LSPD(@"The PGPdisk \"%@\" needs to be converted to a newer format. Click the Convert button below to choose where your converted PGPdisk will be saved. This operation will not destroy any existing data.", @"Conversion needed message")
#define kPGPdiskString_MountFailure					LSPD(@"Mount failure", @"Mount failure caption")
#define kPGPdiskString_UserModFailure				LSPD(@"Failed to modify user set", @"User mod failure caption")
#define kPGPdiskString_ReEncryptFailure				LSPD(@"Failed to re-encrypt disk", @"ReEncryption failure caption")
#define kPGPdiskString_ConversionFailure			LSPD(@"Failed to convert disk", @"Conversion failure caption")
#define kPGPdiskString_BadInstall					LSPD(@"PGPdisk was unable to mount the specified disk because PGP was not installed correctly. It is recommended that you uninstall PGP, and then reinstall.", @"Bad installation message")
#define kPGPdiskString_MountKeyringsMissing			LSPD(@"PGPdisk was unable to mount the specified disk because one or more public key users could not be verified. It is recommended that you verify that your keyrings are set correctly in the Preferences window.", @"")
#define kPGPdiskString_UserModKeyringsMissing		LSPD(@"PGPdisk was unable to modify the user list for the specified disk because one or more public key users could not be verified. It is recommended that you verify that your keyrings are set correctly in the Preferences window.", @"")
#define kPGPdiskString_ReEncryptKeyringsMissing		LSPD(@"PGPdisk was unable to re-encrypt the specified disk because one or more public key users could not be verified. It is recommended that you verify that your keyrings are set correctly in the Preferences window.", @"")
#define kPGPdiskString_ConversionKeyringsMissing	LSPD(@"PGPdisk was unable to convert the specified disk because one of more public key users could not be verified. It is recommended that you verify that your keyrings are set correctly in the Preferences window.", @"")
#define kPGPdiskString_OpenFailed					LSPD(@"Open failure", @"Open failure caption")
#define kPGPdiskString_GenericOpenFailed			LSPD(@"PGPdisk was unable to open the specified disk for an unknown reason. You will not be able to use this disk.", @"Generic open failure message")
#define kPGPdiskString_CantOpenFile					LSPD(@"PGPdisk was unable to open the disk \"%@\" for an unknown reason. If this disk is being accessed from a remote share, please verify that the disk isn't mounted on any other computers.", @"Can't open file message")
#define kPGPdiskString_DiskCorrupted				LSPD(@"PGPdisk was unable to open the disk \"%@\" because it has been corrupted. It is recommended that you restore the disk image from a backup (you did remember to backup didn't you?).", @"Disk corrupt message")
#define kPGPdiskString_EOF							LSPD(@"PGPdisk was unable to open the disk \"%@\" because the end-of-file marker was reached before it was expected.", @"EOF message")
#define kPGPdiskString_FSMaxSizeExceeded			LSPD(@"Disk exceeds volume's file size limit", @"Size exceeded caption")
#define kPGPdiskString_FSHFSSizeExceededMsg			LSPD(@"The converted disk image size will exceed the HFS maximum file size. Please select a location on a non-HFS volume.", @"HFS conversion size exceeded")
#define kPGPdiskString_FSUFSSizeExceededMsg			LSPD(@"The converted disk image size will exceed the UFS maximum file size. Please select a location on a non-UFS volume.", @"UFS conversion size exceeded")
#define kPGPdiskString_SpaceNotAvailable			LSPD(@"Not enough available space", @"No space caption")
#define kPGPdiskString_SpaceNotAvailableMsg			LSPD(@"The specified disk location does not have enough available disk space to save the converted PGPdisk. Please choose another location.", @"No space message")
#define kPGPdiskString_UnableToUnmount				LSPD(@"Unable to unmount", @"Can't unmount caption")
#define kPGPdiskString_UnableToUnmountMsg			LSPD(@"PGPdisk was unable to unmount the specified disk because one or more files are open on the disk.", @"Can't unmount message")
#define kPGPdiskString_SelectDisk					LSPD(@"<Please select a PGPdisk>", @"")
#define kPGPdiskString_PGPdiskImages				LSPD(@"PGPdisk Images", @"")
#define kPGPdiskString_SaveFrameName				@"PGPdisk Main Window"

#define kPGPdiskPopUpNew							5000
#define kPGPdiskPopUpOpen							5001

static PGPdiskController*	sSharedInstance = nil;
static CPGPdiskObject*		sConversionDisk = nil;

static NSString* sFormatSizeString(PGPUInt64 size)
{
	NSMutableString*	string = [[[NSMutableString alloc] init] autorelease];
	char				tempString[256];
	char*				pStr;
	unsigned			length;
	int					i;
	
	snprintf(tempString, sizeof(tempString) - 1, "%qu", size);
	length = strlen(tempString);
	
	if (length % 3)
	{
		pStr = tempString;
		for (i = 0; i < (length % 3); ++i)
			[string appendFormat:@"%c", *pStr++];
		
		if (length > 3) 
			[string appendFormat:@","];
	}
	
	pStr = &tempString[length % 3];
	length -= (length % 3);
	
	for (i = 0; i < length; ++i)
	{
		if (!i || (i % 3))
			[string appendFormat:@"%c", *pStr++];
		else
			[string appendFormat:@",%c", *pStr++];
	}	
	
	return string;
}

static NSString* sGetFormattedSizeString(PGPUInt64 size)
{
	NSMutableString*		string = [[[NSMutableString alloc] init] autorelease];
		
	if ((size >= 0x00000400) && (size < 0x00100000))	/* At least 1K and less than 1MB */
	{
		PGPUInt64		kbSize = size / 0x00000400;
		
		[string appendFormat:@"%@ %@ ", sFormatSizeString(kbSize), LSPD(@"KB", "Kilobytes")];
	}
	else if ((size >= 0x00100000) && (size < 0x40000000))
	{
		PGPUInt64		mbSize = size / 0x00100000;
		
		[string appendFormat:@"%@ %@ ", sFormatSizeString(mbSize), LSPD(@"MB", "Megabytes")];
	}
	else if (size >= 0x40000000)
	{
		PGPUInt64		gbSize = size / 0x40000000;
		
		[string appendFormat:@"%@ %@ ", sFormatSizeString(gbSize), LSPD(@"GB", "Gigabytes")];
	}
	
	[string appendFormat:@"(%@ %@)", sFormatSizeString(size), LSPD(@"bytes", "Bytes")];
	
	return string;
}

static void sigPipeHandler(PGPInt32 signal)
{
	PGPLog((@"Caught SIGPIPE\n"));
}

@implementation PGPdiskController

+ (id)sharedController
{
	return sSharedInstance;
}

+ (NSString*)identifier
{
	return @"PGPdisk.bundle";
}

- (id)init
{
	if (!mInited)
	{
		sSharedInstance = self;
		
		[super init];

		mBundle = [NSBundle bundleForClass:[self class]];
	
		mDisks = [[NSMutableArray alloc] init];
        mMountedDisks = [[NSMutableArray alloc] init];
        
		[self launchDiskEngine];
		
		[[[NSWorkspace sharedWorkspace] notificationCenter]
				addObserver:self
				selector:@selector(finderUnmountedVolume:)
				name:NSWorkspaceDidUnmountNotification
				object:nil];
		
		[[[NSWorkspace sharedWorkspace] notificationCenter]
				addObserver:self
				selector:@selector(finderUnmountedVolume:)	/* Use same selector as unmount notification */
				name:NSWorkspaceDidMountNotification
				object:nil];
		
		[self registerForPrefNotifications];
		
		mInited = YES;
	}
	
	if (!mDiskPreferences)
		mDiskPreferences = [[PGPdiskPreferences alloc] initWithController:self];
	
	if (!mPGPContext && mController)
		mPGPContext = [mController pgpContext];
	
	if (!mDiskContext)
	{
		[self createDiskContext];
		
		[self setPassphraseCaching];
		[self setGlobalDiskTimeout:[self globalDiskTimeout]];
        [self setForcedUnmount:[self forcedUnmount]];
		[self setUnmountOnLogout:[self unmountOnLogout]];
		
        if ([self unmountOnSleep])
        {
            [self setUnmountOnSleep:TRUE];
            [self setPreventSleep:[self preventSleep]];
        }
        else
            [self setUnmountOnSleep:FALSE];
            
		[self setADK:[self adk]];
		[self updateKeyrings];
	}
	
	if (!mLoadedNib)
		[self loadNib:kPGPdiskNib];
	
    return self;
}

- (id)initWithController:(id)controller
{
    mController = controller;

    return [self init];
}

- (id)loadNib:(NSString*)nibFile
{
    NSTableColumn*		usericonColumn;
	NSTableColumn*		usernameColumn;
	NSTableColumn*		readonlyColumn;
	NSTableColumn*		adminColumn;
    NSImageCell*		usericonCell;
    NSTextFieldCell*	tfCell;
	CImageButtonCell*	readonlyColumnCell;
	CImageButtonCell*	adminColumnCell;
	
	if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	
    [mDiskMenu setTitle:kPGPdiskMenu_Disk];
	[mDiskMenu setAutoenablesItems:YES];

	mMountImage = [[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:@"mountpgpdisk36.tiff"]];
	mUnmountImage = [[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:@"unmountpgpdisk36.tiff"]];	
	mDiskIcon = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForImageResource:kPGPdiskDiskCellIcon]];;
	[mDiskIcon setScalesWhenResized:YES];
	[mDiskIcon setSize:NSMakeSize(16, 16)];
	mOS9DiskIcon = [[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPdiskOS9DiskCellIcon]];
	
	if (!mIntFormatter)
	{
		mIntFormatter = [[CIntegerFormatter alloc] init];
		[mIntFormatter setFormat:@"##0"];
		[mIntFormatter setMinimum:[NSDecimalNumber one]];
	}
	
	[mCustomTimeout setFormatter:mIntFormatter];
	
	[mTabView setDelegate:self];
	
	mUserTableSource = [[CPGPdiskUserTable alloc] initWithController:self];
	[mUserTableSource setTable:mUserTable];
	[mUserTable setDataSource:mUserTableSource];
	[mUserTable setDelegate:mUserTableSource];
	
    if (usericonColumn = [mUserTable tableColumnWithIdentifier:kPGPdiskUserIconColumnIdentifier])
    {
        if (usericonCell = [[[NSImageCell alloc] init] autorelease])
			[usericonColumn setDataCell:usericonCell];
    }
    
	if (usernameColumn = [mUserTable tableColumnWithIdentifier:kPGPdiskUsernameColumnIdentifier])
	{
        if (tfCell = [[[NSTextFieldCell alloc] init] autorelease])
        {
            [tfCell setEditable:YES];
            [tfCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
            [usernameColumn setDataCell:tfCell];
        }
	}

	if (readonlyColumn = [mUserTable tableColumnWithIdentifier:kPGPdiskReadOnlyColumnIdentifier])
	{
		if (readonlyColumnCell = [[CImageButtonCell alloc] init])
		{
			[readonlyColumnCell setButtonType:NSPushOnPushOffButton];
			[readonlyColumnCell setBezelStyle:NSShadowlessSquareBezelStyle];
			[readonlyColumnCell setGradientType:NSGradientNone];
			[readonlyColumnCell setBordered:NO];
			[readonlyColumnCell setHighlightsBy:NSContentsCellMask];
			[readonlyColumnCell setAlternateImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotSelect]];
			[readonlyColumnCell setAction:@selector(readonlyPressed:)];
			[readonlyColumnCell setTarget:self];
			[readonlyColumn setDataCell:readonlyColumnCell];
			[readonlyColumnCell release];
		}
	}

	if (adminColumn = [mUserTable tableColumnWithIdentifier:kPGPdiskAdminColumnIdentifier])
	{
		if (adminColumnCell = [[CImageButtonCell alloc] init])
		{
			[adminColumnCell setButtonType:NSPushOnPushOffButton];
			[adminColumnCell setBezelStyle:NSShadowlessSquareBezelStyle];
			[adminColumnCell setGradientType:NSGradientNone];
			[adminColumnCell setBordered:NO];
			[adminColumnCell setHighlightsBy:NSContentsCellMask];
			[adminColumnCell setAlternateImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotSelect]];
			[adminColumnCell setAction:@selector(adminPressed:)];
			[adminColumnCell setTarget:self];
			[adminColumn setDataCell:adminColumnCell];
			[adminColumnCell release];
		}
	}
	
	[mDiskPopup setAutoenablesItems:YES];
	
	[self populateDiskArray];
	[self setupDiskPopup];
	[self diskPopupSelectionChanged:nil];
	[self installDiskCheckerTimer];
	
	[mWindow setDelegate:self];
	[mWindow setFrameUsingName:kPGPdiskString_SaveFrameName];
	
	mLastResponder = mDiskPopup;
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)prefsController
{
	return mDiskPreferences;
}

- (NSWindow*)window
{
	return mWindow;
}

- (NSImage*)menuImage
{
	NSImage*	image;
	
	image = [[[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPdiskToolbarItemImage]] autorelease];
	[image setScalesWhenResized:YES];
	[image setSize:NSMakeSize(16, 16)];
	
	return image;
}

- (NSMenu*)dockMenu
{
	return mDiskDockMenu;
}

- (NSMenuItem*)newMenuItem
{
	NSMenuItem*		item;
	
	item = [[NSMenuItem alloc] initWithTitle:kPGPdiskString_NewPGPdisk action:@selector(newDisk:) keyEquivalent:@"n"];
	[item setKeyEquivalentModifierMask:NSCommandKeyMask];
	[item setTarget:self];
	[item autorelease];
	
	return item;
}

- (void)activate:(BOOL)passive
{
	if (!passive)
		[mWindow makeKeyAndOrderFront:nil];
}

- (void)addMenus
{
	NSMenu*		fileMenu;
	NSMenu*		diskMenu;
	
	[mController addMenu:mDiskMenu withIdentifier:[[self class] identifier] atIndex:[[mController menuItems] count] - 2];
		
	diskMenu = [[[NSApp mainMenu] itemWithTitle:[mDiskMenu title]] submenu];
	fileMenu = [[[NSApp mainMenu] itemWithTitle:kPGPdiskMenu_File] submenu];
	
	mDiskItemDisable = [diskMenu itemWithTitle:kPGPdiskMenuItem_DisableUser];
	
	if (!mDiskItemDisable)
		mDiskItemDisable = [diskMenu itemWithTitle:kPGPdiskMenuItem_EnableUser];
	
	[mUserTableSource tableViewSelectionDidChange:nil];
}

- (void)handleDocument:(NSString*)docFile ofType:(NSString*)docType
{
	if (!(docFile && [docFile length] && docType && [docType isEqualToString:kPGPDiskImageFileType]))
		return;
	
	[self openDisk:docFile mount:YES];
	
	[mController finishedHandlingDocument:docFile];
}

- (void)openDisk:(NSString*)path mount:(BOOL)flag
{
	CPGPdiskObject*		disk = nil;
	PGPError			err = kPGPError_NoErr;
	
	if (!(path && [path length]))
		return;
	
	/*
	If we already have a disk opened at the specified path, select that disk in
	the popup. Otherwise, open the disk and select the new entry in the popup.
	*/

	if (disk = [self diskWithPath:path])
		[self selectDisk:disk];
	else
	{
		if (disk = [CPGPdiskObject diskAtPath:path])
		{
			[disk setController:self];
			
			err = [disk openDiskWithContext:mDiskContext];
			
			if (IsPGPError(err))
				[self displayOpenFailedAlertWithError:err forDisk:disk];
			else
			{
				[disk closeDisk];
				
				[self addDisk:disk];
				[self updateDiskPopup];
				[self selectDisk:disk];
			}
		}
	}
        
	if (IsntPGPError(err) && disk && flag)
		[self mountDisk:disk];
}

- (id)controller
{
	return mController;
}

- (void)setController:(id)controller
{
	if (controller)
		mController = controller;
}

- (void)setupKeyViewLoop
{
	//NSString*		tabViewItemLabel = [[mTabView selectedTabViewItem] label];
	
	/*
	[mNewDiskButton setNextKeyView:mMountButton];
	[mMountButton setNextKeyView:mDiskTable];
	[mDiskTable setNextKeyView:mTabView];
	
	if ([tabViewItemLabel isEqualToString:LSPD(@"General Info", @"")])
	{
		[mTabView setNextKeyView:mReveal];
		[mReveal setNextKeyView:mCipher];
		[mCipher setNextKeyView:mReEncrypt];
		[mReEncrypt setNextKeyView:mNewDiskButton];
	}
	else if ([tabViewItemLabel isEqualToString:LSPD(@"Mounting Options", @"")])
	{
		[mTabView setNextKeyView:mOverrideInactivityTimer];
		[mOverrideInactivityTimer setNextKeyView:[mUseCustomTimeout controlView]];
		[[mUseCustomTimeout controlView] setNextKeyView:mStepper];
		[mStepper setNextKeyView:[mDisableTimer controlView]];
		[[mDisableTimer controlView] setNextKeyView:mOverrideEjectOnLogout];
		[mOverrideEjectOnLogout setNextKeyView:mEjectOnLogout];
		[mEjectOnLogout setNextKeyView:mNewDiskButton];
	}
	else if ([tabViewItemLabel isEqualToString:LSPD(@"Users", @"")])
	{
		[mTabView setNextKeyView:mUserTable];
		[mUserTable setNextKeyView:mNewDiskButton];
	}
	*/
}

- (BOOL)keyViewIsValid:(NSResponder*)keyView
{
	/*
	return ((keyView == mNewDiskButton) ||
			(keyView == mMountButton) ||
			(keyView == mDiskTable) ||
			(keyView == mUserTable));
	*/
	return YES;
}

- (void)registerForPrefNotifications
{
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];
	
	[center addObserver:self
			selector:@selector(passphraseCacheTypeDidChange:)
			name:kPGPclPrefNotification_CacheType
			object:nil];
	
	[center addObserver:self
			selector:@selector(passphraseSecondsDidChange:)
			name:kPGPclPrefNotification_CacheSeconds
			object:nil];
	
	[center addObserver:self
			selector:@selector(passphraseShareCacheDidChange:)
			name:kPGPclPrefNotification_ShareCache
			object:nil];
	
	[center addObserver:self
			selector:@selector(allowForcibleUnmountDidChange:)
			name:kPGPclPrefNotification_ForcibleUnmounts
			object:nil];
	
	[center addObserver:self
			selector:@selector(unmountOnInactivityDidChange:)
			name:kPGPclPrefNotification_AutoUnmount
			object:nil];
	
	[center addObserver:self
			selector:@selector(autoUnmountTimeoutDidChange:)
			name:kPGPclPrefNotification_UnmountTimeout
			object:nil];
	
	[center addObserver:self
			selector:@selector(autoUnmountOnLogoutDidChange:)
			name:kPGPclPrefNotification_UnmountLogout
			object:nil];
	
	[center addObserver:self
			selector:@selector(autoUnmountOnSleepDidChange:)
			name:kPGPclPrefNotification_UnmountSleep
			object:nil];
	
	[center addObserver:self
			selector:@selector(preventSleepDidChange:)
			name:kPGPclPrefNotification_PreventSleep
			object:nil];
}

- (BOOL)releaseIfReadyToShutdown
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];
	
	[mWindow saveFrameUsingName:kPGPdiskString_SaveFrameName];
	
	if (mDiskIcon)
	{
		[mDiskIcon release];
		mDiskIcon = nil;
	}
	
	if (mOS9DiskIcon)
	{
		[mOS9DiskIcon release];
		mOS9DiskIcon = nil;
	}
	
	if (mEngineTimer)
	{
		[mEngineTimer invalidate];
		mEngineTimer = nil;
	}

    if (mDiskCheckerTimer)
    {
        [mDiskCheckerTimer invalidate];
        mDiskCheckerTimer = nil;
    }
	
	if (mIntFormatter)
		[mIntFormatter release];
	
	if (mBundle)
		[mBundle release];
    
    if (mMountedDisks)
        [mMountedDisks release];
	
	[self saveKnownDisksList];
	
	if (mDiskContext)
	{
		PGPdiskFreeContext(mDiskContext);
		mDiskContext = NULL;
	}

	if (mDiskEngineTask)
	{
		[mDiskEngineTask release];
		mDiskEngineTask = nil;
	}
	
	return YES;
}

- (void)keyDBChanged
{
	PGPLog((@"PGPdisk: keyDBChanged"));
}

- (void)passphraseCacheChanged
{
	PGPLog((@"PGPdisk: passphraseCacheChanged"));
}

- (NSMutableArray*)disks
{
	return mDisks;
}

- (void)setDisks:(NSMutableArray*)disks
{
	if (mDisks)
		[mDisks release];

	if (disks)
		mDisks = [disks retain];
	else
		mDisks = nil;
}

- (NSMutableArray*)users
{
	return mUsers;
}

- (NSMutableArray*)mountedDisks
{
	return mMountedDisks;
}

- (PGPContextRef)pgpContext
{
	if (!mPGPContext)
		mPGPContext = [mController pgpContext];
	
	return mPGPContext;
}

- (PGPdiskContextRef)pgpDiskContext
{
	if (!mDiskContext)
		[self createDiskContext];
	
	return mDiskContext;
}

- (PGPPrefRef)pgpPrefRef
{
	if (mController)
		return [mController pgpPrefRef];

	return NULL;
}

- (PGPKeyDBRef)pgpKeyDB
{
	return [mController mainKeyDB];
}

- (char*)pgpPubRingPath
{
	return mPubKeyringPath;
}

- (char*)pgpSecRingPath
{
	return mSecKeyringPath;
}

- (NSTableView*)userTable
{
	return mUserTable;
}

- (CPGPdiskUserTable*)userTableSource
{
    return mUserTableSource;
}

- (NSMenuItem*)disableMenuItem
{
	return mDiskItemDisable;
}

- (NSMenuItem*)changePassphraseMenuItem
{
	return mDiskItemChangePassphrase;
}

- (void)setAddUserEnabled:(BOOL)flag
{
	mAddUser = flag;
}

- (void)setRemoveUserEnabled:(BOOL)flag
{
	mRemoveUser = flag;
}

- (IBAction)openDisk:(id)sender
{
	NSOpenPanel*		openPanel = [NSOpenPanel openPanel];
	NSMutableArray*		types = [[[NSMutableArray alloc] init] autorelease];
	
	[NSApp activateIgnoringOtherApps:YES];
	
	[types addObject:NSFileTypeForHFSTypeCode(kPGPdiskFileOSType)];
	[types addObject:@"pgd"];
	
	[openPanel setPrompt:kPGPdiskString_MountDisk];
	[openPanel setAllowsMultipleSelection:NO];
	
	if ([mWindow isVisible])
	{
		[openPanel beginSheetForDirectory:NSHomeDirectory()
				   file:nil
				   types:types
				   modalForWindow:mWindow
				   modalDelegate:self
				   didEndSelector:@selector(openSheetDidEnd:returnCode:contextInfo:)
				   contextInfo:openPanel];
	}
	else
	{
		if ([openPanel runModalForDirectory:NSHomeDirectory() file:nil types:types] == NSOKButton)
			[self handleDocument:[[openPanel filenames] objectAtIndex:0] ofType:kPGPDiskImageFileType];
	}
}

- (IBAction)convert:(id)sender
{
	PGPError			err = kPGPError_NoErr;
    NSOpenPanel*		openPanel = [NSOpenPanel openPanel];
    NSMutableArray*		types = [[[NSMutableArray alloc] init] autorelease];
    CPGPdiskObject*		disk = [self selectedDisk];
    
    if (disk && (sender != mDiskPopup))
        [self convertDisk:disk displayAlert:NO];
    else
    {
        [types addObject:NSFileTypeForHFSTypeCode(kPGPdiskFileOSType)];
        
		[openPanel setPrompt:kPGPdiskString_Convert];
		[openPanel setAllowsMultipleSelection:NO];
		
		if ([mWindow isVisible])
		{
			[openPanel beginSheetForDirectory:NSHomeDirectory()
					   file:nil
					   types:types
					   modalForWindow:mWindow
					   modalDelegate:self
					   didEndSelector:@selector(convertSheetDidEnd:returnCode:contextInfo:)
					   contextInfo:openPanel];
		}
		else
		{
			if ([openPanel runModalForDirectory:NSHomeDirectory() file:nil types:types] == NSOKButton)
			{
				disk = [[CPGPdiskObject diskAtPath:[[openPanel filenames] objectAtIndex:0]] autorelease];
				[disk setController:self];
				err = [disk openDiskWithContext:mDiskContext];
				
				if (IsPGPError(err))
					[self displayOpenFailedAlertWithError:err forDisk:disk];
				else
				{
					[disk closeDisk];
					[self convertDisk:disk displayAlert:NO];
				}
			}
		}
    }
}

- (IBAction)reveal:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	
	if (disk)
		[[NSWorkspace sharedWorkspace] selectFile:[disk diskPath] inFileViewerRootedAtPath:@""];
}

- (IBAction)cipherChanged:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	NSNumber*			cipher;
	
	if (disk)
	{
		cipher = [NSNumber numberWithInt:[[mCipher selectedItem] tag]];
		
		[self dimTabViewContents:YES];
		[self reEncryptDisk:disk withCipher:cipher];
	}
}

- (IBAction)reEncrypt:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
    NSNumber*			cipher;
	
	if (!disk)
		return;
	
	[self dimTabViewContents:YES];
    
	if ([disk diskNeedsReEncryption])
		[self continueReEncryptingDisk:disk];
	else
	{
		cipher = [NSNumber numberWithInt:[[mCipher selectedItem] tag]];
		[self reEncryptDisk:disk withCipher:cipher];
	}
}

- (IBAction)overrideGlobalInactivityTimer:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	BOOL				flag = NO;
	
	if (!disk)
		return;
	
	if (![mOverrideInactivityTimer isEnabled] || ([mOverrideInactivityTimer state] == NSOffState))
		flag = NO;
	else if ([mOverrideInactivityTimer state] == NSOnState)
		flag = YES;
	
	[mUseCustomTimeout setEnabled:flag];
	[mDisableTimer setEnabled:flag];
	
	if ([mOverrideInactivityTimer state] == NSOnState)
	{
		if ([mOverrideInactivityTimer isEnabled])
			[disk setUsesCustomTimeout:YES];
		
		if ([mUseCustomTimeout state] == NSOnState)
			[self useCustomTimeout:nil];
		else
			[self disableTimer:nil];
	}
	else
	{
		[disk setUsesCustomTimeout:NO];
		
		[mCustomTimeout setEnabled:NO];
		[mCustomTimeout setTextColor:[NSColor disabledControlTextColor]];
		[mStepper setEnabled:NO];
	}
}

- (IBAction)disableTimer:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	
	if (!disk)
		return;
	
	[mUseCustomTimeout setState:NSOffState];
	[mDisableTimer setState:NSOnState];

	[mCustomTimeout setEnabled:NO];
	[mCustomTimeout setTextColor:[NSColor disabledControlTextColor]];
	[mStepper setEnabled:NO];
	
	if ([mDisableTimer isEnabled])
		[disk setCustomTimeout:0];
}

- (IBAction)useCustomTimeout:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	
	if (!disk)
		return;
	
	[mUseCustomTimeout setState:NSOnState];
	[mDisableTimer setState:NSOffState];
	
	if ([mUseCustomTimeout isEnabled])
	{
		[mCustomTimeout setEnabled:YES];
		[mCustomTimeout setTextColor:[NSColor controlTextColor]];
		[mStepper setEnabled:YES];
		
		if ([mCustomTimeout intValue] == 0)
			[mCustomTimeout setStringValue:[[NSNumber numberWithInt:kPGPdiskDefaultUnmountTimeout] stringValue]];
		
		[disk setCustomTimeout:[mCustomTimeout intValue]];
	}
}

- (IBAction)stepperPressed:(id)sender
{
	CPGPdiskObject*		disk = [self selectedDisk];
	int					increment;
    int					currentSize;
    int					stepperValue;
    
	if (!disk)
		return;
	
	currentSize = [mCustomTimeout intValue];
	stepperValue = [mStepper intValue];

	if (stepperValue < 0)
		increment = -1;
	else
		increment = 1;

	[mStepper setIntValue:0];

	if ((currentSize + increment) < 1)
		[mCustomTimeout setIntValue:1];
	else if ((currentSize + increment) <= 999)
		[mCustomTimeout setIntValue:(currentSize + increment)];

	[disk setCustomTimeout:[mCustomTimeout intValue]];
	
	[mCustomTimeout displayIfNeeded];
}

- (void)updateTabViewForDisk:(CPGPdiskObject*)disk
{
	BOOL		diskIsBusy = NO;
	BOOL		openedDisk = NO;
	
	if (![disk diskRef])
	{
		[disk openDiskWithContext:mDiskContext];
		openedDisk = YES;
	}
	
	if ([self diskEngineIsBusy])
	{
		if (openedDisk)
			[disk closeDisk];
		
		return;
	}
	
	if (!disk)
		[self dimTabViewContents:YES];	/* Disable the tabview's controls */
	else
	{
		diskIsBusy = ([disk diskIsBusy] || [disk diskNeedsReEncryption] || ([disk diskFormat] == kPGPdiskDiskFormat_OS9));
		
		if (diskIsBusy || ![disk isWritable])
			[self dimTabViewContents:YES];
		else
			[self dimTabViewContents:NO];
		
		[mReveal setEnabled:YES];
	}
	
	[self setLocationForDisk:disk];
	[self setSizeForDisk:disk];
	[self setCipherForDisk:disk];

	if (disk && [disk isWritable])
	{
		if (![self diskEngineIsBusy] &&
			![disk diskIsBusy] &&
			![disk isMounted] &&
			([disk diskFormat] != kPGPdiskDiskFormat_OS9))
		{
			[mReEncrypt setEnabled:YES];
			
			if (![disk diskNeedsReEncryption])
				[mCipher setEnabled:YES];
		}
		else
		{
			[mReEncrypt setEnabled:NO];
			[mCipher setEnabled:NO];
		}
	}
	else
	{
		[mReEncrypt setEnabled:NO];
		[mCipher setEnabled:NO];
	}
	
	/* Setup the mount options tab */
	[self setAutoUnmountOptionsForDisk:disk];
	
	if (openedDisk)
		[disk closeDisk];
}

- (void)dimTabViewContents:(BOOL)flag
{
	[mReveal setEnabled:!flag];
	[mCipher setEnabled:!flag];
	[mReEncrypt setEnabled:!flag];
	[mOverrideInactivityTimer setEnabled:!flag];
	[mUseCustomTimeout setEnabled:!flag];
	[mDisableTimer setEnabled:!flag];
	[mCustomTimeout setEnabled:!flag];
	[mStepper setEnabled:!flag];
}

- (void)openSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	NSOpenPanel*	openPanel = (NSOpenPanel*) contextInfo;
	
	[sheet orderOut:nil];
	
	if (returnCode == NSOKButton)
	{
		if (openPanel)
			[self handleDocument:[[openPanel filenames] objectAtIndex:0] ofType:kPGPDiskImageFileType];
	}
}

- (void)convertSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	PGPError		err;
	NSOpenPanel*	openPanel = (NSOpenPanel*) contextInfo;
	CPGPdiskObject*	disk;
	
	[sheet orderOut:nil];
	
	if (returnCode == NSOKButton)
	{
		if (openPanel)
		{
			disk = [[[CPGPdiskObject alloc] initWithPath:[[openPanel filenames] objectAtIndex:0]] autorelease];
			[disk setController:self];
			err = [disk openDiskWithContext:mDiskContext];
			
			if (IsPGPError(err))
				[self displayOpenFailedAlertWithError:err forDisk:disk];
			else
			{
				[disk closeDisk];
				[self convertDisk:disk displayAlert:NO];
			}
		}
	}
}

- (void)setLocationForDisk:(CPGPdiskObject*)disk
{
	if (disk)
		[mLocation setStringValue:[disk diskPath]];
	else
		[mLocation setStringValue:kPGPdiskString_SelectDisk];
}

- (void)setSizeForDisk:(CPGPdiskObject*)disk
{
	NSDictionary*		fileAttributes;
	NSString*			sizeString;
	PGPUInt64			size;
	
	if (!disk)
	{
		[mSize setStringValue:kPGPdiskString_SelectDisk];
		return;
	}
	
	fileAttributes = [[NSFileManager defaultManager] fileAttributesAtPath:[disk diskPath] traverseLink:YES];
	
	size = [[fileAttributes objectForKey:NSFileSize] unsignedLongLongValue];
	sizeString = sGetFormattedSizeString(size);
	
	[mSize setStringValue:sizeString];
}

- (void)setCipherForDisk:(CPGPdiskObject*)disk
{
	PGPdiskEncryptionAlgorithm	algorithm = kPGPdiskInvalidAlgorithm;
	
	if (!disk)
		return;
	
	algorithm = [disk diskAlgorithm];

	if (algorithm == kPGPdiskCAST5Algorithm)
		[mCipher selectItemAtIndex:[mCipher indexOfItemWithTag:kPGPdiskCAST5Algorithm]];

	else if (algorithm == kPGPdiskAESAlgorithm)
		[mCipher selectItemAtIndex:[mCipher indexOfItemWithTag:kPGPdiskAESAlgorithm]];
}

- (void)setAutoUnmountOptionsForDisk:(CPGPdiskObject*)disk
{
	PGPUInt32		timeout;
	
	if (!disk)
	{
		[mCustomTimeout setTextColor:[NSColor disabledControlTextColor]];
		
		return;
	}
		
	if ([disk usesCustomTimeout])	/* Disk has a custom timeout */
	{
		[mOverrideInactivityTimer setState:NSOnState];
		
		timeout = [disk customTimeout];
		
		if (!timeout)
		{
			[mUseCustomTimeout setState:NSOffState];
			[mDisableTimer setState:NSOnState];
		}
		else
		{
			[mUseCustomTimeout setState:NSOnState];
			[mDisableTimer setState:NSOffState];
			[mCustomTimeout setStringValue:[[NSNumber numberWithInt:[disk customTimeout]] stringValue]];
		}
	}
	else	/* Use global timeout setting */
	{
		[mOverrideInactivityTimer setState:NSOffState];
		
		[mUseCustomTimeout setState:NSOnState];
		[mDisableTimer setState:NSOffState];
		[mCustomTimeout setStringValue:[[NSNumber numberWithInt:kPGPdiskDefaultUnmountTimeout] stringValue]];
	}
	
	[self overrideGlobalInactivityTimer:nil];	/* Enables/Disables controls */
}

- (void)populateDiskArray
{
	PGPError			err = kPGPError_NoErr;
	NSUserDefaults*		defaults = [NSUserDefaults standardUserDefaults];
	NSDictionary*		dict = [defaults persistentDomainForName:kPGPPrefsDomain];
	NSArray*			disks;
	NSEnumerator*		enumerator;
	NSString*			path;
	CPGPdiskObject*		disk;
	
	if ((disks = [dict objectForKey:kPGPdiskDefault_DiskList]) && [disks count])
	{
		enumerator = [disks objectEnumerator];
		
		while (path = [enumerator nextObject])
		{
			if (![self diskWithPath:path] && [[NSFileManager defaultManager] fileExistsAtPath:path])
			{
				disk = [CPGPdiskObject diskAtPath:path];
				[disk setController:self];
				
				err = [disk openDiskWithContext:mDiskContext];
				
				if (IsPGPError(err))
					[self displayOpenFailedAlertWithError:err forDisk:disk];
				else
				{
					[disk closeDisk];
					[self addDisk:disk];
				}
			}
		}
	}
	else
		[self findDisks];
}

- (void)addDisk:(CPGPdiskObject*)disk
{
	if (!disk)
		return;
	
	[mDisks addObject:disk];
	
	[self saveKnownDisksList];
}

- (void)removeDisk:(CPGPdiskObject*)disk
{
	if (!disk)
		return;
	
	[mDisks removeObject:disk];
	
	[self saveKnownDisksList];
}

- (void)saveKnownDisksList
{
	NSUserDefaults*			defaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary*	dict = [[[defaults persistentDomainForName:kPGPPrefsDomain] mutableCopy] autorelease];
	NSMutableArray*			diskList;
	NSEnumerator*			enumerator;
	CPGPdiskObject*			disk;
	
	if (!mDisks)
		return;
	
	diskList = [[[NSMutableArray alloc] init] autorelease];
	enumerator = [mDisks objectEnumerator];
	
	while (disk = [enumerator nextObject])
		[diskList addObject:[disk diskPath]];
	
	[dict setObject:diskList forKey:kPGPdiskDefault_DiskList];
	[defaults removePersistentDomainForName:kPGPPrefsDomain];
	[defaults setPersistentDomain:dict forName:kPGPPrefsDomain];
	[defaults synchronize];
}

- (PGPError)findDisks
{
	NSString*	desktopPath;
	NSString*	documentsPath;
	NSArray*	searchDirs;
	
	desktopPath = [[NSString pathWithComponents:
						[NSArray arrayWithObjects:NSHomeDirectory(), LSPD(@"Desktop", @"Desktop path"), nil]] stringByStandardizingPath];
	documentsPath = [[NSString pathWithComponents:
						[NSArray arrayWithObjects:NSHomeDirectory(), LSPD(@"Documents", @"Documents path"), nil]] stringByStandardizingPath];
	
	searchDirs = [NSArray arrayWithObjects:NSHomeDirectory(),
										   desktopPath,
										   documentsPath,
										   nil];
	
	return [self findDisksAtPaths:searchDirs];
}

- (PGPError)findDisksAtPaths:(NSArray*)paths
{
	int						i;
	unsigned				dirCount;
	PGPError				err = kPGPError_NoErr;
	NSFileManager*			fileManager = [NSFileManager defaultManager];
	NSString*				currentPath;
	NSString*				file;
	CPGPdiskObject*			disk;
	NSArray*				dirContents;
	NSEnumerator*			dirEnumerator;
	NSString*				fullPath;

	if (!paths || !mDisks)
		return kPGPError_BadParams;

	dirCount = [paths count];
	
	for (i = 0; i < dirCount; ++i)
	{
		currentPath = [paths objectAtIndex:i];	
		dirContents = [fileManager directoryContentsAtPath:currentPath];
		dirEnumerator = [dirContents objectEnumerator];
	
		while (file = [dirEnumerator nextObject])
		{
			if ([[file pathExtension] isEqualToString:@"pgd"])
			{
				fullPath = [NSString pathWithComponents:[NSArray arrayWithObjects:currentPath, file, nil]];
				disk = [CPGPdiskObject diskAtPath:fullPath];
				[disk setController:self];
				
				err = [disk openDiskWithContext:mDiskContext];
				
				if (IsPGPError(err))
					[self displayOpenFailedAlertWithError:err forDisk:disk];
				else
				{
					[self addDisk:disk];
					[disk closeDisk];
				}
			}
		}
	}
	
	return kPGPError_NoErr;
}

- (CPGPdiskObject*)diskWithPath:(NSString*)path
{
    NSEnumerator*		enumerator;
    CPGPdiskObject*		disk;
    
    if (!mDisks || !path)
        return nil;
        
    enumerator = [mDisks objectEnumerator];
    
    while (disk = [enumerator nextObject])
    {
        if ([[disk diskPath] isEqualToString:path])
            return disk;
    }
    
    return nil;
}

- (CPGPdiskObject*)selectedDisk
{
	return [[mDiskPopup selectedItem] representedObject];
}

- (CPGPdiskUser*)selectedUser
{
	int		row = [mUserTable selectedRow];
	
	if (!mUsers || (row == -1))
		return nil;
		
	else
		return [mUsers objectAtIndex:row];
}

- (void)selectDisk:(CPGPdiskObject*)disk
{
	NSEnumerator*		enumerator = [[mDiskPopup itemArray] objectEnumerator];
	NSMenuItem*			item;
	
	if (!disk)
	{
		[[mDiskPopup itemAtIndex:0] setTitle:kPGPdiskString_PGPdiskImages];
		[mDiskPopup selectItemAtIndex:0];
		[self diskPopupSelectionChanged:self];
	}
	else
	{
		while (item = [enumerator nextObject])
		{
			if ([item representedObject] == disk)
			{
				[mDiskPopup selectItem:item];
				[self diskPopupSelectionChanged:self];
				
				break;
			}
		}
	}
}

- (void)setupDiskPopup
{
	if (![mDisks count])
		return;
	
	/* Insert a menu item separator */
	[[mDiskPopup menu] insertItem:[NSMenuItem separatorItem] atIndex:1];
	
	[self populateDiskPopup];
	
	if ([mDisks count])
		[self selectDisk:[mDisks objectAtIndex:0]];
}

- (void)updateDiskPopup
{
	NSEnumerator*		enumerator;
	NSMenuItem*			item;
	
	enumerator = [[mDiskPopup itemArray] objectEnumerator];
	
	/* Remove all disk menu items */
	while (item = [enumerator nextObject])
	{
		if ([mDiskPopup indexOfItem:item] > 0 && ![item isSeparatorItem] && ([item tag] != kPGPdiskPopUpNew) && ([item tag] != kPGPdiskPopUpOpen))
			[mDiskPopup removeItemAtIndex:[mDiskPopup indexOfItem:item]];
		else if ([item isSeparatorItem] || ([item tag] == kPGPdiskPopUpNew) || ([item tag] == kPGPdiskPopUpOpen))
			break;
	}
	
	if (![mDisks count])
	{
		if ([[mDiskPopup itemAtIndex:1] isSeparatorItem])
			[mDiskPopup removeItemAtIndex:1];
		
		[[mDiskPopup itemAtIndex:0] setTitle:kPGPdiskString_PGPdiskImages];
	}
	else
		[self populateDiskPopup];
}

- (void)populateDiskPopup
{
	NSEnumerator*		enumerator;
	NSMenuItem*			item;
	CPGPdiskObject*		disk;
	int					i = 1;
	
	[mDisks sortUsingSelector:@selector(compareName:)];
	
	enumerator = [mDisks objectEnumerator];
	
	while (disk = [enumerator nextObject])
	{
		item = [[[NSMenuItem alloc] initWithTitle:[disk shortName] action:nil keyEquivalent:@""] autorelease];
		[item setTarget:self];
		[item setRepresentedObject:disk];
		
		if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
			[item setImage:mOS9DiskIcon];
		else
			[item setImage:mDiskIcon];
		
		[[mDiskPopup menu] insertItem:item atIndex:i];
		
		++i;
	}
	
	if (![[mDiskPopup itemAtIndex:i] isSeparatorItem])
		[[mDiskPopup menu] insertItem:[NSMenuItem separatorItem] atIndex:i];
}

- (IBAction)diskPopupSelectionChanged:(id)sender
{
	NSMenuItem*			selectedItem = [mDiskPopup selectedItem];
	NSMenuItem*			item;
	NSEnumerator*		enumerator;
	CPGPdiskObject*		disk = nil;
	int					tag = [selectedItem tag];
	
	if ((tag == kPGPdiskPopUpNew) || (tag == kPGPdiskPopUpOpen))
	{
		switch (tag)
		{
			case kPGPdiskPopUpNew:
				[self newDisk:sender];
				break;
			case kPGPdiskPopUpOpen:
				[self openDisk:sender];
				break;
			default:
				break;
		}
		
		[self selectDisk:disk];
	}
	else
	{
		enumerator = [[mDiskPopup itemArray] objectEnumerator];
		
		while (item = [enumerator nextObject])
			[item setState:NSOffState];
		
		if ([[selectedItem representedObject] isKindOfClass:[CPGPdiskObject class]])
		{
			disk = [selectedItem representedObject];
			[selectedItem setState:NSOnState];
			
			[[[mDiskPopup menu] itemAtIndex:0] setTitle:[disk shortName]];
			
			if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
				[[[mDiskPopup menu] itemAtIndex:0] setImage:mOS9DiskIcon];
			else
				[[[mDiskPopup menu] itemAtIndex:0] setImage:mDiskIcon];
			
			[mUserTable deselectAll:self];
			
			[self updateUsersForDisk:disk];
			[self updateTabViewForDisk:disk];
		}
		else
			[[[mDiskPopup menu] itemAtIndex:0] setImage:nil];
		
		[self setMountButtonForDisk:disk];
	}
	
	if (!disk)
	{
		[self updateUsersForDisk:nil];
		[self updateTabViewForDisk:nil];
	}
}

- (BOOL)diskEngineIsBusy
{
    return mDiskEngineIsBusy;
}

- (void)setDiskEngineIsBusy:(BOOL)flag
{
    mDiskEngineIsBusy = flag;
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
	NSString*		title = [menuItem title];
	
	if ([self diskEngineIsBusy])
		return NO;
	
	/* File->Open should always be available, so we just let the last statement return YES */
	
	if ([title isEqualToString:kPGPdiskString_NewPGPdisk])
		return YES;
	
	if ([title isEqualToString:kPGPdiskMenuItem_AddUser])
	{
		if (![mWindow isKeyWindow])
			return NO;
		
		return mAddUser;
	}
		
	if ([title isEqualToString:kPGPdiskMenuItem_RemoveUser])
		return mRemoveUser;
	
	if ([title isEqualToString:kPGPdiskMenuItem_DisableUser] || [title isEqualToString:kPGPdiskMenuItem_EnableUser])
	{
		CPGPdiskObject*	disk = [self selectedDisk];
		CPGPdiskUser*	user = [self selectedUser];
		
		if (![mWindow isKeyWindow])
			return NO;
		
		if (disk && ![disk isMounted] && user)
			return ![user isAdmin];
		
		return NO;
	}
	
	if ([title isEqualToString:kPGPdiskMenuItem_ChangePassphrase])
	{
		CPGPdiskObject*	disk = [self selectedDisk];
		CPGPdiskUser*	user = [self selectedUser];
		
		if (![mWindow isKeyWindow])
			return NO;
		
		if (disk && ![disk isMounted] && user)
			return [user isSymmetricKeyUser];
		
		return NO;
	}
	
    if ([title isEqualToString:kPGPdiskMenuItem_Convert])
    {
        CPGPdiskObject*	disk = [self selectedDisk];
        
        if (!disk)
            return YES;
        else
        {
			if (![mWindow isVisible])
				return NO;
            else if ([disk diskFormat] == kPGPdiskDiskFormat_OS9)
                return YES;
            else
                return NO;
        }
    }
    
	if ([menuItem menu] == [mDiskPopup menu])
	{
		CPGPdiskObject*	disk = nil;
		
		if ([[menuItem representedObject] isKindOfClass:[CPGPdiskObject class]])
		{
			disk = [menuItem representedObject];
			
			return ![disk diskHasMoved];
		}
	}
	
	return YES;
}

- (void)reEncryptDisk:(CPGPdiskObject*)disk withCipher:(NSNumber*)cipherNumber
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	NSArray*				contextArray;
	
	if (disk && cipherNumber)
	{
		contextArray = [[NSArray alloc] initWithObjects:disk, cipherNumber, nil];
	
		NSBeginAlertSheet([NSString stringWithFormat:
            kPGPdiskString_ReEncryptConfirmation,
            [disk shortName]],
            kPGPdiskString_Cancel,		/* defaultButton */
            kPGPdiskString_ReEncrypt,	/* alternateButton */
            nil,						/* otherButton */
            mWindow,					/* docWindow */
            self,						/* modalDelegate */
            nil,						/* willEndSelector */
            @selector(reEncryptWarningDidEnd:returnCode:contextInfo:),	/* didEndSelector */
			contextArray,				/* contextInfo */
			kPGPdiskString_ReEncryptConfirmationMsg);
	}
	
	[pool release];
}

- (void)continueReEncryptingDisk:(CPGPdiskObject*)disk
{
	NSAutoreleasePool*			pool = [[NSAutoreleasePool alloc] init];
	NSArray*					contextArray;
	PGPdiskEncryptionAlgorithm	cipher;		
	
	if (disk)
	{
		cipher = [disk diskAlgorithm];
		contextArray = [[NSArray alloc] initWithObjects:disk, [NSNumber numberWithInt:cipher], nil];

		NSBeginAlertSheet([NSString stringWithFormat:
            kPGPdiskString_ResumeReEncryption,
			[disk shortName]],
            kPGPdiskString_Cancel,		/* defaultButton */
            kPGPdiskString_ReEncrypt,	/* alternateButton */
            nil,						/* otherButton */
            mWindow,					/* docWindow */
            self,						/* modalDelegate */
            nil,						/* willEndSelector */
            @selector(reEncryptWarningDidEnd:returnCode:contextInfo:),	/* didEndSelector */
            contextArray,				/* contextInfo */
            kPGPdiskString_ResumeReEncryptionMsg);
	}
	
	[pool release];
}

- (void)reEncryptWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	NSAutoreleasePool*			pool = [[NSAutoreleasePool alloc] init];
	NSArray*					contextArray = (NSArray*) contextInfo;
	NSConnection*				connection;
	NSPort*						sendPort = [NSPort port];
	NSPort*						receivePort = [NSPort port];
	NSArray*					ports;
	CPGPdiskObject*				disk = nil;
	NSNumber*					cipherNumber = nil;
    CPGPdiskProgressPanel*		panel = [CPGPdiskProgressPanel sharedPanel];
	CPGPdiskReEncryption*		reEncrypt = [[CPGPdiskReEncryption alloc] init];
    char*						passphrase;
    PGPError					err;
	BOOL						openedDisk = NO;
	
	if (contextArray)
	{
		disk = [contextArray objectAtIndex:0];
		cipherNumber = [contextArray objectAtIndex:1];
	}

	if (disk && cipherNumber)
	{
		[disk setDiskIsBusy:YES];
		
        /* The "ReEncrypt" button is mapped to NSCancelButton */
		if (returnCode == NSCancelButton)
		{
			if (![disk diskRef])
			{
				[disk openDiskWithContext:mDiskContext];
				openedDisk = YES;
			}
			
			[self updateUsersForDisk:disk];
			
            err = [disk getAdminPassphrase:&passphrase];
			
            if (IsntPGPError(err))
            {
				mDiskEngineIsBusy = YES;
				
				[self activate:NO];
				
				connection = [[NSConnection alloc] initWithReceivePort:receivePort sendPort:sendPort];
				[connection setRootObject:self];
				
				[panel runSheetModalToWindow:mWindow
							modalDelegate:reEncrypt
							didEndSelector:nil
							stopSelector:@selector(abortReEncryption)
							contextInfo:nil];
				
				ports = [NSArray arrayWithObjects:sendPort, receivePort, nil];
				[reEncrypt setAlgorithm:[cipherNumber intValue]];
				[reEncrypt setDiskPath:[disk diskPath]];
				[reEncrypt setPassphrase:[NSData dataWithBytes:passphrase length:(strlen(passphrase) + 1)]];
				[reEncrypt setServerConnection:connection];
				[reEncrypt setPubRingPath:mPubKeyringPath];
				[reEncrypt setSecRingPath:mSecKeyringPath];
				
				if (openedDisk)
				{
					[disk closeDisk];
					openedDisk = NO;
				}
				
				PGPFreeData(passphrase);
				
				[NSThread detachNewThreadSelector:@selector(reEncryptDiskWithPorts:) toTarget:reEncrypt withObject:ports];
            }
			else
			{
				if (IsPGPError(err) && (err != kPGPError_BadPassphrase))
					[self displayReEncryptionAlertWithError:err forDisk:disk];
				
				if (openedDisk)
					[disk closeDisk];
				
				[disk setDiskIsBusy:NO];
				[self diskPopupSelectionChanged:nil];
			}
		}
        else
		{
			if (openedDisk)
				[disk closeDisk];
			
			[disk setDiskIsBusy:NO];
			[self diskPopupSelectionChanged:nil];
		}
    }
	
	if (contextArray)
		[contextArray release];
    
    if (disk)
        [disk setDiskIsBusy:NO];
    
    [pool release];
}

- (void)convertDisk:(CPGPdiskObject*)disk displayAlert:(BOOL)flag
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	
	if (disk)
		[disk retain];
	
	if (disk && flag)
	{
		NSBeginAlertSheet(kPGPdiskString_OS9Conversion,	/* title */
            kPGPdiskString_Convert,			/* default button */
            kPGPdiskString_Cancel,			/* alternate button */
            nil,							/* other button */
            mWindow,						/* doc window */
            self,							/* delegate */
            nil,							/* willEndSelector */
            @selector(conversionWarningDidEnd:returnCode:contextInfo:),	/* didEndSelector */
            disk,							/* contextInfo */
            [NSString stringWithFormat:kPGPdiskString_OS9ConversionMsg,
                [disk shortName]]);
	}
    else if (disk)
        [self conversionWarningDidEnd:nil returnCode:NSOKButton contextInfo:disk];
	
	[pool release];
}

- (void)conversionWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	NSAutoreleasePool*			pool = [[NSAutoreleasePool alloc] init];
	CPGPdiskObject*				disk = (CPGPdiskObject*) contextInfo;
	NSSavePanel*				savePanel = [NSSavePanel savePanel];
	NSConnection*				connection;
	NSPort*						sendPort = [NSPort port];
	NSPort*						receivePort = [NSPort port];
	NSString*					directory;
	NSString*					filename;
	NSArray*					ports;
	PGPContextRef				context;
    CPGPdiskProgressPanel*		panel = [CPGPdiskProgressPanel sharedPanel];
	CPGPdiskConversion*			converter = [[CPGPdiskConversion alloc] init];
	char*						passphrase = NULL;
	BOOL						openedDisk = NO;
	PGPError					err;

	if (disk)
	{
		if (returnCode == NSOKButton)
		{
			sConversionDisk = disk;
			
			directory = [[disk diskPath] stringByDeletingLastPathComponent];
			filename = [[[disk diskPath] lastPathComponent] stringByAppendingPathExtension:@"pgd"];
			[savePanel setCanSelectHiddenExtension:YES];
			[savePanel setRequiredFileType:@"pgd"];
			[savePanel setDelegate:self];
			[savePanel runModalForDirectory:directory file:filename];
			
			sConversionDisk = nil;
			
			if (filename = [savePanel filename])
			{
				context = [self pgpContext];
				
				if (![disk diskRef])
				{
					[disk openDiskWithContext:mDiskContext];
					openedDisk = YES;
				}
				
				[self updateUsersForDisk:disk];
				
                err = [disk getAdminPassphrase:&passphrase];
				
				if (IsntPGPError(err))
				{
                    mDiskEngineIsBusy = YES;
					
					[self activate:NO];
                    [self updateUsersForDisk:nil];
					
					connection = [[NSConnection alloc] initWithReceivePort:receivePort sendPort:sendPort];
					[connection setRootObject:self];
					
					[panel runSheetModalToWindow:mWindow
								modalDelegate:converter
								didEndSelector:nil
								stopSelector:@selector(abortConversion)
								contextInfo:nil];
					
					ports = [NSArray arrayWithObjects:sendPort, receivePort, nil];
					[converter setOriginalFilename:[disk diskPath]];
					[converter setNewFilename:filename];
					[converter setPassphrase:[NSData dataWithBytes:passphrase length:(strlen(passphrase) + 1)]];
					[converter setServerConnection:connection];
					[converter setPubRingPath:mPubKeyringPath];
					[converter setSecRingPath:mSecKeyringPath];
					
                    PGPFreeData(passphrase);
					
					if (openedDisk)
					{
						[disk closeDisk];
						openedDisk = NO;
					}
					
					[NSThread detachNewThreadSelector:@selector(convertDiskWithPorts:) toTarget:converter withObject:ports];
				}
				else
				{
					if (IsPGPError(err) && (err != kPGPError_BadPassphrase))
						[self displayConversionAlertWithError:err forDisk:disk];
	
					[disk setDiskIsBusy:NO];
					[self diskPopupSelectionChanged:nil];
				}
				
				if (openedDisk)
					[disk closeDisk];
			}
		}
		
		[disk release];
	}
	
	[pool release];
}

- (void)completedReEncryptionOfDisk:(NSString*)filename
{
    CPGPdiskObject*			disk;
	
	mDiskEngineIsBusy = NO;
	
	if (filename)
	{
		disk = [self diskWithPath:filename];
		
		if (disk)
		{
			[disk setDiskIsBusy:NO];
			[self selectDisk:disk];
			
			if ([disk diskNeedsReEncryption])
			{
				PGPLog((@"diskNeedsReEncryption == YES"));
				[self updateUsersForDisk:nil];
			}
			else
				[self updateUsersForDisk:disk];
			
			[self updateTabViewForDisk:disk];
			
			[disk closeDisk];
		}
	}
}

- (void)failedToReEncryptDisk:(NSString*)filename
{
	CPGPdiskObject*			disk;
	
	mDiskEngineIsBusy = NO;
	
	if (filename)
	{
		disk = [self diskWithPath:filename];
		
		if (disk)
		{
			[disk closeDisk];
			[disk setDiskIsBusy:NO];
			[self selectDisk:disk];
		}
	}
}

- (void)completedConversionOfDisk:(NSString*)filename
{
    mDiskEngineIsBusy = NO;

	if (filename)
		[self openDisk:filename mount:NO];
}

- (void)failedToConvertDisk:(NSString*)filename
{
	mDiskEngineIsBusy = NO;
	
	[self diskPopupSelectionChanged:nil];
}

- (BOOL)panel:(id)sender isValidFilename:(NSString*)filename
{
	NSFileManager*      fileManager = [NSFileManager defaultManager];
	NSString*           dir;
	BOOL                isDirectory = NO;
	BOOL                isTooLarge = NO;
	struct statfs       buf;
	PGPInt32            fsType = 0;
	PGPUInt32			blocksHigh = 0;
	PGPUInt32			blocksLow = 0;
	PGPUInt64			diskSize = 0;
	PGPUInt64           maxSize;
	
	memset(&buf, '\0', sizeof(struct statfs));
	
	dir = [filename stringByDeletingLastPathComponent];
	
	if (statfs([dir cString], &buf) == 0)
	{
		/*
		It's slightly difficult to tell the difference between an HFS and HFS+ volume from the statfs struct.
		From playing with it for a while, it seems that the key difference is in the f_bsize member. A value
		of 4096 indicates HFS+, whereas any other value would indicate an older HFS volume.
		*/
		if (!strncmp(buf.f_fstypename, "hfs", strlen("hfs")))
		{
			if (buf.f_bsize == 4096)
				fsType = kPGPdiskHFSExt;
			else
				fsType = kPGPdiskHFS;
		}
		else if (!strncmp(buf.f_fstypename, "ufs", strlen("ufs")))
			fsType = kPGPdiskUFS;
	}
	
	blocksHigh = [sConversionDisk getNumericProperty:kPGPdiskDiskProp_BlocksDiskHigh];
	blocksLow = [sConversionDisk getNumericProperty:kPGPdiskDiskProp_BlocksDiskLow];
	
	diskSize = (PGPUInt64) (blocksLow | ((PGPUInt64) blocksHigh << 32));
	diskSize *= 512;
	
	switch (fsType)
	{
		case kPGPdiskHFS:	/* Max HFS file size is 2GB */
			maxSize = 2 * ((PGPUInt64) 1024 * 1024 * 1024);
	
			if (diskSize > maxSize)
				isTooLarge = YES;
	
			break;
		case kPGPdiskUFS:	/* Max UFS file size is 4GB */
			maxSize = 4 * ((PGPUInt64) 1024 * 1024 * 1024);
	
			if (diskSize > maxSize)
				isTooLarge = YES;
	
			break;
		default:
			break;
	}
	
	if (isTooLarge)
	{
		NSRunAlertPanel(kPGPdiskString_FSMaxSizeExceeded,
			fsType == kPGPdiskHFS ? kPGPdiskString_FSHFSSizeExceededMsg : kPGPdiskString_FSUFSSizeExceededMsg,
			kPGPdiskString_OK,
			nil,
			nil);
	
		return NO;
	}
	
	if (fsType)     /* Only set if statfs didn't fail */
	{
		PGPUInt64   spaceAvailable = ((PGPUInt64) buf.f_bfree * buf.f_bsize);
		double      reducedSize = 0;
		NSString*   sizeUnits = nil;
		
		if (diskSize > spaceAvailable)
		{
			if (spaceAvailable > ((PGPUInt64) 1024 * 1024 * 1024))
			{
				reducedSize = (double) ((double) spaceAvailable / ((PGPUInt64) 1024 * 1024 * 1024));
				sizeUnits = @"GB";
			}
			else if (spaceAvailable > (1024 * 1024))
			{
				reducedSize = (double) ((double) spaceAvailable / (1024 * 1024));
				sizeUnits = @"MB";
			}
			else
			{
				reducedSize = (double) ((double) spaceAvailable / 1024);
				sizeUnits = @"KB";
			}
			
			NSRunAlertPanel(kPGPdiskString_SpaceNotAvailable,
				kPGPdiskString_SpaceNotAvailableMsg,
				kPGPdiskString_OK,
				nil,
				nil);
			
			return NO;
		}
	}
	
	if ([fileManager fileExistsAtPath:dir isDirectory:&isDirectory])
	{
		if ([fileManager isWritableFileAtPath:dir])
			return YES;
		else
		{
			NSRunAlertPanel(kPGPdiskString_CantWriteToLocation,
				kPGPdiskString_CantWriteToLocationMsg,
				kPGPdiskString_OK,
				nil,
				nil);
		}
	}
	else
	{
		NSRunAlertPanel(kPGPdiskString_InvalidDiskLocation,
			kPGPdiskString_InvalidDiskLocationMsg,
			kPGPdiskString_OK,
			nil,
			nil);
	}
	
	return NO;
}

- (BOOL)isDiskEngineRunning
{
    BOOL	isRunning;
	isRunning = PGPdiskIsEngineRunning();
    
    return isRunning;
}

/*
This method will request a new PGPdiskContext from the disk engine.
If the disk engine isn't running, we will launch the engine and wait
as long as it takes before the engine is running and servicing
requests.
*/
- (BOOL)createDiskContext
{
	PGPError		err;
	PGPUInt32		retries = 0;

    [self launchDiskEngine];

	while ((!([self isDiskEngineRunning])) && (retries < kPGPdiskEngineConnectRetries))
	{
		sleep(1);
		
		++retries;
	}

	if (retries == kPGPdiskEngineConnectRetries)
		return NO;

	err = PGPdiskNewContext([self pgpContext], kPGPdiskAPIVersion, &mDiskContext);
	
	if (IsPGPError(err))
    {
        PGPLog((@"PGPdiskNewContext failed with err %d", err));
		return NO;
    }
	else
		return YES;
	
	return NO;
}

/*
Launch the disk engine if it isn't currently running. We determine the status
of the disk engine by calling isDiskEngineRunning.
*/
- (BOOL)launchDiskEngine
{
    NSFileManager*	manager = [NSFileManager defaultManager];
	NSString*		bundlePath = [[NSBundle bundleForClass:[self class]] bundlePath];
	NSString*		launchPath = [bundlePath stringByAppendingPathComponent:kPGPdiskEngineLaunchPath];
	//NSString*		updatePrebinding;
	
	signal(SIGPIPE, sigPipeHandler);
	
	if ([manager fileExistsAtPath:launchPath])
    {
        if (![self isDiskEngineRunning])
        {
			if (!mDiskEngineTask)
			{
				mDiskEngineTask = [[NSTask alloc] init];
				[mDiskEngineTask setLaunchPath:launchPath];
				
				//updatePrebinding = [NSString stringWithFormat:@"/usr/bin/update_prebinding -root / -force -files %@", launchPath];
				//system([updatePrebinding UTF8String]);
				
				[mDiskEngineTask launch];
			}
        }
	}
			
	return YES;
}

- (void)setPassphraseCaching
{
	PGPError			err = kPGPError_NoErr;
	PGPPrefRef			prefRef = [self pgpPrefRef];
	BOOL				prefsChanged = NO;
	static BOOL			firstTimeThrough = YES;
	static PGPUInt32	cacheType = kPGPPrefCacheTypeNone;
	static PGPUInt32	cacheSeconds = kPGPMaxTimeInterval;
	static PGPBoolean	shareCache = FALSE;
	PGPUInt32			nData = 0;
	PGPBoolean			bData = FALSE;
	
	if (firstTimeThrough)
	{
		PGPGetPrefNumber(prefRef, kPGPPrefCacheType, &cacheType);
		PGPGetPrefNumber(prefRef, kPGPPrefCacheSeconds, &cacheSeconds);
		PGPGetPrefBoolean(prefRef, kPGPPrefShareCache, &shareCache);
		
		firstTimeThrough = NO;
		prefsChanged = YES;
	}
	else
	{
		PGPGetPrefNumber(prefRef, kPGPPrefCacheType, &nData);
		
		if (cacheType != nData)
		{
			cacheType = nData;
			prefsChanged = YES;
		}
		
		PGPGetPrefNumber(prefRef, kPGPPrefCacheSeconds, &nData);
		
		if (cacheSeconds != nData)
		{
			cacheSeconds = nData;
			prefsChanged = YES;
		}
		
		PGPGetPrefBoolean(prefRef, kPGPPrefShareCache, &bData);
		
		if (shareCache != bData)
		{
			shareCache = bData;
			prefsChanged = YES;
		}
	}
	
	if (prefsChanged)
	{
		switch (cacheType)
		{
			case kPGPPrefCacheTypeNone:
				err = PGPdiskContextSetPassphraseCaching([self pgpDiskContext], FALSE, 0, FALSE);
				break;
			case kPGPPrefCacheTypeSeconds:
				err = PGPdiskContextSetPassphraseCaching([self pgpDiskContext], TRUE, cacheSeconds, shareCache);
				break;
			case kPGPPrefCacheTypeLogon:
				err = PGPdiskContextSetPassphraseCaching([self pgpDiskContext], TRUE, kPGPMaxTimeInterval, shareCache);
				break;
			default:
				PGPLog((@"Definitely shouldn't get here... Unrecognized cacheType: %d", cacheType));
				break;
		}
		
		if (IsPGPError(err))
			PGPLog((@"Failed to set passphrase caching options (err=%d)", err));
	}
}

- (void)setGlobalDiskTimeout:(PGPUInt32)minutes
{
	PGPError		err;
	
	err = PGPdiskContextSetGlobalDiskTimeout([self pgpDiskContext], minutes);
	
	if (IsPGPError(err))
		PGPLog((@"Failed to set global disk timeout (err=%d)", err));
}

- (PGPUInt32)globalDiskTimeout
{
	PGPError		err;
	PGPBoolean		bData;
	PGPUInt32		nData;
	
	err = PGPGetPrefBoolean([self pgpPrefRef], kPGPPrefDiskUnmountOnInactivity, &bData);
	
	if (IsntPGPError(err) && bData)
	{
		if (IsPGPError(err = PGPGetPrefNumber([self pgpPrefRef], kPGPPrefDiskAutoUnmountTimeout, &nData)))
			nData = 0;
	}
	else if (!bData)
		nData = 0;
		
	return nData;
}

- (void)setUnmountOnLogout:(PGPBoolean)flag
{
	PGPError		err;
	
	err = PGPdiskContextSetGlobalEjectOnLogout([self pgpDiskContext], flag);
	
	if (IsPGPError(err))
		PGPLog((@"Failed to set global unmount on logout (err=%d)", err));
}

- (PGPBoolean)unmountOnLogout
{
	/*
	PGPError		err;
	PGPBoolean		bData = FALSE;
	
	err = PGPGetPrefBoolean([self pgpPrefRef], kPGPPrefDiskAutoUnmountOnLogout, &bData);
	
	return bData;
	*/
	return TRUE;
}

- (void)setUnmountOnSleep:(PGPBoolean)flag
{
	PGPError		err;
	
	err = PGPdiskContextSetUnmountOnSleep([self pgpDiskContext], flag);
	
	if (IsPGPError(err))
		PGPLog((@"Failed to set unmount on sleep (err=%d)", err));
}

- (PGPBoolean)unmountOnSleep
{
	PGPError		err;
	PGPBoolean		bData = FALSE;
	
	err = PGPGetPrefBoolean([self pgpPrefRef], kPGPPrefDiskAutoUnmountOnSleep, &bData);
	
	return bData;
}

- (void)setForcedUnmount:(PGPBoolean)flag
{
	PGPError		err;
	
	err = PGPdiskContextSetForcedAutoDiskUnmount([self pgpDiskContext], flag);
	
	if (IsPGPError(err))
		PGPLog((@"Failed to set unmount on sleep (err=%d)", err));
}

- (PGPBoolean)forcedUnmount
{
	PGPError		err;
	PGPBoolean		bData = FALSE;
	
	err = PGPGetPrefBoolean([self pgpPrefRef], kPGPPrefDiskAllowForcibleUnmounts, &bData);
	
	return bData;
}

- (void)setADK:(PGPKeyDBObjRef)adk
{
	PGPError		err;
	PGPKeyID		keyID;
	
	if (adk)
	{
		[self updateKeyrings];
		
		if (IsPGPError(err = PGPGetKeyID(adk, &keyID)))
		{
			PGPLog((@"Failed to get keyID (err=%d)", err));
			return;
		}
		
		if (IsPGPError(err = PGPdiskContextSetADK([self pgpDiskContext], &keyID)))
			PGPLog((@"Failed to set adk (err=%d)", err));
	}
}

- (PGPKeyDBObjRef)adk
{
	PGPError		err;
	PGPBoolean		haveADK;
	PGPByte*		data;
	PGPSize			dataLength;
	PGPKeyID		adkID;
	PGPKeyDBObjRef	adk = NULL;
	
    err = PGPGetPrefBoolean([self pgpPrefRef], kPGPPrefUsePGPdiskADK, &haveADK);
    
	if(IsntPGPError(err) && haveADK)
    {
        err = PGPGetPrefData([self pgpPrefRef], kPGPPrefPGPdiskADKKeyID, &dataLength, (void**) &data);
        if (IsntPGPError(err))
        {
            adkID = *((PGPKeyID*) data);
            
            err = PGPFindKeyByKeyID([self pgpKeyDB], &adkID, &adk);
            
            PGPFreeData(data);
			
			return adk;
        }
    }
	
	return NULL;
}

- (void)setPreventSleep:(PGPBoolean)flag
{
	PGPError		err;
	
	err = PGPdiskContextSetPreventSleep([self pgpDiskContext], flag);
	
	if (IsPGPError(err))
		PGPLog((@"Failed to set prevent sleep on auto-unmount failure (err=%d)", err));
}

- (PGPBoolean)preventSleep
{
	PGPError		err;
	PGPBoolean		bData = FALSE;
	
	err = PGPGetPrefBoolean([self pgpPrefRef], kPGPPrefDiskNoSleepIfFailAU, &bData);
	
	return bData;
}

- (void)updateKeyrings
{
	char		pubKeyringPath[PATH_MAX];
	char		secKeyringPath[PATH_MAX];
	BOOL		didChange = NO;
		
	pubKeyringPath[0] = '\0';
	secKeyringPath[0] = '\0';

	PGPclGetPath([self pgpPrefRef], kInvalidPGPPrefRef, kPGPclPublicKeyringFile, pubKeyringPath, PATH_MAX - 1);
	PGPclGetPath([self pgpPrefRef], kInvalidPGPPrefRef, kPGPclPrivateKeyringFile, secKeyringPath, PATH_MAX - 1);

	if (strncmp(pubKeyringPath, mPubKeyringPath, PATH_MAX) ||
		strncmp(secKeyringPath, mSecKeyringPath, PATH_MAX))
	{
		didChange = YES;
		
		strncpy(mPubKeyringPath, pubKeyringPath, PATH_MAX);
		strncpy(mSecKeyringPath, secKeyringPath, PATH_MAX);
	}
	
	if (didChange)
	{
		if (mUsers)
			[mUsers removeAllObjects];
		
		if (mPubKeyringPath[0] && mSecKeyringPath[0])
			PGPdiskLoadKeyrings([self pgpDiskContext], mPubKeyringPath, mSecKeyringPath);
		
		[self updateUsersForDisk:[self selectedDisk]];
	}
}

- (void)unmountAllDisks:(NSMutableData*)success
{
	NSArray*			disks;
	NSEnumerator*		enumerator;
	CPGPdiskObject*		disk;
	PGPError			err;
	BOOL				unmounted = YES;
	
	if (!success)
		return;
	
	if (!mMountedDisks || ![mMountedDisks count])
	{
		[success setData:[NSData dataWithBytes:&unmounted length:sizeof(BOOL)]];
		return;
	}
	
	disks = [mMountedDisks copy];
	enumerator = [disks objectEnumerator];
	
	while (disk = [enumerator nextObject])
	{
		PGPLog((@"Unmounting disk: %@", [disk shortName]));
		if (IsPGPError(err = [disk unmount]))
			unmounted = NO;
	}
	
	[disks release];
	
	[success setData:[NSData dataWithBytes:&unmounted length:sizeof(BOOL)]];
}

- (void)diskEnginePID:(NSMutableData*)pid
{
	PGPError			err;
	pid_t				p = getpid();	/* Return our own PID in case of error */
	
	if (!pid)
		return;
	
	if (IsPGPError(err = PGPdiskGetEnginePID(mDiskContext, &p)))
		PGPLog((@"Failed to get diskengine pid (err=%d)", err));
	
	[pid setData:[NSData dataWithBytes:&p length:sizeof(pid_t)]];
}

- (void)displayOpenFailedAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk
{
	NSString*		caption = nil;
	NSString*		message = nil;
	BOOL			critical = NO;
	
	switch (err)
	{
		case kPGPError_CantOpenFile:
			caption = kPGPdiskString_OpenFailed;
			
			if (disk)
				message = [NSString stringWithFormat:kPGPdiskString_CantOpenFile, [disk diskPath]];
			else
				message = [NSString stringWithFormat:kPGPdiskString_CantOpenFile, @"???"];
			break;
		case kPGPError_CorruptData:
			caption = kPGPdiskString_OpenFailed;
			
			if (disk)
				message = [NSString stringWithFormat:kPGPdiskString_DiskCorrupted, [disk diskPath]];
			else
				message = [NSString stringWithFormat:kPGPdiskString_DiskCorrupted, @"???"];
			break;
		case kPGPError_EOF:
			caption = kPGPdiskString_OpenFailed;
			
			if (disk)
				message = [NSString stringWithFormat:kPGPdiskString_EOF, [disk diskPath]];
			else
				[NSString stringWithFormat:kPGPdiskString_EOF, @"???"];
			break;
		default:
			caption = kPGPdiskString_OpenFailed;
			message = kPGPdiskString_GenericOpenFailed;
			PGPLog((@"Unhandled open failure alert (err=%d)", err));
			break;
	}
	
	[self displayAlert:caption message:message critical:critical];
}

- (void)displayMountFailedAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk
{
	NSString*		caption = nil;
	NSString*		message = nil;
	BOOL			critical = NO;
	
	switch (err)
	{
		case kPGPdiskError_NotInstalledCorrectly:
			caption = kPGPdiskString_MountFailure;
			message = kPGPdiskString_BadInstall;
			break;
		
		case kPGPError_ItemNotFound:
			caption = kPGPdiskString_MountFailure;
			message = kPGPdiskString_MountKeyringsMissing;
		
		default:
			PGPLog((@"Unhandled mount failure alert (err=%d)", err));
			break;
	}
	
	[self displayAlert:caption message:message critical:critical];
}

- (void)displayUnmountFailedAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk
{
	NSString*		caption = nil;
	NSString*		message = nil;
	BOOL			critical = NO;
	
	switch (err)
	{
		case kPGPError_FileOpFailed:
			caption = kPGPdiskString_UnableToUnmount;
			message = kPGPdiskString_UnableToUnmountMsg;
			break;
		
		default:
			PGPLog((@"Unhandled unmount failure alert (err=%d)", err));
			break;
	}
	
	[self displayAlert:caption message:message critical:critical];
}

- (void)displayUserModAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk
{
	NSString*		caption = nil;
	NSString*		message = nil;
	BOOL			critical = NO;
	
	switch (err)
	{
		case kPGPError_ItemNotFound:
			caption = kPGPdiskString_UserModFailure;
			message = kPGPdiskString_UserModKeyringsMissing;
		
		default:
			PGPLog((@"Unhandled usermod failure alert (err=%d)", err));
			break;
	}
	
	[self displayAlert:caption message:message critical:critical];
}

- (void)displayReEncryptionAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk
{
	NSString*		caption = nil;
	NSString*		message = nil;
	BOOL			critical = NO;
	
	switch (err)
	{
		case kPGPError_ItemNotFound:
			caption = kPGPdiskString_ReEncryptFailure;
			message = kPGPdiskString_ReEncryptKeyringsMissing;
		
		default:
			PGPLog((@"Unhandled reencryption failure alert (err=%d)", err));
			break;
	}
	
	[self displayAlert:caption message:message critical:critical];
}

- (void)displayConversionAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk
{
	NSString*		caption = nil;
	NSString*		message = nil;
	BOOL			critical = NO;
	
	switch (err)
	{
		case kPGPError_ItemNotFound:
			caption = kPGPdiskString_ConversionFailure;
			message = kPGPdiskString_ConversionKeyringsMissing;
		
		default:
			PGPLog((@"Unhandled conversion failure alert (err=%d)", err));
			break;
	}
	
	[self displayAlert:caption message:message critical:critical];
}

- (void)displayAlert:(NSString*)caption message:(NSString*)message critical:(BOOL)critical
{
	if (caption || message)
	{
		/*
		Because we might not have the main window open (if launched by double-clicking a disk)
		we use NSBeginAlertPanel instead of NSBeginAlertSheet.
		*/
		NSRunAlertPanel(caption,
			message,
			critical ? LSPD(@"Quit", @"") : kPGPdiskString_OK,
			nil,
			nil);
		
		if (critical)
			[NSApp terminate:nil];
	}
}

- (void)installDiskCheckerTimer
{
    if (!mDiskCheckerTimer)
    {
        mDiskCheckerTimer = [NSTimer scheduledTimerWithTimeInterval:kPGPdiskDiskCheckInterval
                                        target:self
                                        selector:@selector(diskChecker:)
                                        userInfo:nil
                                        repeats:YES];
    }
}

- (void)diskChecker:(NSTimer*)timer
{
	NSEnumerator*		enumerator;
	CPGPdiskObject*		disk;
	CPGPdiskObject*		selectedDisk;
	BOOL				removedDisk = NO;
	
	if (!mDisks || mDiskEngineIsBusy)
		return;

	enumerator = [mDisks objectEnumerator];
	selectedDisk = [self selectedDisk];
	
	while (disk = [enumerator nextObject])
	{
		if ([disk diskHasMoved])
		{
			if ([self selectedDisk] == disk)
			{
				selectedDisk = nil;
				
				[self selectDisk:nil];
				[mUserTable deselectAll:self];
				mUsers = nil;
			}
			
			[self removeDisk:disk];
			
			removedDisk = YES;
		}
		else
		{
			if (mMountedDisks && ([mMountedDisks indexOfObject:disk] != NSNotFound))
			{
				if (![disk isMounted])
				{
					[mMountedDisks removeObject:disk];
					removedDisk = YES;
					[disk closeDisk];
				}
			}
		}
	}
	
	if (removedDisk)
	{
		[self updateDiskPopup];						/* Removes the disks from the popup, then repopulates */
		[self selectDisk:selectedDisk];
		[self setMountButtonForDisk:selectedDisk];
	}
}

- (void)tabView:(NSTabView*)tabView didSelectTabViewItem:(NSTabViewItem*)tabViewItem
{
	[self setupKeyViewLoop];
}

- (void)passphraseCacheTypeDidChange:(NSNotification*)notification
{
	[self setPassphraseCaching];
}

- (void)passphraseSecondsDidChange:(NSNotification*)notification
{
	[self setPassphraseCaching];
}

- (void)passphraseShareCacheDidChange:(NSNotification*)notification
{
	[self setPassphraseCaching];
}

- (void)allowForcibleUnmountDidChange:(NSNotification*)notification
{
	[self setForcedUnmount:[self forcedUnmount]];
}

- (void)unmountOnInactivityDidChange:(NSNotification*)notification
{
	[self setGlobalDiskTimeout:[self globalDiskTimeout]];
}

- (void)autoUnmountTimeoutDidChange:(NSNotification*)notification
{
	[self setGlobalDiskTimeout:[self globalDiskTimeout]];
}

- (void)autoUnmountOnLogoutDidChange:(NSNotification*)notification
{
	[self setUnmountOnLogout:[self unmountOnLogout]];
}

- (void)autoUnmountOnSleepDidChange:(NSNotification*)notification
{
	[self setUnmountOnSleep:[self unmountOnSleep]];
}

- (void)preventSleepDidChange:(NSNotification*)notification
{
	[self setPreventSleep:[self preventSleep]];
}

- (void)windowDidBecomeKey:(NSNotification*)notification
{
	[mController setCurrentBundle:self passive:YES];
	
	[mWindow makeFirstResponder:mLastResponder];
}

- (void)windowDidResignKey:(NSNotification*)notification
{
	mLastResponder = [mWindow firstResponder];
}

- (void)finderUnmountedVolume:(NSNotification*)notification
{
	[self diskChecker:nil];
}

- (void)dealloc
{
	if (sSharedInstance == self)
		sSharedInstance = nil;
		
	[super dealloc];
}

@end
