/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPdiskMountDisk.m,v 1.23 2002/10/04 21:06:05 jeffc Exp $
____________________________________________________________________________*/

#import "PGPdiskMountDisk.h"

#define kPGPdiskString_EnterPassphraseForDisk	LSPD(@"Enter passphrase for \"%@\"", @"Get passphrase prompt")
#define kPGPdiskString_MountTooltip				LSPD(@"Mount PGPdisk", @"Mount tooltip")
#define kPGPdiskString_MountDiskTooltip			LSPD(@"Mount PGPdisk \"%@\"", @"Mount disk tooltip")
#define kPGPdiskString_UnmountDiskTooltip		LSPD(@"Unmount PGPdisk \"%@\"", @"Unmount disk tooltip")

@implementation PGPdiskController (PGPdiskMountExtensions)

- (IBAction)mountDiskPressed:(id)sender
{
	CPGPdiskObject*	disk = [self selectedDisk];
	
	/* If no disk selection is currently made, ask the user to choose a disk. */
	if (!disk)
	{
		if (sender == mMountButton)
            [self openDisk:nil];
	}
	else
	{
		if ([disk diskNeedsReEncryption])
			[self continueReEncryptingDisk:disk];
		else
		{
			if ([disk isMounted])
				[self unmountDisk:disk];
			else
				[self mountDisk:disk];
				
			[self setMountButtonForDisk:disk];
		}
	}
}

- (PGPError)mountDisk:(CPGPdiskObject*)disk
{
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context;
	PGPBoolean			canMount = NO;
	NSString*			passphraseCaption;
	char*				passphrase;
	BOOL				openedDisk = NO;
	
	if (!disk)
		return kPGPError_BadParams;

	if (![disk diskRef])
	{
		[disk openDiskWithContext:mDiskContext];
		openedDisk = YES;
	}
	
	if ([disk isMounted])
		return kPGPdiskError_DiskAlreadyMounted;
	
	if (IsPGPError(err = PGPdiskCanPassphraseMountDisk([disk diskRef], NULL, &canMount)))
		PGPLog((@"PGPdiskCanPassphraseMountDisk returned err=%d", err));
	
	if (canMount)
	{
		err = [disk mountWithPassphrase:NULL];
		goto done;
	}
	
	context = [self pgpContext];

retry:
	if (err == kPGPError_BadPassphrase)
		passphraseCaption = kPGPdiskString_IncorrectPassphrase;
	else
	{
		passphraseCaption = [NSString stringWithFormat:
			kPGPdiskString_EnterPassphraseForDisk, [[disk diskPath]
			lastPathComponent]];
	}

	err = PGPPassphraseDialog(context,
				PGPOUIDialogPrompt(context, [passphraseCaption cString]),
				PGPOUIOutputPassphrase(context, &passphrase), PGPOLastOption(context));

	if (IsntPGPError(err))
	{
		err = [disk mountWithPassphrase:passphrase];
					
		PGPFreeData(passphrase);

		if (err == kPGPError_BadPassphrase)
			goto retry;
		else if (IsPGPError(err))
		{
			PGPLog((@"Failed to mount disk: %@ (err=%d)", [disk diskPath], err));
			[self displayMountFailedAlertWithError:err forDisk:disk];
		}
	}

done:
	if (openedDisk)
		[disk closeDisk];
	
	[self setMountButtonForDisk:disk];
	[self updateTabViewForDisk:disk];
	
	return err;
}

- (PGPError)unmountDisk:(CPGPdiskObject*)disk
{
	PGPError		err;
	
	if (!disk)
		return kPGPError_BadParams;
	
	if (![disk isMounted])
		return kPGPdiskError_DiskNotMounted;
		
	if (IsPGPError(err = [disk unmount]))
		[self displayUnmountFailedAlertWithError:err forDisk:disk];
	
	[self updateTabViewForDisk:disk];
	
	return err;
}

- (void)setMountButtonForDiskAtIndex:(int)diskIndex
{
	CPGPdiskObject*	diskEntry;
	
	if (diskIndex == -1)
		[self setMountButtonForDisk:nil];
	else
	{
		diskEntry = [mDisks objectAtIndex:diskIndex];
		[self setMountButtonForDisk:diskEntry];
	}
}

- (void)setMountButtonForDisk:(CPGPdiskObject*)disk
{
	BOOL			isMounted;
	
	if (!disk)
	{
		[mMountButton setEnabled:YES];
		[self setMountButton:nil];
	}
	
	else
	{
		if ([disk diskIsBusy])
			[mMountButton setEnabled:NO];
			
		else
		{
			[mMountButton setEnabled:YES];
			
			if (isMounted = [disk isMounted])
				[self setUnmountButton:disk];
			else
				[self setMountButton:disk];
		}
	}
}

- (void)setUnmountButton:(CPGPdiskObject*)disk
{
	[mMountButton setImage:mUnmountImage];
	
	if (disk)
	{
		[mMountButton setToolTip:[NSString stringWithFormat:
			kPGPdiskString_UnmountDiskTooltip, [disk diskPath]]];
	}
}

- (void)setMountButton:(CPGPdiskObject*)disk
{
	[mMountButton setImage:mMountImage];
	
	if (disk)
	{
		[mMountButton setToolTip:[NSString stringWithFormat:
			kPGPdiskString_MountDiskTooltip, [disk diskPath]]];
	}
	else
		[mMountButton setToolTip:kPGPdiskString_MountTooltip];
}

@end
