/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPdiskPreferences.m,v 1.25 2002/09/30 08:21:15 jeffc Exp $
____________________________________________________________________________*/

#import "PGPdiskPreferences.h"

@implementation PGPdiskPreferences

- (id)init
{
	if (!mInited)
	{
		[super init];
    
		mIdentifiers = [[NSArray alloc] initWithObjects:kPGPdiskPrefsIdentifier, nil];
	
		mInited = YES;
	}

	if (!mLoadedNib)
		[self initWithNib:kPGPdiskPreferencesNib];
	
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
    
	[mAutoUnmountDelay setFormatter:[CIntegerFormatter intFormatter]];

	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller
{
	mController = controller;
	
	return [self init];
}

- (id)controller
{
	return mController;
}

- (NSArray*)identifiers
{
	return mIdentifiers;
}

- (id)prefsViewForIdentifier:(NSString*)identifier
{
	if (identifier && [identifier isEqualToString:kPGPdiskPrefsIdentifier])
		return mPrefsView;
		
	return nil;
}

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier
{
	return mAutoUnmountAfterDelay;
}

- (id)lastPrefsResponderForIdentifier:(NSString*)identifier
{
	return mAllowForcibleUnmounts;
}

- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier
{
	[mAutoUnmountAfterDelay setNextKeyView:mDelayStepper];
	[mDelayStepper setNextKeyView:mUnmountOnSleep];
	[mUnmountOnSleep setNextKeyView:mPreventSleep];
	[mPreventSleep setNextKeyView:mAllowForcibleUnmounts];
}

- (void)restorePanelPrefs:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	BOOL				enabled;
	
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPdiskPrefsIdentifier])
	{
		/* kPGPPrefDiskAllowForcibleUnmounts */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefDiskAllowForcibleUnmounts, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefDiskAllowForcibleUnmounts, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskAllowForcibleUnmounts, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskAllowForcibleUnmounts, err));
				enabled = NO;
			}
			
			[self setAllowForcibleUnmount:bData enabled:enabled];
		}
		
		/* kPGPPrefDiskUnmountOnInactivity */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefDiskUnmountOnInactivity, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefDiskUnmountOnInactivity, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskUnmountOnInactivity, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskUnmountOnInactivity, err));
				enabled = NO;
			}
			
			[self setUnmountOnInactivity:bData enabled:enabled];
		}
		
		/* kPGPPrefDiskAutoUnmountTimeout */
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefDiskAutoUnmountTimeout, &nData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountTimeout, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskAutoUnmountTimeout, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountTimeout, err));
				enabled = NO;
			}
			
			[self setAutoUnmountTimeout:nData enabled:enabled];
		}
		
		/* kPGPPrefDiskAutoUnmountOnSleep */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefDiskAutoUnmountOnSleep, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountOnSleep, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskAutoUnmountOnSleep, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountOnSleep, err));
				enabled = NO;
			}
			
			[self setUnmountOnSleep:bData enabled:enabled];
		}
		
		/* kPGPPrefDiskNoSleepIfFailAU */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefDiskNoSleepIfFailAU, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefDiskNoSleepIfFailAU, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskNoSleepIfFailAU, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskNoSleepIfFailAU, err));
				enabled = NO;
			}
			
			[self setPreventSleep:bData enabled:enabled];
		}
	}
}

- (void)restorePanelDefaults:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	PGPPrefDefinition	prefDef;
	BOOL				enabled;
    
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPdiskPrefsIdentifier])
	{
		/* kPGPPrefDiskAllowForcibleUnmounts */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefDiskAllowForcibleUnmounts, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefDiskAllowForcibleUnmounts, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskAllowForcibleUnmounts, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskAllowForcibleUnmounts, err));
				enabled = NO;
			}
			
			[self setAllowForcibleUnmount:bData enabled:enabled];
		}
		
		/* kPGPPrefDiskUnmountOnInactivity */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefDiskUnmountOnInactivity, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefDiskUnmountOnInactivity, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskUnmountOnInactivity, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskUnmountOnInactivity, err));
				enabled = NO;
			}
			
			[self setUnmountOnInactivity:bData enabled:enabled];
		}
	
		/* kPGPPrefDiskAutoUnmountTimeout */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefDiskAutoUnmountTimeout, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountTimeout, err));
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskAutoUnmountTimeout, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountTimeout, err));
				enabled = NO;
			}
			
			[self setAutoUnmountTimeout:nData enabled:enabled];
		}
		
		/* kPGPPrefDiskAutoUnmountOnSleep */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefDiskAutoUnmountOnSleep, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountOnSleep, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskAutoUnmountOnSleep, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountOnSleep, err));
				enabled = NO;
			}
			
			[self setUnmountOnSleep:bData enabled:enabled];
		}
	
		/* kPGPPrefDiskNoSleepIfFailAU */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefDiskNoSleepIfFailAU, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefDiskNoSleepIfFailAU, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefDiskNoSleepIfFailAU, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefDiskNoSleepIfFailAU, err));
				enabled = NO;
			}
			
			[self setPreventSleep:bData enabled:enabled];
		}
	}
}

- (void)savePanelPrefs:(NSString*)panelIdentifier
{
	PGPError				err;
	PGPPrefRef				prefRef = [mController pgpPrefRef];
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];

	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPdiskPrefsIdentifier])
	{
		/* kPGPPrefDiskAllowForcibleUnmounts */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefDiskAllowForcibleUnmounts, [self getAllowForcibleUnmount])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefDiskAllowForcibleUnmounts, err));
		else
			[center postNotificationName:kPGPclPrefNotification_ForcibleUnmounts object:nil];
	
		/* kPGPPrefDiskUnmountOnInactivity */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefDiskUnmountOnInactivity, [self getUnmountOnInactivity])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefDiskUnmountOnInactivity, err));
		else
			[center postNotificationName:kPGPclPrefNotification_AutoUnmount object:nil];
	
		/* kPGPPrefDiskAutoUnmountTimeout */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefDiskAutoUnmountTimeout, [self getAutoUnmountTimeout])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountTimeout, err));
		else
			[center postNotificationName:kPGPclPrefNotification_UnmountTimeout object:nil];
		
		/* kPGPPrefDiskAutoUnmountOnSleep */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefDiskAutoUnmountOnSleep, [self getUnmountOnSleep])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefDiskAutoUnmountOnSleep, err));
		else
			[center postNotificationName:kPGPclPrefNotification_UnmountSleep object:nil];
	
		/* kPGPPrefDiskNoSleepIfFailAU */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefDiskNoSleepIfFailAU, [self getPreventSleep])))
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefDiskNoSleepIfFailAU, err));
		else
			[center postNotificationName:kPGPclPrefNotification_PreventSleep object:nil];
	}
}

- (void)setAllowForcibleUnmount:(PGPBoolean)value enabled:(BOOL)enabled
{
    [self setCheckbox:mAllowForcibleUnmounts state:value enabled:enabled];
}

- (void)setUnmountOnInactivity:(PGPBoolean)value enabled:(BOOL)enabled
{
    [self setCheckbox:mAutoUnmountAfterDelay state:value enabled:enabled];
	[self unmountAfterDelay:nil];
}

- (void)setAutoUnmountTimeout:(PGPUInt32)value enabled:(BOOL)enabled
{
	[mAutoUnmountDelay setStringValue:@""];
    [mAutoUnmountDelay setIntValue:value];
}

- (void)setUnmountOnSleep:(PGPBoolean)value enabled:(BOOL)enabled
{
    [self setCheckbox:mUnmountOnSleep state:value enabled:enabled];
}

- (void)setPreventSleep:(PGPBoolean)value enabled:(BOOL)enabled
{
    if (value)
        [mPreventSleep setState:NSOnState];
    else
        [mPreventSleep setState:NSOffState];
    
    [self unmountOnSleep:self];
}

- (PGPBoolean)getAllowForcibleUnmount
{
    return [self getCheckbox:mAllowForcibleUnmounts];
}

- (PGPBoolean)getUnmountOnInactivity
{
    return [self getCheckbox:mAutoUnmountAfterDelay];
}

- (PGPUInt32)getAutoUnmountTimeout
{
    return [mAutoUnmountDelay intValue];
}

- (PGPBoolean)getUnmountOnSleep
{
    return [self getCheckbox:mUnmountOnSleep];
}

- (PGPBoolean)getPreventSleep
{
    return [self getCheckbox:mPreventSleep];
}

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled
{
	if (checkBox)
	{
		if (state)
			[checkBox setState:NSOnState];
		else
			[checkBox setState:NSOffState];
		
        [checkBox setEnabled:enabled];
	}
}

- (PGPBoolean)getCheckbox:(NSButton*)checkBox
{
	if (!checkBox)
		return FALSE;
	
	return [checkBox state] == NSOnState ? TRUE : FALSE;
}

- (IBAction)unmountAfterDelay:(id)sender
{
	if ([mAutoUnmountAfterDelay state] == NSOnState)
	{
		[mAutoUnmountDelay setEnabled:YES];
		[mDelayStepper setEnabled:YES];
	}
	else
	{
		[mAutoUnmountDelay setEnabled:NO];
		[mDelayStepper setEnabled:NO];
	}
}

- (IBAction)unmountOnSleep:(id)sender
{
	if ([mUnmountOnSleep state] == NSOnState)
		[mPreventSleep setEnabled:YES];
	else
		[mPreventSleep setEnabled:NO];
}

- (IBAction)stepperPressed:(id)sender
{
   int				increment;
    int				currentSize;
    int				stepperValue;
    
	currentSize = [mAutoUnmountDelay intValue];
	stepperValue = [mDelayStepper intValue];

	if (stepperValue < 0)
		increment = -1;
	else
		increment = 1;

	[mDelayStepper setIntValue:0];

	if ((currentSize + increment) < 1)
		[mAutoUnmountDelay setIntValue:1];
	else if ((currentSize + increment) <= 999)
		[mAutoUnmountDelay setIntValue:(currentSize + increment)];
}

@end
