/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskControllerUserClient.cpp,v 1.3 2002/08/06 20:09:34 dallen Exp $
____________________________________________________________________________*/
#include <IOKit/IOService.h>
#include <IOKit/IOLib.h>
#include <IOKit/system.h>
#include "PGPdiskDriver.h"
#include "PGPdiskController.h"
#include "PGPdiskControllerUserClient.h"

#define super IOUserClient
OSDefineMetaClassAndStructors(PGPdiskControllerUserClient, IOUserClient)

	bool
PGPdiskControllerUserClient::initWithTask(
	task_t			owningTask,
	void			*securityToken,
	UInt32			type )
{
	STATUS_LOG(( "PGPdiskControllerUserClient::initWithTask\n" ));
	
	if( !super::init() )
		return false;
	
	STATUS_LOG(( "   tagging fClient as 0x%08lX\n", (UInt32)owningTask ));
	fClient = owningTask;
	task_reference(fClient);
	
	return true;
}

	IOReturn
PGPdiskControllerUserClient::clientClose( void )
{
	STATUS_LOG(( "PGPdiskControllerUserClient::clientClose\n" ));
	
	if( fService )
	{
		fService->close( this );  // Aborts any outstanding I/O getData
		
		// This should happen automatically but it doesn't yet.
		// be warned this detach will be unnecessary in the future and
		// will probably cause your driver to panic in the future.
		detach( fService );
		fService = NULL;
	}
	
	// Better release our client task reference
	if( fClient )
	{
		task_deallocate( fClient );
		fClient = NULL;
	}
	return kIOReturnSuccess;
}

	IOReturn 
PGPdiskControllerUserClient::message(
	UInt32			type,
	IOService		*provider,
	void			*argument )
{   
	IOReturn		res = kIOReturnSuccess;
	
	switch (type)
	{
		case kIOMessageServiceIsTerminated:
			if( provider )
			{
				STATUS_LOG(( "PGPdiskControllerUserClient::message: kIOMessageServiceIsTerminated from %s\n", provider->getName() ));
				provider->close(this);
			}
			break;
		default:
			res = super::message(type, provider, argument);
			break;
	}
	return res;
}

	bool
PGPdiskControllerUserClient::start(
	IOService *provider)
{
	STATUS_LOG(( "PGPdiskControllerUserClient::start\n" ));
	if( !super::start( provider ) )
		return false;
	
	fService = OSDynamicCast( PGPdiskController, provider);
	if( !fService )	
		return false;
	if( !fService->open( this ) )
		goto abort;
	
	sMethods[kPGPdiskControllerUserClient_CreateDisk].object = fService;
	sMethods[kPGPdiskControllerUserClient_CreateDisk].func = (IOMethod)&PGPdiskController::createDisk;
	sMethods[kPGPdiskControllerUserClient_CreateDisk].count0 = sizeof(PGPdiskCreateDiskParams);
	sMethods[kPGPdiskControllerUserClient_CreateDisk].count1 = 0;
	sMethods[kPGPdiskControllerUserClient_CreateDisk].flags = kIOUCStructIStructO;
	
	STATUS_LOG(( "PGPdiskControllerUserClient::start succeeded\n" ));
	return true;	
abort:
	fService = NULL;
	return false;
}

	IOExternalMethod *
PGPdiskControllerUserClient::getTargetAndMethodForIndex(
	IOService		**target,
	UInt32			index )
{
	STATUS_LOG(( "PGPdiskControllerUserClient::getTargetAndMethodForIndex(%ld)\n", index ));
	if( index < (UInt32) kPGPdiskControllerUserClient_NumCommands )
	{
		*target = sMethods[index].object;
		return &sMethods[index];
	}
	else
		return 0;
}

	void
PGPdiskControllerUserClient::free( void )
{
	STATUS_LOG(( "PGPdiskControllerUserClient::free\n" ));
	super::free();
}

