/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskDevice.cpp,v 1.4 2002/08/06 20:09:34 dallen Exp $
____________________________________________________________________________*/
#include <IOKit/IOLib.h>
#include "PGPdiskDriver.h"
#include "PGPdiskDevice.h"

#define super IOBlockStorageDevice
OSDefineMetaClassAndStructors(PGPdiskDevice, IOBlockStorageDevice)

	bool
PGPdiskDevice::attach(
	IOService	*provider )
{
	if( !super::attach( provider ) )
		return false;
	STATUS_LOG(( "PGPdiskDevice::attach\n" ));
	
	fProvider = OSDynamicCast( PGPdiskDriver, provider );
	if( fProvider == NULL )
	{
		STATUS_LOG(( "PGPdiskDevice: attach; wrong provider type!\n" ));
		return false;
	}
	return true;
}

	void
PGPdiskDevice::detach(
	IOService	*provider )
{
	STATUS_LOG(( "PGPdiskDevice::detach\n" ));
	if( fProvider == provider )
		fProvider = 0;
	super::detach( provider );
}

	void
PGPdiskDevice::stop(
	IOService	*provider )
{
	STATUS_LOG(( "PGPdiskDevice::stop\n" ));
	super::stop( provider );
}

	void
PGPdiskDevice::free( void )
{
	STATUS_LOG(( "PGPdiskDevice::free\n" ));
	super::free();
}

	IOReturn 
PGPdiskDevice::message(
	UInt32		type,
	IOService	*provider,
	void		*argument )
{   
	IOReturn	res = kIOReturnSuccess;
	
	switch (type)
	{         
		case kIOMessageServiceIsTerminated:
			if( provider )
			{
				STATUS_LOG(( "PGPdiskDevice::message: kIOMessageServiceIsTerminated from %s\n", provider->getName() ));
				provider->close(this);
			}
			break;
		default:
			res = super::message(type, provider, argument);
			break;
	}
	return res;
}

	IOReturn
PGPdiskDevice::doAsyncReadWrite(
	IOMemoryDescriptor		*buffer,
	UInt32					block,
	UInt32					nblks,
	IOStorageCompletion		completion )
{
	if( isInactive( ) != false )
	{
		return kIOReturnNotAttached;
	}
	return fProvider->doAsyncReadWrite( buffer, block, nblks, completion );
}

	IOReturn
PGPdiskDevice::doSyncReadWrite(
	IOMemoryDescriptor		*buffer,
	UInt32					block,
	UInt32					nblks )
{
	if( isInactive( ) != false )
	{
		return kIOReturnNotAttached;
	}
	return fProvider->doSyncReadWrite( buffer, block, nblks );
}

	IOReturn
PGPdiskDevice::doEjectMedia( void )
{
	return fProvider->doEjectMedia( );
}

	IOReturn
PGPdiskDevice::doFormatMedia( UInt64 byteCapacity )
{
	return fProvider->doFormatMedia( byteCapacity );
}

	UInt32
PGPdiskDevice::doGetFormatCapacities(
	UInt64		*capacities,
	UInt32   	capacitiesMaxCount ) const
{
	return fProvider->doGetFormatCapacities( capacities, capacitiesMaxCount );
}

	IOReturn
PGPdiskDevice::doLockUnlockMedia( bool doLock )
{
	return fProvider->doLockUnlockMedia( doLock );
}

	IOReturn
PGPdiskDevice::doSynchronizeCache( void )
{
	if( isInactive( ) != false )
	{
		return kIOReturnNotAttached;
	}
	return fProvider->doSynchronizeCache( );
}

	char *
PGPdiskDevice::getVendorString( void )
{
	return fProvider->getVendorString( );
}

	char *
PGPdiskDevice::getProductString( void )
{
	return fProvider->getProductString( );
}

	char *
PGPdiskDevice::getRevisionString( void )
{
	return fProvider->getRevisionString( );
}

	char *
PGPdiskDevice::getAdditionalDeviceInfoString( void )
{
	return fProvider->getAdditionalDeviceInfoString( );
}

	IOReturn
PGPdiskDevice::reportBlockSize( 
	UInt64		*blockSize )
{
	return fProvider->reportBlockSize( blockSize );
}

	IOReturn
PGPdiskDevice::reportEjectability(
	bool		*isEjectable )
{
	return fProvider->reportEjectability( isEjectable );
}

	IOReturn
PGPdiskDevice::reportLockability(
	bool		*isLockable )
{
	return fProvider->reportLockability( isLockable );
}

	IOReturn
PGPdiskDevice::reportPollRequirements(
	bool		*pollIsRequired,
	bool		*pollIsExpensive )
{
	return fProvider->reportPollRequirements( pollIsRequired, pollIsExpensive );
}

	IOReturn
PGPdiskDevice::reportMaxReadTransfer(
	UInt64		blockSize,
	UInt64		*max )
{
	return fProvider->reportMaxReadTransfer( blockSize, max );
}

	IOReturn
PGPdiskDevice::reportMaxValidBlock(
	UInt64		*maxBlock )
{
	return fProvider->reportMaxValidBlock( maxBlock );
}

	IOReturn
PGPdiskDevice::reportMaxWriteTransfer(
	UInt64		blockSize,
	UInt64		*max )
{
	return fProvider->reportMaxWriteTransfer( blockSize, max );
}

	IOReturn
PGPdiskDevice::reportMediaState(
	bool		*mediaPresent,
	bool		*changed )	
{
	return fProvider->reportMediaState( mediaPresent, changed );
}

	IOReturn
PGPdiskDevice::reportRemovability(
	bool		*isRemovable )
{
	return fProvider->reportRemovability( isRemovable );
}

	IOReturn
PGPdiskDevice::reportWriteProtection(
	bool	*isWriteProtected )
{
	return fProvider->reportWriteProtection( isWriteProtected );
}

