/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskDriver.h,v 1.16 2002/10/17 06:59:30 wprice Exp $
____________________________________________________________________________*/
#pragma once

#include <IOKit/storage/IOBlockStorageDriver.h>
#include <IOKit/IOLocks.h>
#include <mach/semaphore.h>
#include "PGPdiskControllerUserClient.h"
#include "PGPdiskDriverUserClient.h"

#define kPGPdiskDriver_ThrottleMaxPending	4

#if ( PGP_DEBUG == 1 )
#define STATUS_LOG(x)			IOLog x ; IOSleep(20)
#define SERIAL_STATUS_LOG(x) 	kprintf x
#else
#define STATUS_LOG(x)
#define SERIAL_STATUS_LOG(x)
#endif

#define kPGPdiskDriver_VendorString			"PGP Corporation"
#define kPGPdiskDriver_ModelString 			"PGPdisk Image Driver"
#define kPGPdiskDriver_ProductString		"PGPdisk"
#define kPGPdiskDriver_RevisionString		"8.0"
#define kPGPdiskDriver_AdditionalString		"[PGPdisk]"

typedef struct PGPdiskWorkQueueEntry
{
	PGPdiskWorkQueueEntry	*next;
	bool					sent;
	IOMemoryDescriptor		*ioBuffer;
	UInt32					startBlock;
	UInt32					numBlocks;
	UInt32					blocksDone;
	IOStorageCompletion		completion;
} PGPdiskWorkQueueEntry;

class PGPdiskDriver : public IOService
{
	OSDeclareDefaultStructors(PGPdiskDriver)

protected:
	UInt32					fDiskID;
	UInt64					fDiskSize;
	bool					fDiskWasLastPresent;
	bool					fLocked;
	bool					fReadOnly;
	bool					fLogoutEject;
	bool					fAutoDiskMount;
	bool					fActivated;
	OSData					*fIconPath;
	void					*fIOBuffer;
	IOLock					*fInDiskControlLock;
	IOLock					*fWorkQueueLock;
	semaphore_t				fWorkQueueSemaphore;
	semaphore_t				fThrottleSemaphore;
	PGPdiskWorkQueueEntry	*fWorkQueue;

public:
	virtual bool		initPGPdisk(	const PGPdiskCreateDiskParams	*diskParams,
										UInt32							diskID,
										OSDictionary					*dictionary = 0 );
	virtual bool		start( IOService *provider );
	virtual void		stop( IOService *provider );
	virtual void		free( void );
	virtual bool		createNub( void );
	virtual IOReturn	activateDrive(	UInt32							*diskID );
	virtual IOReturn	diskControl(	const PGPdiskIOParamBlock		*diskIOIn,
										PGPdiskIOParamBlock				*diskIOOut,
										IOMemoryDescriptor				*dataMD );
	virtual IOReturn	message( UInt32 type, IOService * provider, void * argument );

	virtual IOReturn	doAsyncReadWrite( IOMemoryDescriptor * 	buffer,
										UInt32					block,
										UInt32					nblks,
										IOStorageCompletion		completion );
	
	virtual IOReturn	doSyncReadWrite( IOMemoryDescriptor *	buffer,
										UInt32					block,
										UInt32					nblks );
    virtual IOReturn	doEjectMedia(void);
    virtual IOReturn	doFormatMedia(UInt64 byteCapacity);
    virtual UInt32		doGetFormatCapacities(UInt64 			*capacities,
											UInt32   			capacitiesMaxCount) const;
    virtual IOReturn	doLockUnlockMedia(bool doLock);
    virtual IOReturn	doSynchronizeCache(void);
    virtual char *		getVendorString(void);
    virtual char *		getProductString(void);
    virtual char *		getRevisionString(void);
    virtual char *		getAdditionalDeviceInfoString(void);
    virtual IOReturn	reportBlockSize(UInt64 *blockSize);
    virtual IOReturn	reportEjectability(bool *isEjectable);
    virtual IOReturn	reportLockability(bool *isLockable);
    virtual IOReturn	reportMaxReadTransfer(UInt64 blockSize,UInt64 *max);
    virtual IOReturn	reportMaxWriteTransfer(UInt64 blockSize,UInt64 *max);
    virtual IOReturn	reportMaxValidBlock(UInt64 *maxBlock);
    virtual IOReturn	reportMediaState(bool *mediaPresent,bool *changedState);
    virtual IOReturn	reportPollRequirements(bool *pollRequired,
                                            bool *pollIsExpensive);
    virtual IOReturn	reportRemovability(bool *isRemovable);
    virtual IOReturn	reportWriteProtection(bool *isWriteProtected);
};

