/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: cast5.s,v 1.2 2002/08/06 20:09:35 dallen Exp $
____________________________________________________________________________*/


// 4567890123456789012345678901234567890123456789012345678901234567890
// Register conventions:
// r0 - volatile
// r1 - stack pointer, preserved
// r2 - TOC pointer, preserved
// r3 - First argument (and return value)
// r4-r10 - More arguments, volatile
// r11-r12 - volatile
// r13-r31 - preserved
// LR - return address, volatile
// CTR, XER and MQ are all volatile
//
// The calling stack has some slots reserved for us to use, but
// the interesting part is that there are a few hundred bytes
// *below* the current stack pointer (the so-called "red zone")
// that interrupt handlers won't use and we can use to save out
// registers.

/****************************************************
 * CAST5encrypt
 ****************************************************/

#define t1		r3
#define left	r6
#define right	r7
#define t2		r8
#define s1		r9
#define s2		r10
#define s3		r11
#define s4		r12
#define t3		r13	// Needs to be saved somewhere
#define t4		r14	// Needs to be saved somewhere

// The CAST round.  op1/op2/op3 are add/xor/subf in rotating order.
// These do round n from x into y.
#define ROUND(x,y,op1,op2,op3,n) \
	op3		r0,x,r0			;\
	rlwnm	r0,r0,t1,0,31	;\
	rlwinm	t1,r0,10,22,29	;\
	lwzx	t2,s1,t1		;\
	rlwinm	t1,r0,18,22,29	;\
	lwzx	t3,s2,t1		;\
	rlwinm	t1,r0,26,22,29	;\
	lwzx	t4,s3,t1		;\
	rlwinm	t1,r0,2,22,29	;\
	op1		t2,t3,t2		;\
	lwzx	t3,s4,t1		;\
	op2		t2,t4,t2		;\
	lwz		r0,8*n+8(xkey)	;\
	op3		t2,t3,t2		;\
	lwz		t1,8*n+12(xkey)	;\
	xor		y,t2,y
	
#define SP		r1
#define RTOC	r2

#define in		r3
#define	out		r4
#define xkey	r5

	.globl	_CAST5encrypt
_CAST5encrypt:
	lwz		left,0(in)	// PPC is big-endian, as is CAST,
	lwz		right,4(in) // and unaligned loads are okay
	stw		t3,-8(SP)
	stw		t4,-4(SP)

	/* Load Sboxes into s1..s4 */
	addis	s1,0,hi16(_CASTsboxS1)
	ori		s1,s1,lo16(_CASTsboxS1)
	lwz		s1,0(s1)
	addis	s2,0,hi16(_CASTsboxS2)
	ori		s2,s2,lo16(_CASTsboxS2)
	lwz		s2,0(s2)
	addis	s3,0,hi16(_CASTsboxS3)
	ori		s3,s3,lo16(_CASTsboxS3)
	lwz		s3,0(s3)
	addis	s4,0,hi16(_CASTsboxS4)
	ori		s4,s4,lo16(_CASTsboxS4)
	lwz		s4,0(s4)

	lwz		r0,0(xkey)
	lwz		t1,4(xkey)

	add		r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	xor		t2,t3,t2
	lwzx	t3,s4,t1
	subf	t2,t4,t2
	lwz		r0,8*0+8(xkey)
	add		t2,t3,t2
	lwz		t1,8*0+12(xkey)
	xor		left,t2,left
	
//	ROUND(right, left, xor, subf, add, 0)
	// 1: ROUND(left, right, subf, add, xor, 1)
	xor		r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	subf	t2,t3,t2
	lwzx	t3,s4,t1
	add		t2,t4,t2
	lwz		r0,8*1+8(xkey)
	xor		t2,t3,t2
	lwz		t1,8*1+12(xkey)
	xor		right,t2,right

	// 2: ROUND(right, left, add, xor, subf, 2)
	subf	r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	add		t2,t3,t2
	lwzx	t3,s4,t1
	xor		t2,t4,t2
	lwz		r0,8*2+8(xkey)
	subf	t2,t3,t2
	lwz		t1,8*2+12(xkey)
	xor		left,t2,left

	// 3: ROUND(left, right, xor, subf, add, 3)
	add		r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	xor		t2,t3,t2
	lwzx	t3,s4,t1
	subf	t2,t4,t2
	lwz		r0,8*3+8(xkey)
	add		t2,t3,t2
	lwz		t1,8*3+12(xkey)
	xor		right,t2,right

	// 4: ROUND(right, left, subf, add, xor, 4)
	xor		r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	subf	t2,t3,t2
	lwzx	t3,s4,t1
	add		t2,t4,t2
	lwz		r0,8*4+8(xkey)
	xor		t2,t3,t2
	lwz		t1,8*4+12(xkey)
	xor		left,t2,left
	
	// 5: ROUND(left, right, add, xor, subf, 5)
	subf	r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	add		t2,t3,t2
	lwzx	t3,s4,t1
	xor		t2,t4,t2
	lwz		r0,8*5+8(xkey)
	subf	t2,t3,t2
	lwz		t1,8*5+12(xkey)
	xor		right,t2,right
	
	// 6: ROUND(right, left, xor, subf, add, 6)
	add		r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	xor		t2,t3,t2
	lwzx	t3,s4,t1
	subf	t2,t4,t2
	lwz		r0,8*6+8(xkey)
	add		t2,t3,t2
	lwz		t1,8*6+12(xkey)
	xor		left,t2,left
	
	// 7: ROUND(left, right, subf, add, xor, 7)
	xor		r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	subf	t2,t3,t2
	lwzx	t3,s4,t1
	add		t2,t4,t2
	lwz		r0,8*7+8(xkey)
	xor		t2,t3,t2
	lwz		t1,8*7+12(xkey)
	xor		right,t2,right
	
	// 8: ROUND(right, left, add, xor, subf, 8)
	subf	r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	add		t2,t3,t2
	lwzx	t3,s4,t1
	xor		t2,t4,t2
	lwz		r0,8*8+8(xkey)
	subf	t2,t3,t2
	lwz		t1,8*8+12(xkey)
	xor		left,t2,left
	
	// 9: ROUND(left, right, xor, subf, add, 9)
	add		r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	xor		t2,t3,t2
	lwzx	t3,s4,t1
	subf	t2,t4,t2
	lwz		r0,8*9+8(xkey)
	add		t2,t3,t2
	lwz		t1,8*9+12(xkey)
	xor		right,t2,right
	
	// 10: ROUND(right, left, subf, add, xor, 10)
	xor		r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	subf	t2,t3,t2
	lwzx	t3,s4,t1
	add		t2,t4,t2
	lwz		r0,8*10+8(xkey)
	xor		t2,t3,t2
	lwz		t1,8*10+12(xkey)
	xor		left,t2,left
	
	// 11: ROUND(left, right, add, xor, subf, 11)
	subf	r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	add		t2,t3,t2
	lwzx	t3,s4,t1
	xor		t2,t4,t2
	lwz		r0,8*11+8(xkey)
	subf	t2,t3,t2
	lwz		t1,8*11+12(xkey)
	xor		right,t2,right
	
	// 12: ROUND(right, left, xor, subf, add, 12)
	add		r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	xor		t2,t3,t2
	lwzx	t3,s4,t1
	subf	t2,t4,t2
	lwz		r0,8*12+8(xkey)
	add		t2,t3,t2
	lwz		t1,8*12+12(xkey)
	xor		left,t2,left
	
	// 13: ROUND(left, right, subf, add, xor, 13)
	xor		r0,left,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	subf	t2,t3,t2
	lwzx	t3,s4,t1
	add		t2,t4,t2
	lwz		r0,8*13+8(xkey)
	xor		t2,t3,t2
	lwz		t1,8*13+12(xkey)
	xor		right,t2,right
	
	// 14: ROUND(right, left, add, xor, subf, 14)
	subf	r0,right,r0
	rlwnm	r0,r0,t1,0,31
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	t4,s3,t1
	rlwinm	t1,r0,2,22,29
	add		t2,t3,t2
	lwzx	t3,s4,t1
	xor		t2,t4,t2
	lwz		r0,8*14+8(xkey)
	subf	t2,t3,t2
	lwz		t1,8*14+12(xkey)
	xor		left,t2,left
	
//	ROUND(left, right, xor, subf, add, 15)

// Last round: omit loading of keys for next round
	add		r0,left,r0
	stw		left,4(out)
	rlwnm	r0,r0,t1,0,31
	lwz		t4,-4(SP)
	rlwinm	t1,r0,10,22,29
	lwzx	t2,s1,t1
	rlwinm	t1,r0,18,22,29
	lwzx	t3,s2,t1
	rlwinm	t1,r0,26,22,29
	lwzx	left,s3,t1
	rlwinm	t1,r0,2,22,29
	xor		t2,t3,t2
	lwzx	t3,s4,t1
	subf	t2,left,t2
	add		t2,t3,t2
	lwz		t3,-8(SP)
	xor		right,t2,right
	
	stw		right,0(out)
	blr


// Get rid of those #defines...
#undef SP
#undef RTOC

#undef in
#undef out
#undef xkey

#undef ROUND
#undef left
#undef right
#undef s1
#undef s2
#undef s3
#undef s4
#undef t1
#undef t2
#undef t3
#undef t4

/****************************************************
 * CAST5encryptCFBdbl
 ****************************************************/

#define ROUND(x,y1,y2,b1,b2,op1,op2,op3,n) \
	lwzx   t2,s2,t2;	rlwinm t3,tb,26,22,29;		op3    ta,x,key;		\
	lwzx   t3,s3,t3;	rlwinm tb,tb,2,22,29;		rlwnm  ta,ta,rot,0,31;	\
	op3    t2,t2,t1;	lwzx   tb,s4,tb;			rlwinm t1,ta,10,22,29;	\
	op1    t3,t3,t2;				lwzx t1,s1,t1;	rlwinm t2,ta,18,22,29;	\
	op2    tb,tb,t3;				lwzx t2,s2,t2;	rlwinm t3,ta,26,22,29;	\
	xor    b2,b1,tb;				lwzx t3,s3,t3;	rlwinm ta,ta,2,22,29;	\
	op3    tb,b2,key;				op1  t2,t2,t1;	lwzx   ta,s4,ta;		\
	rlwnm  tb,tb,rot,0,31;			op2  t3,t3,t2;	lwz key,8*n+8(xkey);	\
	rlwinm t1,tb,10,22,29;			op3  ta,ta,t3;	lwz rot,8*n+12(xkey);	\
	lwzx   t1,s1,t1;	rlwinm t2,tb,18,22,29;		xor	   y2,y1,ta;		\

#define out0 r10
#define out1 r11
#define out2 r12
#define out3 r13
#define s1	r14
#define s2	r15
#define s3	r16
#define s4	r17

#define ta	r18
#define tb	r19
#define	t1	r20
#define	t2	r21
#define	t3	r22

#define	key	r23
#define	rot	r24

#define sp		r1
#define RTOC	r2

#define xkey	r3
#define in0		r4
#define in1		r5
#define in2		r6
#define in3		r7
#define src		r8
#define dest	r9
#define len		r10

	.globl	_CAST5encryptCFBdbl
_CAST5encryptCFBdbl:
	mtctr	len					// Repeat len times....
	stw		r13,-48(sp)
	stw		r14,-44(sp)
	stw		r15,-40(sp)
	stw		r16,-36(sp)
	stw		r17,-32(sp)
	stw		r18,-28(sp)
	stw		r19,-24(sp)
	stw		r20,-20(sp)
	stw		r21,-16(sp)
	stw		r22,-12(sp)
	stw		r23,-8(sp)
	stw		r24,-4(sp)

	lwz		key,0(xkey)
	lwz		rot,4(xkey)


	addis	s1,0,hi16(_CASTsboxS1)
	ori		s1,s1,lo16(_CASTsboxS1)
	lwz		s1,0(s1)
	
	// The core CAST function starts here
	
	addis	s2,0,hi16(_CASTsboxS2)
	ori		s2,s2,lo16(_CASTsboxS2)
	lwz		s2,0(s2)
													add    ta,in1,key
	addis	s3,0,hi16(_CASTsboxS3)
	ori		s3,s3,lo16(_CASTsboxS3)
	lwz		s3,0(s3)
													rlwnm  ta,ta,rot,0,31
	addis	s4,0,hi16(_CASTsboxS4)
	ori		s4,s4,lo16(_CASTsboxS4)
	lwz		s4,0(s4)
													rlwinm t1,ta,10,22,29
									lwzx t1,s1,t1
									rlwinm t2,ta,18,22,29
									lwzx t2,s2,t2
									rlwinm t3,ta,26,22,29
	b		CAST5encryptCFBdbl_entry

CAST5encryptCFBdbl_loop:
	subf   t3,t3,t2
							/**/					lwz key,0(xkey)
	add    tb,tb,t3
							/**/					lwz rot,4(xkey)
	xor    out2,out2,tb
							/**/					lwz in2,8(src)
	xor in2,out2,in2
							/**/					lwz in3,12(src)
	stw in0,0(dest)
							/**/					xor in3,out3,in3
	stw in1,4(dest)
							/**/					add    ta,in1,key
	stw in2,8(dest)
							/**/					rlwnm  ta,ta,rot,0,31
	stw in3,12(dest)
							/**/					rlwinm t1,ta,10,22,29
	addi src,src,16
							lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	addi dest,dest,16
							lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
CAST5encryptCFBdbl_entry:
									lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,in3,key
									xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
									subf t3,t3,t2
													lwz key,8(xkey)
	rlwinm t1,tb,10,22,29
									add  ta,ta,t3
													lwz rot,12(xkey)
	lwzx   t1,s1,t1
									rlwinm t2,tb,18,22,29
													xor    out1,in0,ta

//	ROUND(in1,  in0,  out1, XXX,  in3,  xor, subf, add, 0)
	// 1: ROUND(out1, in1,  out0, in2,  out3, subf, add, xor, 1)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,in2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out3,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*1+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*1+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,in1,ta

	// 2: ROUND(out0, out1, out1, in3,  out2, add, xor, subf, 2)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,in3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out2,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*2+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*2+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 3: ROUND(out1, out0, out0, out3, out3, xor, subf, add, 3)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out3,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*3+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*3+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 4: ROUND(out0, out1, out1, out2, out2, subf, add, xor, 4)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out2,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*4+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*4+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 5: ROUND(out1, out0, out0, out3, out3, add, xor, subf, 5)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out3,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*5+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*5+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 6: ROUND(out0, out1, out1, out2, out2, xor, subf, add, 6)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out2,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*6+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*6+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 7: ROUND(out1, out0, out0, out3, out3, subf, add, xor, 7)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out3,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*7+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*7+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 8: ROUND(out0, out1, out1, out2, out2, add, xor, subf, 8)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out2,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*8+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*8+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 9: ROUND(out1, out0, out0, out3, out3, xor, subf, add, 9)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out3,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*9+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*9+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 10: ROUND(out0, out1, out1, out2, out2, subf, add, xor, 10)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out2,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*10+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*10+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 11: ROUND(out1, out0, out0, out3, out3, add, xor, subf, 11)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out3,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*11+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*11+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 12: ROUND(out0, out1, out1, out2, out2, xor, subf, add, 12)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out2,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*12+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*12+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 13: ROUND(out1, out0, out0, out3, out3, subf, add, xor, 13)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out3,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*13+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*13+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 14: ROUND(out0, out1, out1, out2, out2, add, xor, subf, 14)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out2,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*14+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*14+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

//	ROUND(out1, out0, out0, out3, out3, xor, subf, add, 15)
	// Final wrapup
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf   tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out3,key
						xor  t2,t2,t1
													lwzx	ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf t3,t3,t2
													lwz in0,0(src)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz in1,4(src)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	out0,out0,ta
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor in0,out0,in0
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													xor in1,out1,in1
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													/**/

	bdnz	CAST5encryptCFBdbl_loop

	subf   t3,t3,t2
												lwz in2,8(src)
	add    tb,tb,t3
												lwz in3,12(src)
	xor    out2,out2,tb
												stw in0,0(dest)
	xor in2,out2,in2
												stw in1,4(dest)
	xor in3,out3,in3
												stw in2,8(dest)
												stw in3,12(dest)

// Done.  Now reload all the saved registers.  Since we've been
// memory-bound for a long time, there's no advantage to scheduling this.

	lwz		r13,-48(sp)
	lwz		r14,-44(sp)
	lwz		r15,-40(sp)
	lwz		r16,-36(sp)
	lwz		r17,-32(sp)
	lwz		r18,-28(sp)
	lwz		r19,-24(sp)
	lwz		r20,-20(sp)
	lwz		r21,-16(sp)
	lwz		r22,-12(sp)
	lwz		r23,-8(sp)
	lwz		r24,-4(sp)

	blr

/****************************************************
 * CAST5decryptCFBdbl
 ****************************************************/

#define xkey	r3
#define in0		r4
#define in1		r5
#define in2		r6
#define in3		r7
#define src		r8
#define dest	r9
#define len		r10

	.globl	_CAST5decryptCFBdbl
_CAST5decryptCFBdbl:
	mtctr	len					// Repeat len times....
	
	stw		r13,-48(sp)
	stw		r14,-44(sp)
	stw		r15,-40(sp)
	stw		r16,-36(sp)
	stw		r17,-32(sp)
	stw		r18,-28(sp)
	stw		r19,-24(sp)
	stw		r20,-20(sp)
	stw		r21,-16(sp)
	stw		r22,-12(sp)
	stw		r23,-8(sp)
	stw		r24,-4(sp)

	lwz		key,0(xkey)
	lwz		rot,4(xkey)

	addis	s1,0,hi16(_CASTsboxS1)
	ori		s1,s1,lo16(_CASTsboxS1)
	lwz     s1,0(s1)
	
	// The core CAST function starts here
	
	addis	s2,0,hi16(_CASTsboxS2)
	ori		s2,s2,lo16(_CASTsboxS2)
	lwz     s2,0(s2)
													add    ta,in1,key
	addis	s3,0,hi16(_CASTsboxS3)
	ori		s3,s3,lo16(_CASTsboxS3)
	lwz     s3,0(s3)
													rlwnm  ta,ta,rot,0,31
	addis	s4,0,hi16(_CASTsboxS4)
	ori		s4,s4,lo16(_CASTsboxS4)
	lwz     s4,0(s4)
													rlwinm t1,ta,10,22,29
									lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
									lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	b		CAST5decryptCFBdbl_entry
	
CAST5decryptCFBdbl_loop:
	subf   t3,t3,t2
							/**/					 lwz key,0(xkey)
	add    tb,tb,t3
							/**/					 lwz rot,4(xkey)
	xor    out2,out2,tb
							/**/					 lwz in2,8(src)
	 xor out2,out2,in2
							/**/					 lwz in3,12(src)
	 stw out0,0(dest)
							/**/					 xor out3,out3,in3
	 stw out1,4(dest)
							/**/					add    ta,in1,key
	 stw out2,8(dest)
							/**/					rlwnm  ta,ta,rot,0,31
	 stw out3,12(dest)
							/**/					rlwinm t1,ta,10,22,29
	 addi src,src,16
							lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	 addi dest,dest,16
							lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
CAST5decryptCFBdbl_entry:
									lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,in3,key
									xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
									subf t3,t3,t2
													lwz key,8(xkey)
	rlwinm t1,tb,10,22,29
									add  ta,ta,t3
													lwz rot,12(xkey)
	lwzx   t1,s1,t1
									rlwinm t2,tb,18,22,29
													xor    out1,in0,ta

//	ROUND(in1,  in0,  out1, XXX,  in3,  xor, subf, add, 0)
	// 1: ROUND(out1, in1,  out0, in2,  out3, subf, add, xor, 1)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,in2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out3,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*1+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*1+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,in1,ta

	// 2: ROUND(out0, out1, out1, in3,  out2, add, xor, subf, 2)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,in3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out2,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*2+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*2+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 3: ROUND(out1, out0, out0, out3, out3, xor, subf, add, 3)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out3,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*3+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*3+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 4: ROUND(out0, out1, out1, out2, out2, subf, add, xor, 4)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out2,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*4+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*4+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 5: ROUND(out1, out0, out0, out3, out3, add, xor, subf, 5)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out3,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*5+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*5+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 6: ROUND(out0, out1, out1, out2, out2, xor, subf, add, 6)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out2,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*6+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*6+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 7: ROUND(out1, out0, out0, out3, out3, subf, add, xor, 7)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out3,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*7+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*7+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 8: ROUND(out0, out1, out1, out2, out2, add, xor, subf, 8)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out2,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*8+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*8+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 9: ROUND(out1, out0, out0, out3, out3, xor, subf, add, 9)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out3,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*9+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*9+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 10: ROUND(out0, out1, out1, out2, out2, subf, add, xor, 10)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out2,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*10+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*10+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 11: ROUND(out1, out0, out0, out3, out3, add, xor, subf, 11)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out3,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*11+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*11+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 12: ROUND(out0, out1, out1, out2, out2, xor, subf, add, 12)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out2,key
						xor  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf  t3,t3,t2
													lwz key,8*12+8(xkey)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz rot,8*12+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

	// 13: ROUND(out1, out0, out0, out3, out3, subf, add, xor, 13)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	xor    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	subf    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	add    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	xor    tb,out3,key
						subf  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						add  t3,t3,t2
													lwz key,8*13+8(xkey)
	rlwinm t1,tb,10,22,29
						xor  ta,ta,t3
													lwz rot,8*13+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out0,out0,ta

	// 14: ROUND(out0, out1, out1, out2, out2, add, xor, subf, 14)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													subf    ta,out0,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	subf    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	add    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	xor    tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out2,out2,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	subf    tb,out2,key
						add  t2,t2,t1
													lwzx   ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						xor  t3,t3,t2
													lwz key,8*14+8(xkey)
	rlwinm t1,tb,10,22,29
						subf  ta,ta,t3
													lwz rot,8*14+12(xkey)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	   out1,out1,ta

//	ROUND(out1, out0, out0, out3, out3, xor, subf, add, 15)
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													add    ta,out1,key
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													rlwnm  ta,ta,rot,0,31
	add    t2,t2,t1
						lwzx   tb,s4,tb
													rlwinm t1,ta,10,22,29
	xor    t3,t3,t2
						lwzx t1,s1,t1
													rlwinm t2,ta,18,22,29
	subf   tb,tb,t3
						lwzx t2,s2,t2
													rlwinm t3,ta,26,22,29
	xor    out3,out3,tb
						lwzx t3,s3,t3
													rlwinm ta,ta,2,22,29
	add    tb,out3,key
						xor  t2,t2,t1
													lwzx	ta,s4,ta
	rlwnm  tb,tb,rot,0,31
						subf t3,t3,t2
													lwz in0,0(src)
	rlwinm t1,tb,10,22,29
						add  ta,ta,t3
													lwz in1,4(src)
	lwzx   t1,s1,t1
						rlwinm t2,tb,18,22,29
													xor	out0,out0,ta

// Final wrapup.  Steps that are part of the CFB (as opposed to CAST)
// are indented one space.  /**/ marks an instruction slot.
// Note the "bdz" buried in there, just before the cycle that starts
// loading keys to restart.  Since all PPCs have a separate branch unit,
// having two branches takes no time, and it simplifies it considerably.
	lwzx   t2,s2,t2
						rlwinm t3,tb,26,22,29
													xor out0,out0,in0
	lwzx   t3,s3,t3
						rlwinm tb,tb,2,22,29
													xor out1,out1,in1
	xor    t2,t2,t1
						lwzx   tb,s4,tb				 /**/

	bdnz	CAST5decryptCFBdbl_loop	// Should take 0 cycles

// exit:
	subf   t3,t3,t2
												lwz in2,8(src)
	add    tb,tb,t3
												lwz in3,12(src)
	xor    out2,out2,tb
												stw out0,0(dest)
	xor out2,out2,in2
												stw out1,4(dest)
	xor out3,out3,in3
												stw out2,8(dest)
												stw out3,12(dest)

// Done.  Now reload all the saved registers.  Since we've been
// memory-bound for a long time, there's no advantage to scheduling this.

	lwz		r13,-48(sp)
	lwz		r14,-44(sp)
	lwz		r15,-40(sp)
	lwz		r16,-36(sp)
	lwz		r17,-32(sp)
	lwz		r18,-28(sp)
	lwz		r19,-24(sp)
	lwz		r20,-20(sp)
	lwz		r21,-16(sp)
	lwz		r22,-12(sp)
	lwz		r23,-8(sp)
	lwz		r24,-4(sp)

	blr
