/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpDiskEngineDisk.h,v 1.28 2002/10/06 08:46:52 dallen Exp $
____________________________________________________________________________*/
#ifndef Included_pgpDiskEngineDisk_h	/* [ */
#define Included_pgpDiskEngineDisk_h

#include "pgpPubTypes.h"
#include "pgpErrors.h"
#include "pgpDiskBase.h"
#include "pgpDiskEngineTypes.h"


	PGPError
PGPdiskEngineInit( void );

	PGPError
PGPdiskEngineCleanup( void );

/*
 * Note: PGPdiskEngineCreateDisk and PGPdiskEngineOpenDisk need 
 * access to a PGPContextRef that will outlive the current connection,
 * so a thread-specific PGPContextRef will not work.  We need to pass
 * in a context that is guaranteed to be around for the entire
 * life of PGPdiskEngine.
 */
	PGPError
PGPdiskEngineCreateDisk(
	PGPdiskBEContextRef	diskcon,
	const char *		pszPath,
	PGPUInt64			ullSize,
	const char *		pszDefaultMountPoint,
	PGPdiskEncryptionAlgorithm	alg,
	PGPBoolean			bPubKeyUser,
	PGPKeyDBObjRef		key,
	const char *		pszUsername,
	const char *		pszPassphrase,
	PGPInt32			callbackSock,
	PGPdiskIODiskRef *	outDisk );

	PGPError
PGPdiskEngineOpenDisk(
	PGPdiskBEContextRef		diskcon,
	const char *			filename,
	PGPBoolean				readonly,
	PGPdiskIODiskRef *		outDisk );

	PGPError
PGPdiskEngineCloseDisk(
	PGPdiskIODiskRef		disk );

	PGPError
PGPdiskEngineConvertOS9Disk(
	PGPdiskBEContextRef		diskcon,
	const char *			pszOriginalFilename,
	const char *			pszAdminPassphrase,
	const char *			pszNewFilename,
	PGPInt32				callbackSock );

	PGPError
PGPdiskEngineReEncryptDisk(
	PGPdiskBEContextRef	diskcon,
	PGPdiskIODiskRef	disk,
	char *				pszPassphrase,
	PGPdiskEncryptionAlgorithm	newAlg,
	PGPInt32			callbackSock );
	
	PGPError
PGPdiskEngineGetIODiskFilename(
	PGPdiskIODiskRef	disk,
	char *				filename );

	PGPError
PGPdiskEngineDecryptDisk(
	PGPdiskBEContextRef	diskcon,
	PGPdiskIODiskRef	disk,
	PGPdiskIODiskUser *	user,
	const char *		passphrase );

	PGPUInt64
PGPdiskEngineGetDiskSize(
	PGPdiskIODiskRef		disk );
	
	PGPError
PGPdiskEngineCountUsers(
	PGPdiskIODiskRef		disk,
	PGPUInt32 *				outNumUsers );
	
	PGPError
PGPdiskEngineGetIODiskUser(
	PGPdiskBEContextRef		diskcon,
	PGPdiskIODiskRef		disk,
	PGPUInt32				userNumber,
	PGPdiskBEUserRef		user );
	
	PGPdiskIODiskUser *
PGPdiskEngineGetAdminUser(
	PGPdiskIODiskRef		disk );

	PGPBoolean
PGPdiskEngineIsUserOnDisk(
	PGPdiskIODiskRef		disk,
	PGPdiskBEUserRef		user );
	
	PGPError
PGPdiskEngineAddUser(
	PGPdiskBEContextRef		diskcon,
	PGPdiskIODiskRef		disk,
	PGPdiskIODiskUser *		adminUser,
	const char *			adminPassphrase,
	PGPdiskBEUserRef		user );
	
	PGPError
PGPdiskEngineRemoveUser(
	PGPdiskIODiskRef		disk,
	PGPdiskIODiskUser *		user );

	PGPError
PGPdiskEngineIsPassphraseCorrect(
	PGPdiskBEContextRef		diskcon,
	PGPdiskIODiskUser *		user,
	const char *			pszPassphrase,
	PGPBoolean *			bCorrect );

	PGPError
PGPdiskEngineChangeUserPassphrase(
	PGPdiskBEContextRef		diskcon,
	PGPdiskIODiskUser *		user,
	const char *			oldPassphrase,
	const char *			newPassphrase );

	PGPError
PGPdiskEngineSetAdminUser(
	PGPdiskIODiskRef		disk,
	PGPdiskIODiskUser *		newAdminUser );

	PGPError
PGPdiskEngineBEUserModified(
	PGPdiskBEUserRef		user );

	PGPError
PGPdiskEngineGetDiskBooleanProperty(
	PGPdiskIODiskRef		disk,
	PGPUInt32				prop,
	PGPBoolean *			bProperty );

	PGPError
PGPdiskEngineSetDiskBooleanProperty(
	PGPdiskIODiskRef		disk,
	PGPUInt32				prop,
	PGPBoolean				bProperty );

	PGPError
PGPdiskEngineGetDiskNumericProperty(
	PGPdiskIODiskRef		disk,
	PGPUInt32				prop,
	PGPInt32 *				iProperty );

	PGPError
PGPdiskEngineSetDiskNumericProperty(
	PGPdiskIODiskRef		disk,
	PGPUInt32				prop,
	PGPInt32 				iProperty );

	PGPError
PGPdiskEngineGetDiskDataProperty(
	PGPdiskIODiskRef		disk,
	PGPUInt32				prop,
	char **					pszProperty );

	PGPError
PGPdiskEngineSetDiskDataProperty(
	PGPdiskIODiskRef		disk,
	PGPUInt32				prop,
	char *					pszProperty );

	PGPError
PGPdiskEngineSetDiskMounted(
	PGPdiskIODiskRef		disk,
	PGPBoolean				bReadOnly );

	PGPError
PGPdiskEngineSetDiskUnmounted(
	PGPdiskIODiskRef		disk );

	PGPError
PGPdiskEngineUpdateGlobalDiskTimeout(
	PGPUInt32				timeout );

	PGPUInt32
PGPdiskEngineNumMountedDisks( void );

	PGPError
PGPdiskEngineUnmountAllDisks( 
	PGPBoolean				bForced );

	PGPError
PGPdiskEngineSetDiskDeviceName(
	PGPdiskIODiskRef		disk,
	char *					pszDeviceName );

	PGPError
PGPdiskEngineSetDiskKernelConnectionContext(
	PGPdiskIODiskRef					disk,
	PGPdiskKernelConnectionContext *	driverConn );

	PGPError
PGPdiskEngineGetDiskKernelConnectionContext(
	PGPdiskIODiskRef					disk,
	PGPdiskKernelConnectionContext **	driverConn );
	
	PGPError
PGPdiskEngineReadFromDisk(
	PGPdiskIODiskRef		disk,
	PGPUInt64				startBlock,
	PGPUInt32				numBlocks,
	PGPByte *				data );

	PGPError
PGPdiskEngineWriteToDisk(
	PGPdiskIODiskRef		disk,
	PGPUInt64				startBlock,
	PGPUInt32				numBlocks,
	PGPByte *				data );


#endif /* ] Included_pgpDiskEngineDisk_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
