/*
 *  pgpDiskAppMainTest.c
 *  pgpdiskapp
 *
 *  Created by dave on Thu Apr 05 2001.
 *  Copyright (c) 2001 __CompanyName__. All rights reserved.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "pgpUtilities.h"
#include "pgpKeys.h"
#include "pgpErrors.h"
#include "pgpDiskEngine.h"

#define CKERR		if( IsPGPError( err ) ) goto done


#define DISKPATH	"/Users/dave/disk.pgd"
#define DISKSIZE	10 * 1024 * 1024 - 8 * 512

#define KEYIDSTRING1	"0xFCDCC2B6"
#define KEYIDSTRING2	"0xCBB79A31"
#define ADD_SYM_USER 0

PGPError
callback( 
	PGPBoolean	isFormatting,
	PGPUInt32 percent,
	void * userValue )
{
	printf( "\r%d%% done", percent );
	fflush( stdout );
	return kPGPError_NoErr;
}

PGPError
reencrypt_callback( 
	PGPUInt32 percent,
	void * userValue )
{
	printf( "\r%d%% done", percent );
	fflush( stdout );
	return kPGPError_NoErr;
}

int main( int argc, char *argv[] )
{
	PGPError			err			= kPGPError_NoErr;
	PGPContextRef		context		= kInvalidPGPContextRef;
	PGPdiskContextRef	diskcon		= kInvalidPGPdiskContextRef;
	PGPdiskDiskRef		disk		= kInvalidPGPdiskDiskRef;
	char				filename[PATH_MAX];
	PGPUInt32			i			= 0;
	PGPBoolean			bCreate 	= FALSE;
	PGPBoolean			bAddUser	= FALSE;
	PGPBoolean			bRemoveUser	= FALSE;
	PGPBoolean			bMount		= FALSE;
	PGPBoolean			bUnmount	= FALSE;
	PGPBoolean			bUserProps	= FALSE;
	PGPBoolean			bDiskProps	= FALSE;
	PGPBoolean			bReEncrypt	= FALSE;
	PGPBoolean			bAddSymUser	= ADD_SYM_USER;
	PGPdiskEncryptionAlgorithm	alg	= kPGPdiskCAST5Algorithm;
	PGPUInt32			userNum		= 0;
	PGPdiskUserSetRef	userset 	= kInvalidPGPdiskUserSetRef;
	PGPdiskUserIterRef	useriter	= kInvalidPGPdiskUserIterRef;
	PGPdiskUserRef		user		= kInvalidPGPdiskUserRef;
	char				username[128];
	PGPBoolean			bProp		= FALSE;
	PGPUInt32			uProp		= 0;
	char				szProp[512];
	char				szPublicFilename[PATH_MAX];
	char				szPrivateFilename[PATH_MAX];
	PGPFileSpecRef		pubFile		= kInvalidPGPFileSpecRef;
	PGPFileSpecRef		privFile	= kInvalidPGPFileSpecRef;
	PGPKeyDBRef			keydb		= kInvalidPGPKeyDBRef;
	PGPKeyDBObjRef		key			= kInvalidPGPKeyDBObjRef;
	PGPKeyID			keyid;
	
	strcpy( filename, DISKPATH );
	
	for( i = 0; i < argc; i++ )
	{
		if( strcmp( argv[i], "--create" ) == 0 )
			bCreate = TRUE;
		if( strcmp( argv[i], "--adduser" ) == 0 )
			bAddUser = TRUE;
		if( strcmp( argv[i], "--removeuser" ) == 0 )
		{
			bRemoveUser = TRUE;
			userNum = atoi( argv[++i] );
			continue;
		}
		if( strcmp( argv[i], "--mount" ) == 0 )
			bMount = TRUE;
		if( strcmp( argv[i], "--unmount" ) == 0 )
			bUnmount = TRUE;
		if( strcmp( argv[i], "--addsym" ) == 0 )
			bAddSymUser = TRUE;
		if( strcmp( argv[i], "--addpub" ) == 0 )
			bAddSymUser = FALSE;
		if( strcmp( argv[i], "--diskprops" ) == 0 )
			bDiskProps = TRUE;
		if( strcmp( argv[i], "--reencrypt" ) == 0 )
			bReEncrypt = TRUE;
		if( strcmp( argv[i], "--aes" ) == 0 )
			alg = kPGPdiskAESAlgorithm;
		if( strcmp( argv[i], "--cast" ) == 0 )
			alg = kPGPdiskCAST5Algorithm;
		if( strcmp( argv[i], "--userprops" ) == 0 )
		{
			bUserProps = TRUE;
			userNum = atoi( argv[++i] );
			continue;
		}
		if( strcmp( argv[i], "--disk" ) == 0 )
		{
			strcpy( filename, argv[++i] );
			continue;
		}
	}

	if( !PGPdiskIsEngineRunning() )
	{
		printf( "PGPdiskEngine is not running\n" );
		exit( 0 );
	}

	printf( "PGPdiskEngine test app starting\n" );

	err = PGPsdkInit( 0 ); CKERR;
	err = PGPNewContext( kPGPsdkAPIVersion, &context ); CKERR;

	err = PGPdiskNewContext( context, kPGPdiskAPIVersion, &diskcon ); CKERR;

	printf( "Opening keyrings\n" );
	sprintf( szPublicFilename, "%s/.pgp/pubring.pkr", getenv( "HOME" ) );
	err = PGPNewFileSpecFromFullPath( context, szPublicFilename, &pubFile ); CKERR;
	sprintf( szPrivateFilename, "%s/.pgp/secring.skr", getenv( "HOME" ) );
	err = PGPNewFileSpecFromFullPath( context, szPrivateFilename, &privFile ); CKERR;
	
	err = PGPOpenKeyDBFile( context, 
		kPGPOpenKeyDBFileOptions_Create,
		pubFile, privFile, &keydb ); CKERR;
		
	printf( "Telling engine to load keyrings\n" );
	err = PGPdiskLoadKeyrings( diskcon, szPublicFilename, szPrivateFilename ); CKERR;
	
	err = PGPdiskContextSetForcedAutoDiskUnmount( diskcon, TRUE ); CKERR;
	err = PGPdiskContextSetUnmountOnSleep( diskcon, TRUE ); CKERR;

	if( bCreate )
	{
		err = PGPNewKeyIDFromString( KEYIDSTRING1, kPGPPublicKeyAlgorithm_Invalid, &keyid ); CKERR;
		err = PGPFindKeyByKeyID( keydb, &keyid, &key );
		if( IsPGPError( err ) )
		{
			err = PGPNewKeyIDFromString( KEYIDSTRING2, kPGPPublicKeyAlgorithm_Invalid, &keyid ); CKERR;
			err = PGPFindKeyByKeyID( keydb, &keyid, &key ); CKERR;
		}

		printf( "Creating disk\n" );
		err = PGPdiskCreateDisk( diskcon,
			filename,
			DISKSIZE,
			"H:\\ (unused)",
			alg,
			bAddSymUser ? FALSE : TRUE,		/* public key user? */
			key,
			"admin user",
			"adminpassphrase",
			callback, NULL,
			&disk ); CKERR;
		printf( "\n" );
	}
	else
	{
		printf( "Opening disk\n" );
		err = PGPdiskOpenDisk( diskcon, filename, FALSE, &disk ); CKERR;
	}
	
	if( bAddUser )
	{
		printf( "Exporting user set\n" );
		err = PGPdiskNewUserSet( diskcon, &userset ); CKERR;
		err = PGPdiskDiskExportUserSet( disk, userset ); CKERR;
	
		printf( "Adding user to user set\n" );
		if( bAddSymUser )
		{
			err = PGPdiskNewUserWithSymmetricKey( userset,
				"another user",
				"newuserpassphrase",
				FALSE,
				&user ); CKERR;
		}
		else
		{
			err = PGPNewKeyIDFromString( KEYIDSTRING1, kPGPPublicKeyAlgorithm_Invalid, &keyid ); CKERR;
			err = PGPFindKeyByKeyID( keydb, &keyid, &key );
			if( IsPGPError( err ) )
			{
				err = PGPNewKeyIDFromString( KEYIDSTRING2, kPGPPublicKeyAlgorithm_Invalid, &keyid ); CKERR;
				err = PGPFindKeyByKeyID( keydb, &keyid, &key ); CKERR;
			}
			
			err = PGPdiskNewUserWithPublicKey( userset,
				key,
				FALSE,
				FALSE,
				&user ); CKERR;
		}
		
		printf( "Re-importing user set\n" );
		err = PGPdiskDiskImportUserSet( disk, "invalidpassphrase", userset, user ); CKERR;
	}

	if( bRemoveUser )
	{
		printf( "Exporting user set\n" );
		err = PGPdiskNewUserSet( diskcon, &userset ); CKERR;
		err = PGPdiskDiskExportUserSet( disk, userset ); CKERR;
	
		printf( "Removing user from user set\n" );
		err = PGPdiskNewUserIter( userset, &useriter ); CKERR;
		err = PGPdiskUserIterMove( useriter, userNum, TRUE, &user ); CKERR;
		err = PGPdiskUserSetRemoveUser( userset, user ); CKERR;
	
		printf( "Re-importing user set\n" );
		err = PGPdiskUserIterMove( useriter, 0, TRUE, &user ); CKERR;
		err = PGPdiskDiskImportUserSet( disk, "dss", userset, user ); CKERR;
	}

	if( bUserProps )
	{
		printf( "Exporting user set\n" );
		err = PGPdiskNewUserSet( diskcon, &userset ); CKERR;
		err = PGPdiskDiskExportUserSet( disk, userset ); CKERR;
		err = PGPdiskNewUserIter( userset, &useriter ); CKERR;
		err = PGPdiskUserIterMove( useriter, userNum, TRUE, &user ); CKERR;

		err = PGPdiskGetUserDataProp( user, kPGPdiskUserProp_UserName, username, sizeof( username ), NULL ); CKERR;
		printf( "Name: %s\n", username );
		err = PGPdiskGetUserBooleanProp( user, kPGPdiskUserProp_HasSymmetricKey, &bProp ); CKERR;
		if( bProp )
			printf( "Symmetric Key user\n" );
		err = PGPdiskGetUserBooleanProp( user, kPGPdiskUserProp_HasPublicKey, &bProp ); CKERR;
		if( bProp )
			printf( "Public Key user\n" );
		err = PGPdiskGetUserBooleanProp( user, kPGPdiskUserProp_IsReadOnly, &bProp ); CKERR;
		printf( "Read only: %s\n", bProp ? "TRUE" : "FALSE" );
		err = PGPdiskGetUserBooleanProp( user, kPGPdiskUserProp_IsAdminUser, &bProp ); CKERR;
		printf( "Admin user: %s\n", bProp ? "TRUE" : "FALSE" );
		
		err = PGPdiskIsUserPassphraseCorrect( user, "test", &bProp ); CKERR;
		if( bProp )
			printf( "Passphrase is \"test\"\n" );
		err = PGPdiskIsUserPassphraseCorrect( user, "adminpassphrase", &bProp ); CKERR;
		if( bProp )
			printf( "Passphrase is \"adminpassphrase\"\n" );

	}
	
	if( bDiskProps )
	{
		err = PGPdiskGetDiskBooleanProp( disk, kPGPdiskDiskProp_IsMounted, &bProp ); CKERR;
		printf( "Is Mounted: %s\n", bProp ? "TRUE" : "FALSE" );
		err = PGPdiskGetDiskBooleanProp( disk, kPGPdiskDiskProp_IsBeingReEncrypted, &bProp ); CKERR;
		printf( "Is Being ReEncrypted: %s\n", bProp ? "TRUE" : "FALSE" );
		err = PGPdiskGetDiskNumericProp( disk, kPGPdiskDiskProp_Algorithm, &uProp ); CKERR;
		printf( "Algorithm: %d\n", uProp );
		err = PGPdiskGetDiskNumericProp( disk, kPGPdiskDiskProp_DiskFormat, &uProp ); CKERR;
		printf( "Disk Format: %d\n", uProp );
		err = PGPdiskGetDiskDataProp( disk, kPGPdiskDiskProp_Path, szProp, sizeof( szProp ), NULL ); CKERR;
		printf( "Path: %s\n", szProp );
		err = PGPdiskGetDiskDataProp( disk, kPGPdiskDiskProp_DeviceName, szProp, sizeof( szProp ), NULL ); CKERR;
		printf( "Device name: %s\n", szProp );
	}
	
	if( bReEncrypt )
	{
		printf( "Re-encrypting disk\n" );
		err = PGPdiskReEncryptDisk( disk, "test", alg, reencrypt_callback, NULL ); CKERR;
		printf( "\n" );
	}

	if( bMount )
	{
		printf( "Mounting disk\n" );
		err = PGPdiskMountDisk( disk, "mount point (unused)", "testtest", FALSE ); CKERR;
	}

	if( bUnmount )
	{
		printf( "Unounting disk\n" );
		err = PGPdiskUnmountDisk( disk, TRUE ); CKERR;
	}
	
	/*
	err = PGPdiskConvertDisk( diskcon, kPGPdiskDiskFormat_OS9, "/Users/dave/PGPdisk1", "testtest", "/Users/dave/disk.pgd", reencrypt_callback, NULL ); CKERR;
	*/
	
	printf( "PGPdiskEngine test app quitting\n" );

done:
	if( IsPGPError( err ) )
		printf( "Exiting with error %d\n", err );
		
	if( PGPdiskUserSetRefIsValid( userset ) )
		PGPdiskFreeUserSet( userset );
	if( PGPdiskUserIterRefIsValid( useriter ) )
		PGPdiskFreeUserIter( useriter );
	
	PGPdiskCloseDisk( disk );
	PGPdiskFreeContext( diskcon );
	
	if( PGPKeyDBRefIsValid( keydb ) )
		(void) PGPFreeKeyDB( keydb );
	if( PGPFileSpecRefIsValid( pubFile ) )
		(void) PGPFreeFileSpec( pubFile );
	if( PGPFileSpecRefIsValid( privFile ) )
		(void) PGPFreeFileSpec( privFile );
	
	PGPFreeContext( context );
	PGPsdkCleanup();
	
	return 0;
}
