/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeySearch.m,v 1.23 2002/09/22 07:43:16 wprice Exp $
____________________________________________________________________________*/
#import "CKeySearch.h"
#import "CSearchPanel.h"
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pflPrefTypes.h>
#import <PGPclient/pgpMem.h>
#import "PGPkeysCommon.h"
#import "PGPkeysController.h"

#define kCKeySearchNib				@"CKeySearch"
#define kCKeySearchSearchString		LSPK( @"Search", @"Search button title" )
#define kCKeySearchCancelString		LSPK( @"Cancel", @"Cancel button title" )

	static PGPError
serverCallback(
	PGPContextRef 	context,
	PGPEvent 		*event,
	PGPUserValue 	userValue)
{
	CKeySearch		*keySearch = (CKeySearch *)userValue;
	
	return [keySearch serverCallback:event];
}

@implementation CKeySearch

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeySearchNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initWithController:(id)controller
{
	PGPInt32	selection;
	
	[self init];
	mController = controller;
	mPGPContext = [controller pgpContext];
	mTLSContext = [controller tlsContext];
	mPrefRef	= [controller pgpPrefRef];

	mRunning = NO;
	[mSearchProgress retain];
	[mStatusField retain];
	[mSearchProgress removeFromSuperview];
	[mStatusField removeFromSuperview];
	mPGPkeysView = [[CPGPkeysView alloc] initWithController:self mainDB:NO search:YES];
	[[mPGPkeysView view] setFrameSize:[mKeyView frame].size];
	[mKeyView addSubview:[mPGPkeysView view]];
	
	[self loadSourcePopup];
	PGPGetPrefNumber( mPrefRef, kPGPPrefLastSearchWindowSelection, &selection );
	[mSourcePopup selectItemAtIndex:selection];

	mSearchPanelArray = [[NSMutableArray alloc] init];
	mLastSearchType = kKeySearchType_Size;
	[self addSearchPanel];
	
	[mSearchWindow center];
	[mSearchWindow makeKeyAndOrderFront:self];
	[self fixKeyViews:YES];
	return self;
}

- (PGPFilterRef)searchFilter
{
	NSEnumerator		*panelEnumerator = [mSearchPanelArray objectEnumerator];
	CSearchPanel		*panel;
	PGPError			err;
	PGPFilterRef		filter = kInvalidPGPFilterRef,
						filter2 = kInvalidPGPFilterRef;
	
	while( panel = [panelEnumerator nextObject] )
	{
		filter2 = [panel filter];
		if( !PGPFilterRefIsValid( filter2 ) )
		{
			if( PGPFilterRefIsValid( filter ) )
				PGPFreeFilter( filter );
			filter = kInvalidPGPFilterRef;
			break;
		}
		if( PGPFilterRefIsValid( filter ) )
		{
			err = PGPIntersectFilters( filter, filter2, &filter );
			pgpAssertNoErr( err );
		}
		else
			filter = filter2;
	}
	return filter;
}

- (void)fixKeyViews:(BOOL)selectLast
{
	NSMutableArray		*keyViewArray = [NSMutableArray array];
	CSearchPanel		*panel;
	PGPInt32			panelIndex,
						panelCount = [mSearchPanelArray count],
						keyViewIndex,
						keyViewCount,
						lastPanelItem;
	
	[keyViewArray addObject:mSourcePopup];
	for( panelIndex = 0; panelIndex < panelCount; panelIndex++ )
	{
		panel = [mSearchPanelArray objectAtIndex:panelIndex];
		[keyViewArray addObjectsFromArray:[panel keyViews]];
	}
	lastPanelItem = [keyViewArray count] - 1;
	[keyViewArray addObject:mMoreChoicesButton];
	[keyViewArray addObject:mFewerChoicesButton];
	[keyViewArray addObject:mPendingCheckbox];
	[keyViewArray addObject:mSearchButton];
	[keyViewArray addObject:mClearButton];

	keyViewCount = [keyViewArray count];
	for( keyViewIndex = 0; keyViewIndex < keyViewCount; keyViewIndex++ )
	{
		if( keyViewCount == 1 )
			[[keyViewArray objectAtIndex:keyViewIndex] setNextKeyView:[keyViewArray objectAtIndex:keyViewIndex]];
		else if( keyViewIndex + 1 < keyViewCount )
			[[keyViewArray objectAtIndex:keyViewIndex] setNextKeyView:[keyViewArray objectAtIndex:keyViewIndex + 1]];
		else
			[[keyViewArray objectAtIndex:keyViewIndex] setNextKeyView:[keyViewArray objectAtIndex:0]];
	}
	if( selectLast )
		[mSearchWindow makeFirstResponder:[keyViewArray objectAtIndex:lastPanelItem]];
	
}

- (void)addSearchPanel
{
	NSEnumerator		*panelEnumerator = [mSearchPanelArray objectEnumerator];
	CSearchPanel		*panel,
						*otherPanel;
	NSRect				moveRect;
	float				diff;
	NSSize				minSize = [mSearchWindow minSize];
	
	mLastSearchType = ( mLastSearchType + 1 ) % kKeySearchTypeCount;
	panel = [[CSearchPanel alloc] initWithController:self defaultType:mLastSearchType];
	diff = [[panel view] frame].size.height;
	[mSearchPanelArray addObject:panel];
	[panel release];

	moveRect = [mKeyView frame];
	moveRect.size.height -= diff;
	[mKeyView setFrame:moveRect];

	moveRect = [mSearchWindow frame];
	moveRect.origin.y -= diff;
	moveRect.size.height += diff;
	[mSearchWindow setFrame:moveRect display:YES animate:YES];
	minSize.height += diff;
	[mSearchWindow setMinSize:minSize];

	moveRect = [mChoicesView frame];
	moveRect.origin.y -= diff;
	moveRect.size.height += diff;
	[mChoicesView setFrame:moveRect];
	while( otherPanel = [panelEnumerator nextObject] )
	{
		moveRect = [[otherPanel view] frame];
		moveRect.origin.y += diff;
		[[otherPanel view] setFrame:moveRect];
	}
	
	[mCriteriaBox addSubview:[panel view]];
	[mFewerChoicesButton setEnabled:([mSearchPanelArray count] > 1)];
	
	[self fixKeyViews:YES];
}

- (void)removeSearchPanel
{
	NSEnumerator		*panelEnumerator;
	CSearchPanel		*panel = [mSearchPanelArray lastObject],
						*otherPanel;
	NSRect				moveRect;
	float				diff = [[panel view] frame].size.height;
	NSSize				minSize = [mSearchWindow minSize];
	
	if( [mSearchPanelArray count] <= 1 )
		return;
	[[panel view] removeFromSuperview];
	[mSearchPanelArray removeLastObject];
	
	panelEnumerator = [mSearchPanelArray objectEnumerator];
	while( otherPanel = [panelEnumerator nextObject] )
	{
		moveRect = [[otherPanel view] frame];
		moveRect.origin.y -= diff;
		[[otherPanel view] setFrame:moveRect];
	}

	moveRect = [mChoicesView frame];
	moveRect.origin.y += diff;
	moveRect.size.height -= diff;
	[mChoicesView setFrame:moveRect];

	moveRect = [mSearchWindow frame];
	moveRect.origin.y += diff;
	moveRect.size.height -= diff;
	[mSearchWindow setFrame:moveRect display:YES animate:YES];
	minSize.height -= diff;
	[mSearchWindow setMinSize:minSize];

	moveRect = [mKeyView frame];
	moveRect.size.height += diff;
	[mKeyView setFrame:moveRect];
	[mFewerChoicesButton setEnabled:([mSearchPanelArray count] > 1)];
	
	[self fixKeyViews:YES];
}

- (void)loadSourcePopup
{
	PGPInt32	numItems = [mSourcePopup numberOfItems],
				serverIndex;
	
	while( numItems > 2 )
		[mSourcePopup removeItemAtIndex:--numItems];
	[[mSourcePopup menu] addItem:[NSMenuItem separatorItem]];
	if( IsntNull( mServers ) )
	{
		PGPFreeKeyServerList( mServers );
		mServers = NULL;
	}
	if( IsntPGPError( PGPGetKeyServerPrefs( mPrefRef, &mServers, &mNumServers ) ) )
	{
		char	url[kMaxServerNameLength + 1];
		
		for( serverIndex = 0; serverIndex < mNumServers; serverIndex++ )
		{
			if( IsKeyServerListed( mServers[serverIndex].flags ) )
			{
				PGPGetKeyServerURL( &mServers[serverIndex], url );
				[mSourcePopup addItemWithTitle:[NSString stringWithCString:url]];
			}
		}
	}
}

- (IBAction)moreChoicesAction:(id)sender;
{
	if( !mRunning )
		[self addSearchPanel];
}

- (IBAction)fewerChoicesAction:(id)sender;
{
	if( !mRunning )
		[self removeSearchPanel];
}

- (void)endSearch:(PGPKeyDBRef)resultDB
{
	NSEnumerator	*panelEnumerator = [mSearchPanelArray objectEnumerator];
	CSearchPanel	*panel;

	if( mOp == kKeySearchOp_Search )
		[mPGPkeysView loadKeys:resultDB showKeySet:kInvalidPGPKeySetRef];
	[mSearchButton setTitle:kCKeySearchSearchString];
	[mClearButton setEnabled:YES];
	[mSourcePopup setEnabled:YES];
	if( [mSearchPanelArray count] > 1 )
		[mFewerChoicesButton setEnabled:YES]; 
	[mMoreChoicesButton setEnabled:YES];
	[mPendingCheckbox setEnabled:YES];
	while( panel = [panelEnumerator nextObject] )
		[panel setEnabled:YES];
	[mSearchProgress stopAnimation:self];
	[mSearchProgress removeFromSuperview];
	[mStatusField removeFromSuperview];
	if( PGPFilterRefIsValid( mServerFilter ) )
		PGPFreeFilter( mServerFilter );
	mServerFilter = kInvalidPGPFilterRef;
	mRunning = NO;
}

- (PGPKeyServerEntry *)selectedServer
{
	PGPInt32	serverIndex,
				listedServerIndex,
				selectedIndex;
	
	selectedIndex = [mSourcePopup indexOfSelectedItem] - 2;
	for( listedServerIndex = serverIndex = 0; serverIndex < mNumServers; serverIndex++ )
	{
		if( IsKeyServerListed( mServers[serverIndex].flags ) )
			listedServerIndex++;
		if( listedServerIndex == selectedIndex )
			return &mServers[serverIndex];
	}
	return NULL;
}

- (PGPKeyDBObjRef)defaultKey
{
	PGPKeyID		*keyID;
	PGPSize			keyIDLen;
	PGPKeyDBObjRef	defaultKey = kInvalidPGPKeyDBObjRef;
	
	if( IsntPGPError( PGPGetPrefData( mPrefRef, kPGPPrefDefaultKeyID, &keyIDLen, (void **)&keyID ) ) )
	{
		PGPFindKeyByKeyID( [mController mainDB], keyID, &defaultKey );
		PGPFreeData( keyID );
	}
	return defaultKey;
}

- (PGPError)serverCallback:(PGPEvent *)event
{
	PGPError		err = kPGPError_NoErr;
	
	switch( event->type )
	{
		case kPGPEvent_KeyServerEvent:
			if( mLastState != event->data.keyServerData.state )
			{
				NSString	*statusString = nil;
				
				switch( event->data.keyServerData.state )
				{
					case kPGPKeyServerState_Opening:
						statusString = LSPK( @"Opening", @"Server status:Opening" );
						break;
					case kPGPKeyServerState_Querying:
						statusString = LSPK( @"Querying", @"Server status:Querying" );
						break;
					case kPGPKeyServerState_ReceivingResults:
						statusString = LSPK( @"Receiving", @"Server status:Receiving" );
						break;
					case kPGPKeyServerState_ProcessingResults:
						statusString = LSPK( @"Processing", @"Server status:Processing" );
						break;
					case kPGPKeyServerState_Uploading:
						statusString = LSPK( @"Sending", @"Server status:Sending" );
						break;
					case kPGPKeyServerState_Deleting:
						statusString = LSPK( @"Deleting", @"Server status:Deleting" );
						break;
					case kPGPKeyServerState_Disabling:
						statusString = LSPK( @"Disabling", @"Server status:Disabling" );
						break;
					case kPGPKeyServerState_Closing:
						statusString = LSPK( @"Closing", @"Server status:Closing" );
						break;
					default:
						break;
				}
				if( statusString )
				{
					[mStatusField performSelectorOnMainThread:@selector(setStringValue:)
						withObject:statusString waitUntilDone:NO];
				}
				mLastState = event->data.keyServerData.state;
			}
			break;
		case kPGPEvent_KeyServerSignEvent:
		{
			PGPKeyDBObjRef	authKey = kInvalidPGPKeyDBObjRef;
			char			*passphrase = NULL;
			
			err = PGPSigningPassphraseDialog( mPGPContext,
						[mController mainDB], &authKey,
						PGPOUIDefaultKey( mPGPContext, [self defaultKey] ),
						PGPOUIOutputPassphrase( mPGPContext, &passphrase ),
						PGPOLastOption( mPGPContext ) );
			if( IsntPGPError( err ) && IsntNull( passphrase ) )
			{
				err = PGPAddJobOptions( event->job,
						PGPOSignWithKey( mPGPContext, authKey,
							PGPOPassphraseBuffer( mPGPContext, passphrase, strlen( passphrase ) ),
							PGPOLastOption( mPGPContext ) ),
						PGPOClearSign( mPGPContext, TRUE ),
						PGPOLastOption( mPGPContext ) );
				PGPFreeData( passphrase );
			}
			break;
		}
		case kPGPEvent_KeyServerTLSEvent:
		{
			PGPtlsSessionRef		tls = event->data.keyServerTLSData.tlsSession;
			PGPtlsCipherSuiteNum	tlsCipher;
			PGPKeyDBObjRef			remoteKey;
			PGPInt32				remoteKeyAlg;
			PGPKeyID				keyID,
									expectedID;
			char					url[kMaxServerNameLength + 1];
			PGPKeyServerEntry		*serverEntry = [self selectedServer];
			
			err = PGPtlsGetRemoteAuthenticatedKey( tls, &remoteKey, NULL );
			if( IsPGPError( err ) )
				break;
			PGPtlsGetNegotiatedCipherSuite( tls, &tlsCipher );
			[mStatusField setStringValue:LSPK( @"Authenticating", @"Server status:Authenticating" )];
			err = PGPGetKeyID( remoteKey, &keyID );
			if( IsPGPError( err ) )
				break;
			err = PGPGetKeyDBObjNumericProperty( remoteKey, kPGPKeyProperty_AlgorithmID, &remoteKeyAlg );
			if( IsPGPError( err ) )
				break;
			if( serverEntry->authAlg != kPGPPublicKeyAlgorithm_Invalid )
				err = PGPNewKeyIDFromString( serverEntry->authKeyIDString, serverEntry->authAlg, &expectedID );
			if( IsPGPError( err ) )
				break;
			if( ( serverEntry->authAlg == kPGPPublicKeyAlgorithm_Invalid ) ||
				PGPCompareKeyIDs( &expectedID, &keyID ) )
			{
				PGPGetKeyServerURL( serverEntry, url );
				err = PGPclConfirmRemoteKeyDialog( [LSPK(@"The server key is not recognized. Please confirm that the server is authenticating with the correct key.", @"Search Window TLS Key Changed") cString],
						url, remoteKey, tlsCipher, TRUE );
				if( IsntPGPError( err ) )
				{
					err = PGPGetKeyIDString( &keyID, kPGPKeyIDString_Full, serverEntry->authKeyIDString );
					serverEntry->authAlg = remoteKeyAlg;
					PGPSetKeyServerPrefs( mPrefRef, mServers, mNumServers );
				}
			}
			break;
		}
		case kPGPEvent_KeyServerIdleEvent:
			break;
		default:
			break;
	}
	return err;
}

- (void)queryServer:(id)sender
{
	NSAutoreleasePool				*pool = [[NSAutoreleasePool alloc] init];
	PGPKeyServerEntry				*serverEntry = [self selectedServer];
	PGPError						err = kPGPError_NoErr;
	PGPKeyServerAccessType			accessType = kPGPKeyServerAccessType_Administrator;
	PGPKeyServerKeySpace			keySpace = kPGPKeyServerKeySpace_Normal;
	PGPKeyDBRef						resultDB = kInvalidPGPKeyDBRef;
	PGPKeySetRef					failedKeys = kInvalidPGPKeySetRef,
									keySet = kInvalidPGPKeySetRef;
	PGPKeyServerRef					server = kInvalidPGPKeyServerRef;
	PGPtlsSessionRef				tls = kInvalidPGPtlsSessionRef;
	PGPKeySetRef					actionKeySet;
	PGPKeySearchOp					op;
	
	mLastState = kPGPKeyServerState_Invalid;
	actionKeySet = mActionKeySet;
	op = mOp;

	if( [mPendingCheckbox state] )
		keySpace = kPGPKeyServerKeySpace_Pending;
	if( IsNull( serverEntry ) )
		goto done;
	err = PGPNewKeyServer( mPGPContext, serverEntry->type, &server,
				PGPONetHostName( mPGPContext, serverEntry->serverDNS, serverEntry->serverPort ),
				PGPOKeyServerProtocol( mPGPContext, serverEntry->protocol ),
				PGPOKeyServerKeySpace( mPGPContext, keySpace ),
				PGPOKeyServerAccessType( mPGPContext, accessType ),
				PGPOLastOption( mPGPContext ) );
	pgpAssertNoErr( err );
	if( IsntPGPError( err ) )
	{
		mCurrentServer = server;
		PGPSetKeyServerEventHandler( server, serverCallback, self );
		if( serverEntry->protocol == kPGPKeyServerProtocol_LDAPS )
		{
			err = PGPNewTLSSession( mTLSContext, &tls );
			pgpAssertNoErr( err );
		}
		if( IsntPGPError( err ) && ( op != kKeySearchOp_Search ) && PGPtlsSessionRefIsValid( tls ) )
		{
			PGPKeyDBObjRef	authKey		= kInvalidPGPKeyDBObjRef;
			char			*passphrase	= NULL;
			
			err = PGPSigningPassphraseDialog( mPGPContext,
						[mController mainDB], &authKey,
						PGPOUIDefaultKey( mPGPContext, [self defaultKey] ),
						PGPOUIOutputPassphrase( mPGPContext, &passphrase ),
						PGPOLastOption( mPGPContext ) );
			if( IsntPGPError( err ) )
				err = PGPtlsSetLocalPrivateKey( tls, authKey,
						kInvalidPGPKeySetRef,
						PGPOPassphrase( mPGPContext, passphrase ),
						PGPOLastOption( mPGPContext ) );
			if( IsntNull( passphrase ) )
				PGPFreeData( passphrase );
		}
		err = PGPKeyServerOpen( server, tls );
		if( IsntPGPError( err ) )
		{
			switch( op )
			{
				case kKeySearchOp_Search:
					err = PGPQueryKeyServer( server, mServerFilter, &resultDB );
					break;
				case kKeySearchOp_Delete:
					err = PGPDeleteFromKeyServer( server, actionKeySet, &failedKeys );
					mActionKeySet = kInvalidPGPKeySetRef;
					break;
				case kKeySearchOp_Disable:
					err = PGPDisableFromKeyServer( server, actionKeySet, &failedKeys );
					actionKeySet = kInvalidPGPKeySetRef;
					break;
			}
			PGPKeyServerClose( server );
		}
		if( err == kPGPError_ServerPartialSearchResults )
			err = kPGPError_NoErr;
		if( IsntPGPError( err ) && ( op == kKeySearchOp_Search ) )
		{
			[mStatusField setStringValue:LSPK( @"Verifying", @"Server status:verifying" )];
			err = PGPNewKeySet( resultDB, &keySet );
			if( IsntPGPError( err ) )
				err = PGPCheckKeyRingSigs( keySet, [mController mainDB], FALSE, NULL, NULL );
			if( IsntPGPError( err ) )
				err = PGPCalculateTrust( keySet, [mController mainDB] );
			if( PGPKeySetRefIsValid( keySet ) )
				PGPFreeKeySet( keySet );
			keySet = kInvalidPGPKeySetRef;
		}
	}
	
done:
	if( IsntPGPError( err ) && PGPKeyServerRefIsValid( mCurrentServer ) && ( server == mCurrentServer ) )
		[self endSearch:resultDB];
	if( PGPKeyServerRefIsValid( server ) )
		PGPFreeKeyServer( server );
	if( mCurrentServer == server )
		mCurrentServer = kInvalidPGPKeyServerRef;
	if( PGPtlsSessionRefIsValid( tls ) )
		PGPFreeTLSSession( tls );
	if( PGPKeySetRefIsValid( failedKeys ) )
		PGPFreeKeySet( failedKeys );
	if( PGPKeySetRefIsValid( actionKeySet ) )
		PGPFreeKeySet( actionKeySet );
	if( IsPGPError( err ) )
		PGPclErrorBox( err );
	else if( op != kKeySearchOp_Search )
	{
		NSBeginInformationalAlertSheet(
			( op == kKeySearchOp_Delete ) ?
			LSPK( @"Selected Keys Deleted", @"Server delete title" ) :
			LSPK( @"Selected Keys Disabled", @"Server disable title" ),
			kPGPkeysOKButton, nil, nil, mSearchWindow, self, NULL, NULL, NULL,
			( op == kKeySearchOp_Delete ) ?
			LSPK( @"The selected keys have been successfully deleted from the server.", @"Server delete message" ) :
			LSPK( @"The selected keys have been successfully disabled on the server.", @"Server disable message" ) );
	}
	[pool release];
}

- (void)startOpUI
{
	NSEnumerator	*panelEnumerator = [mSearchPanelArray objectEnumerator];
	CSearchPanel	*panel;

	[mStatusField setStringValue:@""];
	[mStatusView addSubview:mSearchProgress];
	[mStatusView addSubview:mStatusField];
	[mSearchProgress startAnimation:self];
	[mSearchButton setTitle:kCKeySearchCancelString];
	[mClearButton setEnabled:NO];
	[mSourcePopup setEnabled:NO];
	[mFewerChoicesButton setEnabled:NO]; 
	[mMoreChoicesButton setEnabled:NO];
	[mPendingCheckbox setEnabled:NO];
	while( panel = [panelEnumerator nextObject] )
		[panel setEnabled:NO];
}

- (IBAction)searchAction:(id)sender
{
	if( !mRunning )
	{
		if( PGPFilterRefIsValid( mServerFilter = [self searchFilter] ) )
		{
			PGPError		err;
			PGPUInt32		source;
			PGPKeyDBRef		lKeyDB = kInvalidPGPKeyDBRef,
							sourceDB = kInvalidPGPKeyDBRef;
			PGPKeySetRef	lKeySet = kInvalidPGPKeySetRef;
			
			mOp = kKeySearchOp_Search;
			[self startOpUI];
			source = [mSourcePopup indexOfSelectedItem];
			switch( source )
			{
				case 0:		// Local Keyring
				case 1:		// Current Results
					sourceDB = ( !source ? [mController mainDB] : [mPGPkeysView keyDB] );
					if( PGPKeyDBRefIsValid( sourceDB ) )
					{
						err = PGPFilterKeyDB( sourceDB, mServerFilter, &lKeySet );
						if( IsntPGPError( err ) )
						{
							err = PGPNewKeyDB( mPGPContext, &lKeyDB );
							if( IsntPGPError(err) )
								err = PGPCopyKeys( lKeySet, lKeyDB, NULL );
							PGPFreeKeySet( lKeySet );
						}
					}
					[self endSearch:lKeyDB];
					break;
				default:
					mRunning = YES;
					[mPGPkeysView emptyKeys];
					[NSThread detachNewThreadSelector:@selector(queryServer:) toTarget:self withObject:self];
					break;
			}
		}
	}
	else
	{
		if( PGPKeyServerRefIsValid( mCurrentServer ) )
			PGPCancelKeyServerCall( mCurrentServer );
		mCurrentServer = kInvalidPGPKeyServerRef;
		[self endSearch:kInvalidPGPKeyDBRef];

	}
}

- (void)serverCommand:(int)command
{
	PGPUInt32		source;
	
	source = [mSourcePopup indexOfSelectedItem];
	if( mRunning )
		return;
	if( source <= 1 )
	{
		NSBeginInformationalAlertSheet(
			LSPK( @"No Server Selected", @"No server title" ),
			kPGPkeysOKButton, nil, nil, mSearchWindow, self, NULL, NULL, NULL,
			LSPK( @"You must select a server in order to perform the operation.", @"No server message" ) );
	}
	else
	{
		if( command == kPGPkeysCommand_ServerDelete )
			mOp = kKeySearchOp_Delete;
		else
			mOp = kKeySearchOp_Disable;
		[self startOpUI];
		if( PGPKeySetRefIsValid( mActionKeySet ) )
			PGPFreeKeySet( mActionKeySet );
		mActionKeySet = kInvalidPGPKeySetRef;
		mActionKeySet = [mPGPkeysView getSelectedKeySet];
		mRunning = YES;
		[NSThread detachNewThreadSelector:@selector(queryServer:) toTarget:self withObject:self];
	}
}

- (IBAction)clearAction:(id)sender;
{
	while( [mSearchPanelArray count] > 1 )
		[self removeSearchPanel];
	[[mSearchPanelArray objectAtIndex:0] setSearchType:kKeySearchType_UserID];
}

- (PGPPrefRef)pgpPrefRef
{
	return mPrefRef;
}

- (PGPContextRef)pgpContext
{
	return mPGPContext;
}

- (PGPtlsContextRef)tlsContext
{
	return mTLSContext;
}

- (NSWindow *)window
{
	return mSearchWindow;
}

- (CPGPkeysView *)pgpKeysView
{
	return mPGPkeysView;
}

- (void)newKey:(id)keysView
{
}

- (void)sign:(id)keysView forArray:(NSMutableArray *)userIDArray
{
	[mController sign:keysView forArray:userIDArray];
}

- (void)split:(id)keysView forKey:(PGPKeyDBObjRef)splitKey;
{
	[mController split:keysView forKey:splitKey];
}

- (void)search:(id)keysView
{
}

- (PGPKeyDBRef)mainDB
{
	return [mController mainDB];
}

- (void)serversUpdated
{
	[self loadSourcePopup];
}

- (BOOL)windowShouldClose:(id)sender
{
	PGPInt32	selection;
	PGPError	err;
	
	if( mRunning )
	{
		if( PGPKeyServerRefIsValid( mCurrentServer ) )
			PGPCancelKeyServerCall( mCurrentServer );
		mCurrentServer = kInvalidPGPKeyServerRef;
	}
	
	selection = [mSourcePopup indexOfSelectedItem];
	err = PGPSetPrefNumber( mPrefRef, kPGPPrefLastSearchWindowSelection, selection );
	pgpAssertNoErr( err );
	[mController keySearchClosed];
	return YES;
}

- (void)dealloc
{
	[mPGPkeysView release];
	[mSearchPanelArray release];
	if( IsntNull( mServers ) )
		PGPFreeKeyServerList( mServers );
	[mSearchProgress release];
	[mStatusField release];
	[super dealloc];
	PGPLog(( @"Deallocated CKeySearch" ));
}

@end

