/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyring.m,v 1.6 2002/11/11 21:41:42 jeffc Exp $
____________________________________________________________________________*/
#import "CKeyring.h"
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pflPrefTypes.h>
#import "PGPkeysCommon.h"
#import "PGPkeysController.h"

#define kCKeyringNib				@"CKeyring"


@implementation CKeyring

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeyringNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
	
	mLoadedNib = YES;
    return [self init];
}

- (id)initWithController:(id)controller
{
	mController = controller;
	mPGPContext = [controller pgpContext];
	mTLSContext = [controller tlsContext];
	mPrefRef	= [controller pgpPrefRef];
	
	[self init];
	
	mPGPkeysView = [[CPGPkeysView alloc] initWithController:self mainDB:NO search:NO];
	[[mPGPkeysView view] setFrameSize:[mViewBox frame].size];
	[mViewBox addSubview:[mPGPkeysView view]];
		
	return self;
}

- (void)loadKeyring:(NSString *)docFile
{
	PGPError				err = kPGPError_NoErr;
	NSString				*docName = nil,
							*path = nil,
							*secGuess = nil,
							*pubGuess = nil;
	const char				*pubC = nil,
							*secC = nil;
	PGPKeyDBRef				keyDB = kInvalidPGPKeyDBRef;
	PGPOpenKeyDBFileOptions	openOptions = kPGPOpenKeyDBFileOptions_Mutable;
	BOOL					hidePub = NO,
							hideSec = NO;
	
	if( docFile )
	{
		docName = [docFile lastPathComponent];
		path = [docFile stringByDeletingLastPathComponent];
		if( [docName isEqual:kPGPkeysPubringOS9Name] )
			secGuess = kPGPkeysSecringOS9Name;
		else if( [docName isEqual:kPGPkeysPubringWinName] )
			secGuess = kPGPkeysSecringWinName;
		else if( [docName isEqual:kPGPkeysPubringOSXName] )
			secGuess = kPGPkeysSecringOSXName;
		else if( [docName isEqual:kPGPkeysPubringAncientName] )
			secGuess = kPGPkeysSecringAncientName;
		if( !secGuess )
		{
			if( [docName isEqual:kPGPkeysSecringOS9Name] )
				pubGuess = kPGPkeysPubringOS9Name;
			else if( [docName isEqual:kPGPkeysSecringWinName] )
				pubGuess = kPGPkeysPubringWinName;
			else if( [docName isEqual:kPGPkeysSecringOSXName] )
				pubGuess = kPGPkeysPubringOSXName;
			else if( [docName isEqual:kPGPkeysSecringAncientName] )
				pubGuess = kPGPkeysPubringAncientName;
		}
		if( pubGuess || secGuess )
		{
			if( pubGuess )
			{
				secGuess = docFile;
				pubGuess = [[NSMutableString stringWithString:path] stringByAppendingPathComponent:pubGuess];
				if( ![[NSFileManager defaultManager] fileExistsAtPath:pubGuess] )
					pubGuess = nil;
			}
			else if( secGuess )
			{
				pubGuess = docFile;
				secGuess = [[NSMutableString stringWithString:path] stringByAppendingPathComponent:secGuess];
				if( ![[NSFileManager defaultManager] fileExistsAtPath:secGuess] )
					secGuess = nil;
			}
		}
		else
		{
			NSOpenPanel	*openPanel = [NSOpenPanel openPanel];

			pubGuess = docFile;
			NSRunAlertPanel( LSPK( @"The associated PGP Private Keyring file could not be found.", @"skr missing title" ),
				LSPK( @"Please select the associated Private Keyring file.", @"skr missing msg" ), kPGPkeysOKButton,
				nil, nil );
			[openPanel setAllowsMultipleSelection:NO];
			if( [openPanel runModalForDirectory:path file:nil types:nil] == NSOKButton )
			{
				secGuess = [openPanel filename];
				if( [secGuess isEqual:pubGuess] )
				{
					NSRunAlertPanel( LSPK( @"The Public and Private Keyring files are the same.", @"skr identical title" ),
						LSPK( @"You must select separate Public and Private Keyring files to open a keyring pair.", @"skr identical msg" ), kPGPkeysOKButton,
						nil, nil );
					err = kPGPError_UserAbort;
				}
			}
			else
				err = kPGPError_UserAbort;
		}
	}
	if( !docFile )
	{
		NSSavePanel	*savePanel = [NSSavePanel savePanel];
		
		[savePanel setRequiredFileType:@"pkr"];
		[savePanel setCanSelectHiddenExtension:YES];
		if( [savePanel runModalForDirectory:NSHomeDirectory()
			file:LSPK(@"PGP Public Keyring",@"Suggested new keyring filename")] == NSOKButton )
		{
			pubGuess = [savePanel filename];
			hidePub = [savePanel isExtensionHidden];
			[savePanel setRequiredFileType:@"skr"];
			if( [savePanel runModalForDirectory:[savePanel directory]
				file:LSPK(@"PGP Private Keyring",@"Suggested new sec keyring filename")] == NSOKButton )
			{
				secGuess = [savePanel filename];
				hideSec = [savePanel isExtensionHidden];
				openOptions |= kPGPOpenKeyDBFileOptions_Create;
			}
			else
				err = kPGPError_UserAbort;
		}
		else
			err = kPGPError_UserAbort;
	}
	if( IsntPGPError( err ) && pubGuess && secGuess )
	{
		pubC = [pubGuess UTF8String];
		secC = [secGuess UTF8String];
		err = PGPclOpenKeyrings( mPGPContext, (char *)pubC, (char *)secC, openOptions, &keyDB );
	}
	if( IsntPGPError( err ) )
	{
		if( openOptions & kPGPOpenKeyDBFileOptions_Create )
		{
			if( hidePub )
				[[NSFileManager defaultManager] changeFileAttributes:
					[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:hidePub] forKey:NSFileExtensionHidden]
					atPath:pubGuess];
			if( hideSec )
				[[NSFileManager defaultManager] changeFileAttributes:
					[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:hideSec] forKey:NSFileExtensionHidden]
					atPath:secGuess];
		}
		if( pubGuess )
			[mKeyringWindow setTitleWithRepresentedFilename:pubGuess];
		[mPGPkeysView loadKeys:keyDB showKeySet:NULL];
		[mKeyringWindow center];
		[mKeyringWindow makeKeyAndOrderFront:self];
		[mPGPkeysView setActive:YES];
	}
	else
	{
		PGPclErrorBox( err );
		[mKeyringWindow performClose:self];
	}
}

- (PGPPrefRef)pgpPrefRef
{
	return mPrefRef;
}

- (PGPContextRef)pgpContext
{
	return mPGPContext;
}

- (PGPtlsContextRef)tlsContext
{
	return mTLSContext;
}

- (NSWindow *)window
{
	return mKeyringWindow;
}

- (CPGPkeysView *)pgpKeysView
{
	return mPGPkeysView;
}

- (void)newKey:(id)keysView
{
	[mController newKey:keysView];
}

- (void)sign:(id)keysView forArray:(NSMutableArray *)userIDArray
{
	[mController sign:keysView forArray:userIDArray];
}

- (void)split:(id)keysView forKey:(PGPKeyDBObjRef)splitKey;
{
	[mController split:keysView forKey:splitKey];
}

- (void)search:(id)keysView
{
	[mController search:keysView];
}

- (PGPKeyDBRef)mainDB
{
	return [mController mainDB];
}

- (BOOL)windowShouldClose:(id)sender
{
	PGPLog((@"CKeyring windowShouldClose:"));
	[mController keyringClosed:self];
	return YES;
}

- (void)dealloc
{
	[mPGPkeysView release];
	[super dealloc];
	PGPLog(( @"Deallocated CKeyring" ));
}

@end

