/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPkeysView.m,v 1.41 2002/11/20 11:27:38 wprice Exp $
____________________________________________________________________________*/
#import "CPGPkeysView.h"
#import "PGPkeysCommon.h"
#import "PGPkeysController.h"
#import "CKeySearch.h"
#import "CKeyServerUpdate.h"
#import <PGPclient/CKeyDBItem.h>
#import <PGPclient/CKeyView.h>
#import <PGPclient/pflPrefTypes.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpClientShared.h>
#import <PGPclient/pgpEndianConversion.h>
#import <PGPclient/pgpMem.h>

#define kCPGPkeysViewNib					@"CPGPkeysView"
#define	kMaxPhotoIDWidth					120
#define kMaxPhotoIDHeight					144

#define kPGPkeysToolbarIdentifier			@"PGPkeys.toolbar"
#define kPGPkeysToolbarItemNew				@"PGPkeys.toolbar.new"
#define kPGPkeysToolbarItemRevoke			@"PGPkeys.toolbar.revoke"
#define kPGPkeysToolbarItemSign				@"PGPkeys.toolbar.sign"
#define kPGPkeysToolbarItemDelete			@"PGPkeys.toolbar.delete"
#define kPGPkeysToolbarItemSearch			@"PGPkeys.toolbar.search"
#define kPGPkeysToolbarItemSendToServer		@"PGPkeys.toolbar.sendtoserver"
#define kPGPkeysToolbarItemUpdate			@"PGPkeys.toolbar.update"
#define kPGPkeysToolbarItemShowInfo			@"PGPkeys.toolbar.showinfo"
#define kPGPkeysToolbarItemImport			@"PGPkeys.toolbar.import"
#define kPGPkeysToolbarItemExport			@"PGPkeys.toolbar.export"

#define kPGPKeyServerString_Connecting		LSPK(@"Connecting", @"Connecting status string")
#define kPGPKeyServerString_Searching		LSPK(@"Searching", @"Searching status string")
#define kPGPKeyServerString_Receiving		LSPK(@"Receiving results", @"Receiving status string")
#define kPGPKeyServerString_Processing		LSPK(@"Processing results", @"Processing status string")
#define kPGPKeyServerString_Sending			LSPK(@"Sending", @"Sending status string")
#define kPGPKeyServerString_Deleting		LSPK(@"Deleting key(s)...", @"Deleting status string")
#define kPGPKeyServerString_Disabling		LSPK(@"Disabling key(s)...", @"Disabling status string")
#define kPGPKeyServerString_Closing			LSPK(@"Cleaning up...", @"Closing status string")

static PGPKeyServerRef	sLastKeyServer = NULL;
static PGPUInt32		sLastKeyServerState = 0;

static PGPError sKeyServerCallback(PGPContextRef context, struct PGPEvent* event, PGPUserValue userValue)
{
	CKeyServerUpdate*		sheet = [CKeyServerUpdate sharedInstance];
	PGPKeyServerRef			eventServer;
	PGPUInt32				serverState;
	PGPError				err = kPGPError_NoErr;
	NSString*				string = nil;
	NSWindow*				window = (NSWindow*) userValue;
	char*					hostname = NULL;
	
	pgpAssert(sheet != nil);
	pgpAssert(event != NULL);
	
	if (!event)
		return kPGPError_BadParams;
	
	if (IsPGPError(err = [sheet returnCode]))
	{
		sLastKeyServer = NULL;
		sLastKeyServerState = 0;
		
		return kPGPError_UserAbort;
	}
	
	switch (event->type)
	{
		case kPGPEvent_KeyServerIdleEvent:
			break;
		
		case kPGPEvent_KeyServerEvent:
			eventServer = event->data.keyServerData.keyServerRef;
			serverState = event->data.keyServerData.state;
			
			[sheet setKeyserver:eventServer];
			
			if (PGPKeyServerRefIsValid(eventServer) && (eventServer != sLastKeyServer))
			{
				sLastKeyServer = eventServer;
				PGPGetKeyServerHostName(sLastKeyServer, &hostname);
				
				if (hostname)
				{
					[[sheet server] setStringValue:[NSString stringWithUTF8String:hostname]];
					[[sheet server] performSelectorOnMainThread:@selector(display) withObject:nil waitUntilDone:NO];
					
					if (![[sheet window] isVisible])
					{
						[NSApp beginSheet:[sheet window]
							   modalForWindow:window
							   modalDelegate:nil
							   didEndSelector:nil
							   contextInfo:NULL];
						[[sheet progress] startAnimation:nil];
					}
					
					PGPFreeData(hostname);
				}
			}
			
			if (serverState != sLastKeyServerState)
			{
				sLastKeyServerState = serverState;
				
				switch (serverState)
				{
					case kPGPKeyServerState_Opening:
						string = kPGPKeyServerString_Connecting;
						break;
						
					case kPGPKeyServerState_Querying:
						string = kPGPKeyServerString_Searching;
						break;
					
					case kPGPKeyServerState_ReceivingResults:
						string = kPGPKeyServerString_Receiving;
						break;
					
					case kPGPKeyServerState_ProcessingResults:
						string = kPGPKeyServerString_Processing;
						break;
						
					case kPGPKeyServerState_Uploading:
						string = kPGPKeyServerString_Sending;
						break;
					
					case kPGPKeyServerState_Deleting:
						string = kPGPKeyServerString_Deleting;
						break;
					
					case kPGPKeyServerState_Disabling:
						string = kPGPKeyServerString_Disabling;
						break;
					
					case kPGPKeyServerState_Closing:
						string = kPGPKeyServerString_Closing;
						break;
					
					default:
						string = @"";
						break;
				}
			}
			
			if (string)
			{
				[[sheet status] setStringValue:string];
				[[sheet status] performSelectorOnMainThread:@selector(display) withObject:nil waitUntilDone:NO];
				
				if (![[sheet window] isVisible])
				{
					[NSApp beginSheet:[sheet window]
							modalForWindow:window
							modalDelegate:nil
							didEndSelector:nil
							contextInfo:NULL];
					[[sheet progress] startAnimation:nil];
				}
			}
		
			break;
		
		default:
			break;
	}
	
	return kPGPError_NoErr;
}

@implementation CPGPkeysView

- (id)init
{	
	if( !mInited )
	{
		[super init];
		mInited = YES;
		mFirstSelection		= kInvalidPGPKeyDBObjRef;
		mMultipleSelected	= NO;
		mSelectFlags		= kPGPclKeyList_None;
		mKeyDB				= kInvalidPGPKeyDBRef;
	}	
	if( !mLoadedNib )
		[self initWithNib:kCPGPkeysViewNib];
	if( !mPGPContext && mController )
	{
		PGPclKeyListColumnInfo	*prefColumnInfo,
								columnInfo;
		PGPSize					len;

		mPGPContext = [mController pgpContext];
		mPrefRef = [mController pgpPrefRef];
		mTLSContext = [mController tlsContext];
		mCKeyView = [[CKeyView alloc] initWithController:self];
		[[mCKeyView view] setFrameSize:[mKeyView frame].size];
		[mKeyView addSubview:[mCKeyView view]];
		
		[mCKeyView setFlags:
			kPGPclKeyList_ValidityButton | kPGPclKeyList_EnableDragOut |
			( mSearch ? 0 : kPGPclKeyList_EnableDropIn )];
		len = sizeof(PGPclKeyListColumnInfo);
		if( IsPGPError( PGPGetPrefData( mPrefRef, kPGPPrefPGPkeysMacColumnData, &len, (void **)&prefColumnInfo ) ) )
		{
			pgpCopyMemory( &kDefaultColumnInfo, &columnInfo, sizeof(PGPclKeyListColumnInfo) );
		}
		else
		{
			pgpCopyMemory( prefColumnInfo, &columnInfo, sizeof(PGPclKeyListColumnInfo) );
			PGPFreeData(prefColumnInfo);
		}
		[mCKeyView setColumns:columnInfo];
	}
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
	
	[self setupKeyViewLoop];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller mainDB:(BOOL)isMainDB search:(BOOL)search
{
    mController = controller;
	mIsMainDB	= isMainDB;
	mSearch		= search;
	
    return [self init];
}

- (void)setupToolbar
{
	NSBundle*			bundle = [NSBundle bundleForClass:[self class]];
	BOOL				firstRun = NO;
	
	if (!mToolbar)
	{
		mToolbar = [[NSToolbar alloc] initWithIdentifier:kPGPkeysToolbarIdentifier];
		[mToolbar setAutosavesConfiguration:YES];
		[mToolbar setAllowsUserCustomization:YES];
		
		if (![[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.pgp.app.PGP"])
		{
			[mToolbar setConfigurationFromDictionary:[mToolbar configurationDictionary]];
			[mToolbar setSizeMode:NSToolbarSizeModeSmall];
			
			firstRun = YES;
		}
		else
			[mToolbar setConfigurationFromDictionary:[mToolbar configurationDictionary]];
		
		mToolbarItemNew = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemNew];
		[mToolbarItemNew setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"newkey.icns"]] autorelease]];
		[mToolbarItemNew setLabel:LSPK(@"New Key", @"New Key ToolbarItem Label")];
		[mToolbarItemNew setPaletteLabel:LSPK(@"New Key", @"New Key ToolbarItem Palette Label")];
		[mToolbarItemNew setToolTip:LSPK(@"Create a new key", @"New Key ToolbarItem ToolTip")];
		[mToolbarItemNew setTag:kPGPkeysCommand_New];
		[mToolbarItemNew setTarget:self];
		[mToolbarItemNew setAction:@selector(keysMenuAction:)];
		
		mToolbarItemRevoke = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemRevoke];
		[mToolbarItemRevoke setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"revoke.icns"]] autorelease]];
		[mToolbarItemRevoke setLabel:LSPK(@"Revoke", @"Revoke Key ToolbarItem Label")];
		[mToolbarItemRevoke setPaletteLabel:LSPK(@"Revoke", @"Revoke Key ToolbarItem Palette Label")];
		[mToolbarItemRevoke setToolTip:LSPK(@"Revoke key", @"Revoke Key ToolbarItem ToolTip")];
		[mToolbarItemRevoke setTag:kPGPkeysCommand_Revoke];
		[mToolbarItemRevoke setTarget:self];
		[mToolbarItemRevoke setAction:@selector(keysMenuAction:)];
		
		mToolbarItemSign = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemSign];
		[mToolbarItemSign setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"sign.icns"]] autorelease]];
		[mToolbarItemSign setLabel:LSPK(@"Sign", @"Sign Key ToolbarItem Label")];
		[mToolbarItemSign setPaletteLabel:LSPK(@"Sign", @"Sign Key ToolbarItem Palette Label")];
		[mToolbarItemSign setToolTip:LSPK(@"Sign", @"Sign Key ToolbarItem ToolTip")];
		[mToolbarItemSign setTag:kPGPkeysCommand_Sign];
		[mToolbarItemSign setTarget:self];
		[mToolbarItemSign setAction:@selector(keysMenuAction:)];
		
		mToolbarItemDelete = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemDelete];
		[mToolbarItemDelete setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"remove.icns"]] autorelease]];
		[mToolbarItemDelete setLabel:LSPK(@"Delete", @"Delete Key ToolbarItem Label")];
		[mToolbarItemDelete setPaletteLabel:LSPK(@"Delete", @"Delete Key ToolbarItem Palette Label")];
		[mToolbarItemDelete setToolTip:LSPK(@"Delete", @"Delete Key ToolbarItem ToolTip")];
		[mToolbarItemDelete setTag:kPGPkeysCommand_Delete];
		[mToolbarItemDelete setTarget:self];
		[mToolbarItemDelete setAction:@selector(keysMenuAction:)];
		
		mToolbarItemSearch = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemSearch];
		[mToolbarItemSearch setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"search.icns"]] autorelease]];
		[mToolbarItemSearch setLabel:LSPK(@"Search", @"Search Key ToolbarItem Label")];
		[mToolbarItemSearch setPaletteLabel:LSPK(@"Search", @"Search Key ToolbarItem Palette Label")];
		[mToolbarItemSearch setToolTip:LSPK(@"Search for a key", @"Search Key ToolbarItem ToolTip")];
		[mToolbarItemSearch setTag:kPGPkeysCommand_Search];
		[mToolbarItemSearch setTarget:self];
		[mToolbarItemSearch setAction:@selector(keysMenuAction:)];
		
		mToolbarItemSendToServer = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemSendToServer];
		[mToolbarItemSendToServer setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"send.icns"]] autorelease]];
		[mToolbarItemSendToServer setLabel:LSPK(@"Send To Server", @"SendToServer Key ToolbarItem Label")];
		[mToolbarItemSendToServer setPaletteLabel:LSPK(@"Send To Server", @"SendToServer Key ToolbarItem Palette Label")];
		[mToolbarItemSendToServer setToolTip:LSPK(@"Send key to server", @"SendToServer Key ToolbarItem ToolTip")];
		[mToolbarItemSendToServer setTag:kPGPkeysCommand_SendToServer];
		[mToolbarItemSendToServer setTarget:self];
		[mToolbarItemSendToServer setAction:@selector(keysMenuAction:)];
		
		mToolbarItemUpdate = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemUpdate];
		[mToolbarItemUpdate setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"update.icns"]] autorelease]];
		[mToolbarItemUpdate setLabel:LSPK(@"Update", @"Update Key ToolbarItem Label")];
		[mToolbarItemUpdate setPaletteLabel:LSPK(@"Update From Server", @"Update Key ToolbarItem Palette Label")];
		[mToolbarItemUpdate setToolTip:LSPK(@"Update from server", @"Update Key ToolbarItem ToolTip")];
		[mToolbarItemUpdate setTag:kPGPkeysCommand_UpdateFromServer];
		[mToolbarItemUpdate setTarget:self];
		[mToolbarItemUpdate setAction:@selector(keysMenuAction:)];
		
		mToolbarItemShowInfo = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemShowInfo];
		[mToolbarItemShowInfo setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"info.icns"]] autorelease]];
		[mToolbarItemShowInfo setLabel:LSPK(@"Info", @"ShowInfo Key ToolbarItem Label")];
		[mToolbarItemShowInfo setPaletteLabel:LSPK(@"Show Info", @"ShowInfo Key ToolbarItem Palette Label")];
		[mToolbarItemShowInfo setToolTip:LSPK(@"Show info", @"ShowInfo Key ToolbarItem ToolTip")];
		[mToolbarItemShowInfo setTag:kPGPkeysCommand_Info];
		[mToolbarItemShowInfo setTarget:self];
		[mToolbarItemShowInfo setAction:@selector(keysMenuAction:)];
		
		mToolbarItemImport = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemImport];
		[mToolbarItemImport setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"import.icns"]] autorelease]];
		[mToolbarItemImport setLabel:LSPK(@"Import", @"Import Key ToolbarItem Label")];
		[mToolbarItemImport setPaletteLabel:LSPK(@"Import", @"Import Key ToolbarItem Palette Label")];
		[mToolbarItemImport setToolTip:LSPK(@"Import", @"Import Key ToolbarItem ToolTip")];
		[mToolbarItemImport setTag:kPGPkeysCommand_Import];
		[mToolbarItemImport setTarget:self];
		[mToolbarItemImport setAction:@selector(keysMenuAction:)];
		
		mToolbarItemExport = [[NSToolbarItem alloc] initWithItemIdentifier:kPGPkeysToolbarItemExport];
		[mToolbarItemExport setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"export.icns"]] autorelease]];
		[mToolbarItemExport setLabel:LSPK(@"Export", @"Export Key ToolbarItem Label")];
		[mToolbarItemExport setPaletteLabel:LSPK(@"Export", @"Export Key ToolbarItem Palette Label")];
		[mToolbarItemExport setToolTip:LSPK(@"Export", @"Export Key ToolbarItem ToolTip")];
		[mToolbarItemExport setTag:kPGPkeysCommand_Export];
		[mToolbarItemExport setTarget:self];
		[mToolbarItemExport setAction:@selector(keysMenuAction:)];
		
		[mToolbar setDelegate:self];
		[[self window] setToolbar:mToolbar];
		
		if (firstRun)
			[[self window] toggleToolbarShown:nil];
	}
}

- (void)loadKeys:(PGPKeyDBRef)keyDB showKeySet:(PGPKeySetRef)keySet
{
	[self emptyKeys];
	mKeyDB = keyDB;
	[mCKeyView loadKeys:keyDB showKeySet:keySet];
	[self selectionChanged];
}

- (void)emptyKeys
{
	[mCKeyView emptyKeys];
	if( !mIsMainDB && PGPKeyDBRefIsValid( mKeyDB ) )
		PGPFreeKeyDB( mKeyDB );
	mKeyDB = kInvalidPGPKeyDBRef;
}

- (PGPContextRef)pgpContext
{
	return mPGPContext;
}

- (PGPtlsContextRef)tlsContext
{
	return mTLSContext;
}

- (PGPPrefRef)pgpPrefRef
{
	return mPrefRef;
}

- (PGPKeyDBRef)mainDB
{
	return [mController mainDB];
}

- (void)setActive:(BOOL)flag
{
	[self setupToolbar];
	
	[mCKeyView setActive:flag];
	[self selectionChanged];
}

- (BOOL)columnIsActive:(PGPInt16) columnIndex
{
	PGPInt16				cIndex;
	PGPclKeyListColumnInfo	columnInfo;
	
	columnInfo = [mCKeyView columns];
	for( cIndex = 0; cIndex < columnInfo.numActiveColumns; cIndex++ )
	{
		if( columnInfo.columns[cIndex] == columnIndex )
			return YES;
	}
	return NO;
}

- (void)toggleColumn:(PGPInt16)columnIndex
{
	PGPclKeyListColumnInfo	columnInfo;

	columnInfo = [mCKeyView columns];
	if( [self columnIsActive:columnIndex] )
	{
		PGPInt16	cIndex;
		BOOL		found = FALSE;
		
		for( cIndex = 0; cIndex < columnInfo.numActiveColumns; cIndex++ )
		{
			if( found )
				columnInfo.columns[cIndex - 1] = columnInfo.columns[cIndex];
			else if( columnInfo.columns[cIndex] == columnIndex )
				found = TRUE;
		}
		columnInfo.numActiveColumns--;
	}
	else
		columnInfo.columns[columnInfo.numActiveColumns++] = columnIndex;
	[mCKeyView setColumns:columnInfo];
}

- (void)selectionChanged
{
	mFirstSelection = [mCKeyView selectionInfo:&mMultipleSelected withFlags:&mSelectFlags];
	
	[[[mKeyView window] toolbar] validateVisibleItems];
}

- (void)setupKeyViewLoop
{
}

- (BOOL)keyViewIsValid:(NSResponder*)keyView
{
	return YES;
}

- (void)importDocument:(NSString *)docFile
{
	[mCKeyView importKeyDB:[mCKeyView importFileList:[NSArray arrayWithObject:docFile]] confirm:YES];
}

- (void)importSheetDidEnd:(NSOpenPanel*)sheet returnCode:(int)code contextInfo:(void*)info
{
	[sheet orderOut:self];
	if( code == NSOKButton )
	{
		[mCKeyView importKeyDB:[mCKeyView importFileList:[sheet filenames]] confirm:YES];
	}
}

- (void)exportSheetDidEnd:(NSSavePanel*)sheet returnCode:(int)code contextInfo:(void*)info
{
	[sheet orderOut:self];
	[sheet setAccessoryView:nil];
	if( code == NSOKButton )
	{
		[mCKeyView exportSelection:[sheet filename]
			privates:[mIncludePrivatesCheckbox state]
			extensions:[mInclude6XCheckbox state]
			hideSuffix:[sheet isExtensionHidden]];
	}
}

- (BOOL)validateCommand:(int)command item:(NSMenuItem *)menuItem
{
	PGPBoolean	secret,
				revoke,
				canSign,
				allowCertReq;
	PGPInt32	keyVersion;
	BOOL		enabled = NO;
	char		serverURL[256];
	
	switch( command )
	{
		case kPGPkeysCommand_ViewValidity:
		case kPGPkeysCommand_ViewTrust:
		case kPGPkeysCommand_ViewSize:
		case kPGPkeysCommand_ViewCreation:
		case kPGPkeysCommand_ViewExpiration:
		case kPGPkeysCommand_ViewKeyID:
		case kPGPkeysCommand_ViewADK:
		case kPGPkeysCommand_ViewDescription:
			if( menuItem )
				[menuItem setState:[self columnIsActive:command - kPGPkeysCommand_ViewValidity + 1]];
			enabled = YES;
			break;
		case kPGPkeysCommand_UpdateRevocations:
			if( IsntPGPError( PGPGetPrefStringBuffer( mPrefRef, kPGPPrefCARevocationServerURL,
				sizeof(serverURL), serverURL ) ) && serverURL[0] )
				enabled = YES;
			break;
		case kPGPkeysCommand_AddCertificate:
		case kPGPkeysCommand_RetrieveCert:
			PGPGetPrefBoolean( mPrefRef, kPGPPrefAllowManualX509CertRequest, &allowCertReq );
			if( !allowCertReq )
				break;
			if( !mMultipleSelected && ( ( mSelectFlags & kPGPclKeyList_UserID ) ||
				( mSelectFlags & kPGPclKeyList_Key ) ) &&
				IsntPGPError( PGPGetKeyDBObjBooleanProperty( PGPPeekKeyDBObjKey( mFirstSelection ),
				kPGPKeyProperty_CanSign, &canSign ) ) && canSign &&
				IsntPGPError( PGPGetPrefStringBuffer( mPrefRef, kPGPPrefCAServerURL,
				sizeof(serverURL), serverURL ) ) && serverURL[0] )
			{
				enabled = YES;
			}
			break;
		case kPGPkeysCommand_Split:
		case kPGPkeysCommand_AddName:
		case kPGPkeysCommand_AddPhoto:
		case kPGPkeysCommand_AddRevoker:
			if( !mMultipleSelected && ( mSelectFlags & kPGPclKeyList_Key ) &&
				IsntPGPError( PGPGetKeyDBObjBooleanProperty( mFirstSelection,
				kPGPKeyProperty_CanSign, &canSign ) ) && canSign )
			{
				if( ( command == kPGPkeysCommand_AddPhoto ) ||
					( command == kPGPkeysCommand_AddRevoker ) )
				{
					PGPGetKeyDBObjNumericProperty( mFirstSelection, kPGPKeyProperty_Version, &keyVersion);
					if( keyVersion == kPGPKeyVersion_V4 )
						enabled = YES;
				}
				else
					enabled = YES;
			}
			break;
		case kPGPkeysCommand_Revoke:
			if( !mMultipleSelected && ( ( ( mSelectFlags & kPGPclKeyList_Key ) &&
				IsntPGPError( PGPGetKeyDBObjBooleanProperty( mFirstSelection,
				kPGPKeyProperty_IsRevocable, &revoke ) ) && revoke ) ||
				( ( mSelectFlags & kPGPclKeyList_Signature ) &&
				IsntPGPError( PGPGetKeyDBObjBooleanProperty( mFirstSelection,
				kPGPSigProperty_IsRevoked, &revoke ) ) && !revoke ) ) )
			{
				enabled = YES;
			}
			break;
		case kPGPkeysCommand_SetDefault:
			if( mIsMainDB && !mMultipleSelected && ( ( mSelectFlags & kPGPclKeyList_UserID ) ||
				( ( mSelectFlags & kPGPclKeyList_Key ) &&
				IsntPGPError( PGPGetKeyDBObjBooleanProperty( mFirstSelection,
				kPGPKeyProperty_IsSecret, &secret ) ) && secret ) ) )
			{
				enabled = YES;
			}
			break;
		case kPGPkeysCommand_New:
		case kPGPkeysCommand_Import:
			if( mSearch )
				break;
		case kPGPkeysCommand_Search:
		case kPGPkeysCommand_NewKeyring:
			enabled = YES;
			break;
		case kPGPkeysCommand_UpdateFromServer:
		case kPGPkeysCommand_Export:
		case kPGPkeysCommand_Info:
		case kPGPkeysCommand_Delete:
		case kPGPkeysCommand_SendToServer:
		case kPGPkeysCommand_SendToThisServer:
			if( PGPKeyDBObjRefIsValid( mFirstSelection ) )
				enabled = YES;
			break;
		case kPGPkeysCommand_ServerDelete:
		case kPGPkeysCommand_ServerDisable:
			if( mSearch && PGPKeyDBObjRefIsValid( mFirstSelection ) )
				enabled = YES;
			break;
		case kPGPkeysCommand_Enable:
		case kPGPkeysCommand_Disable:
		case kPGPkeysCommand_ReverifySignatures:
			if( PGPKeyDBObjRefIsValid( mFirstSelection ) && !( mSelectFlags & kPGPclKeyList_Signature ) &&
				!( mSelectFlags & kPGPclKeyList_UserID ))
				enabled = YES;
			break;
		case kPGPkeysCommand_Sign:
			if( PGPKeyDBObjRefIsValid( mFirstSelection ) && !( mSelectFlags & kPGPclKeyList_Signature ) )
				enabled = YES;
			break;
		default:
			break;
	}
	return enabled;
}

- (BOOL)validateToolbarItem:(NSToolbarItem*)item
{
	return [self validateCommand:[item tag] item:nil];
}

- (void)reload
{
	[mCKeyView reload];
}

- (PGPKeyDBObjRef)lookupDefaultKey
{
	return [mCKeyView lookupDefaultKey];
}

- (PGPKeyDBRef)keyDB
{
	return mKeyDB;
}

- (NSView *)view
{
	return mPGPkeysView;
}

- (NSView *)keyView
{
	return [mCKeyView keyView];
}

- (NSOutlineView *)outlineView
{
	return [mCKeyView outlineView];
}

- (NSWindow *)window
{
	return [mPGPkeysView window];
}

- (void)sign
{
	[mController sign:self forArray:[mCKeyView selectedItemArray]];
}

- (void)newKey
{
	[mController newKey:self];
}

- (void)search
{
	[mController search:self];
}

- (void)validityButtonPressed:(id)sender
{
	[self sign];
}

- (PGPKeySetRef)getSelectedKeySet
{
	return [mCKeyView getEnumeratorKeySet:[mCKeyView selectedItemEnumerator]];
}

- (void)reverifySigs
{
	PGPError		err = kPGPError_NoErr;
	PGPKeySetRef	keySet = [self getSelectedKeySet];
	
	if( PGPKeySetRefIsValid( keySet ) )
	{
		err = PGPCheckKeyRingSigs( keySet, [mController mainDB], TRUE, NULL, NULL );
		PGPFreeKeySet( keySet );
		[mCKeyView reload];
	}
}

- (void)didEndRevokeConfirmSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
 	PGPError			err = kPGPError_NoErr;
	CKeyDBItem			*item = [[mCKeyView selectedItemArray] objectAtIndex:0];
	PGPBoolean			ksSync,
						split = FALSE;
	PGPKeyIterRef		keyIter = kInvalidPGPKeyIterRef;
 	char				*passphrase = NULL;
	PGPByte				*passKey = NULL;
	PGPSize				passKeySize;
 
	if( returnCode != NSAlertDefaultReturn )
		return;
	err = PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyServerSyncOnRevocation, &ksSync );	CKERR;
	if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
	{
		PGPKeyDBObjRef		revokerKey = kInvalidPGPKeyDBObjRef;
		PGPBoolean			secret;
		
		if( ksSync )
			if( IsntPGPError( [self updateKeyDBObj:item atomic:YES] ) )
				[mCKeyView reload];
		PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPKeyProperty_IsSecret, &secret );
		if( secret )
			revokerKey = [item keyDBItem];
		else
		{
			PGPUInt32		numRevokers,
							revIndex;
			PGPBoolean		revSecret,
							revSplit;
			PGPKeyDBObjRef	key;
			
			err = PGPCountRevocationKeys( [item keyDBItem], &numRevokers );	CKERR;
			for( revIndex = 0; revIndex < numRevokers; revIndex++ )
			{
				err = PGPGetIndexedRevocationKey( [item keyDBItem], revIndex, &key, NULL ); CKERR;
				if( PGPKeyDBObjRefIsValid( key ) )
				{
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsSecret, &revSecret );
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsSecretShared, &revSplit );
					if( revSecret )
					{
						if( !PGPKeyDBObjRefIsValid( revokerKey ) || !revSplit )
							revokerKey = key;
						if( !revSplit )
							break;
					}
				}
			}
		}
		if( PGPKeyDBObjRefIsValid( revokerKey ) )
			err = PGPclGetKeyPassphrase( revokerKey, mTLSContext, mPrefRef, &split, &passphrase, &passKey, &passKeySize );
		else
			err = kPGPError_ItemNotFound;
		if( IsntPGPError( err ) )
		{
			err = PGPNewKeyIterFromKeyDB( mKeyDB, &keyIter );	CKERR;
			if( PGPKeyIterSeek( keyIter, [item keyDBItem] ) >= 0 )
			{
				PGPKeyDBObjRef	subKey;
				
				while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
				{
					err = PGPclGuaranteeMinimumEntropy( mPGPContext );	CKERR;
					err = PGPRevoke( subKey, split ?
								PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
								PGPOPassphrase( mPGPContext, passphrase ),
								PGPOLastOption( mPGPContext ) );
				}
				if( err == kPGPError_EndOfIteration )
					err = kPGPError_NoErr;
			}
			else
				err = kPGPError_SecretKeyNotFound;
		}
		
		if( IsntPGPError( err ) )
		{
			err = PGPclGuaranteeMinimumEntropy( mPGPContext );	CKERR;
			err = PGPRevoke( [item keyDBItem], split ?
					PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
					PGPOPassphrase( mPGPContext, passphrase ),
					PGPOLastOption( mPGPContext ) );
			if( ksSync )
				[self sendKeyDBObj:[item keyDBItem]];
		}
	}
	else if( [item keyDBObjType] == kPGPKeyDBObjType_Signature )
	{
		PGPKeyDBObjRef	certKey;
		PGPBoolean		x509;
		
		PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPSigProperty_IsX509, &x509 );
		if( x509 )
		{
			NSRunAlertPanel( LSPK( @"Can't Revoke", @"Can't Revoke title" ),
				LSPK( @"X.509 Certificates can only be revoked by the Certificate Authority.", @"No X.509 Revoke message" ), 
				kPGPkeysOKButton, nil, nil );
		}
		else
		{
			err = PGPGetSigCertifierKey( [item keyDBItem], mKeyDB, &certKey );
			if( IsntPGPError( err ) && PGPKeyDBObjRefIsValid( certKey ) )
			{
				if(ksSync)
				{
					if( IsntPGPError( [self updateKeyDBObj:[[[CKeyDBItem alloc] initWithItem:certKey withParent:nil] autorelease] atomic:YES] ) )
						[mCKeyView reload];
				}
				err = PGPclGetKeyPassphrase( certKey, mTLSContext, mPrefRef, &split, &passphrase, &passKey, &passKeySize );
				if( IsntPGPError( err ) )
				{	
					err = PGPclGuaranteeMinimumEntropy( mPGPContext );	CKERR;
					err = PGPRevokeSig( [item keyDBItem], split ?
							PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
							PGPOPassphrase( mPGPContext, passphrase),
							PGPOLastOption( mPGPContext ) );
					if( IsntPGPError( err ) && ksSync )
						[self sendKeyDBObj:certKey];
				}
			}
		}
	}
done:
	if( err == kPGPError_ItemNotFound )
	{
		NSRunAlertPanel( LSPK( @"Revoker Not Found", @"Revoker Not Found title" ),
			LSPK( @"A revoker private key for the selected key was not found.", @"Revoker missing message" ), 
			kPGPkeysOKButton, nil, nil );
	}
	else if( IsPGPError( err ) )
	{
		NSRunAlertPanel( LSPK( @"Revocation Error", @"Revocation error title" ),
			[NSString stringWithFormat:LSPK( @"An error occurred revoking the key:%@.",
			@"Revocation error message" ), PGPGetErrorNSString( err )], 
			kPGPkeysOKButton, nil, nil );
	}
	else
		[mCKeyView reload];
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
	if( PGPKeyIterRefIsValid( keyIter ) )
		PGPFreeKeyIter( keyIter );
}

- (void)revoke
{
	NSBeginAlertSheet(
		LSPK( @"Confirm Revocation", @"Title of Revocation confirmation" ),
		kPGPkeysOKButton, kPGPkeysCancelButton, nil, [mPGPkeysView window], 
		self, NULL, @selector(didEndRevokeConfirmSheet:returnCode:contextInfo:), NULL,
		LSPK( @"Are you sure you want to revoke the selected items?", @"Confirm revocation" ) );
}

- (void)revokerSelectDidEnd:(PGPError)resultCode selectedKeys:(PGPKeySetRef)keySet
{
 	PGPError			err = kPGPError_NoErr;
	CKeyDBItem			*item = [[mCKeyView selectedItemArray] objectAtIndex:0];
	PGPBoolean			ksSync,
						split = FALSE;
 	char				*passphrase = NULL;
	PGPByte				*passKey = NULL;
	PGPSize				passKeySize;
	PGPUInt32			numRevokers = 0;

	if( IsntPGPError( resultCode ) && PGPKeySetRefIsValid( keySet ) )
	{
		PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyServerSyncOnAdd, &ksSync );
		if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
		{
			if( ksSync )
				if( IsntPGPError( [self updateKeyDBObj:item atomic:YES] ) )
					[mCKeyView reload];
			PGPCountKeys( keySet, &numRevokers );
			if( numRevokers > 0 )
			{
				err = PGPclGetKeyPassphrase( [item keyDBItem], mTLSContext, mPrefRef, &split, &passphrase, &passKey, &passKeySize );	CKERR;
				err = PGPAddKeyOptions( [item keyDBItem], split ?
						PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
						PGPOPassphrase( mPGPContext, passphrase ),
						PGPORevocationKeySet( mPGPContext, keySet ),
						PGPOLastOption( mPGPContext ) ); CKERR;
				if(ksSync)
					[self sendKeyDBObj:[item keyDBItem]];
				NSBeginInformationalAlertSheet( LSPK( @"Revokers Updated", @"Added Revokers informational" ),
					kPGPkeysOKButton, nil, nil, [mPGPkeysView window], 
					self, NULL, NULL, NULL,
					[NSString stringWithFormat:LSPK(
					@"%u revoker(s) have been added to the selected key.", @"Added Revokers message" ), numRevokers] );
			}
		}
	}
done:
	if( IsPGPError( err ) && ( err != kPGPError_UserAbort ) )
	{
		NSBeginInformationalAlertSheet( LSPK( @"Error Adding Revoker", @"AddRevoker error title" ),
			kPGPkeysOKButton, nil, nil, [mPGPkeysView window], 
			self, NULL, NULL, NULL,
			[NSString stringWithFormat:LSPK( @"An error occurred adding the revoker:%@.",
			@"Revocation error message" ), PGPGetErrorNSString( err )] );
	}
	else if( IsntPGPError( resultCode ) )
		[mCKeyView reload];
	if( PGPKeySetRefIsValid( keySet ) )
		PGPFreeKeySet( keySet );
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
}

- (void)addRevoker
{
	PGPError		err = kPGPError_NoErr;
	PGPKeySetRef	keyV4Set = kInvalidPGPKeySetRef;
	PGPFilterRef	keyV4Filter = kInvalidPGPFilterRef;
	
	err = PGPNewKeyDBObjNumericFilter( mPGPContext, kPGPKeyProperty_Version, kPGPKeyVersion_V4,
			kPGPMatchCriterion_Equal, &keyV4Filter );	CKERR;
	err = PGPFilterKeySet( PGPPeekKeyDBRootKeySet(mKeyDB), keyV4Filter, &keyV4Set );	CKERR;
	PGPclBeginKeySelectSheet( LSPK( @"Please select the revoker to add:", @"AddRevoker Prompt" ),
		[mPGPkeysView window], mPrefRef, keyV4Set,
		mKeyDB, kPGPclKeySelect_FreeKeySet, self, @selector(revokerSelectDidEnd:selectedKeys:) );
done:
	if( PGPFilterRefIsValid( keyV4Filter ) )
		PGPFreeFilter( keyV4Filter );
}

- (IBAction)addNameAction:(id)sender
{
	[[NSApplication sharedApplication] endSheet:mAddNameWindow
		returnCode:[sender tag] ? kPGPError_UserAbort : kPGPError_NoErr ];
}

- (void)addNameSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	PGPError			err = kPGPError_NoErr;
	PGPBoolean			split = FALSE;
 	char				*passphrase = NULL;
	PGPByte				*passKey = NULL;
	PGPSize				passKeySize;
	BOOL				valid = YES;
	PGPBoolean			ksSync;
	
	[sheet orderOut:self];
	if( IsntPGPError( returnCode ) )
	{
		NSMutableString			*nameString;
		NSString				*emailString;
		
		nameString	= [NSMutableString stringWithString:[mAddNameNameField stringValue]];
		emailString	= [mAddNameEmailField stringValue];
		if( [nameString length] )
		{
			CKeyDBItem			*item = [[mCKeyView selectedItemArray] objectAtIndex:0];
			PGPBoolean			secret;
			
			PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPKeyProperty_IsSecret, &secret );
			if( secret )
			{
				err = PGPclGetKeyPassphrase( [item keyDBItem], mTLSContext, mPrefRef, &split, &passphrase, &passKey, &passKeySize );	CKERR;
				err = PGPclGuaranteeMinimumEntropy( mPGPContext );												CKERR;
				if( [emailString length] )
					[nameString appendFormat:@" <%@>", emailString];
				
				err = PGPAddUserID( [item keyDBItem], [nameString UTF8String], split ?
							PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
							PGPOPassphrase( mPGPContext, passphrase),
							PGPOLastOption( mPGPContext ) );													CKERR;
				[mCKeyView reload];
				PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyServerSyncOnAdd, &ksSync );
				if( ksSync )
					[self sendKeyDBObj:[item keyDBItem]];
			}
		}
		else
			valid = NO;
	}
done:
	if( !valid )
	{
		NSBeginAlertSheet( LSPK( @"Name Required", @"Add Name:Name Required Error" ),
			kPGPkeysOKButton, nil, nil, [mPGPkeysView window], 
			self, nil, nil, nil,
			LSPK( @"You must enter a name.", @"Add Name:Name Required Message" ) );
	}
	else if( IsPGPError( err ) )
		PGPclErrorBox( err );
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
}

- (void)addName
{
	CKeyDBItem		*item = [[mCKeyView selectedItemArray] objectAtIndex:0];
	PGPBoolean		ksSync;
	
	if( !item )
		return;
	PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyServerSyncOnAdd, &ksSync );
	if( ksSync )
		if( IsntPGPError( [self updateKeyDBObj:item atomic:YES] ) )
			[mCKeyView reload];
	[mAddNameNameField setStringValue:@""];
	[mAddNameEmailField setStringValue:@""];
	[[NSApplication sharedApplication] beginSheet:mAddNameWindow modalForWindow:[mPGPkeysView window]
		modalDelegate:self didEndSelector:@selector(addNameSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
}

- (IBAction)addPhotoAction:(id)sender
{
	[[NSApplication sharedApplication] endSheet:mAddPhotoWindow
		returnCode:[sender tag] ? kPGPError_UserAbort : kPGPError_NoErr ];
}

- (void)addPhotoSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	PGPError			err = kPGPError_NoErr;
	PGPBoolean			split = FALSE;
 	char				*passphrase = NULL;
	PGPByte				*passKey = NULL;
	PGPSize				passKeySize;
	BOOL				valid = YES;
	PGPBoolean			ksSync;
	PGPByte				*photoData = NULL;
	
	[sheet orderOut:self];
	if( IsntPGPError( returnCode ) )
	{
		NSImage			*image = [mAddPhotoImageView image];
		
		if( image )
		{
			CKeyDBItem			*item = [[mCKeyView selectedItemArray] objectAtIndex:0];
			PGPBoolean			secret;
			
			PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPKeyProperty_IsSecret, &secret );
			if( secret )
			{
				NSData					*imageData;
				PGPSize					photoDataSize;
				PGPPhotoUserIDHeader	*photoHeader;
				PGPUInt16				headerLength;
				
				err = PGPclGetKeyPassphrase( [item keyDBItem], mTLSContext, mPrefRef,
							&split, &passphrase, &passKey, &passKeySize );										CKERR;
				err = PGPclGuaranteeMinimumEntropy( mPGPContext );												CKERR;
				
				[image setScalesWhenResized:YES];
				[image setDataRetained:YES];
				[image setSize:NSMakeSize( kMaxPhotoIDWidth, kMaxPhotoIDHeight )];
				[image recache];
				PGPLog(( @"representations: %@", [image representations] ));
				imageData = [NSBitmapImageRep representationOfImageRepsInArray:[image representations]
								usingType:NSJPEGFileType
								properties:[NSDictionary dictionaryWithObject:
								[NSNumber numberWithFloat:0.55] forKey:NSImageCompressionFactor]];
				PGPLog(( @"imageData length = %u", [imageData length] ));
				photoDataSize = [imageData length] + sizeof(PGPPhotoUserIDHeader);
				photoData = PGPNewData( PGPPeekContextMemoryMgr( mPGPContext ), [imageData length] + sizeof(PGPPhotoUserIDHeader),
								kPGPMemoryMgrFlags_Clear );
				if( IsntNull( photoData ) )
				{
					photoHeader = (PGPPhotoUserIDHeader *)photoData;
					headerLength = sizeof( *photoHeader );
					photoHeader->headerVersion 	= kPGPPhotoIDHeaderVersion;
					photoHeader->photoIDFormat	= kPGPPhotoIDFormat_JPEG;
					PGPUInt16ToStorage( headerLength, (PGPByte *) &photoHeader->headerLength );
					[imageData getBytes:(PGPByte *) photoData + headerLength];
					
					err = PGPAddAttributeUserID( [item keyDBItem],
							kPGPAttribute_Image, photoData, photoDataSize,
							split ?	PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
									PGPOPassphrase( mPGPContext, passphrase),
							PGPOLastOption( mPGPContext ) );														CKERR;
					[mCKeyView reload];
					PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyServerSyncOnAdd, &ksSync );
					if( ksSync )
						[self sendKeyDBObj:[item keyDBItem]];
				}
				else
					err = kPGPError_OutOfMemory;
			}
		}
		else
			valid = NO;
	}
done:
	if( !valid )
	{
		NSBeginAlertSheet( LSPK( @"Image Required", @"Add Photo:Image Required Error" ),
			kPGPkeysOKButton, nil, nil, [mPGPkeysView window], 
			self, nil, nil, nil,
			LSPK( @"You must specify an image in order to add a Photo ID.", @"Add Photo:Image Required Message" ) );
	}
	else if( IsPGPError( err ) )
		PGPclErrorBox( err );
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
	if( IsntNull( photoData ) )
		PGPFreeData( photoData );
}

- (void)addPhoto
{
	PGPError		err = kPGPError_NoErr;
	CKeyDBItem		*item = [[mCKeyView selectedItemArray] objectAtIndex:0];
	PGPBoolean		ksSync;
	PGPKeyIterRef	keyIter = kInvalidPGPKeyIterRef;
	PGPKeyDBObjRef	userID;
	PGPBoolean		isAttributeID;
	
	if( !item )
		return;
	PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyServerSyncOnAdd, &ksSync );
	if( ksSync )
		if( IsntPGPError( [self updateKeyDBObj:item atomic:YES] ) )
			[mCKeyView reload];
	
	err = PGPNewKeyIterFromKeyDB( mKeyDB, &keyIter );		CKERR;
	PGPKeyIterSeek( keyIter, [item keyDBItem] );
	while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_UserID, &userID ) ) )
	{
		PGPGetKeyDBObjBooleanProperty( userID, kPGPUserIDProperty_IsAttribute, &isAttributeID );
		if( isAttributeID )
		{
			PGPInt32	attributeType;
			
			PGPGetKeyDBObjNumericProperty( userID, kPGPUserIDProperty_AttributeType, &attributeType );
			if( attributeType == kPGPAttribute_Image )
			{
				NSBeginAlertSheet( LSPK( @"You cannot add more than one Photo ID.", @"Add Photo:Limit Error" ),
					kPGPkeysOKButton, nil, nil, [mPGPkeysView window], 
					self, nil, nil, nil,
					LSPK( @"You may add a new Photo ID by deleting the old Photo ID first.", @"Add Name:Limit Message" ) );
				return;
			}
		}
	}
	[mAddPhotoImageView setImage:nil];
	[[NSApplication sharedApplication] beginSheet:mAddPhotoWindow modalForWindow:[mPGPkeysView window]
		modalDelegate:self didEndSelector:@selector(addPhotoSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
done:
	if( PGPKeyIterRefIsValid( keyIter ) )
		PGPFreeKeyIter( keyIter );
}

- (IBAction)progressCancelAction:(id)sender
{
	mProgressAbort = YES;
}

- (void)doProgress
{
	mProgressAbort	= NO;
	[mProgressWindow center];
	[mProgressProgressIndicator startAnimation:self];
	[[NSApplication sharedApplication] runModalForWindow:mProgressWindow];
}

- (void)endCAProgress:(id)sender resultCode:(PGPError)err
{
	NSDate		*sleepUntil = [NSDate dateWithTimeIntervalSinceNow:1];

	while( ![[NSApplication sharedApplication] modalWindow] )
	{
		[NSThread sleepUntilDate:sleepUntil];
		[sleepUntil addTimeInterval:1];
	}
	[mProgressProgressIndicator stopAnimation:self];
	[[NSApplication sharedApplication] stopModal];
	[mProgressWindow orderOut:self];
	if( IsntPGPError( err ) )
	{
		NSString	*str = nil;
		
		switch( mCAOp )
		{
			case kPGPCA_OP_RequestCert:
				if( mCARequestType == kPGPPrefCertRequestPKCS10 )
					str = LSPK( @"Your certificate request has been placed on the clipboard. Please follow your administrator's instructions to complete the certificate request process.", @"PKCS10 Success" );
				else
					str = LSPK( @"Your certificate has been successfully requested. You will be notified when it is ready.", @"CRS Success" );
				break;
			case kPGPCA_OP_RetrieveCert:
				[mCKeyView reload];
				str = LSPK( @"Your certificate has been successfully retrieved!", @"CertRetrieve Success" );
				break;
			case kPGPCA_OP_UpdateCRL:
				str = LSPK( @"The Certificate Revocation List has been successfully updated.", @"CRL Updated" );
				break;
			default:
				break;
		}
		if( str )
		{
			NSBeginInformationalAlertSheet( LSPK( @"Success", @"CA Success Title" ),
				kPGPkeysOKButton, nil, nil, [[NSApplication sharedApplication] keyWindow], 
				self, NULL, NULL, NULL, str );
		}
	}
	else
		PGPclErrorBox( err );
	[sender autorelease];
}

- (void)certOp
{
	PGPError			err = kPGPError_NoErr;
	PGPKeyDBObjRef		userID = kInvalidPGPKeyDBObjRef;
	PGPUInt32			objType;
	PGPKeyServerClass	serverClass;
	PGPAttributeValue	*avList = NULL;
	PGPUInt32			numAVs = 0;
	
	mCARequestType = kPGPPrefCertRequestCRS;
	if( PGPKeyDBObjRefIsValid( mFirstSelection ) ||
		( mCAOp == kPGPCA_OP_UpdateCRL ) &&
		!(	( serverClass == kPGPKeyServerClass_Verisign ) ||
			( serverClass == kPGPKeyServerClass_Entrust ) ) )
	{
		PGPGetPrefNumber( mPrefRef, kPGPPrefCAType, (PGPUInt32 *)&serverClass );
		PGPGetPrefNumber( mPrefRef, kPGPPrefCertRequestType, (PGPUInt32 *)&mCARequestType );
		if( PGPKeyDBObjRefIsValid( mFirstSelection ) )
		{
			PGPGetKeyDBObjNumericProperty( mFirstSelection, kPGPKeyDBObjProperty_ObjectType, &objType );
			if( objType == kPGPKeyDBObjType_Key )
				PGPGetPrimaryUserID( mFirstSelection, &userID );
			else if( objType == kPGPKeyDBObjType_UserID )
				userID = mFirstSelection;
			mCATargetKey	= PGPPeekKeyDBObjKey( userID );
			mCATargetUser	= userID;
		}
		else
		{
			mCATargetKey	= kInvalidPGPKeyDBObjRef;
			mCATargetUser	= kInvalidPGPKeyDBObjRef;
		}
		if( mCAOp == kPGPCA_OP_RequestCert )
		{
			err = PGPclGetCACertReqAVList( mPGPContext, mPrefRef, FALSE, userID,
					serverClass, &mCARequestType, &avList, &numAVs );
		}
		if( IsntPGPError( err ) )
		{
			CCAThread	*caThread = [[CCAThread alloc] initCAThread:mPGPContext prefRef:mPrefRef
										tlsContext:mTLSContext op:mCAOp reqType:mCARequestType
										keyDB:mKeyDB targetKey:mCATargetKey passphrase:NULL
										avList:avList numAVs:numAVs status:mProgressStatusField
										abort:&mProgressAbort target:self
										didEndSelector:@selector(endCAProgress:resultCode:)];
			
			[mProgressStatusField setStringValue:@""];
			switch( mCAOp )
			{
				case kPGPCA_OP_RequestCert:
					[mProgressWindow setTitle:LSPK( @"Requesting Certificate", @"ReqCert Window Title" )];
					break;
				case kPGPCA_OP_RetrieveCert:
					[mProgressWindow setTitle:LSPK( @"Retrieving Certificate", @"RetCert Window Title" )];
					break;
				case kPGPCA_OP_UpdateCRL:
					[mProgressWindow setTitle:LSPK( @"Updating Revocations", @"UpdateCRL Window Title" )];
					break;
				default:
					break;
			}
			err = [caThread run];
			if( IsntPGPError( err ) )
			{
				[self doProgress];
			}
			else
			{
				if( IsntNull( avList ) )
					PGPclFreeCACertReqAVList( avList, numAVs );
			}
		}
	}
	if( IsPGPError( err ) )
		PGPclErrorBox( err );
}

- (PGPError)updateKeyDBObj:(CKeyDBItem*)keyItem atomic:(BOOL)atomic
{
	PGPError				err = kPGPError_NoErr;
	PGPContextRef			context = kInvalidPGPContextRef;
	NSMutableDictionary*	dict = [NSMutableDictionary dictionary];
	
	if (!keyItem)
		return kPGPError_BadParams;
	
	if (IsPGPError(err = PGPNewContext(kPGPsdkAPIVersion, &context)))
		return err;
	
	[dict setObject:[NSArray arrayWithObject:keyItem] forKey:@"keys"];
	[dict setObject:[NSData dataWithBytes:&context length:sizeof(PGPContextRef)] forKey:@"context"];
	[dict setObject:[NSNumber numberWithBool:atomic] forKey:@"flush"];
	
	[NSThread detachNewThreadSelector:@selector(updateKeys:) toTarget:self withObject:dict];
	
	return err;
}

- (void)updateSelection
{
	NSEnumerator*			selectedItemEnumerator = [mCKeyView selectedItemEnumerator];
	NSMutableArray*			keys = [NSMutableArray array];
	NSMutableDictionary*	dict = [NSMutableDictionary dictionary];
	CKeyDBItem*				item;
	PGPContextRef			context = kInvalidPGPContextRef;
	PGPError				err = kPGPError_NoErr;
	
	err = PGPNewContext(kPGPsdkAPIVersion, &context);
	pgpAssertNoErr(err);
	
	if (IsPGPError(err))
		return;
	
	while (item = [selectedItemEnumerator nextObject])
		[keys addObject:item];
	
	[dict setObject:keys forKey:@"keys"];
	[dict setObject:[NSData dataWithBytes:&context length:sizeof(PGPContextRef)] forKey:@"context"];
	[dict setObject:[NSNumber numberWithBool:YES] forKey:@"flush"];
	
	[NSThread detachNewThreadSelector:@selector(updateKeys:) toTarget:self withObject:dict];
}

- (void)updateKeys:(NSDictionary*)info
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSEnumerator*		enumerator;
	NSArray*			keys;
	NSMutableArray*		keyIDs = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray*		domains = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray*		keyserverEntryArray = [[[NSMutableArray alloc] init] autorelease];
	NSString*			keyIDString;
	CKeyDBItem*			item;
	CKeyServerUpdate*	sheet = nil;
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context;
	PGPKeyDBRef			collectedKeys = kInvalidPGPKeyDBRef;
	PGPKeyDBObjRef		keyObj;
	PGPKeyServerEntry*	keyserverList;
	PGPUInt32			listCount;
	PGPUInt32			keysCount = 0;
	PGPUInt32			itemCount = 0;
	PGPInt32			objType;
	PGPFilterRef		filter = kInvalidPGPFilterRef;
	PGPFilterRef		idFilter = kInvalidPGPFilterRef;
	PGPKeyID			keyID;
	PGPSize				len;
	int					i;
	char				domainStr[256];
	char				userID[256];
	char				keyIDStr[128];
	
	if (!info)
		return;
	
	if (IsPGPError(err = PGPKeyServerInit()))
		goto done;
	
	sheet = [CKeyServerUpdate sharedInstance];
	[NSApp beginSheet:[sheet window] modalForWindow:[self window] modalDelegate:nil didEndSelector:nil contextInfo:NULL];
	[[sheet progress] startAnimation:nil];
	
	keys = (NSArray*) [info objectForKey:@"keys"];
	context = *((PGPContextRef*) [(NSData*) [info objectForKey:@"context"] bytes]);
	
	enumerator = [keys objectEnumerator];
	
	while (item = [enumerator nextObject])
	{
		PGPKeyDBObjRef		topKey = kInvalidPGPKeyDBObjRef;
		
		domainStr[0] = '\0';
		len = sizeof(domainStr);
		keyObj = [item keyDBItem];
		pgpAssert(PGPKeyDBObjRefIsValid(keyObj));
		
		/* Obtain the keyID of the selected object */
		PGPGetKeyDBObjNumericProperty(keyObj, kPGPKeyDBObjProperty_ObjectType, &objType);
		switch (objType)
		{
			case kPGPKeyDBObjType_UserID:
				topKey = PGPPeekKeyDBObjKey(keyObj);
				/* Intentionally fall through */
			case kPGPKeyDBObjType_Key:
				if (!PGPKeyDBObjRefIsValid(topKey))
					topKey = keyObj;
				
				err = PGPGetKeyID(topKey, &keyID);
				pgpAssertNoErr(err);
				
				err = PGPGetKeyIDString(&keyID, kPGPKeyIDString_Full, keyIDStr);
				pgpAssertNoErr(err);
				
				keyIDString = [NSString stringWithUTF8String:keyIDStr];
				if (![keyIDs containsObject:keyIDString])
				{
					[keyIDs addObject:keyIDString];
					++itemCount;
				}
				
				/* Note the domain so that we can search for an appropriate keyserver */
				err = PGPGetPrimaryUserIDName(topKey, userID, sizeof(userID) - 1, &len);
				pgpAssertNoErr(err);
				
				userID[len] = '\0';
				
				PGPFindEmailDomain(userID, domainStr);
				
				if (domainStr[0])
				{
					NSString*	domain = [NSString stringWithUTF8String:domainStr];
					
					if (![domains containsObject:domain])
						[domains addObject:domain];
				}
				
				break;
			case kPGPKeyDBObjType_Signature:
				err = PGPGetKeyDBObjDataProperty(keyObj, kPGPSigProperty_KeyID, &keyID, sizeof(PGPKeyID), &len);
				pgpAssertNoErr(err);
				
				err = PGPGetKeyIDString(&keyID, kPGPKeyIDString_Full, keyIDStr);
				pgpAssertNoErr(err);
				
				keyIDString = [NSString stringWithUTF8String:keyIDStr];
				if (![keyIDs containsObject:keyIDString])
				{
					[keyIDs addObject:keyIDString];
					++itemCount;
				}
				
				break;
			default:
				break;
		}
		
		/* Create a filter based on the collected keyIDs */
		if (IsntPGPError(err))
		{
			err = PGPNewKeyDBObjDataFilter(context,
						kPGPKeyProperty_KeyID,
						&keyID,
						sizeof(PGPKeyID),
						kPGPMatchCriterion_Equal,
						&filter);
			
			if (IsntPGPError(err))
			{
				if (PGPFilterRefIsValid(idFilter))
				{
					PGPFilterRef	combinedFilter = kInvalidPGPFilterRef;
					
					err = PGPUnionFilters(idFilter, filter, &combinedFilter);
					pgpAssertNoErr(err);
					
					idFilter = combinedFilter;
				}
				else
					idFilter = filter;
			}
		}
	}
	
	if (IsPGPError(err))
		goto done;
	
	if (IsPGPError(err = PGPGetKeyServerPrefs(mPrefRef, &keyserverList, &listCount)))
		goto done;
	
	/* Determine if any of the keyservers handle the domains for the selected key objects */
	if ([domains count])
	{
		NSEnumerator*		domainEnumerator = [domains objectEnumerator];
		NSEnumerator*		keyserverEnumerator;
		NSData*				data;
		NSString*			domain;
		BOOL				found = NO;
		
		while (domain = [domainEnumerator nextObject])
		{
			for (i = 0; i < listCount; ++i)
			{
				if (!strncasecmp(keyserverList[i].domain, [domain UTF8String], sizeof(domainStr)))
				{
					NSData*				keyserverEntry = [NSData dataWithBytes:&keyserverList[i] length:sizeof(PGPKeyServerEntry)];
					PGPKeyServerEntry	ksEntry;
					
					keyserverEnumerator = [keyserverEntryArray objectEnumerator];
					while ((data = [keyserverEnumerator nextObject]) && !found)
					{
						[data getBytes:&ksEntry length:sizeof(PGPKeyServerEntry)];
						
						if (!strncasecmp(keyserverList[i].serverDNS, ksEntry.serverDNS, sizeof(ksEntry.serverDNS)))
							found = YES;
					}
					
					if (!found)
					{
						PGPLog((@"adding keyserver: %s", keyserverList[i].serverDNS));
						[keyserverEntryArray addObject:keyserverEntry];
					}
				}
			}
		}
	}
	
	/* Add keyservers which handle all domains */
	for (i = 0; i < listCount; ++i)
	{
		if (keyserverList[i].domain[0] == '\0')
		{
			NSEnumerator*		enumerator = [keyserverEntryArray objectEnumerator];
			NSData*				data;
			NSData*				keyserverEntry = [NSData dataWithBytes:&keyserverList[i] length:sizeof(PGPKeyServerEntry)];
			PGPKeyServerEntry	ksEntry;
			BOOL				found = NO;
			
			while ((data = [enumerator nextObject]) && !found)
			{
				[data getBytes:&ksEntry length:sizeof(PGPKeyServerEntry)];
				
				if (!strncasecmp(keyserverList[i].serverDNS, ksEntry.serverDNS, sizeof(ksEntry.serverDNS)))
					found = YES;
			}
			
			if (!found)
			{
				PGPLog((@"adding keyserver: %s", keyserverList[i].serverDNS));
				[keyserverEntryArray addObject:keyserverEntry];
			}
		}
	}
	
	PGPFreeKeyServerList(keyserverList);
	
	if (IsPGPError(err = PGPNewKeyDB(context, &collectedKeys)))
		goto done;
	
	/* Search each keyserver until all key objects are found */
	if ([keyserverEntryArray count])
	{
		NSEnumerator*			keyserverEnumerator = [keyserverEntryArray objectEnumerator];
		NSData*					keyserverEntryData;
		PGPKeyServerEntry*		keyserverEntry;
		PGPKeyServerRef			keyserver;
		PGPKeyServerProtocol	protocol;
		PGPtlsSessionRef		tls;
		PGPFilterRef			httpFilter;
		PGPKeyDBRef				results;
		PGPKeyDBObjRef			tmpKey;
		PGPKeyIterRef			iter;
		
		while (IsntPGPError(err) && [keyIDs count] && (keyserverEntryData = [keyserverEnumerator nextObject]))
		{
			keyserverEntry = (PGPKeyServerEntry*) [keyserverEntryData bytes];
			keyserver = kInvalidPGPKeyServerRef;
			tls	= kInvalidPGPtlsSessionRef;
			httpFilter = kInvalidPGPFilterRef;
			results = kInvalidPGPKeyDBRef;
			
			PGPLog((@"connecting to server: %s", keyserverEntry->serverDNS));
			
			err = PGPNewKeyServer(context,
						keyserverEntry->type,
						&keyserver,
						PGPONetHostName(context, keyserverEntry->serverDNS, keyserverEntry->serverPort),
						PGPOKeyServerProtocol(context, keyserverEntry->protocol),
						PGPOKeyServerKeySpace(context, kPGPKeyServerKeySpace_Normal),
						PGPOKeyServerAccessType(context, kPGPKeyServerAccessType_Normal),
						PGPOLastOption(context));
			
			[[sheet server] setStringValue:[NSString stringWithUTF8String:keyserverEntry->serverDNS]];
			[[sheet server] performSelectorOnMainThread:@selector(display) withObject:nil waitUntilDone:NO];
			
			if (IsntPGPError(err))
			{
				err = PGPSetKeyServerEventHandler(keyserver, sKeyServerCallback, [self window]);
				pgpAssertNoErr(err);
				
				err = PGPGetKeyServerProtocol(keyserver, &protocol);
				if (IsntPGPError(err) && (protocol == kPGPKeyServerProtocol_LDAPS))
					err = PGPNewTLSSession(mTLSContext, &tls);
				
				if (IsntPGPError(err))
				{
					err = PGPKeyServerOpen(keyserver, tls);
					if (IsntPGPError(err))
					{
						if ((protocol == kPGPKeyServerProtocol_HTTP) || (itemCount > 50))
						{
							enumerator = [keyIDs objectEnumerator];
							
							while (IsntPGPError(err) && (keyIDString = [enumerator nextObject]))
							{
								err = PGPNewKeyIDFromString([keyIDString UTF8String], kPGPPublicKeyAlgorithm_Invalid, &keyID);
								pgpAssertNoErr(err);
								
								if (IsntPGPError(err))
								{
									err = PGPNewKeyDBObjDataFilter(context,
												kPGPKeyProperty_KeyID,
												&keyID,
												sizeof(PGPKeyID),
												kPGPMatchCriterion_Equal,
												&httpFilter);
									pgpAssertNoErr(err);
									
									if (IsntPGPError(err))
									{
										err = PGPQueryKeyServer(keyserver, httpFilter, &results);
										
										if (IsntPGPError(err))
										{
											err = PGPNewKeyIterFromKeyDB(results, &iter);
											pgpAssertNoErr(err);
											
											while (IsntPGPError(err = PGPKeyIterNextKeyDBObj(iter, kPGPKeyDBObjType_Key, &tmpKey)))
											{
												err = PGPGetKeyID(tmpKey, &keyID);
												pgpAssertNoErr(err);
												
												err = PGPGetKeyIDString(&keyID, kPGPKeyIDString_Full, keyIDStr);
												pgpAssertNoErr(err);
												
												keyIDString = [NSString stringWithUTF8String:keyIDStr];
												if ([keyIDs containsObject:keyIDString])
													[keyIDs removeObject:keyIDString];
											}
											
											PGPLog((@"%d keys remaining after searching server %s", [keyIDs count], keyserverEntry->serverDNS));
											
											if (PGPKeyIterRefIsValid(iter))
												PGPFreeKeyIter(iter);
											
											err = PGPCopyKeys(PGPPeekKeyDBRootKeySet(results), collectedKeys, NULL);
											pgpAssertNoErr(err);
											
											if (PGPKeyDBRefIsValid(results))
												PGPFreeKeyDB(results);
										}
										else
											PGPLog((@"PGPQueryKeyServer returned err: %d", err));
									}
									
									if (PGPFilterRefIsValid(httpFilter))
										PGPFreeFilter(httpFilter);
								}
							}
						}
						else	/* General case is that we're searching an LDAP(S) keyserver */
						{
							err = PGPQueryKeyServer(keyserver, idFilter, &results);
							
							if (IsntPGPError(err))
							{
								err = PGPNewKeyIterFromKeyDB(results, &iter);
								pgpAssertNoErr(err);
								
								while (IsntPGPError(err = PGPKeyIterNextKeyDBObj(iter, kPGPKeyDBObjType_Key, &tmpKey)))
								{
									err = PGPGetKeyID(tmpKey, &keyID);
									pgpAssertNoErr(err);
									
									err = PGPGetKeyIDString(&keyID, kPGPKeyIDString_Full, keyIDStr);
									pgpAssertNoErr(err);
									
									keyIDString = [NSString stringWithUTF8String:keyIDStr];
									if ([keyIDs containsObject:keyIDString])
										[keyIDs removeObject:keyIDString];
								}
								
								PGPLog((@"%d keys remaining after searching server %s", [keyIDs count], keyserverEntry->serverDNS));
								
								if (PGPKeyIterRefIsValid(iter))
									PGPFreeKeyIter(iter);
								
								err = PGPCopyKeys(PGPPeekKeyDBRootKeySet(results), collectedKeys, NULL);
								pgpAssertNoErr(err);
								
								if (PGPKeyDBRefIsValid(results))
									PGPFreeKeyDB(results);
							}
							else
								PGPLog((@"PGPQueryKeyServer returned err: %d", err));
						}
					}
				}
				else
					PGPLog((@"PGPGetKeyServerProtocol returned err: %d", err));
				
				if (PGPtlsSessionRefIsValid(tls))
					PGPFreeTLSSession(tls);
				
				PGPKeyServerClose(keyserver);
				PGPFreeKeyServer(keyserver);
				
				if (err != kPGPError_UserAbort)
					err = kPGPError_NoErr;
			}
		}
	}
	
	PGPCountKeysInKeyDB(collectedKeys, &keysCount);
	
done:
	PGPKeyServerCleanup();
	
	if (keysCount)
	{
		NSMutableDictionary*	dict = [NSMutableDictionary dictionary];
		
		[dict setObject:[NSData dataWithBytes:&collectedKeys length:sizeof(PGPKeyDBRef)] forKey:@"keyDB"];
		[dict setObject:[info objectForKey:@"flush"] forKey:@"flush"];
		
		PGPLog((@"Collected %d keys", keysCount));
		
		[self performSelectorOnMainThread:@selector(updateMainKeyDBWithKeys:) withObject:dict waitUntilDone:YES];
	}
	
	if (PGPFilterRefIsValid(idFilter))
		PGPFreeFilter(idFilter);
	
	if (PGPKeyDBRefIsValid(collectedKeys))
		PGPFreeKeyDB(collectedKeys);
	
	if (PGPContextRefIsValid(context))
		PGPFreeContext(context);
	
	if (sheet && [[sheet window] isVisible])
	{
		[[sheet progress] stopAnimation:nil];
		[NSApp endSheet:[sheet window]];
		[sheet close];
		
		[[sheet server] setStringValue:@""];
		[[sheet status] setStringValue:@""];
	}
	
	[pool release];
}

- (void)updateMainKeyDBWithKeys:(NSDictionary*)info
{
	PGPError		err;
	PGPKeyDBRef		newKeys = kInvalidPGPKeyDBRef;
	BOOL			flush = YES;
	
	if (!info)
		return;
	
	newKeys = *((PGPKeyDBRef*) [(NSData*) [info objectForKey:@"keyDB"] bytes]);
	
	err = PGPCopyKeys(PGPPeekKeyDBRootKeySet(newKeys), mKeyDB, NULL);
	pgpAssertNoErr(err);
	
	flush = [(NSNumber*) [info objectForKey:@"flush"] boolValue];
	
	if (flush)
	{
		PGPFlushKeyDB(mKeyDB);
		[self reload];
	}
}

- (PGPError)sendKeyDBObj:(PGPKeyDBObjRef)keyDBObj
{
	PGPError		err = kPGPError_NoErr;
	
	keyDBObj = PGPPeekKeyDBObjKey( keyDBObj );
	err = PGPclSendKeyToServer( mPGPContext, mTLSContext, mPrefRef, keyDBObj, mTargetServer );
	return err;
}

- (void)sendSelection
{
	NSEnumerator	*selectedItemEnumerator = [mCKeyView selectedItemEnumerator];
	CKeyDBItem		*item;
	PGPInt32		successes = 0;
	PGPError		err = kPGPError_NoErr;
	
	while( item = [selectedItemEnumerator nextObject] )
	{
		err = [self sendKeyDBObj:[item keyDBItem]];
		if( IsPGPError( err ) )
		{
			PGPclErrorBox( err );
			break;
		}
		else
			successes++;
	}
	if( successes )
	{
		NSBeginInformationalAlertSheet( LSPK( @"Success", @"Server send title" ),
			kPGPkeysOKButton, nil, nil, [mPGPkeysView window], 
			self, NULL, NULL, NULL, [NSString stringWithFormat:
			LSPK( @"%d key(s) were successfully sent to the server.",
			@"Server send message" ), successes] );
	}
}

- (void)keysMenuAction:(id)sender
{
	switch( [sender tag] )
	{
		case kPGPkeysCommand_New:
			[self newKey];
			break;
		case kPGPkeysCommand_Sign:
			[self sign];
			break;
		case kPGPkeysCommand_Revoke:
			[self revoke];
			break;
		case kPGPkeysCommand_SetDefault:
			[mCKeyView setDefault];
			break;
		case kPGPkeysCommand_AddName:
			[self addName];
			break;
		case kPGPkeysCommand_AddPhoto:
			[self addPhoto];
			break;
		case kPGPkeysCommand_AddRevoker:
			[self addRevoker];
			break;
		case kPGPkeysCommand_AddCertificate:
			mCAOp	= kPGPCA_OP_RequestCert;
			[self certOp];
			break;
		case kPGPkeysCommand_RetrieveCert:
			mCAOp	= kPGPCA_OP_RetrieveCert;
			[self certOp];
			break;
		case kPGPkeysCommand_UpdateRevocations:
			mCAOp	= kPGPCA_OP_UpdateCRL;
			[self certOp];
			break;
		case kPGPkeysCommand_SendToThisServer:
			mTargetServer = [[sender representedObject] mutableBytes];
			[self sendSelection];
			break;
		case kPGPkeysCommand_SendToServer:
			mTargetServer = NULL;
			[self sendSelection];
			break;
		case kPGPkeysCommand_UpdateFromServer:
			[self updateSelection];
			break;
		case kPGPkeysCommand_Enable:
			[mCKeyView enableSelection:YES];
			break;
		case kPGPkeysCommand_Disable:
			[mCKeyView enableSelection:NO];
			break;
		case kPGPkeysCommand_ReverifySignatures:
			[self reverifySigs];
			break;
		case kPGPkeysCommand_Split:
			[mController split:self forKey:mFirstSelection];
			break;
		case kPGPkeysCommand_Reconstruct:
			break;
		case kPGPkeysCommand_Search:
			[self search];
			break;
		case kPGPkeysCommand_Import:
		{
			NSArray		*fileTypes = [[[NSArray alloc] initWithObjects:@"asc", @"pkr", @"skr", @"pfx", @"p12", @"gpg",
										NSFileTypeForHFSTypeCode('TEXT'),
										NSFileTypeForHFSTypeCode('pgPR'),
										NSFileTypeForHFSTypeCode('pgRR'), nil] autorelease];
			NSOpenPanel	*openPanel = [NSOpenPanel openPanel];
			
			[openPanel setAllowsMultipleSelection:YES];
			[openPanel beginSheetForDirectory:NSHomeDirectory()
				file:nil types:fileTypes
				modalForWindow:[mPGPkeysView window]
				modalDelegate:self
				didEndSelector:@selector(importSheetDidEnd:returnCode:contextInfo:)
				contextInfo:NULL];
			break;
		}
		case kPGPkeysCommand_Export:
		{
			NSSavePanel	*savePanel = [NSSavePanel savePanel];
			
			[savePanel setRequiredFileType:@"asc"];
			[savePanel setCanSelectHiddenExtension:YES];
			[savePanel setAccessoryView:mSavePanelAuxView];
			[savePanel beginSheetForDirectory:NSHomeDirectory()
				file:LSPK(@"Exported Keys",@"Suggested export filename")
				modalForWindow:[mPGPkeysView window]
				modalDelegate:self
				didEndSelector:@selector(exportSheetDidEnd:returnCode:contextInfo:)
				contextInfo:NULL];
			break;
		}
		case kPGPkeysCommand_Info:
			[mCKeyView showInfo];
			break;
		case kPGPkeysCommand_ServerDelete:
		case kPGPkeysCommand_ServerDisable:
		{
			CKeySearch	*keySearch = mController;
			
			[keySearch serverCommand:[sender tag]];
			break;
		}
		case kPGPkeysCommand_Delete:
			if( PGPKeyDBObjRefIsValid( mFirstSelection ) )
				[mCKeyView clear:self];
			else
				NSBeep();
			break;
		default:
			break;
	}
}

- (NSToolbarItem*)toolbar:(NSToolbar*)toolbar itemForItemIdentifier:(NSString*)identifier willBeInsertedIntoToolbar:(BOOL)flag
{
	if ([identifier isEqualToString:kPGPkeysToolbarItemNew])
		return mToolbarItemNew;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemRevoke])
		return mToolbarItemRevoke;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemSign])
		return mToolbarItemSign;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemDelete])
		return mToolbarItemDelete;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemSearch])
		return mToolbarItemSearch;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemSendToServer])
		return mToolbarItemSendToServer;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemUpdate])
		return mToolbarItemUpdate;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemShowInfo])
		return mToolbarItemShowInfo;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemImport])
		return mToolbarItemImport;
	else if ([identifier isEqualToString:kPGPkeysToolbarItemExport])
		return mToolbarItemExport;
	else
		return nil;
}

- (NSArray*)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar
{
	return [NSArray arrayWithObjects:kPGPkeysToolbarItemNew,
					kPGPkeysToolbarItemRevoke,
					kPGPkeysToolbarItemSign,
					kPGPkeysToolbarItemDelete,
					kPGPkeysToolbarItemSearch,
					kPGPkeysToolbarItemSendToServer,
					kPGPkeysToolbarItemUpdate,
					kPGPkeysToolbarItemShowInfo,
					kPGPkeysToolbarItemImport,
					kPGPkeysToolbarItemExport,
					NSToolbarSeparatorItemIdentifier,
					NSToolbarSpaceItemIdentifier,
					NSToolbarFlexibleSpaceItemIdentifier,
					NSToolbarCustomizeToolbarItemIdentifier,
					nil];
}

- (NSArray*)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar
{
	return [NSArray arrayWithObjects:kPGPkeysToolbarItemNew,
					kPGPkeysToolbarItemRevoke,
					kPGPkeysToolbarItemSign,
					kPGPkeysToolbarItemDelete,
					kPGPkeysToolbarItemShowInfo,
					NSToolbarSeparatorItemIdentifier,
					kPGPkeysToolbarItemSendToServer,
					kPGPkeysToolbarItemUpdate,
					NSToolbarSeparatorItemIdentifier,
					kPGPkeysToolbarItemImport,
					kPGPkeysToolbarItemExport,
					NSToolbarSeparatorItemIdentifier,
					kPGPkeysToolbarItemSearch,
					nil];
}

- (void)dealloc
{
	PGPError				err;
	PGPclKeyListColumnInfo	columnInfo;
	
	if( mIsMainDB )
	{
		columnInfo = [mCKeyView columns];
		err = PGPSetPrefData( mPrefRef, kPGPPrefPGPkeysMacColumnData, sizeof(PGPclKeyListColumnInfo), &columnInfo );
		if( err )
			PGPLog(( @"Couldnt set column pref data, err =%d", err ));
		PGPclWritePrefs( mPrefRef );
	}
	[mCKeyView cleanPasteboards];
	[mCKeyView release];
	mCKeyView = nil;
	[mPGPkeysView release];
	[mSavePanelAuxView release];
	[mAddPhotoWindow release];
	[mAddNameWindow release];
	[mProgressWindow release];
	
	[mToolbar release];
	[mToolbarItemNew release];
	[mToolbarItemRevoke release];
	[mToolbarItemSign release];
	[mToolbarItemDelete release];
	[mToolbarItemSearch release];
	[mToolbarItemSendToServer release];
	[mToolbarItemUpdate release];
	[mToolbarItemShowInfo release];
	[mToolbarItemImport release];
	[mToolbarItemExport release];
	
	if( !mIsMainDB && PGPKeyDBRefIsValid( mKeyDB) )
	{
		PGPFreeKeyDB( mKeyDB );
		mKeyDB = kInvalidPGPKeyDBRef;
	}
	PGPLog(( @"Deallocated CPGPkeysView" ));	
	[super dealloc];
}

@end

