/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPkeysController.m,v 1.82 2002/11/11 21:41:42 jeffc Exp $
____________________________________________________________________________*/
#import <PGPclient/pgpKeyServerPrefs.h>
#import "PGPkeysController.h"
#import "PGPkeysPreferences.h"
#import "CPGPkeysView.h"
#import "CKeyring.h"
#import "CKeySign.h"
#import "CKeySplit.h"
#import "CKeySearch.h"
#import "CSendShares.h"
#import <PGPclient/CKeyGen.h>
#import <PGPclient/CKeyDBItem.h>
#import <PGPclient/pgpMem.h>

@implementation PGPkeysController

+ (NSString*)identifier
{
	return @"PGPkeys.bundle";
}

- (id)init
{
	if( !mInited )
	{
		[super init];
		[self registerForPrefNotifications];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kPGPkeysNib];
	if( !mPGPContext )
	{
		mPGPContext = [mController pgpContext];
		mPrefRef = [mController pgpPrefRef];
		mPGPKeysPreferences = [[PGPkeysPreferences alloc] initWithController:self];
		mMainDBPGPkeysView = [[CPGPkeysView alloc] initWithController:self mainDB:YES search:NO];
		[[mMainDBPGPkeysView view] setFrameSize:[mMainDBView frame].size];
		[mMainDBView addSubview:[mMainDBPGPkeysView view]];
		mCKeyrings = [[NSMutableArray alloc] init];
	}
	
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
    
    [mKeysMenu setTitle:kPGPkeysKeysMenuTitle];
    [mServerMenu setTitle:kPGPkeysServerMenuTitle];
    
	[mWindow setDelegate:self];
	[mWindow setFrameUsingName:kPGPkeysSaveFrameName];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller
{
    mController = controller;
    
	[self init];
	
	[mWindow setInitialFirstResponder:[mMainDBPGPkeysView outlineView]];
	[mWindow makeFirstResponder:[mMainDBPGPkeysView outlineView]];
	
	return self;
}

- (void)registerForPrefNotifications
{
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];
	
	[center addObserver:self
			selector:@selector(serversDidChange:)
			name:kPGPclPrefNotification_KeyServerList
			object:nil];
}

- (BOOL)releaseIfReadyToShutdown
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[mWindow saveFrameUsingName:kPGPkeysSaveFrameName];
	
	[self closeAuxWindows];
	[mCKeyrings release];
	[mMainDBPGPkeysView release];
	PGPLog(( @"Deallocated PGPkeys" ));	
	return YES;
}

- (void)keyDBChanged
{
	PGPLog((@"PGPkeys: keyDBChanged"));
	[mMainDBPGPkeysView reload];
}

- (void)passphraseCacheChanged
{
	PGPLog((@"PGPkeys: passphraseCacheChanged"));
}

- (void)dealloc
{
	[super dealloc];
}

- (id)controller
{
	return mController;
}

- (void)setController:(id)controller
{
	if( mController )
		mController = controller;
}

- (id)prefsController
{
	return mPGPKeysPreferences;
}

- (PGPContextRef)pgpContext
{
	return mPGPContext;
}

- (PGPKeyDBRef)mainDB
{
	return [mController mainKeyDB];
}

- (PGPtlsContextRef)tlsContext
{
	return [mController tlsContext];
}

- (PGPPrefRef)pgpPrefRef
{
	return mPrefRef;
}

- (NSWindow*)window
{
	return mWindow;
}

- (NSImage*)menuImage
{
	NSImage*	image;
	
	image = [[[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPkeysToolbarItemImage]] autorelease];
	[image setScalesWhenResized:YES];
	[image setSize:NSMakeSize(16, 16)];
	
	return image;
}

- (NSMenu*)dockMenu
{
	return nil;
}

- (NSMenuItem*)newMenuItem
{
	NSMenuItem*		item;
	
	item = [[NSMenuItem alloc] initWithTitle:kPGPkeysNewKeyring action:@selector(newKeyring:) keyEquivalent:@"n"];
	[item setKeyEquivalentModifierMask:(NSCommandKeyMask | NSShiftKeyMask)];
	[item setTarget:self];
	[item autorelease];
	
	return item;
}

- (void)activate:(BOOL)passive
{
	[self updateKeyrings];
	
	if (!passive)
	{
		[mMainDBPGPkeysView setActive:YES];
		[mWindow makeKeyAndOrderFront:nil];
	}
}

- (void)deactivate
{
	[mMainDBPGPkeysView setActive:NO];
}

- (void)addMenus
{
	[mController addMenu:mKeysMenu withIdentifier:[[self class] identifier] atIndex:[[mController menuItems] count] - 2];
	[mController addMenu:mServerMenu withIdentifier:[[self class] identifier] atIndex:[[mController menuItems] count] - 2];
	
	[self updateServerMenu];
}

- (void)removeMenus
{
}

- (void)newKeyring:(id)sender
{
	CKeyring	*altKeyring;

	altKeyring = [[CKeyring alloc] initWithController:self];
	[mCKeyrings addObject:altKeyring];
	[altKeyring loadKeyring:nil];
	[[altKeyring window] setInitialFirstResponder:[[altKeyring pgpKeysView] outlineView]];
	[[altKeyring window] makeFirstResponder:[[altKeyring pgpKeysView] outlineView]];
	[altKeyring release];
}

- (void)handleDocument:(NSString*)docFile ofType:(NSString*)docType
{
	CKeyring	*altKeyring;
	
	[self updateKeyrings];
	if(	[docType isEqualToString:kPGPPubKeyringFileType] ||
		[docType isEqualToString:kPGPSecKeyringFileType] )
	{
		altKeyring = [[CKeyring alloc] initWithController:self];
		[mCKeyrings addObject:altKeyring];
		[altKeyring loadKeyring:docFile];
		[altKeyring release];
	}
	else if( [docType isEqualToString:kPGPKeyShareFileType] )
	{
		if( mCSendShares )
			[[mCSendShares window] makeKeyAndOrderFront:self];
		else
		{
			mCSendShares = [[CSendShares alloc] initWithController:self shareFile:docFile];
			[mCSendShares run];
		}
	}
	else if(	[docType isEqualToString:kPGPExportedKeyFileType] ||
				[docType isEqualToString:kPGPCertificateFileType] )
	{
		[mMainDBPGPkeysView importDocument:docFile];
	}
	
	[mController finishedHandlingDocument:docFile];
}

- (void)toggleColumnSelection:(id)sender
{
	CPGPkeysView	*frontKeyView = [self frontKeyView];
	
	if( frontKeyView )
		[frontKeyView toggleColumn:[sender tag] - kPGPkeysCommand_ViewValidity + 1];
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	CPGPkeysView	*frontKeyView = [self frontKeyView];
	
	if ([[menuItem title] isEqualToString:kPGPkeysNewKeyring])
		return YES;
	if( !frontKeyView || ![[frontKeyView window] isKeyWindow] )
		return NO;
	return [frontKeyView validateCommand:[menuItem tag] item:menuItem];
}

- (void)keyringClosed:(CKeyring *)keyring
{
	[mCKeyrings removeObject:keyring];
}

- (void)keySearchClosed
{
	[mCKeySearch release];
	mCKeySearch = nil;
}

- (void)sendSharesClosed
{
	[mCSendShares release];
	mCSendShares = nil;
}

- (void)endKeySign:(id)keysView success:(BOOL)signedKeys
{
	[mCKeySign release];
	mCKeySign = nil;
	if( signedKeys )
		[keysView reload];
}

- (void)endKeyGen:(id)keysView success:(BOOL)generated
{
	[mCKeyGen release];
	mCKeyGen = nil;
	if( generated )
		[keysView reload];
}

- (void)endKeySplit:(id)keysView success:(BOOL)split
{
	[mCKeySplit release];
	mCKeySplit = nil;
	if( split )
	{
		[keysView reload];
		NSBeginInformationalAlertSheet( LSPK( @"Split Successful", @"Split Successful" ),
			kPGPkeysOKButton, nil, nil, [keysView window], 
			self, NULL, NULL, NULL,
			LSPK( @"The key has been successfully split.", @"Split Successful msg" ) );
	}
}

- (void)sign:(CPGPkeysView *)keysView forArray:(NSMutableArray *)userIDArray
{
	if( mCKeySign )
		[[mCKeySign window] makeKeyAndOrderFront:self];
	else
	{
		mCKeySign = [[CKeySign alloc] initWithController:self keysView:keysView
			forArray:userIDArray forKeyDB:[keysView keyDB]
			defaultSigner:[mMainDBPGPkeysView lookupDefaultKey]];
	}
}

- (void)split:(id)keysView forKey:(PGPKeyDBObjRef)splitKey
{
	if( mCKeySplit )
		[[mCKeySplit window] makeKeyAndOrderFront:self];
	else
	{
		mCKeySplit = [[CKeySplit alloc] initWithController:self keysView:keysView splitKey:splitKey];
	}
}

- (void)newKey:(CPGPkeysView *)keysView
{
	if( mCKeyGen )
		[[mCKeyGen window] makeKeyAndOrderFront:self];
	else
		mCKeyGen = [[CKeyGen alloc] initWithController:self keysView:keysView forKeyDB:[keysView keyDB]];
}

- (void)search:(CPGPkeysView *)keysView
{
	if( mCKeySearch )
		[[mCKeySearch window] makeKeyAndOrderFront:self];
	else
		mCKeySearch = [[CKeySearch alloc] initWithController:self];
}

- (CPGPkeysView *)frontKeyView
{
	CPGPkeysView	*frontView = nil;
	
	if( mMainDBPGPkeysView && [[mMainDBPGPkeysView window] isKeyWindow] )
		frontView = mMainDBPGPkeysView;
	else if( mCKeySearch && [[mCKeySearch window] isKeyWindow] )
		frontView = [mCKeySearch pgpKeysView];
	else if( [mCKeyrings count] )
	{
		int			krIndex,
					krCount = [mCKeyrings count];
		CKeyring *	kr;
		
		for( krIndex = 0; krIndex < krCount; krIndex++ )
		{
			kr = [mCKeyrings objectAtIndex:krIndex];
			if( [[kr window] isKeyWindow] )
			{
				frontView = [kr pgpKeysView];
				break;
			}
		}
	}
	return frontView;
}

- (void)closeAuxWindows
{
	if( mCSendShares )
		[[mCSendShares window] performClose:self];
	if( mCKeySplit )
		[[mCKeySplit window] performClose:self];
	if( mCKeySign )
		[[mCKeySign window] performClose:self];
	if( mCKeyGen )
		[[mCKeyGen window] performClose:self];
	if( mCKeySearch )
		[[mCKeySearch window] performClose:self];
	while( [mCKeyrings count] > 0 )
		[[[mCKeyrings objectAtIndex:0] window] performClose:self];
}

- (void)keysMenuAction:(id)sender
{
	CPGPkeysView	*frontView = [self frontKeyView];
	
	if( !frontView || ![[frontView window] isKeyWindow] )
		return;
	return [frontView keysMenuAction:sender];
}

- (void)mainDBInvalidated
{
	[mMainDBPGPkeysView emptyKeys];
}

- (BOOL)updateKeyrings
{
	BOOL	keyGen = NO;
	
	if( PGPKeyDBRefIsValid( [mController mainKeyDB] ) && ( [mController mainKeyDB] != [mMainDBPGPkeysView keyDB] ) )
	{
		PGPBoolean	generatedKey;
		
		[self closeAuxWindows];
		PGPLog((@"PGPkeys: updateKeyrings"));
		[mMainDBPGPkeysView loadKeys:[mController mainKeyDB] showKeySet:NULL];
		PGPGetPrefBoolean( mPrefRef, kPGPPrefFirstKeyGenerated, &generatedKey );
		if( PGPKeyDBRefIsValid( [mController mainKeyDB] ) && !generatedKey )
		{
			PGPKeyIterRef	keyIter;
			PGPKeyDBObjRef	key;
			PGPBoolean		secret = FALSE;

			PGPLog((@"\tkPGPPrefFirstKeyGenerated = FALSE"));
			PGPSetPrefBoolean( mPrefRef, kPGPPrefFirstKeyGenerated, TRUE );
			if( IsntPGPError( PGPNewKeyIterFromKeyDB( [mController mainKeyDB], &keyIter ) ) )
			{
				while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_Key, &key ) ) )
				{
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsSecret, &secret );
					if( secret )
						break;
				}
				PGPFreeKeyIter( keyIter );
			}
			if( !secret )
			{
				[self performSelector:@selector(newKey:) withObject:mMainDBPGPkeysView afterDelay:0.0];
				keyGen = YES;
			}
		}
	}
	return keyGen;
}

- (void)updateServerMenu
{
	NSMenuItem				*topServerItem = [[NSApp mainMenu] itemWithTitle:kPGPkeysServerMenuTitle];
	NSMenu					*serverMenu;
	PGPKeyServerEntry		*servers = NULL;
	PGPUInt32				numServers;
	
	if( !topServerItem )
		return;
	serverMenu	= [[[topServerItem submenu] itemWithTitle:LSPK(@"Send To", @"")] submenu];
	if( serverMenu )
	{
		PGPInt32	numItems = [serverMenu numberOfItems],
					serverIndex;
		
		while( numItems > 3 )
			[serverMenu removeItemAtIndex:--numItems];
		if( IsntPGPError( PGPGetKeyServerPrefs( mPrefRef, &servers, &numServers ) ) )
		{
			char	url[kMaxServerNameLength + 1];
			
			for( serverIndex = 0; serverIndex < numServers; serverIndex++ )
			{
				if( IsKeyServerListed( servers[serverIndex].flags ) )
				{
					NSMenuItem	*newItem;
					
					PGPGetKeyServerURL( &servers[serverIndex], url );
					newItem = [serverMenu addItemWithTitle:[NSString stringWithCString:url]
								action:@selector(keysMenuAction:)
								keyEquivalent:@""];
					[newItem setTarget:self];
					[newItem setTag:kPGPkeysCommand_SendToThisServer];
					[newItem setRepresentedObject:[NSMutableData dataWithBytes:&servers[serverIndex]
						length:sizeof(PGPKeyServerEntry)]];
				}
			}
		}
	}
	if( IsntNull( servers ) )
		PGPFreeKeyServerList( servers );
}

- (void)serversDidChange:(NSNotification*)notification
{
	if( mCKeySearch )
		[mCKeySearch serversUpdated];
	[self updateServerMenu];
}

- (void)windowDidBecomeKey:(NSNotification*)notification
{
	[mController setCurrentBundle:self passive:YES];
}

- (BOOL)windowShouldClose:(id)sender
{
	[mMainDBPGPkeysView setActive:NO];
	
	return YES;
}

@end

