/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPkeysPreferences.h,v 1.16 2002/09/07 09:56:31 wprice Exp $
____________________________________________________________________________*/

#import <PGP/pgpDebug.h>
#import <PGP/pgpPubTypes.h>
#import <PGP/pgpKeyServer.h>
#import <PGPclient/pgpKeyServerPrefs.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPui/CBrowserCell.h>
#import <PGPclient/PGPOpenPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/CIntegerFormatter.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <Cocoa/Cocoa.h>
#import <PGPui/CPGPIcons.h>

#import "PGPPrefsController.h"
#import "PGPkeysController.h"

#define kPGPkeysPreferencesNib				@"PGPkeysPreferences"

#define kPGPkeysPrefsFilesIdentifier		LSPK(@"Files", @"Files Pref Panel")
#define kPGPkeysPrefsServersIdentifier		LSPK(@"Server", @"Server Pref Panel")
#define kPGPkeysPrefsCAIdentifier			LSPK(@"CA", @"CA Pref Panel")
#define kPGPkeysPrefsServerColumnIdentifier	@"Server"
#define kPGPkeysPrefsDomainColumnIdentifier @"Domain"
#define kPGPkeysPrefsListedColumnIdentifier @"Listed"
#define kPGPkeysAnyDomainCellIcon			@"anydomain.tiff"
#define kPGPkeysServerEditModeNone			1000
#define kPGPkeysServerEditModeNew			1001
#define kPGPkeysServerEditModeEdit			1002

enum
{
	kPGPkeysServerType_Invalid = 0,
	
	kPGPkeysServerType_PGPKeyserverHTTP = 3000,
	kPGPkeysServerType_PGPKeyserverLDAP,
	kPGPkeysServerType_PGPKeyserverLDAPS,
	kPGPkeysServerType_X509DirectoryLDAP,
	kPGPkeysServerType_X509DirectoryLDAPS
};

@class NSDictionary, NSMutableDictionary;

@interface PGPkeysPreferences : NSObject <PGPPrefsController>
{
	PGPkeysController*		mController;
	NSArray*				mIdentifiers;
	NSMutableArray*			mKeyServerEntries;
	NSImage*				mAnyDomainIcon;
	CIntegerFormatter*		mIntFormatter;
	PGPUInt32				mServerEditMode;
	PGPKeyDBObjRef			mX509AuthCert;
	NSString				*mLastDirectory;

	/* Files preferences panel */
	IBOutlet NSView*		mFilesPrefView;
	IBOutlet NSTextField*	mPubKeyringLocation;
	IBOutlet NSButton*		mChangePubKeyring;
	IBOutlet NSTextField*	mSecKeyringLocation;
	IBOutlet NSButton*		mChangeSecKeyring;

	/* Servers preferences panel */
	IBOutlet NSView*		mServersPrefView;
	IBOutlet NSTableView*	mServersTableView;
	IBOutlet NSButton*		mNewServer;
	IBOutlet NSButton*		mRemoveServer;
	IBOutlet NSButton*		mEditServer;
	IBOutlet NSButton*		mSetAsRoot;
	IBOutlet NSButton*		mMoveUp;
	IBOutlet NSButton*		mMoveDown;
	IBOutlet NSButton*		mEncryptToUnknown;
	IBOutlet NSButton*		mSigningKeys;
	IBOutlet NSButton*		mAddingNamesPhotosRevokers;
	IBOutlet NSButton*		mRevocation;
	IBOutlet NSButton*		mVerification;
	
	/* Server edit window */
	IBOutlet NSWindow*		mServerEditWindow;
	IBOutlet NSPopUpButton*	mServerType;
	IBOutlet NSTextField*	mServerDNS;
	IBOutlet NSTextField*	mServerPort;
	IBOutlet NSTextField*	mAuthKeyID;
	IBOutlet NSButton*		mAnyDomain;
	IBOutlet NSButton*		mSpecificDomain;
	IBOutlet NSTextField*	mDomain;
	IBOutlet NSButton*		mListInSearchWindow;

	/* CA preferences panel */
	IBOutlet NSView*		mCAPrefView;
	IBOutlet NSTextField*	mURL;
	IBOutlet NSTextField*	mRevocationURL;
	IBOutlet NSPopUpButton*	mCAType;
	IBOutlet NSImageView*	mCertIcon;
	IBOutlet NSTextField*	mCertName;
	IBOutlet NSButton*		mSelectCert;
	IBOutlet NSButton*		mClearCert;

	BOOL					mInited;
	BOOL					mLoadedNib;
}

- (id)init;
- (id)initWithNib:(NSString*)nibFile;
- (id)initWithController:(id)controller;

- (id)controller;
- (NSArray*)identifiers;
- (id)prefsViewForIdentifier:(NSString*)identifier;

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier;
- (id)lastPrefsResponderForIdentifier:(NSString*)identifier;
- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier;

- (void)restorePanelPrefs:(NSString*)panelIdentifier;
- (void)restorePanelDefaults:(NSString*)panelIdentifier;
- (void)savePanelPrefs:(NSString*)panelIdentifier;

- (void)setPubKeyringLocation:(const char*)value enabled:(BOOL)enabled;
- (void)setSecKeyringLocation:(const char*)value enabled:(BOOL)enabled;
- (void)setSyncOnUnknownKeys:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setSyncOnKeySign:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setSyncOnAdd:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setSyncOnRevocation:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setSyncOnVerify:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setServerURL:(const char*)value enabled:(BOOL)enabled;
- (void)setRevocationServerURL:(const char*)value enabled:(BOOL)enabled;
- (void)setCAType:(PGPUInt32)value enabled:(BOOL)enabled;
- (void)setRootCertificate:(PGPKeyDBObjRef)cert enabled:(BOOL)enabled;

- (const char*)getPubKeyringLocation;
- (const char*)getSecKeyringLocation;
- (PGPBoolean)getSyncOnUnknownKeys;
- (PGPBoolean)getSyncOnKeySign;
- (PGPBoolean)getSyncOnAdd;
- (PGPBoolean)getSyncOnRevocation;
- (PGPBoolean)getSyncOnVerify;
- (const char*)getServerURL;
- (const char*)getRevocationServerURL;
- (PGPUInt32)getCAType;

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled;
- (PGPBoolean)getCheckbox:(NSButton*)checkBox;

- (IBAction)changePubKeyring:(id)sender;
- (IBAction)changeSecKeyring:(id)sender;
- (IBAction)newServer:(id)sender;
- (IBAction)removeServer:(id)sender;
- (IBAction)editServer:(id)sender;
- (IBAction)setServerAsRoot:(id)sender;
- (IBAction)moveServerUp:(id)sender;
- (IBAction)moveServerDown:(id)sender;
- (IBAction)anyDomain:(id)sender;
- (IBAction)specificDomain:(id)sender;
- (IBAction)serverEditOK:(id)sender;
- (IBAction)serverEditCancel:(id)sender;
- (IBAction)selectCert:(id)sender;
- (IBAction)clearCert:(id)sender;

- (void)remove:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)certSelectDidEnd:(PGPError)resultCode selectedCert:(PGPKeyDBObjRef)cert;

- (void)updateServerButtons;
- (BOOL)validateServerEditWindow;
- (void)setupServerEditWindow:(PGPKeyServerEntry*)entry;
- (PGPError)saveServerEditEntry:(PGPKeyServerEntry*)entry;

- (void)tableViewSelectionDidChange:(NSNotification*)notification;
- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row;
- (int)numberOfRowsInTableView:(NSTableView*)tableView;
- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row;

- (void)dealloc;

@end
