/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPVerificationDataSource.m,v 1.10 2002/11/11 05:26:31 wprice Exp $
____________________________________________________________________________*/

#import "CPGPVerificationDataSource.h"
#import "CPGPVerification.h"

#define kPGPmailFileNameColumnIdentifier	@"mFileName"
#define kPGPmailSignedColumnIdentifier		@"mSigned"
#define kPGPmailVerifiedColumnIdentifier	@"mVerified"

@implementation CPGPVerificationDataSource

- (id)init
{
    self = [super init];
    [self setVerifications: [NSMutableArray array]];
    return( self );
}

- (void)dealloc
{
    [mVerifications release];
    [super dealloc];
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    return( [mVerifications count] );
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)rowIndex
{
    CPGPVerification 	*theVerification = nil;
    id					theValue = nil;
    
    NSParameterAssert( rowIndex >= 0 && rowIndex < [mVerifications count] );
    
    theVerification = [mVerifications objectAtIndex:rowIndex];
    
    if( [[tableColumn identifier] isEqualToString:kPGPmailSignedColumnIdentifier] ||
        [[tableColumn identifier] isEqualToString:kPGPmailVerifiedColumnIdentifier] )
    {
        NSUserDefaults	*defaults = [NSUserDefaults standardUserDefaults];
        NSCalendarDate	*date = [theVerification valueForKey: [tableColumn identifier]];
        
		theValue = [date descriptionWithCalendarFormat:[NSString stringWithFormat:@"%@ %@%%p",
					[defaults stringForKey:NSShortDateFormatString],
					[defaults stringForKey:NSTimeFormatString]]
					locale:[defaults dictionaryRepresentation]];
    }
    else
    {
        theValue = [theVerification valueForKey:[tableColumn identifier]];
    }
    
    return(  theValue );
}

- (NSMutableArray *)verifications
{
    return( mVerifications );
}

- (void)setVerifications:(NSMutableArray *)newVerifications
{
    [mVerifications autorelease];
    mVerifications = [newVerifications retain];
}

- (void)addRow:(id)rowData
{
    [mVerifications addObject:rowData];
}

- (void)tableView:(NSTableView*)tableView willDisplayCell:(id)tableCell forTableColumn:(NSTableColumn*)tableColumn row:(int)rowIndex
{
    if( [[tableColumn identifier] isEqualToString:kPGPmailFileNameColumnIdentifier] )
    {
        NSImage				*image;
        CPGPVerification 	*theVerification = [mVerifications objectAtIndex:rowIndex];

        if( [theVerification isSigValid] )
            image = [[NSImage alloc] initWithContentsOfFile:
                [[NSBundle bundleForClass:[self class]] pathForImageResource:@"pencil.tiff"]];
        else
            image = [[NSImage alloc] initWithContentsOfFile:
                [[NSBundle bundleForClass:[self class]] pathForImageResource:@"nopencil.tiff"]];

        [tableCell setImage:image];
        [image release];
    }
}

@end

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/