/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPmailButton.m,v 1.4 2002/10/16 04:44:09 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPmailButton.h"

@implementation CPGPmailButton : NSButton

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	[[self cell] setShowsStateBy:NSContentsCellMask];
	[[self cell] setHighlightsBy:NSContentsCellMask];
	
	return self;
}

- (void)setSupportedFileExtensions:(NSArray*)fileExtensions
{
	if (mSupportedFileExtensions)
	{
		[mSupportedFileExtensions release];
		mSupportedFileExtensions = nil;
	}
	
	mSupportedFileExtensions = [[NSMutableArray arrayWithArray:fileExtensions] retain];
}

- (NSArray*)supportedFileExtensions
{
	return mSupportedFileExtensions;
}

- (void)setSupportedFileTypes:(NSArray*)fileTypes
{
	if (mSupportedFileTypes)
	{
		[mSupportedFileTypes release];
		mSupportedFileTypes = nil;
	}
	
	mSupportedFileTypes = [[NSMutableArray arrayWithArray:fileTypes] retain];
}

- (NSArray*)supportedFileTypes
{
	return mSupportedFileTypes;
}

- (void)setDragSelector:(SEL)selector
{
	mDragSelector = selector;
}

- (SEL)dragSelector
{
	return mDragSelector;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
	[[self cell] setHighlighted:NO];
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
	NSPasteboard*		pboard = [sender draggingPasteboard];
	NSEnumerator*		enumerator;
	NSEnumerator*		typeEnumerator;
	NSEnumerator*		extEnumerator;
	NSString*			type;
	NSString*			plist;
	NSString*			filename;
	NSString*			fileType;
	NSString*			fileExt;
	NSString*			hfsFileType;
	NSArray*			filenames;
	
	/*
	We want to prevent any local drag & drop (from the keyview for example).
	If draggingSource is != nil the drag source is PGP.app.
	*/
	if ([sender draggingSource])
		return NSDragOperationNone;
	
    type = [pboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]];
	
	if (!type)
		return NSDragOperationNone;
	else if ([type isEqualToString:NSFilenamesPboardType])
	{
		if (![mSupportedFileTypes count] && ![mSupportedFileExtensions count])
		{
			[[self cell] setHighlighted:YES];
			return NSDragOperationGeneric;
		}
		else
		{
			plist = [[[NSString alloc] initWithData:[pboard dataForType:type] encoding:NSUTF8StringEncoding] autorelease];
			
			if (filenames = [plist propertyList])
			{
				enumerator = [filenames objectEnumerator];
				
				while (filename = [enumerator nextObject])
				{
					if ([mSupportedFileExtensions count])
					{
						extEnumerator = [mSupportedFileExtensions objectEnumerator];
						
						while (fileExt = [extEnumerator nextObject])
						{
							if ([[filename pathExtension] isEqualToString:fileExt])
							{
								[[self cell] setHighlighted:YES];
								return NSDragOperationGeneric;
							}
						}
					}
					
					if ([mSupportedFileTypes count])
					{
						typeEnumerator = [mSupportedFileTypes objectEnumerator];
						hfsFileType = NSHFSTypeOfFile(filename);
						
						while (fileType = [typeEnumerator nextObject])
						{
							if ([hfsFileType isEqualToString:fileType])
							{
								[[self cell] setHighlighted:YES];
								return NSDragOperationGeneric;
							}
						}
					}
				}
			}
		}
		
		return NSDragOperationNone;
	}
	else
		return NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
	[[self cell] setHighlighted:NO];
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard*		pboard = [sender draggingPasteboard];
	NSEnumerator*		enumerator;
	NSEnumerator*		typeEnumerator;
	NSEnumerator*		extEnumerator;
	NSString*			type;
	NSString*			plist;
	NSString*			filename;
	NSString*			fileType;
	NSString*			fileExt;
	NSString*			hfsFileType;
	NSArray*			filenames;
	
    type = [pboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]];
	
	if (!type)
		return NO;
	else if ([type isEqualToString:NSFilenamesPboardType])
	{
		if (![mSupportedFileTypes count] && ![mSupportedFileExtensions count])
			return YES;
		else
		{
			plist = [[[NSString alloc] initWithData:[pboard dataForType:type] encoding:NSUTF8StringEncoding] autorelease];
			
			if (filenames = [plist propertyList])
			{
				enumerator = [filenames objectEnumerator];
				
				while (filename = [enumerator nextObject])
				{
					if ([mSupportedFileExtensions count])
					{
						extEnumerator = [mSupportedFileExtensions objectEnumerator];
						
						while (fileExt = [extEnumerator nextObject])
						{
							if ([[filename pathExtension] isEqualToString:fileExt])
								return YES;
						}
					}
					
					if ([mSupportedFileTypes count])
					{
						typeEnumerator = [mSupportedFileTypes objectEnumerator];
						hfsFileType = NSHFSTypeOfFile(filename);
						
						while (fileType = [typeEnumerator nextObject])
						{
							if ([hfsFileType isEqualToString:fileType])
								return YES;
						}
					}
				}
			}
		}
		
		return NSDragOperationNone;
	}
	else
		return NSDragOperationNone;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard*		pboard = [sender draggingPasteboard];
	NSString*			type;
	NSString*			plist;
	NSArray*			filenames;
	
    type = [pboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]];
	
	if (!type)
		return NO;
	else if ([type isEqualToString:NSFilenamesPboardType])
	{
		plist = [[[NSString alloc] initWithData:[pboard dataForType:type] encoding:NSUTF8StringEncoding] autorelease];
		
		if (filenames = [plist propertyList])
		{
			[[self target] performSelector:mDragSelector withObject:filenames];
			return YES;
		}
	}
	
	return NO;
}

- (void)awakeFromNib
{
	[self setBordered:YES];
	[self setEnabled:YES];
	[self setButtonType:NSMomentaryPushButton];
	[self setBezelStyle:NSRegularSquareBezelStyle];
	[self setImagePosition:NSImageOnly];
	[self setTitle:@""];
}

- (void)dealloc
{
	if (mSupportedFileTypes)
	{
		[mSupportedFileTypes release];
		mSupportedFileTypes = nil;
	}
	
	if (mSupportedFileExtensions)
	{
		[mSupportedFileExtensions release];
		mSupportedFileExtensions = nil;
	}
	
	[super dealloc];
}

@end
