/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPmailEncodeDecodeFile.h,v 1.24 2002/10/16 04:44:09 jeffc Exp $
____________________________________________________________________________*/
#ifndef Included_CPGPmailEncodeDecodeFile_h	/* [ */
#define Included_CPGPmailEncodeDecodeFile_h

#import <PGP/PGP.h>
#import <Cocoa/Cocoa.h>

#import "PGPmailCommon.h"
#import "CPGPmailController.h"
#import "CPGPmailProgress.h"

@class CPGPmailEncodeDecodeFile;

typedef struct ProcessingInfo
{
    NSString*   src;
    NSString*	resFork;
    NSString*	destFile;
    BOOL		isFolder;
    PGPUInt32   resSize;
    PGPUInt32   dataSize;
    NSString*	fileType;
    OSType		typeCode;
    OSType		creatorCode;
} ProcessingInfo;

typedef struct PGPmailEncodeDecodeParams
{
	PGPKeyDBRef					allKeys;
	PGPOptionListRef			encodeOptions;
	PGPBoolean					encrypting;
	PGPBoolean					conventionalEncrypt;
	PGPBoolean					signing;
	PGPBoolean					detachedSignature;
	PGPBoolean					textOutput;
	PGPBoolean					wipeOriginal;
	PGPBoolean					encryptingToSelf;
	PGPBoolean					signingWithSplitKey;
	PGPBoolean					fyeo;
	PGPBoolean					selfDecryptingArchive;
	PGPKeyDBObjRef				signingKey;
	PGPLocalEncodingFlags		encodingFlags;
	char						conventionalPassphrase[256];
	char						signingPassphrase[256];
	PGPByte*					signingPasskey;
	PGPSize						signingPasskeySize;
} PGPmailEncodeDecodeParams;

typedef struct PGPmailEventHandlerData
{
    CPGPmailEncodeDecodeFile*		mTask;
    PGPmailEncodeDecodeParams 		mParams;
    PGPBoolean						retryingSigVerification;
    PGPBoolean						isFYEO;
    void *							outputData;
    PGPSize							outputDataSize;
    PGPClientDecodeEventHandlerData	clientHandlerData;
} PGPmailEventHandlerData;

@interface CPGPmailEncodeDecodeFile : NSObject
{
    CPGPmailController*				mAppController;
	CPGPmailProgress*				mProgressDialog;
    
    PGPContextRef					mContext;
    PGPtlsContextRef				mTLSContext;
    PGPPrefRef						mPrefRef;
	PGPKeyDBRef						mAllKeys;
    PGPMemoryMgrRef					mMemoryMgr;
    PGPmailOperation				mOperation;
 	PGPUInt32						mBadPassphraseCount;
    unsigned int					mModifierFlags;
	PGPAnalyzeType					mDataType;
	PGPmailSignatureData			mSignatureData;
	PGPBoolean						mSignatureDataValid;
    PGPFileSpecRef					mOutputFile;
    
    struct
	{
		PGPBoolean			warnOnWipe;
		PGPCipherAlgorithm	convAlgorithm;		
		char				comment[ 256 ];
		PGPCipherAlgorithm	*allowedAlgorithms;
		PGPUInt32			numAllowedAlgorithms;
		
	} mTaskPrefs;
    
	NSMutableArray*		mWipeTasks;
    NSArray*			mTaskList;
    
	BOOL				mTaskListHasFileTasks;

    NSString*			mSource;				// Original target file or folder
	BOOL				mSourceIsFolder;
	PGPBoolean			mProcessInPlace;
	NSString*			mWorkingFile;			// Current child target (if any)
	NSString*			mDetachedSigInput;
    
    NSFileManager*		mFileManager;
    BOOL				mInited;
}

- (id)init;

- (id)initWithController:
    (CPGPmailController *)inController;

- (void)dealloc;

- (void)processFiles:(NSArray*)inFiles operation:(PGPmailOperation)inOperation lock:(NSLock*)lock;

- (void)stopProcessing;

@end

#endif /* ] Included_CPGPmailEncodeDecodeFile_h */

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/