/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPmailFSWiper.m,v 1.14 2002/10/22 06:59:35 jason Exp $
____________________________________________________________________________*/

#import <sys/param.h>
#import <sys/mount.h>
#import <unistd.h>
#import <PGP/PGP.h>
#import <pgpMem.h>
#import <PGPclient/pgpClientLibCocoa.h>

#import "CPGPmailFSWiper.h"

@interface CPGPmailFSWiper( PrivateMethods )

- (PGPError)wipeInternal:(NSFileHandle *)file
    length:(PGPUInt64)wipeLength
    passes:(PGPUInt32)numPasses
    patterns:(PGPUInt32)numPatterns
    list:(PGPWipingPattern *)patternList
    buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength;
								
- (PGPError)wipe:(NSFileHandle *)file
    passes:(PGPUInt32)numPasses
	patterns:(PGPUInt32)numPatterns
	list:(PGPWipingPattern *)patternList
	buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength;
					
- (PGPError)wipeFork:(NSString *)fileName
    fork:(PGPmailForkKind)whichFork
	passes:(PGPUInt32)numPasses
    patterns:(PGPUInt32)numPatterns
	list:(PGPWipingPattern *)patternList
	buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength;
				                                
- (BOOL)progress:(NSString *)fileName
    done:(PGPUInt64)additionalDone;
	
@end

@implementation CPGPmailFSWiper

- (id)init
{
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }
    
    if( !mController )
        PGPLog((@"%s:%d: Warning: mController has not been set.", __FILE__, __LINE__));

    mFileManager = [NSFileManager defaultManager];

	mWipeFile	= nil;
	mTotalDone	= 0;
    
    return self;
}

- (id)initWithController:(CPGPmailWipeFile *)controller
{
    mController = controller;
    
    return [self init];
}

- (void)dealloc
{
    [super dealloc];
}

- (PGPError)wipeInternal:(NSFileHandle *)file
    length:(PGPUInt64)wipeLength
    passes:(PGPUInt32)numPasses
    patterns:(PGPUInt32)numPatterns
    list:(PGPWipingPattern *)patternList
    buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength
{
	PGPError			theError = kPGPError_NoErr;

	pgpAssertAddrValid( patternList, PGPWipingPattern );
	pgpAssert( wipeBufferLength != 0 );
	pgpAssert( wipeBufferLength % sizeof( PGPWipingPattern ) == 0 );
	
	if( wipeLength != 0 )
	{
		PGPUInt32	passIndex = 0;
        BOOL		stopRequested = NO;
        
		[file synchronizeFile];
		
		for( passIndex = 0; IsntPGPError(theError) && (passIndex < numPasses); passIndex++ )
		{
			const PGPWipingPattern	*curPattern;
			PGPUInt64				bytesRemaining;
			
			curPattern = &patternList[ passIndex % numPatterns ];
			
			pgpCopyPattern( &curPattern->bytes[0], sizeof( *curPattern ),
					wipeBuffer, wipeBufferLength );
					
			[file seekToFileOffset:0];
		
			bytesRemaining = wipeLength;
			while( bytesRemaining != 0 )
			{
				long		count	= MIN( wipeBufferLength, bytesRemaining );
				NSData *	wipeData = [[NSData alloc] initWithBytesNoCopy:wipeBuffer length:count
											freeWhenDone:NO];
				
				[file writeData:wipeData];
				[wipeData release];
                [file synchronizeFile];

				bytesRemaining -= count;
				
                stopRequested = [self progress:mWipeFile done:count];
                
                if( stopRequested )
                {
                    theError = kPGPError_UserAbort;
                    break;
                }
			}
        }
	}
	
	[file synchronizeFile];

	return theError;
}
								
- (PGPError)wipe:(NSFileHandle *)file
    passes:(PGPUInt32)numPasses
	patterns:(PGPUInt32)numPatterns
	list:(PGPWipingPattern *)patternList
	buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength
{
    PGPError				theError = kPGPError_NoErr;
	struct statfs			systemStat;
    
	if( 0 == fstatfs( [file fileDescriptor], &systemStat) )
	{
        PGPUInt64	wipeLength;
        PGPUInt64	fileSize = [file seekToEndOfFile];
        UInt32		allocBlockSize = systemStat.f_bsize;
		
		// wipe length should round up to next allocation block
		wipeLength = ( fileSize + allocBlockSize - 1 ) / allocBlockSize * allocBlockSize;
        
		theError = [self wipeInternal:file length:[file seekToEndOfFile] passes:numPasses patterns:numPatterns
					list:patternList buffer:wipeBuffer size:wipeBufferLength];
	}
    
	return theError;
}
					
- (PGPError)wipeFork:(NSString *)fileName
    fork:(PGPmailForkKind)whichFork
	passes:(PGPUInt32)numPasses
    patterns:(PGPUInt32)numPatterns
	list:(PGPWipingPattern *)patternList
	buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength
{
	PGPError			theError = kPGPError_NoErr;
    NSMutableString*	file = [NSMutableString string];
    
    [file setString:fileName];
    
    if( whichFork == kPGPmailForkKindResource )
        [file appendString:kFileResourceFork];
    
    if( [mFileManager fileExistsAtPath:file] )
    {
        NSFileHandle*		fileRef = [NSFileHandle fileHandleForUpdatingAtPath:file];
        
        if( nil != fileRef )
        {
            theError = [self wipe:fileRef passes:numPasses patterns:numPatterns
                            list:patternList buffer:wipeBuffer size:wipeBufferLength];
    
            if( IsntPGPError( theError ) )
                [fileRef seekToFileOffset:0];
            
            [fileRef closeFile];
        }
	}
    
	return theError;
}
				
- (PGPError)wipeFile:(NSString *)file
	passes:(PGPUInt32)numPasses
    patterns:(PGPUInt32)numPatterns
	list:(PGPWipingPattern *)patternList
	buffer:(void *)wipeBuffer
    size:(PGPSize)wipeBufferLength
{
	PGPError			theError = kPGPError_NoErr;
        
    if( ![mFileManager isReadableFileAtPath:file] )
        // we cant read the file we cant do anyting with it.
        theError = kPGPError_FileLocked;
    else if( [mFileManager isWritableFileAtPath:file] && [mFileManager isDeletableFileAtPath:file] )
    {
        // if the file is not writeable we cant over write it.
        
        BOOL			allocatedBuffer = NO;

        if( IsNull( wipeBuffer ) || wipeBufferLength < sizeof( PGPWipingPattern ) )
        {
            UInt32	bufferSize = 512L * 1024L;
            
            wipeBuffer = PGPNewData( PGPPeekContextMemoryMgr([mController pgpContext]),
                                bufferSize, kPGPMemoryMgrFlags_None );
            if( IsntNull( wipeBuffer ) )
            {
                allocatedBuffer 	= YES;
                wipeBufferLength	= bufferSize;
            }
            else
                theError = kPGPError_OutOfMemory;
        }
        
        if( IsntPGPError( theError ) )
        {
            //FInfo	fileInfo;
            
            // The wiping buffer must be a multiple of the pattern size
            
            wipeBufferLength = ( wipeBufferLength / 
                                    sizeof( PGPWipingPattern ) ) *
                                    sizeof( PGPWipingPattern );

            theError = [self wipeFork:file fork:kPGPmailForkKindData passes:numPasses
                            patterns:numPatterns list:patternList buffer:wipeBuffer
                            size:wipeBufferLength];
                            
            if( IsntPGPError( theError ) )
                theError = [self wipeFork:file fork:kPGPmailForkKindResource passes:numPasses
                            patterns:numPatterns list:patternList buffer:wipeBuffer
                            size:wipeBufferLength];
        }
        
        if( allocatedBuffer )
            PGPFreeData( wipeBuffer );
    }
    else
        theError = kPGPError_FilePermissions;
        
	return theError;
}
                                
- (BOOL)progress:(NSString *)file
    done:(PGPUInt64)additionalDone
{
    mTotalDone	+= additionalDone;
    
    return [mController progress:additionalDone];
}
	
- (PGPError)wipeAndDelete:(NSString *)file
	passes:(PGPUInt32)numPasses
    patterns:(PGPUInt32)numPatterns
    list:(PGPWipingPattern *)patternList
	buffer:(void *)wipeBuffer
	size:(PGPSize)wipeBufferLength
{
	PGPError		theError = kPGPError_NoErr; 
    
	mTotalDone		= 0;
	mWipeFile		= file;
	
	// wipe the data
	theError	= [self wipeFile:file passes:numPasses patterns:numPatterns list:patternList
                    buffer:wipeBuffer size:wipeBufferLength];
	if( IsntPGPError( theError ) )
	{
        sync();
		if( IsntPGPError( theError ) )
		{
			unsigned			fileNameLength = [[file lastPathComponent] length];
			NSMutableString *	bogusName = [NSMutableString string];
            NSString *			tempFile = nil;
			int					x;
            
			for (x = 0; x < fileNameLength; x++) {
				[bogusName appendString:@"x"];
			}
            
            tempFile = [[file stringByDeletingLastPathComponent]
                           stringByAppendingPathComponent:bogusName];
            
            if( [mFileManager movePath:file toPath:tempFile handler:nil] )
				sync();
                
            theError = [mFileManager removeFileAtPath:tempFile handler:nil] ?
                    kPGPError_NoErr : kPGPError_FileOpFailed ;
 		}
	}
	
	// caller is responsible for flushing volume
	
	return theError;
}

@end
