/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPmailProgress.m,v 1.16 2002/10/22 09:25:50 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPmailProgress.h"
#import "CPGPmailController.h"
#import <PGPclient/pgpClientLibCocoa.h>

#define kPGPmailProgressNib			@"CPGPmailProgress"	

static CPGPmailProgress*	sharedProgressObject = nil;

@implementation CPGPmailProgressJob

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mLock = [[NSLock alloc] init];
		
		mInited = YES;
	}
	
	return self;
}

- (id)initWithFile:(NSString*)file operation:(PGPmailOperation)operation selector:(SEL)selector target:(id)target
{
	[self init];
	[self setFile:file];
	[self setOperation:operation];
	[self setSelector:selector];
	[self setTarget:target];
	
	return self;
}

- (void)lock
{
	[mLock lock];
}

- (void)unlock
{
	[mLock unlock];
}

- (NSString*)file
{
	return mFile;
}

- (void)setFile:(NSString*)file
{
	[mFile release];
	mFile = nil;
	
	if (file)
		mFile = [file retain];
}

- (PGPmailOperation)operation
{
	return mOperation;
}

- (void)setOperation:(PGPmailOperation)operation
{
	mOperation = operation;
}

- (SEL)selector
{
	return mSelector;
}

- (void)setSelector:(SEL)selector
{
	mSelector = selector;
}

- (id)target
{
	return mTarget;
}

- (void)setTarget:(id)target
{
	[mTarget release];
	mTarget = nil;
	
	if (target)
		mTarget = [target retain];
}

- (void)dealloc
{
	[mLock release];
	mLock = nil;
	
	[self setFile:nil];
	[self setSelector:nil];
	[self setTarget:nil];
	
	[super dealloc];
}

@end

@implementation CPGPmailProgress

+ (id)sharedInstance
{    
	if( !sharedProgressObject )
		sharedProgressObject = [[CPGPmailProgress allocWithZone:[self zone]] init];
		
	return sharedProgressObject;
}

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mPendingJobs = [[NSMutableArray alloc] init];
		mLock = [[NSLock alloc] init];
		
		mInited = YES;
	}
    
	if (!mLoadedNib)
		[self loadNib:kPGPmailProgressNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	NSBundle*	bundle = [NSBundle bundleForClass:[self class]];
	
	[NSBundle loadNibNamed:nibFile owner:self];
	
	mEncryptImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"encrypt.icns"]];
	mSignImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"sign.icns"]];
	mEncryptSignImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"signandencrypt.icns"]];
	mDecryptVerifyImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"decrypt.icns"]];
	mWipeImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"wipe.icns"]];
	mCollapsed = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"collapsed.tiff"]];
	mExpanded = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"expanded.tiff"]];
	
	mPendingView = [[NSView alloc] initWithFrame:NSZeroRect];
	
	[mPendingButton setAction:@selector(expandCollapsePending:)];
	[mPendingButton setTarget:self];
	[mPendingButton retain];
	[mPendingButton removeFromSuperview];
	
	[mPendingOpsText retain];
	[mPendingOpsText removeFromSuperview];
	
	[mProgressWindow setHidesOnDeactivate:NO];
	[mProgressWindow setLevel:NSNormalWindowLevel];
	[mProgressWindow setExcludedFromWindowsMenu:YES];
	[mProgressIndicator setUsesThreadedAnimation:YES];
	
	mLoadedNib = YES;
}

- (NSString*)fileName
{
    return [mFileNameText stringValue];
}

- (void)setFileName:(NSString*)fileName
{
	[mLock lock];
	
    [mFileNameText performSelectorOnMainThread:@selector(setStringValue:) withObject:fileName waitUntilDone:NO];
	
	[mLock unlock];
}

- (NSString*)message
{
    return [mMessageText stringValue];
}

- (void)setMessage:(NSString*)message
{
	[mLock lock];
	
    [mMessageText performSelectorOnMainThread:@selector(setStringValue:) withObject:message waitUntilDone:NO];
	
	[mLock unlock];
}

- (NSView*)pendingView
{
	return mPendingView;
}

- (void)setPendingView:(NSView*)pendingView
{
	[mPendingView removeFromSuperview];
	[mPendingView release];
	
	mPendingView = nil;
	
	if (pendingView)
		mPendingView = [pendingView retain];
}

- (BOOL)jobsPending
{
	return [mPendingJobs count] ? YES : NO;
}

- (void)startProgressForJob:(CPGPmailProgressJob*)job
{
	if (!job)
		return;
	
	[job retain];
	
	[self addPendingJob:job];
	[self updateDialog];
	
	[job lock];	/* Pending operations wait here until the current op finishes. */
	
	[mProgressIndicator setDoubleValue:0.0];
}

- (void)updateProgressForJob:(CPGPmailProgressJob*)job
{
	if (!job)
		return;
	
	[self setMessage:[self messageTextForOperation:[job operation]]];
	[self setFileName:[job file]];
	
	if ([job selector] && [job target])
	{
		if ([mProgressIndicator isIndeterminate])
		{
			[mProgressIndicator stopAnimation:nil];
			[mProgressIndicator setIndeterminate:NO];
		}
		
		[mStopButton setEnabled:YES];
	}
	else
	{
		if (![mProgressIndicator isIndeterminate])
			[mProgressIndicator setIndeterminate:YES];
		
		[mProgressIndicator startAnimation:nil];
		
		[mStopButton setEnabled:NO];
	}
	
	if (![mProgressWindow isVisible])
		[mProgressWindow performSelectorOnMainThread:@selector(orderFront:) withObject:nil waitUntilDone:NO];
}

- (void)finishedJob:(CPGPmailProgressJob*)job
{
	CPGPmailProgressJob*	nextJob;
	
	[job unlock];
	[job release];
	
	if ([mProgressIndicator isIndeterminate])
		[mProgressIndicator stopAnimation:nil];
	
	if ([mPendingJobs count])
	{
		[mPendingJobs removeObjectAtIndex:0];
		
		if ([mPendingJobs count])
		{
			nextJob = [mPendingJobs objectAtIndex:0];
			
			[self updateDialog];
			[self updateProgressForJob:nextJob];
			
			[nextJob unlock];
		}
		else
			[mProgressWindow performSelectorOnMainThread:@selector(close) withObject:nil waitUntilDone:NO];
	}
}

- (void)addPendingJob:(CPGPmailProgressJob*)job
{
	if ([mPendingJobs count])
		[job lock];
	
	[mPendingJobs addObject:job];
}

- (void)updateDialog
{
	NSView*					contentView = [mProgressWindow contentView];
	NSView*					jobView;
	NSView*					jobContainer;
	NSRect					jobFrame;
	NSRect					frame;
	NSMutableArray*			pendingViews = [[[NSMutableArray alloc] init] autorelease];
	NSEnumerator*			enumerator;
	CPGPmailProgressJob*	job;
	
	[mLock lock];
	
	if ([mPendingJobs count] == 1)
	{
		[mPendingView removeFromSuperview];
		[mPendingButton removeFromSuperview];
		[mPendingOpsText removeFromSuperview];
	}
	else if ([mPendingJobs count] > 1)
	{
		[mProgressView addSubview:mPendingButton];
		[mProgressView addSubview:mPendingOpsText];
	}
	
	if ([mPendingButton state] == NSOnState)
	{
		frame = NSMakeRect(0, 0, NSWidth([mProgressWindow frame]), 0);
		
		enumerator = [mPendingJobs objectEnumerator];
		
		job = [enumerator nextObject];	/* Toss out the first pending job */
		while (job = [enumerator nextObject])
		{
			jobView = [self viewForPendingJob:job];
			frame.size.height += NSHeight([jobView frame]);
			
			[pendingViews addObject:jobView];
		}
		
		jobContainer = [[[NSView alloc] initWithFrame:frame] autorelease];
		frame.origin.y = frame.size.height;
		
		enumerator = [pendingViews objectEnumerator];
		
		while (jobView = [enumerator nextObject])
		{
			jobFrame = [jobView frame];
			frame.origin.y -= NSHeight(jobFrame);
			jobFrame.origin.y = frame.origin.y;
			[jobView setFrame:jobFrame];
			[jobContainer addSubview:jobView];
		}
		
		frame = NSMakeRect(0, 0, NSWidth([contentView frame]), NSHeight([mProgressView frame]) + NSHeight([jobContainer frame]));
		contentView = [[[NSView alloc] initWithFrame:frame] autorelease];
		
		frame = NSMakeRect(0, NSHeight([jobContainer frame]), NSWidth([mProgressView frame]), NSHeight([mProgressView frame]));
		[mProgressView setFrame:frame];
		
		frame = NSMakeRect(0, 0, NSWidth([jobContainer frame]), NSHeight([jobContainer frame]));
		[jobContainer setFrame:frame];
		
		[contentView addSubview:mProgressView];
		[contentView addSubview:jobContainer];
	}
	else
	{
		frame = NSMakeRect(0, 0, NSWidth([mProgressView frame]), NSHeight([mProgressView frame]));
		contentView = [[[NSView alloc] initWithFrame:frame] autorelease];
		
		frame = NSMakeRect(0, 0, NSWidth([mProgressView frame]), NSHeight([mProgressView frame]));
		[mProgressView setFrame:frame];
		
		[contentView addSubview:mProgressView];
	}
	
	frame = [mProgressWindow frame];
	
	frame.origin.y -= NSHeight([contentView frame]) - NSHeight([[mProgressWindow contentView] frame]);
	frame.size.height -= NSHeight([[mProgressWindow contentView] frame]);
	frame.size.height += NSHeight([contentView frame]);
	[mProgressWindow setContentView:contentView];
	[mProgressWindow setFrame:frame display:YES];
	
	[mLock unlock];
}

- (IBAction)expandCollapsePending:(id)sender
{
	if ([mPendingButton state] == NSOnState)
		[mPendingButton setImage:mExpanded];
	else
		[mPendingButton setImage:mCollapsed];
	
	[self updateDialog];
}

- (IBAction)stopRequested:(id)sender
{
	CPGPmailProgressJob*	job;
	SEL						selector = nil;
	id						target = nil;
	NSInvocation*			invocation = nil;
	NSMethodSignature*		methodSig;
	
	if (!mPendingJobs)
		return;
	
	job = [mPendingJobs objectAtIndex:0];
	selector = [job selector];
	target = [job target];
	
	if (selector && target)
	{
		methodSig = [target methodSignatureForSelector:selector];
		invocation = [NSInvocation invocationWithMethodSignature:methodSig];
		[invocation setSelector:selector];
		[invocation setTarget:target];
		[invocation invoke];
	}
}

- (IBAction)cancelPendingJob:(id)sender
{
	CPGPmailProgressJob*	job;
	
	job = [[(NSButton*) sender cell] representedObject];
	
	if (job)
	{
		[mPendingJobs removeObject:job];
		[self updateDialog];
	}
}

- (NSView*)viewForPendingJob:(CPGPmailProgressJob*)job
{
	NSView*				container;
	NSBox*				divider;
	NSImageView*		imageView;
	NSTextField*		filename;
	NSButton*			cancel;
	NSRect				frame;
	
	if (!job)
		return nil;
	
	frame = NSMakeRect(0, 0, NSWidth([mProgressWindow frame]), 38);
	container = [[[NSView alloc] initWithFrame:frame] autorelease];
	
	frame = NSMakeRect(0, 31, NSWidth([mProgressWindow frame]), 5);
	divider = [[[NSBox alloc] initWithFrame:frame] autorelease];
	[divider setBoxType:NSBoxSeparator];
	[divider setBorderType:NSLineBorder];
	
	frame = NSMakeRect(20, 4, 24, 24);
	imageView = [[[NSImageView alloc] initWithFrame:frame] autorelease];
	[imageView setEditable:NO];
	[imageView setImage:[self imageForOperation:[job operation]]];
	
	frame = NSMakeRect(49, 9, 249, 14);
	filename = [[[NSTextField alloc] initWithFrame:frame] autorelease];
	[filename setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[filename setEditable:NO];
	[filename setBezeled:NO];
	[filename setDrawsBackground:NO];
	[filename setStringValue:[job file]];
	
	frame = NSMakeRect(298, 2, 84, 25);
	cancel = [[[NSButton alloc] initWithFrame:frame] autorelease];
	[cancel setButtonType:NSMomentaryPushButton];
	[cancel setBordered:YES];
	[cancel setBezelStyle:NSRoundedBezelStyle];
	[cancel setTitle:LSPM(@"Cancel", @"")];
	[cancel setAction:@selector(cancelPendingJob:)];
	[cancel setTarget:self];
	[[cancel cell] setControlSize:NSSmallControlSize];
	[[cancel cell] setRepresentedObject:job];
	
	[container addSubview:divider];
	[container addSubview:imageView];
	[container addSubview:filename];
	[container addSubview:cancel];
	
	return container;
}

- (NSString*)messageTextForOperation:(PGPmailOperation)operation
{
	switch (operation)
	{
		case kPGPmailEncryptOperation:
			return kPGPmailEncryptStr;
			break;
		
		case kPGPmailSignOperation:
			return kPGPmailSignStr;
			break;
		
		case kPGPmailEncryptSignOperation:
			return kPGPmailEncryptSignStr;
			break;
		
		case kPGPmailDecryptVerifyOperation:
			return kPGPmailDecryptVerifyStr;
			break;
		
		case kPGPmailWipeOperation:
			return kPGPmailWipeFileStr;
			break;
		
		default:
			return kPGPmailProcessingStr;
			break;
	}
}

- (NSImage*)imageForOperation:(PGPmailOperation)operation
{
	switch (operation)
	{
		case kPGPmailEncryptOperation:
			return mEncryptImage;
			break;
		
		case kPGPmailSignOperation:
			return mSignImage;
			break;
		
		case kPGPmailEncryptSignOperation:
			return mEncryptSignImage;
			break;
		
		case kPGPmailDecryptVerifyOperation:
			return mDecryptVerifyImage;
			break;
		
		case kPGPmailWipeOperation:
			return mWipeImage;
			break;
		
		default:
			return nil;
			break;
	}
}

- (void)doProgress:(UInt64)bytes total:(UInt64)total
{
	if (!bytes && !total)
		[mProgressIndicator setDoubleValue:0];
	else
		[mProgressIndicator setDoubleValue:((double)bytes/(double)total)*100];
}

- (void)dealloc
{
	if (self == sharedProgressObject)
		sharedProgressObject = nil;
	
	[mLock release];
	mLock = nil;
	
	[mPendingView release];
	mPendingView = nil;
	
	[mPendingButton release];
	mPendingButton = nil;
	
	[mPendingOpsText release];
	mPendingOpsText = nil;
	
	[mPendingJobs release];
	mPendingJobs = nil;
	
	[mEncryptImage release];
	mEncryptImage = nil;
	
	[mSignImage release];
	mSignImage = nil;
	
	[mEncryptSignImage release];
	mEncryptSignImage = nil;
	
	[mDecryptVerifyImage release];
	mDecryptVerifyImage = nil;
	
	[mWipeImage release];
	mWipeImage = nil;
	
	[mCollapsed release];
	mCollapsed = nil;
	
	[mExpanded release];
	mExpanded = nil;
	
	[super dealloc];
}

@end
