/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPmailWipeFile.m,v 1.22 2002/11/05 02:21:02 jeffc Exp $
____________________________________________________________________________*/

#import <unistd.h>
#import <pgpMem.h>
#import <sys/param.h>
#import <sys/mount.h>
#import <PGPclient/pflPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpDiskWiper.h>

#import <Foundation/Foundation.h>

#import "CPGPmailFSWiper.h"
#import "CPGPmailWipeFile.h"

static int sSortDirectories(NSString* str1, NSString* str2, void* context)
{
	NSScanner*	scanner;
	unsigned	slashCount1 = 0;
	unsigned	slashCount2 = 0;
	
	scanner = [NSScanner scannerWithString:str1];
	
	/* Scan initial / */
	if ([scanner scanString:@"/" intoString:nil])
		++slashCount1;
	
	while ([scanner scanUpToString:@"/" intoString:nil])
	{
		if ([scanner scanString:@"/" intoString:nil])
			++slashCount1;
	}
	
	scanner = [NSScanner scannerWithString:str2];
	
	/* Scan initial / */
	if ([scanner scanString:@"/" intoString:nil])
		++slashCount2;
	
	while ([scanner scanUpToString:@"/" intoString:nil])
	{
		if ([scanner scanString:@"/" intoString:nil])
			++slashCount2;
	}
	
	return slashCount1 > slashCount2 ? NSOrderedAscending : NSOrderedDescending;
}

static BOOL	sPGPmailWipeStop = NO;

@interface CPGPmailWipeFile( PrivateMethods )

- (void)prescanFolder:(NSString *)folder;

- (void)prescan:(NSArray *)inFiles;

- (PGPError)wipeFolder:(NSString *)folder;

- (PGPError)getWipingPatternList:(PGPUInt32)numPasses
    list:(PGPWipingPattern **)patterns;

- (PGPError)wipe;

- (PGPError)wipeFile:(NSString *)file;

- (void)wipeFiles:(NSArray *)files;

- (void)warningSheetDidEnd:(NSWindow *)sheet
    returnCode:(int)returnCode
    contextInfo:(void *)contextInfo;

- (PGPUInt64)fileSize:(NSString *)inFile;

- (void)startProcessing;

@end

@implementation CPGPmailWipeFile

- (id)init
{
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }
    
    if( !mController )
        PGPLog((@"%s:%d: Warning: mController has not been set.", __FILE__, __LINE__));
    
    if( !mContext && mController )
        mContext = [mController pgpContext];
    
    if( !mPrefRef && mController )
        mPrefRef = [mController pgpPrefRef];
        
    mFileManager		= [[NSFileManager defaultManager] retain];
	mProgressDialog		= [CPGPmailProgress sharedInstance];
    
    mWipeFile			= nil;
	mNumWipePasses 		= 0;
    mTotalBytesToWipe 	= 0L;
    mTotalBytesDone		= 0L;
    mWipeBuffer 		= NULL;
	mWipeBufferSize 	= 0;
	mPatternList		= NULL;
    sPGPmailWipeStop	= NO;
    mFilesToWipe		= nil;
    
    return self;
}

- (id)initWithController:(CPGPmailController *)controller
{
    mController = controller;

    return [self init];
}

- (void)dealloc
{
	if( IsntNull( mPatternList ) )
	{
		PGPFreeData( mPatternList );
		mPatternList = NULL;
	}
    if( nil != mFilesToWipe )
        [mFilesToWipe release];
        
    [super dealloc];
}

- (PGPContextRef)pgpContext
{
    return mContext;
}

- (BOOL)progress:(PGPUInt64)inTotalDone
{
	mTotalBytesDone += inTotalDone;
    
	[mProgressDialog doProgress:mTotalBytesDone total:mTotalBytesToWipe];
    
	if (sPGPmailWipeStop)
	{
		sPGPmailWipeStop = NO;
		return YES;
	}
	else
		return NO;
}

- (void)setLock:(NSLock*)lock
{
	mLock = lock;
}

- (void)processFiles:(NSArray *)inFiles
{
    int					result = 0;
    PGPBoolean			warn = YES;
    PGPError			theError = kPGPError_NoErr;
    
	theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefWarnOnWipe, &warn);
    mFilesToWipe = [inFiles copy];
    
    if( warn )
    {
        NSBeep();
        if( [[mController window] isVisible] )
            NSBeginCriticalAlertSheet(kConfirmWipeStr, kOKStr, kCancelStr, nil, [mController window], self, nil,
                @selector(warningSheetDidEnd:returnCode:contextInfo:), NULL, kCannotUndoStr);
        else
            result = NSRunCriticalAlertPanel(kConfirmWipeStr, kCannotUndoStr, kOKStr, kCancelStr, nil);
    }
    else
        result = NSAlertDefaultReturn;
    
    if( NSAlertDefaultReturn == result )
        [self startProcessing];
}

- (PGPError)wipe
{
	PGPError		theError = kPGPError_NoErr;

	pgpAssert( mNumWipePasses != 0 );
	    
	theError = [self getWipingPatternList:mNumWipePasses list:&mPatternList];
	if( IsntPGPError( theError ) )
	{
		mWipeBufferSize = 512L * 1024L;
		
		mWipeBuffer = PGPNewData( PGPPeekContextMemoryMgr([mController pgpContext]),
                            mWipeBufferSize, kPGPMemoryMgrFlags_None);
		if( IsntNull( mWipeBuffer ) )
		{		
			theError = [self wipeFile:mWipeFile];
			
			PGPFreeData( mWipeBuffer );
			mWipeBuffer = NULL;
		}
		else
			theError = kPGPError_OutOfMemory;
		
		PGPFreeData( mPatternList );
		mPatternList = NULL;
	}
    
	return( theError );
}

- (PGPUInt64)fileSize:(NSString *)inFile
{
    NSDictionary	*fattrs = nil;
    NSNumber 		*fileSize = nil;
    
    fattrs = [mFileManager fileAttributesAtPath:inFile traverseLink:NO];
    fileSize = [fattrs objectForKey:NSFileSize];
	return [fileSize unsignedLongLongValue];
}

//____________________________________________________________________________
//	Recursively scan the spec and its subfolders, though spec could be a file.
//____________________________________________________________________________
- (void)prescanFolder:(NSString *)folder
{
    NSString				*theFile = nil;
    NSDirectoryEnumerator	*theEnumerator = [mFileManager enumeratorAtPath:folder];
    
    while( theFile = [theEnumerator nextObject] )
    {
        BOOL			isDir = NO;
        NSString		*fullPath = [folder stringByAppendingPathComponent:theFile];
        
        if([mFileManager fileExistsAtPath:fullPath isDirectory:&isDir]  && !isDir)
            mTotalBytesToWipe += [self fileSize:fullPath];
    }
}

- (void)prescan:(NSArray *)inFiles
{
	NSEnumerator*	enumerator;
	NSString*		path;
	BOOL			isDir;
	
	enumerator = [inFiles objectEnumerator];
	
	while (path = [enumerator nextObject])
	{
		if ([mFileManager fileExistsAtPath:path isDirectory:&isDir] && isDir)
			[self prescanFolder:path];
		else
			mTotalBytesToWipe += [self fileSize:path];
	}
}

- (PGPError)wipeFile:(NSString *)file
{
	PGPError			theError = kPGPError_NoErr;
    CPGPmailFSWiper*	wiper = [[CPGPmailFSWiper alloc] initWithController:self];
    	
	// this takes care of accumulating each files progress count
	[mProgressDialog setFileName:[mFileManager displayNameAtPath:file]];
	
	// wipe the data
	theError = [wiper wipeAndDelete:file passes:mNumWipePasses
                    patterns:mNumWipePasses list:mPatternList
                    buffer:mWipeBuffer size:mWipeBufferSize];
    [wiper release];
    
	return( theError );
}

- (PGPError)wipeFolder:(NSString*)folder
{
    PGPError 				err = kPGPError_NoErr;
	NSDirectoryEnumerator*	dirEnumerator;
	NSMutableArray*			directories = [[[NSMutableArray alloc] init] autorelease];
	NSArray*				subpaths;
	NSEnumerator*			enumerator;
	NSString*				path;
	NSString*				fullPath;
	NSString*				tempFile;
	struct statfs			pathStat;
	char					bogusName[32];
	BOOL					isDir;
    
	[directories addObject:folder];
	
	subpaths = [mFileManager subpathsAtPath:folder];
	enumerator = [subpaths objectEnumerator];
	
	if ([subpaths count] > 50)
	{
		[mJob setSelector:nil];
		[mJob setTarget:nil];
		[mJob setFile:[NSString stringWithFormat:LSPM(@"Processing %@", @""), folder]];
		[mProgressDialog updateProgressForJob:mJob];
	}
	
	while (path = [enumerator nextObject])
	{
		fullPath = [folder stringByAppendingPathComponent:path];
		if ([mFileManager fileExistsAtPath:fullPath isDirectory:&isDir] && isDir)
			[directories addObject:fullPath];
	}
	
	[directories sortUsingFunction:sSortDirectories context:NULL];
	
	if ([subpaths count] > 50)
	{
		[mJob setSelector:@selector(stopRequested)];
		[mJob setTarget:self];
		[mJob setFile:@""];
		[mProgressDialog updateProgressForJob:mJob];
	}
	
	enumerator = [directories objectEnumerator];
	
	while (IsntPGPError(err) && (path = [enumerator nextObject]))
	{
		if (statfs([path UTF8String], &pathStat) == 0)
		{
			NSString*	trashDir = [[NSString stringWithUTF8String:pathStat.f_mntonname] stringByAppendingPathComponent:@".Trashes"];

			dirEnumerator = [mFileManager enumeratorAtPath:path];
			
			while (IsntPGPError(err) && (mWipeFile = [dirEnumerator nextObject]))
			{
				mWipeFile = [path stringByAppendingPathComponent:mWipeFile];
				
				/* Skip over .Trashes */
				if ([mWipeFile isEqualToString:trashDir])
					continue;
				
				err = [self wipe];
			}
			
			if (IsntPGPError(err))
			{
				/* Skip over .Trashes and the volume mount point (if wiping a volume) */
				if ([path isEqualToString:trashDir] || [path isEqualToString:[NSString stringWithUTF8String:pathStat.f_mntonname]])
					continue;
				
				pgpFillMemory(bogusName, sizeof(bogusName) - 1, 'x');
				bogusName[31] = '\0';
				
				tempFile = [[path stringByDeletingLastPathComponent] stringByAppendingPathComponent:[NSString stringWithUTF8String:bogusName]];
				
				if ([mFileManager movePath:path toPath:tempFile handler:nil])
					sync();
				
				err = [mFileManager removeFileAtPath:tempFile handler:nil] ? kPGPError_NoErr : kPGPError_FileOpFailed;
			}
		}
		else
			PGPLog((@"Failed to statfs path: %@", path));
	}
	
	return err;
}

- (void)wipeFiles:(NSArray *)files
{
    NSAutoreleasePool		*thePool = [[NSAutoreleasePool alloc] init];
    NSString*				filename;
	PGPError				theError = kPGPError_NoErr;
    int						index = 0;
	BOOL					isDir;
	
	if (!files || ![files count])
		goto done;
	
	if ([files count] > 1)
		filename = [NSString stringWithFormat:@"%@, ", [mFileManager displayNameAtPath:[files objectAtIndex:0]]];
	else
		filename = [mFileManager displayNameAtPath:[files objectAtIndex:0]];
	
	mJob = [[CPGPmailProgressJob alloc] initWithFile:filename operation:kPGPmailWipeOperation selector:nil target:nil];
	[mProgressDialog startProgressForJob:mJob];
	
	if ([mFileManager fileExistsAtPath:[files objectAtIndex:0] isDirectory:&isDir] && isDir)
	{
		[mJob setFile:LSPM(@"Calculating file sizes", @"")];
		[mProgressDialog updateProgressForJob:mJob];
	}
	
    [self prescan:files];
	
	[mJob setSelector:@selector(stopRequested)];
	[mJob setTarget:self];
	[mJob setFile:@""];
	[mProgressDialog updateProgressForJob:mJob];
	
    theError = PGPGetPrefNumber( mPrefRef, kPGPPrefFileWipePasses, &mNumWipePasses );
	
    mTotalBytesToWipe *= mNumWipePasses;
    
    pgpAssertNoErr( theError );
    
    if( mNumWipePasses == 0 )
        mNumWipePasses = 3;
	
	[mProgressDialog updateProgressForJob:mJob];
	
    for( index = 0; IsntPGPError(theError) && (index < [files count]); index++ )
    {
        NSString		*tmpFile = [files objectAtIndex:index];
        BOOL			isFolder = NO;
        
        [mFileManager fileExistsAtPath:tmpFile isDirectory:&isFolder];
        if( isFolder )
            theError = [self wipeFolder:tmpFile];
        else
        {
            mWipeFile = tmpFile;
            theError = [self wipe];
        }
    }
    
    if (mJob)
    {
        [mProgressDialog finishedJob:mJob];
        [mJob release];
		mJob = nil;
    }
	
done:
	if (mLock)
		[mLock unlock];
	
    mWipeFile = nil;
    [mFileManager release];
    [thePool release];
    [self release];
}

- (PGPError)getWipingPatternList:(PGPUInt32)numPasses
    list:(PGPWipingPattern **)patterns
{
	PGPDiskWipeRef		wipeRef;
	PGPError			theError = kPGPError_NoErr;
	
	*patterns = NULL;
	
	theError = PGPCreateDiskWiper( mContext, &wipeRef, numPasses );
	if( IsntPGPError( theError ) )
	{
		PGPWipingPattern	*patternList;
		
		patternList = (PGPWipingPattern *) PGPNewData(
							PGPPeekContextMemoryMgr( mContext ),
							numPasses * sizeof( *patternList ), 0 );
		if( IsntNull( patternList ) )
		{
			PGPUInt32	patternIndex;

			for( patternIndex = 0; patternIndex < numPasses; patternIndex++ )
			{
			#if FIXED_PATTERN	/* [ */
			
				PGPUInt32	*cur;
				
				cur = (PGPUInt32 *) &patternList[patternIndex].bytes[0];
				for( PGPUInt32 index = 0; index < 128; index++ )
					*cur++ = 'DAVE';
					
			#else	/* ] FIXED_PATTERN [ */
				
				PGPInt32 	buffer[256];
				
				theError = PGPGetDiskWipeBuffer( wipeRef, buffer );
				if( IsPGPError( theError ) )
					break;
					
				pgpCopyMemory( buffer, &patternList[patternIndex],
						sizeof( patternList[0] ) );
			#endif	/* ] FIXED_PATTERN */
			}
			
			*patterns = patternList;
		}
		else
			theError = kPGPError_OutOfMemory;
		
		PGPDestroyDiskWiper( wipeRef );
	}

	return( theError );
}

- (void)warningSheetDidEnd:(NSWindow *)sheet
    returnCode:(int)returnCode
    contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
	if( NSAlertDefaultReturn == returnCode )
        [self startProcessing];
}

- (void)startProcessing
{
	[NSThread detachNewThreadSelector:@selector(wipeFiles:) toTarget:self withObject:mFilesToWipe];
}

- (void)stopRequested
{
    sPGPmailWipeStop = YES;
}

@end
