/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPmailFileUtilities.m,v 1.18 2002/11/05 02:21:02 jeffc Exp $
____________________________________________________________________________*/

#import "PGPmailCommon.h"
#import "PGPmailFileUtilities.h"
#import "CPGPmailController.h"
#import <Cocoa/Cocoa.h>
#import <sys/param.h>
#import <sys/mount.h>

#define kLocateDetachedSigSourceFilePromptStr	LSPM(@"Please locate the source file for the signature file:", @"")

	void
GetDefaultEncryptSignName(
    NSString* 		srcName,
    BOOL 			textOutput,
    BOOL			detachedSignature,
    NSString*		*destName)
{
	NSString*			suffix = nil;
	NSString*			path = [srcName stringByDeletingLastPathComponent];
	NSString*			fileName = [srcName lastPathComponent];
	NSMutableString*	desiredName = [NSMutableString string];
	struct statfs		pathStat;
	unsigned			maxFileLen = kPGPMaxFileNameLength;
	unsigned			diff;
    
	if (textOutput && !detachedSignature)
		suffix = kASCIIFileSuffixStr;
	else if (detachedSignature)
		suffix = kDetachedSigFileSuffixStr;
	else
		suffix = kBinaryFileSuffixStr;
	
	[desiredName setString:[fileName stringByAppendingString:suffix]];
	
	if (statfs([srcName UTF8String], &pathStat) == 0)
	{
		if (!strncmp(pathStat.f_fstypename, "hfs", strlen("hfs")))
		{
			if (pathStat.f_bsize != 4096)
				maxFileLen = 31;
		}
	}
	
	if ([desiredName length] > maxFileLen)
	{
		diff = [desiredName length] - maxFileLen;
		
		if ([[@"." stringByAppendingString:[fileName pathExtension]] length] > diff)
			[desiredName setString:[[fileName stringByDeletingPathExtension] stringByAppendingString:suffix]];
		else
		{
			diff += [suffix length];
			[desiredName setString:[[fileName substringWithRange:NSMakeRange(0, [desiredName length] - diff)] stringByAppendingString:suffix]];
		}
	}
	
	*destName = [path stringByAppendingPathComponent:desiredName];
}

/*____________________________________________________________________________
	Given a source spec, determine the correct default decrypt output spec.
	
	Examples:
	"X.pgp"	=>	"X"
	"X.asc"	=>	"X"
	"X"		=>	"X"
	
	If no suffix is detected, then the name is not changed, and the caller
	has to figure out a name.
____________________________________________________________________________*/

	BOOL
GetDefaultDecryptVerifyName(
    NSString* 		srcName,
    NSString*		*destName )
{
	BOOL				foundSuffix	= NO;

    *destName = [NSString stringWithString:srcName];
    
	foundSuffix = [*destName hasSuffix:kBinaryFileSuffixStr];
	if (! foundSuffix)
	{
		foundSuffix = [*destName hasSuffix:kGPGBinaryFileSuffixStr];
	}
	if( ! foundSuffix )
	{
		foundSuffix	= [*destName hasSuffix:kASCIIFileSuffixStr];
	}
	
	if( foundSuffix )
	{
		*destName = [srcName stringByDeletingPathExtension];
	}
	
	return( foundSuffix );
}

    BOOL
GetDetachedSigFileSourceName(
    NSString*		sigFileName,
    NSString*		*sourceFileName)
{
	BOOL				matchedSuffix = NO;

    *sourceFileName = [NSString stringWithString:sigFileName];
    
	if( [*sourceFileName hasSuffix:kDetachedSigFileSuffixStr] )
	{
		*sourceFileName 	= [sigFileName stringByDeletingPathExtension];
		matchedSuffix		= TRUE;
	}
	else
	{
		if( [*sourceFileName hasSuffix:kASCIIFileSuffixStr] )
		{
			*sourceFileName 	= [sigFileName stringByDeletingPathExtension];
			matchedSuffix		= TRUE;
		}
		else
		{
			if( [*sourceFileName hasSuffix:kBinaryFileSuffixStr] )
			{
				*sourceFileName 	= [sigFileName stringByDeletingPathExtension];
				matchedSuffix		= TRUE;
			}
		}
	}
	
	return( matchedSuffix );
}

    void
GetUniqueFileName(
    NSString*		inFile,
    NSString*		*outFile)
{
    NSFileManager*		manager = [NSFileManager defaultManager];
    NSString*			fileExt = [inFile pathExtension];
    NSString*			fileName = [inFile stringByDeletingPathExtension];
    NSMutableString*	newFile = [NSMutableString string];
    int					count = 0;
    
    [newFile setString:inFile];
    while( [manager fileExistsAtPath:newFile] )
    {
        [newFile setString:fileName];
        [newFile appendString:[NSString stringWithFormat:@"%d", ++count]];
        if( [fileExt length] != 0 )
        {
            [newFile appendString:@"."];
            [newFile appendString:fileExt];
        }
    }
    *outFile = [newFile copy];
}

    void
GetPGPFileKindInfo(
    NSString*		inFile,
    FileKindInfo*	info )
{
    NSDictionary*		fattrs = [[NSFileManager defaultManager]
                                    fileAttributesAtPath:inFile traverseLink:NO];
    OSType				fileType = [fattrs fileHFSTypeCode];
    OSType				fileCreator = [fattrs fileHFSCreatorCode];
    
    info->isEncrypted = info->isSigned = info->isDetachedSig = NO;
    info->isKeyShareFile = info->isPGPFile = info->maybePGPFile = NO;
    
    if( kPGPMacFileCreator_PGPtools == fileCreator)
	{
		// it's a PGP file
		info->isPGPFile	= TRUE;
		
		switch( fileType )
		{
			default:
			case kPGPMacFileType_ArmorFile:
				break;
				
			case kPGPMacFileType_EncryptedData:
				info->isEncrypted = TRUE;
				break;
				
			case kPGPMacFileType_SignedData:
				info->isSigned = TRUE;
				break;
				
			case kPGPMacFileType_DetachedSig:
				info->isDetachedSig	= TRUE;
				break;
		}
	}
	else if( kPGPMacFileCreator_PGPkeys == fileCreator )
	{
		if( kPGPMacFileType_KeyShares == fileType )
		{
			info->isPGPFile			= TRUE;
			info->isKeyShareFile 	= TRUE;
		}
	}
	else if( 'TEXT' == fileType )
	{
		info->maybePGPFile = TRUE;
	}
	else
	{
		// look for file name extensions
		if( [inFile hasSuffix:kBinaryFileSuffixStr] )
		{
			info->maybePGPFile = TRUE;
		}
		else if( [inFile hasSuffix:kDetachedSigFileSuffixStr] )
		{
			info->maybePGPFile = TRUE;
		}
		else if( [inFile hasSuffix:kASCIIFileSuffixStr] )
		{
			info->maybePGPFile = TRUE;
		}
        else if( [inFile hasSuffix:kKeyShareFileSuffixStr] )
        {
			info->isPGPFile			= TRUE;
            info->isKeyShareFile	= TRUE;
        }
	}
}

	BOOL
PromptForSignatureSourceFile(
	NSString*		sigFile,
	NSString*		*sourceFile)
{
    NSOpenPanel*		oPanel;
    BOOL				status = YES;
    int					result;
    	
    oPanel = [NSOpenPanel openPanel];
    [oPanel setAllowsMultipleSelection:NO];
    
    result = [oPanel runModalForDirectory:[sigFile stringByDeletingLastPathComponent]
            file:[sigFile lastPathComponent] types:nil];
    if( result == NSOKButton )
    {
        NSArray *filesToOpen = [oPanel filenames];
        *sourceFile = [filesToOpen objectAtIndex:0];
    }
	else
	{
		status = NO;
	}
			
	return( status );
}

    PGPError
MacBinaryConversion(
    PGPFileSpecRef fileRef)
{
	PGPError 		theError = kPGPError_NoErr;
	PGPUInt32		macCreator,macTypeCode;
	PGPFileSpecRef 	deMacifiedFSpec = 0;

	theError=PGPMacBinaryToLocal(fileRef,
                &deMacifiedFSpec,
                &macCreator,&macTypeCode );

	if(deMacifiedFSpec)
		PGPFreeFileSpec(deMacifiedFSpec);

	return( theError );
}
