/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: MessageEditor.h,v 1.2 2002/08/24 01:32:31 wmalik Exp $
____________________________________________________________________________*/

#import <Foundation/NSObject.h>
#import <Foundation/NSGeometry.h>
#import <Foundation/Foundation.h>

@class NSButton;
@class TilingView;
@class AddressTextField;
@class NSPopUpButton;
@class MessageTextView;
@class CompletionController;
@class TextDocumentViewer;
@class NSTextField;
@class ComposeBackEnd;
@class NSMutableArray;
@class Favorites;
@class NSToolbar;
@class MailDelivery;
@class NSView;
@class NSPanel;
@class OldCompletionController;
@class NSWindow;

@interface MessageEditor:NSObject
{
    TilingView *headerContainer;
    AddressTextField *fieldPrototype;
    NSView *fromAndSignatureContainer;
    NSPopUpButton *fromPopup;
    NSTextField *fromLabel;
    NSPopUpButton *signaturePopup;
    NSTextField *signatureLabel;
    MessageTextView *composeText;
    OldCompletionController *completionController;
    TextDocumentViewer *textDocumentViewer;
    NSView *composeContentView;
    NSView *viewerContentView;
    ComposeBackEnd *backEnd;
    NSMutableArray *accessoryViewOwners;
    MailDelivery *delivery;
    //struct ? _flags;
    long _flags;
    NSToolbar *_toolbar;
    NSMutableDictionary *_toolbarItems;
    int _messageType;
    NSPanel *_deliveryFallbackPanel;
    id _deliveryFallbackErrorLabel;
    id _deliveryFallbackPopupButton;
    NSWindow *becomeRichSheetWindow;
    char _chatShouldBeEnabled;
    char _userKnowsSaveFailed;
}

+ createEditorWithType:(int)fp12 originalMessage:fp16 forwardedText:fp20 showAllHeaders:(char)fp24;
+ existingEditorViewingMessage:fp12;
+ editorViewingMessage:fp12 showAllHeaders:(char)fp16;
+ (void)handleFailedDeliveryOfMessage:fp12 store:fp16 reason:fp20;
+ (void)restoreDraftMessage:fp12 withSavedState:fp16;
+ (void)restoreTornOffMessage:fp12 withSavedState:fp16;
+ (void)saveDefaults;
+ (void)restoreFromDefaults;
+ (void)setIsInlineSpellCheckingEnabled:(char)fp12;
+ (void)setNeedsAutosave;
+ (void)autosaveTimerFired;
- (void)show;
- (void)readDefaultsFromDictionary:fp12;
- (void)writeDefaultsToArray:fp12;
- (void)setHeaders:fp12;
- (void)appendAttributedString:fp12;
- window;
- (void)appendAddresses:fp12 toHeader:fp16;
- (void)addBccHeader:fp12;
- (void)addReplyToHeader:fp12;
- (void)saveMessageToDrafts:fp12;
- (void)insertOriginalAttachments:fp12;
- (void)removeAttachments:fp12;
- (void)_toggleRichSheetDidEnd:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)toggleRich:fp12;
- (void)makeRichText:fp12;
- (void)makePlainText:fp12;
- (void)showPrintPanel:fp12;
- (void)pasteAsQuotation:fp12;
- (void)searchIndex:fp12;
- (void)saveAs:fp12;
- (char)_validateAction:(SEL)fp12 tag:(int)fp16;
- (char)validateMenuItem:fp12;
- (void)send:fp12;
- (void)malformedAddressSheetClosed:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)changeReplyMode:fp12;
- (void)insertFile:fp12;
- (void)showAddressPanel:fp12;
- (void)appendMessages:fp12;
- (void)chatWithRecipients:fp12;
- (void)changeHeaderField:fp12;
- (void)nowWouldBeAGoodTimeToTerminate:fp12;
- windowWillReturnFieldEditor:fp12 toObject:fp16;
- (char)textView:fp12 doCommandBySelector:(SEL)fp16;
- (void)stayPlainOrBecomeRich:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (char)windowShouldClose:fp12;
- (void)windowDidResize:fp12;
- (void)windowDidMove:fp12;
- (void)changeFromHeader:fp12;
- (void)undoSignatureChange:fp12;
- (void)changeSignature:fp12;
- (void)textDidChange:fp12;
- attachmentDirectory;
- (void)textView:fp12 doubleClickedOnCell:fp16 inRect:(struct _NSRect)fp20 atIndex:(unsigned int)fp32;
- (void)textView:fp12 clickedOnCell:fp16 inRect:(struct _NSRect)fp20 atIndex:(unsigned int)fp32;
- (void)textView:fp12 draggedCell:fp16 inRect:(struct _NSRect)fp20 event:fp32 atIndex:(unsigned int)fp36;
- (char)textView:fp12 shouldReadSelectionFromPasteboard:fp16 type:fp20 result:(char *)fp24;
- attachmentContextMenu;
- (void)enableCompletion:(char)fp12 forTextField:fp16;
- backEnd;
- (void)setBackEnd:fp12;
- (void)updateUIAfterAppleScriptModification:fp12;
- (void)deliveryFallbackPanelSendLater:fp12;
- (void)deliveryFallbackPanelEditMessage:fp12;
- (void)deliveryFallbackPanelTryOtherAccount:fp12;
- (void)presenceChanged:fp12;
- (void)presenceIconChanged:fp12;
- (void)updatePresenceButtonStateEditedField:fp12;
- (void)addSenderToAddressBook:fp12;
@end

@interface MessageEditor(Internal)
+ sharedContextMenu;
+ (void)showEditorWithSavedState:fp12;
- _frameSaveName;
- (void)dealloc;
- init;
- (void)subjectChanged;
- (void)addressFieldChanged;
- (char)_sendButtonShouldBeEnabled;
- (void)_setupTextViewInlineSpellChecking;
- (void)_configureComposeWindowForType:(int)fp12 message:fp16;
- initWithType:(int)fp12 message:fp16 showAllHeaders:(char)fp20;
- (char)_prepareMessageToBeSaved:fp12;
- (void)saveMessage;
- (void)updateFromAndSignatureView;
- (void)updateHeaderFields;
- (void)configureInitialText:fp12;
- (char)isRichText;
- (void)setRichText:(char)fp12;
- (void)backEndDidChange:fp12;
- (void)reloadContentsFromMessage:fp12;
- (void)messageWillNeedRecoloring:fp12;
- (void)_setMessageStatusOnOriginalMessage;
- (void)backEndDidAppendMessageToOutbox:fp12 result:(int)fp16;
- (char)backEnd:fp12 shouldDeliverMessage:fp16;
- (char)backEnd:fp12 shouldSaveMessage:fp16;
- (void)_setUserSavedMessage:(char)fp12;
- (void)backEndDidSaveMessage:fp12 result:(int)fp16;
- (void)deliveryCompleted:fp12;
- (void)reportDeliveryFailure:fp12;
- (void)chooseAlternateDeliveryAccountSheetClosed:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)deliveryFailureSheetDidClose:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)cleanupUIAfterDeliveryNotification:fp12;
- (void)completionControllerDidSelectAddress:fp12;
- (void)textFieldBeganOrEndedEditing:fp12;
- (void)appendMessageArray:fp12;
- (void)addFileWrappersForPaths:fp12 turnFoldersIntoLinks:(char)fp16;
- (void)openPanelSheetDidEnd:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)saveState;
- (void)forceWindowClose;
- (void)closeConfirmSheetDidEnd:fp12 returnCode:(int)fp16 forSave:(void *)fp20;
- (char)_isViewingMessage;
- _messageBeingViewed;
- (void)_configureTornOffMessageWindowForMessage:fp12;
- (void)_messageWillBeDisplayed:fp12;
- (void)messageFlagsChanged:fp12;
- (void)_monitorStoreForChanges;
- (void)_stopMonitoringStoreForChanges;
- (void)deleteMessages:fp12;
- (void)deleteMessage:fp12;
- (void)_returnToSenderSheetDidEnd:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)markAsRead:fp12;
- (void)markAsUnread:fp12;
- (void)markAsFlagged:fp12;
- (void)markAsUnflagged:fp12;
- (void)returnToSender:fp12;
- (void)_convertWindowToType:(int)fp12 initialText:fp16;
- (void)replyMessage:fp12;
- (void)replyAllMessage:fp12;
- (void)forwardMessage:fp12;
- (void)redirectMessage:fp12;
- (char)performKeyEquivalent:fp12;
- (void)presencePreferenceChanged:fp12;
- (void)updatePresenceButtonState;
- (void)updatePresenceButtonStateForAddresses:fp12;
@end

@interface MessageEditor(MessageEditorToolbar)
- (char)validateToolbarItem:fp12;
- (void)_setupToolBarOnWindow:fp12 messageType:(int)fp16;
- (void)_synchronizeChangeReplyItem:fp12 messageType:(int)fp16;
- (void)toggleReplytype:(int)fp12;
- (void)_updateSendButtonStateInToolbar;
- (void)_updateSendButtonInToolbarWithImage:fp12;
- (void)toolbarWillAddItem:fp12;
- toolbar:fp12 itemForItemIdentifier:fp16 willBeInsertedIntoToolbar:(char)fp20;
- toolbarDefaultItemIdentifiers:fp12;
- toolbarAllowedItemIdentifiers:fp12;
@end
