/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: MessageViewer.h,v 1.1 2002/08/21 03:17:17 wmalik Exp $
____________________________________________________________________________*/

#import <AppKit/NSResponder.h>
#import <Foundation/NSGeometry.h>

@class MessageStore;
@class TableViewManager;
@class TextDocumentViewer;
@class NSWindow;
@class NSSplitView;
@class NSBox;
@class NSToolbar;
@class NSButton;
@class NSTextField;
@class NSPopUpButton;
@class NSProgressIndicator;
@class ASExtendedOutlineView;
@class MailboxesOutlineViewOwner;
@class NSDrawer;
@class StoreViewingAttributes;
@class SearchTextField;
@class NSView;
@class NSMutableDictionary;
@class MailboxUid;
@class MessageMall;
@class ExpandingSplitView;
@class AsyncArrows;
@class NSToolbarItem;
@class NSMenuItem;

@interface MessageViewer:NSResponder
{
    MessageMall *_messageMall;
    TableViewManager *_tableManager;
    TextDocumentViewer *_textViewer;
    NSWindow *_window;
    ExpandingSplitView *_splitView;
    NSTextField *_statusField;
    AsyncArrows *_progressIndicator;
    NSBox *_messageViewerBox;
    NSBox *_mailboxesDrawerBox;
    ASExtendedOutlineView *_outlineView;
    SearchTextField *_searchField;
    NSView *_searchFieldView;
    NSToolbarItem *_searchViewItem;
    NSPopUpButton *_searchPopup;
    NSString *_lastSearchPhrase;
    NSMenuItem *_selectedMailboxesItem;
    MailboxesOutlineViewOwner *_outlineViewOwner;
    NSDrawer *_mailboxesDrawer;
    NSString *_sizeStatusString;
    NSString *_uiStatusMessage;
    char _shouldCloseMailboxListingAfterDragOperation;
    char _shouldPreventTableViewResize;
    char _shouldMakeTableViewSelectionVisible;
    float _lastDragXPosition;
    float _lastSplitViewPosition;
    NSArray *_mailboxesToDisplayWhenVisible;
    NSToolbar *_toolbar;
    NSMutableDictionary *_toolbarItems;
    NSDictionary *_savedDefaults;
}

+ allMessageViewers;
+ existingViewerForStore:fp12;
+ existingViewerForMailboxUid:fp12;
+ (void)registerNewViewer:fp12;
+ (void)deregisterViewer:fp12;
+ (void)showAllViewers;
+ (void)updateStatusLineForAllViewers;
+ (void)saveDefaults;
+ (void)restoreFromDefaults;
- _mailboxUidsFromDefaults:fp12;
- initWithSavedDefaults:fp12;
- initWithMailboxUids:fp12;
- (void)dealloc;
- (void)_registerForApplicationNotifications;
- (void)_unregisterForApplicationNotifications;
- (void)_registerForStoreNotifications;
- (void)_unregisterForStoreNotifications;
- (void)storeBeingInvalidated:fp12;
- (void)preferencesChanged:fp12;
- (char)mailboxListingIsShowing;
- (void)_setStore:fp12;
- (char)_isViewingMailboxUid:fp12;
- _accountBeingViewed;
- messageTextView;
- window;
- (void)show;
- (void)showAndMakeKey:(char)fp12;
- (void)_setupUI;
- (void)_loadDrawerAndMessages;
- (void)_setupMailboxDrawerOutlineView;
- (void)_setupMailboxesDrawer;
- (void)_setupNextKeyViewLoop;
- (void)takeOverAsSelectionOwner;
- (void)resignAsSelectionOwner;
- (void)windowDidBecomeMain:fp12;
- (void)nowWouldBeAGoodTimeToTerminate:fp12;
- (char)windowShouldClose:fp12;
- (void)windowWillMiniaturize:fp12;
- (struct _NSSize)windowWillResize:fp16 toSize:(struct _NSSize)fp20;
- (void)splitViewDidResizeSubviews:fp12;
- (void)splitViewWillResizeSubviews:fp12;
- (void)splitViewDoubleClickedOnDivider:fp12;
- (void)splitView:fp12 resizeSubviewsWithOldSize:(struct _NSSize)fp16;
- (char)splitView:fp12 canCollapseSubview:fp16;
- (float)splitView:fp12 constrainMinCoordinate:(float)fp40 ofSubviewAt:(int)fp20;
- (float)splitView:fp12 constrainMaxCoordinate:(float)fp40 ofSubviewAt:(int)fp20;
- (void)outlineViewDoubleClick:fp12;
- (void)selectMailbox:fp12;
- (void)keyDown:fp12;
- (void)keyUp:fp12;
- (void)_mailboxWasRenamed:fp12;
- (void)mailboxSelectionChanged:fp12;
- (void)_setMailboxUids:fp12;
- (char)_selectionContainsMessagesWithDeletedStatusEqualTo:(char)fp12;
- (char)_selectionContainsMessagesWithReadStatusEqualTo:(char)fp12;
- (char)_selectionContainsMessagesWithFlaggedStatusEqualTo:(char)fp12;
- (char)_selectionContainsMessagesWithJunkMailLevelEqualTo:(int)fp12;
- (char)_selectionContainsMessagesWithAttachments;
- (char)_validateAction:(SEL)fp12 tag:(int)fp16;
- (char)validateMenuItem:fp12;
- (void)messageWasDisplayedInTextView:fp12;
- (void)checkNewMail:fp12;
- (void)replyMessage:fp12;
- (void)replyAllMessage:fp12;
- (void)showComposeWindow:fp12;
- (void)showAddressPanel:fp12;
- (void)undeleteMessages:fp12;
- (void)deleteMessages:fp12;
- (void)replyViaIM:fp12;
- tableManager;
- (void)jumpToSelection:fp12;
- (void)redirectMessage:fp12;
- (void)forwardMessage:fp12;
- (void)_replyMessageToAll:(char)fp12;
- (void)displaySelectedMessageInSeparateWindow:fp12;
- (void)transferToMailbox:fp12;
- (void)_changeFlag:fp12 state:(char)fp16 forMessages:fp20 undoActionName:fp24;
- (void)markAsRead:fp12;
- (void)markAsUnread:fp12;
- (void)markAsUnflagged:fp12;
- (void)markAsFlagged:fp12;
- (void)changeColor:fp12;
- (void)rebuildTableOfContents:fp12;
- (void)copy:fp12;
- (void)cut:fp12;
- (void)paste:fp12;
- (void)showPrintPanel:fp12;
- (char)restoreDraft:fp12;
- findTarget;
- (void)saveAs:fp12;
- (void)saveAllAttachments:fp12;
- (void)updateMenuCheckmarkForSearchScope:fp12;
- (void)searchPopupChanged:fp12;
- (void)searchIndex:fp12;
- (void)clearSearch:fp12;
- (void)searchFinished:fp12;
- (void)performSearch:fp12;
- (void)controlTextDidEndEditing:fp12;
- (void)controlTextDidChange:fp12;
- (void)sortByTagOfSender:fp12;
- (void)focus:fp12;
- (void)unfocus:fp12;
- (void)showDeletions:fp12;
- (void)hideDeletions:fp12;
- (void)toggleContentsColumn:fp12;
- (void)toggleMessageNumbersColumn:fp12;
- (void)toggleMessageFlagsColumn:fp12;
- (void)toggleFromColumn:fp12;
- (void)togglePresenceColumn:fp12;
- (void)toggleToColumn:fp12;
- (void)toggleDateSentColumn:fp12;
- (void)toggleDateReceivedColumn:fp12;
- (void)toggleLocationColumn:fp12;
- (void)toggleSizeColumn:fp12;
- (void)_showMailboxesPanel;
- (void)showMailboxesPanel:fp12;
- (void)writeDefaultsToArray:fp12;
- _saveDefaults;
- (void)_setupFromDefaults;
- _sizeStatus;
- (void)showStatusMessage:fp12;
- (void)_updateStatusLine;
- (void)_updateStatusLineAndWindowTitle;
- (void)_updateMessageMallUids:fp12;
- (void)showStatusLine:fp12;
- (void)hideStatusLine:fp12;
- (void)messageWasSelected:fp12;
- (void)tableView:fp12 willStartDragWithEvent:fp16;
- (void)draggedImage:fp12 movedTo:(struct _NSPoint)fp16;
- (void)draggingUpdatedAtLocation:(struct _NSPoint)fp12;
- (void)tableViewDidEndDragging:fp12;
- (void)transferAgain:fp12;
- (void)reapplySortingRules:fp12;
- (void)_returnToSenderSheetDidEnd:fp12 returnCode:(int)fp16 contextInfo:(void *)fp20;
- (void)returnToSender:fp12;
- (void)addSenderToAddressBook:fp12;
- (void)showMainThreadIsBusy:fp12;
- (void)showMainThreadIsNotBusy:fp12;
- (void)_performJunkMailActionOnMessages:fp12;
- (void)markAsJunkMail:fp12;
- (void)synchronouslyMarkAsJunkMail:fp12 delete:fp16;
- (void)markAsNotJunkMail:fp12;
- (void)synchronouslyMarkAsNotJunkMail:fp12;
- (void)_updateSearchItemLabel;
- (void)showSearchTypeMenu:fp12;
@end

@interface MessageViewer(MessageViewerToolbar)
- (void)_setupToolBar;
- (char)validateToolbarItem:fp12;
- toolbar:fp12 itemForItemIdentifier:fp16 willBeInsertedIntoToolbar:(char)fp20;
- (void)setupSearchView:(char)fp12;
- toolbarDefaultItemIdentifiers:fp12;
- toolbarAllowedItemIdentifiers:fp12;
- (void)toolbarDidRemoveItem:fp12;
@end
