/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailBundle.m,v 1.12 2002/11/21 03:40:37 jeffc Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGP/PGPPrivate.h>
#import <PGPclient/pgpVersionHeader.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>

#import "PGPMailBundle.h"
#import "PGPMailTextDocumentViewer.h"
#import "PGPMailMessageViewer.h"
#import "PGPMailMessageBody.h"
#import "PGPMailMessage.h"

#import <Message.h>
#import <MessageHeaders.h>
#import <MessageBody.h>
#import <TextDocumentViewer.h>

#import "PGPMailMessageEditor.h"
#import "PGPMailPreferences.h"
#import "PGPMailTableViewManager.h"
#import "PGPMailProgress.h"

#import "PGPMailCommon.h"
#import <AppKit/AppKit.h>

#define kPGPmailComposeAccessoryNib			@"PGPMailCompose"
#define kPGPmailViewerNib					@"PGPMailMenu"			

#define kPGPmailMessageViewer				@"MessageViewer"
#define kPGPmailTornOffViewer				@"TornOffViewer"
#define kPGPmailComposeNewOrDraft			@"ComposeNewOrDraft"
#define kPGPmailComposeReplyOrForward		@"ComposeReplyOrForward"

#define kPGPmailMessage						@"Message"
#define kPGPmailMessageKey					@"MessageKey"
#define kPGPmailIsPGPMessage				@"isPGPMessage"

#define kPGPmailViewer						@"viewer"

#define kPGPMailViewerPreferencesName		@"PGPMailViewerPreferences"

#define kPGPmailNoDelegate					LSPV(@"PGPmail: toolbar %@ has no delegate!", @"TOOLBAR_HAS_NO_DELEGATE")
#define kPGPmailMessageDecryptVerify		LSPV(@"PGPmail: Decrypting/Verifying", @"MESSAGE_DECRYPTING_VERIFYING")
#define kPGPmailMessageDone					LSPV(@"PGPmail: Done", @"MESSAGE_DONE")
#define kPGPmailMessageBlank				LSPV(@" ", @"MESSAGE_BLANK")
#define kPGPmailUnableToAddMenu				LSPV(@"Unable to add PGP menu", @"UNABLE_TO_ADD_PGP_MENU")
#define kPGPmailUnableToLoadKeyDB			LSPV(@"%s:%d: Warning: could not load main KeyDB(PGPerror=%d).", @"UNABLE_TO_LOAD_KEYDB")

static PGPMailBundle			*sAppController = nil;

@interface NSObject(PGPMailBundle)
- (void)mailTo:(NSPasteboard *)pasteboard userData:(NSString *)userData error:(NSString **)error;
@end

@interface NSApplication(PGPMailBundle_Revealed)
- (NSArray *)messageEditors;
@end

@interface PGPMailBundle(Private)
- (void)pgpDecryptVerifyMessage:(Message *)inMessage inViewer:(TextDocumentViewer *)inViewer;
@end

@implementation PGPMailBundle
+ (void)initialize
{
    NSBundle	*theBundle;

    [super initialize];
    
    theBundle = [NSBundle bundleForClass:self];
    
	[(NSImage *)[[NSImage alloc] initByReferencingFile:[theBundle pathForImageResource:kPGPMailEncryptedIconFile]] setName:kPGPMailEncryptedIconName];
	[(NSImage *)[[NSImage alloc] initByReferencingFile:[theBundle pathForImageResource:kPGPMailClearIconFile]] setName:kPGPMailClearIconName];
	[(NSImage *)[[NSImage alloc] initByReferencingFile:[theBundle pathForImageResource:kPGPMailSignedIconFile]] setName:kPGPMailSignedIconName];
	[(NSImage *)[[NSImage alloc] initByReferencingFile:[theBundle pathForImageResource:kPGPMailUnsignedIconFile]] setName:kPGPMailUnsignedIconName];
	[(NSImage *)[[NSImage alloc] initByReferencingFile:[theBundle pathForImageResource:kPGPMailIconFile]] setName:kPGPMailIconName];
	[(NSImage *)[[NSImage alloc] initByReferencingFile:[theBundle pathForImageResource:kPGPMailDecryptVerifyFile]] setName:kPGPMailDecryptVerifyIconName];
        
    [self registerBundle];
        
    NSLog(@"Loaded %@", [self pgpVersion]);
}

+ (BOOL)hasPreferencesPanel
{
    return NO;
}

+ (PGPMailBundle *)pgpController
{
    return sAppController;
}

+ (NSString *)pgpVersion
{
    return [NSString stringWithCString:pgpVersionHeaderString()];
}

- (NSMenuItem *)newMenuItemWithTitle:(NSString *)title action:(SEL)action andKeyEquivalent:(NSString *)keyEquivalent inMenu:(NSMenu *)menu underItemWithSelector:(SEL)selector
{
    NSMenuItem		*theMenuItem = nil;
    NSArray 		*theItems = [menu itemArray];
    int				index = 0;
    
    if( !keyEquivalent )
        keyEquivalent = @"";
    
    for( index = 0; index < [theItems count]; index++ )
    {
        theMenuItem = [theItems objectAtIndex:index];
        
        if( [theMenuItem action] == selector )
            return [menu insertItemWithTitle:title action:action keyEquivalent:keyEquivalent atIndex:index+1];
        else if( [[theMenuItem target] isKindOfClass:[NSMenu class]] )
        {
            theMenuItem = [self newMenuItemWithTitle:title action:action andKeyEquivalent:keyEquivalent
                    inMenu:[theMenuItem target] underItemWithSelector:selector];
            if( theMenuItem )
                return theMenuItem;
        }
    }
    
    return nil;
}

- (void)setPGPMenu:(NSMenu *)inMenu
{
    NSMenuItem 	*theMenuItem = [self newMenuItemWithTitle:@"PGP" action:NULL andKeyEquivalent:@"" inMenu:[[NSApplication sharedApplication] mainMenu] underItemWithSelector:@selector(reapplySortingRules:)];
    
    if( !theMenuItem )
        NSLog( @"-[%@ %@]: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), kPGPmailUnableToAddMenu );
    else
    {
        [[theMenuItem menu] insertItem:[NSMenuItem separatorItem] atIndex:[[theMenuItem menu] indexOfItem:theMenuItem]];
        [[theMenuItem menu] setSubmenu:inMenu forItem:theMenuItem];
        [mEncryptMenuItem setState:([self pgpEncryptNewMessagesByDefault] ? NSOnState:NSOffState)];
        [mSignMenuItem setState:([self pgpSignNewMessagesByDefault] ? NSOnState:NSOffState)];
    }
}

- (id)realDelegateForToolbar:(NSToolbar *)toolbar
{
    if(![[toolbar delegate] isKindOfClass:[MVMailBundle class]])
    {
        [mRealToolbarDelegates removeObjectForKey:[NSValue valueWithNonretainedObject:toolbar]];
        return nil;
    }
    else
        return [[mRealToolbarDelegates objectForKey:[NSValue valueWithNonretainedObject:toolbar]] nonretainedObjectValue];
}

- (void)addAdditionalContextualMenuItemsToMessageViewer:(MessageViewer *)viewer
{
    NSMenu		*theMenu = [[viewer pgpTableManager] pgpContextualMenu];
    NSMenu		*pgpMenu = [[self pgpEncryptMenuItem] menu];
    NSMenuItem	*newMenuItem = nil;

    newMenuItem = [pgpMenu itemAtIndex:0];
    if( -1 == [theMenu indexOfItemWithTitle:[newMenuItem title]] )
    {
        newMenuItem = [NSMenuItem separatorItem];
        [theMenu addItem:newMenuItem];
        newMenuItem = [[pgpMenu itemAtIndex:0] copyWithZone:[theMenu zone]];
        [newMenuItem setKeyEquivalent:@""];
        [theMenu addItem:newMenuItem];
        [newMenuItem release];
    }
}

- (id)usurpToolbarDelegate:(NSToolbar *)toolbar
{
	NSArray		*theIdentifers = [mAdditionalToolbarItemIdentifiers objectForKey:[toolbar identifier]];
    id			theDelegate = [toolbar delegate];
    
    if( nil == theDelegate )
    {
        NSLog( kPGPmailNoDelegate, [toolbar identifier] );
        return nil;
    }
    
	if(theIdentifers != nil && [theIdentifers count] > 0)
    {
//        NSEnumerator	*theEnum = [theIdentifers objectEnumerator];
//        NSString		*theIdentifier = nil;

        if([theDelegate isKindOfClass:[MessageViewer class]])
            [self addAdditionalContextualMenuItemsToMessageViewer:theDelegate];
        else if([theDelegate respondsToSelector:@selector(realDelegateForToolbar:)]){
            id	usurpedDelegate = [theDelegate realDelegateForToolbar:toolbar];

            if([usurpedDelegate isKindOfClass:[MessageViewer class]])
                [self addAdditionalContextualMenuItemsToMessageViewer:usurpedDelegate];
        }
        [mRealToolbarDelegates setObject:[NSValue valueWithNonretainedObject:theDelegate] forKey:[NSValue valueWithNonretainedObject:toolbar]];
        [toolbar setDelegate:self];
#if 0        
        while(theIdentifier = [theEnum nextObject])
        {
            NSString	*tmpString = [[NSUserDefaults standardUserDefaults] stringForKey:[theIdentifier stringByAppendingFormat:@".%@", [toolbar identifier]]];
            
            if(tmpString != nil)
            {
                NSArray		*items = [toolbar items];
                int			index = [tmpString intValue];
                int			maxIndex = [items count];
                
                if(index > maxIndex)
                    index = maxIndex;

                if(index == maxIndex || ![[[items objectAtIndex:index] itemIdentifier] isEqualToString:theIdentifier])
                {
                    NSEnumerator	*itemEnum = [items objectEnumerator];
                    NSToolbarItem	*theItem;
                    BOOL			add = YES;
                    
                    while(theItem = [itemEnum nextObject])
                    {
                        if([[theItem itemIdentifier] isEqualToString:theIdentifier])
                        {
                            add = NO;
                            break;
                        }
                    }
                    if(add)
                        [toolbar insertItemWithItemIdentifier:theIdentifier atIndex:index];
                }
            }
        }
#endif
        return theDelegate;
    }
    else
        return nil;
}

- (NSToolbarItem *)createToolbarItemWithItemIdentifier:(NSString *)itemIdentifier label:(NSString *)label tooltip:(NSString *)tooltip target:(id)target action:(SEL)action imageNamed:(NSString *)imageName
{
    NSToolbarItem	*theItem = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
        
    [theItem setLabel:label];
    [theItem setPaletteLabel:[theItem label]];
    [theItem setToolTip:tooltip];
    [theItem setTarget:target];
    [theItem setAction:action];
    [theItem setImage:[NSImage imageNamed:imageName]];

    return [theItem autorelease];
}

- (BOOL)itemForItemIdentifier:(NSString *)itemIdentifier alreadyInToolbar:(NSToolbar *)toolbar
{
    NSEnumerator	*theEnum = [[toolbar items] objectEnumerator];
    NSToolbarItem	*theItem = nil;
    
    while( theItem = [theEnum nextObject] )
        if( [[theItem itemIdentifier] isEqualToString:itemIdentifier] && ![theItem allowsDuplicatesInToolbar] )
            return YES;
    return NO;
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier
    willBeInsertedIntoToolbar:(BOOL)flag
{
	if( [itemIdentifier isEqualToString:kPGPToolbarIdentifierDecryptVerify] )
        return [self createToolbarItemWithItemIdentifier:itemIdentifier label:kPGPToolbarItemDecryptVerify tooltip:kPGPToolbarItemTooltipDecryptVerify target:self action:@selector(pgpDecryptVerifyMessage:) imageNamed:kPGPMailDecryptVerifyIconName];
    else if([itemIdentifier isEqualToString:kPGPToolbarIdentifierEncrypt])
    {
        NSToolbarItem	*newItem = [self createToolbarItemWithItemIdentifier:itemIdentifier label:kPGPToolbarItemEncrypted tooltip:kPGPToolbarItemTooltipEncrypted target:self action:@selector(pgpToggleEncryptionForNewMessage:) imageNamed:kPGPMailEncryptedIconName];
        return newItem;
    }
    else if([itemIdentifier isEqualToString:kPGPToolbarIdentifierSign])
    {
        NSToolbarItem	*newItem = [self createToolbarItemWithItemIdentifier:itemIdentifier label:kPGPToolbarItemSigned tooltip:kPGPToolbarItemTooltipSigned target:self action:@selector(pgpToggleSignatureForNewMessage:) imageNamed:kPGPMailSignedIconName];
        return newItem;
    }
    else
        return [[self realDelegateForToolbar:toolbar] toolbar:toolbar itemForItemIdentifier:itemIdentifier willBeInsertedIntoToolbar:flag];
}
    
- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar
{
	return [[self realDelegateForToolbar:toolbar] toolbarDefaultItemIdentifiers:toolbar];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar
{
    NSArray		*theIdentifiers = [mAdditionalToolbarItemIdentifiers objectForKey:[toolbar identifier]];

    if( nil != theIdentifiers )
    {
        NSEnumerator	*theEnum = [theIdentifiers objectEnumerator];
        NSString		*theIdentifier = nil;
        NSMutableArray	*identifiers = [NSMutableArray arrayWithArray: [[self realDelegateForToolbar:toolbar] toolbarAllowedItemIdentifiers:toolbar]];
        
        while(theIdentifier = [theEnum nextObject])
            [identifiers addObject:theIdentifier];
            
        return identifiers;
    }
    else
        return [[self realDelegateForToolbar:toolbar] toolbarAllowedItemIdentifiers:toolbar];
}
#if 0
- (void)toolbarDidAddItem:(NSNotification *)notification
{
    NSToolbar	*toolbar = [notification object];
    NSArray		*items = [toolbar items];
    int			index = 0;
    NSArray		*theIdentifers = [mAdditionalToolbarItemIdentifiers objectForKey:[toolbar identifier]];

    for(index = [items count] - 1; index >= 0; index--)
    {
        NSString	*itemIdentifier = [[items objectAtIndex:index] itemIdentifier];
        
        if([theIdentifers containsObject:itemIdentifier])
        {
            [[NSUserDefaults standardUserDefaults] setInteger:index forKey:[itemIdentifier stringByAppendingFormat:@".%@", [toolbar identifier]]];
            break;
        }
    }
}
#endif
- (void)toolbarWillAddItem:(NSNotification *)notification notifyRealDelegate:(BOOL)notifyRealDelegate
{
    NSToolbar		*theToolbar = [notification object];
    
    if( notifyRealDelegate )
    {
        id	realDelegate = [self realDelegateForToolbar:theToolbar];
        
        if( [realDelegate respondsToSelector:@selector(toolbarWillAddItem:)] )
            [realDelegate performSelector:@selector(toolbarWillAddItem:) withObject:notification];
    }
}

- (void)anyToolbarWillAddItem:(NSNotification *)notification
{
    NSToolbar		*theToolbar = [notification object];
    id				realDelegate = [self realDelegateForToolbar:theToolbar];

    if(realDelegate == nil)
    {
        realDelegate = [self usurpToolbarDelegate:theToolbar];
        if(realDelegate != nil)
            [self toolbarWillAddItem:notification notifyRealDelegate:NO];
    }
}

- (void)toolbarWillAddItem:(NSNotification *)notification
{
	[self toolbarWillAddItem:notification notifyRealDelegate:YES];
}

- (void) toolbarDidRemoveItem: (NSNotification *)notification
{
    NSToolbar	*toolbar = [notification object];
    id		realDelegate = [self realDelegateForToolbar:toolbar];

    
    if([realDelegate respondsToSelector:_cmd])
        [realDelegate performSelector:_cmd withObject:notification];
}

- (id)init
{
	PGPInt32		days;
	
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }

    [[NSNotificationCenter defaultCenter] addObserver:self
        selector:@selector(applicationWillTerminate:)name:NSApplicationWillTerminateNotification
        object:[NSApplication sharedApplication]];
    
    [self pgpInitSDK];
    [self pgpInitPrefs];
	
	PGPclLicenseCheck(mPGPContext);
	
	days = PGPclLicenseDaysLeft();
	
	if ((PGPclIsAuthorized() && PGPclPGPmailEnabled()) || (PGPclPGPmailEnabled() && ((days + PGP_LICENSEGRACEPERIOD) > 0)))
	{
		if( !mLoadedNib )
			[self initWithNib:kPGPmailViewerNib];
    
		mCleanupDone = NO;
	}

{
            NSEnumerator	*anEnum = [[MessageViewer allMessageViewers] objectEnumerator];
            MessageViewer	*aViewer;
			
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(anyToolbarWillAddItem:) name:NSToolbarWillAddItemNotification object:nil];
            while(aViewer = [anEnum nextObject])
                [self usurpToolbarDelegate:[aViewer pgpToolbar]];
}	
	mDecodeError = kPGPError_NoErr;
	
	return self;
}

- (id)initWithNib:(NSString *)inNibFile
{
    if( inNibFile && [inNibFile length] )
        [NSBundle loadNibNamed:inNibFile owner:self];
	else
		NSLog( kPGPmailBadNibFile, __FILE__, __LINE__ );

    mRealToolbarDelegates = [[NSMutableDictionary allocWithZone:[self zone]] init];
    mAdditionalToolbarItemIdentifiers = [[NSDictionary allocWithZone:[self zone]] initWithObjectsAndKeys:
        [NSArray arrayWithObjects:kPGPToolbarIdentifierDecryptVerify, nil], kPGPmailMessageViewer,
        [NSArray arrayWithObjects:kPGPToolbarIdentifierDecryptVerify, nil], kPGPmailTornOffViewer,
        [NSArray arrayWithObjects:kPGPToolbarIdentifierEncrypt, kPGPToolbarIdentifierSign, nil], kPGPmailComposeNewOrDraft,
        [NSArray arrayWithObjects:kPGPToolbarIdentifierEncrypt, kPGPToolbarIdentifierSign, nil], kPGPmailComposeReplyOrForward,  nil];

	mLoadedNib = YES;
    mCleanupDone = NO;
    return [self init];
}

- (void)dealloc
{
    [mRealToolbarDelegates release];
    [mAdditionalToolbarItemIdentifiers release];

    [super dealloc];
}

- (NSString *)pgpVersion
{
    return [[self class] pgpVersion];
}

+ (BOOL)hasComposeAccessoryViewOwner
{
	return YES;
}

+ (NSString *)composeAccessoryViewOwnerClassName
{
    return @"PGPMailComposeAccessoryViewOwner";
}

- (void)pgpInitSDK
{
	if( !mSDKInited )
	{
        PGPError		theError = kPGPError_NoErr;

		theError = PGPsdkInit(0);
		pgpAssertNoErr(theError);
		theError = PGPsdkUILibInit(0);
		pgpAssertNoErr(theError);
		theError = PGPNewContext(kPGPsdkAPIVersion, &mPGPContext);
		pgpAssertNoErr(theError);
		theError = PGPKeyServerInit();
		pgpAssertNoErr(theError);
		theError = PGPNewTLSContext(mPGPContext, &mTLSContext);
		pgpAssertNoErr(theError);
		mMainKeyDB = kInvalidPGPKeyDBRef;
        
        mSDKInited = YES;
    }
}

- (void)pgpInitPrefs
{
    if( !mPrefsInited )
    {
        mPrefs = [[PGPMailPreferences alloc] initWithController:self];
        mPrefsInited = YES;
    }
}

- (void)pgpCleanUp
{
    if( mPrefsInited )
        [mPrefs release];
	
	PGPclLicenseFree(mPGPContext);
	
    if( mPGPContext )
    {
        if( PGPKeyDBRefIsValid(mMainKeyDB) )
        {
            PGPFreeKeyDB(mMainKeyDB);
            mMainKeyDB = kInvalidPGPKeyDBRef;
        }

        if( PGPtlsContextRefIsValid(mTLSContext) )
        {
            PGPFreeTLSContext(mTLSContext);
            mTLSContext = kInvalidPGPtlsContextRef;
        }

        if( PGPContextRefIsValid(mPGPContext) )
        {
            PGPFreeContext(mPGPContext);
            mPGPContext = kInvalidPGPContextRef;
        }

        PGPKeyServerCleanup();
        PGPsdkCleanup();
    }
}

- (PGPMailPreferences *)pgpPrefsManager
{
    return mPrefs;
}

- (PGPContextRef)pgpContext
{
    return mPGPContext;
}

- (PGPtlsContextRef)pgpTLSContext
{
    return mTLSContext;
}

- (PGPPrefRef)pgpPrefRef
{
    return [mPrefs pgpPrefRef];
}

- (PGPKeyDBRef)pgpMainKeyDB
{
	if( !PGPKeyDBRefIsValid( mMainKeyDB ) )
	{
		PGPError	theError = kPGPError_NoErr;

		theError = PGPclGetPath([self pgpPrefRef], kInvalidPGPPrefRef, kPGPclPublicKeyringFile, mPubKeyringPath, PATH_MAX);
		theError = PGPclGetPath([self pgpPrefRef], kInvalidPGPPrefRef, kPGPclPrivateKeyringFile, mSecKeyringPath, PATH_MAX);

		theError = PGPclOpenDefaultKeyrings( mPGPContext, [self pgpPrefRef], kPGPOpenKeyDBFileOptions_Mutable, &mMainKeyDB );
		if( !PGPKeyDBRefIsValid( mMainKeyDB ) )
			PGPLog( (kPGPmailUnableToLoadKeyDB, __FILE__, __LINE__, theError) );
	}
	return mMainKeyDB;
}

- (void)applicationWillTerminate:(NSNotification *)notification
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:[notification name]
            object:[notification object]];
    [self pgpCleanUp];
}

- (NSMenuItem *)pgpSignMenuItem
{
    return mSignMenuItem;
}

- (NSMenuItem *)pgpEncryptMenuItem
{
    return mEncryptMenuItem;
}

- (NSMenuItem *)pgpDecryptVerifyMenuItem
{
    return mDecryptVerifyMenuItem;
}

- (void)pgpSetEncryptNewMessagesByDefault:(BOOL)inFlag
{
    if( ![mEncryptMenuItem isEnabled] )
        [mEncryptMenuItem setState:(inFlag ? NSOnState : NSOffState)];
}

- (BOOL)pgpEncryptNewMessagesByDefault
{
    return [mPrefs pgpEncryptNewMessagesByDefault]?YES:NO;
}

- (void)pgpSetSignNewMessagesByDefault:(BOOL)inFlag
{
    if( ![mSignMenuItem isEnabled] )
        [mSignMenuItem setState:(inFlag ? NSOnState : NSOffState)];
}

- (BOOL)pgpSignNewMessagesByDefault
{
    return [mPrefs pgpSignNewMessagesByDefault]?YES:NO;
}

- (BOOL)pgpUsePGPMIMEWhenSendingMail
{
    return [mPrefs pgpUsePGPMimeWhenSendingMail]?YES:NO;
}

- (BOOL)pgpAutomaticallyDecryptVerifyMessages
{
    return [mPrefs pgpAutoDecryptVerifyWhenOpeningMessges]?YES:NO;
}

- (BOOL)pgpUsesEncapsulatedSignature
{
    return [mPrefs pgpUsesEncapsulatedSignature]?YES:NO;
}

- (void)mailTo:(id)sender
{
    NSString 		*theError = nil;
    NSPasteboard	*pBoard = [NSPasteboard pasteboardWithUniqueName];
    NSArray			*pbTypes = [NSArray arrayWithObject:NSStringPboardType];
    id				serviceProvider = [NSApplication sharedApplication];
    
    (void)[pBoard declareTypes:pbTypes owner:nil];
    (void)[pBoard addTypes:pbTypes owner:nil];
    (void)[pBoard setString:@"pgp@pgp.com" forType:NSStringPboardType];
    
    if([serviceProvider respondsToSelector:@selector(mailTo:userData:error:)])
        [serviceProvider mailTo:pBoard userData:nil error:&theError];
    if( theError )
        NSBeep();
}

- (IBAction)pgpToggleEncryptionForNewMessage:(id)sender
{
    id		messageEditor = [[NSApplication sharedApplication] targetForAction:_cmd];

    if( messageEditor && [messageEditor respondsToSelector:_cmd] )
        [messageEditor performSelector:_cmd withObject:sender];
}

- (IBAction)pgpToggleSignatureForNewMessage:(id)sender
{
    id		messageEditor = [[NSApplication sharedApplication] targetForAction:_cmd];
    
    if( messageEditor && [messageEditor respondsToSelector:_cmd] )
        [messageEditor performSelector:_cmd withObject:sender];
}

- (IBAction)pgpDecryptVerifyMessage:(id)sender
{
    MessageViewer		*messageViewer = [[NSApplication sharedApplication] targetForAction:@selector(pgpTextViewer:)];
    TextDocumentViewer	*theViewer = [messageViewer pgpTextViewer:nil];
  
    [self pgpDecryptVerifyMessage:[theViewer pgpMessage] inViewer:theViewer];
}

- (Message *)targetMessageForToolbarItem:(NSToolbarItem *)item
{
    if(item == nil)
    {
        MessageViewer		*messageViewer = [[NSApplication sharedApplication] targetForAction:@selector(pgpTextViewer:)];
        TextDocumentViewer	*viewer = [messageViewer pgpTextViewer:nil];
        
        return [viewer pgpMessage];
    }
    else
    {
        NSEnumerator	*theEnum = [[MessageViewer allMessageViewers] objectEnumerator];
        MessageViewer	*aViewer;
        MessageEditor	*anEditor;
        
        while(aViewer = [theEnum nextObject]){
            NSToolbar	*aToolbar = [aViewer pgpToolbar];
            
            if([[aToolbar items] containsObject:item])
                return [[aViewer pgpTextViewer:nil] pgpMessage];
        }

        theEnum = [(NSArray *)[NSApp messageEditors] objectEnumerator];
        while(anEditor = [theEnum nextObject]){
            NSToolbar	*aToolbar = [anEditor pgpToolbar];
            
            if([[aToolbar items] containsObject:item])
                return [[anEditor pgpTextViewer:nil] pgpMessage];
        }
    }
    
    return nil;
}

- (BOOL)_validateAction:(SEL)theAction toolbarItem:(NSToolbarItem *)item
{
    if(theAction == @selector(pgpDecryptVerifyMessage:))
    {
        Message	*theMessage = [self targetMessageForToolbarItem:item];

        if(theMessage != nil)
        {
            NSMutableDictionary	*theDictionary = [[item menuFormRepresentation] representedObject];
            Message				*lastDisplayedMessage = [[theDictionary objectForKey:kPGPmailMessage] nonretainedObjectValue];
            NSNumber			*aNumber = [theDictionary objectForKey:kPGPmailIsPGPMessage];
                
            if(!aNumber || lastDisplayedMessage != theMessage)
            {
                if(!theDictionary)
                    theDictionary = [NSMutableDictionary dictionary];
                [theDictionary setObject:[NSValue valueWithNonretainedObject:theMessage] forKey:kPGPmailMessage];
                aNumber = [NSNumber numberWithBool:[theMessage isPGPMessage]];
                [theDictionary setObject:aNumber forKey:kPGPmailIsPGPMessage];
                [[item menuFormRepresentation] setRepresentedObject:theDictionary];
            }
            return [aNumber boolValue];
        }
        else
            return NO;
    }
    else if(theAction == @selector(pgpToggleEncryptionForNewMessage:) || theAction == @selector(pgpToggleSignatureForNewMessage:))
    {
        id	messageEditor = [[NSApplication sharedApplication] targetForAction:theAction];
        
        if(messageEditor != nil)
            return ([messageEditor respondsToSelector:@selector(pgpIsRealEditor)] && [messageEditor pgpIsRealEditor]);
        else
            return NO;
    }

    return NO;
}

- (BOOL)validateMenuItem:(NSMenuItem *)inItem
{
    return [self _validateAction:[inItem action] toolbarItem:nil];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)inItem
{
    return [self _validateAction:[inItem action] toolbarItem:inItem];
}

- (void)_messageHasBeenDisplayed:(NSNotification *)notification
{
	static Message *	sLastMessage = nil;
    Message		*		theMessage = [[notification userInfo] objectForKey:kPGPmailMessageKey];

    [[NSNotificationCenter defaultCenter] removeObserver:self name:[notification name] object:nil];
    
    if([self pgpAutomaticallyDecryptVerifyMessages] && (IsntPGPError(mDecodeError) || (sLastMessage != theMessage))) {
        [self pgpDecryptVerifyMessage:theMessage inViewer:[notification object]];
	}
	sLastMessage = theMessage;
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(messageWillBeDisplayedInView:) name:[notification name] object:nil];
}

- (void)messageWillBeDisplayedInView:(NSNotification *)notification
{
    [self performSelector:@selector(_messageHasBeenDisplayed:) withObject:notification afterDelay:0.0];
}

- (void)sheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSDictionary	*theDict = (NSDictionary *)contextInfo;
    
    if( NSAlertDefaultReturn == returnCode )
        [self pgpDecryptVerifyMessage:[theDict objectForKey:kPGPmailMessage] inViewer:[theDict objectForKey:kPGPmailViewer]];
            
    [theDict release];
}

- (void)progressIndicatorDidCancel:(PGPMailProgress *)controller
{
    [PGPMailEncodeDecode pgpCancelDecodeOperation];
}

- (void)pgpDecryptVerifyMessage:(Message *)inMessage inViewer:(TextDocumentViewer *)inViewer
{
    PGPMailDecodeOptions		theDecodeOptions = PGPMakeDefaultDecodeOptions();

    if( [inMessage isPGPMessage:&theDecodeOptions] )
    {
        NSException				*theException = nil;
        
        if( [[[[inViewer textView] window] delegate] respondsToSelector:@selector(showStatusMessage:)] )
            [[[[inViewer textView] window] delegate] showStatusMessage:kPGPmailMessageDecryptVerify];
        
        NS_DURING
            Message		*decodedMessage = [inMessage pgpDecryptVerifyMessage:&theDecodeOptions];

            if( nil != decodedMessage )
            {
                [inViewer fadeToEmpty];
                [inViewer setMessage:decodedMessage];
            }
        NS_HANDLER
            theException = localException;
        NS_ENDHANDLER
        
        if( [[[[inViewer textView] window] delegate] respondsToSelector:@selector(showStatusMessage:)] )
            [[[[inViewer textView] window] delegate] showStatusMessage:kPGPmailMessageBlank];
        
        if( nil == theException )
            [inViewer reloadCurrentMessage];
    }
	mDecodeError = theDecodeOptions.mDecodeError;
}

@end
