/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailCommon.h,v 1.15 2002/10/23 09:52:07 jason Exp $
____________________________________________________________________________*/

#import <Foundation/NSObject.h>
#import <CoreFoundation/CFString.h>
#import "PGPErrors.h"

#ifndef LSPV
#define LSPV(key, comment) \
	    [[NSBundle bundleForClass:[PGPMailBundle class]] localizedStringForKey:(key) value:@"" table:(nil)]
#endif

typedef enum
{
    PGPNoSignature,
    PGPInlineSignature,
    PGPDetachedSignature,
    PGPEncapsulatedSignature
}PGPSignatureType;

typedef struct PGPMailEncodeOptions
{
    BOOL				mSign;
    BOOL				mEncrypt;
    BOOL				mUseMIME;
    BOOL				mDetachedSignature;
    BOOL				mEncapsulatedSignature;
	BOOL				mFlowed;
	PGPError			mEncodeError;
    CFStringEncoding	mStringEncoding;
    PGPSignatureType	mSignatureType;
} PGPMailEncodeOptions;

typedef struct PGPMailDecodeOptions
{
    BOOL				mIsSigned;
    BOOL				mIsEncrypted;
    BOOL				mIsOpenPGPSigned;
    BOOL				mIsOpenPGPEncrypted;
	BOOL				mFlowed;
	PGPError			mDecodeError;
    CFStringEncoding	mStringEncoding;
    NSString			*mSignature;
} PGPMailDecodeOptions;

#define kPGPMailIconFile					@"PGPMail"
#define kPGPMailIconName					@"pgpMailIcon"
#define kPGPMailEncryptedIconFile			@"pgpencrypted"
#define kPGPMailEncryptedIconName			@"pgpEncryptedIcon"
#define kPGPMailClearIconFile				@"pgpclear"
#define kPGPMailClearIconName				@"pgpClearIcon"
#define kPGPMailSignedIconFile				@"pgpsigned"
#define kPGPMailSignedIconName				@"pgpSignedIcon"
#define kPGPMailUnsignedIconFile			@"pgpunsigned"
#define kPGPMailUnsignedIconName			@"pgpUnsignedIcon"
#define kPGPMailDecryptVerifyFile			@"pgpdecryptverify"
#define kPGPMailDecryptVerifyIconName		@"pgpDecryptVerifyIcon"

#define kPGPToolbarIdentifierSign			@"PGPSignMessage"
#define kPGPToolbarIdentifierEncrypt		@"PGPEncryptMessage"
#define kPGPToolbarIdentifierDecryptVerify	@"PGPDecryptVerifyMessage"

#define kPGPToolbarItemSigned				LSPV(@"Signed", @"TOOLBAR_ITEM_SIGNED")
#define kPGPToolbarItemUnsigned				LSPV(@"Unsigned", @"TOOLBAR_ITEM_UNSIGNED")
#define kPGPToolbarItemEncrypted			LSPV(@"Encrypted", @"TOOLBAR_ITEM_ENCRYPT")
#define kPGPToolbarItemClear				LSPV(@"Unencrypted", @"TOOLBAR_ITEM_CLEAR")
#define kPGPToolbarItemDecryptVerify		LSPV(@"Decrypt/Verify", @"TOOLBAR_ITEM_DECRYPT_VERIFY")

#define kPGPToolbarItemTooltipSigned		LSPV(@"The message will be PGP signed", @"TOOLTIP_SIGNED")
#define kPGPToolbarItemTooltipUnsigned		LSPV(@"The message will not be PGP signed", @"TOOLTIP_UNSIGNED")
#define kPGPToolbarItemTooltipEncrypted		LSPV(@"The message will be PGP encrypted.", @"TOOLTIP_ENCRYPTED")
#define kPGPToolbarItemTooltipClear			LSPV(@"The message will not be PGP encrypted.", @"TOOLTIP_CLEAR")
#define kPGPToolbarItemTooltipDecryptVerify	LSPV(@"PGP Decrypt/Verify", @"TOOLTIP_DECRYPT_VERIFY")

#define kPGPmailBadNibFile					LSPV(@"%s:%d: Bad nibFile specified", @"BAD_NIB_FILE")
#define kUtilImportKeysStr					LSPV(@"Please select the keys you wish to import:", @"MESSAGE_IMPORT_KEYS")
#define kPGPmailUnexpectedCase				LSPV(@"PGPmail -[%@ %@]: Unexpected case.", @"UNEXPECTED_CASE")
#define kPGPmailDataMissing					LSPV(@"-[%@ %@] No data to encode", @"NO_DATA_TO_ENCODE")

#define kPGPmailSigLabelBeginStr			LSPV(@"*** PGP SIGNATURE VERIFICATION ***\n", @"PGP_SIGNATURE_BEGIN_STRING")
#define kPGPmailSigBeginVerifiedStr			LSPV(@"*** BEGIN PGP VERIFIED MESSAGE ***\n", @"Beging Verified")
#define kPGPmailSigEndVerifiedStr			LSPV(@"\n*** END PGP VERIFIED MESSAGE ***\n", @"End Verified")
#define kPGPmailSigBeginDecryptStr			LSPV(@"*** BEGIN PGP DECRYPTED/VERIFIED MESSAGE ***\n", @"Begin Decrypted")
#define kPGPmailSigEndDecryptStr			LSPV(@"\n*** END PGP DECRYPTED/VERIFIED MESSAGE ***\n", @"End Decrypted")
#define kPGPmailCancelButton				LSPV(@"Cancel", @"PGP_CANCEL_BUTTON")
#define kPGPmailOKButton					LSPV(@"OK", @"PGP_OK_BUTTON")

#define kPGPmailUtilSignatureWarningStr		LSPV(@"Signature Warning", @"Signature Warning")
#define kPGPmailUtilBadSignatureStr			LSPV(@"The data in at least one section of this message does not match the data that was signed.", @"PGP_BAD_SIGNATURE_STRING")
#define kPGPmailUtilSignatureNotVerifiedStr	LSPV(@"Unable to verify signature on at least one section of this message.", @"PGP_SIGNATURE_NOT_VERIFIED")
#define kPGPmailUtilDecodeErrorStr			LSPV(@"Decode Error", @"Decode Error")
#define kPGPmailUtilNotPlaintextStr			LSPV(@"PGP is unable to decode the message; perhaps because the text is formatted.", @"Not plain text error message")


#define kPGPmailHeaderTo					@"to"
#define kPGPmailHeaderCC					@"cc"
#define kPGPmailHeaderBcc					@"bcc"
#define kPGPmailHeaderPGPPrefix				@"pgp-"
#define kPGPmailHeaderApplication			@"application"
#define kPGPmailHeaderQuotedPrintable		@"quoted-printable"
#define kPGPmailHeaderMultipart				@"multipart"
#define kPGPmailHeaderBoundry				@"boundary"
#define kPGPmailHeaderProtocol				@"protocol"
#define kPGPmailHeaderSigned				@"signed"
#define kPGPmailHeaderMixed					@"mixed"
#define kPGPmailHeaderAlternative			@"alternative"
#define kPGPmailHeaderPGPSignature			@"pgp-signature"
#define KPGPmailHeaderPGPAppSignature		@"application/pgp-signature"
#define kPGPmailHeaderEncrypted				@"encrypted"
#define kPGPmailHeaderPGPEncrypted			@"pgp-encrypted"
#define kPGPmailHeaderPGPAppEncrypted		@"application/pgp-encrypted"
#define kPGPmailHeaderText					@"text"
#define kPGPmailHeaderPlain					@"plain"
#define kPGPmailHeaderName					@"name"
#define kPGPmailHeaderSigName				@"PGP.sig"
#define kPGPmailHeaderOctetStream			@"octet-stream"
#define kPGPmailHeaderCharset				@"charset"
#define kPGPmailHeaderVersion				@"version: 1"
#define kPGPmailHeaderMIMEVersion			@"mime-version"
#define kPGPmailHeader7Bit					@"7bit"
#define kPGPmailHeaderContentType			@"content-type"
#define kPGPmailHeaderTransferEncoding		@"content-transfer-encoding"
#define kPGPmailHeaderXMacType				@"x-mac-type"
#define kPGPmailHeaderAlgorithm				@"micalg"
#define kPGPmailHeaderBase64				@"base64"
#define kPGPmailHeaderContentDisposition	@"content-disposition"
#define kPGPmailHeaderInline				@"inline"
#define kPGPmailHeaderFormatIsFlowed		@"format=flowed"
#define kPGPmailHeaderFormatIsFixed			@"format=fixed"
#define kPGPmailHeaderXPGPRFC2646Fix		@"X-PGP-RFC2646-Fix"

PGPMailEncodeOptions PGPMakeDefaultEncodeOptions( void );
PGPMailDecodeOptions PGPMakeDefaultDecodeOptions( void );
