/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailComposeAccessoryViewOwner.m,v 1.7 2002/10/23 09:52:07 jason Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>

#import "PGPMailComposeAccessoryViewOwner.h"
#import "PGPMailBundle.h"
#import "PGPMailMessage.h"
#import "PGPMailMessageEditor.h"

#import "PGPMailProgress.h"

#import <NSStringMessage.h>
#import "PGPMailCommon.h"

#define kPGPmailComposeAccessoryNib			@"PGPMailCompose"

#define kPGPmailMessageEncrypting			LSPV(@"Encrypting", @"MESSAGE_ENCRYPTING")
#define kPGPmailMessageSigning				LSPV(@"Signing", @"MESSAGE_SIGNING")
#define kPGPmailMessageEncryptingSigning	LSPV(@"Encrypting and Signing", @"MESSAGE_ENCRYPTING_SIGNING")
#define kPGPmailMessageDecryptingVerifying	LSPV(@"Decrypting/Verifying", @"MESSAGE_DECRYPTING_VERIFYING")

#define kPGPmailButtonTitleTryAgain			LSPV(@"Try Again", @"BUTTON_TRY_AGAIN")
#define kPGPmailButtonTitleCancel			LSPV(@"Cancel", @"BUTTON_CANCEL")

@interface PGPMailComposeAccessoryViewOwner(Private)
- (void)pgpSetEncryptMessage:(BOOL)inFlag;
- (void)pgpSetSignMessage:(BOOL)inFlag;
@end

@implementation PGPMailComposeAccessoryViewOwner
+ (NSString *)composeAccessoryViewNibName
{
    return kPGPmailComposeAccessoryNib;
}

- (id)init
{
    [super init];
    
    mEncodeOptions = PGPMakeDefaultEncodeOptions();
    
    return self;
}

- (void)dealloc
{
    PGPMailBundle	*theBundle = [PGPMailBundle sharedInstance];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSToolbarWillAddItemNotification object:nil];
    
    [[theBundle pgpSignMenuItem] setState:[theBundle pgpSignNewMessagesByDefault]];
    [[theBundle pgpEncryptMenuItem] setState:[theBundle pgpEncryptNewMessagesByDefault]];
    
    [super dealloc];
}

- (void)windowDidResignMain:(NSNotification *)notification
{
    if( [[self composeAccessoryView] isDescendantOf:[[notification object] contentView]] )
    {
        PGPMailBundle	*theBundle = [PGPMailBundle sharedInstance];
        
        [[theBundle pgpSignMenuItem] setState:([theBundle pgpSignNewMessagesByDefault] ? NSOnState:NSOffState)];
        [[theBundle pgpEncryptMenuItem] setState:([theBundle pgpEncryptNewMessagesByDefault] ? NSOnState:NSOffState)];
    }
}

- (void)windowDidBecomeMain:(NSNotification *)notification
{
    if( [[self composeAccessoryView] isDescendantOf:[[notification object] contentView]] )
    {
        MessageEditor			*theEditor = [[[self composeAccessoryView] window] delegate];
        PGPMailBundle			*theBundle = [PGPMailBundle sharedInstance];
                    
        [[theBundle pgpSignMenuItem] setState:(mEncodeOptions.mSign ? NSOnState:NSOffState)];
        [[theBundle pgpEncryptMenuItem] setState:(mEncodeOptions.mEncrypt ? NSOnState:NSOffState)];

        if([[theEditor pgpAccessoryViewOwners] lastObject] != self)
        {
            NSMutableArray	*theOwners = [NSMutableArray arrayWithArray:[theEditor pgpAccessoryViewOwners]];
    
            [self retain];
            [theOwners removeObject:self];
            [theOwners addObject:self];
            [self release];
            [theEditor pgpSetAccessoryViewOwners:theOwners];
        }
    }
}

- (void)toolbarWillAddItem:(NSNotification *)notification
{
    if( [notification object] == [[[self composeAccessoryView] window] toolbar] )
    {
        NSToolbarItem	*theItem = [[notification userInfo] objectForKey:@"item"];
        
        if(	[[theItem itemIdentifier] isEqualToString:kPGPToolbarIdentifierEncrypt] ||
            [[theItem itemIdentifier] isEqualToString:kPGPToolbarIdentifierSign] )
        {
            [theItem setTarget:self];
            [self performSelector:@selector(updateToolbarAndMenuItems) withObject:nil afterDelay:0.0];
        }
    }
}

- (void)setupUIForMessage:(Message *)message
{
    [super setupUIForMessage:message];
    
    [self performSelector:@selector(updateToolbarAndMenuItems) withObject:nil afterDelay:0.0];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidBecomeMain:) name:NSWindowDidBecomeMainNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidResignMain:) name:NSWindowDidResignMainNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(toolbarWillAddItem:) name:NSToolbarWillAddItemNotification object:nil];
}

- (void)updateToolbarAndMenuItems
{
    [self pgpSetEncryptMessage:[[PGPMailBundle sharedInstance] pgpEncryptNewMessagesByDefault]];
    [self pgpSetSignMessage:[[PGPMailBundle sharedInstance] pgpSignNewMessagesByDefault]];
}

- (void)pgpSetEncryptMessage:(BOOL)inFlag
{
    NSEnumerator	*theEnum = [[[[[self composeAccessoryView] window] toolbar] items] objectEnumerator];
    NSToolbarItem	*theItem = nil;

	mEncodeOptions.mEncrypt = inFlag;
    mEncodeOptions.mUseMIME = [[PGPMailBundle sharedInstance] pgpUsePGPMIMEWhenSendingMail];
    
    while( theItem = [theEnum nextObject] )
        if( [[theItem itemIdentifier] isEqualToString:kPGPToolbarIdentifierEncrypt] )
        {
            [theItem setLabel:(mEncodeOptions.mEncrypt ? kPGPToolbarItemEncrypted:kPGPToolbarItemClear)];
            [theItem setImage:[NSImage imageNamed:(mEncodeOptions.mEncrypt ? kPGPMailEncryptedIconName:kPGPMailClearIconName)]];
            [theItem setToolTip:(mEncodeOptions.mEncrypt ? kPGPToolbarItemTooltipEncrypted:kPGPToolbarItemTooltipClear)];
        }

    [[[PGPMailBundle sharedInstance] pgpEncryptMenuItem] setState:(mEncodeOptions.mEncrypt ? NSOnState:NSOffState)];
}

- (void)pgpSetSignMessage:(BOOL)inFlag
{
    NSEnumerator	*theEnum = [[[[[self composeAccessoryView] window] toolbar] items] objectEnumerator];
    NSToolbarItem	*theItem = nil;

    mEncodeOptions.mSign = inFlag;
    mEncodeOptions.mUseMIME = [[PGPMailBundle sharedInstance] pgpUsePGPMIMEWhenSendingMail];
        
    while( theItem = [theEnum nextObject] )
        if( [[theItem itemIdentifier] isEqualToString:kPGPToolbarIdentifierSign] )
        {
            [theItem setLabel:mEncodeOptions.mSign ? kPGPToolbarItemSigned:kPGPToolbarItemUnsigned];
            [theItem setImage:[NSImage imageNamed:(mEncodeOptions.mSign ? kPGPMailSignedIconName:kPGPMailUnsignedIconName)]];
            [theItem setToolTip:(mEncodeOptions.mSign ? kPGPToolbarItemTooltipSigned:kPGPToolbarItemTooltipUnsigned)];
        }

    [[[PGPMailBundle sharedInstance] pgpSignMenuItem] setState:(mEncodeOptions.mSign ? NSOnState:NSOffState)];
}

- (void)pgpToggleEncryptionForNewMessage:(id)sender
{
    [self pgpSetEncryptMessage:!mEncodeOptions.mEncrypt];
}

- (void)pgpToggleSignatureForNewMessage:(id)sender
{
    [self pgpSetSignMessage:!mEncodeOptions.mSign];
}

- (void)sheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if( NSAlertDefaultReturn == returnCode )
    {
        id			theTarget = [[[self composeAccessoryView] window] delegate];
        SEL			theSelector = NSSelectorFromString(@"send:");
        
        if( theTarget && [theTarget respondsToSelector:theSelector] )
            [theTarget performSelector:theSelector withObject:nil];
    }
}

- (void)displayException:(NSException *)exception
{
    NSString 	*theTitle = mEncodeOptions.mEncrypt ? kPGPmailMessageEncrypting : kPGPmailMessageSigning;
            
    NSBeginAlertSheet(theTitle, nil, nil, nil, [[self composeAccessoryView] window], nil, NULL, NULL, NULL, [exception reason]);
    [exception release];
}

- (BOOL)messageWillBeDelivered:(Message *)inMessage
{
    BOOL			theResult = [super messageWillBeDelivered:inMessage];
    
    if( theResult && (mEncodeOptions.mEncrypt || mEncodeOptions.mSign) )
    {
        PGPMailProgress	*progress = [PGPMailProgress sharedInstance];
        NSString		*thePrompt = nil;

        if( mEncodeOptions.mEncrypt && mEncodeOptions.mSign )
            thePrompt = kPGPmailMessageEncryptingSigning;
        else if( mEncodeOptions.mEncrypt )
            thePrompt = kPGPmailMessageEncrypting;
        else
            thePrompt = kPGPmailMessageSigning;

        [progress startWithMessage:thePrompt delegate:self];
        NS_DURING
			mEncodeOptions.mEncodeError = kPGPError_NoErr;
            [inMessage pgpEncodeMessage:&mEncodeOptions];
			if (IsPGPError(mEncodeOptions.mEncodeError)) {
				theResult = NO;
			}
        NS_HANDLER
            theResult = NO;
            [self performSelector:@selector(displayException:)
                withObject:[localException retain] afterDelay:0.0];
        NS_ENDHANDLER

        [progress stop];
     }

    return theResult;
}

- (void)progressIndicatorDidCancel:(PGPMailProgress *)controller
{
    [PGPMailEncodeDecode pgpCancelEncodeOperation];
}
@end
