/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailMessageBody.m,v 1.7 2002/09/21 02:54:20 jason Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>

#import "PGPMailMessageBody.h"
#import "PGPMailNSData.h"
#import "PGPMailBundle.h"

#import <NSDataMessage.h>
#import <NSStringMessage.h>
#import <Message.h>
#import <MessageHeaders.h>
#import <MimeBody.h>
#import <MimePart.h>
#import <MutableMessageHeaders.h>
#import <ObjectCache.h>

#import <Foundation/Foundation.h>

#define kPGPmailUnableToModify			LSPV(@"Unable to modify body", @"UNABLE_TO_MODIFY_BODY")
#define kPGPmailCharsetMissing			LSPV(@"did not find charset %@ in %@", @"DID_NOT_FIND_CHARSET")

@implementation MessageBody(PGPMail)

- (BOOL)isPGPMessage
{
    [NSException raise:NSGenericException format:kPGPmailUnexpectedCase, NSStringFromClass([self class]), NSStringFromSelector(_cmd) ];
    return NO;
}

- (BOOL)isPGPMessage:(PGPMailDecodeOptions *)ioOptions
{
    [NSException raise:NSGenericException format:kPGPmailUnexpectedCase, NSStringFromClass([self class]), NSStringFromSelector(_cmd) ];
    return NO;
}

- (MessageBody *)pgpDecryptedBody:(MessageHeaders **)ioDecryptedMessageHeaders options:(PGPMailDecodeOptions *)ioOptions;
{
    [NSException raise:NSGenericException format:kPGPmailUnexpectedCase, NSStringFromClass([self class]), NSStringFromSelector(_cmd) ];
    return nil;
}

- (NSData *)pgpEncode:(PGPMailEncodeOptions *)ioOptions recipients:(NSArray *)inRecipients
{
    NSData					*modifiedData, *originalData = [self rawData];
    BOOL					usesQuotedPrintable = NO, usedQuotedPrintable = NO;
    BOOL					usesBase64 = NO, usedBase64 = NO;
    CFStringEncoding		newEncoding, originalEncoding;
    MutableMessageHeaders	*newHeaders;
    PGPMailEncodeDecode		*theHandler;
	NSString				*contentType = [[[self message] headers] firstHeaderForKey:kPGPmailHeaderContentType];

    NSAssert2( 0 < [originalData length], kPGPmailDataMissing, NSStringFromClass([self class]), NSStringFromSelector(_cmd) );

    usedQuotedPrintable = [[[[self message] headers] firstHeaderForKey:kPGPmailHeaderTransferEncoding] isEqualToString:kPGPmailHeaderQuotedPrintable];
    if(usedQuotedPrintable)
        originalData = [originalData decodeQuotedPrintableForText:YES];
    else{
        usedBase64 = [[[[self message] headers] firstHeaderForKey:kPGPmailHeaderTransferEncoding] isEqualToString:kPGPmailHeaderBase64];
        if(usedBase64)
            originalData = [originalData decodeBase64];
    }
	ioOptions->mFlowed = ([contentType rangeOfString:kPGPmailHeaderFormatIsFlowed options:NSCaseInsensitiveSearch].location != NSNotFound);
    if(ioOptions->mSign)
        originalData = [originalData pgpConvertEndOfLines];

    newEncoding = originalEncoding = [[(MimeBody *)self topLevelPart] textEncoding];

    theHandler = [PGPMailEncodeDecode handler];
    originalData = [PGPMailEncodeDecode pgpConvertedStringData:originalData fromEncoding:originalEncoding toEncoding:&newEncoding];
    if(ioOptions->mEncrypt)
        modifiedData = [theHandler pgpEncodeData:originalData options:ioOptions recipients:inRecipients];
    else
        modifiedData = [theHandler pgpEncodeData:originalData options:ioOptions recipients:nil];
    
    if(usedBase64)
    {
        modifiedData = [modifiedData encodeBase64];
        usesBase64 = YES;
    }
    else if(usedQuotedPrintable || [modifiedData pgpContainsNonASCIICharacter])
    {
        modifiedData = [modifiedData encodeQuotedPrintableForText:YES];
        usesQuotedPrintable = YES;
    }
    
    [self setEncodedBody:modifiedData];
    newHeaders = [[[self message] headers] mutableCopy];
	if ((!ioOptions->mUseMIME) && (!ioOptions->mEncrypt) && (ioOptions->mFlowed)) {
		NSMutableString *	newContentType = [NSMutableString stringWithString:contentType];
		
		[newContentType replaceOccurrencesOfString:kPGPmailHeaderFormatIsFlowed withString:kPGPmailHeaderFormatIsFixed
			options:NSCaseInsensitiveSearch range:NSMakeRange(0, [newContentType length])];
		[newHeaders setHeader:newContentType forKey:kPGPmailHeaderContentType];
		[newHeaders setHeader:@"1" forKey:kPGPmailHeaderXPGPRFC2646Fix];
	}

    if(usesQuotedPrintable && !usedQuotedPrintable)
        [newHeaders setHeader:kPGPmailHeaderQuotedPrintable forKey:kPGPmailHeaderTransferEncoding];
    else if(usesBase64 && !usedBase64)
        [newHeaders setHeader:kPGPmailHeaderBase64 forKey:kPGPmailHeaderTransferEncoding];
    if(newEncoding != originalEncoding)
    {
        NSMutableString	*newContentType = [NSMutableString stringWithString:[newHeaders firstHeaderForKey:kPGPmailHeaderContentType]];
        NSString		*charset = [[(MimeBody *)self topLevelPart] bodyParameterForKey:kPGPmailHeaderCharset];
        NSRange			charsetRange = [newContentType rangeOfString:charset];
        
        NSAssert2(charsetRange.location != NSNotFound, kPGPmailCharsetMissing, charset, newContentType);
        [newContentType replaceCharactersInRange:charsetRange withString:(NSString *)CFStringConvertEncodingToIANACharSetName(newEncoding)];
        [newHeaders setHeader:newContentType forKey:kPGPmailHeaderContentType];
    }
    [[[self message] headers] flushCachedData];
    [[self message] setHeaders:newHeaders];
    [newHeaders release];

    return modifiedData;
}
@end

#import <MessageStore.h>

@implementation MessageBody(PGPMail_MacOSX)
- (void)setEncodedBody:(NSData *)data
{
    if([[self message] messageStore])
        [((MessageStore *)[[self message] messageStore])->_bodyDataCache setObject:data forKey:[self message]];
    else
    {
        NSLog(@"Unable to modify encodedBody");
    }
}
@end
