/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: PGPdiskPubTypes.h,v 1.2 2002/08/06 20:10:18 dallen Exp $
____________________________________________________________________________*/

#ifndef Included_PGPdiskPubTypes_h	/* [ */
#define Included_PGPdiskPubTypes_h

#include "pgpPubTypes.h"
#include "pgpDiskBase.h"

/* Types */

#ifdef PGPDISK_ENGINE

namespace PGP
{
	class CPGPdiskContext;
	class CPGPdiskDisk;
	class CPGPdiskDiskSet;
	class CPGPdiskDiskIter;
	class CPGPdiskUser;
	class CPGPdiskUserSet;
	class CPGPdiskUserIter;
}

typedef PGP::CPGPdiskContext	*PGPdiskContextRef;
typedef PGP::CPGPdiskDisk		*PGPdiskDiskRef;
typedef PGP::CPGPdiskDiskSet	*PGPdiskDiskSetRef;
typedef PGP::CPGPdiskDiskIter	*PGPdiskDiskIterRef;
typedef PGP::CPGPdiskUser		*PGPdiskUserRef;
typedef PGP::CPGPdiskUserSet	*PGPdiskUserSetRef;
typedef PGP::CPGPdiskUserIter	*PGPdiskUserIterRef;

#else

typedef struct CPGPdiskContextExt	*PGPdiskContextRef;
typedef struct CPGPdiskDiskExt		*PGPdiskDiskRef;
typedef struct CPGPdiskDiskSetExt	*PGPdiskDiskSetRef;
typedef struct CPGPdiskDiskIterExt	*PGPdiskDiskIterRef;
typedef struct CPGPdiskUserExt		*PGPdiskUserRef;
typedef struct CPGPdiskUserIterExt	*PGPdiskUserIterRef;
typedef struct CPGPdiskUserSetExt	*PGPdiskUserSetRef;

#endif	/* PGPDISK_ENGINE */

/* Types */

typedef enum 
{
	kPGPdiskAnyPassType				= 1, 
	kPGPdiskNonPublicKeyPassType	= 2, 
	kPGPdiskMasterPassType			= 3, 
	kPGPdiskAlternatePassType		= 4, 
	kPGPdiskPublicKeyPassType		= 5

} PGPdiskPassphraseType;

typedef enum
{
	kPGPdiskInvalidAlgorithm	= 0, 
	kPGPdiskCopyDataAlgorithm	= 1, 
	kPGPdiskOldCAST5Algorithm	= 2, 
	kPGPdiskCAST5Algorithm		= 3, 
	kPGPdiskTwoFishAlgorithm	= 4, 

	// All user-defined algorithms must be > 1000.
	kPGPdiskDummyAlgorithm 		= ULONG_MAX		// force size to 4 bytes

} PGPdiskEncryptionAlgorithm;

typedef struct 
{
	char	path[kPGPdiskMaxPathLength];
	char	root[kPGPdiskMaxPathLength];
	char	deviceName[kPGPdiskMaxPathLength];

	PGPBoolean	isReadOnly;
	PGPBoolean	hasTimedOut;

} PGPdiskInfo;

typedef enum 
{
	kPGPdiskUserProp_HasPublicKey		= 100,		// boolean
	kPGPdiskUserProp_HasSymmetricKey	= 200,		// boolean
	kPGPdiskUserProp_IsReadOnly			= 300,		// boolean, settable
	kPGPdiskUserProp_IsLocked			= 400,		// boolean, settable
	kPGPdiskUserProp_IsAdminUser		= 500,		// boolean
	kPGPdiskUserProp_IsDisabled			= 600,		// boolean, settable
	kPGPdiskUserProp_IsCipheringEnabled	= 700,		// boolean
	kPGPdiskUserProp_CanSetReadOnly		= 800,		// boolean
	kPGPdiskUserProp_CanSetLocked		= 900,		// boolean
	kPGPdiskUserProp_CanSetUserName		= 1000,		// boolean

	kPGPdiskUserProp_UserName			= 1100,		// string, settable

} PGPdiskUserPropName;

typedef enum 
{
	kPGPdiskDiskProp_IsMounted			= 100,		// boolean
	kPGPdiskDiskProp_IsOpened			= 200, 		// boolean
	kPGPdiskDiskProp_IsReadOnly			= 300, 		// boolean
	kPGPdiskDiskProp_IsLockedForFormat	= 310, 		// boolean
	kPGPdiskDiskProp_IsLockedForIO		= 320, 		// boolean
	kPGPdiskDiskProp_IsBeingReEncrypted	= 400, 		// boolean
	kPGPdiskDiskProp_HasOldHeaders		= 500, 		// boolean
	kPGPdiskDiskProp_AreUsersWiped		= 600, 		// boolean
	kPGPdiskDiskProp_HasTimedOut		= 700,		// boolean
	kPGPdiskDiskProp_UsesCustomTimeout	= 710,		// boolean, settable

	kPGPdiskDiskProp_Algorithm			= 800, 		// number
	kPGPdiskDiskProp_CustomTimeout		= 900, 		// number, settable
	kPGPdiskDiskProp_BlocksDiskLow		= 1000,		// number
	kPGPdiskDiskProp_BlocksDiskHigh		= 1100,		// number

	kPGPdiskDiskProp_Path				= 1200,		// string
	kPGPdiskDiskProp_Root				= 1300, 	// string
	kPGPdiskDiskProp_DefaultRoot		= 1400, 	// string, settable
	kPGPdiskDiskProp_DeviceName			= 1500		// string

} PGPdiskDiskPropName;

typedef PGPBoolean (*ReEncryptDiskStatusFuncType)(PGPUInt32 percent, 
	void *userVal);

typedef PGPBoolean (*NewDiskStatusFuncType)(PGPBoolean areFormatting, 
	PGPUInt32 percent, void *userVal);


/* Constants */

#define	kInvalidPGPdiskContextRef	((PGPdiskContextRef) NULL)
#define	kInvalidPGPdiskDiskRef		((PGPdiskDiskRef) NULL)
#define	kInvalidPGPdiskDiskSetRef	((PGPdiskDiskSetRef) NULL)
#define	kInvalidPGPdiskDiskIterRef	((PGPdiskDiskIterRef) NULL)
#define	kInvalidPGPdiskUserRef		((PGPdiskUserRef) NULL)
#define	kInvalidPGPdiskUserSetRef	((PGPdiskUserSetRef) NULL)
#define	kInvalidPGPdiskUserIterRef	((PGPdiskUserIterRef) NULL)


/* Macros */

#define PGPdiskContextRefIsValid(ref)	((ref) != kInvalidPGPdiskContextRef)
#define PGPdiskDiskRefIsValid(ref)		((ref) != kInvalidPGPdiskDiskRef)
#define PGPdiskDiskSetRefIsValid(ref)	((ref) != kInvalidPGPdiskDiskSetRef)
#define PGPdiskDiskIterRefIsValid(ref)	((ref) != kInvalidPGPdiskDiskIterRef)
#define PGPdiskUserRefIsValid(ref)		((ref) != kInvalidPGPdiskUserRef)
#define PGPdiskUserSetRefIsValid(ref)	((ref) != kInvalidPGPdiskUserSetRef)
#define PGPdiskUserIterRefIsValid(ref)	((ref) != kInvalidPGPdiskUserIterRef)

#endif	/* ] Included_PGPdiskPubTypes_h */
