/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpNetService.h,v 1.25 2002/08/06 20:10:26 dallen Exp $
____________________________________________________________________________*/
#ifndef Included_pgpNetService_h	/* [ */
#define Included_pgpNetService_h

#include "pgpNetPrefs.h"
#include "pgpUtilities.h"
#include "pgpIKE.h"
#include "pgpRPCPacker.h"

#if PGP_OSX
#include <CoreFoundation/CFSocket.h>
#include "PGPnetDriverComm.h"
#endif /* PGP_OSX */

#include "pgpNetKernelXChng.h"
#include "pgpNetLog.h"

#define kPGPnetSocket			"/var/run/pgpnet.sock"
#define kPGPnetUISocket			"/tmp/pgpnetui.sock"
#define kPGPnetSocketMode		0666		/* rw-rw-rw- */
#define kPGPnetUISocketMode		0666		/* rw-rw-rw- */

/* Enums */

typedef enum
{
	/* Client messages to server (and server's responses) */
	kPGPnet_MT_ClientConnect,				/* C->S: [PGPBoolean][path to client sock] */
	
	kPGPnet_MT_GetSAList,					/* C->S: No data */
	kPGPnet_MT_SendSAList,					/* S->C: [PGPUInt32][array of PGPipsecSA] */

	/* For connecting to hosts on host list */
	kPGPnet_MT_Connect,						/* C->S: [PGPBoolean][PGPnetSAIdent] */
	kPGPnet_MT_IKEAlert,					/* S->C: [PGPikeMTAlert] */
	kPGPnet_MT_AuthCheckRequest,			/* S->C: [PGPikeMTAuthCheck] */
	kPGPnet_MT_AuthCheckReply,				/* C->S: [PGPikeMTAuthCheck] */
	kPGPnet_MT_SAFinished,					/* S->C: [PGPnetConnectionStatus] */

	kPGPnet_MT_Disconnect,					/* C->S: [PGPipsecSPI] */

	kPGPnet_MT_SendPassphrase,				/* C->S: [Passphrase type][Passphrase] */
	kPGPnet_MT_PassphraseIsOK,				/* S->C: [PGPBoolean] */

	kPGPnet_MT_FlushPassphrases,			/* C->S: No data */

	kPGPnet_MT_GetRemoteAuthKey,			/* C->S: [PGPipsecSPI] */
	kPGPnet_MT_SendRemoteAuthKey,			/* S->C: [PGPBytes of authKey] */
	
	kPGPnet_MT_LogTraffic,					/* C->S: [PGPBoolean (allowed)][PGPBoolean (blocked)] */

	kPGPnet_MT_GetPrefs,					/* C->S: No data */
	kPGPnet_MT_SendPrefs,					/* S->C: [exported pref buffer] */
	kPGPnet_MT_SetPrefs,					/* C->S: [exported pref buffer] */

	kPGPnet_MT_RereadConfig,				/* C->S: No data */
	kPGPnet_MT_ClearLogFile,				/* C->S: No data */
	
	/* Server messages to client (and client's responses) */
	kPGPnet_MT_SAInProgress,				/* S->C: [PGPnetSAIdent] */
	kPGPnet_MT_NewSA,						/* S->C: [PGPipsecSA] */
	kPGPnet_MT_SADied,						/* S->C: [PGPipsecSA] */
	kPGPnet_MT_SAFailed,					/* S->C: [PGPnetSAIdent] */
	kPGPnet_MT_Identity,					/* S->C: [PGPikeMTIdentity] */
	kPGPnet_MT_SAStatusUpdate,				/* S->C: [PGPUInt32][array of PGPnetStat] */
	kPGPnet_MT_NewAttacker,					/* S->C: [PGPNetPrefBlockedEntry] */
	kPGPnet_MT_LogEvent,					/* S->C: [PGPnetLogEvent] */
	kPGPnet_MT_IKEDebug,					/* S->C: [string of IKE debug message] */
	kPGPnet_MT_PrefsChanged,				/* S->C: [exported pref buffer] */
	kPGPnet_MT_IntrudersChanged,			/* S->C: [PGPUInt32][array of PGPNetPrefBlockedEntry] */
											/* C->S: No data */
	kPGPnet_MT_DriverNotInstalled,			/* S->C: No data */

	/* Force PGPnetMessageType to 32 bits */
	kPGPnet_MT_None					= 0xFFFFFFFF
} PGPnetMessageType;

typedef enum
{
	kPGPnetPassphraseType_PGP		= 1,
	kPGPnetPassphraseType_X509		= 2,

	/* Force to 32 bits */
	kPGPnetPassphraseType_None		= 0xFFFFFFFF
} PGPnetPassphraseType;

/* Structs */

/* Used to match an SA with a PGPNetPrefHostEntry */
typedef struct PGPnetSAIdent
{
	PGPUInt32	ipAddress;
	PGPUInt32	ipAddrStart;
	PGPUInt32	ipMaskEnd;
	PGPUInt32	destIsRange;
} PGPnetSAIdent;

typedef struct PGPnetContextPriv
{
	PGPContextRef	context;
	PGPInt32		sockToService;			/* write() data here to send messages to service */
	PGPInt32		sockFromService;		/* read() data from here to get messages from service */
} PGPnetContextPriv;

typedef struct PGPnetContextPriv *		PGPnetContextRef;

#define kInvalidPGPnetContextRef		((PGPnetContextRef) NULL)
#define PGPnetContextRefIsValid(ref)	((ref) != kInvalidPGPnetContextRef)
#define PGPnetValidateContextRef(ref)	PGPValidateParam(PGPnetContextRefIsValid(ref))


/* Functions */

/* Init/cleanup functions */

	PGPError
PGPnetNewContext(
	PGPContextRef		context,
	PGPnetContextRef	*outNetContext );
	
	PGPError
PGPnetFreeContext(
	PGPnetContextRef	netContext );
	
	PGPContextRef
PGPnetPeekContextPGPContext(
	PGPnetContextRef	netContext );
	
/* Functions for sending messages to the service */

	PGPError
PGPnetRereadConfig(
	PGPnetContextRef	netContext );
	
	PGPError
PGPnetTellServiceToClearLogFile(
	PGPnetContextRef	netContext );
	
/*
 * eventHandler, userValue, and status are only used if eventHandler is non-NULL.
 * This is used as a callback mechanism so that a client can get status info
 * about the connection process without having to read the events that the
 * service sends to the client.  Note that this will block if you use an event
 * handler.  Most GUI apps will set eventHandler to NULL and not worry about it.
 */
	PGPError
PGPnetConnectToHost(
	PGPnetContextRef		netContext,
	PGPNetPrefHostEntry *	hostEntry );
	
	PGPError
PGPnetDisconnectFromHost(
	PGPnetContextRef		netContext,
	PGPipsecSA *			sa );

	PGPError
PGPnetGetSAList(
	PGPnetContextRef		netContext,
	PGPipsecSA **			sa,				/* Needs to be freed by the caller with PGPFreeData */
	PGPSize *				numSAs );
	
	PGPError
PGPnetGetRemoteAuthKey(
	PGPnetContextRef		netContext,
	PGPipsecSA *			sa,
	PGPByte **				remoteAuthKey,		/* Needs to be freed by the caller with PGPFreeData */
	PGPSize *				remoteAuthKeySize );
	
	PGPError
PGPnetFlushPassphrases(
	PGPnetContextRef		netContext );
	
	PGPError
PGPnetLogTraffic(
	PGPnetContextRef		netContext,
	PGPBoolean				allowed,			/* Log allowed traffic */
	PGPBoolean				blocked );			/* Log blocked traffic */
	
/* Functions for getting messages from the service */
	
#if PGP_OSX
	PGPError
PGPnetAddMessageCallbackToCFRunLoop(
	PGPnetContextRef	netContext,
	CFRunLoopRef		rl,
	CFSocketCallBack	callback,
	PGPUserValue		userValue );	/* Given to the callback in the 'info' parameter */
#endif

/*
 * select() on netContext->sockFromService, waiting for messages from the service.
 * When we get one, we'll return, setting msg to point to the message we just
 * received.   outMsg should be freed by the caller with PGPFreeData.
 */
	PGPError
PGPnetReadMessageFromService(
	PGPnetContextRef	netContext,
	PGPrpcMessage **	outMsg );
	
	PGPError
PGPnetReadSAInProgressMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPnetSAIdent *		said );

	PGPError
PGPnetReadNewSAMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPipsecSA *		sa );

	PGPError
PGPnetReadSADiedMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPipsecSA *		sa );

	PGPError
PGPnetReadSAFailedMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPnetSAIdent *		said );
	
	PGPError
PGPnetReadIdentityMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPikeMTIdentity *	identity );
	
	PGPError
PGPnetReadSAStatusUpdateMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPnetStat **		stats,
	PGPSize *			numStats );
	
	PGPError
PGPnetReadNewAttackerMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPNetPrefBlockedEntry *	attacker );

	PGPError
PGPnetReadLogEventMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPnetLogEvent *	event );
	
	PGPError
PGPnetReadIntrudersChangedMessage(
	PGPnetContextRef			netContext,
	PGPrpcMessage *				msg,
	PGPNetPrefBlockedEntry **	blockedHosts,
	PGPUInt32 *					numBlockedHosts );

	PGPError
PGPnetReadIKEDebugMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	char **				string );		/* Needs to be freed by the caller with PGPFreeData */

	PGPError
PGPnetReadPrefsChangedMessage(
	PGPnetContextRef	netContext,
	PGPrpcMessage *		msg,
	PGPPrefRef *		netPrefs );
	
/* Prefs management */	
	
	PGPError
PGPnetGetPrefs(
	PGPnetContextRef	netContext,
	PGPPrefRef *		netPrefs );
	
	PGPError
PGPnetSetPrefs(
	PGPnetContextRef	netContext,
	PGPPrefRef			netPrefs );
	
	PGPError
PGPnetIntrudersChanged(
	PGPnetContextRef	netContext );
	

#endif /* ] Included_pgpNetService_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/

