/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPSharedEncryptDecrypt.c,v 1.5 2002/08/06 20:10:27 dallen Exp $
____________________________________________________________________________*/

#include <PGPclient/pgpClientLibOSX.h>

#include "PGPSharedEncryptDecrypt.h"

	PGPError
PGPSharedForEachKeyDo(
	PGPContextRef					context,
    PGPPrefRef						prefRef,
	PGPKeyOrdering 					keyOrdering,
	PGPBoolean						reverseOrder,
	PGPSharedForEackKeyDoProcPtr	callbackProc,
	void							*callbackData)
{
	PGPError		err = kPGPError_NoErr;
	PGPKeyDBRef		keyDBRef;
		
	err = PGPclOpenDefaultKeyrings( context, prefRef, kPGPOpenKeyDBFileOptions_None, &keyDBRef );
	if( IsntPGPError( err ) )
	{
		PGPKeySetRef	keySetRef;
		
		err = PGPNewKeySet( keyDBRef, &keySetRef );
		if( IsntPGPError( err ) )
		{
			PGPKeyListRef	keyListRef;
			
			err = PGPOrderKeySet( keySetRef, keyOrdering, reverseOrder, &keyListRef );
			if( IsntPGPError( err ) )
			{
				PGPKeyIterRef	keyIterator;
				
				err = PGPNewKeyIter( keyListRef, &keyIterator );
				if( IsntPGPError( err ) )
				{
					PGPKeyDBObjRef	theKey;
					
					err = PGPKeyIterNextKeyDBObj( keyIterator, kPGPKeyDBObjType_Key, &theKey );
					while( IsntPGPError( err ) )
					{
						err = (*callbackProc)( keyIterator, theKey,
									callbackData );
						if( IsPGPError( err ) )
							break;
							
						err = PGPKeyIterNextKeyDBObj( keyIterator, kPGPKeyDBObjType_Key, &theKey );
					}
					
					if( err == kPGPError_EndOfIteration )
						err = kPGPError_NoErr;
						
					PGPFreeKeyIter( keyIterator );
				}
				
				PGPFreeKeyList( keyListRef );
			}
		
			PGPFreeKeySet( keySetRef );
		}
		
		PGPFreeKeyDB( keyDBRef );
	}
	
	if( err == kPGPError_UserAbort )
		err = kPGPError_NoErr;

	return( err );
}

typedef struct KeyCounts
{
	PGPUInt32	publicKeys;
	PGPUInt32	privateKeys;

} KeyCounts;

	static PGPError
CountKeysInDefaultKeyringCallbackProc(
	PGPKeyIterRef	keyIterator,
	PGPKeyDBObjRef 	theKey,
	void 			*userData)
{
	PGPBoolean	isPrivateKey = FALSE;
	
	#pragma unused( keyIterator )
	
	++((KeyCounts *) userData)->publicKeys;
	
	PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsSecret, &isPrivateKey );
	if( isPrivateKey )
	{
		++((KeyCounts *) userData)->privateKeys;
	}
	
	return( kPGPError_NoErr );
}

	PGPError
CountKeysInDefaultKeyring(
	PGPContextRef	context,
    PGPPrefRef		prefRef,
	PGPUInt32 		*numPublicKeys,
	PGPUInt32 		*numPrivateKeys)
{
	PGPError	err = kPGPError_NoErr;
	KeyCounts	keyCounts;
	
    // *****
	//AssertAddrNullOrValid( numPublicKeys, PGPUInt32, "CountKeysInDefaultKeyring" );
	//AssertAddrNullOrValid( numPrivateKeys, PGPUInt32, "CountKeysInDefaultKeyring");
	
	keyCounts.publicKeys 	= 0;
	keyCounts.privateKeys	= 0;
	
	err = PGPSharedForEachKeyDo( context, prefRef, kPGPKeyOrdering_Any, FALSE,
				CountKeysInDefaultKeyringCallbackProc, &keyCounts );

	if( IsntNull( numPublicKeys ) )
		*numPublicKeys = keyCounts.publicKeys;

	if( IsntNull( numPrivateKeys ) )
		*numPrivateKeys = keyCounts.privateKeys;
		
	return( err );
}

	static PGPError
GetKeySimpleProperties(
	PGPKeyDBObjRef			theKey,
	PGPSharedKeyProperties	*properties)
{
	PGPError	err = kPGPError_NoErr;
	
    // *****
	// pgpAssertAddrValid( theKey, VoidAlign );
	pgpAssertAddrValid( properties, PGPSharedKeyProperties );
	
	pgpClearMemory( properties, sizeof( *properties ) );
	
	err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsSecret,
				&properties->isSecret );
	
	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsAxiomatic,
					&properties->isAxiomatic );
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsSecretShared,
					&properties->isSplit );
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsRevoked,
					&properties->isRevoked );
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsDisabled,
					&properties->isDisabled );
	}
	
	if( IsntPGPError( err ) )
	{
		PGPBoolean	isNotCorrupt;
		
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsNotCorrupt,
					&isNotCorrupt );
		properties->isCorrupt = ! isNotCorrupt;
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_IsExpired,
					&properties->isExpired );
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_CanSign,
					&properties->canSign );
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_CanEncrypt,
					&properties->canEncrypt );
	}

	if( IsntPGPError( err ) )
	{
		err = PGPGetKeyDBObjBooleanProperty( theKey, kPGPKeyProperty_CanEncrypt,
					&properties->canEncrypt );
	}

	return( err );
}

	PGPError
PGPSharedGetKeyProperties(
	PGPKeyDBObjRef			theKey,
	const char				*userID,
	PGPSharedKeyProperties	*properties)
{
	PGPError	err = kPGPError_NoErr;
	
    // *****
	// AssertAddrNullOrValid( userID, char, "PGPSharedGetKeyProperties" );
	pgpAssertAddrValid( properties, PGPSharedKeyProperties );
	
	err = GetKeySimpleProperties( theKey, properties );
	if( IsntPGPError( err ) )
	{
		if( IsntNull( userID ) )
		{
			PGPKeySetRef	keySet = kInvalidPGPKeySetRef;
			
			properties->userValidity = kPGPValidity_Unknown;
						
			err = PGPNewOneKeySet( theKey, &keySet );
			if( IsntPGPError( err ) )
			{
				PGPKeyListRef	keyList = kInvalidPGPKeyListRef;
				
				err = PGPOrderKeySet( keySet, kPGPKeyOrdering_Any, FALSE, &keyList );
				if( IsntPGPError( err ) )
				{
					PGPKeyIterRef	iterator;
					
					err = PGPNewKeyIter( keyList, &iterator );
					if( IsntPGPError( err ) )
					{
						PGPKeyDBObjRef		curUserID = kInvalidPGPKeyDBObjRef;
						PGPBoolean			foundUserID = FALSE;
						PGPKeyDBObjRef		keyRef = kInvalidPGPKeyDBObjRef;
						
						/* must advance to first key */
						err	= PGPKeyIterNextKeyDBObj( iterator, kPGPKeyDBObjType_Key, &keyRef );
						pgpAssertNoErr( err );
						
						while( IsntPGPError(
							err = PGPKeyIterNextKeyDBObj(iterator, kPGPKeyDBObjType_UserID, &curUserID)))
						{
							char		checkUserID[256];
							PGPSize		totalSize;
							
							err = PGPGetKeyDBObjDataProperty( curUserID,
										kPGPUserIDProperty_Name,
										checkUserID, sizeof( checkUserID ) - 1,
										&totalSize );
							if( IsntPGPError( err ) )
							{
								if( totalSize > sizeof(checkUserID) - 1 )
									checkUserID[sizeof(checkUserID) - 1] = '\0';
								else
									checkUserID[totalSize] = '\0';
								
								if( strcmp( checkUserID, userID ) == 0 )
								{
									PGPInt32	validity;
									
									err = PGPGetKeyDBObjNumericProperty( curUserID,
											kPGPUserIDProperty_Validity,
											&validity );
									
									properties->userValidity =
										(PGPValidity) validity;
									
									foundUserID = TRUE;
									break;
								}
							}
						}
						
						if( err == kPGPError_EndOfIteration )
							err = kPGPError_NoErr;
							
						(void) PGPFreeKeyIter( iterator );
					}
					
					(void) PGPFreeKeyList( keyList );
				}
				
				(void) PGPFreeKeySet( keySet );
			}
		}
		else
		{
			err = PGPGetPrimaryUserIDValidity( theKey, &properties->userValidity );
		}
	}
	return( err );
}

	PGPError
PGPSharedGetIndKeyFromKeySet(
	PGPKeySetRef 	keySet,
	PGPKeyOrdering 	keyOrdering,
	PGPBoolean		reverseOrdering,
	PGPUInt32		searchIndex,
	PGPKeyDBObjRef	*theKey)
{
	PGPError		err = kPGPError_NoErr;
	PGPKeyListRef	keyListRef = kInvalidPGPKeyListRef;
	
	*theKey = kInvalidPGPKeyDBObjRef;

	err = PGPOrderKeySet( keySet, keyOrdering, reverseOrdering, &keyListRef );
	if( IsntPGPError( err ) )
	{
		PGPKeyIterRef	keyIterator;
		
		err = PGPNewKeyIter( keyListRef, &keyIterator );
		if( IsntPGPError( err ) )
		{
			PGPUInt32	keyIndex = 0;
			
			for( keyIndex = 1; keyIndex <= searchIndex; ++keyIndex )
			{
				err = PGPKeyIterNextKeyDBObj( keyIterator, kPGPKeyDBObjType_Key, theKey );
				if( IsPGPError( err ) )
				{
					*theKey = kInvalidPGPKeyDBObjRef;
					break;
				}
			}

			PGPFreeKeyIter( keyIterator );
		}
		
		PGPFreeKeyList( keyListRef );
	}
	
	if( err == kPGPError_EndOfIteration )
		err = kPGPError_ItemNotFound;
		
	return( err );
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
