/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpClientLibOSX.h,v 1.53 2002/08/06 20:10:27 dallen Exp $
____________________________________________________________________________*/
#ifndef Included_pgpClientLibOSX_h	/* [ */
#define Included_pgpClientLibOSX_h

#include <PGP/pgpKeys.h>
#include <PGP/pgpTLS.h>
#include <PGP/pgpKeyServer.h>
#include <PGPui/pgpUserInterface.h>
#include <PGP/pgpUtilities.h>
#include <PGPclient/pgpBuild.h>
#include <PGPclient/pflPrefs.h>
#include <PGPclient/pgpClientLib.h>
#include <PGPclient/pgpKeyServerPrefs.h>

PGP_BEGIN_C_DECLARATIONS

#define kPGPcl_MaxPath				256
#define kPGPcl_MaxPGPUserIDLength	256
#define kPGPcl_OneYearInSeconds		31536000

#define kPGPRecipientSettingsDefault		0
#define kPGPGetPassphraseSettingsDefault	0

typedef PGPFlags	PGPUISettings;
typedef PGPFlags	PGPRecipientSettings;
typedef PGPFlags	PGPSelectKeysSettings;
typedef PGPFlags	PGPSelect509Settings;

typedef PGPFlags	PGPGetPassphraseOptions;
typedef PGPFlags	PGPGetPassphraseSettings;

// PGPRecipientOptions
typedef PGPFlags	PGPRecipientOptions;

typedef struct PGPClientEncodeDecodeData *			PGPClientEncodeDecodeDataRef;
#define	kInvalidPGPClientEncodeDecodeDataRef		((PGPClientEncodeDecodeDataRef) NULL)
#define PGPClientEncodeDecodeDataRefIsValid( ref )	( (ref) != kInvalidPGPClientEncodeDecodeDataRef )

typedef enum PGPclPathIndex
{
	kPGPclPublicKeyringFile			= 0,
	kPGPclPrivateKeyringFile,
	kPGPclNetPublicKeyringFile,
	kPGPclNetPrivateKeyringFile,
	kPGPclNetLogFile,
	kPGPclNetLogEventDataFolder,
	kPGPclRandseedFile,
	kPGPclNetRandseedFile,
	kPGPclNumPathIndices
} PGPclPathIndex;

enum
{
	kPGPRecipientOptionsHideFileOptions		= (1L << 0),
	kPGPRecipientOptionsAlwaysShowDialog	= (1L << 1),
	kPGPRecipientOptionsShowFYEO			= (1L << 2),
	kPGPRecipientOptionsShowSDA				= (1L << 3),

	kPGPRecipientOptionsDefault = 0
};

enum
{
	// relevant signing and encryption for files
	kPGPUISettingsTextOutput		= (1UL << 0 ),
	kPGPUISettingsNoMacBinary		= (1UL << 1),
	kPGPUISettingsSmartMacBinary	= (1UL << 2),
	kPGPUISettingsForceMacBinary	= (1UL << 3),

	// encryption only
	kPGPRecipientSettingsWipeOriginal		= (1UL << 8), // files
	kPGPRecipientSettingsConvEncrypt		= (1UL << 9),
	kPGPRecipientSettingsFYEO				= (1UL << 10),
	kPGPRecipientSettingsSDA				= (1UL << 11), // files

	// signing only
	kPGPGetPassphraseSettingsDetachedSig	= (1UL << 16)
};

enum
{
	kPGPGetPassphraseOptionsHideFileOptions	= (1L << 0),
	kPGPGetPassphraseOptionsDefault = 0
};

enum
{
	kPGPSelectKeysImportVariation	= (1L << 0),
	kPGPSelectKeysSingleKey			= (1L << 1),
	kPGPSelectKeysAuthKeysOnly		= (1L << 2),
	kPGPSelectKeysNoRSAV3			= (1L << 3),

	kPGPSelectKeysDefault = 0
};

typedef struct PGPClientDecodeEventHandlerData
{
	PGPContextRef			context;
	PGPtlsContextRef		tlsContext;
	PGPKeyDBRef				allKeys;
    PGPPrefRef				prefRef;
	PGPKeyDBObjRef			decryptionKey;
	PGPBoolean				conventional;
	PGPBoolean				split;
	PGPUInt32				cacheType;
	PGPUInt32				cacheSeconds;
	PGPBoolean				cacheGlobal;
	PGPKeyDBRef				newKeys;
	PGPKeySetRef			recipientKeySet;
	PGPKeyID				*recipientKeyIDList;
	PGPUInt32				recipientKeyIDCount;
	char					prompt[256];
	PGPByte					*outputBuffer;
	PGPSize					outputBufferSize;
	PGPEventSignatureData	signatureData;
	PGPBoolean				signatureDataValid;
	PGPAnalyzeType			sectionType;
} PGPClientDecodeEventHandlerData;

	PGPError
PGPclInitEncodeDecode(
	PGPContextRef					inContext,
	PGPtlsContextRef				inTLSContext,
    PGPKeyDBRef						inKeyDBRef,
    PGPPrefRef						inPrefRef,    
	PGPClientEncodeDecodeDataRef	*outRef);

	void
PGPclCleanupEncodeDecode(
	PGPClientEncodeDecodeDataRef	inRef);

	PGPError
PGPclEncode(
	PGPClientEncodeDecodeDataRef	inRef,
	PGPBoolean						inSign,
	PGPGetPassphraseSettings		inSignSettings,
	PGPGetPassphraseOptions			inSignOptions,
	PGPBoolean						inUseSigningCache,
	PGPBoolean						inClearSign,
	PGPBoolean						inDataIsText,
	PGPBoolean						inEncrypt,
	PGPRecipientSettings			inEncryptSettings,
	PGPRecipientOptions				inEncryptOptions,
	PGPUInt32						inNumDefaultRecipients,
	const PGPRecipientSpec *		inDefaultRecipients,
	PGPBoolean						inArmorOutput,
	const PGPByte *					inData,
	PGPSize							inDataSize,
	PGPByte **						outData,
	PGPSize *						outDataSize);

 	PGPError
PGPclDecode(
    PGPClientEncodeDecodeDataRef	inRef,
	const PGPByte *					inData,
	PGPSize							inDataSize,
	PGPByte **						outData,
	PGPSize *						outDataSize);
   
    PGPError
PGPclInitDecodeEventHandlerData(
	PGPContextRef						context,
	PGPtlsContextRef					tlsContext,
	PGPKeyDBRef							allKeys,
	PGPPrefRef							prefRef,
	PGPClientDecodeEventHandlerData *	data );

    void
PGPclCleanupDecodeEventHandlerData(
    PGPClientDecodeEventHandlerData 	*data );

    PGPError
PGPclDecodeEventHandler(
    PGPContextRef 						context,
    struct PGPEvent 					*event,
    PGPClientDecodeEventHandlerData 	*data );

	PGPError
PGPclGetPath(
	PGPPrefRef					prefRef,
	PGPPrefRef					netPrefRef,
	PGPclPathIndex				index,
	char						*pszPath,
	PGPInt32					iLen);

	PGPError
PGPclOpenDefaultKeyrings(
	PGPContextRef				context,
	PGPPrefRef					prefRef,
	PGPOpenKeyDBFileOptions		options,
	PGPKeyDBRef*				pkeydbOut);

	PGPError
PGPclOpenNetKeyrings(
	PGPContextRef				context,
	PGPPrefRef					netPrefRef,
	PGPOpenKeyDBFileOptions		options,
	PGPKeyDBRef*				pkeydbOut );

	PGPError
PGPclOpenKeyrings(
	PGPContextRef				context,
	char*						pszPubPath,
	char*						pszPrivPath,
	PGPOpenKeyDBFileOptions		options,
	PGPKeyDBRef*				pkeydbOut );

	void
PGPclGetFingerprintString(
	PGPKeyDBObjRef				key,
	char						*fingerString );

	PGPError
PGPclGetKeyPassphrase(
	PGPKeyDBObjRef				key,
	PGPtlsContextRef			tlsContext,
	PGPPrefRef					prefRef,
	PGPBoolean					*isSplit,
	char						**passphrase,
	PGPByte						**passKey,
	PGPSize						*passKeySize );

   PGPError
PGPclSigningPassphraseDialog(
    PGPContextRef               inContext,
    PGPPrefRef					inPrefRef,
    PGPKeyDBRef                 inKeyDBRef, 
    const char                  *ioPrompt,
    PGPGetPassphraseOptions     inSignOptions,
    PGPGetPassphraseSettings	inSignSettings,
    PGPKeyDBObjRef              inDefaultSigningKey,
    char                        ioPassphrase[kPGPcl_MaxPGPUserIDLength],
    PGPGetPassphraseSettings    *ioUserSettings,
    PGPKeyDBObjRef              *ioActualSigningKey);
    
    PGPError
PGPclRecipientDialog(
    PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
    PGPtlsContextRef			inTLSContext,
    PGPKeyDBRef					inKeyDBRef,
    PGPUInt32					inNumDefaultRecipients,
    const PGPRecipientSpec		*ioDefaultRecipients,
    PGPRecipientOptions			inDialogOptions,
    PGPRecipientSettings		inDefaultSettings,
    PGPRecipientSettings		*ioUserSettings,
    PGPKeyDBRef					*outActualRecipients,
    PGPKeyDBRef					*outNewKeys);

	PGPError
PGPclSelectKeysDialog(
	PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
	PGPSelectKeysSettings		settings,
	const char					*prompt,
	PGPKeyDBRef					fromDB,
	PGPKeyDBRef					validityDB,		// can be NULL
	PGPKeySetRef				*selectedSet);	// Caller is responsible for
                                                // freeing selectedSet

	PGPError
PGPclForYourEyesOnlyDialog(
	PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
	char *						inString);

	PGPError
PGPclOutputDialog(
	PGPContextRef	inContext,
    PGPPrefRef		inPrefRef,
	const char *	inString );

	PGPError
PGPclConfirmRemoteKeyDialog(
	const char				*prompt,
	const char				*remoteHostName,
	PGPKeyDBObjRef			keyToConfirm,
	PGPtlsCipherSuiteNum	tlsCipher,
	PGPBoolean				askConfirm );

	PGPError
PGPclReconstitutionDialog(
	PGPKeyDBObjRef				keyToCollect,
	PGPKeyDBRef					keyDB,
	PGPtlsContextRef			tlsContext,
    PGPPrefRef					prefRef,
	PGPByte **					passKey,
	PGPSize *					passKeySize);

	PGPError
PGPclGuaranteeMinimumEntropy(
    PGPContextRef 		inContext );

	PGPError
PGPclDecryptionPassphraseDialog(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
    PGPPrefRef			prefRef,
	const char			*prompt,
	PGPKeySetRef		recipientKeys,
	const PGPKeyID		keyIDList[],		
	PGPUInt32			keyIDCount,
	PGPOptionListRef	additionalOptions,
	char				**passphrase,
	PGPKeyDBObjRef 		*decryptionKey,
	PGPKeyDBRef			*newKeys);

	PGPError
PGPclGetKeyFromServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPKeyDBObjRef		key,
	PGPKeyDBRef			*resultDB);

	PGPError
PGPclGetKeyIDFromServer(
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPKeyID const		*keyID,
	PGPKeyDBRef			*resultDB);

	PGPError
PGPclSendKeyToServer(   
	PGPContextRef		context,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPKeyDBObjRef		key,
	PGPKeyServerEntry*	targetServer);

	PGPError
PGPclGetCACertReqAVList(
	PGPContextRef			context,
	PGPPrefRef				prefRef,
	PGPBoolean				force,
	PGPKeyDBObjRef			userID,
	PGPKeyServerClass		serverClass,
	PGPUInt32 *				requestType,
	PGPAttributeValue **	pAVList,
	PGPUInt32 *				pNumAVs );

	PGPError
PGPclFreeCACertReqAVList(
	PGPAttributeValue		*pAVList,
	PGPUInt32				numAVs );

	PGPError
PGPclErrorBox( PGPError theError );

	PGPError
PGPclGetPrefDefault(
	PGPPrefIndex		prefID,
	PGPPrefDefinition	*prefDef);

	PGPError
PGPclGetNetPrefDefault(
	PGPPrefIndex		prefID,
	PGPPrefDefinition	*prefDef);
    
	PGPError
PGPclRSASupported(
	PGPBoolean *		hasRSA);

PGPError PGPclHideFileExtensions(PGPBoolean* hideExtensions);

PGPError PGPclKeyboardUIEnabled(PGPBoolean* keyboardNav);

PGPError PGPclIsOSSupported(void);

PGPError PGPclIsExpired(void);

PGP_END_C_DECLARATIONS


#endif /* ] Included_pgpClientLibOSX_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
