/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpClientPrefs.c,v 1.41 2002/11/16 19:26:15 pbj Exp $
____________________________________________________________________________*/
#include "pgpClientPrefs.h"
#include "pgpKeyServerPrefs.h"
#include "pgpMem.h"

/*	beginning of code formerly in pgpClientPrefs.h .... */
static PGPByte sBlankAdminHash[] = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

static PGPPrefStructMember sLDAPKeyServerArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPKeyServerProtocol_LDAP},
	{kPGPPrefType_Number, (void *) kPGPKeyServerClass_PGP},
	{kPGPPrefType_String, (void *) ""},
	{kPGPPrefType_String, (void *) "keyserver.pgp.com"},
	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_String, (void *) ""},
	{kPGPPrefType_Number, (void *) kPGPPublicKeyAlgorithm_Invalid},
	{kPGPPrefType_Number, (void *) (kKeyServerListed|kKeyServerIsRoot)},
	{kPGPPrefType_String, (void *) ""},
};

static PGPPrefStructMember sEuroKeyServerArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPKeyServerProtocol_LDAP},
	{kPGPPrefType_Number, (void *) kPGPKeyServerClass_PGP},
	{kPGPPrefType_String, (void *) ""},
	{kPGPPrefType_String, (void *) "europe.keys.pgp.com"},
	{kPGPPrefType_Number, (void *) 11370},
	{kPGPPrefType_String, (void *) ""},
	{kPGPPrefType_Number, (void *) kPGPPublicKeyAlgorithm_Invalid},
	{kPGPPrefType_Number, (void *) (kKeyServerListed)},
	{kPGPPrefType_String, (void *) ""},
};

PGPPrefStruct LDAPKeyServer =
{
	sizeof(sLDAPKeyServerArray) / sizeof(PGPPrefStructMember),
	sLDAPKeyServerArray
};

PGPPrefStruct EuroKeyServer =
{
	sizeof(sEuroKeyServerArray) / sizeof(PGPPrefStructMember),
	sEuroKeyServerArray
};

static PGPPrefArrayElement sDefaultKeyServerArray[] =
{
	{&LDAPKeyServer},{&EuroKeyServer}
};

PGPPrefArray DefaultKeyServer =
{
	kPGPPrefType_Struct,
	sizeof(sDefaultKeyServerArray) / sizeof(PGPPrefArrayElement),
	sDefaultKeyServerArray,
	&LDAPKeyServer
};

static PGPPrefArrayElement sAllowedAlgorithmsArray[] =
{
	{(void *) kPGPCipherAlgorithm_AES256},
	{(void *) kPGPCipherAlgorithm_CAST5},
	{(void *) kPGPCipherAlgorithm_3DES},
	{(void *) kPGPCipherAlgorithm_IDEA},
	{(void *) kPGPCipherAlgorithm_Twofish256}
};

static PGPPrefArray sAllowedAlgorithms =
{
	kPGPPrefType_Number,
	sizeof(sAllowedAlgorithmsArray) / sizeof(PGPPrefArrayElement),
	sAllowedAlgorithmsArray
};

PGPPrefArray DefaultKeys =
{
	kPGPPrefType_Byte,
	0,
	NULL
};

#if	PGP_MACINTOSH	/* [ */

static const PGPmenuHotkeys	sInitPGPmenuHotkeys[] =
{
	kMenuOptionModifier,		'E',
	kMenuOptionModifier,		'S',
	kMenuOptionModifier,		'C',
	kMenuOptionModifier,		'D',
	kMenuOptionModifier,		'L',
	kMenuOptionModifier,		'P'
};

static const PGPmenuApps	sInitPGPmenuApps[] =
{
	{TRUE,	TRUE,	'AOp3', "\pAmerica Online"},		/* Verified -> 5.0 */
	{TRUE,	FALSE,	'R*ch', "\pBBEdit"},				/* Verified -> 5.0	*/
	{FALSE,	FALSE,	'MMan', "\pEmailer"},				/* Verified -> EOL */
	{TRUE,	TRUE,	'OPIM', "\pMicrosoft Entourage"},	/* Verified -> 9.0 */
	{TRUE,	TRUE,	'MOSS', "\pNetscape Communicator"},	/* Verified -> 4.73 */
	{TRUE,	TRUE,	'MSNM', "\pOutlook Express"},		/* Verified -> 5.0 */
	{FALSE,	FALSE,	'ttxt', "\pSimpleText"},			/* Verified -> 1.4 */
	{FALSE,	TRUE,	'Cmlt',	"\pPowerMail"}//,				/* Verified -> 3.0.3 */
//	{FALSE,	FALSE,	'BLTO',	"\pMailsmith"}				/* Verified -> 1.1 */
};

#endif	/* ] */

const PGPPrefDefinition clientDefaults[] =
{
	/* Boolean prefs */

	{kPGPPrefEncryptToSelf,
	"EncryptToSelf",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWordWrapEnable,
	"WordWrap",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefShareCache,
	"ShareCache",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefMailEncryptPGPMIME,
	"MailEncryptPGPMIME",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefMailSignPGPMIME,
	"MailSignPGPMIME",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefFastKeyGen,
	"FastKeyGen",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefMailEncryptDefault,
	"MailEncryptDefault",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefMailSignDefault,
	"MailSignDefault",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefMarginalIsInvalid,
	"MarginalValidityIsInvalid",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefDisplayMarginalValidity,
	"DisplayMarginalValidity",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefFirstKeyGenerated,
	"FirstKeyGenerated",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefWarnOnRSARecipAndNonRSASigner,
	"WarnOnRSARecipAndNonRSASigner",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefKeyServerSyncUnknownKeys,
	"KeyServerSyncUnknownKeys",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefKeyServerSyncOnAdd,
	"KeyServerSyncOnAdd",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefKeyServerSyncOnKeySign,
	"KeyServerSyncOnKeySign",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefKeyServerSyncOnRevocation,
	"KeyServerSyncOnRevocation",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefKeyServerSyncOnVerify,
	"KeyServerSyncOnVerify",
	kPGPPrefType_Boolean,
	(void *) TRUE},

    {kPGPPrefDiskDefaultMountAtStartup,
	"DiskDefaultMountAtStartup",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWarnOnReadOnlyKeyRings,
	"WarnOnReadOnlyKeyRings",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWarnOnWipe,
	"WarnOnWipe",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAutoDecrypt,
	"AutoDecrypt",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefWarnOnADK,
	"WarnOnADK",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWarnOnMemLockFailure,
	"WarnOnMemLockFailure",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefExportKeysCompatible,
	"ExportKeysCompatible",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefHookCBTWin,
	"HookCBTWin",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAlwaysUseSecureViewer,
	"AlwaysUseSecureViewer",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefUseHexFingerprint,
	"UseHexFingerprint",
	kPGPPrefType_Boolean,
	(void *) FALSE},

    {kPGPPrefDiskWarnOnDeletion,
	"DiskWarnOnDeletion",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefDiskAutoUnmountOnLogout,
	"DiskAutoUnmountOnLogout",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefDiskAutoUnmountOnSleep,
	"DiskAutoUnmountOnSleep",
	kPGPPrefType_Boolean,
	(void *) TRUE},

/*	{kPGPPrefDiskBrowseAfterMount,
	"DiskBrowseAfterMount",
	kPGPPrefType_Boolean,
	(void *) TRUE},*/

	{kPGPPrefDiskNoSleepIfFailAU,
	"DiskNoSleepIfFailAU",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefDiskUnmountOnInactivity,
	"DiskUnmountOnInactivity",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefDiskWarnIfUnmountFail,
	"DiskWarnIfUnmountFail",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefDiskAllowForcibleUnmounts,
	"DiskAllowForcibleUnmounts",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefDiskAlwaysForceUnmounts,
	"DiskAlwaysForceUnmounts",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefTrashIntegration,
	"IntegrateWipeWithTrash",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAutoWipeOnDelete,
	"AutoWipeOnDelete",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefShowWipeProgress,
	"ShowWipeProgress",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefShowTrayIcon,
	"ShowTrayIcon",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefShowPGPkeysToolbar,
	"ShowPGPkeysToolbar",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefUseOutgoingADK,
	"UseOutgoingADK",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefUseDHADK,
	"UseDHADK",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefEnforceIncomingADK,
	"EnforceIncomingADK",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefEnforceMinChars,
	"EnforceMinChars",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefEnforceMinQuality,
	"EnforceMinQuality",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefWarnNotCertByCorpIsNoLongerUsed,
	"Unused",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAutoSignTrustCorp,
	"AutoSignTrustCorp",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefMetaIntroducerCorp,
	"MetaIntroducerCorp",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAllowConventionalEncryption,
	"AllowConventionalEncryption",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefEnforceOutgoingADK,
	"EnforceOutgoingADK",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefEnforceRemoteADKClass,
	"EnforceRemoteADKClass",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefUpdateAllKeys,
	"UpdateAllKeys",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefUpdateTrustedIntroducers,
	"UpdateTrustedIntroducers",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAutoAddRevoker,
	"AutoAddRevoker",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefUsePGPdiskADK,
	"UsePGPdiskADK",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefKeyGenX509CertRequest,
	"KeyGenX509CertRequest",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAllowManualX509CertRequest,
	"AllowManualX509CertRequest",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAutoUpdateX509CRL,
	"AutoUpdateX509CRL",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAllowKeyGeneration,
	"AllowKeyGeneration",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefPresetKeyGeneration,
	"PresetKeyGeneration",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAlwaysSendKeys,
	"AlwaysSendKeys",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAutoKeyReconstruction,
	"AutoKeyReconstruction",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefUpdatePrefs,
	"UpdatePrefs",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAddLotusNotesUserID,
	"AddLotusNotesUserID",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefTempestFont,
	"TempestFont",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefIdleAutoLockScreen,
	"LockScreen",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAutoBackupKeyring,
	"AutoBackupKeyring",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAutoBackupToKeyringFolder,
	"BackupToKeyringFolder",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAutoSignRootCAKey,
	"AutoSignRootCAKey",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAllowEncryptInvalidKeys,
	"AllowEncryptInvalid",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefAllowKeySigning,
	"AllowKeySigning",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefDiskUseDefaultKey,
	"DiskUseDefaultKey",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefCopyClientPrefs, 
	"CopyClientPrefs", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefCopyNetHosts, 
	"CopyNetHosts", 
	kPGPPrefType_Boolean, 
	(void *) FALSE},

	{kPGPPrefMergeNetHosts, 
	"MergeNetHosts", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefPreselectInstall, 
	"PreselectInstall", 
	kPGPPrefType_Boolean, 
	(void *) FALSE},

	{kPGPPrefInstallPrograms, 
	"InstallPrograms", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallLogin, 
	"InstallLogin", 
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefInstallNet, 
	"InstallNet", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallFire, 
	"InstallFire", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallDisk, 
	"InstallDisk", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallEudora, 
	"InstallEudora", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallOutlook, 
	"InstallOutlook", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallOutlookExpress, 
	"InstallOutlookExpress", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallICQ, 
	"InstallICQ", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallLotusNotes, 
	"InstallLotusNotes", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefDiskCreate, 
	"DiskCreate", 
	kPGPPrefType_Boolean, 
	(void *) FALSE},

	{kPGPPrefInstallGroupwise, 
	"InstallGroupwise", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefInstallDocumentation, 
	"InstallDocumentation", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefUninstallOldPGP, 
	"UninstallOldPGP", 
	kPGPPrefType_Boolean, 
	(void *) FALSE}, // Changed to FALSE by wjb

	{kPGPPrefRebootAfterSilentInstall, 
	"RebootAfterSilentInstall", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefRemainSilentAfterInstall,
	"RemainSilentAfterInstall",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefAutoAddExchangeServerAddress, 
	"AutoAddExchangeServerAddress", 
	kPGPPrefType_Boolean, 
	(void *) TRUE},

	{kPGPPrefGenerateKeysOnSmartCard, 
	"GenerateKeysOnSmartCard", 
	kPGPPrefType_Boolean, 
	(void *) FALSE},

	{kPGPPrefDisableAutoUpgrade, 
	"DisableAutoUpgrade", 
	kPGPPrefType_Boolean, 
	(void *) FALSE},

	{kPGPPrefReportToEPO, 
	"ReportToEPO", 
	kPGPPrefType_Boolean, 
	(void *) FALSE},
	
#if PGP_WIN32
	
	{kPGPPrefWin32OutlookConvertMessages,
	"OutlookConvertToPlainText",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPPrefWin32OutlookAttachment,
	"OutlookWarnAttachment",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWin32OutlookHTMLEncrypt,
	"OutlookWarnHTMLEncrypt",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWin32OutlookHTMLDecrypt,
	"OutlookWarnHTMLRTFDecrypt",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWin32EudoraOutputPGPMIME,
	"EudoraWarnOutputPGPMIME",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWin32OutlookNotFromExplorer,
	"OutlookWarnNotFromExplorer",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPPrefWin32OutlookExchangeID,
	"OutlookExchangeID",
	kPGPPrefType_Boolean,
	(void *) TRUE},

#endif

	/* Number prefs */

	{kPGPPrefWordWrapWidth,
	"WordWrapWidth",
	kPGPPrefType_Number,
	(void *) 70},

	{kPGPPrefCacheType,
	"CacheType",
	kPGPPrefType_Number,
	(void *) kPGPPrefCacheTypeSeconds},

	{kPGPPrefCacheSeconds,
	"CacheSeconds",
	kPGPPrefType_Number,
	(void *) 120},

	{kPGPPrefPreferredAlgorithm,
	"PreferredAlgorithm",
	kPGPPrefType_Number,
	(void *) kPGPCipherAlgorithm_AES256},

	{kPGPPrefMacBinaryDefault,
	"MacBinaryDefault",
	kPGPPrefType_Number,
	(void *) kPGPPrefMacBinaryOff},

	{kPGPPrefFileWipePasses,
	"FileWipePasses",
	kPGPPrefType_Number,
	(void *) 3},

	{kPGPPrefDiskWipePasses,
	"DiskWipePasses",
	kPGPPrefType_Number,
	(void *) 3},

	{kPGPPrefLastAllKeysUpdate,
	"LastAllKeysUpdate",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefLastTrustedIntroducersUpdate,
	"LastTrustedIntroducersUpdate",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefCAType,
	"CAType",
	kPGPPrefType_Number,
	(void *) kPGPKeyServerClass_Netscape},

	{kPGPPrefNextAutoCRLUpdate,
	"NextAutoCRLUpdate",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefDiskLastCreateSize,
	"DiskLastCreateSize",
	kPGPPrefType_Number,
	(void *) 100},

	{kPGPPrefDiskLastCreateScale,
	"DiskLastCreateScale",
	kPGPPrefType_Number,
	(void *) 1},	/* is PGPdiskMbScale */

	{kPGPPrefDiskAutoUnmountTimeout,
	"DiskAutoUnmountTimeout",
	kPGPPrefType_Number,
	(void *) 15},

	{kPGPPrefDiskLastAlgorithm,
	"DiskLastAlgorithm",
	kPGPPrefType_Number,
	(void *) 3},	/* is kPGPdiskCAST5Algorithm */

	{kPGPPrefDiskLastFileSystem,
	"DiskLastFileSystem",
	kPGPPrefType_Number,
	(void *) 0},	/* is kFat16FileSys */

	{kPGPPrefLastPrefsUpdate,
	"LastPrefsUpdate",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefMetaIntroducerTrustDepth,
	"MetaIntroducerTrustDepth",
	kPGPPrefType_Number,
	(void *) 2},

	{kPGPPrefIdleAutoLockScreenMinutes,
	"LockScreenMinutes",
	kPGPPrefType_Number,
	(void *) 60},

	{kPGPPrefMinChars,
	"MinPassphraseChars",
	kPGPPrefType_Number,
	(void *) 5},

	{kPGPPrefMinQuality,
	"MinPassphraseQuality",
	kPGPPrefType_Number,
	(void *) 60},

	{kPGPPrefDaysUpdateAllKeys,
	"DaysUpdateAllKeys",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefDaysUpdateTrustedIntroducers,
	"DaysUpdateTrustedIntroducers",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefKeySize,
	"KeySize",
	kPGPPrefType_Number,
	(void *) 2048},

	{kPGPPrefDaysUpdatePrefs,
	"DaysUpdatePrefs",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefKeyPublicKeyAlgorithm,
	"KeyPublicKeyAlgorithm",
	kPGPPrefType_Number,
	(void *) kPGPPublicKeyAlgorithm_DSA},

	{kPGPPrefKeyExpiration,
	"KeyExpiration",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefLDAPReconServerType,
	"LDAPReconServerType",
	kPGPPrefType_Number,
	(void *) kPGPKeyServerClass_PGP},

	{kPGPPrefCertRequestType,
	"CertRequestType",
	kPGPPrefType_Number,
	(void *) kPGPPrefCertRequestCRS},

	{kPGPPrefCorpKeyTrustDepth,
	"CorpKeyTrustDepth",
	kPGPPrefType_Number,
	(void *) 2},

	{kPGPPrefDiskSize,
	"DiskSize",
	kPGPPrefType_Number,
	(void *) 204800},

	{kPGPPrefDiskAlgorithm,
	"DiskAlgorithm",
	kPGPPrefType_Number,
	(void *) 3},

	{kPGPPrefDiskFileSystem,
	"DiskFileSystem",
	kPGPPrefType_Number,
	(void *) 1},

	{kPGPPrefLastCertRequest,
	"LastCertRequest",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPPrefLDAPPrefsServerAlg,
	"LDAPPrefsServerAlg",
	kPGPPrefType_Number,
	(void *) kPGPPublicKeyAlgorithm_Invalid},

	{kPGPPrefLastSearchWindowSelection,
	"LastSearchWindowSelection",
	kPGPPrefType_Number,
#if PGP_WIN32
	(void *) 2},
#else
	(void *) 3},
#endif

#if	PGP_MACINTOSH	/* [ */

	{kPGPPrefAutoBackupDirIDMac,
	"AutoBackupDirIDMac",
	kPGPPrefType_Number,
	(void *) 0},

#elif PGP_WIN32

	{kPGPPrefUserAccountStatus,
	"UserAccountStatus",
	kPGPPrefType_Number,
	(void *) kPGPPrefPGPNotInitialized},

#endif	/* ] */

	/* String prefs */

	{kPGPPrefComment,
	"Comment",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefCAServerURL,
	"CAServerURL",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefCARevocationServerURL,
	"CARevocationServerURL",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefDiskLastOpenDir,
	"DiskLastOpenDir",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefDiskLastSaveDir,
	"DiskLastSaveDir",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefDiskLastCreateRoot,
	"DiskLastCreateRoot",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefPublicKeyringFile,
	"PublicKeyringFile",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefPrivateKeyringFile,
	"PrivateKeyringFile",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefRNGSeedFile,
	"RNGSeedFile",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefPKCS11DrvFile,
	"PKCS11DrvFile",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefLicenseName,
	"LicenseName",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefLicenseCompany,
	"LicenseCompany",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefLDAPPrefsServer,
	"LDAPPrefsServer",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefLDAPReconServer,
	"LDAPReconServer",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefDiskMountAtStartupPaths,
	"DiskMountAtStartupPaths",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefAutoBackupFolder,
	"KeyringBackupFolder",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefInstallDir,
	"InstallDir",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPPrefLDAPPrefsServerKeyID,
	"LDAPPrefsServerKeyID",
	kPGPPrefType_String,
	(void *) ""},

#if PGP_MACINTOSH

	{kPGPPrefCompanyName,
	"Company Name",
	kPGPPrefType_String,
	(void *) ""},
	
#endif

	/* Byte prefs */

#if	PGP_MACINTOSH	/* [ */

	{kPGPPrefPGPmenuMacAppSignatures,
	"PGPmenuMacAppSignatures",
	kPGPPrefType_Byte,
	(void *) sInitPGPmenuApps,
	sizeof(sInitPGPmenuApps)},

	{kPGPPrefHotKeyDataMac,
	"HotKeysDataMac",
	kPGPPrefType_Byte,
	(void *) sInitPGPmenuHotkeys,
	sizeof(sInitPGPmenuHotkeys)},

	{kPGPPrefPGPtoolsMacPrivateData,
	"PGPtoolsMacPrivateData",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefDiskEditorWinPosMac,
	"DiskEditorWinPosMac",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefPublicKeyringFileMac,
	"PublicKeyringFileMac",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefPrivateKeyringFileMac,
	"PrivateKeyringFileMac",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefRNGSeedFileMac,
	"RNGSeedFileMac",
	kPGPPrefType_Byte,
	NULL},

#endif	/* ] */

#if PGP_MACINTOSH || PGP_OSX

	{kPGPPrefPGPkeysMacColumnData,
	"PGPkeysMacColumnData",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefPGPkeysMacMainWinPos,
	"PGPkeysMacMainWinPos",
	kPGPPrefType_Byte,
	NULL},
	
#endif

#if PGP_WIN32	/* [ */

	{kPGPPrefPGPkeysWinMainWinPos,
	"PGPkeysWinMainWinPos",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefPGPkeysWinColumnData,
	"PGPkeysWinColumnData",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefPGPkeysWinGroupsData,
	"PGPkeysWinGroupsData",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefDiskEditorWinPos,
	"DiskEditorWinPos",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefHotKeyData,
	"HotKeyData",
	kPGPPrefType_Byte,
	NULL},

#endif	/* ] */

	{kPGPPrefCARootKeyID,
	"CARootKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefCARootX509IASN,
	"CARootX509IASN",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefCAServerAuthKeyID,
	"CAServerAuthKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefCARevocationServerAuthKeyID,
	"CARevocationServerAuthKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefOutgoingADKID,
	"OutgoingADKKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefDHADKID,
	"DiffieHellmanADKKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefCorpKeyID,
	"CorporateKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefRevokerKeyID,
	"RevokerKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefPGPdiskADKKeyID,
	"PGPdiskADKKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefDefaultKeyID,
	"DefaultKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefLoginWindowsInfo,
	"LoginWindowsInfo",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefExtraAVPairs,
	"ExtraAVPairs",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefRequestedCertKeyID,
	"RequestedCertKeyID",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefLicenseNumber,
	"LicenseNumber",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefLicenseAuthorization,
	"LicenseAuthorization",
	kPGPPrefType_Byte,
	NULL},

	{kPGPPrefAdminHash,
	"AdminHash",
	kPGPPrefType_Byte,
	(void *) sBlankAdminHash,
	sizeof(sBlankAdminHash)},

	/* Array prefs */

	{kPGPPrefDefaultKeys,
	"DefaultKeys",
	kPGPPrefType_Array,
	(void *) &DefaultKeys,
	sizeof(DefaultKeys)},

	{kPGPPrefKeyServerList,
	"KeyServerList",
	kPGPPrefType_Array,
	(void *) &DefaultKeyServer,
	sizeof(DefaultKeyServer)},

	{kPGPPrefAllowedAlgorithmsList,
	"AllowedAlgorithmsList",
	kPGPPrefType_Array,
	(void *) &sAllowedAlgorithms,
	sizeof(sAllowedAlgorithms)}
};

const PGPSize clientDefaultsSize =
						sizeof(clientDefaults) / sizeof(PGPPrefDefinition);

/*	... end of code formerly in pgpClientPrefs.h */
