/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPMacOSCompatibility.mm,v 1.4 2002/08/06 20:10:55 dallen Exp $
____________________________________________________________________________*/

#import <Foundation/Foundation.h>

#include "pgpBase.h"
#include "pflTypes.h"
#include "pgpFileSpec.h"
#include "pgpMacFileMapping.h"
#include "pgpMacUtils.h"
#include "MacFiles.h"

#include "PGPMacOSCompatibility.h"



	PGPError 
PFLNewFileSpecFromFSSpec(
	PGPMemoryMgrRef			context,
	struct FSSpec const *	spec,
	PFLFileSpecRef *		outRef )
{
	PGPError			result;
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSString *			name = [((NSString *) CFStringCreateWithPascalString(NULL, spec->name, CFStringGetSystemEncoding()))
									autorelease];
	FSSpec				par;
	OSStatus			err;
	FSRef				fsRef;
	
	err = FSMakeFSSpec(spec->vRefNum, spec->parID, nil, &par);
	if (! err) {
		err = FSpMakeFSRef(&par, &fsRef);
		if (! err) {
			NSURL *		url = [((NSURL *) CFURLCreateFromFSRef(NULL, &fsRef)) autorelease];
			NSString *	path = [((NSString *) CFURLCopyFileSystemPath((CFURLRef) url, kCFURLPOSIXPathStyle)) autorelease];

			result = PFLNewFileSpecFromFullPath(context, [[path stringByAppendingPathComponent:name] cString], outRef);
		}
	}
	if (err) {
		result = pgpErrorFromMacError(err, kPGPError_UnknownError);
	}
	[pool release];
	
	return result;
}



	PGPError
PFLGetFSSpecFromFileSpec(
	PFLConstFileSpecRef		fileRef,
	struct FSSpec *			spec)
{
	PGPError	result;
	OSStatus	err;
	char *		path;
	
	result = PFLGetFullPathFromFileSpec(fileRef, &path);
	if (IsntPGPError(result)) {
		NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
		NSURL *				url = [NSURL fileURLWithPath:[NSString stringWithCString:path]];
		NSURL *				parURL = [((NSURL *) CFURLCreateCopyDeletingLastPathComponent(NULL, (CFURLRef) url)) autorelease];
		NSString *			name = [((NSString *) CFURLCopyLastPathComponent((CFURLRef) url)) autorelease];
		FSRef				parRef;
		FSCatalogInfo		catInfo;
		Str32				pName;
		
		PGPFreeData(path);
		if (CFURLGetFSRef(((CFURLRef) parURL), &parRef)) {
			CFStringGetPascalString((CFStringRef) name, pName, kHFSMaxFileNameChars + 1, CFStringGetSystemEncoding());
			err = FSGetCatalogInfo(&parRef, kFSCatInfoVolume | kFSCatInfoNodeID, &catInfo, NULL, NULL, NULL);
			if (! err) {
				err = FSMakeFSSpec(catInfo.volume, catInfo.nodeID, pName, spec);
				if (err == fnfErr) {
					err = noErr;
				}
			}
			if (err) {
				result = pgpErrorFromMacError(err, kPGPError_UnknownError);
			} else {
				result = kPGPError_NoErr;
			}
		} else {
			result = kPGPError_UnknownError;
		}
		[pool release];
	}

	return result;
}



	void
pgpOSXSetFileCreatorAndType(
	const char *	inPath,
	OSType			inCreator,
	OSType			inType)
{
	if (inCreator || inType) {
		NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
		
		[[NSFileManager defaultManager] changeFileAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithLong:inCreator], NSFileHFSCreatorCode, [NSNumber numberWithLong:inType], NSFileHFSTypeCode, NULL]
			atPath:[NSString stringWithCString:inPath]];
		[pool release];
	}
}



	OSStatus
FSpGetUniqueSpec(
	const FSSpec *	spec,
	FSSpec *		outSpec)
{
	OSStatus			result;

	result = FSMakeFSSpec(spec->vRefNum, spec->parID, spec->name, outSpec);
	if (! result) {
		NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
		NSMutableString *	base = [NSMutableString string];
		NSMutableString *	n;
		Str63				name;
		int					i = 1;

		CFStringAppendPascalString((CFMutableStringRef) base, spec->name, CFStringGetSystemEncoding());
		do {
			n = [NSMutableString stringWithFormat:@" %d", i];
			while (([base length] + [n length]) > kHFSMaxFileNameChars) {
				[base deleteCharactersInRange: NSMakeRange([base length] - 1, 1)];
			}
			[n insertString:base atIndex:0];
			if (CFStringGetPascalString((CFStringRef) n, name, kHFSMaxFileNameChars + 1, CFStringGetSystemEncoding())) {
				result = FSMakeFSSpec(spec->vRefNum, spec->parID, name, outSpec);
			} else {
				result = bdNamErr;
			}
			i++;
		} while (! result);
		[pool release];
	}
	if (result == fnfErr) {
		result = noErr;
	}
	
	return result;
}



	PGPBoolean
PStringHasSuffix(
	ConstStr255Param	str,
	ConstStr255Param	suffix,
	PGPBoolean			caseSensitive)
{
	PGPBoolean			result = FALSE;
	
	if (str[0] >= suffix[0]) {
		int 			i;
		unsigned char	a;
		unsigned char	b;
		
		for (i = 0; i < suffix[0]; i++) {
			if (caseSensitive) {
				a = str[str[0] - i];
				b = suffix[suffix[0] - i];
			} else {
				a = toupper(str[str[0] - i]);
				b = toupper(suffix[suffix[0] - i]);
			}
			if (a != b) {
				break;
			}
		}
		if (i == suffix[0]) {
			result = TRUE;
		}
	}
	
	return result;
}
