/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: CPGPservice.mm,v 1.19 2002/09/30 06:41:23 jason Exp $
____________________________________________________________________________*/

#import <unistd.h>
#import <grp.h>
#import <pwd.h>
#import <mach/mach_error.h>
#import <mach/task.h>
#import <mach/mach_traps.h>

#import "CPGPservice.h"
#import "pgpMem.h"


class StSetCredentials {
public:
	static void				SetPGPContext(PGPContextRef inContext) { sContext = inContext; }

	explicit				StSetCredentials(NSMachPort * inPort, PGPError * outResult);
							~StSetCredentials();

private:
	static PGPContextRef	sContext;
	BOOL					mRestore;
	uid_t					mOldUID;
	gid_t					mOldGID;
};



PGPContextRef			StSetCredentials::sContext = kInvalidPGPContextRef;
static int				sSignalPipeDescriptor = -1;
CPGPservice *			gApp = NULL;


	int
main(
	int				argc,
	const char *	argv[])
{
	NS_DURING
		gApp = [[CPGPservice alloc] init];
		[[NSRunLoop currentRunLoop] run];
	NS_HANDLER
		[gApp terminate:NULL];
		NSLog(@"%@", localException);
	NS_ENDHANDLER
	
	return 0;
}



	static void
sHandleSignals(
	int	sig)
{
	write(sSignalPipeDescriptor, &sig, sizeof(sig));
}
	


@implementation CPGPservice

	- (id)
init
{
	if (self = [super init]) {
		NSAutoreleasePool *		pool = [[NSAutoreleasePool alloc] init];
		NSNotificationCenter *	center = [NSNotificationCenter defaultCenter];
		NSConnection *			connection = [NSConnection defaultConnection];
		struct sigaction		act;
		
		stopTimerSeconds = 300;

		//	Handle signals
		signalPipe = [[NSPipe alloc] init];
		NSAssert(signalPipe, @"Couldn't create pipe for signaling");
		sSignalPipeDescriptor = [[signalPipe fileHandleForWriting] fileDescriptor];
		[[signalPipe fileHandleForReading] readInBackgroundAndNotify];
		[center addObserver:self selector:@selector(handleSignals:) name:NSFileHandleReadCompletionNotification object:NULL];
		act.sa_handler = sHandleSignals;
		sigemptyset(&act.sa_mask);
		act.sa_flags = 0;
		NSAssert(! sigaction(SIGTERM, &act, NULL), @"Couldn't install SIGTERM handler");
		NSAssert(! sigaction(SIGINT, &act, NULL), @"Couldn't install SIGINT handler");
		NSAssert(! sigaction(SIGHUP, &act, NULL), @"Couldn't install SIGHUP handler");
		
		//	Vend ourselves with DO
		[connection setRootObject:[NSProtocolChecker protocolCheckerWithTarget:self protocol:@protocol(PGPserviceProtocol)]];
		NSAssert([connection registerName:kPGPserviceID], @"Unable to register PGPservice with bootstrap server");
		[connection setIndependentConversationQueueing:YES];
		[center addObserver:self selector:@selector(connectionInitialized:) name:NSConnectionDidInitializeNotification
			object:NULL];
		[center addObserver:self selector:@selector(connectionDied:) name:NSConnectionDidDieNotification object:NULL];
		
		//	Listen for logouts
		[[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self selector:@selector(handleSignals:) name:NSWorkspaceWillPowerOffNotification object:nil];
		
		//	Initialize the sdk
		NSAssert(IsntPGPError(PGPsdkInit(kPGPFlags_ForceLocalExecution | kPGPFlags_SuppressCacheThread)), @"Unable to initialize PGP sdk");
		sdkInitialized = YES;
		NSAssert(IsntPGPError(PGPNewContext(kPGPsdkAPIVersion, &pgpContext)), @"Unable to create a pgp context");
		StSetCredentials::SetPGPContext(pgpContext);
		
		[self scheduleStopTimer];
		[self startCacheTimer];
		[pool release];
	}
	
	return self;
}



	- (void)
dealloc
{
	if (PGPContextRefIsValid(pgpContext)) {
		PGPFreeContext(pgpContext);
	}
	if (sdkInitialized) {
		PGPsdkCleanup();
	}
	sSignalPipeDescriptor = -1;
	[signalPipe release];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];
	[super dealloc];
}



	- (void)
terminate
	:(id)	sender
{
	NSConnection *	connection = [NSConnection defaultConnection];
	
	[connection registerName:NULL];
	[connection setRootObject:NULL];
	[stopTimer invalidate];
	stopTimer = NULL;
	if (connCount == 0) {
		[cacheTimer invalidate];
		cacheTimer = NULL;
		[self release];
		NSLog(@"Shutting down");
		exit(0);
	} else {
		NSLog(@"Shutting down: Waiting for %d open connection%s...", (int) connCount, (connCount == 1) ? "" : "s");
	}
}



	- (void)
handleSignals
	:(id)	notification
{
	stopTimerSeconds = 0;
	[self terminate:notification];
}


	- (void)
scheduleStopTimer
{
	[self cancelStopTimer];
	stopTimer = [NSTimer scheduledTimerWithTimeInterval:stopTimerSeconds target:self selector:@selector(terminate:) userInfo:NULL repeats:NO];
}



	- (void)
cancelStopTimer
{
	[stopTimer invalidate];
	stopTimer = NULL;
}



	- (void)
startCacheTimer
{
	cacheTimer = [NSTimer scheduledTimerWithTimeInterval:15 target:self selector:@selector(checkCacheExpirations:) userInfo:NULL repeats:YES];
}



	- (void)
checkCacheExpirations
	:(NSTimer *)	timer
{
	pgpExpirePassphraseCache(pgpContext);
	pgpExpireKeyDBCache(pgpContext);
}



	- (void)
connectionInitialized
	:(NSNotification *)	notification
{
	connCount++;
	[self cancelStopTimer];
}



	- (void)
connectionDied
	:(NSNotification *)	notification
{
	connCount--;
	if (connCount == 0) {
		[self scheduleStopTimer];
	}
}



	- (PGPError)
pgpConnect_back
	:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);

	if (IsntPGPError(result)) {
		pgpPassphraseCacheAddClient(pgpContext, pgpContextGetConnectRef(pgpContext));
	}
	
	return result;
}



	- (PGPError)
pgpReconnect_back
	:(NSMachPort *)	task
{
	return kPGPError_NoErr;
}



	- (PGPError)
pgpDisconnect_back
	:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);

	if (IsntPGPError(result)) {
		pgpPassphraseCacheRemoveClient(pgpContextGetConnectRef(pgpContext));
	}

	return result;
}



	- (PGPError)
pgpFetchObjectData_back
	:(PGPUInt32)				objID
	bufPtr:(out NSData **)		bufPtr
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpFetchObjectData_back(pgpContext, objID, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*bufPtr = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpGetKeyByKeyID_back
	:(PGPUInt32)		dbid
	keyIDIn:(NSData *)	keyIDIn
	dummyOK:(BOOL)		dummyOK
	deletedOK:(BOOL)	deletedOK
	outID:(PGPUInt32 *)	outID
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpGetKeyByKeyID_back(pgpContext, dbid, (const PGPKeyID *) [keyIDIn bytes], dummyOK, deletedOK, outID);
	}
	
	return result;
}



	- (PGPError)
pgpKeyEncrypt_back
	:(PGPUInt32)						objID
	inbuf:(NSData *)					inbuf
	format:(PGPPublicKeyMessageFormat)	format
	outbuf:(NSData **)					outbuf
	task:(NSMachPort *)					task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpKeyEncrypt_back(pgpContext, objID, (const PGPByte *) [inbuf bytes], [inbuf length], format, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*outbuf = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpKeyDecrypt_back
	:(PGPUInt32)						objID
	passphrase:(NSData *)				passphrase
	hashedPhrase:(BOOL)					hashedPhrase
	cacheTimeOut:(PGPUInt32)			cacheTimeOut
	cacheGlobal:(BOOL)					cacheGlobal
	inbuf:(NSData *)					inbuf
	format:(PGPPublicKeyMessageFormat)	format
	outbuf:(NSData **)					outbuf
	task:(NSMachPort *)					task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpKeyDecrypt_back(pgpContext, objID, (const PGPByte *) [passphrase bytes], [passphrase length], hashedPhrase,
					cacheTimeOut, cacheGlobal, (const PGPByte *) [inbuf bytes], [inbuf length], format, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*outbuf = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPInt32)
pgpKeyVerify_back
	:(PGPUInt32)						objID
	inbuf:(NSData *)					inbuf
	hashalg:(PGPHashAlgorithm)			hashalg
	hash:(NSData *)						hash
	format:(PGPPublicKeyMessageFormat)	format
	task:(NSMachPort *)					task
{
	PGPError			err;
	StSetCredentials	creds(task, &err);
	PGPInt32			result;

	if (IsntPGPError(err)) {
		result = pgpKeyVerify_back(pgpContext, objID, (const PGPByte *) [inbuf bytes], [inbuf length], hashalg, (const PGPByte *) [hash bytes], [hash length],
					format);
	} else {
		result = -1;
	}
	
	return result;
}



	- (PGPError)
pgpKeySign_back
	:(PGPUInt32)						objID
	passphrase:(NSData *)				passphrase
	hashedPhrase:(BOOL)					hashedPhrase
	cacheTimeOut:(PGPUInt32)			cacheTimeOut
	cacheGlobal:(BOOL)					cacheGlobal
	hashalg:(PGPHashAlgorithm)			hashalg
	hash:(NSData *)						hash
	format:(PGPPublicKeyMessageFormat)	format
	outbuf:(NSData **)					outbuf
	task:(NSMachPort *)					task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpKeySign_back(pgpContext, objID, (const PGPByte *) [passphrase bytes], [passphrase length], hashedPhrase, cacheTimeOut,
					cacheGlobal, hashalg, (const PGPByte *) [hash bytes], [hash length], format, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*outbuf = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (BOOL)
pgpSecPassphraseOK_back
	:(PGPUInt32)				objID
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	task:(NSMachPort *)			task
{
	PGPError			err;
	StSetCredentials	creds(task, &err);
	BOOL				result;
	
	if (IsntPGPError(err)) {
		result = pgpSecPassphraseOK_back(pgpContext, objID, (const PGPByte *) [passphrase bytes], [passphrase length], hashedPhrase, cacheTimeOut,
					cacheGlobal);
	} else {
		result = FALSE;
	}
	
	return result;
}



	- (PGPError)
pgpPurgePassphraseCache_back
	:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpPurgePassphraseCache_back(pgpContext);
	}
	
	return result;
}



	- (PGPError)
pgpCountCachedPassphrases_back
	:(PGPUInt32 *)				nLocals
	nGlobals:(PGPUInt32 *)		nGlobals
	nOtherLocals:(PGPUInt32 *)	nOtherLocals
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpCountCachedPassphrases_back(pgpContext, nLocals, nGlobals, nOtherLocals);
	}
	
	return result;
}



	- (PGPError)
pgpKeyMaxSizes_back
	:(PGPUInt32)						objID
	maxEncryption:(PGPUInt32 *)			maxEncryption
	maxDecryption:(PGPUInt32 *)			maxDecryption
	maxSignature:(PGPUInt32 *)			maxSignature
	format:(PGPPublicKeyMessageFormat)	format
	task:(NSMachPort *)					task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpKeyMaxSizes_back(pgpContext, objID, maxEncryption, maxDecryption, maxSignature, format);
	}
	
	return result;
}



	- (PGPError)
pgpSecProperties_back
	:(PGPUInt32)					objID
	needsPassphrase:(BOOL *)		needsPassphrase
	isSecretShared:(BOOL *)			isSecretShared
	lockAlg:(PGPCipherAlgorithm *)	lockAlg
	lockBits:(PGPUInt32 *)			lockBits
	task:(NSMachPort *)				task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpSecProperties_back(pgpContext, objID, reinterpret_cast<PGPBoolean *>(needsPassphrase),
					reinterpret_cast<PGPBoolean *>(isSecretShared), lockAlg, lockBits);
	}
	
	return result;
}



	- (PGPError)
pgpFetchKeyInfo_back
	:(PGPUInt32)		objID
	bufPtr:(NSData **)	bufPtr
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpFetchKeyInfo_back(pgpContext, objID, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*bufPtr = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpOpenKeyDBFile_back
	:(PGPOpenKeyDBFileOptions)	openFlags
	pub:(NSData *)				pub
	priv:(NSData *)				priv
	kdbid:(PGPUInt32 *)			kdbid
	numKeys:(PGPUInt32 *)		numKeys
	keyBuffer:(NSData **)		keyBuffer
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	PFLFileSpecRef		pubref;
	PFLFileSpecRef		privref;
	
	if (IsntPGPError(result)) {
		result = PFLImportFileSpec(PGPPeekContextMemoryMgr(pgpContext), (const PGPByte *) [pub bytes], [pub length], &pubref);
	}
	if (IsntPGPError(result)) {
		result = PFLImportFileSpec(PGPPeekContextMemoryMgr(pgpContext), (const PGPByte *) [priv bytes], [priv length], &privref);
	}
	if (IsntPGPError(result)) {
		result = pgpOpenKeyDBFile_back(pgpContext, openFlags, pubref, privref, kdbid, numKeys, &ptr, &len);
		PFLFreeFileSpec(pubref);
		PFLFreeFileSpec(privref);
	}
	if (IsntPGPError(result)) {
		*keyBuffer = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpImportKeyBinary_back
	:(NSData *)				buffer
	kdbid:(PGPUInt32 *)		kdbid
	numKeys:(PGPUInt32 *)	numKeys
	keyArray:(NSData **)	keyArray
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpImportKeyBinary_back(pgpContext, (const PGPByte *) [buffer bytes], [buffer length], kdbid, numKeys, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*keyArray = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpKeyDBFlush_back
	:(PGPUInt32)				objID
	changedkeylist:(NSData **)	changedkeylist
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpKeyDBFlush_back(pgpContext, objID, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*changedkeylist = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (void)
pgpFreeKeyDB_back
	:(PGPUInt32)		objID
	task:(NSMachPort *)	task
{
	PGPError			err;
	StSetCredentials	creds(task, &err);
	
	if (IsntPGPError(err)) {
		pgpFreeKeyDB_back(pgpContext, objID);
	}
}



	- (PGPError)
pgpCacheKeyDB_back
	:(PGPUInt32)				objID
	timeoutSeconds:(PGPUInt32)	timeoutSeconds
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpCacheKeyDB_back(pgpContext, objID, timeoutSeconds);
	}
	
	return result;
}



	- (PGPError)
pgpPurgeKeyDBCache_back
	:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpPurgeKeyDBCache_back(pgpContext);
	}
	
	return result;
}



	- (PGPError)
pgpSetKeyEnabled_back
	:(PGPUInt32)		objID
	enable:(BOOL)		enable
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpSetKeyEnabled_back(pgpContext, objID, enable);
	}
	
	return result;
}



	- (PGPError)
pgpSetKeyAxiomatic_back
	:(PGPUInt32)				objID
	setAxiomatic:(BOOL)			setAxiomatic
	checkPassphrase:(BOOL)		checkPassphrase
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpSetKeyAxiomatic_back(pgpContext, objID, setAxiomatic, checkPassphrase, (const char *) [passphrase bytes],
					[passphrase length], hashedPhrase, cacheTimeOut, cacheGlobal);
	}
	
	return result;
}



	- (PGPError)
pgpPropagateTrust_back
	:(PGPUInt32)				setid
	keylist:(NSData *)			keylist
	altid:(PGPUInt32)			altid
	timenow:(PGPUInt32)			timenow
	changedkeylist:(NSData **)	changedkeylist
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [keylist length];
	
	if (IsntPGPError(result)) {
		ptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[keylist getBytes:ptr];
		result = pgpPropagateTrust_back(pgpContext, setid, ptr, len, altid, timenow, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*changedkeylist = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpCheckKeyRingSigs_back
	:(PGPUInt32)				setid
	keylist:(NSData *)			keylist
	altid:(PGPUInt32)			altid
	checkAll:(BOOL)				checkAll
	changedkeylist:(NSData **)	changedkeylist
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [keylist length];
	
	if (IsntPGPError(result)) {
		ptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[keylist getBytes:ptr];
		result = pgpCheckKeyRingSigs_back(pgpContext, setid, ptr, len, altid, checkAll, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*changedkeylist = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpPrepareToCheckKeyRingSigs_back
	:(PGPUInt32)				setid
	keylist:(NSData *)			keylist
	altid:(PGPUInt32)			altid
	checkAll:(BOOL)				checkAll
	nsigs:(PGPUInt32 *)			nsigs
	changedkeylist:(NSData **)	changedkeylist
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [keylist length];
	
	if (IsntPGPError(result)) {
		ptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[keylist getBytes:ptr];
		result = pgpPrepareToCheckKeyRingSigs_back(pgpContext, setid, ptr, len, altid, checkAll, nsigs,
					&ptr, &len);
	}
	if (IsntPGPError(result)) {
		*changedkeylist = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpCheckSig_back
	:(PGPUInt32)			setid
	objid:(PGPUInt32)		objid
	altid:(PGPUInt32)		altid
	checkAll:(BOOL)			checkAll
	revocationonly:(BOOL)	revocationonly
	handled:(BOOL *)		handled
	changed:(BOOL *)		changed
	verified:(BOOL *)		verified
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpCheckSig_back(pgpContext, setid, objid, altid, checkAll, revocationonly, reinterpret_cast<PGPBoolean *>(handled),
					reinterpret_cast<PGPBoolean *>(changed), reinterpret_cast<PGPBoolean *>(verified));
	}
	
	return result;
}



	- (PGPError)
pgpDoGenerateKey_back
	:(PGPUInt32)						keydbID
	masterkeyID:(PGPUInt32)				masterkeyID
	pkalg:(PGPByte)						pkalg
	bits:(PGPUInt32)					bits
	creationDate:(PGPTime)				creationDate
	expirationDays:(PGPUInt16)			expirationDays
	name:(NSData *)						name
	passphrase:(NSData *)				passphrase
	passphraseIsKey:(BOOL)				passphraseIsKey
	masterpass:(NSData *)				masterpass
	cacheTimeOut:(PGPUInt32)			cacheTimeOut
	cacheGlobal:(BOOL)					cacheGlobal
	progress:(PGPEventHandlerProcPtr)	progress
	userValue:(PGPUserValue)			userValue
	fastgen:(BOOL)						fastgen
	checkentropy:(BOOL)					checkentropy
	usetoken:(BOOL)						usetoken
	tokenid:(PGPUInt32)					tokenid
	adkkeylist:(NSData *)				adkkeylist
	adkclass:(PGPByte)					adkclass
	rakkeylist:(NSData *)				rakkeylist
	rakclass:(PGPByte)					rakclass
	prefalg:(NSData *)					prefalg
	prefkeyserv:(NSData *)				prefkeyserv
	keyflags:(PGPUInt32)				keyflags
	fkeyflags:(BOOL)					fkeyflags
	keyservprefs:(PGPUInt32)			keyservprefs
	fkeyservprefs:(BOOL)				fkeyservprefs
	newobjs:(NSData **)					newobjs
	newKey:(PGPUInt32 *)				newKey
	task:(NSMachPort *)					task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [adkkeylist length];
	PGPUInt32 *			ptr2;
	PGPSize				len2 = [rakkeylist length];
	
	if (IsntPGPError(result)) {
		ptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		ptr2 = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len2, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr2)) {
			PGPFreeData(ptr);
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[adkkeylist getBytes:ptr];
		[rakkeylist getBytes:ptr2];
		result = pgpDoGenerateKey_back(pgpContext, keydbID, masterkeyID, pkalg, bits, creationDate, expirationDays,
					(char const *) [name bytes], [name length], (char const *) [passphrase bytes], [passphrase length], passphraseIsKey, (char const *) [masterpass bytes],
					[masterpass length], cacheTimeOut, cacheGlobal, progress, userValue, fastgen, checkentropy, usetoken,
					tokenid, ptr, len, adkclass, ptr2, len2, rakclass, (PGPCipherAlgorithm const *) [prefalg bytes], [prefalg length], (PGPByte const *) [prefkeyserv bytes],
					[prefkeyserv length], keyflags, fkeyflags, keyservprefs, fkeyservprefs, &ptr, &len, newKey);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpCreateKeypair_back
	:(PGPUInt32)				keydbID
	seckeyBuf:(NSData *)		seckeyBuf
	keyspecBuf:(NSData *)		keyspecBuf
	name:(NSData *)				name
	adklist:(NSData *)			adklist
	adkclass:(PGPByte)			adkclass
	raklist:(NSData *)			raklist
	rakclass:(PGPByte)			rakclass
	passphrase:(NSData *)		passphrase
	passphraseIsKey:(BOOL)		passphraseIsKey
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	newobjs:(NSData **)			newobjs
	newkeyid:(PGPUInt32 *)		newkeyid
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [seckeyBuf length];
	PGPByte *			seckeyptr;
	PGPByte *			keyspecptr;
	PGPSize				keyspeclen = [keyspecBuf length];
	PGPUInt32 *			adkptr;
	PGPSize				adklen = [adklist length];
	PGPUInt32 *			rakptr;
	PGPSize				raklen = [raklist length];
	
	if (IsntPGPError(result)) {
		seckeyptr = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(seckeyptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		keyspecptr = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), keyspeclen, kPGPMemoryMgrFlags_None);
		if (IsNull(keyspecptr)) {
			PGPFreeData(seckeyptr);
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		adkptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), adklen, kPGPMemoryMgrFlags_None);
		if (IsNull(adkptr)) {
			PGPFreeData(seckeyptr);
			PGPFreeData(keyspecptr);
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		rakptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), raklen, kPGPMemoryMgrFlags_None);
		if (IsNull(rakptr)) {
			PGPFreeData(seckeyptr);
			PGPFreeData(keyspecptr);
			PGPFreeData(adkptr);
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[seckeyBuf getBytes:seckeyptr];
		[keyspecBuf getBytes:keyspecptr];
		[adklist getBytes:adkptr];
		[raklist getBytes:rakptr];
		result = pgpCreateKeypair_back(pgpContext, keydbID, seckeyptr, len, keyspecptr, keyspeclen, (const char *) [name bytes],
					[name length], adkptr, adklen, adkclass, rakptr, raklen, rakclass, (const char *) [passphrase bytes],
					[passphrase length], passphraseIsKey, cacheTimeOut, cacheGlobal, &ptr, &len, newkeyid);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpCreateSubkeypair_back
	:(PGPUInt32)				masterkeyID
	seckeyBuf:(NSData *)		seckeyBuf
	keyspecBuf:(NSData *)		keyspecBuf
	passphrase:(NSData *)		passphrase
	passphraseIsKey:(BOOL)		passphraseIsKey
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	newobjs:(NSData **)			newobjs
	newkeyid:(PGPUInt32 *)		newkeyid
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	PGPByte *			secptr;
	PGPSize				seclen = [seckeyBuf length];
	PGPByte *			keyptr;
	PGPSize				keylen = [keyspecBuf length];
	
	if (IsntPGPError(result)) {
		secptr = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), seclen, kPGPMemoryMgrFlags_None);
		if (IsNull(secptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		keyptr = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), keylen, kPGPMemoryMgrFlags_None);
		if (IsNull(keyptr)) {
			PGPFreeData(secptr);
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[seckeyBuf getBytes:secptr];
		[keyspecBuf getBytes:keyptr];
		result = pgpCreateSubkeypair_back(pgpContext, masterkeyID, secptr, seclen, keyptr, keylen, (const char *) [passphrase bytes],
					[passphrase length], passphraseIsKey, cacheTimeOut, cacheGlobal, &ptr, &len, newkeyid);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpCountTokens_back
	:(PGPUInt32 *)		numTokens
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpCountTokens_back(pgpContext, numTokens);
	}
	
	return result;
}



	- (PGPError)
pgpGetTokenInfo_back
	:(PGPUInt32)			tokNumber
	info:(PGPTokenInfo *)	info
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpGetTokenInfo_back(pgpContext, tokNumber, info);
	}
	
	return result;
}



	- (PGPError)
pgpDeleteKeyOnToken_back
	:(PGPUInt32)			keydbID
	keyID:(NSData *)		keyID 
	tokenNumber:(PGPUInt32)	tokenNumber
	pin:(NSData *)			pin
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpDeleteKeyOnToken_back(pgpContext, keydbID, (const PGPKeyID *) [keyID bytes], tokenNumber, (PGPByte *) [pin bytes], [pin length]);
	}
	
	return result;
}



	- (PGPError)
pgpWipeToken_back
	:(PGPUInt32)			tokNumber
	passphrase:(NSData *)	passphrase
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpWipeToken_back(pgpContext, tokNumber, (PGPByte *) [passphrase bytes], [passphrase length]);
	}
	
	return result;
}



	- (PGPError)
pgpFormatToken_back
	:(PGPUInt32)			tokNumber
	adminPin:(NSData *)		adminPin
	newUserPin:(NSData *)	newUserPin
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpFormatToken_back(pgpContext, tokNumber, (PGPByte *) [adminPin bytes], [adminPin length],
					(PGPByte *) [newUserPin bytes], [newUserPin length]);
	}
	
	return result;
}



	- (PGPError)
pgpTokenPassphraseIsValid_back
	:(PGPUInt32)			tokNumber
	passphrase:(NSData *)	passphrase
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpTokenPassphraseIsValid_back(pgpContext, tokNumber, (PGPByte *) [passphrase bytes], [passphrase length]);
	}
	
	return result;
}



	- (PGPError)
pgpCopyKeyToToken_back
	:(PGPUInt32)				objID
	tokNumber:(PGPUInt32)		tokNumber
	isMaster:(BOOL)				isMaster
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	PIN:(NSData *)				PIN
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpCopyKeyToToken_back(pgpContext, objID, tokNumber, isMaster, (const char *) [passphrase bytes], [passphrase length],
					hashedPhrase, (const char *) [PIN bytes], [PIN length], cacheTimeOut, cacheGlobal);
	}
	
	return result;
}



	- (PGPError)
pgpTokenImportX509_back
	:(NSData *)			keyID
	userID:(NSData *)	userID
	x509:(NSData *)		x509
	password:(NSData *)	password
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpTokenImportX509_back(pgpContext, (PGPByte *) [keyID bytes], (PGPByte *) [userID bytes], [userID length], (PGPByte *) [x509 bytes],
					[x509 length], (PGPByte *) [password bytes], [password length]);
	}
	
	return result;
}



	- (PGPError)
pgpAddUserID_back
	:(PGPUInt32)						objid
	isAttribute:(BOOL)					isAttribute
	attributeType:(PGPAttributeType)	attributeType
	userIDData:(NSData *)				userIDData
	passphrase:(NSData *)				passphrase
	hashedPhrase:(BOOL)					hashedPhrase
	cacheTimeOut:(PGPUInt32)			cacheTimeOut
	cacheGlobal:(BOOL)					cacheGlobal
	newobjs:(NSData **)					newobjs
	task:(NSMachPort *)					task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpAddUserID_back(pgpContext, objid, isAttribute, attributeType, (const char *) [userIDData bytes], [userIDData length],
					(const char *) [passphrase bytes], [passphrase length], hashedPhrase, cacheTimeOut, cacheGlobal, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpCertifyUserID_back
	:(PGPUInt32)					useridid
	certifying_keyid:(PGPUInt32)	certifying_keyid
	passphrase:(NSData *)			passphrase
	hashedPhrase:(BOOL)				hashedPhrase
	cacheTimeOut:(PGPUInt32)		cacheTimeOut
	cacheGlobal:(BOOL)				cacheGlobal
	exportable:(BOOL)				exportable
	creationDate:(PGPTime)			creationDate
	expiration:(PGPUInt32)			expiration
	trustDepth:(PGPByte)			trustDepth
	trustValue:(PGPByte)			trustValue
	sRegExp:(NSData *)				sRegExp
	newobjs:(NSData **)				newobjs
	task:(NSMachPort *)				task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpCertifyUserID_back(pgpContext, useridid, certifying_keyid, (const char *) [passphrase bytes], [passphrase length],
					hashedPhrase, cacheTimeOut, cacheGlobal, exportable, creationDate, expiration, trustDepth,
					trustValue, (const char *) [sRegExp bytes], [sRegExp length], &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpRevokeSig_back
	:(PGPUInt32)				sigid
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	newobjs:(NSData **)			newobjs
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpRevokeSig_back(pgpContext, sigid, (const char *) [passphrase bytes], [passphrase length], hashedPhrase, cacheTimeOut,
					cacheGlobal, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpRevokeKey_back
	:(PGPUInt32)				key_id
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	newobjs:(NSData **)			newobjs
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpRevokeKey_back(pgpContext, key_id, (const char *) [passphrase bytes], [passphrase length], hashedPhrase, cacheTimeOut,
					cacheGlobal, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpDoChangePassphrase_back
	:(PGPUInt32)				keydb_id
	key_id:(PGPUInt32)			key_id
	masterkey_id:(PGPUInt32)	masterkey_id
	oldphrase:(NSData *)		oldphrase
	newphrase:(NSData *)		newphrase
	newPassphraseIsKey:(BOOL)	newPassphraseIsKey
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpDoChangePassphrase_back(pgpContext, keydb_id, key_id, masterkey_id, (const char *) [oldphrase bytes], [oldphrase length],
					(const char *) [newphrase bytes], [newphrase length], newPassphraseIsKey, cacheTimeOut, cacheGlobal);
	}
	
	return result;
}



	- (PGPError)
pgpPassphraseIsValid_back
	:(PGPUInt32)				key_id
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	isValid:(BOOL *)			isValid
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpPassphraseIsValid_back(pgpContext, key_id, (const char *) [passphrase bytes], [passphrase length], hashedPhrase,
					cacheTimeOut, cacheGlobal, reinterpret_cast<PGPBoolean *>(isValid));
	}
	
	return result;
}



	- (PGPError)
pgpSetKeyTrust_back
	:(PGPUInt32)		key_id
	trust:(PGPUInt32)	trust
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpSetKeyTrust_back(pgpContext, key_id, trust);
	}
	
	return result;
}



	- (PGPError)
pgpGetPasskeyBuffer_back
	:(PGPUInt32)				key_id
	passphrase:(NSData *)		passphrase
	passkeyBuffer:(NSData **)	passkeyBuffer
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpGetPasskeyBuffer_back(pgpContext, key_id, (const char *) [passphrase bytes], [passphrase length], &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*passkeyBuffer = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpAddKeyOptions_back
	:(PGPUInt32)				key_id
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	raklist:(NSData *)			raklist
	rakclass:(PGPUInt32)		rakclass
	newobjs:(NSData **)			newobjs
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [raklist length];
	
	if (IsntPGPError(result)) {
		ptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[raklist getBytes:ptr];
		result = pgpAddKeyOptions_back(pgpContext, key_id, (const char *) [passphrase bytes], [passphrase length], hashedPhrase, cacheTimeOut,
					cacheGlobal, ptr, len, rakclass, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpUpdateKeyOptions_back
	:(PGPUInt32)				key_id
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	prefalg:(NSData *)			prefalg
	prefkeyserv:(NSData *)		prefkeyserv
	keyflags:(PGPUInt32)		keyflags
	fkeyflags:(BOOL)			fkeyflags
	keyservprefs:(PGPUInt32)	keyservprefs
	fkeyservprefs:(BOOL)		fkeyservprefs
	newobjs:(NSData **)			newobjs
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpUpdateKeyOptions_back(pgpContext, key_id, (const char *) [passphrase bytes], [passphrase length], hashedPhrase,
					cacheTimeOut, cacheGlobal, (const PGPCipherAlgorithm *) [prefalg bytes], [prefalg length], (PGPByte *) [prefkeyserv bytes], [prefkeyserv length],
					keyflags, fkeyflags, keyservprefs, fkeyservprefs, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpKeyDBAddObject_back
	:(PGPUInt32)			kdb_id
	obj_id:(PGPUInt32)		obj_id
	newobjs:(NSData **)		newobjs
	pnewobj:(PGPUInt32 *)	pnewobj
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpKeyDBAddObject_back(pgpContext, kdb_id, obj_id, &ptr, &len, pnewobj);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpKeyDBRemoveObject_back
	:(PGPUInt32)		kdb_id
	obj_id:(PGPUInt32)	obj_id
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpKeyDBRemoveObject_back(pgpContext, kdb_id, obj_id);
	}
	
	return result;
}



	- (PGPError)
pgpCopyKeys_back
	:(PGPUInt32)			srcid
	dstid:(PGPUInt32)		dstid
	keylist:(NSData *)		keylist
	neednewkeylist:(BOOL)	neednewkeylist
	newkeylist:(NSData **)	newkeylist
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len = [keylist length];
	
	if (IsntPGPError(result)) {
		ptr = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(pgpContext), len, kPGPMemoryMgrFlags_None);
		if (IsNull(ptr)) {
			result = kPGPError_OutOfMemory;
		}
	}
	if (IsntPGPError(result)) {
		[keylist getBytes:ptr];
		result = pgpCopyKeys_back(pgpContext, srcid, dstid, ptr, len, neednewkeylist, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newkeylist = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpKeyDBArray_back
	:(PGPUInt32)			kdbid
	numKeys:(PGPUInt32 *)	numKeys
	keyArray:(NSData **)	keyArray
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpKeyDBArray_back(pgpContext, kdbid, numKeys, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*keyArray = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpUpdateKeyDB_back
	:(PGPUInt32)				kdbid
	numNewKeys:(PGPUInt32 *)	numNewKeys
	newKeyArray:(NSData **)		newKeyArray
	changedkeylist:(NSData **)	changedkeylist
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	PGPUInt32 *			ptr2;
	PGPSize				len2;
	
	if (IsntPGPError(result)) {
		result = pgpUpdateKeyDB_back(pgpContext, kdbid, numNewKeys, &ptr, &len, &ptr2, &len2);
	}
	if (IsntPGPError(result)) {
		*newKeyArray = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
		*changedkeylist = [NSData dataWithBytes:ptr2 length:len2];
		PGPFreeData(ptr2);
	}
	
	return result;
}



	- (PGPError)
pgpNewKeyDB_back
	:(PGPUInt32 *)		keydbid
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpNewKeyDB_back(pgpContext, keydbid);
	}
	
	return result;
}



	- (PGPError)
pgpCertifyPrimaryUserID_back
	:(PGPUInt32)				useridid
	passphrase:(NSData *)		passphrase
	hashedPhrase:(BOOL)			hashedPhrase
	cacheTimeOut:(PGPUInt32)	cacheTimeOut
	cacheGlobal:(BOOL)			cacheGlobal
	newobjs:(NSData **)			newobjs
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPUInt32 *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpCertifyPrimaryUserID_back(pgpContext, useridid, (const char *) [passphrase bytes], [passphrase length], hashedPhrase,
					cacheTimeOut, cacheGlobal, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*newobjs = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpGlobalRandomPoolAddState_back
	:(BOOL)					addKeyState
	keyEvent:(PGPInt32)		keyEvent
	addMouseState:(BOOL)	addMouseState
	addSystemState:(BOOL)	addSystemState
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpGlobalRandomPoolAddState_back(addKeyState, keyEvent, addMouseState, addSystemState);
	}
	
	return result;
}



	- (PGPError)
pgpGlobalRandomPoolGetInfo_back
	:(PGPUInt32 *)				entropy
	size:(PGPUInt32 *)			size
	minEntropy:(PGPUInt32 *)	minEntropy
	hasMinEntropy:(BOOL *)		hasMinEntropy
	hasIntelRNG:(BOOL *)		hasIntelRNG
	task:(NSMachPort *)			task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpGlobalRandomPoolGetInfo_back(entropy, size, minEntropy, reinterpret_cast<PGPBoolean *>(hasMinEntropy),
					reinterpret_cast<PGPBoolean *>(hasIntelRNG));
	}
	
	return result;
}



	- (PGPError)
pgpRandomGetBytesEntropy_back
	:(PGPUInt32)			requestBytes
	entropyBits:(PGPUInt32)	entropyBits
	randBuf:(NSData **)		randBuf
	task:(NSMachPort *)		task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpRandomGetBytesEntropy_back(pgpContext, requestBytes, entropyBits, &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*randBuf = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpRandomAddBytes_back
	:(NSData *)			buf
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpRandomAddBytes_back(pgpContext, (PGPByte *) [buf bytes], [buf length]);
	}
	
	return result;
}



	- (PGPError)
pgpRandomStir_back
	:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpRandomStir_back(pgpContext);
	}
	
	return result;
}



	- (PGPError)
pgpSetRandSeedFile_back
	:(NSData *)			randFile
	task:(NSMachPort *)	task
{
	PGPError				result;
	StSetCredentials		creds(task, &result);
	PFLFileSpecRef			ref;

	if (IsntPGPError(result)) {
		result = PFLImportFileSpec(PGPPeekContextMemoryMgr(pgpContext), (PGPByte *) [randFile bytes], [randFile length], &ref);
	}
	if (IsntPGPError(result)) {
		result = pgpSetRandSeedFile_back(ref);
		PFLFreeFileSpec(ref);
	}
	
	return result;
}



	- (PGPError)
pgpTokenPutKeyContainer_back
	:(NSData *)			keyID
	password:(NSData *)	password
	cont:(NSData *)		cont
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpTokenPutKeyContainer_back(pgpContext, (PGPByte *) [keyID bytes], (PGPByte *) [password bytes], [password length], (PGPByte *) [cont bytes],
					[cont length]);
	}
	
	return result;
}



	- (PGPError)
pgpTokenGetKeyContainer_back
	:(NSData *)			keyID
	password:(NSData *)	password
	contOut:(NSData **)	contOut
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	PGPByte *			ptr;
	PGPSize				len;
	
	if (IsntPGPError(result)) {
		result = pgpTokenGetKeyContainer_back(pgpContext, (PGPByte *) [keyID bytes], (PGPByte *) [password bytes], [password length], &ptr, &len);
	}
	if (IsntPGPError(result)) {
		*contOut = [NSData dataWithBytes:ptr length:len];
		PGPFreeData(ptr);
	}
	
	return result;
}



	- (PGPError)
pgpSetPKCS11DrvFile_back
	:(NSData *)			module
	task:(NSMachPort *)	task
{
	PGPError			result;
	StSetCredentials	creds(task, &result);
	
	if (IsntPGPError(result)) {
		result = pgpSetPKCS11DrvFile_back((PGPByte *) [module bytes]);
	}
	
	return result;
}

@end



StSetCredentials::StSetCredentials(
	NSMachPort *	inPort,
	PGPError *		outResult)
{
	kern_return_t			kerr;
	mach_port_t				p = [inPort machPort];
	security_token_t		tok;
	mach_msg_type_number_t	cnt = TASK_SECURITY_TOKEN_COUNT;
	int						pid;

	*outResult = kPGPError_SecurityOpFailed;
	kerr = pid_for_task(p, &pid);
	if (kerr) {
		NSLog(@"Unable to get the process id for the port. Error = %d (%s)", kerr, mach_error_string(kerr));
	} else {
		pgpContextSetConnectRef(sContext, pid);
		kerr = task_info(p, TASK_SECURITY_TOKEN, reinterpret_cast<task_info_t>(&tok), &cnt);
		if (kerr) {
			NSLog(@"Unable to get the security token. Error = %d (%s)", kerr, mach_error_string(kerr));
		} else {
			mOldGID = getegid();
			mOldUID = geteuid();
			seteuid(0);	//	Makes it easier to switch to other groups
			if (setegid(tok.val[1])) {
				NSLog(@"Unable to setegid(%d (%@)), uid: %d (%@) euid: %d (%@) gid: %d (%@) egid: %d (%@). Error = %d (%s)",
					tok.val[1], [NSString stringWithCString:getgrgid(tok.val[1])->gr_name], getuid(),
					[NSString stringWithCString:getpwuid(getuid())->pw_name], geteuid(),
					[NSString stringWithCString:getpwuid(geteuid())->pw_name], getgid(),
					[NSString stringWithCString:getgrgid(getgid())->gr_name], getegid(),
					[NSString stringWithCString:getgrgid(getegid())->gr_name], errno, strerror(errno));
			} else {
				if (seteuid(tok.val[0])) {
					NSLog(@"Unable to seteuid(%d (%@)), uid: %d (%@) euid: %d (%@) gid: %d (%@) egid: %d (%@). Error = %d (%s)",
						tok.val[0], [NSString stringWithCString:getpwuid(tok.val[0])->pw_name], getuid(),
						[NSString stringWithCString:getpwuid(getuid())->pw_name], geteuid(),
						[NSString stringWithCString:getpwuid(geteuid())->pw_name], getgid(),
						[NSString stringWithCString:getgrgid(getgid())->gr_name], getegid(),
						[NSString stringWithCString:getgrgid(getegid())->gr_name], errno, strerror(errno));
					setegid(mOldGID);
				} else {
					*outResult = kPGPError_NoErr;
				}
			}
		}
	}
	mRestore = IsntPGPError(*outResult);
}



StSetCredentials::~StSetCredentials()
{
	if (mRestore) {
		seteuid(0);
		if (setegid(mOldGID)) {
			NSLog(@"Unable to restore egid(%d)", mOldGID);
		}
		if (seteuid(mOldUID)) {
			NSLog(@"Unable to restore euid(%d)", mOldUID);
		}
	}
}
