/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: CPGPPassphraseTextView.m,v 1.6 2002/10/22 09:25:50 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPPassphraseTextView.h"

#include "pgpUtilities.h"
#include "pgpPassphraseUtils.h"

#define NSEscapeCharacter		0x1B

@implementation CPGPPassphraseTextView

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	return self;
}

- (id)initWithFrame:(NSRect)frame
{
	return [super initWithFrame:frame];
}

- (BOOL)becomeFirstResponder
{
	KeyMap		keyboardState;
        
    GetKeys(keyboardState);
	
	if (keyboardState[1] & 0x2)
	{
		if ([[self delegate] respondsToSelector:@selector(capsLockStatusDidChange:)])
			[(id <PGPPassphraseViewDelegate>) [self delegate] capsLockStatusDidChange:YES];
	}
        
    return [super becomeFirstResponder];
}

- (void)clearPassphrase
{
	[self setString:[NSString string]];
	[[self layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth([self frame]), 14)
												   usedRect:NSMakeRect(0, 0, 10, 14)
												   textContainer:[self textContainer]];
	
	[self estimateQuality:[self string]];
}

- (void)estimateQuality:(NSString*)string
{
	id <PGPPassphraseViewDelegate>	delegate = [self delegate];
	
	if (string)
		[delegate updatePassphraseQuality:pgpEstimatePassphraseQuality([string UTF8String]) sender:self];
}

- (void)flagsChanged:(NSEvent*)event
{
	unsigned		flags;
	
	flags = [event modifierFlags];
	
	if ([[self delegate] respondsToSelector:@selector(capsLockStatusDidChange:)])
	{
		if (flags & NSAlphaShiftKeyMask)
			[(id <PGPPassphraseViewDelegate>) [self delegate] capsLockStatusDidChange:YES];
		else
			[(id <PGPPassphraseViewDelegate>) [self delegate] capsLockStatusDidChange:NO];
	}
		
	[super flagsChanged:event];
}

- (void)insertText:(id)text
{
	if ([[self delegate] respondsToSelector:@selector(passphraseView:shouldInsertText:)])
	{
		if ([(id <PGPPassphraseViewDelegate>) [self delegate] passphraseView:self shouldInsertText:text])
			[self superInsertText:text];
	}
}

- (void)superInsertText:(id)text
{
	[super insertText:text];
	
	[self estimateQuality:[self string]];
}

- (void)deleteBackward:(id)sender
{
	if ([[self delegate] respondsToSelector:@selector(passphraseView:shouldDeleteBackward:)])
	{
		if ([(id <PGPPassphraseViewDelegate>) [self delegate] passphraseView:self shouldDeleteBackward:sender])
			[self superDeleteBackward:sender];
	}
}

- (void)superDeleteBackward:(id)sender
{
	[super deleteBackward:sender];
	
	[self estimateQuality:[self string]];
}

- (void)setMarkedText:(id)text selectedRange:(NSRange)range
{
	if ([[self delegate] respondsToSelector:@selector(shouldPassphraseViewDisplayMarkedText:)])
	{
		if ([(id <PGPPassphraseViewDelegate>) [self delegate] shouldPassphraseViewDisplayMarkedText:self])
			[super setMarkedText:text selectedRange:range];
	}
	
	[self estimateQuality:[self string]];
}

- (void)insertNewline:(id)sender
{
	NSButtonCell*	buttonCell;
	
	if (buttonCell = [[self window] defaultButtonCell])
		[buttonCell performClick:sender];
}

- (void)insertBacktab:(id)sender
{
    if ([self previousKeyView])
        [[self window] makeFirstResponder:[self previousKeyView]];
}

- (void)insertTab:(id)sender
{
	if ([self nextKeyView])
		[[self window] makeFirstResponder:[self nextKeyView]];
}

- (void)moveBackward:(id)sender
{
}

- (void)moveBackwardAndModifySelection:(id)sender
{
}

- (void)moveDown:(id)sender
{
}

- (void)moveDownAndModifySelection:(id)sender
{
}

- (void)moveForward:(id)sender
{
}

- (void)moveForwardAndModifySelection:(id)sender
{
}

- (void)moveLeft:(id)sender
{
}

- (void)moveRight:(id)sender
{
}

- (void)moveToBeginningOfDocument:(id)sender
{
}

- (void)moveToBeginningOfLine:(id)sender
{
}

- (void)moveToBeginningOfParagraph:(id)sender
{
}

- (void)moveToEndOfDocument:(id)sender
{
}

- (void)moveToEndOfLine:(id)sender
{
}

- (void)moveToEndOfParagraph:(id)sender
{
}

- (void)moveUp:(id)sender
{
}

- (void)moveUpAndModifySelection:(id)sender
{
}

- (void)moveWordBackward:(id)sender
{
}

- (void)moveWordBackwardAndModifySelection:(id)sender
{
}

- (void)moveWordForward:(id)sender
{
}

- (void)moveWordForwardAndModifySelection:(id)sender
{
}

- (void)cut:(id)sender
{
}

- (void)copy:(id)sender
{
}

- (void)paste:(id)sender
{
}

- (void)selectAll:(id)sender
{
}

- (void)setSelectedRange:(NSRange)range affinity:(NSSelectionAffinity)affinity stillSelecting:(BOOL)flag
{
	if (range.length)
		range.location = range.length;
	
	[super setSelectedRange:range affinity:affinity stillSelecting:flag];
}

/*
This method gets called on a mouseDown event. By default it would place the
insertion point wherever the mouseDown event occurred. In our case, we always
put the selection at the end of the current string.
*/
- (NSRange)selectionRangeForProposedRange:(NSRange)range granularity:(NSSelectionGranularity)granularity
{
	range.location = [[self string] length];
	range.length = 0;
	
	return range;
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
	SEL		menuAction = [menuItem action];

	if (menuAction == @selector(cut:))
		return NO;
	
	if (menuAction == @selector(copy:))
		return NO;
	
	if (menuAction == @selector(paste:))
		return NO;
	
	if (menuAction == @selector(selectAll:))
		return NO;
	
	if (menuAction == @selector(delete:))
		return NO;;
	
	return YES;
}

- (void)dealloc
{
	[super dealloc];
}

@end
