/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPDialogClasses.h,v 1.9 2002/11/07 23:01:58 jeffc Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>

#import "PGPDialogProxies.h"
#import "PGPUICocoaPriv.h"
#import "CPGPSelectedTableView.h"

#include "pgpPassphraseUtils.h"
#include "pgpRecipientDialogCommon.h"
#include "pgpOptionListPriv.h"

@class CPGPPassphraseTextView;

/*
Interface Builder has a limitation which prevents it from correctly parsing
object hierarchies which span multiple header files. Consequently, all of our
class declarations need to appear in a single header (implementations will
probably span several files).
*/

@interface CPGPDialog : NSResponder
{
	@protected

		IBOutlet NSWindow*			mWindow;		
		IBOutlet NSTextField*		mPrompt;

		PGPDialogProxyObjRef		mProxyObject;
		PGPContextRef				mContext;
		PGPOptionListRef			mClientOptions;
		PGPKeyServerSpec*			mServerList;
		PGPUInt32					mServerCount;
		PGPBoolean					mSearchBeforeDisplay;
		PGPKeyDBRef*				mNewKeys;		/* Server search results */
		PGPtlsContextRef			mTLSContext;
		PGPKeyDBObjRef				mDefaultKey;
		
		PGPError					mReturnCode;
		BOOL						mLoadedNib;
}

- (PGPDialogProxyObjRef)proxyObject;
- (void)setProxyObject:(PGPDialogProxyObjRef)proxyObject;

- (PGPContextRef)context;
- (void)setContext:(PGPContextRef)context;

- (PGPOptionListRef)clientOptions;
- (void)setClientOptions:(PGPOptionListRef)options;

- (NSString*)windowTitle;
- (void)setWindowTitle:(NSString*)title;

- (PGPKeyServerSpec*)serverList;
- (void)setServerList:(PGPKeyServerSpec*)serverList;

- (PGPUInt32)serverCount;
- (void)setServerCount:(PGPUInt32)serverCount;

- (PGPBoolean)searchBeforeDisplay;
- (void)setSearchBeforeDisplay:(PGPBoolean)flag;

- (PGPKeyDBRef*)newKeys;
- (void)setNewKeys:(PGPKeyDBRef*)newKeys;

- (PGPtlsContextRef)tlsContext;
- (void)setTLSContext:(PGPtlsContextRef)tlsContext;

- (PGPKeyDBObjRef)defaultKey;
- (void)setDefaultKey:(PGPKeyDBObjRef)key;

- (NSString*)prompt;
- (void)setPrompt:(NSString*)prompt;

- (PGPError)runDialog;
- (PGPError)returnCode;

- (BOOL)windowShouldClose:(id)sender;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPOptionsDialog : CPGPDialog
{
	@protected
		
		IBOutlet NSView*				mOptionsView;
		
		NSMutableDictionary*			mOptionMapping;
		PGPOptionListRef				mOptions;
}

- (id)initWithOptions:(PGPOptionListRef)options;
- (void)loadNib:(NSString*)nibFile;

- (NSView*)optionsView;
- (PGPError)buildCheckboxesInView:(NSView*)checkboxContainer withOptionList:(PGPOptionListRef)optionList horizontalPadding:(PGPUInt32)hPadding verticalPadding:(PGPUInt32)vPadding parentCheckbox:(NSButton*)parent;
- (PGPError)buildPopUpButtonsInView:(NSView*)popupContainer withOptionList:(PGPOptionListRef)optionList horizontalPadding:(PGPUInt32)hPadding verticalPadding:(PGPUInt32)vPadding;
- (NSButton*)createCheckboxButton:(PGPOUICheckboxDesc*)description parentCheckbox:(NSButton*)parent;
- (NSTextField*)createPopUpTitle:(PGPOUIPopupListDesc*)description;
- (NSPopUpButton*)createPopUpButton:(PGPOUIPopupListDesc*)description;

- (IBAction)checkboxPressed:(id)sender;
- (IBAction)popupButtonPressed:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPCollectRandomDataDialog : CPGPDialog
{
	@protected
	
		IBOutlet NSProgressIndicator*	mRandomDataCollected;
		IBOutlet NSButton*				mDismissButton;
		IBOutlet NSTextField*			mKeyThief;
		
		PGPUInt32						mNeededEntropyBits;
		PGPUInt32						mStartEntropy;
		PGPUInt32						mNeededEntropy;
		Point							mLastMouse;
		NSTimer*						mEntropyTimer;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (PGPUInt32)neededEntropyBits;
- (void)setNeededEntropyBits:(PGPUInt32)neededEntropyBits;

- (void)startTimer;
- (void)stopTimer;
- (void)collectEntropy:(NSTimer*)timer;
- (void)textDidChange:(NSNotification*)notification;

- (void)updateProgressBar:(id)sender;

- (PGPError)runDialog;
- (void)close;

- (IBAction)cancel:(id)sender;
- (IBAction)ok:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPPassphraseDialog : CPGPDialog <PGPPassphraseViewDelegate>
{
	@protected

		IBOutlet NSScrollView*				mScrollView;
		IBOutlet NSButton*					mHideTypingCheckbox;
		IBOutlet NSTextField*				mCapsLockWarningField;
		IBOutlet NSButton*					mOptionsButton;
		IBOutlet NSButton*					mOkButton;
		IBOutlet NSButton*					mCancelButton;
		
		char**								mPassphrasePtr;
		unsigned char						mSpaceCountList[kMaxPassphraseLength];
		PGPOptionListRef					mDialogOptions;
		PGPUInt32							mMinPassphraseLength;
		PGPUInt32							mMinPassphraseQuality;
		PGPUInt32							mDialogTimeout;
		PGPUInt32							mPassphraseQuality;
		NSTimer*							mDialogTimeoutTimer;
		CPGPPassphraseTextView*				mPassphraseView;
		CPGPPassphraseTextView*				mSpaceView;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (void)setPrompt:(NSString*)prompt;

- (char**)passphrasePtr;
- (void)setPassphrasePtr:(char**)passphrasePtr;

- (PGPOptionListRef)dialogOptions;
- (void)setDialogOptions:(PGPOptionListRef)dialogOptions;

- (PGPUInt32)minPassphraseLength;
- (void)setMinPassphraseLength:(PGPUInt32)minPassphraseLength;

- (PGPUInt32)minPassphraseQuality;
- (void)setMinPassphraseQuality:(PGPUInt32)minPassphraseQuality;

- (BOOL)passphraseLengthAndQualityOk:(const char*)passphrase;

- (void)updatePassphraseQuality:(PGPUInt32)quality sender:(id)sender;

- (PGPUInt32)dialogTimeout;
- (void)setDialogTimeout:(PGPUInt32)timeout;
- (void)startTimer;
- (void)resetTimer;
- (void)dialogTimedOut:(NSTimer*)timer;

- (PGPError)runDialog;
- (void)close;

- (IBAction)hideText:(id)sender;
- (IBAction)cancel:(id)sender;
- (IBAction)ok:(id)sender;

/* CPGPPassphraseTextView Delegate Methods */
- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldInsertText:(NSString*)string;
- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldDeleteBackward:(id)sender;
- (BOOL)shouldPassphraseViewDisplayMarkedText:(CPGPPassphraseTextView*)passphraseView;
- (void)capsLockStatusDidChange:(BOOL)enabled;

/* NSWindow Delegate Methods */
- (BOOL)windowShouldClose:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPKeyPassphraseDialog : CPGPPassphraseDialog
{
	@protected

		IBOutlet NSTextField*		mKeyField;
		
		PGPBoolean					mVerifyPassphrase;
		PGPBoolean					mCache;
		PGPBoolean					mCacheGlobal;
		PGPUInt32					mCacheTimeout;		
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (PGPBoolean)verifyPassphrase:(const char*)passphrase;
- (void)rejectPassphraseWithMessage:(NSString*)message;
- (PGPBoolean)verifiesPassphrase;
- (void)setVerifiesPassphrase:(PGPBoolean)flag;

- (PGPBoolean)cache;
- (void)setCache:(PGPBoolean)flag;

- (PGPBoolean)cacheGlobal;
- (void)setCacheGlobal:(PGPBoolean)flag;

- (PGPUInt32)cacheTimeout;
- (void)setCacheTimeout:(PGPUInt32)cacheTimeout;

- (PGPError)runDialog;

- (IBAction)ok:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPKeySetPassphraseDialog : CPGPKeyPassphraseDialog
{
	@protected
	
		PGPBoolean					mFindMatchingKey;
		PGPKeySetRef				mKeySet;
		PGPKeyDBObjRef*				mPassphraseKeyPtr;
}

- (PGPBoolean)findMatchingKey;
- (void)setFindMatchingKey:(PGPBoolean)flag;

- (PGPKeySetRef)keySet;
- (void)setKeySet:(PGPKeySetRef)keySet;

- (PGPKeyDBObjRef*)passphraseKeyPtr;
- (void)setPassphraseKeyPtr:(PGPKeyDBObjRef*)keyPtr;

- (NSString*)generateSigningKeyString:(PGPKeyDBObjRef)key;
- (NSString*)generateEncryptionKeyString:(PGPKeyDBObjRef)key;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPSignPassphraseDialog : CPGPKeySetPassphraseDialog
{
	@protected

		IBOutlet NSTextField*		mKeySelectPrompt;
		IBOutlet NSPopUpButton*		mKeySelectPopup;
		IBOutlet NSTextField*		mMessageText;
		IBOutlet NSBox*				mOptionsBox;
		IBOutlet NSView*			mContainer;
		IBOutlet NSView*			mButtonContainer;
		
		NSView*						mOptionsView;
		CPGPOptionsDialog*			mOptionsDialog;
		
	@private
	
		BOOL						mFoundMatchingKey;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (void)setKeySet:(PGPKeySetRef)keySet;

- (PGPError)buildKeyList;
- (PGPError)validatePassphrase:(const char*)passphrase forKey:(PGPKeyDBObjRef)key;

- (IBAction)signingKeyChanged:(id)sender;
- (IBAction)showOptions:(id)sender;
- (IBAction)hideText:(id)sender;
- (IBAction)ok:(id)sender;

- (void)capsLockStatusDidChange:(BOOL)enabled;

- (void)sheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPDecryptionPassphraseDialog : CPGPKeySetPassphraseDialog
{
	@protected

		IBOutlet NSTextField*		mPrompt2;
		IBOutlet NSTableView*		mRecipientsTable;
		IBOutlet NSTextField*		mMessageText;
		
		PGPKeyID*					mKeyIDList;
		PGPUInt32					mKeyIDCount;
		PGPKeyID*					mMissingKeyIDList;
		PGPUInt32					mMissingKeyIDCount;
		PGPBoolean					mHaveSecretKey;
		
	@private
	
		NSMutableArray*				mRecipientStrings;
		NSMutableArray*				mRecipientKeys;
		BOOL						mFoundMatchingKey;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (PGPKeyID*)keyIDList;
- (void)setKeyIDList:(PGPKeyID*)keyIDList;

- (PGPUInt32)keyIDCount;
- (void)setKeyIDCount:(PGPUInt32)keyIDCount;

- (PGPKeyID*)missingKeyIDList;
- (void)setMissingKeyIDList:(PGPKeyID*)missingKeyIDList;

- (PGPUInt32)missingKeyIDCount;
- (void)setMissingKeyIDCount:(PGPUInt32)missingKeyIDCount;

- (void)setNewKeys:(PGPKeyDBRef*)newKeys;
- (void)setKeySet:(PGPKeySetRef)keySet;

- (PGPError)addKeySet:(PGPKeySetRef)keySet toRecipientsTable:(NSTableView*)table;
- (PGPError)runDialog;

- (IBAction)hideText:(id)sender;
- (IBAction)ok:(id)sender;

- (int)numberOfRowsInTableView:(NSTableView*)tableView;
- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)tableColumn row:(int)row;
- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row;
- (void)tableViewSelectionDidChange:(NSNotification*)notification;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPConfirmationPassphraseDialog : CPGPPassphraseDialog
{
	@protected

		IBOutlet NSTextField*				mPassphraseQualityPrompt;
		IBOutlet NSProgressIndicator*		mPassphraseQualityIndicator;
		IBOutlet NSTextField*				mMinimumQualityPrompt;
		IBOutlet NSProgressIndicator*		mMinimumQualityIndicator;
		IBOutlet NSScrollView*				mConfirmationScrollView;
		
		CPGPPassphraseTextView*				mConfirmationPassphraseView;
		CPGPPassphraseTextView*				mConfirmationSpaceView;
		PGPBoolean							mShowPassphraseQuality;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (PGPBoolean)showsPassphraseQuality;
- (void)setShowsPassphraseQuality:(PGPBoolean)flag;

- (void)updatePassphraseQuality:(PGPUInt32)quality sender:(id)sender;

- (PGPError)runDialog;
- (void)setupKeyViewLoop;
- (void)close;

- (IBAction)hideText:(id)sender;
- (IBAction)ok:(id)sender;

/* CPGPPassphraseTextView Delegate Methods */
- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldInsertText:(NSString*)string;
- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldDeleteBackward:(id)sender;
- (BOOL)shouldPassphraseViewDisplayMarkedText:(CPGPPassphraseTextView*)passphraseView;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPKeyServerDialog : CPGPDialog
{
	@protected

		IBOutlet NSTextField*				mServerLabel;
		IBOutlet NSTextField*				mServerName;
		IBOutlet NSTextField*				mKeyLabel;
		IBOutlet NSTextField*				mKeyName;
		IBOutlet NSTextField*				mStatusLabel;
		IBOutlet NSTextField*				mStatus;
		IBOutlet NSProgressIndicator*		mProgressIndicator;
		IBOutlet NSButton*					mCancel;
		
		BOOL								mThreadEnded;
		BOOL								mAbort;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (NSString*)prompt;
- (void)setPrompt:(NSString*)prompt;

- (NSTextField*)serverNameField;
- (NSTextField*)keyNameField;
- (NSTextField*)statusField;

- (void)close;

- (IBAction)cancel:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPSearchKeyServerDialog : CPGPKeyServerDialog
{
	@protected
	
		PGPBoolean                      mSearchAllServers;
        PGPFilterRef            		mFilter;
        char                            mKeyDescription[256];
}

- (id)init;

- (PGPBoolean)searchAllServers;
- (void)setSearchAllServers:(PGPBoolean)flag;

- (PGPFilterRef)filter;
- (void)setFilter:(PGPFilterRef)filter;

- (char*)keyDescription;
- (void)setKeyDescription:(char*)keyDescription;

- (PGPError)runDialog;

- (void)keyServerThread;
- (IBAction)cancel:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPSendToKeyServerDialog : CPGPKeyServerDialog
{
	@protected
	
		PGPKeySetRef					mKeysToSend;
		PGPKeySetRef*					mFailedKeys;
}

- (id)init;

- (PGPKeySetRef)keysToSend;
- (void)setKeysToSend:(PGPKeySetRef)keysToSend;

- (PGPKeySetRef*)failedKeys;
- (void)setFailedKeys:(PGPKeySetRef*)failedKeys;

- (PGPError)runDialog;

- (void)keyServerThread;
- (IBAction)cancel:(id)sender;

- (void)dealloc;

@end

/* ------------------------------------------------------------------------- */

@interface CPGPRecipientDialog : CPGPDialog
{
	@protected
	
		IBOutlet NSTableView*		mUsersTable;
		IBOutlet NSTableView*		mRecipientsTable;
		IBOutlet NSButton*			mOptions;
		IBOutlet NSButton*			mUpdateFromServer;
		IBOutlet NSButton*			mCancel;
		IBOutlet NSButton*			mOK;
		IBOutlet NSView*			mTableViewContainer;
		IBOutlet NSView*			mButtonContainer;
		IBOutlet NSBox*				mOptionsBox;

		NSMutableArray*				mUserIDArray;
		NSMutableArray*				mRecipientArray;
        NSMutableString*			mSearchString;
		NSMutableData*				mDragData;
        NSTimeInterval				mLastKeypress;
		NSImage*					mValidDot;
		NSImage*					mInvalidDot;
		NSLock*						mLock;
        NSTableView*				mSelectedTable;
		NSView*						mOptionsView;
		NSSize						mMinSize;
		CPGPOptionsDialog*			mOptionsDialog;
		BOOL						mUpdatingMissingRecipients;
		BOOL						mConventionalEncryptionEnabled;
        PGPInt32					mLastUserRow;
        PGPInt32					mLastRecipientRow;
		PGPRecipientsList			mRecipients;
		PGPOptionListRef			mDialogOptions;
		PGPUInt32					mNumDefaultRecipients;
		PGPUInt32					mNumMissingRecipients;
		PGPRecipientSpec*			mDefaultRecipients;
		PGPBoolean					mDisplayMarginalValidity;
		PGPBoolean					mIgnoreMarginalValidity;
		PGPGroupSetRef				mGroupSet;
		PGPKeySetRef				mClientKeySet;
		PGPBoolean					mAlwaysDisplay;
		PGPBoolean					mAlwaysDisplayWithARRs;
		PGPKeyDBRef*				mRecipientKeysPtr;
		PGPUInt32*					mRecipientCount;
		PGPRecipientSpec**			mRecipientList;
		
		PGPAdditionalRecipientRequestEnforcement	mEnforcement;
}

- (id)init;
- (void)loadNib:(NSString*)nibFile;

- (PGPOptionListRef)dialogOptions;
- (void)setDialogOptions:(PGPOptionListRef)dialogOptions;

- (PGPUInt32)numDefaultRecipients;
- (void)setNumDefaultRecipients:(PGPUInt32)numDefaultRecipients;

- (const PGPRecipientSpec*)defaultRecipients;
- (void)setDefaultRecipients:(const PGPRecipientSpec*)defaultRecipients;

/* Displaying marginally valid keys is not supported at this time. */
- (PGPBoolean)displayMarginalValidity;
- (void)setDisplayMarginalValidity:(PGPBoolean)displayMarginalValidity;

- (PGPBoolean)ignoreMarginalValidity;
- (void)setIgnoreMarginalValidity:(PGPBoolean)ignoreMarginalValidity;

- (PGPGroupSetRef)groupSet;
- (void)setGroupSet:(PGPGroupSetRef)groupSet;

- (PGPKeySetRef)clientKeySet;
- (void)setClientKeySet:(PGPKeySetRef)clientKeySet;

- (PGPBoolean)alwaysDisplay;
- (void)setAlwaysDisplay:(PGPBoolean)alwaysDisplay;

- (PGPBoolean)alwaysDisplayWithARRs;
- (void)setAlwaysDisplayWithARRs:(PGPBoolean)alwaysDisplayWithARRs;

- (PGPKeyDBRef*)recipientKeysPtr;
- (void)setRecipientKeysPtr:(PGPKeyDBRef*)recipientKeysPtr;

- (PGPUInt32*)recipientCount;
- (void)setRecipientCount:(PGPUInt32*)recipientCount;

- (PGPRecipientSpec**)recipientList;
- (void)setRecipientList:(PGPRecipientSpec**)recipientList;

- (PGPAdditionalRecipientRequestEnforcement)enforcement;
- (void)setEnforcement:(PGPAdditionalRecipientRequestEnforcement)enforcement;

- (PGPError)runDialog;
- (void)close;

- (void)setupConventionalEncryption;
- (void)setupTableViews;
- (NSTableView*)selectedTable;
- (void)selectTable:(NSTableView*)table;
- (void)registerDragTypes;
- (void)unregisterDragTypes;
- (PGPError)constructDataSources;
- (PGPError)populateDataSource:(NSMutableArray*)dataSource withKeys:(PGPRecipientKey*)keys ofKind:(PGPRecipientUserKind)kind fromLocation:(PGPRecipientUserLocation)location;
- (PGPError)populateDataSource:(NSMutableArray*)dataSource withUsers:(PGPRecipientUser*)users ofKind:(PGPRecipientUserKind)kind fromLocation:(PGPRecipientUserLocation)location;
- (PGPRecipientUser*)userForTableView:(NSTableView*)table row:(int)row;
- (PGPError)moveUsers:(NSData*)users toLocation:(PGPRecipientUserLocation)location;

- (IBAction)userDoubleClicked:(id)sender;
- (IBAction)showOptions:(id)sender;
- (IBAction)updateFromServer:(id)sender;
- (IBAction)cancel:(id)sender;
- (IBAction)ok:(id)sender;
- (IBAction)conventionalEncrypt:(id)sender;

- (BOOL)acceptsFirstResponder;
- (void)selectSearchString;
- (void)keyDown:(NSEvent*)event;

/* NSTableView delegate methods */
- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row;
- (BOOL)tableView:(NSTableView*)table shouldSelectRow:(int)row;

/* Methods which qualify us for NSTableDataSource status */
- (int)numberOfRowsInTableView:(NSTableView*)tableView;
- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row;

- (BOOL)tableView:(NSTableView*)tableView writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pBoard;
- (void)pasteboard:(NSPasteboard*)sender provideDataForType:(NSString*)type;
- (NSDragOperation)tableView:(NSTableView*)table validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation;
- (BOOL)tableView:(NSTableView*)table acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation;

- (void)dealloc;

@end
