/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPDialogProxies.h,v 1.2 2002/08/06 20:10:56 dallen Exp $
____________________________________________________________________________*/

#include "pgpUserInterface.h"
#include "pgpUtilities.h"

typedef enum
{
	kPGPDialogClass_PassphraseDialog					= 4000,
	kPGPDialogClass_OptionsDialog,
	kPGPDialogClass_RecipientDialog,
	kPGPDialogClass_RandomDataDialog,
	kPGPDialogClass_KeyServerDialog,
} PGPDialogClass;

typedef enum
{
	kPGPDialogProxyType_RandomDataOptions				= 3000,
	kPGPDialogProxyType_PassphraseOptions,
	kPGPDialogProxyType_KeyPassphraseOptions,
	kPGPDialogProxyType_SigningPassphraseOptions,
	kPGPDialogProxyType_DecryptionPassphraseOptions,
	kPGPDialogProxyType_ConfirmationPassphraseOptions,
	kPGPDialogProxyType_SearchKeyServerOptions,
	kPGPDialogProxyType_SendToKeyServerOptions,
	kPGPDialogProxyType_RecipientOptions,
} PGPDialogProxyType;

typedef struct
{
	PGPContextRef		context;
	void*				instance;
	void*				options;	/* The data type is hidden to satisfy the Obj-C compiler */
	PGPDialogClass		dlgClass;
	PGPDialogProxyType	type;
} PGPDialogProxyObj;

typedef PGPDialogProxyObj* PGPDialogProxyObjRef;

#ifdef __cplusplus
extern "C"
{
#endif

PGPError displayDialog(PGPDialogProxyObjRef proxyObject);
PGPError createDialogInstance(PGPDialogProxyObjRef proxyObject);
PGPError releaseDialogInstance(PGPDialogProxyObjRef proxyObject);
PGPError configureDialog(PGPDialogProxyObjRef proxyObject);

/* Generic dialog options */
PGPError setContext(PGPDialogProxyObjRef proxyObject, PGPContextRef context);
PGPError setClientOptions(PGPDialogProxyObjRef proxyObject, PGPOptionListRef optionList);
PGPError setWindowTitle(PGPDialogProxyObjRef proxyObject, const char* windowTitle);
PGPError setServerList(PGPDialogProxyObjRef proxyObject, PGPKeyServerSpec* serverList);
PGPError setServerCount(PGPDialogProxyObjRef proxyObject, PGPUInt32 serverCount);
PGPError setSearchBeforeDisplay(PGPDialogProxyObjRef proxyObject, PGPBoolean searchBeforeDisplay);
PGPError setNewKeys(PGPDialogProxyObjRef proxyObject, PGPKeyDBRef* newKeys);
PGPError setTLSContext(PGPDialogProxyObjRef proxyObject, PGPtlsContextRef tlsContext);
PGPError setDefaultKey(PGPDialogProxyObjRef proxyObject, PGPKeyDBObjRef defaultKey);
PGPError setPrompt(PGPDialogProxyObjRef proxyObject, const char* prompt);

/* Random data dialog options */
PGPError setNeededEntropyBits(PGPDialogProxyObjRef proxyObject, PGPUInt32 neededEntropyBits);

/* Passphrase dialog options */
PGPError setPassphrasePtr(PGPDialogProxyObjRef proxyObject, char** passphrasePtr);
PGPError setDialogOptions(PGPDialogProxyObjRef proxyObject, PGPOptionListRef optionList);
PGPError setMinPassphraseLength(PGPDialogProxyObjRef proxyObject, PGPUInt32 minPassphraseLength);
PGPError setMinPassphraseQuality(PGPDialogProxyObjRef proxyObject, PGPUInt32 minPassphraseQuality);
PGPError setDialogTimeout(PGPDialogProxyObjRef proxyObject, PGPUInt32 timeout);

/* Key passphrase dialog options */
PGPError setVerifyPassphrase(PGPDialogProxyObjRef proxyObject, PGPBoolean verifyPassphrase);
PGPError setCache(PGPDialogProxyObjRef proxyObject, PGPBoolean cache);
PGPError setCacheGlobal(PGPDialogProxyObjRef proxyObject, PGPBoolean cacheGlobal);
PGPError setCacheTimeout(PGPDialogProxyObjRef proxyObject, PGPUInt32 cacheTimeout);

/* Confirmation passphrase dialog options */
PGPError setShowsPassphraseQuality(PGPDialogProxyObjRef proxyObject, PGPBoolean showPassphraseQuality);

/* KeySet passphrase dialog options */
PGPError setFindMatchingKey(PGPDialogProxyObjRef proxyObject, PGPBoolean findMatchingKey);
PGPError setKeySet(PGPDialogProxyObjRef proxyObject, PGPKeySetRef keySet);
PGPError setPassphraseKeyPtr(PGPDialogProxyObjRef proxyObject, PGPKeyDBObjRef* passphraseKeyPtr);

/* Decryption passphrase dialog options */
PGPError setKeyIDList(PGPDialogProxyObjRef proxyObject, PGPKeyID* keyIDList);
PGPError setKeyIDCount(PGPDialogProxyObjRef proxyObject, PGPUInt32 keyIDCount);
PGPError setMissingKeyIDList(PGPDialogProxyObjRef proxyObject, PGPKeyID* missingKeyIDList);
PGPError setMissingKeyIDCount(PGPDialogProxyObjRef proxyObject, PGPUInt32 missingKeyIDCount);

/* KeyServer dialog options */

/* Search KeyServer dialog options */
PGPError setSearchAllServers(PGPDialogProxyObjRef proxyObject, PGPBoolean searchAllServers);
PGPError setFilter(PGPDialogProxyObjRef proxyObject, PGPFilterRef filter);
PGPError setKeyDescription(PGPDialogProxyObjRef proxyObject, char* keyDescription);

/* SendTo KeyServer dialog options */
PGPError setKeysToSend(PGPDialogProxyObjRef proxyObject, PGPKeySetRef keysToSend);
PGPError setFailedKeys(PGPDialogProxyObjRef proxyObject, PGPKeySetRef* failedKeys);

/* Recipient dialog options */
PGPError setRecipientDialogOptions(PGPDialogProxyObjRef proxyObject, PGPOptionListRef dialogOptions);
PGPError setNumDefaultRecipients(PGPDialogProxyObjRef proxyObject, PGPUInt32 numDefaultRecipients);
PGPError setDefaultRecipients(PGPDialogProxyObjRef proxyObject, const PGPRecipientSpec* defaultRecipients);
PGPError setDisplayMarginalValidity(PGPDialogProxyObjRef proxyObject, PGPBoolean displayMarginalValidity);
PGPError setIgnoreMarginalValidity(PGPDialogProxyObjRef proxyObject, PGPBoolean ignoreMarginalValidity);
PGPError setGroupSet(PGPDialogProxyObjRef proxyObject, PGPGroupSetRef groupSet);
PGPError setClientKeySet(PGPDialogProxyObjRef proxyObject, PGPKeySetRef clientKeySet);
PGPError setAlwaysDisplay(PGPDialogProxyObjRef proxyObject, PGPBoolean alwaysDisplay);
PGPError setAlwaysDisplayWithARRs(PGPDialogProxyObjRef proxyObject, PGPBoolean alwaysDisplayWithARRs);
PGPError setRecipientKeysPtr(PGPDialogProxyObjRef proxyObject, PGPKeyDBRef* recipientKeysPtr);
PGPError setRecipientCount(PGPDialogProxyObjRef proxyObject, PGPUInt32* recipientCount);
PGPError setRecipientList(PGPDialogProxyObjRef proxyObject, PGPRecipientSpec** recipientList);
PGPError setEnforcement(PGPDialogProxyObjRef proxyObject, PGPAdditionalRecipientRequestEnforcement enforcement);

#ifdef __cplusplus
};
#endif
