/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPDialogProxies.m,v 1.4 2002/10/22 09:25:50 jeffc Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>
#import "PGPDialogProxies.h"
#import "PGPDialogClasses.h"

/*____________________________________________________________
	Generic dialog utility functions 
  ____________________________________________________________*/

PGPError displayDialog(PGPDialogProxyObjRef proxyObject)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
	
	err = [(CPGPDialog*) proxyObject->instance runDialog];
	
	return err;
}

PGPError createDialogInstance(PGPDialogProxyObjRef proxyObject)
{
	PGPError			err = kPGPError_NoErr;

	if (!proxyObject)
		return kPGPError_BadParams;

	proxyObject->instance = NULL;

	switch (proxyObject->type)
	{
		case kPGPDialogProxyType_RandomDataOptions:
			proxyObject->instance = (void*) [[CPGPCollectRandomDataDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_RandomDataDialog;
			
			err = configureDialog(proxyObject);
			
			break;
			
		case kPGPDialogProxyType_PassphraseOptions:
			proxyObject->instance = (void*) [[CPGPPassphraseDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_PassphraseDialog;
			
			err = configureDialog(proxyObject);
			
			break;
			
		case  kPGPDialogProxyType_KeyPassphraseOptions:
			proxyObject->instance = (void*) [[CPGPKeyPassphraseDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_PassphraseDialog;
			
			err = configureDialog(proxyObject);
			
			break;
			
		case kPGPDialogProxyType_SigningPassphraseOptions:
			proxyObject->instance = (void*) [[CPGPSignPassphraseDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_PassphraseDialog;
			
			err = configureDialog(proxyObject);
			
			break;
		
		case kPGPDialogProxyType_DecryptionPassphraseOptions:
			proxyObject->instance = (void*) [[CPGPDecryptionPassphraseDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_PassphraseDialog;
			
			err = configureDialog(proxyObject);
			
			break;
		
		case kPGPDialogProxyType_ConfirmationPassphraseOptions:
			proxyObject->instance = (void*) [[CPGPConfirmationPassphraseDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_PassphraseDialog;
			
			err = configureDialog(proxyObject);
					
			break;

		case kPGPDialogProxyType_SearchKeyServerOptions:
			proxyObject->instance = (void*) [[CPGPSearchKeyServerDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_KeyServerDialog;
			
			err = configureDialog(proxyObject);
			
			break;
		
		case kPGPDialogProxyType_SendToKeyServerOptions:
			proxyObject->instance = (void*) [[CPGPSendToKeyServerDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_KeyServerDialog;
			
			err = configureDialog(proxyObject);
			
			break;

		case kPGPDialogProxyType_RecipientOptions:
			proxyObject->instance = (void*) [[CPGPRecipientDialog alloc] init];
			[(CPGPDialog*) proxyObject->instance setProxyObject:proxyObject];
			proxyObject->dlgClass = kPGPDialogClass_RecipientDialog;
			
			err = configureDialog(proxyObject);
			
			break;			

		default:
			break;
	}
	
	return err;
}

PGPError releaseDialogInstance(PGPDialogProxyObjRef proxyObject)
{
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(NSObject*) proxyObject->instance release];
	
	proxyObject->instance = NULL;
	
	return kPGPError_NoErr;
}

/*____________________________________________________________
	Generic dialog options 
  ____________________________________________________________*/

PGPError setContext(PGPDialogProxyObjRef proxyObject, PGPContextRef context) 
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !context || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setContext:context];
	
	return err;
}

PGPError setClientOptions(PGPDialogProxyObjRef proxyObject, PGPOptionListRef optionList)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !optionList || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setClientOptions:optionList];
	
	return err;
}

PGPError setWindowTitle(PGPDialogProxyObjRef proxyObject, const char* windowTitle)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !windowTitle || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setWindowTitle:[NSString stringWithUTF8String:windowTitle]];
	
	return err;
}

PGPError setServerList(PGPDialogProxyObjRef proxyObject, PGPKeyServerSpec* serverList)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !serverList || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setServerList:serverList];
	
	return err;
}

PGPError setServerCount(PGPDialogProxyObjRef proxyObject, PGPUInt32 serverCount)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setServerCount:serverCount];
	
	return err;
}

PGPError setSearchBeforeDisplay(PGPDialogProxyObjRef proxyObject, PGPBoolean searchBeforeDisplay)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setSearchBeforeDisplay:searchBeforeDisplay];
	
	return err;
}

PGPError setNewKeys(PGPDialogProxyObjRef proxyObject, PGPKeyDBRef* newKeys)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !newKeys || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPDialog*) proxyObject->instance setNewKeys:newKeys];
	
	return err;
}

PGPError setTLSContext(PGPDialogProxyObjRef proxyObject, PGPtlsContextRef tlsContext)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !tlsContext || !proxyObject->instance)
		return kPGPError_BadParams;

	[(CPGPDialog*) proxyObject->instance setTLSContext:tlsContext];
	
	return err;
}

PGPError setDefaultKey(PGPDialogProxyObjRef proxyObject, PGPKeyDBObjRef defaultKey)
{
	PGPError		err = kPGPError_NoErr;

	if (!proxyObject || !defaultKey || !proxyObject->instance)
		return kPGPError_BadParams;
	
	[(CPGPKeyPassphraseDialog*) proxyObject->instance setDefaultKey:defaultKey];

	return err;
}

PGPError setPrompt(PGPDialogProxyObjRef proxyObject, const char* prompt)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !prompt || !proxyObject->instance)
		return kPGPError_BadParams;
	
	[(CPGPDialog*) proxyObject->instance setPrompt:[NSString stringWithUTF8String:prompt]];

	return err;
}

/*____________________________________________________________
	Random data dialog options 
  ____________________________________________________________*/

PGPError setNeededEntropyBits(PGPDialogProxyObjRef proxyObject, PGPUInt32 neededEntropyBits)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPCollectRandomDataDialog*) proxyObject->instance setNeededEntropyBits:neededEntropyBits];
	
	return err;
}

/*____________________________________________________________
	Passphrase dialog options 
  ____________________________________________________________*/

PGPError setPassphrasePtr(PGPDialogProxyObjRef proxyObject, char** passphrasePtr)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !passphrasePtr || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPPassphraseDialog*) proxyObject->instance setPassphrasePtr:passphrasePtr];
	
	return err;
}

PGPError setDialogOptions(PGPDialogProxyObjRef proxyObject, PGPOptionListRef optionList)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !optionList || !proxyObject->instance)
		return kPGPError_BadParams;
			
	[(CPGPPassphraseDialog*) proxyObject->instance setDialogOptions:optionList];
	
	return err;
}

PGPError setMinPassphraseQuality(PGPDialogProxyObjRef proxyObject, PGPUInt32 minPassphraseQuality)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
	
	[(CPGPPassphraseDialog*) proxyObject->instance setMinPassphraseQuality:minPassphraseQuality];

	return err;
}

PGPError setMinPassphraseLength(PGPDialogProxyObjRef proxyObject, PGPUInt32 minPassphraseLength)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
	
	[(CPGPPassphraseDialog*) proxyObject->instance setMinPassphraseLength:minPassphraseLength];
	
	return err;
}

PGPError setDialogTimeout(PGPDialogProxyObjRef proxyObject, PGPUInt32 timeout)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
	
	[(CPGPPassphraseDialog*) proxyObject->instance setDialogTimeout:timeout];
	
	return err;
}

/*____________________________________________________________
	Key passphrase dialog options 
  ____________________________________________________________*/

PGPError setVerifyPassphrase(PGPDialogProxyObjRef proxyObject, PGPBoolean verifyPassphrase)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
	
	[(CPGPKeyPassphraseDialog*) proxyObject->instance setVerifiesPassphrase:verifyPassphrase];

	return err;
}

PGPError setCache(PGPDialogProxyObjRef proxyObject, PGPBoolean cache)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPKeyPassphraseDialog*) proxyObject->instance setCache:cache];

	return err;
}

PGPError setCacheGlobal(PGPDialogProxyObjRef proxyObject, PGPBoolean cacheGlobal)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPKeyPassphraseDialog*) proxyObject->instance setCacheGlobal:cacheGlobal];

	return err;
}

PGPError setCacheTimeout(PGPDialogProxyObjRef proxyObject, PGPUInt32 cacheTimeout)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPKeyPassphraseDialog*) proxyObject->instance setCacheTimeout:cacheTimeout];

	return err;
}

/*____________________________________________________________
	Confirmation passphrase dialog options 
  ____________________________________________________________*/
  
PGPError setShowsPassphraseQuality(PGPDialogProxyObjRef proxyObject, PGPBoolean showPassphraseQuality)
{  
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPConfirmationPassphraseDialog*) proxyObject->instance setShowsPassphraseQuality:showPassphraseQuality];

	return err;
}

/*____________________________________________________________
	KeySet passphrase dialog options 
  ____________________________________________________________*/

PGPError setFindMatchingKey(PGPDialogProxyObjRef proxyObject, PGPBoolean findMatchingKey)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPKeySetPassphraseDialog*) proxyObject->instance setFindMatchingKey:findMatchingKey];

	return err;
}

PGPError setKeySet(PGPDialogProxyObjRef proxyObject, PGPKeySetRef keySet)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !keySet || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPKeySetPassphraseDialog*) proxyObject->instance setKeySet:keySet];

	return err;
}

PGPError setPassphraseKeyPtr(PGPDialogProxyObjRef proxyObject, PGPKeyDBObjRef* passphraseKeyPtr)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !passphraseKeyPtr || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPKeySetPassphraseDialog*) proxyObject->instance setPassphraseKeyPtr:passphraseKeyPtr];

	return err;
}

/*____________________________________________________________
	Decryption passphrase dialog options 
  ____________________________________________________________*/
  
PGPError setKeyIDList(PGPDialogProxyObjRef proxyObject, PGPKeyID* keyIDList)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !keyIDList || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPDecryptionPassphraseDialog*) proxyObject->instance setKeyIDList:keyIDList];

	return err;
}

PGPError setKeyIDCount(PGPDialogProxyObjRef proxyObject, PGPUInt32 keyIDCount)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPDecryptionPassphraseDialog*) proxyObject->instance setKeyIDCount:keyIDCount];

	return err;
}

PGPError setMissingKeyIDList(PGPDialogProxyObjRef proxyObject, PGPKeyID* missingKeyIDList)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !missingKeyIDList || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPDecryptionPassphraseDialog*) proxyObject->instance setMissingKeyIDList:missingKeyIDList];

	return err;
}

PGPError setMissingKeyIDCount(PGPDialogProxyObjRef proxyObject, PGPUInt32 missingKeyIDCount)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPDecryptionPassphraseDialog*) proxyObject->instance setMissingKeyIDCount:missingKeyIDCount];

	return err;
}

/*____________________________________________________________
	KeyServer dialog options 
  ____________________________________________________________*/

/*____________________________________________________________
	Search KeyServer dialog options 
  ____________________________________________________________*/

PGPError setSearchAllServers(PGPDialogProxyObjRef proxyObject, PGPBoolean searchAllServers)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPSearchKeyServerDialog*) proxyObject->instance setSearchAllServers:searchAllServers];

	return err;
}

PGPError setFilter(PGPDialogProxyObjRef proxyObject, PGPFilterRef filter)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !filter || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPSearchKeyServerDialog*) proxyObject->instance setFilter:filter];

	return err;
}

PGPError setKeyDescription(PGPDialogProxyObjRef proxyObject, char* keyDescription)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPSearchKeyServerDialog*) proxyObject->instance setKeyDescription:keyDescription];

	return err;
}

/*____________________________________________________________
	SendTo KeyServer dialog options 
  ____________________________________________________________*/

PGPError setKeysToSend(PGPDialogProxyObjRef proxyObject, PGPKeySetRef keysToSend)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !keysToSend || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPSendToKeyServerDialog*) proxyObject->instance setKeysToSend:keysToSend];

	return err;
}

PGPError setFailedKeys(PGPDialogProxyObjRef proxyObject, PGPKeySetRef* failedKeys)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !failedKeys || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPSendToKeyServerDialog*) proxyObject->instance setFailedKeys:failedKeys];

	return err;
}

/*____________________________________________________________
	Recipient dialog options 
  ____________________________________________________________*/

PGPError setRecipientDialogOptions(PGPDialogProxyObjRef proxyObject, PGPOptionListRef dialogOptions)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !dialogOptions || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setDialogOptions:dialogOptions];

	return err;
}

PGPError setNumDefaultRecipients(PGPDialogProxyObjRef proxyObject, PGPUInt32 numDefaultRecipients)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setNumDefaultRecipients:numDefaultRecipients];

	return err;
}

PGPError setDefaultRecipients(PGPDialogProxyObjRef proxyObject, const PGPRecipientSpec* defaultRecipients)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !defaultRecipients || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setDefaultRecipients:defaultRecipients];

	return err;
}

PGPError setDisplayMarginalValidity(PGPDialogProxyObjRef proxyObject, PGPBoolean displayMarginalValidity)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setDisplayMarginalValidity:displayMarginalValidity];

	return err;
}

PGPError setIgnoreMarginalValidity(PGPDialogProxyObjRef proxyObject, PGPBoolean ignoreMarginalValidity)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setIgnoreMarginalValidity:ignoreMarginalValidity];

	return err;
}

PGPError setGroupSet(PGPDialogProxyObjRef proxyObject, PGPGroupSetRef groupSet)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !groupSet || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setGroupSet:groupSet];

	return err;
}

PGPError setClientKeySet(PGPDialogProxyObjRef proxyObject, PGPKeySetRef clientKeySet)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !clientKeySet || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setClientKeySet:clientKeySet];

	return err;
}

PGPError setAlwaysDisplay(PGPDialogProxyObjRef proxyObject, PGPBoolean alwaysDisplay)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setAlwaysDisplay:alwaysDisplay];

	return err;
}

PGPError setAlwaysDisplayWithARRs(PGPDialogProxyObjRef proxyObject, PGPBoolean alwaysDisplayWithARRs)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setAlwaysDisplayWithARRs:alwaysDisplayWithARRs];

	return err;
}

PGPError setRecipientKeysPtr(PGPDialogProxyObjRef proxyObject, PGPKeyDBRef* recipientKeysPtr)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !recipientKeysPtr || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setRecipientKeysPtr:recipientKeysPtr];

	return err;
}

PGPError setRecipientCount(PGPDialogProxyObjRef proxyObject, PGPUInt32* recipientCount)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !recipientCount || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setRecipientCount:recipientCount];

	return err;
}

PGPError setRecipientList(PGPDialogProxyObjRef proxyObject, PGPRecipientSpec** recipientList)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !recipientList || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setRecipientList:recipientList];

	return err;
}

PGPError setEnforcement(PGPDialogProxyObjRef proxyObject, PGPAdditionalRecipientRequestEnforcement enforcement)
{
	PGPError		err = kPGPError_NoErr;
	
	if (!proxyObject || !proxyObject->instance)
		return kPGPError_BadParams;
		
	[(CPGPRecipientDialog*) proxyObject->instance setEnforcement:enforcement];

	return err;
}