/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPKeyServerDialogs.m,v 1.7 2002/10/08 09:50:45 jeffc Exp $
____________________________________________________________________________*/

#import "pgpDebug.h"
#import "PGPDialogClasses.h"
#import "pgpKeyServer.h"
#import "pgpKeyServerDialogCommon.h"
#import <unistd.h>

#define self CPGPKeyServerDialog

#define kPGPKeyServerDialogNib				@"PGPKeyServerDialog"
#define kPGPKeyServerString_Connecting		LSPU(@"Connecting", @"Connecting status string")
#define kPGPKeyServerString_Searching		LSPU(@"Searching", @"Searching status string")
#define kPGPKeyServerString_Receiving		LSPU(@"Receiving results", @"Receiving status string")
#define kPGPKeyServerString_Processing		LSPU(@"Processing results", @"Processing status string")
#define kPGPKeyServerString_Sending			LSPU(@"Sending", @"Sending status string")
#define kPGPKeyServerString_Deleting		LSPU(@"Deleting key(s)...", @"Deleting status string")
#define kPGPKeyServerString_Disabling		LSPU(@"Disabling key(s)...", @"Disabling status string")
#define kPGPKeyServerString_Closing			LSPU(@"Cleaning up...", @"Closing status string")

static PGPKeyServerRef	sLastKeyServer = NULL;
static PGPUInt32		sLastKeyServerState = 0;

static PGPError sKeyServerCallbackProc(PGPContextRef context, struct PGPEvent* event, PGPUserValue userValue)
{
	CPGPKeyServerDialog*	dialog;
	PGPKeyServerRef			eventServer;
	PGPUInt32				serverState;
	NSString*				string = nil;
	char*					hostname = NULL;
	
	dialog = (void*)userValue;
	
	pgpAssert(event != NULL);
	pgpAssert(dialog != nil);
	
	if (!event)
		return kPGPError_BadParams;
	
	if ([dialog returnCode] == kPGPError_UserAbort)
	{
		sLastKeyServer = NULL;
		sLastKeyServerState = 0;
		[dialog cancel:nil];
		
		return kPGPError_UserAbort;
	}
	
	switch (event->type)
	{
		case kPGPEvent_KeyServerIdleEvent:
			break;
		
		case kPGPEvent_KeyServerEvent:
			eventServer = event->data.keyServerData.keyServerRef;
			serverState = event->data.keyServerData.state;
			
			if (PGPKeyServerRefIsValid(eventServer) && (eventServer != sLastKeyServer))
			{
				sLastKeyServer = eventServer;
				PGPGetKeyServerHostName(sLastKeyServer, &hostname);
				
				if (hostname)
				{
					[[dialog serverNameField] setStringValue:[NSString stringWithCString:hostname]];
					[[dialog serverNameField] displayIfNeeded];
					PGPFreeData(hostname);
				}
			}
			
			if (serverState != sLastKeyServerState)
			{
				sLastKeyServerState = serverState;
				
				switch (serverState)
				{
					case kPGPKeyServerState_Opening:
						string = kPGPKeyServerString_Connecting;
						break;
						
					case kPGPKeyServerState_Querying:
						string = kPGPKeyServerString_Searching;
						break;
					
					case kPGPKeyServerState_ReceivingResults:
						string = kPGPKeyServerString_Receiving;
						break;
					
					case kPGPKeyServerState_ProcessingResults:
						string = kPGPKeyServerString_Processing;
						break;
						
					case kPGPKeyServerState_Uploading:
						string = kPGPKeyServerString_Sending;
						break;
					
					case kPGPKeyServerState_Deleting:
						string = kPGPKeyServerString_Deleting;
						break;
					
					case kPGPKeyServerState_Disabling:
						string = kPGPKeyServerString_Disabling;
						break;
					
					case kPGPKeyServerState_Closing:
						string = kPGPKeyServerString_Closing;
						break;
					
					default:
						string = @"";
						break;
				}
			}
			
			if (string)
			{
				[[dialog statusField] setStringValue:string];
				[[dialog statusField] displayIfNeeded];
			}
		
			break;
		
		default:
			break;
	}
	
	return kPGPError_NoErr;
}

#undef self

@implementation CPGPKeyServerDialog

- (id)init
{
	if (!mLoadedNib)
		[self loadNib:kPGPKeyServerDialogNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	else
		PGPLog((@"%s:%d: Bad nibFile specified", __FILE__, __LINE__));
    
	[mKeyName retain];
	[mKeyLabel retain];
	
	[mWindow setDelegate:self];
}

- (NSString*)prompt
{
	return nil;
}

- (void)setPrompt:(NSString*)prompt
{
}

- (NSTextField*)serverNameField
{
	return mServerName;
}

- (NSTextField*)keyNameField
{
	return mKeyName;
}

- (NSTextField*)statusField
{
	return mStatus;
}

- (void)close
{
	[mProgressIndicator stopAnimation:self];
	
	[mWindow orderOut:self];
}

- (IBAction)cancel:(id)sender
{
	mReturnCode = kPGPError_UserAbort;
	
	[NSApp abortModal];
}

- (void)dealloc
{
	[mWindow setDelegate:nil];
	
	if (mKeyName)
		[mKeyName release];
		
	if (mKeyLabel)
		[mKeyLabel release];
	
	[super dealloc];
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPSearchKeyServerDialog

- (id)init
{
	[super init];
		
	return self;
}

- (PGPBoolean)searchAllServers
{
	return mSearchAllServers;
}

- (void)setSearchAllServers:(PGPBoolean)flag
{
	mSearchAllServers = flag;
}

- (PGPFilterRef)filter
{
	return mFilter;
}

- (void)setFilter:(PGPFilterRef)filter
{
	mFilter = filter;
}

- (char*)keyDescription
{
	return mKeyDescription;
}

- (void)setKeyDescription:(char*)keyDescription
{
	if (keyDescription)
		strncpy(mKeyDescription, keyDescription, sizeof(mKeyDescription));
	else
		mKeyDescription[0] = '\0';
}

- (PGPError)runDialog
{
	NSRect			oldFrame;
	NSRect			newFrame;
	NSString*		string;
	NSData*			data;
	
	if (mKeyDescription[0])
	{
		if (!(string = [NSString stringWithUTF8String:mKeyDescription]))
		{
			data = [NSData dataWithBytes:mKeyDescription length:strlen(mKeyDescription)];
			string = [[[NSString alloc] initWithData:data encoding:NSMacOSRomanStringEncoding] autorelease];
		}
		
		[mKeyName setStringValue:string];
	}
	else
	{
		[mKeyName removeFromSuperview];
		[mKeyLabel removeFromSuperview];
		
		oldFrame = [mServerName frame];
		newFrame = [mKeyName frame];
		newFrame.size.width = NSWidth(oldFrame);
		[mServerName setFrame:newFrame];
		
		oldFrame = [mServerLabel frame];
		newFrame = [mKeyLabel frame];
		newFrame.size.width = NSWidth(oldFrame);
		[mServerLabel setFrame:newFrame];
		
		newFrame = [mWindow frame];
		newFrame.size.height -= NSHeight([mKeyName frame]) + 8;

		[mWindow setFrame:newFrame display:NO];
	}

	mThreadEnded = NO;
	[NSApplication detachDrawingThread:@selector(keyServerThread) toTarget:self withObject:nil];

	[mWindow center];
	[NSApp runModalForWindow:mWindow];
	
	[mWindow close];

	return mReturnCode;
}

- (void)keyServerThread
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	PGPKeyDBRef			newKeys = kInvalidPGPKeyDBRef;
	PGPKeyServerSpec*	serverList = NULL;
	PGPUInt32			serverIndex = 0;
	
	while (![mWindow isVisible])
		usleep(333);
	
	[mWindow display];
	
	[mProgressIndicator setUsesThreadedAnimation:YES];	/* Necessary since we run in a modal event loop */	
	[mProgressIndicator startAnimation:self];
	
	/* Copy the server list in case we get orphaned */
	serverList = PGPNewData(PGPPeekContextMemoryMgr(mContext),
		mServerCount * sizeof(PGPKeyServerSpec),
		kPGPMemoryMgrFlags_Clear);
	if (serverList != NULL)
	{
		for (serverIndex = 0; serverIndex < mServerCount; serverIndex++)
		{
			serverList[serverIndex].server = mServerList[serverIndex].server;
			if (PGPKeyServerRefIsValid (serverList[serverIndex].server))
				PGPIncKeyServerRefCount (serverList[serverIndex].server);
		}
	
		mAbort = NO;
		mReturnCode = PGPSearchKeyServerDialogCommon(mContext,
							serverList,
							mServerCount,
							mTLSContext,
							mFilter,
							sKeyServerCallbackProc,
							(PGPUserValue) self,	/* User Value */
							mSearchAllServers,
							&newKeys);
		
		PGPFreeData (serverList);
	}
	
	if (!mAbort)
	{
		*mNewKeys = newKeys;
		sLastKeyServer = NULL;
		sLastKeyServerState = 0;
		
		[mProgressIndicator stopAnimation:self];
		
		[NSApp abortModal];
		
		mThreadEnded = YES;
	}
	[pool release];
}

- (IBAction)cancel:(id)sender
{
	if (sLastKeyServer)
		PGPCancelKeyServerCall(sLastKeyServer);

	mReturnCode = kPGPError_UserAbort;
	mAbort = YES;
	[NSApp abortModal];
}

- (void)dealloc
{
	[super dealloc];
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPSendToKeyServerDialog

- (id)init
{
	[super init];
		
	return self;
}

- (PGPKeySetRef)keysToSend
{
	return mKeysToSend;
}

- (void)setKeysToSend:(PGPKeySetRef)keysToSend
{
	mKeysToSend = keysToSend;
}

- (PGPKeySetRef*)failedKeys
{
	return mFailedKeys;
}

- (void)setFailedKeys:(PGPKeySetRef*)failedKeys
{
	mFailedKeys = failedKeys;
}

- (PGPError)runDialog
{
	NSRect			oldFrame;
	NSRect			newFrame;
	NSModalSession	session;

	/* Remove the key name field and label. Move and resize window components accordingly. */
	[mKeyName removeFromSuperview];
	[mKeyLabel removeFromSuperview];
		
	oldFrame = [mServerName frame];
	newFrame = [mKeyName frame];
	newFrame.size.width = NSWidth(oldFrame);
	[mServerName setFrame:newFrame];
	
	oldFrame = [mServerLabel frame];
	newFrame = [mKeyLabel frame];
	newFrame.size.width = NSWidth(oldFrame);
	[mServerLabel setFrame:newFrame];
	
	newFrame = [mWindow frame];
	newFrame.size.height -= NSHeight([mKeyName frame]) + 8;

	[mWindow setFrame:newFrame display:NO];
	
	mThreadEnded = NO;
	[NSThread detachNewThreadSelector:@selector(keyServerThread) toTarget:self withObject:nil];

	[mWindow center];
	
	session = [NSApp beginModalSessionForWindow:mWindow];
	for (;;)
	{
		if (mAbort || ([NSApp runModalSession:session] != NSRunContinuesResponse))
			break;
	}
	[NSApp endModalSession:session];

	while (!mAbort && !mThreadEnded)
		usleep(333);
	[mWindow close];
		
	return mReturnCode;
}

- (void)keyServerThread
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	PGPKeyServerSpec*	serverList = NULL;
	PGPUInt32			serverIndex = 0;
	PGPKeySetRef		failedKeys = kInvalidPGPKeySetRef;

	while (![mWindow isVisible])
		usleep(333);

	[mProgressIndicator setUsesThreadedAnimation:YES];	/* Necessary since we run in a modal event loop */	
	[mProgressIndicator startAnimation:self];

	/* Copy the server list in case we get orphaned */
	serverList = PGPNewData(PGPPeekContextMemoryMgr(mContext),
		mServerCount * sizeof(PGPKeyServerSpec),
		kPGPMemoryMgrFlags_Clear);
	if (serverList != NULL)
	{
		for (serverIndex = 0; serverIndex < mServerCount; serverIndex++)
		{
			serverList[serverIndex].server = mServerList[serverIndex].server;
			if (PGPKeyServerRefIsValid (serverList[serverIndex].server))
				PGPIncKeyServerRefCount (serverList[serverIndex].server);
		}
	
		mAbort = NO;
	
		mReturnCode = PGPSendToKeyServerDialogCommon(mContext,
							serverList,
							mTLSContext,
							mKeysToSend,
							sKeyServerCallbackProc,
							(PGPUserValue) self,	/* User Value */
							&failedKeys);

		PGPFreeData (serverList);
	}
	
	if (!mAbort)
	{
		*mFailedKeys = failedKeys;
		sLastKeyServer = NULL;
		sLastKeyServerState = 0;
		
		[NSApp stopModal];
		mThreadEnded = YES;
	}
	
	[pool release];
}

- (IBAction)cancel:(id)sender
{
	if (sLastKeyServer)
		PGPCancelKeyServerCall(sLastKeyServer);

	mReturnCode = kPGPError_UserAbort;
	mAbort = YES;

	[super cancel:sender];
}

- (void)dealloc
{
	[super dealloc];
}

@end
