/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPRecipientDialog.m,v 1.15 2002/10/22 09:25:50 jeffc Exp $
____________________________________________________________________________*/

#import "PGPDialogClasses.h"
#import "pgpPFLPriv.h"
#import <unistd.h>

#define kPGPRecipientDialogNib								@"PGPRecipientDialog"
#define kUserTableColumnIdentifier_Users					@"Users"
#define kUserTableColumnIdentifier_Validity					@"Validity"
#define kUserTableColumnIdentifier_Size						@"Size"
#define kRecipientTableColumnIdentifier_Recipients			@"Recipients"
#define kRecipientTableColumnIdentifier_Validity			@"Validity"
#define kRecipientTableColumnIdentifier_Size				@"Size"
#define kUserTablePBoardType								@"PGPUserTablePBoardType"
#define kRecipientTablePBoardType							@"PGPRecipientTablePBoardType"

#define kPGPRecipientDialogString_MissingKeyID				LSPU(@"Unknown User (Key ID is %s)", @"Missing key ID")
#define kPGPRecipientDialogString_DSSDHKeySize				LSPU(@"%d/%d", @"DSS/DH key size")
#define kPGPRecipientDialogString_MissingRecipientKeySize	LSPU(@"No Match", @"No Match")
#define kPGPRecipientDialogString_ADKError					LSPU(@"ADK Error", @"ADK Error")
#define kPGPRecipientDialogString_ADKWarning				LSPU(@"ADK Warning", @"ADK Warning")
#define kPGPRecipientDialogString_ADKNotFoundError			LSPU(@"The user \"%@\" has a missing Additional Decryption Key. This user cannot be used for encryption without the Additional Decryption Key.", @"ADK not found error")
#define kPGPRecipientDialogString_ADKNotFoundWarning		LSPU(@"The user \"%@\" has a missing Additional Decryption Key. Contact the owner of the key to obtain the Additional Decryption Key.", @"ADK not found warning")
#define kPGPRecipientDialogString_GroupADKNotFoundError		LSPU(@"The group \"%@\" contains one or more keys with missing Additional Decryption Keys. This group cannot be used for encryption without these Additional Decryption Keys.", @"Group ADK not found error")
#define kPGPRecipientDialogString_GroupADKNotFoundWarning	LSPU(@"The group \"%@\" contains one or more keys with missing Additional Decryption Keys. Contact the owners of these keys to obtain the Additional Decryption Keys.", @"Group ADK not found warning")
#define kPGPRecipientDialogString_ADKViolatesPolicy			LSPU(@"Removing the Additional Decryption Key \"%@\" from the recipient list may violate the policy established for other keys in the recipient list.", @"ADK violates policy")
#define kPGPRecipientDialogString_ADKAbort					LSPU(@"Some of the keys you have chosen as recipients have Additional Decryption Key requests, and these ADKs have not yet been signed as valid on your keyring. Please verify the validity of the ADK before encrypting.", @"ADK abort")
#define kPGPRecipientDialogString_ADKAdded					LSPU(@"Some of the keys you have chosen as recipients have Additional Decryption Key requests, and these ADKs were added to the recipient list.", @"ADK added")
#define kConvEncryptTitleStr 								LSPU(@"Conventional Encrypt", @"Conventional encryption title")

#define self CPGPRecipientDialog

#define kMaxKeypressInterval 1.0

static PGPError sRecipientEventHandler(PGPRecipientsList* recipients, PGPRecipientEvent* event, PGPUserValue userValue)
{
	NSString*		title = nil;
	NSString*		messageString = nil;
	NSString*		name;
	int				len;
	
	switch (event->type)
	{
		case kPGPRecipientEvent_MoveUserFailedEvent:
			title = kPGPRecipientDialogString_ADKError;
			
			if (event->user->kind == kPGPRecipientUserKind_Key)
				messageString = kPGPRecipientDialogString_ADKNotFoundError;
			else
				messageString = kPGPRecipientDialogString_GroupADKNotFoundError;
			
			break;
		
		case kPGPRecipientEvent_MoveUserWarningEvent:
			title = kPGPRecipientDialogString_ADKWarning;
			
			if (event->user->kind == kPGPRecipientUserKind_Key)
				messageString = kPGPRecipientDialogString_ADKNotFoundWarning;
			else
				messageString = kPGPRecipientDialogString_GroupADKNotFoundWarning;
			
			break;
		
		case kPGPRecipientEvent_MovedARRWarningEvent:
			title = kPGPRecipientDialogString_ADKWarning;
			messageString = kPGPRecipientDialogString_ADKViolatesPolicy;
			
			break;
		
		default:
			break;
	}
	
	if (messageString)
	{
		name = [NSString stringWithUTF8String:PGPGetRecipientUserNamePtr(event->user)];
		
		if (!name)
		{
			len = strlen(PGPGetRecipientUserNamePtr(event->user));
			name = [[[NSString alloc] initWithData:[NSData dataWithBytes:PGPGetRecipientUserNamePtr(event->user) length:len] encoding:NSMacOSRomanStringEncoding] autorelease];
		}
		
		NSBeginAlertSheet(title,
				kPGPUIString_OK,
				nil,
				nil,
				[NSApp keyWindow],
				nil,
				nil,
				nil,
				nil,
				[NSString stringWithFormat:messageString, name]);
	}
	
	return kPGPError_NoErr;
}

static int compareRecipients(id recipientData1, id recipientData2, void* context)
{
	PGPRecipientUser*		recipient1;
	NSString*				recipient1Name;
	PGPRecipientUser*		recipient2;
	NSString*				recipient2Name;
	int						len;
	
	recipient1 = *((PGPRecipientUser**) [(NSData*) recipientData1 bytes]);
	recipient2 = *((PGPRecipientUser**) [(NSData*) recipientData2 bytes]);
	
	recipient1Name = [NSString stringWithUTF8String:PGPGetRecipientUserNamePtr(recipient1)];
	if (!recipient1Name)
	{
		len = strlen(PGPGetRecipientUserNamePtr(recipient1));
		recipient1Name = [[[NSString alloc] initWithData:[NSData dataWithBytes:PGPGetRecipientUserNamePtr(recipient1) length:len] encoding:NSMacOSRomanStringEncoding] autorelease];
	}
	
	recipient2Name = [NSString stringWithUTF8String:PGPGetRecipientUserNamePtr(recipient2)];
	if (!recipient2Name)
	{
		len = strlen(PGPGetRecipientUserNamePtr(recipient2));
		recipient2Name = [[[NSString alloc] initWithData:[NSData dataWithBytes:PGPGetRecipientUserNamePtr(recipient2) length:len] encoding:NSMacOSRomanStringEncoding] autorelease];
	}
	
	return [recipient1Name caseInsensitiveCompare:recipient2Name];
}

static PGPInt16 sMatchUserIDs(const char* uid1, const char* uid2)
{
	PGPInt16	matchChars = 0;
	char		a;
    char		b;
	
	while (*uid1 && *uid2)
	{
		a = *uid1++;
        b = *uid2++;
		
        if ((a >= 'a') && (a <= 'z'))
			a -= ('a' - 'A');
		if ((b >= 'a') && (b <= 'z'))
			b -= ('a' - 'A');
		if (a == b)
			matchChars++;
		else
			break;
	}
    
	return matchChars;
}

PGPError pgpGetMissingRecipientKeyIDStringPlatform(PGPContextRef context,
	const PGPKeyID* keyID, char keyIDString[256])
{
	PGPError        err;
	NSString*		keyString;
	char			tempStr[256];

	err = PGPGetKeyIDString(keyID, kPGPKeyIDString_Abbreviated, keyIDString);
	if(IsntPGPError(err))
	{
		strncpy(tempStr, keyIDString, 255);
		tempStr[255] = '\0';
		
		keyString = [NSString stringWithFormat:kPGPRecipientDialogString_MissingKeyID, tempStr];
		[keyString getCString:keyIDString maxLength:(sizeof(keyIDString) - 1)];
	}       

	return err;
}

#undef self

@implementation CPGPRecipientDialog

- (id)init
{
	if (!mLock)
		mLock = [[NSLock alloc] init];
		
	if (!mLoadedNib)
		[self loadNib:kPGPRecipientDialogNib];
	
    if (!mSearchString)
        mSearchString = [[NSMutableString alloc] init];
    
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	
	if (!mValidDot)
		mValidDot = [[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotGreen] retain];
	
	if (!mInvalidDot)
		mInvalidDot = [[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed] retain];
	
	[mOptions retain];
	[mUpdateFromServer retain];
	[mOptionsBox retain];
	[mTableViewContainer retain];
	[mButtonContainer retain];
    
    [mWindow makeFirstResponder:mUsersTable];
    [mUsersTable setNextResponder:mRecipientsTable];
    [mRecipientsTable setNextResponder:self];
    [self setNextResponder:mWindow];
	
	mMinSize = [mWindow minSize];
	
    [mWindow setMaxSize:[[NSScreen mainScreen] frame].size];
	[mWindow setDelegate:self];
	
	mLoadedNib = YES;
}

- (PGPOptionListRef)dialogOptions
{
	return mDialogOptions;
}

- (void)setDialogOptions:(PGPOptionListRef)dialogOptions
{
	mDialogOptions = dialogOptions;
}

- (PGPUInt32)numDefaultRecipients
{
	return mNumDefaultRecipients;
}

- (void)setNumDefaultRecipients:(PGPUInt32)numDefaultRecipients
{
	mNumDefaultRecipients = numDefaultRecipients;
}

- (const PGPRecipientSpec*)defaultRecipients
{
	return mDefaultRecipients;
}

- (void)setDefaultRecipients:(const PGPRecipientSpec*)defaultRecipients
{
	mDefaultRecipients = (PGPRecipientSpec*) defaultRecipients;
}

/* Displaying marginally valid keys is not supported at this time. */
- (PGPBoolean)displayMarginalValidity
{
	return mDisplayMarginalValidity;
}

- (void)setDisplayMarginalValidity:(PGPBoolean)displayMarginalValidity
{
	mDisplayMarginalValidity = displayMarginalValidity;
}

- (PGPBoolean)ignoreMarginalValidity
{
	return mIgnoreMarginalValidity;
}

- (void)setIgnoreMarginalValidity:(PGPBoolean)ignoreMarginalValidity
{
	mIgnoreMarginalValidity = ignoreMarginalValidity;
}

- (PGPGroupSetRef)groupSet
{
	return mGroupSet;
}

- (void)setGroupSet:(PGPGroupSetRef)groupSet
{
	mGroupSet = groupSet;
}

- (PGPKeySetRef)clientKeySet
{
	return mClientKeySet;
}

- (void)setClientKeySet:(PGPKeySetRef)clientKeySet
{
	mClientKeySet = clientKeySet;
}

- (PGPBoolean)alwaysDisplay
{
	return mAlwaysDisplay;
}

- (void)setAlwaysDisplay:(PGPBoolean)alwaysDisplay
{
	mAlwaysDisplay = alwaysDisplay;
}

- (PGPBoolean)alwaysDisplayWithARRs
{
	return mAlwaysDisplayWithARRs;
}

- (void)setAlwaysDisplayWithARRs:(PGPBoolean)alwaysDisplayWithARRs
{
	mAlwaysDisplayWithARRs = alwaysDisplayWithARRs;
}

- (PGPKeyDBRef*)recipientKeysPtr
{
	return mRecipientKeysPtr;
}

- (void)setRecipientKeysPtr:(PGPKeyDBRef*)recipientKeysPtr
{
	mRecipientKeysPtr = recipientKeysPtr;
}

- (PGPUInt32*)recipientCount
{
	return mRecipientCount;
}

- (void)setRecipientCount:(PGPUInt32*)recipientCount
{
	mRecipientCount = recipientCount;
}

- (PGPRecipientSpec**)recipientList
{
	return mRecipientList;
}

- (void)setRecipientList:(PGPRecipientSpec**)recipientList
{
	mRecipientList = recipientList;
}

- (PGPAdditionalRecipientRequestEnforcement)enforcement
{
	return mEnforcement;
}

- (void)setEnforcement:(PGPAdditionalRecipientRequestEnforcement)enforcement
{
	mEnforcement = enforcement;	
}

- (PGPError)runDialog
{
	PGPError			err = kPGPError_NoErr;
	PGPBoolean			haveDefaultARRs;
	PGPBoolean      	bInvalidADKs;
	PGPBoolean			bValidADKs;
	PGPBoolean			bShowDialog = [self alwaysDisplay];
	NSRect				frame;
	NSRect				newFrame;
	KeyMap				keyboardState;
	
	err = PGPBuildRecipientsList(NULL,
				mContext,
				mClientKeySet,
				mGroupSet,
				mNumDefaultRecipients,
				mDefaultRecipients,
				mServerCount,
				mServerList,
				mTLSContext,
				mDefaultKey,
				mSearchBeforeDisplay,
				mEnforcement,
				sRecipientEventHandler,
				NULL,
				&mRecipients,
				&haveDefaultARRs);
	
	if (IsPGPError(err))
		goto done;
	
	if (haveDefaultARRs && [self alwaysDisplayWithARRs])
		bShowDialog = TRUE;
	
	err = [self constructDataSources];
	
	if (IsPGPError(err))
	{
		PGPLog((@"Failed to construct table data sources (err=%d)", err));
		goto done;
	}
	
	if (bShowDialog == FALSE)
	{
		NSEnumerator*	enumerator;
		NSData*			recipientData;
		PGPInt32    	minimumValidity;
		
		if (![mRecipientArray count])
			bShowDialog = TRUE;
		
		if ([self ignoreMarginalValidity])
			minimumValidity = (PGPInt32) kPGPValidity_Complete;
		else
			minimumValidity = (PGPInt32) kPGPValidity_Marginal;
		
		/*
		Check to see if the dialog needs to be displayed.
		Show the dialog if there are missing, ambiguous, invalid,
		or no default recipients
		*/
		enumerator = [mRecipientArray objectEnumerator];
		
		while (recipientData = [enumerator nextObject])
		{
			PGPRecipientUser*	user;
			
			user = *((PGPRecipientUser**) [(NSData*) recipientData bytes]);
			
			if (user->multipleMatch ||
				user->kind == kPGPRecipientUserKind_MissingRecipient ||
				user->validity < minimumValidity)
			{
				bShowDialog = TRUE;
				break;
			}
		}
	}
	
	/* If the options key is pressed, we force the display of the dialog. */
    GetKeys(keyboardState);
	
	if (keyboardState[1] & 0x4)	/* Options key */
		bShowDialog = TRUE;
	
	if (bShowDialog && mDialogOptions)
	{
		mOptionsDialog = [[CPGPOptionsDialog alloc] initWithOptions:mDialogOptions];
		
		if (mOptionsView = [mOptionsDialog optionsView])
		{
			[mOptionsBox setContentView:mOptionsView];
			[mOptionsBox sizeToFit];
			[mOptionsBox removeFromSuperview];
			
			/*
			Determine if conventional encryption was passed as a dialog option.
			If so, we need make some UI modifications to support the option.
			*/
			[self setupConventionalEncryption];
		}
	}
	else
	{
		[mOptions removeFromSuperview];
		[mOptionsBox removeFromSuperview];
	}
	
	if (bShowDialog && mServerList && mNumMissingRecipients)
	{
		/* If mOptions is hidden, move the Update From Server button */
		if (![mOptions superview])
		{
			frame = [mOptions frame];
			newFrame = [mUpdateFromServer frame];
			
			newFrame.origin.x = frame.origin.x;
			newFrame.origin.y = frame.origin.y;
			
			[mUpdateFromServer setFrame:newFrame];
		}
	}
	else
		[mUpdateFromServer removeFromSuperview];
		
	if (IsPGPError(err))
		goto done;
	
	if (bShowDialog)
	{
		[self setupTableViews];

		[mUsersTable reloadData];
		[mRecipientsTable reloadData];
		
		[self selectTable:mUsersTable];
		
		err = [super runDialog];
	}
	
	while (mUpdatingMissingRecipients)
		usleep(333);
	
	if (IsntPGPError(err))
	{
		err = PGPGetRecipientKeys(&mRecipients,
					mRecipientKeysPtr,
					mNewKeys,
					mRecipientCount,
					mRecipientList,
                    &bValidADKs,
					&bInvalidADKs);

		if (IsntPGPError(err) && mAlwaysDisplayWithARRs)
		{
			if (bInvalidADKs)
			{
				NSBeginAlertSheet(kPGPRecipientDialogString_ADKError,
					kPGPUIString_OK,
					nil,
					nil,
					[NSApp keyWindow],
					nil,
					nil,
					nil,
					nil,
					kPGPRecipientDialogString_ADKAbort);
				
				err = kPGPError_UserAbort;
			}
			else if (bValidADKs)
			{
				NSBeginAlertSheet(kPGPRecipientDialogString_ADKWarning,
					kPGPUIString_OK,
					nil,
					nil,
					[NSApp keyWindow],
					nil,
					nil,
					nil,
					nil,
					kPGPRecipientDialogString_ADKAdded);
				
				err = kPGPError_NoErr;
			}
		}
	}
	
done:
	PGPDisposeRecipientsList(&mRecipients);
	
	return err;
}

- (void)close
{
	[NSApp stopModal];
	[mWindow close];
}

- (void)setupConventionalEncryption
{
	PGPUInt32				optionIndex = 0;
	PGPOption				optionData;
	PGPOUICheckboxDesc*		descriptor = nil;
	NSButton*				convEncryptionButton = nil;
	BOOL					haveConvEncryption = NO;
	
	pgpAssert(mOptionsView != nil);
	
	while (IsntPGPError(pgpGetIndexedOption(mDialogOptions, optionIndex, FALSE, &optionData)))
	{
		if (optionData.type == kPGPOptionType_Checkbox)
		{
			descriptor = (PGPOUICheckboxDesc*) optionData.value.asPtr;
	
			if (descriptor && [kConvEncryptTitleStr isEqualToString:[NSString stringWithUTF8String:descriptor->title]])
			{
				haveConvEncryption = YES;
				break;
			}
		}
		
		++optionIndex;
	}
	
	if (haveConvEncryption)
	{
		convEncryptionButton = [mOptionsView viewWithTag:descriptor->itemID];
		
		if (convEncryptionButton)
		{
			[convEncryptionButton setTarget:self];
			[convEncryptionButton setAction:@selector(conventionalEncrypt:)];
			[self conventionalEncrypt:convEncryptionButton];
		}
	}
}

- (void)setupTableViews
{
	NSTableColumn*		column;
	NSImageCell*		iCell;
	CBrowserCell*		bCell;
	
	[mUsersTable setDataSource:self];
	[mUsersTable setDelegate:self];
	[mUsersTable setTarget:self];
	[mUsersTable setDoubleAction:@selector(userDoubleClicked:)];
	
	[mRecipientsTable setDataSource:self];
	[mRecipientsTable setDelegate:self];
	[mRecipientsTable setTarget:self];
	[mRecipientsTable setDoubleAction:@selector(userDoubleClicked:)];

	[self registerDragTypes];
	
	if (column = [mUsersTable tableColumnWithIdentifier:kUserTableColumnIdentifier_Users])
	{
		bCell = [[[CBrowserCell alloc] init] autorelease];
		[bCell setLeaf:YES];
		[bCell setWraps:NO];
		[bCell setFont:[NSFont systemFontOfSize:10]];
		
		[column setDataCell:bCell];
	}
	
	if (column = [mUsersTable tableColumnWithIdentifier:kUserTableColumnIdentifier_Validity])
	{
		iCell = [[[NSImageCell alloc] initImageCell:nil] autorelease];
		[iCell setImageAlignment:NSImageAlignCenter];
		[iCell setImageFrameStyle:NSImageFrameNone];
		[iCell setImageScaling:NSScaleNone];
		
		[column setDataCell:iCell];
	}

	if (column = [mUsersTable tableColumnWithIdentifier:kUserTableColumnIdentifier_Size])
	{
		bCell = [[[CBrowserCell alloc] init] autorelease];
		[bCell setLeaf:YES];
		[bCell setWraps:NO];
		[bCell setFont:[NSFont systemFontOfSize:10]];
		
		[column setDataCell:bCell];
	}

	if (column = [mRecipientsTable tableColumnWithIdentifier:kRecipientTableColumnIdentifier_Recipients])
	{
		bCell = [[[CBrowserCell alloc] init] autorelease];
		[bCell setLeaf:YES];
		[bCell setWraps:NO];
		[bCell setFont:[NSFont systemFontOfSize:10]];
		
		[column setDataCell:bCell];
	}

	if (column = [mRecipientsTable tableColumnWithIdentifier:kRecipientTableColumnIdentifier_Validity])
	{
		iCell = [[[NSImageCell alloc] initImageCell:nil] autorelease];
		[iCell setImageAlignment:NSImageAlignCenter];
		[iCell setImageFrameStyle:NSImageFrameNone];
		[iCell setImageScaling:NSScaleNone];
		
		[column setDataCell:iCell];
	}

	if (column = [mRecipientsTable tableColumnWithIdentifier:kRecipientTableColumnIdentifier_Size])
	{
		bCell = [[[CBrowserCell alloc] init] autorelease];
		[bCell setLeaf:YES];
		[bCell setWraps:NO];
		[bCell setFont:[NSFont systemFontOfSize:10]];
		
		[column setDataCell:bCell];
	}
}

- (NSTableView*)selectedTable
{
    return mSelectedTable;
}

- (void)selectTable:(NSTableView*)table
{
	NSClipView*				clipView;
	NSScrollView*			scrollView;
	CPGPSelectedTableView*	selectedTableView;
	
	if (table != mSelectedTable)
	{
		if (mSelectedTable)
		{
			if ([[mSelectedTable superview] isKindOfClass:[NSClipView class]])
			{
				clipView = (NSClipView*) [mSelectedTable superview];
				
				if ([[clipView superview] isKindOfClass:[NSScrollView class]])
				{
					scrollView = (NSScrollView*) [clipView superview];
					
					if ([[scrollView superview] isKindOfClass:[CPGPSelectedTableView class]])
					{
						selectedTableView = (CPGPSelectedTableView*) [scrollView superview];
						
						[selectedTableView setIsSelected:NO];
					}
				}
			}
		}
		
		mSelectedTable = table;
		
		if (mSelectedTable)
		{
			if ([[mSelectedTable superview] isKindOfClass:[NSClipView class]])
			{
				clipView = (NSClipView*) [mSelectedTable superview];
				
				if ([[clipView superview] isKindOfClass:[NSScrollView class]])
				{
					scrollView = (NSScrollView*) [clipView superview];
					
					if ([[scrollView superview] isKindOfClass:[CPGPSelectedTableView class]])
					{
						selectedTableView = (CPGPSelectedTableView*) [scrollView superview];
						
						[selectedTableView setIsSelected:YES];
					}
				}
			}
		}
	}
}

- (void)registerDragTypes
{
	[mUsersTable registerForDraggedTypes:[NSArray arrayWithObject:kUserTablePBoardType]];
	[mRecipientsTable registerForDraggedTypes:[NSArray arrayWithObject:kRecipientTablePBoardType]];
}

- (void)unregisterDragTypes
{
	[mUsersTable unregisterDraggedTypes];
	[mRecipientsTable unregisterDraggedTypes];
}

- (PGPError)constructDataSources
{
	PGPError			err;

	if (mUserIDArray)
		[mUserIDArray release];
	
	mUserIDArray = [[NSMutableArray alloc] init];
	
	if (mRecipientArray)
		[mRecipientArray release];
	
	mRecipientArray = [[NSMutableArray alloc] init];

	mNumMissingRecipients = 0;

	err = [self populateDataSource:mUserIDArray
				withKeys:mRecipients.keys
				ofKind:kPGPRecipientUserKind_Key
				fromLocation:kPGPRecipientUserLocation_UserList];
	
	if (IsPGPError(err))
		return err;
	
	err = [self populateDataSource:mUserIDArray
				withUsers:mRecipients.groups
				ofKind:kPGPRecipientUserKind_Group
				fromLocation:kPGPRecipientUserLocation_UserList];

	if (IsPGPError(err))
		return err;
	
	err = [self populateDataSource:mRecipientArray
				withKeys:mRecipients.keys
				ofKind:kPGPRecipientUserKind_Key
				fromLocation:kPGPRecipientUserLocation_RecipientList];

	if (IsPGPError(err))
		return err;
	
	err = [self populateDataSource:mRecipientArray
				withKeys:mRecipients.keys
				ofKind:kPGPRecipientUserKind_Group
				fromLocation:kPGPRecipientUserLocation_RecipientList];

	if (IsPGPError(err))
		return err;
	
	err = [self populateDataSource:mRecipientArray
				withUsers:mRecipients.missingRecipients
				ofKind:kPGPRecipientUserKind_MissingRecipient
				fromLocation:kPGPRecipientUserLocation_RecipientList];
		
	[mUserIDArray sortUsingFunction:compareRecipients context:nil];
	[mRecipientArray sortUsingFunction:compareRecipients context:nil];
	
	return err;
}

- (PGPError)populateDataSource:(NSMutableArray*)dataSource withKeys:(PGPRecipientKey*)keys ofKind:(PGPRecipientUserKind)kind fromLocation:(PGPRecipientUserLocation)location
{
	PGPError			err = kPGPError_NoErr;
	PGPRecipientKey*	curKey = keys;

	while (curKey)
	{
		err = [self populateDataSource:dataSource withUsers:curKey->users ofKind:kind fromLocation:location];
		
		if (IsPGPError(err))
			return err;
		
		curKey = curKey->nextKey;
	}
	
	return err;
}

- (PGPError)populateDataSource:(NSMutableArray*)dataSource withUsers:(PGPRecipientUser*)users ofKind:(PGPRecipientUserKind)kind fromLocation:(PGPRecipientUserLocation)location
{
	PGPRecipientUser*	curUser;

	PGPValidateParam(dataSource);
	PGPValidateParam(kind != kPGPRecipientUserKind_Invalid);
	PGPValidateParam(location != kPGPRecipientUserLocation_Invalid);
	
	if (dataSource)
	{
		curUser = users;
		while (curUser)
		{
			if (curUser->kind == kind)
			{
				if (curUser->location == location)
				{
					/* Our data source is comprised of pointers to the actual PGPRecipientUser structs */
					[dataSource addObject:[NSData dataWithBytes:&curUser length:sizeof(PGPRecipientUser*)]];
					
					if (location == kPGPRecipientUserLocation_RecipientList)
					{
						if (kind == kPGPRecipientUserKind_MissingRecipient)
							++mNumMissingRecipients;
					}
				}
			}
			
			curUser = curUser->nextUser;
		}
	}
	
	return kPGPError_NoErr;
}

- (PGPRecipientUser*)userForTableView:(NSTableView*)table row:(int)row
{
	NSArray*			dataSource = nil;
	PGPRecipientUser*	user;
	
	if (table == mUsersTable)
		dataSource = mUserIDArray;
	
	else if (table == mRecipientsTable)
		dataSource = mRecipientArray;
	
	if (!dataSource)
		return nil;
	
	if (row > [dataSource count])
		return nil;		/* Row is out of bounds */
	
	user = *((PGPRecipientUser**) [(NSData*) [dataSource objectAtIndex:row] bytes]);
	
	return user;
}

- (PGPError)moveUsers:(NSData*)users toLocation:(PGPRecipientUserLocation)location
{
	PGPError			err;
	PGPRecipientUser**	recipients;
	PGPUInt32			numRecipients;
	PGPUInt32			numMovedUsers;
	PGPBoolean			movedARRs;
	NSMutableArray*		source = nil;
	NSMutableArray*		destination = nil;
	
	PGPValidateParam(users);
	PGPValidateParam(location != kPGPRecipientUserLocation_Invalid);
	
	if (location == kPGPRecipientUserLocation_RecipientList)
	{
		source = mUserIDArray;
		destination = mRecipientArray;
	}

	else if (location == kPGPRecipientUserLocation_UserList)
	{
		source = mRecipientArray;
		destination = mUserIDArray;
	}
	
	if (!source || !destination)
		return kPGPError_UnknownError;
	
	recipients = (PGPRecipientUser**) [users bytes];
	
	if (!recipients)
		return kPGPError_BadParams;
	
	numRecipients = [users length] / sizeof(PGPRecipientUser*);
		
	err = PGPMoveRecipients(&mRecipients, location, numRecipients, recipients, &numMovedUsers, &movedARRs);
	if (IsPGPError(err))
	{
		PGPLog((@"PGPMoveRecipients failed with %d", err));
		return err;
	}
	
	err = [self constructDataSources];
	if (IsPGPError(err))
		PGPLog((@"Failed to construct table data sources (err=%d)", err));
	else
	{
		[mUsersTable reloadData];
		[mRecipientsTable reloadData];
	}
	
	if (!mNumMissingRecipients)
		[mUpdateFromServer setEnabled:NO];
	
	return kPGPError_NoErr;
}

- (IBAction)userDoubleClicked:(id)sender
{
	PGPError					err;
	NSArray*					source;
	PGPRecipientUserLocation	location;
	
	if (sender == mUsersTable)
	{
		location = kPGPRecipientUserLocation_RecipientList;
		source = mUserIDArray;
	}
	else
	{
		location = kPGPRecipientUserLocation_UserList;
		source = mRecipientArray;
	}
	
	err = [self moveUsers:[source objectAtIndex:[sender selectedRow]] toLocation:location];
	if (IsPGPError(err))
		PGPLog((@"Failed to move user (err=%d)", err));
	
	[mUsersTable reloadData];
	[mRecipientsTable reloadData];
}

- (IBAction)showOptions:(id)sender
{
	NSRect			frame;
	NSView*			contentView;

	[self unregisterDragTypes];
	
	if ([mOptionsBox superview])
	{
		//frame = NSMakeRect(0, 0, NSWidth([[mWindow contentView] frame]), NSHeight([[mWindow contentView] frame]) - NSHeight([mOptionsBox frame]) - 20);
		frame = NSMakeRect(0, 0, NSWidth([[mWindow contentView] frame]), (NSHeight([mTableViewContainer frame]) + NSHeight([mButtonContainer frame])) + 28);
		contentView = [[[NSView alloc] initWithFrame:frame] autorelease];
		
		/* Place the table view container to the top of the content view */
		[mTableViewContainer removeFromSuperview];
		frame = [mTableViewContainer frame];
		frame.origin.y = NSMaxY([contentView frame]) - NSHeight(frame);
		[mTableViewContainer setFrame:frame];
		[contentView addSubview:mTableViewContainer];
		
		[mOptionsBox removeFromSuperview];
		
		[mOptions removeFromSuperview];
		frame = [mOptions frame];
		frame.origin.y = NSMinY([mTableViewContainer frame]) - NSHeight(frame) - 6;
		[mOptions setFrame:frame];
		[contentView addSubview:mOptions];
		
		/* Place the button container at the bottom of the content view */
		[mButtonContainer removeFromSuperview];
		frame = [mButtonContainer frame];
		frame.origin.y = 0;
		[mButtonContainer setFrame:frame];
		[contentView addSubview:mButtonContainer];
		
		/* Resize the window. Set its new minSize and maxSize */
		frame = [mWindow frame];
		frame.origin.y += NSHeight([[mWindow contentView] frame]) - NSHeight([contentView frame]);
		frame.size.height -= NSHeight([[mWindow contentView] frame]);
		frame.size.height += NSHeight([contentView frame]);
		[mWindow setContentView:contentView];
		[mWindow setFrame:frame display:YES animate:NO];
		[mWindow setMinSize:mMinSize];
	}
	else
	{
		frame = NSMakeRect(0, 0, NSWidth([[mWindow contentView] frame]), NSHeight([mTableViewContainer frame]) + NSHeight([mOptionsBox frame]) + NSHeight([mButtonContainer frame]) + 8);
		contentView = [[[NSView alloc] initWithFrame:frame] autorelease];
		
		/* Place the table view container to the top of the content view */
		[mTableViewContainer removeFromSuperview];
		frame = [mTableViewContainer frame];
		frame.origin.y = NSMaxY([contentView frame]) - NSHeight(frame);
		[mTableViewContainer setFrame:frame];
		[contentView addSubview:mTableViewContainer];
		
		/* Place the options box underneath the table view container */
		frame = [mOptionsBox frame];
		frame.origin.y = NSMinY([mTableViewContainer frame]) - NSHeight(frame) - 8;
		frame.size.width = NSWidth([[mWindow contentView] frame]) - 75;
		[mOptionsBox setFrame:frame];
		[contentView addSubview:mOptionsBox];
		
		/* Place the options button next to the options box */
		frame = [mOptions frame];
		frame.origin.y = NSMinY([mTableViewContainer frame]) - NSHeight(frame) - 6;
		[mOptions setFrame:frame];
		[contentView addSubview:mOptions];
		
		/* Place the button container at the bottom of the content view */
		[mButtonContainer removeFromSuperview];
		frame = [mButtonContainer frame];
		frame.origin.y = 0;
		[mButtonContainer setFrame:frame];
		[contentView addSubview:mButtonContainer];
		
		/* Resize the window. Set its new minSize and maxSize */
		frame = [mWindow frame];
		frame.origin.y += NSHeight([[mWindow contentView] frame]) - NSHeight([contentView frame]);
		frame.size.height -= NSHeight([[mWindow contentView] frame]);
		frame.size.height += NSHeight([contentView frame]);
		[mWindow setContentView:contentView];
		[mWindow setFrame:frame display:YES animate:NO];
		[mWindow setMinSize:NSMakeSize([mWindow minSize].width, ([mWindow minSize].height + NSHeight([mOptionsBox frame])) - 20)];
	}
	
	[self registerDragTypes];
}

- (IBAction)updateFromServer:(id)sender
{
	[mUsersTable deselectAll:nil];
	[mRecipientsTable deselectAll:nil];
	
	[NSThread detachNewThreadSelector:@selector(updateMissingRecipients) toTarget:self withObject:nil];
}

- (void)updateMissingRecipients
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	PGPError			err;
	PGPBoolean			haveNewKeys;

	mUpdatingMissingRecipients = YES;
	
	err = PGPUpdateMissingRecipients(NULL, &mRecipients, &haveNewKeys);

	if (IsntPGPError(err) && haveNewKeys)
	{
		mNumMissingRecipients = 0;
		
		err = [self constructDataSources];
		mUpdatingMissingRecipients = NO;
		
		if (IsPGPError(err))
			PGPLog((@"Failed to construct table data sources (err=%d)", err));
		else
		{
			[mUsersTable reloadData];
			[mRecipientsTable reloadData];
		}
		
		if (!mNumMissingRecipients)
			[mUpdateFromServer setEnabled:NO];
	}
	
	mUpdatingMissingRecipients = NO;
	
	[pool release];
}

- (IBAction)cancel:(id)sender
{
	mReturnCode = kPGPError_UserAbort;
	
	[self close];
}

- (IBAction)ok:(id)sender
{
	mReturnCode = kPGPError_NoErr;
	
	[self close];
}

- (IBAction)conventionalEncrypt:(id)sender
{
	if ([sender state] == NSOnState)
	{
		mConventionalEncryptionEnabled = YES;
		[mUsersTable setBackgroundColor:[NSColor lightGrayColor]];
		[mRecipientsTable setBackgroundColor:[NSColor lightGrayColor]];
	}
	else
	{
		mConventionalEncryptionEnabled = NO;
		[mUsersTable setBackgroundColor:[NSColor whiteColor]];
		[mRecipientsTable setBackgroundColor:[NSColor whiteColor]];
	}
	
    [mOptionsDialog checkboxPressed:sender];
    
	[mUsersTable noteNumberOfRowsChanged];
	[mRecipientsTable noteNumberOfRowsChanged];
}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (void)selectSearchString
{
    NSArray*			dataSource;
	NSEnumerator*		enumerator;
    NSData*				bestUserData = nil;
    NSData*				userData;
    PGPRecipientUser*	user;
	PGPInt32			searchScore;
	PGPInt32			bestSearchScore = 0;
	
    if ([self selectedTable] == mUsersTable)
        dataSource = mUserIDArray;
    else
        dataSource = mRecipientArray;
    
    enumerator = [dataSource objectEnumerator];
    
    while (userData = [enumerator nextObject])
    {
        user = *((PGPRecipientUser**) [userData bytes]);
        
        searchScore = sMatchUserIDs(PGPGetRecipientUserNamePtr(user), [mSearchString UTF8String]);
        
        if (searchScore > bestSearchScore)
        {
            bestSearchScore = searchScore;
            bestUserData = userData;
        }
    }
    
    if (!bestUserData)
        [mSearchString deleteCharactersInRange:NSMakeRange(0, [mSearchString length])];
    else
    {
        PGPInt32		bestRow = [dataSource indexOfObject:bestUserData];
        [[self selectedTable] selectRow:bestRow byExtendingSelection:NO];
        [[self selectedTable] scrollRowToVisible:bestRow];
    }
}

- (void)keyDown:(NSEvent*)event
{
	NSString*	keyString = [event characters];
    unichar		key = 0;

    if (keyString)
        key = [keyString characterAtIndex:0];
    
    if (key)
    {
        if (((key >= ' ') && (key <= 'Z')) ||
            ((key >= 'a') && (key <= 'z')))
        {
            if ([event timestamp] - kMaxKeypressInterval > mLastKeypress)
                [mSearchString deleteCharactersInRange:NSMakeRange(0,[mSearchString length])];
            
            [mSearchString appendString:[NSString stringWithCharacters:&key length:1]];
            mLastKeypress = [event timestamp];
            [self selectSearchString];
        }
        else if (key == NSCarriageReturnCharacter)
            [mOK performClick:nil];
        else if ((key == NSEnterCharacter) || (key == NSRightArrowFunctionKey))
        {
            if ([[self selectedTable] selectedRow] != -1)
                [self userDoubleClicked:[self selectedTable]];
        }
        else if ((key == NSDeleteCharacter) || (key == NSLeftArrowFunctionKey))
        {
            if ([[self selectedTable] selectedRow] != -1)
            {
                if ([self selectedTable] == mRecipientsTable)
                    [self userDoubleClicked:[self selectedTable]];
            }
        }
        else if ((key == NSTabCharacter) || (key == NSBackTabCharacter))
        {
            if ([self selectedTable] == mRecipientsTable)
            {
                mLastRecipientRow = [mRecipientsTable selectedRow];
                [mRecipientsTable deselectAll:nil];
                [mWindow makeFirstResponder:mUsersTable];
				[mUsersTable selectRow:mLastUserRow byExtendingSelection:NO];
                [mUsersTable scrollRowToVisible:mLastUserRow];
				[self selectTable:mUsersTable];
            }
            else
            {
                mLastUserRow = [mUsersTable selectedRow];
                [mUsersTable deselectAll:nil];
                [mWindow makeFirstResponder:mRecipientsTable];
				[mRecipientsTable selectRow:mLastRecipientRow byExtendingSelection:NO];
                [mRecipientsTable scrollRowToVisible:mLastRecipientRow];
				[self selectTable:mRecipientsTable];
            }
        }
        else
            [self interpretKeyEvents:[NSArray arrayWithObject:event]];
    }
}

- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row
{
	PGPRecipientUser*	user;
	
	if (!mUpdatingMissingRecipients)
	{
		user = [self userForTableView:table row:row];
		
		if (user)
		{
			if ([[column identifier] isEqualToString:kUserTableColumnIdentifier_Users] ||
				[[column identifier] isEqualToString:kRecipientTableColumnIdentifier_Recipients])
			{
				/* Set the icon for the user */
				if (user->kind == kPGPRecipientUserKind_Key)
				{
					if (user->userInfo.key->algorithm == kPGPPublicKeyAlgorithm_RSA)
						[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_RSAUser]];
					else
						[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DSAUser]];
				}
				else if (user->kind == kPGPRecipientUserKind_MissingRecipient)
					[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_UnknownUser]];
				else if (user->kind == kPGPRecipientUserKind_Group)
					[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_Group]];
			}
		}
	}
}

- (BOOL)tableView:(NSTableView*)table shouldSelectRow:(int)row
{
	if (table == mUsersTable)
		[mRecipientsTable deselectAll:nil];
	else
		[mUsersTable deselectAll:nil];
	
	[self selectTable:table];
    
	return YES;
}

- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	if (mUpdatingMissingRecipients || mConventionalEncryptionEnabled)
		return 0;
		
	if (tableView == mUsersTable)
	{
		if (mUserIDArray)
			return [mUserIDArray count];
	}
	
	else if (tableView == mRecipientsTable)
	{
		if (mRecipientArray)
			return [mRecipientArray count];
	}
	
	return 0;
}

- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row
{
	id					value = nil;
	PGPRecipientUser*	user;
	int					len;

	user = [self userForTableView:tableView row:row];
	
	if (!user || mUpdatingMissingRecipients)
		return nil;
	
	if ([[column identifier] isEqualToString:kUserTableColumnIdentifier_Users] ||
		[[column identifier] isEqualToString:kRecipientTableColumnIdentifier_Recipients])
	{
		value = [NSString stringWithUTF8String:PGPGetRecipientUserNamePtr(user)];
		
		if (!value)
		{
			len = strlen(PGPGetRecipientUserNamePtr(user));
			value = [[[NSString alloc] initWithData:[NSData dataWithBytes:PGPGetRecipientUserNamePtr(user) length:len] encoding:NSMacOSRomanStringEncoding] autorelease];
		}
	}
	else if ([[column identifier] isEqualToString:kUserTableColumnIdentifier_Size] ||
			 [[column identifier] isEqualToString:kRecipientTableColumnIdentifier_Size])
	{
		if (user->kind == kPGPRecipientUserKind_Key)
		{
			if (user->userInfo.key->subKeyBits != 0)
			{
				value = [NSString stringWithFormat:kPGPRecipientDialogString_DSSDHKeySize,
							user->userInfo.key->subKeyBits, user->userInfo.key->keyBits];
			}
			else
				value = [NSString stringWithFormat:@"%d", user->userInfo.key->keyBits];
		}
		else if (user->kind == kPGPRecipientUserKind_MissingRecipient)
			value = kPGPRecipientDialogString_MissingRecipientKeySize;
	}
	else if ([[column identifier] isEqualToString:kUserTableColumnIdentifier_Validity] ||
			 [[column identifier] isEqualToString:kRecipientTableColumnIdentifier_Validity])
	{
		if (user->validity == kPGPValidity_Complete)
			value = mValidDot;
		else
			value = mInvalidDot;
	}

	return value;
}

- (BOOL)tableView:(NSTableView*)tableView writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pBoard
{
	NSEnumerator*		enumerator;
	NSNumber*			row;
	NSArray*			source;
	PGPRecipientUser**	userPtr;
	
	if (mUpdatingMissingRecipients)
		return NO;
	
	if (mDragData)
		[mDragData release];
	
	if (tableView == mUsersTable)
		source = mUserIDArray;
	else
		source = mRecipientArray;
	
	mDragData = [[NSMutableData alloc] init];
	enumerator = [rows objectEnumerator];
	
	while (row = [enumerator nextObject])
	{
		userPtr = (PGPRecipientUser**) [(NSData*) [source objectAtIndex:[row intValue]] bytes];
		[mDragData appendBytes:userPtr length:sizeof(PGPRecipientUser*)];
	}
	
	if (tableView == mRecipientsTable)
		[pBoard declareTypes:[NSArray arrayWithObject:kUserTablePBoardType] owner:self];
	else
		[pBoard declareTypes:[NSArray arrayWithObject:kRecipientTablePBoardType] owner:self];

	return YES;
}

- (void)pasteboard:(NSPasteboard*)sender provideDataForType:(NSString*)type
{
	if (mDragData)
	{
		if ([type isEqualToString:kRecipientTablePBoardType])
			[sender setData:mDragData forType:kRecipientTablePBoardType];
		else if ([type isEqualToString:kUserTablePBoardType])
			[sender setData:mDragData forType:kUserTablePBoardType];
	}
}

- (NSDragOperation)tableView:(NSTableView*)table validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
{
    NSPasteboard*	pboard = [info draggingPasteboard];
    NSString*		type;
    
	if (mUpdatingMissingRecipients)
		return NSDragOperationNone;
	
	if (table == mRecipientsTable)
	{
		type = [pboard availableTypeFromArray:[NSArray arrayWithObject:kRecipientTablePBoardType]];
		
		if (type)
		{
			[table setDropRow:-1 dropOperation:NSTableViewDropOn];
		
			return NSDragOperationGeneric;
		}
	}
	else
	{
		type = [pboard availableTypeFromArray:[NSArray arrayWithObject:kUserTablePBoardType]];
		
		if (type)
		{
			[table setDropRow:-1 dropOperation:NSTableViewDropOn];
		
			return NSDragOperationGeneric;
		}
	}
		
    return NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView*)table acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation
{
    NSPasteboard*				pboard = [info draggingPasteboard];
    NSString*					type;
	PGPRecipientUserLocation	location;
	PGPError					err;
    
	if (mUpdatingMissingRecipients)
		return NO;
	
	if (table == mRecipientsTable)
	{
	    type = [pboard availableTypeFromArray:[NSArray arrayWithObject:kRecipientTablePBoardType]];

		if (type)
		{
			location = kPGPRecipientUserLocation_RecipientList;
			
			err = [self moveUsers:[pboard dataForType:kRecipientTablePBoardType] toLocation:location];
		
			if (IsPGPError(err))
			{
				PGPLog((@"Failed to accept drop (err=%d)", err));
				return NO;
			}
			
			[mUsersTable reloadData];
			[mRecipientsTable reloadData];			
			[mUsersTable deselectAll:nil];
			
			return YES;
		}
	}

	else
	{
		type = [pboard availableTypeFromArray:[NSArray arrayWithObject:kUserTablePBoardType]];

		if (type)
		{
			location = kPGPRecipientUserLocation_UserList;
			
			err = [self moveUsers:[pboard dataForType:kUserTablePBoardType] toLocation:location];
			
			if (IsPGPError(err))
			{
				PGPLog((@"Failed to accept drop (err=%d)", err));
				return NO;
			}
		
			[mUsersTable reloadData];
			[mRecipientsTable reloadData];
			[mRecipientsTable deselectAll:nil];
		
			return YES;
		}
    }
    
	return NO;
}

- (void)dealloc
{
	[mWindow setDelegate:nil];
	
	[self unregisterDragTypes];
	
	if (mOptions)
		[mOptions release];
	
	if (mUpdateFromServer)
		[mUpdateFromServer release];
	
	if (mUserIDArray)
		[mUserIDArray release];
		
	if (mRecipientArray)
		[mRecipientArray release];
	
	if (mDragData)
		[mDragData release];
	
	if (mValidDot)
		[mValidDot release];
	
	if (mInvalidDot)
		[mInvalidDot release];
	
	if (mOptionsDialog)
		[mOptionsDialog release];
	
	if (mOptionsBox)
		[mOptionsBox release];
	
	if (mTableViewContainer)
		[mTableViewContainer release];
	
	if (mButtonContainer)
		[mButtonContainer release];
	
    if (mSearchString)
        [mSearchString release];
	
	[super dealloc];
}

@end