/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpFrontEndThread.c,v 1.6 2002/08/06 20:11:18 dallen Exp $
____________________________________________________________________________*/
#include <pthread.h>
#include <unistd.h>
#include <signal.h>
#include "pgpBase.h"
#include "pgpMem.h"
#include "pgpThreads.h"
#include "pgpContext.h"
#include "pgpPassCach.h"
#include "pgpRPCMsg.h"
#include "pgpKeyPriv.h"

static PGPBoolean	frontEndThreadRunning;
PGPMutex_t			sRPCMutex;
PGPBoolean			haveSomethingCached;


static void *
sFrontEndThread(
	void *			arg )
{
	PGPContextRef	context		= kInvalidPGPContextRef;

	(void) arg;

	while( frontEndThreadRunning )
	{
		sleep( 1 );

		context = kInvalidPGPContextRef;
		while( IsntNull( context = pgpContextNextContext( context ) ) )
		{
			PGPMutexLock( &sRPCMutex );

			pgpExpirePassphraseCache( context );
			pgpExpireKeyDBCache( context );

			if( pgpIsPassphraseCached( context ) || pgpIsKeyDBCached( context ) )
				haveSomethingCached = TRUE;
			else
				haveSomethingCached = FALSE;

			PGPMutexUnlock( &sRPCMutex );
		}
	}

	return NULL;
}

void
pgpInitFrontEndThread(
	PGPBoolean		clientSidePwdCache )
{
	pthread_t		thread;

	(void) clientSidePwdCache;

	frontEndThreadRunning = TRUE;

#if PGP_UNIX_HPUX
	pthread_create( &thread, pthread_attr_default, sFrontEndThread, NULL );
	pthread_detach( &thread );
#else
	pthread_create( &thread, NULL, sFrontEndThread, NULL );
	pthread_detach( thread );
#endif

	return;
}

void
pgpKillFrontEndThread( void )
{
	frontEndThreadRunning = FALSE;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
